/*************************************************************************
 * CARD.C - Contains file routines for reading, writing & polling PSX ram
 *		cards.
 *
 *	Started on - 11-02-95
 *	Written by - Mike Heilemann
 *************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include "prjhdrs.h"

#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <sys/file.h>
#include "psxpad.h"

#define	IO_WAIT	22

extern Sint32 YesNoBox(char *message, int x, int y, int width);

/* here is some global stuff for Cards */
unsigned long ev0,ev1,ev2,ev3;
//unsigned long ev10,ev11,ev12,ev13;
char		card[2];					// Used to keep track of inserted cards...
card_t card_info;
memfile_t MemFile[16];
char *CARD_WRITE_MSG = 	  "WRITING MEMORY CARD: Do NOT remove";
//char *CARD_OPEN_ERROR = "FILE OPEN ERROR";
//char *CARD_DELETE_ERROR = "FILE DELETE ERROR";
char *CARD_READ_ERROR =	  "MEMORY CARD ERROR";
char *CARD_WRITE_ERROR =  "MEMORY CARD ERROR";
char *CARD_SPACE_ERROR =  "Not enough room on Memory Card to save file.";
char *CARD_MISSING_ERROR ="This option needs a Memory Card in Slot 1";

extern int WriteMessageFlag;

int dialog_num;

Sint16 CardSlot = -1;


void wait(int ms)
{
	long time;

	time = GetVBlankCount();
	while (time + ms < GetVBlankCount())
		;
}


void ResetCard(void)
{
	InitCARD(0);
//	InitCARD(1);	// Async access.
	StartCARD();
	_bu_init();
}


/*************************************************************************
 *	INIT CARD - Initializes and Establishes proper Events for card polling.
 *************************************************************************/
void InitCard(void)
{
	ev0 = OpenEvent(SwCARD, EvSpIOE,		EvMdNOINTR, NULL);
	ev1 = OpenEvent(SwCARD, EvSpERROR,	EvMdNOINTR, NULL);
	ev2 = OpenEvent(SwCARD, EvSpTIMOUT,	EvMdNOINTR, NULL);
	ev3 = OpenEvent(SwCARD, EvSpNEW,		EvMdNOINTR, NULL);
	EnableEvent(ev0);
	EnableEvent(ev1);
	EnableEvent(ev2);
	EnableEvent(ev3);

//	ev10 = OpenEvent(HwCARD, EvSpIOE, EvMdNOINTR, NULL);
//	ev11 = OpenEvent(HwCARD, EvSpERROR, EvMdNOINTR, NULL);
//	ev12 = OpenEvent(HwCARD, EvSpTIMOUT, EvMdNOINTR, NULL);
//	ev13 = OpenEvent(HwCARD, EvSpNEW, EvMdNOINTR, NULL);
//	EnableEvent(ev10);
//	EnableEvent(ev11);
//	EnableEvent(ev12);
//	EnableEvent(ev13);


	InitCARD(0);	// No Async access.
	StartCARD();
	_bu_init();
}


/*************************************************************************
 *	CHECK CARD - Tests system for card on port passed in.
 *
 *		Returns 1 if Card in port
 *		Returns 0 if no card in port
 *************************************************************************/
short CheckCard(char port, char bank)
{
	long ret;
	char s[20];


	_new_card();

	_clear_event();					// Be sure it is clear or stuff might fail

	_card_info(port * 16 + bank);
	ret = _card_event();

	if (ret == 1 || ret == 2)		// No card
		return 0;

	if (ret == 3) {					// New Card
		_clear_event();
		_card_clear(port * 16 + bank);
		ret = _card_event();

		if (ret == 1 || ret == 2)	// No card
			return 0;
	}

	return(1);
}


/*************************************************************************
 *	CHECK FORMAT - Tests card on port passed to see if it is formatted.
 *
 *		Returns  1 if Card formatted
 *		Returns  0 if Card not formatted
 *		Returns -1 if no Card in specified port
 *************************************************************************/
short CheckFormat(char port, char bank)
{
	int error = 0;
	long ret;
//	char s[128];
#if 1
// My way saves 37 bytes

	_clear_event();						// Clear previous card events
	_card_load(port * 16 + bank);		// Test format request
	ret = _card_event();					// Get results

// REQUIREMENT: AUTO FORMAT NOT ALLOWED.
	while (ret == 3)					 	// If new card result its unformatted...
	{
		ResetCard();

		_clear_event();					// Clear previous card events
		if(_card_load(port * 16 + bank))	// Test format request
			ret = _card_event();				// Get results

		if (error++ == 5)	 	// If new card result its unformatted...
			if (DialogBoxYN(256, 110, "Card is Unformatted: Format Card?"))
				if (!FormatCard(port, bank))
					return(0);
			else
				return(0);
	}
	return(1);

#else
// Sony Way - Slower, requires reading of card

	_clear_event();						// Clear previous card events
	_card_read(port * 16 + bank, 0, s);	// read header @ start of card
	while (s[0] != 'M' && s[1] != 'C')
	{
		_clear_event();						// Clear previous card events
		_card_read(port * 16 + bank, 0, s);	// read header @ start of card

		if (error++ == MAX_RETRYS)	 	// If new card result its unformatted...
			if (DialogBoxYN(256, 110, "Card is Unformatted: Format Card?"))
				if (!FormatCard(port, bank))
					return(0);
			else
				return(0);
	}
	return(1);	// FORMATTED
#endif
}


/*************************************************************************
 * FORMAT CARD - Formats card destroying all data.
 *
 * RETURNS:
 * 	0 = failure
 *		1 = success
 *************************************************************************/
short FormatCard(char port, char bank)
{
	long ret;
	char s[20];

//	if (!CheckCard(port, bank))	// If no card, return -1
//		return(-1);

	dialog_num = DialogBoxMessage(256, 112, CARD_WRITE_MSG);
	_card_load(port * 16 + bank);
	_card_wait(port);					// Need to wait in between commands

	sprintf(s, "bu%d%d:", port, bank);
	ret = format(s);
	CloseDialogBoxMessage(dialog_num);
	if (ret == 1)
		return(1);
	else
		return(0);
}


/*************************************************************************
 * OPEN FILE - Opens a file on the memory card for writing.
 *
 * NOTE: Description may be up to 15 characters long (SONY BUG)
 *************************************************************************/
int OpenFile(char *buffer, long length, char port, char bank, char *file_name, char *description)
{
	int i,
		 error = 0;
	char *ptr1,				// For Icons
		  *ptr2,				// For Icons
		  *ptr3,				// For Icons
		  s[80];
	long fp,
		  len;


	if (!CheckFormat(port, bank))	// Unformatted or format error
		return(FILE_ERROR);

	if (WriteMessageFlag)
		dialog_num = DialogBoxMessage(256, 112, CARD_WRITE_MSG);
//	else
//		WriteMessageFlag = 1;

	len = length + 4;												//tom size of sectors with 2 byte checksum applied +4 for length
	len = (len/126*2) + (len%126 ? 2 : 0);	//tom account for checksum in each sector
	len += sizeof(card_t);									//tom size of 1st sector with no checksum applied

	card_info.magic[0] = 'S';
	card_info.magic[1] = 'C';
	card_info.type = 0x13;	// 0x11 = 1 anim, 0x12 = 2 anim, 0x13 = 3 anim frames
	card_info.blocks = (len / 8192) + (len % 8192 ? 1:0); // Calculate blocks
	sprintf(card_info.name, "%s", description);
//	card_info.pad[28];

	// Load Icons from CD
	ptr1 = (char *)LoadCDFile("\\ICON\\FBALL1.tim");	// ICON FILES 16 X 16 X 16 color
	ptr2 = (char *)LoadCDFile("\\ICON\\FBALL2.tim");
	ptr3 = (char *)LoadCDFile("\\ICON\\FBALL3.tim");

	memcpy(card_info.clut, (char *)(ptr1+20), 32);
	memcpy(card_info.icon[0], (char *)(ptr1+64), 128);
	memcpy(card_info.icon[1], (char *)(ptr2+64), 128);
	memcpy(card_info.icon[2], (char *)(ptr3+64), 128);

	// Free the loaded cd files here
	FreeFileData(ptr1);
	FreeFileData(ptr2);
	FreeFileData(ptr3);

	sprintf(s, "bu%d%d:%s", port, bank, file_name);		// Opens file on port 0 or 1, Using file_name

// Delete old file
	i = DeleteFile(port, bank, file_name);				 	// Get rid of old one
	if (!i)
	{
		wait(20);
		DeleteFile(port, bank, file_name);				 	// Get rid of old one
	}
	error = 0;


// Check for space on card
	GetFiles(port, bank);
	for (i = 0; i < 15; i++)
		if (MemFile[i].blocks)
			error += MemFile[i].blocks;
		else
			break;

	if (error > 15-card_info.blocks)
	{
		if (WriteMessageFlag)
			CloseDialogBoxMessage(dialog_num);
		TimedMessage(5, CARD_SPACE_ERROR);
		return(FILE_ERROR);
	}
	error = 0;


	fp = open(s, O_CREAT | (card_info.blocks<<16));
	while (fp == -1)											// Retry open
	{
		ResetCard();
		if (error++ == MAX_RETRYS)
		{
			if (WriteMessageFlag)
				CloseDialogBoxMessage(dialog_num);
			TimedMessage(5, CARD_WRITE_ERROR);
			return(FILE_ERROR);
		}
		fp = open(s, O_CREAT | (card_info.blocks<<16));
	}

	close(fp);

	wait(IO_WAIT);

	fp = open(s, O_WRONLY);
	while (fp == -1)											// Retry open
	{
		if (error++ == MAX_RETRYS)
		{
			if (WriteMessageFlag)
				CloseDialogBoxMessage(dialog_num);
			TimedMessage(5, CARD_WRITE_ERROR);
			return(FILE_ERROR);								// Open it read only
		}
		fp = open(s, O_WRONLY);
	}
	error = 0;

	wait(IO_WAIT);

	i = write(fp, &card_info, sizeof(card_t));

	if (i == 0)
	{
		if (WriteMessageFlag)
			CloseDialogBoxMessage(dialog_num);
		TimedMessage(5, CARD_WRITE_ERROR);
		return(FILE_ERROR);
	}

	while(i == -1)					// RETRY 10 times before quitting
	{
		if (error++ == MAX_RETRYS)
		{
			if (WriteMessageFlag)
				CloseDialogBoxMessage(dialog_num);
			TimedMessage(5, CARD_WRITE_ERROR);
			return(FILE_ERROR);
		}

		wait(IO_WAIT);

		i = write(fp, &card_info, sizeof(card_t));
	}

	wait(IO_WAIT);

	error = WriteFile(fp, (unsigned short *)buffer, length);	// This closes the file as well...
	close(fp);

	return(error);
}


/*************************************************************************
 * READ FILE - Reads in and checks a file in 126 byte blocks. (127 & 128
 *		are checksum data.
 *
 *	returns 0 if checksum is correct
 *	returns 1 if checksum fails
 *************************************************************************/
char ReadFile(unsigned short *buffer, char port, char bank, char *file)
{
	register int i,
					 data_read,
					 read_error = 0;
	int check_error = 0;

	long length;
	unsigned short temp[64];
	unsigned short check;

	char s[80];
	unsigned long fp;

	if (!CheckFormat(port, bank))	// Unformatted or format error
		return(FILE_ERROR);

	sprintf(s, "bu%d%d:%s", port, bank, file);			// Opens file on port.

	fp = open(s, O_RDONLY);

	while (fp == -1)
	{
		if (read_error++ == 10)
		{
//			TimedMessage(5, CARD_READ_ERROR);
			return(OPEN_ERROR);								// Open it read only
		}
		fp = open(s, O_RDONLY);
	}

	wait(IO_WAIT);

	read_error = 0;

	data_read = read(fp, &card_info, sizeof(card_t));	// Read Header 512 bytes
	while(data_read == -1)
	{
		if (read_error++ == 10)
		{
			close(fp);
//			TimedMessage(5, CARD_READ_ERROR);
			return(FILE_ERROR);
		}
		wait(IO_WAIT);

		data_read = read(fp, &card_info, sizeof(card_t));
	}
	read_error = 0;

	wait(IO_WAIT);

	data_read = read(fp, &temp, 128);
	while(data_read == -1)
	{
		if (read_error++ == MAX_RETRYS)
		{
			close(fp);
//			TimedMessage(5, CARD_READ_ERROR);
			return(FILE_ERROR);
		}
		data_read = read(fp, &temp, 128);
	}
	read_error = 0;

	length = (temp[0] << 16) + temp[1];
	i = 2;												// Skip the length

	while (length > 0)
	{
		check = 0;
		for (; i < data_read / 2 -1; i++)		// Checksum calculating
		{
			check ^= temp[i];

			if(length-i > 0) *buffer++ = temp[i];
			else buffer++;
  		}

		if (temp[i] != check)						// Checksum Failed (Bad File)
		{
			lseek(fp, -data_read, SEEK_CUR);		// RETRY 100 times before quitting
			wait(20);

			if (check_error++ == MAX_RETRYS)
			{
				close(fp);
//				TimedMessage(5, "UNABLE TO READ BLOCK");
				return(FILE_ERROR);
			}

		// Adjusted by Nigel to not count checksum word ie. -1

			length += data_read / 2 -1;
			buffer -= data_read / 2 -1;			// Reset buffer pointer
		}
		else
		{
			check_error = 0;
		}

		data_read = read(fp, &temp, 128);
		if (data_read == 0)
		{
			close(fp);
//			TimedMessage(5, CARD_READ_ERROR);
			return(FILE_ERROR);
		}

		while(data_read == -1)						// RETRY 100 times before quitting
		{
			wait(20);
			if (read_error++ == MAX_RETRYS)
			{
				close(fp);
//				TimedMessage(5, CARD_READ_ERROR);
				return(FILE_ERROR);
			}
			data_read = read(fp, &temp, 128);
		}
		read_error = 0;

		// Adjusted by Nigel to not count checksum word ie. -1

		length -= data_read / 2 -1;					// Adjust length to read (words)

		i = 0;										// Reset index
	}
	close(fp);

	return(0);
}


/*************************************************************************
 * WRITE FILE - Writes buffer to file handle passed in with checksums.
 *		Length should be passed in as bytes.
 *
 *	Note: LENGTH is really important, as read does not detect EOF so
 *		we must save it ourselves and keep track of it on the read.
 *************************************************************************/
int WriteFile(long fp, unsigned short *buffer, long length)
{
	long i,
		  j,
		  error = 0;
	unsigned short temp[64];	  	// Temporary buffer for data.
	unsigned short check;


	length /= 2;						// Convert to words
	length += (length % 2) ? 3 : 2;	// Convert to words & add on length

	for (i = 0; i < length; i += 63)
	{
		check = 0;
		for (j = 0; j < 63; j++) {
			if (i == 0 && j == 0)	// Beginning of file Write length first
			{
				temp[0] = length >> 16;
				temp[1] = length & 0x0000FFFF;
				j = 1;					// Advance pointer past length & don't do checksum on length
			}
			else
			{
				if (i + j >= length)
					temp[j] = 0;
				else
					temp[j] = *buffer++;	// Store character in buffer
				check ^= temp[j];
			}
		}

	   temp[63] = check;			  	// Put checksum in temporary buffer

		j = write(fp, &temp, 128);
		if (j == 0)
		{
			close(fp);
			if (WriteMessageFlag)
				CloseDialogBoxMessage(dialog_num);
			return(FILE_ERROR);
		}

		while(j == -1)					// RETRY 100 times before quitting
		{
			if (error++ == MAX_RETRYS)
			{
				close(fp);
				if (WriteMessageFlag)
					CloseDialogBoxMessage(dialog_num);
				return(FILE_ERROR);
			}
			j = write(fp, &temp, 128);
		}
		error = 0;
	}

	close(fp);
	if (WriteMessageFlag)
		CloseDialogBoxMessage(dialog_num);
	return(0);	// Return success
}


/*************************************************************************
 * READ HEADER - Reads in and checks a file in 126 byte blocks. (127 & 128
 *		are checksum data.
 *
 *	returns -1 if open fails
 *	returns  0 if read fails
 *	returns  1 if read is successful
 *************************************************************************/
char ReadHeader(char *buffer, char port, char bank, char *file)
{
	int i,
		 error = 0;
	char s[80];
	long fp;

	sprintf(s, "bu%d%d:%s", port, bank, file);// Opens file on port.

	fp = open(s, O_RDONLY);							// Open it read only
	while (fp == -1)
	{
		if (error++ == MAX_RETRYS)
		{
			TimedMessage(5, "FILE OPEN ERROR");
			return(fp);
		}
		fp = open(s, O_RDONLY);							// Open it read only
	}

	wait(IO_WAIT);

	while((i = read(fp, &card_info, sizeof(card_t))) == -1)
		;

	if (i)
		strcpy(buffer, card_info.name);
	else
	{
		close(fp);
		TimedMessage(5, "FILE READ ERROR");
		return(0);
	}
	close(fp);

	return(1);
}


/*************************************************************************
 * GET FILES - Gets file names and slot sizes from memory card
 *************************************************************************/
char GetFiles(char port, char bank)
{
	int i,j,k;
	int fnum[2];
	char file[40];
	struct DIRENTRY dir;


	do
	{
		for(j=0; j<2; j++)
		{
			if (!CheckFormat(port, bank))	// Unformatted or format error
			{
//			return(FILE_ERROR);
			return(-1);
			}



			for (i = 0; i < 15; i++)
			{
				MemFile[i].name[0] = 0;	// if it is NULL it is the last file
				MemFile[i].blocks  = 0;	// if it is NULL it is the last file
			}

			i = 0;

			sprintf(file, "bu%d%d:*", port, bank);

			if (k = firstfile(file, &dir) != 0)
			{
				strcpy(MemFile[i].name, dir.name);
				MemFile[i].blocks = dir.size / 8192;
				i++;

				wait(IO_WAIT);

				while (k = nextfile(&dir) != 0)
				{
					strcpy(MemFile[i].name, dir.name);
					MemFile[i].blocks = dir.size / 8192;
					i++;
				}
			}

			fnum[j] = i;
		}

	} while(fnum[0] != fnum[1]);	// if the 2 results weren't equal then an error occurred (brute force as nextfile can return false values)

	return(fnum[0]);	//
}


/*************************************************************************
 * CHECK FILE - Checks for a file
 *
 *	RETURNS: 1 - File Exists
 *				0 - File does not Exist
 *************************************************************************/
int CheckFile(char port, char bank, char *file)
{
	int i;

	GetFiles(port, bank);
	for (i = 0; i < 15; i++)
	{
		if (!strcmp(file, MemFile[i].name))
			return(1);
	}

	return(0);
}


/*************************************************************************
 * PROMPT FOR OVERWRITE - Checks for a file and prompts for overwrite.
 *
 *	RETURNS: 1 - Ok to Overwrite
 *				0 - Don't Overwrite
 *************************************************************************/
int PromptForOverwrite(char port, char bank, int x, int y, char *file)
{
	int i,
		 tmp = 0;

	if (CheckFile(port, bank, file))	// If it is there we know we have space
  		return(DialogBoxYN(x, y, "OVERWRITE current file?"));
	else			// Check for room
	{
		for (i = 0; i < 15; i++)
			tmp += MemFile[i].blocks;
		if (tmp < 15)
			return(1);
		else
			TimedMessage(4, CARD_SPACE_ERROR);
	}

	return(0);
}


/*************************************************************************
 * DELETE FILE - Pass in what port and the file name and it gets deleted
 *		from Ram card.
 *
 *	Returns 0 - Failed
 *			  1 - File was Deleted
 *************************************************************************/
long DeleteFile(char port, char bank, char *file_name)
{
	char s[40];

	sprintf(s, "bu%d%d:%s", port, bank, file_name);
	return (delete(s));
}



short _card_event(void)
{
	while(1) {
		if (TestEvent(ev0) == 1)
			return(0);
		if (TestEvent(ev1) == 1)
			return(1);
		if (TestEvent(ev2) == 1)
			return(2);
		if (TestEvent(ev3) == 1)
			return(3);
	}
}

void _clear_event(void)
{
	TestEvent(ev0);
	TestEvent(ev1);
	TestEvent(ev2);
	TestEvent(ev3);
}


//short _card_event_x(void)		// Hardware function
//{
//	while(1) {
//		if (TestEvent(ev10) == 1)
//			return(0);
//		if (TestEvent(ev11) == 1)
//			return(1);
//		if (TestEvent(ev12) == 1)
//			return(2);
//		if (TestEvent(ev13) == 1)
//			return(3);
//	}
//}

//void _clear_event_x(void)		// Hardware function
//{
//	TestEvent(ev10);
//	TestEvent(ev11);
//	TestEvent(ev12);
//	TestEvent(ev13);
//}


/*************************************************************************
 * READ PLAYHEADER - Reads in and checks a file in 126 byte blocks. (127 & 128
 *		are checksum data.
 *
 *	returns 0 if checksum is correct
 *	returns 1 if checksum fails
 *************************************************************************/
char ReadPlayHeader(unsigned short *buffer, char port, char bank, char *file)
{
	int i,
		 j,
		 read_error = 0,
		 check_error = 0,
		 data_read;
	long length;
	unsigned short temp[64];
	unsigned short check;

	char s[80];
	long fp;


	sprintf(s, "bu%d%d:%s", port, bank, file);// Opens file on port.

// OPEN W/ ERROR CHECKING
	fp = open(s, O_RDONLY);							// Open it read only
	while (fp == -1)
	{
		ResetCard();
		if (read_error++ == MAX_RETRYS)
		{
//			TimedMessage(5, CARD_OPEN_ERROR);
			return(FILE_ERROR);						// Open it read only
		}
		fp = open(s, O_RDONLY);
	}
	read_error = 0;

	wait(IO_WAIT);

// READ FILE HEADER W/ ERROR CHECKING
	data_read = read(fp, &card_info, sizeof(card_t));	// Read Header 512 bytes
	while(data_read == -1)
	{
		if (read_error++ == MAX_RETRYS)
		{
			close(fp);
			TimedMessage(5, CARD_READ_ERROR);
			return(FILE_ERROR);
		}
		data_read = read(fp, &card_info, sizeof(card_t));
	}
	read_error = 0;

// READ DATA W/ ERROR CHECKING
	data_read = read(fp, &temp, 128);
	while(data_read == -1)
	{
		if (read_error++ == MAX_RETRYS)
		{
			close(fp);
			TimedMessage(5, CARD_READ_ERROR);
			return(FILE_ERROR);
		}
		data_read = read(fp, &temp, 128);
	}
	read_error = 0;

	length = (temp[0] << 16) + temp[1];
	i = 2;												// Skip the length

	j = 3;												// 3 blocks contain play names string

// CHECKSUM ERROR CHECKING
	while ((length > 0) && j)
	{
		check = 0;
		for (; i < 63; i++)							// Checksum calculating
		{
			check ^= temp[i];
			*buffer++ = temp[i];
		}

		if (temp[i] != check)						// Checksum Failed (Bad File)
		{
			lseek(fp, -data_read, SEEK_CUR);		// RETRY 100 times before quitting

			if (check_error++ == MAX_RETRYS)
			{
				close(fp);
				TimedMessage(5, "FILE CHECKSUM ERROR");
				return(FILE_ERROR);
			}

			length += data_read / 2 -1;			//tom added -1
			buffer -= data_read / 2 -1;			// Reset buffer pointer
		}
		else
		{
			check_error = 0;
		}

// READ DATA W/ ERROR CHECKING
		data_read = read(fp, &temp, 128);
		if (data_read == 0)
		{
			close(fp);
			TimedMessage(5, CARD_READ_ERROR);
			return(FILE_ERROR);
		}

		while(data_read == -1)						// RETRY 100 times before quitting
		{
			if (read_error++ == MAX_RETRYS)
			{
				close(fp);
				TimedMessage(5, CARD_READ_ERROR);
				return(FILE_ERROR);
			}
			data_read = read(fp, &temp, 128);
		}
		read_error = 0;

		i = 0;											// Reset index
		length -= data_read / 2;					// Adjust length to read (words)

		j--;
	}
	close(fp);
	return(0);
}

/****************************************************************************/

int GetCardBank(void)
{
	int i, j;

	CardSlot = -1;

	j = 1;							// Try 1 times

	while(CardSlot < 0)
	{
		if(CheckCard(0, 0))
			CardSlot = 0;

		if(!j--)
		{
			return(FALSE);
		}
	}

	return(TRUE);
}


/****************************************************************************/
// GetPadPort
// PURPOSE : obtain Port & Bank values for memory card associated with a
// pad controller
// INPUT: Pad number
// OUTPUT: 16 bits value containing Port in hi byte & Bank in lo byte
/****************************************************************************/

UWORD GetPadPortBank(int pad)
{
	UWORD i;
	UWORD port;
	UWORD bank;
	UWORD mtap[2]={0,0};

	for(i=0; i<2; i++)
	{
		if(MultiTapID[i]) mtap[i] = 1;
	}

	if(!pad) return(0);			// Pad 0 always the same irregardless of multitap configuration

	port = (pad < (mtap[0] << 2)) ? 0:1;

	if(!port) return(pad);		// hi byte already 0

	bank = pad - ((mtap[0]*3)+1); // subtract offset

	return(bank | 0x100);
}

/****************************************************************************/
