/***************************************************************
	CD File handler routines

	Copyright 1995 GameTek
	All rights reserved.

	By Christopher G. Deel
****************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include <kernel.h>
#include <libcd.h>

#include "prjhdrs.h"	/* Project header files */

//#define DEBUG_THIS 1	/* Debug this module 0,1 or 2 */
//#define TEST_THIS 1 	// Test this mudule

int gReTrys;
char gCurrentDir[MAXPATH];

enum BGSTATES 
    { 
    	IDLE=0, STARTING_SEEK, SEEKING, READING, ERROR, COMPLETE
    } gBackgroundLoadState;
    
ULONG gBackgroundSectorID;
char *gBackgroundLoadDest;

#define MAX_LOAD_LIST 10

typedef struct
{
	ULONG sector;
	char *dest;
} LoadListType;

LoadListType gLoadList[MAX_LOAD_LIST+1];

int gLoadListIn, gLoadListOut, gLoadListState, gBackgroundLoadingFlag;

#define PAUSE_CD_TIME (1 * 60 * VBlanksPerSec)
#define STOP_CD_TIME (3 * 60 * VBlanksPerSec)
static ULONG gPauseCDTime = 0, gStopCDTime = 0;
char gSpinDownEnabled = TRUE;


/*-------------------------------------------------------------
  ShowDiskErrorMessage ~ 
-------------------------------------------------------------*/
void ShowDiskErrorMessage(char *msg)
{
	int i;
	char *dirty = "Disk is dirty or damaged\n\n";
	
	SetBackgroundColor(0x50, 0x10, 0x10); 
	FntLoad(960, 256);
	SetDumpFnt(FntOpen(30, 30, 500, 200, 0, 128));
	for (i = 0; i < 4; i++)
	{
		InitFrame();
		ShowFrame();
		FntPrint(dirty);
		FntPrint(msg);
		FntFlush(-1);
	}
	GsSwapDispBuff();
	for (i = 0; i < 3 * 60; i++)
		VSync(0);
	getch();
	SetBackgroundColor(0, 0, 0); 
	GsSwapDispBuff();
}

#if 0
/*-------------------------------------------------------------
  ShowDiskErrorMessage ~ 
-------------------------------------------------------------*/
void ShowDiskErrorMessage(char *msg)
{
	int i, str, str2;
	
	ResetArcadeStrings();
	SetBackgroundColor(0x50, 0x10, 0x10); 
	str = CreateDbShlString("Disk is dirty or damaged", 70, 100);
	str2 = CreateDbShlString(msg, 70, 200);
	
	for (i = 0; i < 2; i++)	
	{
		InitFrame();
		DisplayAllStrings();
		ShowFrame();
	}
	getch();	
	PopString(str);
	PopString(str2);
	SetBackgroundColor(0, 0, 0); 
}  /* End ShowDiskErrorMessage() */
#endif

/*-------------------------------------------------------------
  FatalDiskError ~ 
-------------------------------------------------------------*/
void FatalDiskError(char *cfile, int line, char *fname)
{
	char msg[80];
	
	*msg = 0;
	DebugPrintf(0, "Fatal disk error %s %d\n", cfile, line);	
	if (fname)
		sprintf(msg, "%s not found   %s %d", fname, cfile, line);
	else	
		sprintf(msg, "%s #%d", cfile, line);
	ShowDiskErrorMessage(msg);
}  /* End FatalDiskError() */

/*-------------------------------------------------------------
  ResetSpinDown ~
-------------------------------------------------------------*/
void ResetSpinDown(void)
{
	register ULONG vb;

	vb = GetVBlankCount();
	gPauseCDTime = vb + PAUSE_CD_TIME;
	gStopCDTime = vb + STOP_CD_TIME;
}

/*-------------------------------------------------------------
  EnableSpinDown ~
-------------------------------------------------------------*/
void EnableSpinDown(int on)
{
	gSpinDownEnabled = on;
	ResetSpinDown();
}

/*-------------------------------------------------------------
  CheckSpinDown ~
-------------------------------------------------------------*/
void CheckSpinDown(void)
{
	register ULONG vb;
	static ULONG vbCheck = 0;

	if (gSpinDownEnabled 
	    	&& (gLoadListState == IDLE || gLoadListState == COMPLETE) )
	{
		vb = GetVBlankCount();
		if (vb > vbCheck)
		{
			vbCheck = vb + (2 * VBlanksPerSec);
			if (vb > gStopCDTime)
			{
				StopCD();
//				 DebugPrintf(0, "AutoStop\n");
			}
			else if (vb > gPauseCDTime)
			{
				PauseCD();
//				 DebugPrintf(0, "AutoPause\n");
			}
		}
	}
}

/*-------------------------------------------------------------
  InitCDHandler ~
-------------------------------------------------------------*/
void InitCDHandler(void)
{
	CdlFILE	fdat;

	CdInit();
//	CdInitFileSystem();
#if DEBUG_THIS
	CdSetDebug(DEBUG_THIS);
#else
	CdSetDebug(0);
#endif
//	ChangeDir("/");
	gBackgroundLoadState = IDLE;
	if (!SyncStatusCD())
		DebugPrintf(0, "CD Init Error\n");
//	SeekCDFile("PSX.EXE", 1, &fdat);
	ResetSpinDown();
}


/*-------------------------------------------------------------
  ChangeDir ~ Change Default directory
-------------------------------------------------------------*/
//void ChangeDir(char *dir)
//{
//	register char *ptr1, *ptr2;

//	ptr1 = dir;
//	ptr2 = gCurrentDir;
//	while (*ptr1)
//	{
//		if(*ptr1 >= 'a' & *ptr1 <= 'z')
//			*ptr2++ = toupper(*ptr1++);
//		else if (*ptr1 == '/')
//			{
//				*ptr2++ = '\\';
//				ptr1++;
//			}
//		else
//			*ptr2++ = *ptr1++;
//	}
//	if (*(ptr2-1) != '\\')
//		*ptr2++ = '\\';
//	*ptr2 = '\0';
//}

/*-------------------------------------------------------------
  FixCDFname ~ Clean up CD File name
-------------------------------------------------------------*/
void FixCDFname(char *inFname, char *outFname)
{
	register char *ptr1, *ptr2;

	ptr2 = outFname;
	if (*inFname != '/' || *inFname != '\\')
	{
		ptr1 = gCurrentDir;
		while (*ptr1)
			*ptr2++ = *ptr1++;
	}
	ptr1 = inFname;
	while (*ptr1)
	{
		if(*ptr1 >= 'a' & *ptr1 <= 'z')
			*ptr2++ = toupper(*ptr1++);
		else if (*ptr1 == '/')
			{
				*ptr2++ = '\\';
				ptr1++;
			}
		else
			*ptr2++ = *ptr1++;
	}
	*ptr2++ = ';';
	*ptr2++ = '1';
	*ptr2 = '\0';
#if DEBUG_THIS
	 DebugPrintf(0, "(%s)", outFname);
#endif
}

/*-------------------------------------------------------------
  PauseCD ~
-------------------------------------------------------------*/
void PauseCD(void)
{
	unsigned char buff[8];

//	CdControlB(CdlPause, buff, 0);
	CdControlF(CdlPause, buff);
}

/*-------------------------------------------------------------
  StopCD ~
-------------------------------------------------------------*/
void StopCD(void)
{
	unsigned char buff[8];

//	CdControlB(CdlStop, buff, 0);
	CdControlF(CdlStop, buff);
}

/*-------------------------------------------------------------
  SyncStatusCD ~ Sync with CD and check status with retrys
-------------------------------------------------------------*/
BOOL SyncStatusCD(void)
{
	UBYTE resultbuf[8] = {0};
	int result, retrys = 0;
	BOOL ret;

	do
	{
		result = CdSync(0, resultbuf);
		if (result == CdlDiskError)
		{
			retrys++;
			CdInit();
			Cerror();  // Retry warning
		}
		if (retrys < 3)
			ret = True;
		else
		{
			ret = False;
			Cerror();
		}
	} while (result == CdlDiskError && retrys < 4);
	return ret;
}

/*-------------------------------------------------------------
  SearchCDFile ~
-------------------------------------------------------------*/
BOOL SearchCDFile(char *fname, CdlFILE *cdfp)
{
	int retrys = 0;
	BOOL ret = True;
	CdlFILE	fdat, *result = NULL;
	CdlLOC	pos;
	int	mode = CdlModeSpeed;
	int	nsector;
	char fname2[MAXPATH];

	do
	{
		ResetSpinDown();
		FixCDFname(fname, fname2);
//		CdSync(0, &result);
		if ( !SyncStatusCD() )
			return False; // Error

		gBackgroundLoadState = IDLE;
		result = CdSearchFile(cdfp, fname2);
		retrys++;
	} while (result == NULL && retrys < 5);
	if (result == NULL)
	{
		DebugPrintf(0, "File not found (%s)\n", fname);
		Cerror();
//		cdfp->size = 0;
		ret = False;
		FatalDiskError(__FILE__, __LINE__, fname);
	}
	return ret;
}

/*-------------------------------------------------------------
  SeekCDFile ~ Seek to CD File
-------------------------------------------------------------*/
BOOL SeekCDFile(char *fname, int wait, CdlFILE *cdfp)
{
	BOOL result;
	CdlLOC	*pos;
//	int	mode = CdlModeSpeed;
//	int	nsector;
//	char fname2[40];

//	FixCDFname(fname, fname2);
//	CdSync(0, &result);
//	result = CdSearchFile(cdfp, fname2);
	result = SearchCDFile(fname, cdfp);
	if (result)
	{
		pos = &(cdfp->pos);
//		result = True;
		if (wait)
			CdControlB(CdlSetloc, (u_char *)pos, 0);
		else
			CdControl(CdlSetloc, (u_char *)pos, 0);

	}
	ResetSpinDown();
	return result;
}

/*-------------------------------------------------------------
  PreSeek ~ Seek to location where file is
-------------------------------------------------------------*/
BOOL PreSeek(char *fname)
{
	UBYTE result = 1;
	CdlFILE	fdat;

#if USE_CD
	result = SeekCDFile(fname, 1, &fdat);
	CdPause();
#endif
	ResetSpinDown();
	return result;
}

/*-------------------------------------------------------------
  LoadCDFile ~ Load Entire CD file
    Open File, Alloc mem, Load file, Close file
    Return pointer to File data
-------------------------------------------------------------*/
static char * LoadCDFileSub(char *fname, char *dest)
{
	UBYTE resultbuf[8] = {0};
	ULONG *ptr;
	UBYTE result, error = False;
	CdlFILE	fdat, *cdfp;
	int	mode = CdlModeSpeed;
	int	size, nsector;
	int result2;

#if USE_CD
	cdfp = &fdat;
	result = SeekCDFile(fname, 1, cdfp);
	if (!result)
		return(NULL);   // error

	nsector = (cdfp->size+2047) / 2048;
	if (nsector > MAXSECTOR)
		nsector = MAXSECTOR;
	size = (nsector) * SIZE_1SECTOR;
	size += 4;  /* for safety */
	if (dest == NULL)
		ptr = (ULONG*)AllocMem(size);
	else
		ptr = (ULONG *)dest;
#if DEBUG_THIS
	 DebugPrintf(0, " %d Sectors %d bytes\n", nsector, size);
#endif
	if ( !CdRead(nsector, ptr, mode) )
	{
		Cerror();
		error = True;
	}
	result2 = CdReadSync(0, resultbuf);
	if (result2 != 0)
	{
		Cerror();
		error = True;
	}

#if DEBUG_THIS
	 DebugPrintf(0, " result: %d\n ", result);
	 DumpMemory(ptr, 2, 1);
#endif

#else // (not USE_CD)
	ptr = GetMultiFilePtr(FAKE_CD, fname);
#endif
	if (error && (dest == NULL))
	{
		FreeMem(ptr);
		ptr = NULL;
	}
	ResetSpinDown();
	CdPause();
	return (char *)ptr;
}

/*-------------------------------------------------------------
  LoadCDFileTo ~ Load Entire CD file   (With Retrys)
    Open File, Load file, Close file
-------------------------------------------------------------*/
char * LoadCDFileTo(char *fname, char *dest)
{
	int retrys;
	char *ptr = NULL;

	retrys = 0;
	do
	{
		ptr = LoadCDFileSub(fname, dest);
		if (ptr == NULL)
		{
			retrys++;
			CdInit();
			Cmarker();
			if (retrys == 8)
			{
				FatalDiskError(__FILE__, __LINE__, fname);
			}
		}
	} while (ptr == NULL && retrys < 12);
//	if (ptr == NULL)
//		return 0;
//	else
//		return 1;
	return ptr;
}

/*-------------------------------------------------------------
  LoadCDFile ~ Load Entire CD file   (With Retrys)
    Open File, Alloc mem, Load file, Close file
    Return pointer to File data
-------------------------------------------------------------*/
char * LoadCDFile(char *fname)
{
	int retrys = 0;
	char *ptr = NULL;

	ptr = LoadCDFileTo(fname, NULL);
	return ptr;

#if DEBUG_MALLOC
	 DebugPrintf(0, "Loading %s\n", fname);
#endif
}

/*-------------------------------------------------------------
  FreeFileData ~ Free memory allocated by LoadCDFile
-------------------------------------------------------------*/
BOOL FreeFileData(char *mem)
{
#if USE_CD
	FreeMem(mem);
#endif
}

/*******************************************************************
	Fast Sector Direct routines
	  Christopher Deel

		Combined Sector size info (sectorID)
			High byte: 		(Number of sectors in file) /2
			Low 3 bytes:	Starting Sector #

*******************************************************************/

/*-------------------------------------------------------------
  CalcSectorID ~
		sector = starting sector
		length = length in SECTORS
-------------------------------------------------------------*/
ULONG CalcSectorID(int sector, int length)
{
	ULONG id;

//	length /= 2;
	if (length > MAXSECTOR)
		length = 0;
	id = (length & 0xFF) << 24;
	id = id | (sector & SECTOR_MASK);
	return id;
}

/*-------------------------------------------------------------
  GetFileSector ~ Get the starting sector & size of a file
    return Sector ID
-------------------------------------------------------------*/
ULONG GetFileSector(char *fname)
{
	CdlFILE fdat;
	int sector, length, result;
	ULONG id = 0;

	result = SearchCDFile(fname, &fdat);
	if (result)
	{
		length = (fdat.size+2047) / 2048;
		sector = CdPosToInt(&(fdat.pos));
		id = CalcSectorID(sector, length);
	}
	ResetSpinDown();
	return id;
}

/*-------------------------------------------------------------
  SeekSector ~ Seek to Sector
-------------------------------------------------------------*/
BOOL SeekSector(ULONG sector, BOOL wait)
{
	CdlLOC pos;
	unsigned char result[8];

	sector = sector & SECTOR_MASK;
	CdIntToPos(sector, &pos);
	if (wait)
		CdControlB(CdlSetloc, (u_char *)&pos, result);
	else
	{
//		CdControl(CdlSetloc, (u_char *)&pos, 0);
		CdControlF(CdlSetloc, (u_char *)&pos);
	}
	ResetSpinDown();
}

/*-------------------------------------------------------------
  ReadNextSectors ~ Read # of sectors into RAM
-------------------------------------------------------------*/
int ReadNextSectors(int sectors, ULONG *dest, int wait)
{
	UBYTE resultbuf[8] = {0};
	int result = 0;

	CdRead(sectors, dest, CdlModeSpeed);
	if (wait)
	{
		result = CdReadSync(0, resultbuf);
		CdPause();
	}
	ResetSpinDown();
	return result;
}

/*-------------------------------------------------------------
  ReadCDSectors ~ Read sectors from CD at specific place
  		return number of sectors read
-------------------------------------------------------------*/
int ReadCDSectors(ULONG sector, int num, ULONG *dest)
{
	int result, retrys = 0;
	
//	do 
//	{
		SeekSector(sector, 1);
		ReadNextSectors(num, dest, 1);
//	} while 
}

/*------------------------------------------------------------------
  LoadFileSectors ~ Read all of file into RAM using Sector access
    Open File, Alloc mem, Load file, Close file
    Return pointer to File data
------------------------------------------------------------------*/
char *LoadFileSectors(ULONG sectorID)
{
	int length, result, bytes;
	ULONG *ptr;

	ResetSpinDown();
	length = MGetSectorIDSize(sectorID);
	bytes = length * SIZE_1SECTOR;
	ptr = (ULONG*)AllocMem(bytes);
	ReadCDSectors(sectorID, length, ptr);
	return (char *)ptr;
}

/*------------------------------------------------------------------
  LoadFileSectorsTo ~ Read all of file into RAM using Sector access
    Open File, Load file, Close file
------------------------------------------------------------------*/
int LoadFileSectorsTo(ULONG sectorID, char *dest)
{
	int length, result, bytes;
	ULONG *ptr;

	ResetSpinDown();
	ptr = (ULONG*)dest;
	length = MGetSectorIDSize(sectorID);
	ReadCDSectors(sectorID, length, ptr);
	return True;
}

/********************* Background load routines **********************/
/*-------------------------------------------------------------
  IsBackgroundLoading ~
-------------------------------------------------------------*/
int IsBackgroundLoading(void)
{
	int retval;
	static int completeCount = 0;
	
	retval = TRUE;
	if (gLoadListState == COMPLETE)  // safety Net
	{
		if (completeCount++ > 20)
		{
		    retval = FALSE;
		    completeCount = 0;
		}
	}
	else
	{
		completeCount = 0;
	}  //End safety Net
	
	
	if (gLoadListState == IDLE)
	{
		retval = FALSE;	
	}
//	return gLoadListState;
	return retval;
}

/*-------------------------------------------------------------
  StartBackgroundLoad ~ Start load of background file
-------------------------------------------------------------*/
int StartBackgroundLoad(int sectorID, char *dest)
{
	ResetSpinDown();
	gBackgroundSectorID = sectorID;
	gBackgroundLoadDest = dest;
	gBackgroundLoadState = STARTING_SEEK;
}

/*-------------------------------------------------------------
  CheckBackgroundLoad ~ Check to see if background load
  		is finished. Must be called peridocally.
  		return 0=Not finished, 1=Finished, -1=error
-------------------------------------------------------------*/
int CheckBackgroundLoad(void)
{
	UBYTE resultbuf[8] = {0};
	int ret;
	int status;
	int length;
	static int retrys = 0;

	ret = 0;
	switch (gBackgroundLoadState)
	{
		case IDLE:
			break;
		case STARTING_SEEK:
			SeekSector(gBackgroundSectorID, 0);
			gBackgroundLoadState = SEEKING;
			ResetSpinDown();
			break;
		case SEEKING:
			status = CdSync(1, resultbuf);
			if (status == CdlDiskError)
		    {
				gBackgroundLoadState = STARTING_SEEK;
		    	retrys++;
		    }
			else if (status == CdlComplete)
			{
//				retrys = 0;
				length = MGetSectorIDSectors(gBackgroundSectorID);
				ReadNextSectors(length, (ULONG *)gBackgroundLoadDest, 0);
				gBackgroundLoadState = READING;
				ResetSpinDown();
			}
			break;
		case READING:
			status = CdReadSync(1, resultbuf);
			if (status == -1)
		    {
				gBackgroundLoadState = STARTING_SEEK;
		    	retrys++;
		    }
			else if (status == 0)
			{
				ResetSpinDown();
				ret = 1;
				retrys = 0;
			}
			break;
		case ERROR:
			ret = -1;
			break;
	}
	if (retrys == 10)
	{
		FatalDiskError(__FILE__, __LINE__, NULL);
		if (retrys > 16)
		{
			ret = -1;
			retrys = 0;
		}
	}
	return ret;
}

// ***************** Background Load list routines ****************

/*-------------------------------------------------------------
  ClearLoadList ~ Clear the backgournd load list
-------------------------------------------------------------*/
void ClearLoadList(void)
{
	int i;

	for (i= 0; i <= MAX_LOAD_LIST; i++)
	{
		gLoadList[i].sector = 0;
	}
	gLoadListIn = 0;
	gLoadListOut = 0;
	gLoadListState = IDLE;
}

/*-------------------------------------------------------------
  AddLoadList ~ Add Sector id to load list
-------------------------------------------------------------*/
int AddLoadList(ULONG sector, char *dest)
{
	int ret;

	if (gLoadListIn < MAX_LOAD_LIST)
	{
		gLoadList[gLoadListIn].sector = sector;
		gLoadList[gLoadListIn].dest = dest;
		gLoadListIn++;
		gLoadList[gLoadListIn].sector = 0; // Mark end
		ret = True;
	}
	else
	{
		Cerror();  // Too many files in list
	}
	ret = False;
	return ret;
}

/*-------------------------------------------------------------
  AllocAddLoadList ~ Allocate memory and add File to load list
-------------------------------------------------------------*/
char * AllocAddLoadList(ULONG sector)
{
	int length, bytes;
	char *ptr;

	length = MGetSectorIDSize(sector);
	bytes = length * SIZE_1SECTOR;
	ptr = (char *)AllocMem(bytes);
	AddLoadList(sector, ptr);
//	 DebugPrintf(0, "AllocAddLoadList %X %X %X\n", sector, ptr, bytes);
	return ptr;
}

/*-------------------------------------------------------------
  NextLoadListFile ~ Start next file in load list
-------------------------------------------------------------*/
static int NextLoadListFile(void)
{
	ResetSpinDown();
	StartBackgroundLoad(gLoadList[gLoadListOut].sector,
						gLoadList[gLoadListOut].dest);
	gLoadListOut++;
}

/*-------------------------------------------------------------
  StartLoadList ~ Start loading load list in background
-------------------------------------------------------------*/
void StartLoadList(void)
{
	gLoadListOut = 0;
	NextLoadListFile();
	gLoadListState = READING;
}

/*-------------------------------------------------------------
  CheckLoadList ~ Check to see if load list is finished
  		Must be called peridocally.
  		return 0=Not finished, 1=Finished, -1=error
-------------------------------------------------------------*/
int CheckLoadList(void)
{
	int ret,stat;

	switch (gLoadListState)	
	{
		case IDLE:
//			return 0;
			ret = 0;
			break;
			
		case COMPLETE:
		//	return 1;  // finished
			ret = 1;  // finished
			break;
			
//		case READING:
		default:
			ret = 0;
			stat = CheckBackgroundLoad();
			if (stat == 1)
			{
				if (gLoadList[gLoadListOut].sector)
					NextLoadListFile();
				else
				{
					ret = 1;  // finished
					gLoadListState = COMPLETE;
				}
			}
			else if (stat < 0)
			{
				Cerror();	// background load error
				ret = -1; // error
			}
			break;
	}
	return ret;
}

// ***************** Test routines ***********************
#if TEST_THIS

/*-------------------------------------------------------------
  TestBackgroundCD ~
-------------------------------------------------------------*/
void TestBackgroundCD(void)
{
    char *fname = "/anim/anims.mfl";
    char *buff;
    int stat, stat2, tm2,tm1, i;
    ULONG sector, bytes;

//    PreSeek("/psx.exe");
	 Cmarker();
	 InitTimeMarks();
    sector = GetFileSector(fname);
    bytes = GetSectorIDBytes(sector);
	MarkTime();
    buff = AllocMem(bytes);
	DebugPrintf(0, "S=%X  %d bytes  to %X\n", sector, bytes, buff);
	PreSeek("/tmp/control.tim");
	MarkTime();
    StartBackgroundLoad(sector, buff);
   	stat2 = gBackgroundLoadState;
	MarkTime();
   	stat = CheckBackgroundLoad();
//	DebugPrintf(0, "(%d %d)  \n", stat, stat2);
	i = 0;
    while (stat == 0)
    {
    	i++;
    	stat = CheckBackgroundLoad();
    	if (gBackgroundLoadState != stat2)
    		MarkTime();
    	stat2 = gBackgroundLoadState;
//    	DebugPrintf(0, "(%d %d)   ", stat, gBackgroundLoadState);
    }
	MarkTime();
	DumpTimeMarks();
    DebugPrintf(0, "\n i:%d  ", i);
	FreeMem(buff);
}

/*-------------------------------------------------------------
  TestBackgroundCD ~
-------------------------------------------------------------*/
void TestLoadList(void)
{
    char *fname[5] = {
		"/tmp/control.tim",
		"/tmp/credits.tim",
		"/tmp/psxlogo.tim",
		"/tmp/control.tim",
		NULL };
    char *buff[5];
    int stat, stat2, tm2,tm1, i;
    ULONG sector, bytes;

//    PreSeek("/psx.exe");
	 Cmarker();
	 InitTimeMarks();
	ClearLoadList();
	for (i = 0; i < 4; i++)
	{
	    sector = GetFileSector(fname[i]);
	    bytes = GetSectorIDBytes(sector);
	    buff[i] = AllocMem(bytes);
	    AddLoadList(sector, buff[i]);
	}
	MarkTime();
//	DebugPrintf(0, "S=%X  %d bytes  to %X\n", sector, bytes, buff);
//	PreSeek("/tmp/control.tim");
	MarkTime();
    StartLoadList();
   	stat2 = gBackgroundLoadState;
	MarkTime();
   	stat = CheckLoadList();
	DebugPrintf(0, "(%d %d)  \n", stat, stat2);
	i = 0;
    while (stat == 0)
    {
    	i++;
    	stat = CheckLoadList();
    	if (gBackgroundLoadState != stat2)
    		MarkTime();
    	stat2 = gBackgroundLoadState;
    	DebugPrintf(0, "(%d %d)   ", stat, gBackgroundLoadState);
    }
	MarkTime();
	DumpTimeMarks();
    DebugPrintf(0, "\n i:%d  ", i);
    for (i = 3; i >= 0; i--)
    	DumpMemory(buff[i], 2, 1);
    for (i = 3; i >= 0; i--)
    {
		FreeMem(buff[i]);
    }
}

#endif
