/***************************************************************
	CD File handler routines

	Copyright 1995 GameTek
	All rights reserved.

	By Christopher G. Deel 
****************************************************************/

#ifndef CdlFILE
#include <libcd.h>
#endif


// Like PC 
#define MAXPATH   80
#define MAXDIR    66
#define MAXFILE   9
#define MAXEXT    5

#define SIZE_1SECTOR	2048
#define SIZE_2SECTOR	(SIZE_1SECTOR * 2)
#define MAXSECTOR	256	

#define SECTOR_MASK 0xFFffFF
#define MGetSectorIDSize(id) ((id >> 24) & 0xFF)
#define MGetSectorIDSectors(id) (MGetSectorIDSize(id))
#define GetSectorIDBytes(id) (MGetSectorIDSectors(id) * SIZE_1SECTOR)
					 
/*-------------------------------------------------------------
  LoadCDFile ~ Load Entire CD file
    Open File, Alloc mem, Load file, Close file
    Return pointer to File data
-------------------------------------------------------------*/

/*-------------------------------------------------------------
  FreeFileData ~ Free memory allocated by LoadCDFile
-------------------------------------------------------------*/
/*******************************************************************
	Fast Sector Direct routines
	  Christopher Deel
			
		Combined Sector size info (sectorID)			
			High byte: 		Number of sectors in file
			Low 3 bytes:	Starting Sector #

*******************************************************************/

/*-------------------------------------------------------------
  GetFileSector ~ Get the starting sector & size of a file
-------------------------------------------------------------*/

/*-------------------------------------------------------------
  SeekSector ~ Seek to Sector
-------------------------------------------------------------*/

/*-------------------------------------------------------------
  ReadNextSectors ~ Read # of sectors into RAM
-------------------------------------------------------------*/

/*-------------------------------------------------------------
  ReadCDSectors ~ Read sectors from CD at specific place
  		return number of sectors read
-------------------------------------------------------------*/

/*-------------------------------------------------------------
  LoadFileSectors ~ Read all of file into RAM using Sector access
    Open File, Alloc mem, Load file, Close file
    Return pointer to File data
-------------------------------------------------------------*/


/* cdhandlr.c */
void ResetSpinDown ( void );
void EnableSpinDown ( int on );
void CheckSpinDown ( void );
void InitCDHandler ( void );
void ChangeDir ( char *dir );
void FixCDFname ( char *inFname , char *outFname );
void PauseCD ( void );
void StopCD ( void );

BOOL SyncStatusCD ( void );
BOOL SearchCDFile ( char *fname , CdlFILE *cdfp );
BOOL SeekCDFile ( char *fname , int wait , CdlFILE *cdfp );
BOOL PreSeek ( char *fname );
char *LoadCDFile ( char *fname );
BOOL FreeFileData ( char *mem );
ULONG CalcSectorID ( int sector , int length );
ULONG GetFileSector ( char *fname );
BOOL SeekSector ( ULONG sector , BOOL wait );
int ReadNextSectors ( int sectors , ULONG *dest , int wait );
int ReadCDSectors ( ULONG sector , int num , ULONG *dest );
char *LoadFileSectors ( ULONG sectorID );
int LoadFileSectorsTo(ULONG sectorID, char *dest);
int StartBackgroundLoad ( int sectorID , char *dest );
int CheckBackgroundLoad ( void );
//*****************Background Load list routines ****************
void ClearLoadList ( void );
int AddLoadList ( ULONG sector , char *dest );
char *AllocAddLoadList ( ULONG sector );
void StartLoadList ( void );
int CheckLoadList ( void );

int IsBackgroundLoading(void);
