/****************************************************************************/
#define TITLE		"Coin Toss Test  "
                             
/****************************************************************************/

#include <sys/types.h>

#include <libetc.h>			   /* For Controller routines */
#include <libgte.h>			   /* For LIBGS */
#include <libgpu.h>			   /* For LIBGS */
#include <libgs.h>			   /* Graphics Library */
#include <libsn.h>


#include "prjtypes.h"
#include "prjhdrs.h"    /* Project header files */
#include "camera.h"
#include "gameloop.h"
#include "arcade.h"
#include "coinlog.h"
#include "coingraf.h"
#include "deel.h"
#include "teamstad.h"
//#include "logowide.h"

#include "fbequ.h"
#include "fbram.h"

//#define DEBUG_THIS 1

//#define NO_COIN_GRAPHIC 1

#define COIN_HEADS_FRAME 8
#define COIN_TAILS_FRAME 3

#define COIN_SCREEN_WIDTH  128
#define COIN_SCREEN_HEIGHT 80

#define CPU_MENU_TIME  (5 * (VBlanksPerSec/2))
#define COIN_FLIP_TIME  (3 * VBlanksPerSec)

ULONG gCoinAnimSector = NULL;
char *gCoinTossFile = NULL;

SubTexture_t gCoinTex;
PolySpritePtr gCoinSprite;
int gCoinFrame;
char *gCoinBuff = NULL;

static ULONG gCpuMenuTime = 0;

int CoinForceValue = 0;
int gCoinPad = 0;
SVECTOR gSpiralCameraOffset = {0, -700, 8800};
SVECTOR gSpiralCameraOffsetDome = {0, -2200, 7800};
SVECTOR gCoinCameraOffset = {-2000, -1900, 0};

#if DEBUG_THIS
	void DumpCoinState(void);
#else
	#define DumpCoinState()  /*nothing*/
#endif /* DEBUG_THIS */


enum CoinMenuFlipEnum
{
//	COIN_MENU_FLIP_TEAM,
	COIN_MENU_FLIP_HEADS,
	COIN_MENU_FLIP_TAILS,
	COIN_MENU_FLIP_MAX,
};

#define COIN_MENU_FLIP_WIDTH  160
#define COIN_MENU_FLIP_HEIGHT 65
#define COIN_MENU_FLIP_X  (255 - (COIN_MENU_FLIP_WIDTH/2))
#define COIN_MENU_FLIP_Y 105

MenuText_t gCoinFlipText[COIN_MENU_FLIP_MAX + 1] =
	{
//		"TEAM                 ", 1,COIN_MENU_FLIP_X +10,
//									COIN_MENU_FLIP_Y +10, 0,
		"HEADS", 1,COIN_MENU_FLIP_X +15,
									COIN_MENU_FLIP_Y +40, 0,
		"TAILS", 1,COIN_MENU_FLIP_X +15,
									COIN_MENU_FLIP_Y +40+(1*12), 0,
		NULL,0,0,0,0
	};

enum CoinMenuWinEnum
{
//	COIN_MENU_WIN_TEAM,
	COIN_MENU_WIN_KICK,
	COIN_MENU_WIN_RECEIVE,
	COIN_MENU_WIN_SIDE,
	COIN_MENU_WIN_MAX,
};

#define COIN_MENU_WIN_WIDTH  160
#define COIN_MENU_WIN_HEIGHT 80
#define COIN_MENU_WIN_X  (255 - (COIN_MENU_WIN_WIDTH/2))
#define COIN_MENU_WIN_Y 105

MenuText_t gCoinWinText[COIN_MENU_WIN_MAX + 1] =
	{
//		"TEAM                 ", 1,COIN_MENU_WIN_X +10,
//									COIN_MENU_WIN_Y +10, 0,
		"KICK", 1,COIN_MENU_WIN_X +10,
									COIN_MENU_WIN_Y +40, 0,
		"RECEIVE", 1,COIN_MENU_WIN_X +10,
									COIN_MENU_WIN_Y +40+(1*12), 0,
		"CHOOSE SIDE", 1,COIN_MENU_WIN_X +10,
									COIN_MENU_WIN_Y +40+(2*12), 0,
		NULL,0,0,0,0
	};

	
enum CoinMenuKickEnum
{
//	COIN_MENU_KICK_TEAM,
	COIN_MENU_KICK_KICK,
	COIN_MENU_KICK_RECEIVE,
	COIN_MENU_KICK_MAX,
};

#define COIN_MENU_KICK_WIDTH  160
#define COIN_MENU_KICK_HEIGHT 70
#define COIN_MENU_KICK_X  (255 - (COIN_MENU_KICK_WIDTH/2))
#define COIN_MENU_KICK_Y 105

MenuText_t gCoinKickText[COIN_MENU_KICK_MAX + 1] =
	{
//		"TEAM                 ", 1,COIN_MENU_KICK_X +10,
//									COIN_MENU_KICK_Y +10, 0,
		"KICK          ", 1,COIN_MENU_KICK_X +10,
									COIN_MENU_KICK_Y +40, 0,
		"RECEIVE      ", 1,COIN_MENU_KICK_X +10,
									COIN_MENU_KICK_Y +40+(1*12), 0,
		NULL,0,0,0,0
	};
	
		
enum CoinMenuSideEnum
{
//	COIN_MENU_SIDE_TEAM,
	COIN_MENU_SIDE_EAST,
	COIN_MENU_SIDE_WEST,
	COIN_MENU_SIDE_MAX,
};

#define COIN_MENU_SIDE_WIDTH  160
#define COIN_MENU_SIDE_HEIGHT 70
#define COIN_MENU_SIDE_X  (255 - (COIN_MENU_SIDE_WIDTH/2))
#define COIN_MENU_SIDE_Y 105

MenuText_t gCoinSideText[COIN_MENU_SIDE_MAX + 1] =
	{
//		"TEAM                 ", 1,COIN_MENU_SIDE_X +10,
//									COIN_MENU_SIDE_Y +10, 0,
		"SOUTH EASTxxx", 1,COIN_MENU_SIDE_X +10,
									COIN_MENU_SIDE_Y +40, 0,
		"SOUTH WESTxxx", 1,COIN_MENU_SIDE_X +20,
									COIN_MENU_SIDE_Y +40+(1*12), 0,
		NULL,0,0,0,0
	};

struct 
{
	char *left;
	char *right;
} gDirectionNames[] =
	{
	    {"NORTH", "SOUTH"},
	    {"NORTH WEST", "SOUTH EAST"},
	    {"WEST", "EAST"},
	    {"SOUTH WEST", "NORTH EAST"},
	};
	


/*-------------------------------------------------------------
  LoadCoinGraphics ~ 
-------------------------------------------------------------*/
void LoadCoinGraphic(void)
{
	
}


/*-------------------------------------------------------------
  InitCoinTossCamera ~ 
-------------------------------------------------------------*/
void InitCoinTossCamera(void)
{
	register int team;
	SVECTOR vec;
	
//	team = GameTeamData[0].TeamNumber; // home team
	gGameLoopFlags2.WasCoinToss = TRUE;
	DebugPrintf(0, "Init Coin Dome=%d\n", gDomeStadium);
	if (gDomeStadium)
	{
		SetCameraOffsetVector(&gSpiralCameraOffsetDome);
	}
	else
	{
		SetCameraOffsetVector(&gSpiralCameraOffset);
	}
	TranslateAIVector(MID_FIELDX, MID_FIELDY, 25 * YARD, &vec);
//	SetCameraReference(vec.vx, vec.vy, vec.vz);
	SetCameraReference(vec.vx, -700, vec.vz);
	CalcCameraRadius();
	CalcCameraAngle();
//	SetCameraAngle(DegreesToPsxAngle(0));
	if (CoinOvertimeFlag)
	{
		SetCameraOffsetVector(&gCoinCameraOffset);
		SetCameraReference(vec.vx, 0, vec.vz);
		MoveCameraInstant();
	}
	else
	{
		MoveCameraAngle(0);
	}
}

/*-------------------------------------------------------------
  InitOvertimeGraphics ~ 
-------------------------------------------------------------*/
void InitOvertimeGraphics(void)
{
	 Cmarker();
	FreeArcadeTemp();
	ResetArcadeStrings();
	TemplateBuff = NULL;
	gPermTexLoaded = FALSE;
	
	initVRamHandling(); 
	InitGameLoop();
	UpdateAllScoreboard();
//	LoadStadiumTims(1);
//	InitStadiumTextures();
	DrawSync(0);
	SyncFlag = FALSE;	// for arcade timing
//	DoAI();
	TranslateAI();
	ResetArcadeStrings();
	
}  /* End InitOvertimeGraphics() */

/*-------------------------------------------------------------
  DoSpiralCameraIntro ~ 
-------------------------------------------------------------*/
int DoSpiralCameraIntro(int reset)
{
	int done, in, refDown, offUp, team;
	static int count = 0, freeze = FALSE;
	int targetAngle;
	
//	 return TRUE;  // for debug
	if (reset)
	{
		count = 0;
		return;
	}
	
	targetAngle = DegreesToPsxAngle(270);
	done = FALSE;
	
	team = GameTeamData[0].TeamNumber; // home team
	if (gDomeStadium)
	{
		in = 50;
		refDown = 5;
		offUp = 15;
	}
	else	
	{
		in = 60;
		refDown = 5;
		offUp = 15;
	}
	
	if (count == 0)
	{
		SetGameBackground(0);
		InitCoinTossCamera();
		SetCameraMax(31000, -200, 31000);
		SetCameraMin(-31000,-2000, -31000);
		gCamera.viewMin.vy = -2000;
		if (!CoinOvertimeFlag && GameType != CHAMPIONSHIP_GAME)
			PlayPregame();     // Play welcome... sound
		freeze = FALSE;
		 DumpCamera();
	}
    else
    {
	    if (!freeze)
	    {
		    MoveCameraAngle(42);
		}
		MoveCameraInstant();
	    if (!freeze && count > 80)
	    {
		    gCamera.radius -= in;
		    gCamera.ref.vy += refDown;
		    gCamera.offsetTarget.vy -= offUp;
//			SetCameraOffset(gCamera.offsetTarget.vx, 
//					-((gCamera.radius * 8) / 47), gCamera.offsetTarget.vz);
		}
		limitRange(gCamera.radius, 1100, 8800);
		gCamera.angle %= DegreesToPsxAngle(360);
	}
	
    if (count > 96 && (gCamera.angle < targetAngle)
        			&& (gCamera.angle > targetAngle - 42) )
    {
    	SetCameraAngle(targetAngle);
    	gCamera.offset.vz = 0;
    	gCamera.offsetTarget.vz = 0;
    	if (!freeze)
    	{
	    	freeze = TRUE;
    	}
    }
	
    count++;
	if (count > 200)
   	{
//		SetCameraAngle(targetAngle);
//		gCamera.offset.vz = 0;
   		done = TRUE;
		freeze = FALSE;
   		count = 0;
		FreePregame();
    		 
   		 DebugPrintf(0, "angle %d %d\n", gCamera.angle, 
    	 		PsxAngleToDegrees(gCamera.angle));
   	}
   	if (CoinOvertimeFlag)
   	{
   		count += 3;
   		count = 200;
		SetCameraOffsetVector(&gCoinCameraOffset);
	    gCamera.ref.vy = 0;
	    
   	}
	return done;
}

/*-------------------------------------------------------------
  SetCoinFrame ~ 
-------------------------------------------------------------*/
void UpdateCoinFrame(void)
{
	char name[14];
	char *ptr;

#if NO_COIN_GRAPHIC
	 return;
#endif	 
//	 if (gCoinFrame == 0)
//	 	DumpMemory(gCoinTossFile, 10, 1);
	sprintf(name, "coin%02d.cmp", gCoinFrame);
	ptr = (char*)GetMultiFilePtr(gCoinTossFile, name);
	DecompressFrameBytes(gCoinBuff, ptr);
	LoadSubTexturePixels((ULONG*)gCoinBuff, &gCoinTex);
}

/*-------------------------------------------------------------
  InitCoinGraphics ~ 
-------------------------------------------------------------*/
void InitCoinGraphic(void)
{
	vram_t *texture, *clut;
	char *clutPtr;
	
#if NO_COIN_GRAPHIC
	 return;	 
#endif	 
	gCoinBuff = (char*)AllocMem(128*64);
	texture = gStadTexture[11].texture;
	clutPtr = (char*)GetMultiFilePtr(gCoinTossFile, "coin.clt");
	DrawSync(0);
	clut = register_CLUT((ULONG*)clutPtr, VRAM_CLUT16);
	InitSubTexture(&gCoinTex, texture, clut, 128,128, 128,128, 0, 0);
	gCoinSprite = CreatePolySprite(COIN_SCREEN_WIDTH, COIN_SCREEN_HEIGHT);
	MakePolySprite2d(gCoinSprite);
	MovePolySpriteTo(-210,-90,5, gCoinSprite);
	SetPolySpriteSubTexture(&gCoinTex, gCoinSprite);
	gCoinFrame = 0;
	UpdateCoinFrame();
	DrawSync(0);
}

/*-------------------------------------------------------------
  FreeCoinGraphics ~ 
-------------------------------------------------------------*/
void FreeCoinGraphics(void)
{
#if NO_COIN_GRAPHIC
	 return;	 
#endif	 
	if (gCoinBuff)
	{
		DeletePolySprite(gCoinSprite);
		gCoinSprite = NULL;
		FreeMem(gCoinBuff);
		gCoinBuff = NULL;
	}
}
	
/*-------------------------------------------------------------
  DoCoinPickFlipMenu ~ 
   on: 1=Show & Update,  0=Hide
   team: 0=home, 1=visitor
   return: CHOSE_NONE, CHOSE_HEADS, CHOSE_TAILS 
-------------------------------------------------------------*/
int DoCoinPickFlipMenu(int on, int team)
{
	static int active = 0;
	static int box = -1;
	int retval, cmd;
	RECT rec;
	 ULONG db;  //debug
	 	
	cmd = ARCADE_MENU_NONE;  // no choice made
	if (on)
	{
		if (!active)  // Start Menu
		{
			 DumpCoinState();
			active = TRUE;
//			InitCoinGraphic();
//			gCoinFrame = COIN_HEADS_FRAME;
//			UpdateCoinFrame();
			gMenuInfo.selection = COIN_MENU_FLIP_HEADS;
			gMenuInfo.team = team;
			gCoinPad = GameTeamData[team].ControllerNum[0];
			PlayCoinToss(HEADS_TAILS, 1);	// Play speach
			if (gCoinPad < 0)
			{
				gMenuInfo.selection = PickCpuCoinFlip() -1;
				gCpuMenuTime = GetVBlankCount() + CPU_MENU_TIME;
				 Cmarker();
			}
//			sprintf(gCoinFlipText[COIN_MENU_FLIP_TEAM].text, "%s", 
//				SpecificTeamData[GameTeamData[team].TeamNumber].TeamCity);
			SetTeamNameSpriteOn(team, TRUE, -1, COIN_MENU_FLIP_Y+10);
//			MovePolySpriteTo(-GetTeamNameWidth(team)/2, COIN_MENU_FLIP_Y+10-115,
//						0, TeamNameSprite[team]);
			
//			MovePolySpriteTo(COIN_MENU_FLIP_X+10-255, COIN_MENU_FLIP_Y+10-115,
//						0, TeamNameSprite[team]);

			StartMenuText(gCoinFlipText, COIN_MENU_FLIP_HEADS);
//			setRECT(&rec, COIN_MENU_FLIP_X, COIN_MENU_FLIP_Y, 
//					COIN_MENU_FLIP_WIDTH, COIN_MENU_FLIP_HEIGHT);
//			ShowMenuBorder(TRUE, &rec, 0); 
			box = ShowArcadeMenuBox(COIN_MENU_FLIP_X, COIN_MENU_FLIP_Y, 
					COIN_MENU_FLIP_WIDTH, COIN_MENU_FLIP_HEIGHT, -1);
			MoveMenuSelect(0);
		}
		else		// Do Menu
		{
			if (gCoinPad < 0) // CPU
			{
				MoveMenuSelect(0);
				
				if (GetVBlankCount() > gCpuMenuTime )
				{
					cmd = MoveMenuSelect(0);
				}
			}
			else
				cmd = DoMenuPad(gCoinPad);
		}
	}
	else  // Erase the menu
	{
		active = FALSE;
		EndMenuText();
//		ShowMenuBorder(FALSE, NULL, 0);
		FreeDialogBox(box);
		SetTeamNameSpriteOn(0, FALSE, 0,0);
		SetTeamNameSpriteOn(1, FALSE, 0,0);
		 Cmarker();
		 DumpCoinState();
	}
	if (cmd == COIN_MENU_FLIP_HEADS)
		retval = CHOSE_HEADS;
	else if (cmd == COIN_MENU_FLIP_TAILS)
		retval = CHOSE_TAILS;
	else 
		retval = CHOSE_NONE;
	return retval;
}

/*-------------------------------------------------------------
  DoCoinPickFirstMenu ~ Winner of flip
   Receiver, Kick, Which side
   on: 1=Show & Update,  0=Hide
   team: 0=home, 1=visitor
   return: CHOSE_NONE, CHOSE_KICK, CHOSE_RECEIVE, CHOSE_SIDE 
-------------------------------------------------------------*/
int DoCoinPickFirstMenu(int on, int team)
{
	static int active = 0, box = -1;
	int retval;
	int cmd;
	RECT rec;
	
	cmd = ARCADE_MENU_NONE;  // no choice made
	if (on)
	{
		if (!active)
		{
			 DumpCoinState();
			active = TRUE;
			gMenuInfo.selection = COIN_MENU_WIN_RECEIVE;
			gMenuInfo.team = team;
			gCoinPad = GameTeamData[team].ControllerNum[0];
			if (gCoinPad < 0)
			{
				gMenuInfo.selection = PickCpuCoinWin() -1;
				gCpuMenuTime = GetVBlankCount() + CPU_MENU_TIME;
				 Cmarker();
			}
			SetTeamNameSpriteOn(team, TRUE, -1, COIN_MENU_WIN_Y+10);
		
			StartMenuText(gCoinWinText, COIN_MENU_WIN_KICK);
//			setRECT(&rec, COIN_MENU_WIN_X, COIN_MENU_WIN_Y, 
//					COIN_MENU_WIN_WIDTH, COIN_MENU_WIN_HEIGHT);
//			ShowMenuBorder(TRUE, &rec, 0); 
			box = ShowArcadeMenuBox(COIN_MENU_WIN_X, COIN_MENU_WIN_Y, 
					COIN_MENU_WIN_WIDTH, COIN_MENU_WIN_HEIGHT, -1);
			MoveMenuSelect(0);
			PlayCoinToss(CHOOSE, team);	// Play speach
		}
		else		// Do Menu
		{
			if (gCoinPad < 0) // CPU
			{
				MoveMenuSelect(0);
				if (GetVBlankCount() > gCpuMenuTime )
				{
					cmd = MoveMenuSelect(0);
				}
			}
			else
				cmd = DoMenuPad(gCoinPad);
		}
	}
	else
	{
//		 DumpCoinState();
		active = FALSE;
		SetTeamNameSpriteOn(0, FALSE ,0,0);
		SetTeamNameSpriteOn(1, FALSE ,0,0);
		EndMenuText();
//		ShowMenuBorder(FALSE, NULL, 0);
		FreeDialogBox(box);
//		gCpuMenuTime = GetVBlankCount() + CPU_MENU_TIME;
		FreeCoinGraphics();
		 Cmarker();
	}
	if (cmd == COIN_MENU_WIN_KICK)
		retval = CHOSE_KICK;
	else if (cmd == COIN_MENU_WIN_RECEIVE)
		retval = CHOSE_RECEIVE;
	else if (cmd == COIN_MENU_WIN_SIDE)
		retval = CHOSE_SIDE;
	else 
		retval = CHOSE_NONE;
	return retval;
}


/*-------------------------------------------------------------
  DoCoinPickSideMenu ~ 
   on: 1=Show & Update,  0=Hide
   team: 0=home, 1=visitor
   return: CHOSE_NONE, CHOSE_EAST, CHOSE_WEST 
-------------------------------------------------------------*/
int DoCoinPickSideMenu(int on, int team)
{
	static int active = 0, box = -1;
	int retval;
	int cmd, stadDir;
	RECT rec;
	
	cmd = ARCADE_MENU_NONE;  // no choice made
	if (on)
	{
		if (!active)
		{
//			DebugPrintf(0, "Activate pick side menu\n");
			 DumpCoinState();
			active = TRUE;
//			InitWindVane();
			PlayCoinToss(CHOOSE_SIDES, team);	// Play speach
			gMenuInfo.selection = COIN_MENU_SIDE_EAST;
			gMenuInfo.team = team;
			gCoinPad = GameTeamData[team].ControllerNum[0];
			if (gCoinPad < 0)
			{
				gMenuInfo.selection = PickCpuCoinSide() -1;
				gCpuMenuTime = GetVBlankCount() + CPU_MENU_TIME;
				 Cmarker();
			}
//			sprintf(gCoinSideText[COIN_MENU_SIDE_TEAM].text, "%s", 
//				SpecificTeamData[GameTeamData[team].TeamNumber].TeamCity);
			SetTeamNameSpriteOn(team, TRUE, -1, COIN_MENU_SIDE_Y+10);
//			MovePolySpriteTo(COIN_MENU_SIDE_X+10-255, COIN_MENU_SIDE_Y+10-115,
//						0, TeamNameSprite[team]);
//			MovePolySpriteTo(-GetTeamNameWidth(team)/2, COIN_MENU_SIDE_Y+10-115,
//						0, TeamNameSprite[team]);

			stadDir = gTeamStadiumInfo[GameTeamData[0].TeamNumber].faces;
			stadDir = min(stadDir, SOUTH_WEST_STADIUM);
			strcpy(gCoinSideText[COIN_MENU_SIDE_EAST].text, 
					gDirectionNames[stadDir].left);
			strcpy(gCoinSideText[COIN_MENU_SIDE_WEST].text,
					gDirectionNames[stadDir].right);
			
			 DebugPrintf(0, "%s-%s\n",
						 gCoinSideText[COIN_MENU_SIDE_EAST].text,
						 gCoinSideText[COIN_MENU_SIDE_WEST].text );
						 
			StartMenuText(gCoinSideText, COIN_MENU_SIDE_EAST);
//			setRECT(&rec, COIN_MENU_SIDE_X, COIN_MENU_SIDE_Y, 
//					COIN_MENU_SIDE_WIDTH, COIN_MENU_SIDE_HEIGHT);
//			ShowMenuBorder(TRUE, &rec, 0); 
			box = ShowArcadeMenuBox(COIN_MENU_SIDE_X, COIN_MENU_SIDE_Y, 
					COIN_MENU_SIDE_WIDTH, COIN_MENU_SIDE_HEIGHT, -1);
			MoveMenuSelect(0);
		}
		else		// Do Menu
		{
			if (gCoinPad < 0) // CPU
			{
				MoveMenuSelect(0);
				
				if (GetVBlankCount() > gCpuMenuTime )
				{
					cmd = MoveMenuSelect(0);
				}
			}
			else
				cmd = DoMenuPad(gCoinPad);
		}
	}
	else
	{
//		DebugPrintf(0, "Hide pick side menu\n");
//		 DumpCoinState();
		active = FALSE;
		SetTeamNameSpriteOn(0, FALSE,0,0);
		SetTeamNameSpriteOn(1, FALSE,0,0);
		EndMenuText();
//		ShowMenuBorder(FALSE, NULL, 0);
		FreeDialogBox(box);
//		gCpuMenuTime = GetVBlankCount() + CPU_MENU_TIME;
	}
	if (cmd == COIN_MENU_SIDE_EAST)
		retval = CHOSE_EAST;
	else if (cmd == COIN_MENU_SIDE_WEST)
		retval = CHOSE_WEST;
	else 
		retval = CHOSE_NONE;

	return retval;
}

/*-------------------------------------------------------------
  DoCoinPickReceiveMenu ~ 
   on: 1=Show & Update,  0=Hide
   team: 0=home, 1=visitor
   return: CHOSE_NONE, CHOSE_KICK, CHOSE_RECEIVE 
-------------------------------------------------------------*/
int DoCoinPickReceiveMenu(int on, int team)
{
	static int active = 0, box = -1;
	int retval;
	int cmd;
	RECT rec;
	
	cmd = ARCADE_MENU_NONE;  // no choice made
	if (on)
	{
		if (!active)
		{
//				DebugPrintf(0, "Activate pick receive menu\n");
				DumpCoinState();
			active = TRUE;
			gMenuInfo.selection = COIN_MENU_KICK_RECEIVE;
			gMenuInfo.team = team;
			gCoinPad = GameTeamData[team].ControllerNum[0];
			if (gCoinPad < 0)
			{
				gMenuInfo.selection = PickCpuCoinKick() -1;
				gCpuMenuTime = GetVBlankCount() + CPU_MENU_TIME;
				 Cmarker();
			}
			SetTeamNameSpriteOn(team, TRUE, -1, COIN_MENU_KICK_Y+10);
		
			StartMenuText(gCoinKickText, COIN_MENU_KICK_KICK);
//			setRECT(&rec, COIN_MENU_KICK_X, COIN_MENU_KICK_Y, 
//					COIN_MENU_KICK_WIDTH, COIN_MENU_KICK_HEIGHT);
//			ShowMenuBorder(TRUE, &rec, 0); 
			box = ShowArcadeMenuBox(COIN_MENU_KICK_X, COIN_MENU_KICK_Y, 
					COIN_MENU_KICK_WIDTH, COIN_MENU_KICK_HEIGHT, -1);
			MoveMenuSelect(0);
			PlayCoinToss(CHOOSE, team);	// Play speach
		}
		else		// Do Menu
		{
			if (gCoinPad < 0) // CPU
			{
				MoveMenuSelect(0);
				
				if (GetVBlankCount() > gCpuMenuTime )
				{
					cmd = MoveMenuSelect(0);
				}
			}
			else  // Not CPU
				cmd = DoMenuPad(gCoinPad);
		}
	}
	else  // Not on
	{
//		 DebugPrintf(0, "Hide pick receive menu\n");
//		 DumpCoinState();
		active = FALSE;
		SetTeamNameSpriteOn(0, FALSE,0,0);
		SetTeamNameSpriteOn(1, FALSE,0,0);
		EndMenuText();
//		ShowMenuBorder(FALSE, NULL, 0);
		FreeDialogBox(box);
//		gCpuMenuTime = GetVBlankCount() + CPU_MENU_TIME;
	}
	if (cmd == COIN_MENU_KICK_KICK)
		retval = CHOSE_KICK;
	else if (cmd == COIN_MENU_KICK_RECEIVE)
		retval = CHOSE_RECEIVE;
	else 
		retval = CHOSE_NONE;
	return retval;
}

/*-------------------------------------------------------------
  DoCoinFlippingGraphic ~ 
-------------------------------------------------------------*/
int DoCoinFlippingGraphic(void)
{
	static ULONG vbWait = 0;
	static short hold  = -1;
	int retval;
	
#if NO_COIN_GRAPHIC
	 return	 TRUE;
#endif	 
	if (vbWait == 0)
	{
		InitCoinGraphic();
		vbWait = GetVBlankCount() + COIN_FLIP_TIME;
		hold = -1;
	}
	gCoinFrame++;
	if (gCoinFrame > 10)
		gCoinFrame = 0;
	if (hold >= 0)
		gCoinFrame = hold;
	UpdateCoinFrame();
	
	if (GetVBlankCount() > (vbWait - VBlanksPerSec) )
//	if (JpadData[0]) // debug
	{
		if (CoinHeadsFlag && (gCoinFrame == COIN_HEADS_FRAME))
		{
			hold = COIN_HEADS_FRAME;
		}
		if (!CoinHeadsFlag && (gCoinFrame == COIN_TAILS_FRAME))
		{
			hold = COIN_TAILS_FRAME;
		}
	}
	if (GetVBlankCount() > vbWait)
	{
		vbWait = 0;
		retval = TRUE;
	}
	else		
		retval = FALSE;
	return retval;
}

/*-------------------------------------------------------------
  DoCoinRunBackGraphic ~ 
-------------------------------------------------------------*/
int DoCoinRunBackGraphic(void)
{
	static int count = 0;
	int done;
	
	done = FALSE;

	if (count == 0)
	{
		Cmarker();
		 DumpCoinState();
		InitGameDataPart2();
		SetCameraByOffenseDir();
		InitKickoffCamera();
		if (gGameCameraStyle != NORMAL_CAMERA_STYLE)
		{
			MoveCameraInstant();
			DoCamera();
		}
		else
		{
			 Cmarker();
			 DumpCamera();
			gHoldPlayCallVB = GetVBlankCount() + 3 * VBlanksPerSec;
		}
		InitPlayerColors(0);
		DrawSync(0);
	}
		
	count++;
	if (count > 1)
	{
		count = 0;
		done = TRUE;
		gHoldPlayCallVB = GetVBlankCount() + 5 * (VBlanksPerSec/2);
		FreeCoinToss();		// Free Coin toss sound memory
		InitKickoffCamera();
		InitPlayerColors(0);
		DrawSync(0);
	}
	
	return done;
	
}  /* End DoCoinRunBackGraphic() */

//********************** Debug routines *********************
#if DEBUG_THIS

/*-------------------------------------------------------------
  FakeCoinResult ~ 
-------------------------------------------------------------*/
void FakeCoinResult(void)
{
	CoinHeadsFlag = rand() & 1;
	CoinVisitorKickFlag = rand() & 1; 
	CoinVisitorEastFlag = rand() & 1; 
	DumpCoinState();
}

/*-------------------------------------------------------------
  DumpCoinState ~ 
-------------------------------------------------------------*/
void DumpCoinState(void)
{
	DebugPrintf(0, "CoinTossState: %d   ", CoinTossState);
	if (CoinActiveFlag)
		DebugPrintf(0, "A ");
	if (CoinFlippedFlag)
	{
		if (CoinHeadsFlag)
			DebugPrintf(0, "H ");
		else	
			DebugPrintf(0, "T ");
		DebugPrintf(0, "F ");
	}
	if (CoinVisitorPickingFlag)
		DebugPrintf(0, "VP ");
	if (CoinPickedHeadsFlag)
		DebugPrintf(0, "PH ");
	if (CoinVisitorWonFlag)
		DebugPrintf(0, "WIN ");
	if (CoinWinnerPickedSideFlag)
		DebugPrintf(0, "PS ");
	if (CoinVisitorKickFlag) 
		DebugPrintf(0, "VK ");
	if (CoinVisitorEastFlag) 
		DebugPrintf(0, "VE ");
	DebugPrintf(0, "   %d  %d \n", GetVBlankCount(), gCpuMenuTime);
}

#endif

