/*************************************************************************
	Coin Toss Logic

	By Christopher Deel

		Copyright (C) 1996 Gametek Inc.
		All rights Reserved

*************************************************************************/

#define COINLOG_C 1

#include "prjtypes.h"
#include "coinlog.h"
#include "coingraf.h"

short CoinTossState ;

/*-------------------------------------------------------------
  InitCoinTossLogic ~ 
-------------------------------------------------------------*/
void InitCoinTossLogic(void)
{
	CoinActiveFlag = 0;
	CoinHeadsFlag = 0; 
	CoinFlippedFlag = 0;
	CoinVisitorKickFlag = 0; 
	CoinVisitorEastFlag = 0; 
	CoinVisitorPickingFlag = 0;
	CoinPickedHeadsFlag = 0;
	CoinVisitorWonFlag = 0;
	CoinWinnerPickedSideFlag = 0;
	CoinOvertimeFlag = 0;
	CoinTossState = COIN_TOSS_NULL;
}

/*-------------------------------------------------------------
  StartCoinTossLogic ~ 
-------------------------------------------------------------*/
void StartCoinTossLogic(void)
{
	CoinActiveFlag = True;
	DoSpiralCameraIntro(1);	// reset spiral
	CoinTossState = COIN_TOSS_SPIRAL;
//	PlayPregame();
}


/*-------------------------------------------------------------
  StartOvertimeCoinTossLogic ~ 
-------------------------------------------------------------*/
void StartOvertimeCoinTossLogic(void)
{
	CoinActiveFlag = True;
	CoinOvertimeFlag = True;
	CoinTossState = COIN_TOSS_SPIRAL;
}  /* End StartOvertimeCoinTossLogic() */

/*-------------------------------------------------------------
  SignalOvertimeCoinToss ~ 
-------------------------------------------------------------*/
void SignalOverTimeCoinToss(void)
{
	InitCoinTossLogic();
	CoinOvertimeFlag = True;
}  /* End SignalOvertimeCoinToss() */

/*-------------------------------------------------------------
  FlipTheCoin ~ 
-------------------------------------------------------------*/
static int FlipTheCoin(void)
{
	int val;
	
//	val = rand() & 1;
	val = GetVBlankCount() & 1;
	if (val)
		CoinHeadsFlag = TRUE;
	else
		CoinHeadsFlag = FALSE;
	return val;
}

/*-------------------------------------------------------------
  DoSpiralState ~ 
-------------------------------------------------------------*/
static void DoSpiralState(void)
{
//	if (!CoinOvertimeFlag)
	if (1)
	{
		if (DoSpiralCameraIntro(0))
		{
			CoinTossState = COIN_TOSS_RUN_OUT; 
//			FreePregame();
		}
	}
	else
	{
		CoinTossState = COIN_TOSS_RUN_OUT; 
	}
}

/*-------------------------------------------------------------
  DoRunOutState ~ 
-------------------------------------------------------------*/
static void DoRunOutState(void)
{
	CoinVisitorPickingFlag = TRUE;
	CoinTossState = COIN_TOSS_PICK_FLIP;
}

/*-------------------------------------------------------------
  DoPickingFlipState ~ 
-------------------------------------------------------------*/
static void DoPickingFlipState(void)
{
	int val;
	
	val = DoCoinPickFlipMenu(TRUE, CoinVisitorPickingFlag);
	if (val != 0)
	{
		if (val == CHOSE_HEADS)
			CoinPickedHeadsFlag = TRUE;
		else
			CoinPickedHeadsFlag = FALSE;
		FlipTheCoin();
		DoCoinPickFlipMenu(FALSE, CoinVisitorPickingFlag);
		CoinTossState = COIN_TOSS_FLIPPING;
	}
}

/*-------------------------------------------------------------
  DoFlippingState ~ 
-------------------------------------------------------------*/
static void DoFlippingState(void)
{
	if (DoCoinFlippingGraphic())
	{
		if (CoinPickedHeadsFlag == CoinHeadsFlag)
			CoinVisitorWonFlag = TRUE;
		else
			CoinVisitorWonFlag = False;
		CoinVisitorPickingFlag = CoinVisitorWonFlag;
		CoinFlippedFlag = TRUE;
		CoinTossState = COIN_TOSS_PICK_FIRST;
	}
}

/*-------------------------------------------------------------
  DoPickingFirstState ~ 
-------------------------------------------------------------*/
static void DoPickingFirstState(void)
{
	int val;
	
	val = DoCoinPickFirstMenu(TRUE, CoinVisitorPickingFlag);
	if (val != CHOSE_NONE)
	{
		switch (val)
		{
			case CHOSE_KICK: // Kick
				if (CoinVisitorPickingFlag)
				{
					CoinVisitorKickFlag = True;
					CoinVisitorPickingFlag = False;
					CoinWinnerPickedSideFlag = False;
				}
				else
				{
					CoinVisitorKickFlag = False;
					CoinVisitorPickingFlag = True;
					CoinWinnerPickedSideFlag = False;
				}
				CoinTossState = COIN_TOSS_PICK_SIDE;
				break;
			
			case CHOSE_RECEIVE: // Receive
				if (CoinVisitorPickingFlag)
				{
					CoinVisitorKickFlag = False;
					CoinVisitorPickingFlag = False;
				}
				else
				{
					CoinVisitorKickFlag = True;
					CoinVisitorPickingFlag = True;
				}
				CoinTossState = COIN_TOSS_PICK_SIDE;
				break;
			
			case CHOSE_SIDE: // Side
				CoinWinnerPickedSideFlag = True;
				CoinTossState = COIN_TOSS_PICK_SIDE;
				break;
		}
		DoCoinPickFirstMenu(FALSE, CoinVisitorPickingFlag);
		InitGameDataPart1();
	}
}

/*-------------------------------------------------------------
  DoPickingReceiveState ~ 
-------------------------------------------------------------*/
static void DoPickingReceiveState(void)
{
	int val;
	
	val = DoCoinPickReceiveMenu(TRUE, CoinVisitorPickingFlag);
	if (val != CHOSE_NONE)
	{
		if (CoinVisitorPickingFlag)
		{
			if (val == CHOSE_KICK)
				CoinVisitorKickFlag = TRUE;
			else
				CoinVisitorKickFlag = FALSE;
			CoinVisitorPickingFlag = FALSE;
		}                                       
		else
		{
			if (val == CHOSE_KICK)
				CoinVisitorKickFlag = FALSE;
			else
				CoinVisitorKickFlag = TRUE;
			CoinVisitorPickingFlag = TRUE;
		}
		DoCoinPickReceiveMenu(FALSE, CoinVisitorPickingFlag);
		CoinTossState = COIN_TOSS_RUN_BACK;
		InitGameDataPart1();
	}
}

/*-------------------------------------------------------------
  DoPickingSideState ~ 
-------------------------------------------------------------*/
static void DoPickingSideState(void)
{
	int val;
	
	val = DoCoinPickSideMenu(TRUE, CoinVisitorPickingFlag);
	if (val != CHOSE_NONE)
	{
		if (CoinVisitorPickingFlag)
		{
			if (val == CHOSE_EAST)
				CoinVisitorEastFlag = TRUE;
			else 	// 2
				CoinVisitorEastFlag = FALSE;
			CoinVisitorPickingFlag = FALSE;
		}
		else  // Home Picked
		{
			if (val == CHOSE_EAST)
				CoinVisitorEastFlag = FALSE;
			else 	// 2
				CoinVisitorEastFlag = TRUE;
			CoinVisitorPickingFlag = TRUE;
		}
		DoCoinPickSideMenu(FALSE, CoinVisitorPickingFlag);
		if (CoinWinnerPickedSideFlag)
			CoinTossState = COIN_TOSS_PICK_RECEIVE;
		else
			CoinTossState = COIN_TOSS_RUN_BACK;
		InitGameDataPart1();
	}
}

/*-------------------------------------------------------------
  DoRunBackState ~ 
-------------------------------------------------------------*/
static void DoRunBackState(void)
{
	if (DoCoinRunBackGraphic())
		CoinTossState = COIN_TOSS_COMPLETE;
}

/*-------------------------------------------------------------
  DoCompleteState ~ 
-------------------------------------------------------------*/
static void DoCompleteState(void)
{
//	InitGameDataPart2();
	InitPlayerColors(0);
	CoinActiveFlag = False;
	CoinOvertimeFlag = 0;
}

/*-------------------------------------------------------------
  DoCoinTossLogic ~ 
-------------------------------------------------------------*/
void DoCoinTossLogic(void)
{
	switch (CoinTossState)
	{
		case COIN_TOSS_NULL:
			break;
		case COIN_TOSS_SPIRAL:
			DoSpiralState();
			break;
		case COIN_TOSS_RUN_OUT:
			DoRunOutState();
			break;
		case COIN_TOSS_PICK_FLIP:
			DoPickingFlipState();
			break;
		case COIN_TOSS_FLIPPING:
			DoFlippingState();
			break;
		case COIN_TOSS_PICK_FIRST:
			DoPickingFirstState();
			break;
		case COIN_TOSS_PICK_RECEIVE:
			DoPickingReceiveState();
			break;
		case COIN_TOSS_PICK_SIDE:
			DoPickingSideState();
			break;
		case COIN_TOSS_RUN_BACK:
			DoRunBackState();
			break;
		case COIN_TOSS_COMPLETE:
			DoCompleteState();
			break;
	}
}

/*-------------------------------------------------------------
  PickCpuCoinFlip ~ 
-------------------------------------------------------------*/
int PickCpuCoinFlip(void)
{
	if (rand() & 1)
		return CHOSE_HEADS;	
	else	
		return CHOSE_TAILS;	
}

/*-------------------------------------------------------------
  PickCpuCoinKick ~ 
-------------------------------------------------------------*/
int PickCpuCoinKick(void)
{
	return CHOSE_RECEIVE;	
}

/*-------------------------------------------------------------
  PickCpuCoinWin ~ 
-------------------------------------------------------------*/
int PickCpuCoinWin(void)
{
	return CHOSE_RECEIVE;	
}

/*-------------------------------------------------------------
  PickCpuCoinSide ~ 
-------------------------------------------------------------*/
int PickCpuCoinSide(void)
{
	if (rand() & 1)
		return CHOSE_EAST;	
	else
		return CHOSE_EAST;	
}
