/*************************************************************************
 * DIALOG.C - Contains all polygon dialog box routines.
 *
 * Written by: Mike Heilemann
 *************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include <strings.h>
#include <memory.h>
#include "prjhdrs.h"

#define	NO_BACKOUT	0
#define	BACKOUT	1

int		num_dialogs = 0;

extern int Std_Menu_Frame();
extern char *CurFont;

dialog_t dialog[MAX_DIALOGS];
char DialogFont = 0;

/*************************************************************************
 * POPUP DIALOG BOX - This routine will create up to 5 dialog window boxes
 *		that overlap each other and are semi-tranparent.
 *
 *		x,y,w,h - are dimensions of the box displayed without shadow
 *		shadow  - height in pixels of shadow (assumes 512 X 240 screen)
 *		r,g,b	  - RGB values for polygons, 150% light, 100% box, 50% dark
 *
 *		RETURNS: Dialog number for current dialog
 *
 *	NOTE: The way the rgb stuff is set up, it goes from 150%, to 125%,
 *		to 100%, to 75%, then finally to 50% in the loop...
 *************************************************************************/

int PopupDialogBox(int x, int y, int w, int h, int shadow, int r, int g, int b)
{
	register int i,
					 x_shadow,
					 db;

	num_dialogs = 0;

	for(db = 0; db < MAX_DIALOGS; db++)
	{
		if (dialog[db].used) num_dialogs++;
	}

	db = 0;

	while(db < MAX_DIALOGS)
	{
		if(!dialog[db].used)
		{

			// We found an unused slot so initialize the structure for this box

			for (i = 0; i < MAX_DIALOG_STRINGS; i++)
				dialog[db].string[i] = -1;

			// Dont go over max layers

			if (dialog[db].layers > MAX_TRANS_LAYERS)
				dialog[db].layers = MAX_TRANS_LAYERS;

			dialog[db].depth = 12 - num_dialogs * 2;
			dialog[db].layers = MAX_TRANS_LAYERS;
			dialog[db].shadow = shadow;

			for (i = 0; i < MAX_TRANS_LAYERS * 5; i++)
			{
				setPolyF4(&dialog[db].poly[i]);			// Init poly
				setSemiTrans(&dialog[db].poly[i], 1); 	// Semi-Transparent
			}

			MoveDialog(db, x, y);				// Move to X, Y
			ResizeDialog(db, w, h);				// Size Dialog Box
			ChangeDialogColor(db, r, g, b);	// Color Dialog Box
			dialog[db].used = TRUE;

			return(db);
		}

		db++;
	}

	// Error

	Cerror();

	return(-1);
}


/*************************************************************************
 * DISPLAY DIALOG BOX - Displays all current dialog boxes.
 *************************************************************************/
void DisplayDialogBox(void)
{
	register int i,
					 j;
	for (i = 0; i < MAX_DIALOGS; i++)
	{
		if(dialog[i].used)
		{
			for (j = 0; j < dialog[i].layers * 5; j++)
				GsSortPoly(&dialog[i].poly[j], gPtrGsOt, dialog[i].depth);
		}
	}
}


/*************************************************************************
 * CHANGE DIALOG DEPTH -
 *************************************************************************/
void ChangeDialogDepth(int num, unsigned short depth)
{
	dialog[num].depth = depth;
}


/*************************************************************************
 * CHANGE DIALOG LAYERS -
 *************************************************************************/
void ChangeDialogLayers(int num, unsigned short layers)
{
	int old_layers = dialog[num].layers << 1;

	dialog[num].layers = layers;
	ResizeDialog(num, dialog[num].poly[old_layers].x1 - dialog[num].poly[old_layers].x0, dialog[num].poly[old_layers].y2 - dialog[num].poly[old_layers].y0);
	ChangeDialogColor(num, dialog[num].r, dialog[num].g, dialog[num].b);
}


/*************************************************************************
 * FREE DIALOG BOX -
 *************************************************************************/
void FreeDialogBox(int boxnum)
{
	dialog[boxnum].used = FALSE;

//	if (num_dialogs)
//		num_dialogs--;
}


/*************************************************************************
 * FREE ALL DIALOG BOXES -
 *************************************************************************/
void FreeAllDialogBoxes(void)
{
	int i;

	for(i=0; i<MAX_DIALOGS; i++)
		dialog[i].used = FALSE;

//	num_dialogs = 0;
}


/*************************************************************************
 * MOVE DIALOG -
 *************************************************************************/
void MoveDialog(int num, int x, int y)
{
	int i,
		 x_offset,
		 y_offset;

	if (dialog[num].shadow)	// OFFSET FROM DRAWING SPACE
	{
		x_offset = x - 256 - dialog[num].poly[dialog[num].layers * 2].x0;
		y_offset = y - 120 - dialog[num].poly[dialog[num].layers * 2].y0;
	}
	else
	{
		x_offset = x - 256 - dialog[num].poly[0].x0;
		y_offset = y - 120 - dialog[num].poly[0].y0;

		for (i = 0; i < dialog[num_dialogs].layers; i++)
			setXY4(&dialog[num].poly[i], dialog[num].poly[i].x0+x_offset, dialog[num].poly[i].y0+y_offset, dialog[num].poly[i].x1+x_offset, dialog[num].poly[i].y1+y_offset, dialog[num].poly[i].x2+x_offset, dialog[num].poly[i].y2+y_offset, dialog[num].poly[i].x3+x_offset, dialog[num].poly[i].y3+y_offset);

		for (; i < dialog[num_dialogs].layers * 5; i++)
			setXYWH(&dialog[num_dialogs].poly[i], 0, 0, 0, 0);
		return;
	}

	// All bars
	for (i = 0; i < dialog[num].layers * 5; i++)
		setXY4(&dialog[num].poly[i], dialog[num].poly[i].x0+x_offset, dialog[num].poly[i].y0+y_offset, dialog[num].poly[i].x1+x_offset, dialog[num].poly[i].y1+y_offset, dialog[num].poly[i].x2+x_offset, dialog[num].poly[i].y2+y_offset, dialog[num].poly[i].x3+x_offset, dialog[num].poly[i].y3+y_offset);
}


/*************************************************************************
 * Resize Dialog -
 *************************************************************************/
void ResizeDialog(int num, int w, int h)
{
	register int i,
				x,
				y;
	int		shadow,
				x_shadow;


	shadow = dialog[num].shadow;
	x_shadow = shadow * 2;

	if (dialog[num].shadow)	// OFFSET FROM DRAWING SPACE
	{
		x = dialog[num].poly[dialog[num].layers * 2].x0;
		y = dialog[num].poly[dialog[num].layers * 2].y0;
	}
	else
	{
		for (i = 0; i < dialog[num_dialogs].layers; i++)
			setXYWH(&dialog[num_dialogs].poly[i], dialog[num].poly[0].x0, dialog[num].poly[0].y0, w, h);

		for (; i < dialog[num_dialogs].layers * 5; i++)
			setXYWH(&dialog[num_dialogs].poly[i], 0, 0, 0, 0);
		return;
	}

	// Upper bar
	for (i = 0; i < dialog[num].layers; i++)
		setXY4(&dialog[num].poly[i], x-x_shadow, y-shadow, x+w+x_shadow, y-shadow, x, y, x+w, y);

	// Left Bar
	for (; i < dialog[num].layers * 2; i++)
		setXY4(&dialog[num].poly[i], x-x_shadow, y-shadow, x, y, x-x_shadow, y+h+shadow, x, y+h);

	// Center box
	for (; i < dialog[num].layers * 3; i++)
		setXYWH(&dialog[num].poly[i], x, y, w, h);

	// Right bar
	for (; i < dialog[num].layers * 4; i++)
		setXY4(&dialog[num].poly[i], x+w, y, x+w+x_shadow, y-shadow, x+w, y+h, x+w+x_shadow, y+h+shadow);

	// Lower bar
	for (; i < dialog[num].layers * 5; i++)
		setXY4(&dialog[num].poly[i], x, y+h, x+w, y+h, x-x_shadow, y+h+shadow, x+w+x_shadow, y+h+shadow);
}


/*************************************************************************
 * CHANGE DIALOG COLOR -
 *************************************************************************/
void ChangeDialogColor(int num, int r, int g, int b)
{
	int i,
		 j,
		 red,
		 green,
		 blue;


	dialog[num].r = r;
	dialog[num].g = g;
	dialog[num].b = b;

	if (dialog[num].shadow)	// Do Box with shadow
	{
		// Setup RGB values for all polys in box
		red	= r/4;
		green	= g/4;
		blue	= b/4;
		r += r/2;
		g += g/2;
		b += b/2;
	}

	for(i = 0; i < dialog[num].layers * 5; i += dialog[num].layers)
	{
		for (j = 0; j < dialog[num].layers; j++)
		{
			setRGB0(&dialog[num].poly[i+j], r, g, b);
		}
		r -= red;
		g -= green;
		b -= blue;
		if (r < 1 && g < 1 && b < 1)
			r = g = b = 1;
	}
}

/*************************************************************************
 *	ADD DIALOG TEXT CENTERED -
 *************************************************************************/
int AddDialogTextCentered(int num, int x, int y, char *text, char color, char *spacing_table, char flags)
{
	int string;
//	char_poly_t *ptr;


	for (string = 0; string < MAX_DIALOG_STRINGS; string++)
		if (dialog[num].string[string] == -1)
			break;

	if (string == MAX_DIALOG_STRINGS)
		return(-1);

	dialog[num].string[string] = CreateString(text, spacing_table, x, y, flags);
	CenterString(dialog[num].string[string], text, x);
	ChangeString(dialog[num].string[string], text, flags);
	ChangeStringDepth(strings.string[dialog[num].string[string]], dialog[num].depth - 1);
	ChangeStringCLUT(strings.string[dialog[num].string[string]], FontClutIDs[color]->CLUT_ID);

//	ptr = strings.string[dialog[num].string[string]];
	return (string);
}


/*************************************************************************
 *	ADD DIALOG TEXT -
 *************************************************************************/
int AddDialogText(int num, int x, int y, char *text, char color, char *spacing_table, char flags)
{
	int string;
//	char_poly_t *ptr;


	for (string = 0; string < MAX_DIALOG_STRINGS; string++)
		if (dialog[num].string[string] == -1)
			break;

	if (string == MAX_DIALOG_STRINGS)
		return;

	dialog[num].string[string] = CreateString(text, spacing_table, x, y, flags);
	ChangeString(dialog[num].string[string], text, flags);
	ChangeStringDepth(strings.string[dialog[num].string[string]], dialog[num].depth - 1);
	ChangeStringCLUT(strings.string[dialog[num].string[string]], FontClutIDs[color]->CLUT_ID);

//	ptr = strings.string[dialog[num].string[string]];
	return(string);
}


/*************************************************************************
 * DIALOG BOX MESSAGE -
 *************************************************************************/
//int DialogBoxMessage(int x, int y, char *text, char *spacing_table, char flags)
int DialogBoxMessage(int x, int y, char *text)
{
	int num,
		 width;


//	width = GetStrLen(text, spacing_table);
	if (!CheckFont())
	{
		InitFonts();
		LoadFont("fontdpc2.tim", 3);
		Register_Font_Cluts();
		CurFont = fontdpc2;
		DialogFont = 1;
	}

  	width = GetStrLen(text, CurFont);

  	// Create
  	num = PopupDialogBox(x-10 - (width/2), y, width+20, 20, 2, 0x50, 0x10, 0x10);
  	ChangeDialogDepth(num, 2);

//	string = AddDialogTextCentered(num, x, y+3, text, 0, spacing_table, flags);
	AddDialogTextCentered(num, x, y+3, text, 0, CurFont, FONT0 | PROPORTIONAL);

	DoFrameLoop(1, Std_Menu_Frame);	// Display dialog
	DoFrameLoop(1, Std_Menu_Frame);
	return(num);
}


/*************************************************************************
 * CLOSE DIALOG BOX MESSAGE -
 *************************************************************************/
int CloseDialogBoxMessage(int num)
{
	int i;

	// Cleanup
	for (i = 0; i < MAX_DIALOG_STRINGS; i++)
		if (dialog[num].string[i] > -1)
		{
			PopString(dialog[num].string[i]);
			dialog[num].string[i] = -1;
		}

	FreeDialogBox(num);

	if (DialogFont)
		FreeFonts();

	DialogFont = 0;
	DoFrameLoop(1, Std_Menu_Frame);	// Dissapear dialog
	DoFrameLoop(1, Std_Menu_Frame);
}


/*************************************************************************
 * TIMED MESSAGE XY-
 *************************************************************************/
void TimedMessageXY(int x, int y, int seconds, char *text)
{
int num;
long time;

	GetAllPads(5);
	time = GetVBlankCount();
	num = DialogBoxMessage(x, y, text);

	do
	{
		ReadPads();
	}while (GetVBlankCount() < time + 60 * seconds && !GetAllPads());

	CloseDialogBoxMessage(num);
}


/*************************************************************************
 * TIMED MESSAGE -
 *************************************************************************/
void TimedMessage(int seconds, char *text)
{
	TimedMessageXY(256, 60, seconds, text);
}


/*************************************************************************
 * INPUT DIALOG BOX - 2 item
 *************************************************************************/
Sint32 InputDialogBox(int x, int y, char *text, char *yes_text, char *no_text, char backout)
{
	int num,
		 width,
		 widthY,
		 widthN,
		 string,
		 yes,
		 no,
		 flash = 0,
		 pads = 0,
		 done = 0,
		 val = 0;


	width = GetStrLen(text, CurFont);
	widthY = GetStrLen(yes_text, CurFont);
	widthN = GetStrLen(no_text, CurFont);

	// Create
	num = PopupDialogBox(x-10 - (width/2), y, width+20, 34, 2, 0x50, 0x00, 0x00);
	ChangeDialogDepth(num, 2);

	width = x - ((widthY + widthN)/2) - 8;
	string = AddDialogTextCentered(num, x, y+3, text, 0, CurFont, FONT0 | PROPORTIONAL);
	yes = AddDialogText(num, width, y+17, yes_text, 0, CurFont, FONT0 | PROPORTIONAL);
	no = AddDialogText(num,  width + widthY + 16, y+17, no_text, 3, CurFont, FONT0 | PROPORTIONAL);

	// Do YN Box input
	width = 0;
	pads = GetAllPads(1);
	while(!done)
	{
		pads = GetAllPads(15);
		switch(pads)
		{
			case JPADRup:		if(backout)	// Allow backout if required
								{
									done++;
									val = -1;
								}
								break;

			case JPADRdown:		done++;
								PlayEffect(SELECT_SFX);
								break;

			case JPADLleft:
			case JPADLright:
								PlayEffect(CURSOR_SFX);
								val ^= 1;
								if (val)
								{
									ChangeStringCLUT(strings.string[dialog[num].string[yes]], FontClutIDs[2]->CLUT_ID);
									ChangeStringCLUT(strings.string[dialog[num].string[no]], FontClutIDs[0]->CLUT_ID);
								}
								else
								{
									ChangeStringCLUT(strings.string[dialog[num].string[yes]], FontClutIDs[0]->CLUT_ID);
									ChangeStringCLUT(strings.string[dialog[num].string[no]], FontClutIDs[2]->CLUT_ID);
								}
								flash = 0;
			default:			break;
		}

		if (flash++ & 0x50)
		{
			flash = 0;
			width ^= 1;
			ChangeStringCLUT(strings.string[dialog[num].string[no-val]], FontClutIDs[width+2]->CLUT_ID);
		}
		DoFrameLoop(1, Std_Menu_Frame);
	}

	// Cleanup
	FreeDialogBox(num);
	PopString(dialog[num].string[string]);
	dialog[num].string[string] = NULL;
	PopString(dialog[num].string[yes]);
	dialog[num].string[yes] = NULL;
	PopString(dialog[num].string[no]);
	dialog[num].string[no] = NULL;

	DoFrameLoop(1, Std_Menu_Frame);	// Dissapear dialog
	DoFrameLoop(1, Std_Menu_Frame);

	return (val);
}


/*************************************************************************
 * DIALOG BOX YN -
 *************************************************************************/
int DialogBoxYN(int x, int y, char *text)
{
	FlushPads();
	return (InputDialogBox(x, y, text, "Yes", "No",NO_BACKOUT));
}
