/**************************************************************


					FBAIDATA.C

	AUTHOR: M. Knauer				Start Date: 6/13/95
	Modify Date:


	Copyright 1995  GameTek, Inc.



	This file contains Initialized Data Tables for Football AI.

	It is an INCLUDED file.

	Also see FBAIDATA.h  which has all tables in here identified
	as "externs".


**************************************************************/


	#include	"FBEQU.H"
	#include	"FBForms.H"

/*----------------------------------------------------------------

				S P E E D    D A T A

		Speeds based on 15 frames/sec.
	Average player speed = 8 equates to 1/2 yard "steps" per frame.
	This equals a 5.3 second 40 yard dash.

	Acceleration is built into the MAX_NUM_SPEED entries by making
	the curve non-linear (the deltas start off small, then become large,
	then are small again near the average speed
	Uses Table lookup as opposed to figuring X, Y sizes of "steps".

	"Average" speed is 8 (most QBs) slowest is 6[linemen], fastest is
	13, but could be higher during speed burst.


	WARNING - even though there are only 16 speeds(0-15 -  INC Speed checks
	this), there is an "extra" speed (#16) used by the DIVE

------------------------------------------------------------------*/

#define MSIN90  0x80		/* half a yard	- 128*/
#define MSIN60  0x76		// 118 decimal
#define MSIN45  0x5A        // 90 decimal
#define MSIN30  0x31        // 49 decimal
#define MSIN00  ZERO

#define MCOS00  MSIN90
#define MCOS30  MSIN60
#define MCOS45  MSIN45
#define MCOS60  MSIN30
#define MCOS90  MSIN00

#define ST(x) \
	MCOS90*(x)/100,-MSIN90*(x)/100,   /*  N */    \
	MCOS60*(x)/100,-MSIN60*(x)/100,   /*  NNE */    \
	MCOS45*(x)/100,-MSIN45*(x)/100,   /*  NE  */    \
	MCOS30*(x)/100,-MSIN30*(x)/100,   /*  ENE */    \
														\
	MCOS00*(x)/100,MSIN00*(x)/100,    /*  E   */    \
	MCOS30*(x)/100,MSIN30*(x)/100,    /*  ESE */    \
	MCOS45*(x)/100,MSIN45*(x)/100,    /*  SE  */    \
	MCOS60*(x)/100,MSIN60*(x)/100,    /*  SSE */    \
														\
	MCOS90*(x)/100,MSIN90*(x)/100,    /*  S */    \
	-MCOS60*(x)/100,MSIN60*(x)/100,   /*  SSW */    \
	-MCOS45*(x)/100,MSIN45*(x)/100,   /*  SW  */    \
	-MCOS30*(x)/100,MSIN30*(x)/100,   /*  WSW */    \
														\
	-MCOS00*(x)/100,MSIN00*(x)/100,   /*  W */    \
	-MCOS30*(x)/100,-MSIN30*(x)/100,  /*  WNW */    \
	-MCOS45*(x)/100,-MSIN45*(x)/100,  /*  NW  */    \
	-MCOS60*(x)/100,-MSIN60*(x)/100   /*  NNW */



#define STT(a,b,c,d,e,f,g,h,i,j,k,l,m,n,o,p,q,r,s,t,u,v,w,x,y)\
ST(a),ST(b),ST(c),ST(d),ST(e),ST(f),ST(g),ST(h), \
ST(i),ST(j),ST(k),ST(l),ST(m),ST(n),ST(o),ST(p), \
ST(q),ST(r),ST(s),ST(t),ST(u),ST(v),ST(w),ST(x),ST(y)




const struct s8XY SpeedTable[MAX_NUM_SPEEDS][MAX_DIRECTIONS] = {

/*-------------------------------------------------------------------------
	The values in ST(x) represent percentages of the actual speed,
	therefore if index #8 = 100% = 1/2 yard per "frame" or "move"
	 (i.e. 1/15 second)

	MAJOR CHANGE 7/25/96   back to 20 htz movement rate (i.e. slower
	   speeds then 15 htz rate

	MAJOR CHANGE 7/5/96	add more speeds -now 26 (0-25)
	PlayerSpeedRating 0-100 is scaled 0 - HIGHEST_NORMAL_SPEED(19?)
	Other speeds (20-24) are needed if Speed Burst or FAST menu option
	( FAST menu option = +2, SLOW menu option = -2 from Normal)
	Speed #25 is a special "cheat" speed used to return to huddle.

	MAJOR CHANGE 2/6/96  Speeds were too fast - made "normal" speed (8) not
	100% but more like 80% - see FBGRAVE.TXT for old table

	16 different speeds makes a 1K data structure


	4/3/96  - we were running @ 20 Hertz(we thought it was 15)
	- anims and AI now run at 15 HTz but movement runs @ 30 Htz
	(by halfing old and new)
	Anims appear to look best @ 20, but are okay at 15 if @ proper
	speed, which is why I keeping tweaking the table below

---------------------------------------------------------------------------*/
#ifdef FRAME_RATE_20HERTZ

// new tables for MAX_NUM_SPEEDS = 20 (20 HTZ AI rate - 7/25/96)
STT(14,20,25,28,32, 36,39,42,45,48, 50,53,55,58,60, 62,64,66,69,71, 74,76,  78,80, 83)
,ST(92)	// this is the DIVE speed


#endif

#if 0	//#ifdef FRAME_RATE_30HERTZ
// here are new tables for MAX_NUM_SPEEDS = 20 (15 HTZ AI rate)
  STT(15,25,35,40,44, 48,52,56,60,64, 68,71,74,77,80, 83,86,89,92,95, 98,101,104,107,110)

//STT(15,25,35,40,44, 48,52,56,60,64, 68,72,76,80,84, 88,92,96,100,104)

//idx(  0,  1,  2,  3,     4,  5,  6,  7,    8,  9, 10, 11,    12, 13, 14, 15)
// below is the 20HTZ speed converted to 15 HTZ (roughly base x 1.33)
//STT( 15, 25, 35, 40,    45, 50, 55, 60,   65, 70, 75, 80,    85, 90, 95,100)

// below is the last of the 20HTZ speeds born 3/10/96, died 4/3/96
//			  - base speed of 7(45) is okay
//STT( 10, 15, 20, 25,    30, 35, 40, 45,   50, 55, 60, 65,    70, 75, 80, 85)

// below is original speed as of 6/13/95 whiched proved to be too fast for human control
//STT( 15, 30, 45, 55,    60, 65, 70, 75,   80, 85, 90, 95,   100,105,110,115)


	,ST(120)	// this is the DIVE speed

#endif

};




const struct s8XY WindSpeedTable[MAX_NUM_WIND_SPEEDS][MAX_DIRECTIONS] = {
	ST(14),
	ST(24)
};



/*  Table to translate directions into flipped directions	*/

const WORD FlipDir[MAX_DIRECTIONS] = {

SOUTH_DIR,		//	NORTH_DIR		0
SSE_DIR,		//	NNE_DIR			1
SE_DIR,		//	NE_DIR			2
ESE_DIR,		//	ENE_DIR			3
EAST_DIR,		//	EAST_DIR		4
ENE_DIR,		//	ESE_DIR			5
NE_DIR,		//	SE_DIR			6
NNE_DIR,		//	SSE_DIR			7
NORTH_DIR,		//	SOUTH_DIR		8
NNW_DIR,		//	SSW_DIR			9
NW_DIR,		//	SW_DIR			10
WNW_DIR,		//	WSW_DIR			11
WEST_DIR,		//	WEST_DIR		12
WSW_DIR,		//	WNW_DIR			13
SW_DIR,		//	NW_DIR			14
SSW_DIR		//	NNW_DIR			15
};







/***** HIGHEST PRIORITY AT END of List  ********/

const WORD  TryManyMovesTable[MAX_DIR][4] = {

// TryGoingNORTHTable1:
	EAST_DIR,NW_DIR,NE_DIR,NORTH_DIR,

// TryGoingNNETable1:
	ESE_DIR,NNW_DIR,ENE_DIR,NNE_DIR,

// TryGoingNETable1:
	SE_DIR,NORTH_DIR,EAST_DIR,NE_DIR,

// TryGoingENETable1:
	SSE_DIR,NNE_DIR,ESE_DIR,ENE_DIR,

// TryGoingEASTTable1:
	SOUTH_DIR,SE_DIR,NE_DIR,EAST_DIR,

// TryGoingESETable1:
	NNE_DIR,SSE_DIR,ENE_DIR,ESE_DIR,

// TryGoingSETable1:
	NE_DIR,SOUTH_DIR,EAST_DIR,SE_DIR,

// TryGoingSSETable1:
	ENE_DIR,SSW_DIR,ESE_DIR,SSE_DIR,

// TryGoingSOUTHTable1:
	EAST_DIR,SW_DIR,SE_DIR,SOUTH_DIR,

// TryGoingSSWTable1:
	WNW_DIR,SSE_DIR,WSW_DIR,SSW_DIR,

// TryGoingSWTable1:
	NW_DIR,SOUTH_DIR,WEST_DIR,SW_DIR,

// TryGoingWSWTable1:
	NNW_DIR,SSW_DIR,WNW_DIR,WSW_DIR,

// TryGoingWESTTable1:
	NORTH_DIR,NW_DIR,SW_DIR,WEST_DIR,

// TryGoingWNWTable1:
	SSW_DIR,NNW_DIR,WSW_DIR,WNW_DIR,

// TryGoingNWTable1:
	NE_DIR,NORTH_DIR,WEST_DIR,NW_DIR,

// TryGoingNNWTable1:
	WSW_DIR,NNE_DIR,WNW_DIR,NNW_DIR
};




/*************************************************************************
 *
 *		 D E F E N S E   C H A S E     D I R E C T I O N S
 *
 *	This table is used by defenders to "cut down the angle" of the
 *	ball carrier based on 1) the current direction from
 *	defender to BC, and then 2) the direction BC is going.
 *
 *************************************************************************/



// dir to target = DUE NORTH (0)

const BYTE DirVsDirTable[16][16] = {{	NNE_DIR,	// NORTH_DIR
											NE_DIR,	// NNE_DIR
											NE_DIR,	// NE_DIR
											ENE_DIR,	// ENE_DIR
											ENE_DIR,	// EAST_DIR
											NE_DIR,	// ESE_DIR
											NE_DIR,	// SE_DIR
											NE_DIR,	// SSE_DIR
											NNE_DIR,	// SOUTH_DIR
											NORTH_DIR,	// SSW_DIR
											NNW_DIR,	// SW_DIR
											NNW_DIR,	// WSW_DIR
											NNW_DIR,	// WEST_DIR
											NORTH_DIR,	// WNW_DIR
											NORTH_DIR,	// NW_DIR
											NORTH_DIR},	// NNW_DIR


// dir to target = NNE (1)

										{	NE_DIR,	// NORTH_DIR
											NE_DIR,	// NNE_DIR
											NE_DIR,	// NE_DIR
											ENE_DIR,	// ENE_DIR
											EAST_DIR,	// EAST_DIR
											EAST_DIR,	// ESE_DIR
											ENE_DIR,	// SE_DIR
											ENE_DIR,	// SSE_DIR
											ENE_DIR,	// SOUTH_DIR
											NE_DIR,	// SSW_DIR
											NNE_DIR,	// SW_DIR
											NNE_DIR,	// WSW_DIR
											NNE_DIR,	// WEST_DIR
											NNE_DIR,	// WNW_DIR
											NNE_DIR,	// NW_DIR
											NE_DIR},	// NNW_DIR


// dir to target = NE (2) 

										{	NE_DIR,	// NORTH_DIR
											NE_DIR,	// NNE_DIR
											NE_DIR,	// NE_DIR
											ENE_DIR,	// ENE_DIR
											EAST_DIR,	// EAST_DIR
											EAST_DIR,	// ESE_DIR
											EAST_DIR,	// SE_DIR
											EAST_DIR,	// SSE_DIR
											EAST_DIR,	// SOUTH_DIR
											EAST_DIR,	// SSW_DIR
											ENE_DIR,	// SW_DIR
											NE_DIR,	// WSW_DIR
											NE_DIR,	// WEST_DIR
											NE_DIR,	// WNW_DIR
											NE_DIR,	// NW_DIR
											NE_DIR},	// NNW_DIR


// dir to target = ENE (3)

										{	ENE_DIR,	// NORTH_DIR
											NE_DIR,	// NNE_DIR
											ENE_DIR,	// NE_DIR
											ENE_DIR,	// ENE_DIR
											EAST_DIR,	// EAST_DIR
											EAST_DIR,	// ESE_DIR
											EAST_DIR,	// SE_DIR
											EAST_DIR,	// SSE_DIR
											ESE_DIR,	// SOUTH_DIR
											EAST_DIR,	// SSW_DIR
											EAST_DIR,	// SW_DIR
											ENE_DIR,	// WSW_DIR
											NE_DIR,	// WEST_DIR
											NE_DIR,	// WNW_DIR
											NE_DIR,	// NW_DIR
											ENE_DIR},	// NNW_DIR

// dir to target = DUE EAST (4) 

										{	ENE_DIR,	// NORTH_DIR
											EAST_DIR,	// NNE_DIR
											EAST_DIR,	// NE_DIR
											EAST_DIR,	// ENE_DIR
											EAST_DIR,	// EAST_DIR
											EAST_DIR,	// ESE_DIR
											EAST_DIR,	// SE_DIR
											EAST_DIR,	// SSE_DIR
											ESE_DIR,	// SOUTH_DIR
											ESE_DIR,	// SSW_DIR
											ESE_DIR,	// SW_DIR
											ESE_DIR,	// WSW_DIR
											EAST_DIR,	// WEST_DIR
											ENE_DIR,	// WNW_DIR
											ENE_DIR,	// NW_DIR
											ENE_DIR},	// NNW_DIR

// dir to target = ESE (5) 

										{	EAST_DIR,	// NORTH_DIR
											EAST_DIR,	// NNE_DIR
											EAST_DIR,	// NE_DIR
											EAST_DIR,	// ENE_DIR
											EAST_DIR,	// EAST_DIR
											ESE_DIR,	// ESE_DIR
											ESE_DIR,	// SE_DIR
											ESE_DIR,	// SSE_DIR
											SE_DIR,	// SOUTH_DIR
											SE_DIR,	// SSW_DIR
											SE_DIR,	// SW_DIR
											SE_DIR,	// WSW_DIR
											SE_DIR,	// WEST_DIR
											ESE_DIR,	// WNW_DIR
											EAST_DIR,	// NW_DIR
											EAST_DIR},	// NNW_DIR

// dir to target = SE (6)

										{	EAST_DIR,	// NORTH_DIR
											EAST_DIR,	// NNE_DIR
											EAST_DIR,	// NE_DIR
											EAST_DIR,	// ENE_DIR
											EAST_DIR,	// EAST_DIR
											EAST_DIR,	// ESE_DIR
											ESE_DIR,	// SE_DIR
											ESE_DIR,	// SSE_DIR
											SE_DIR,	// SOUTH_DIR
											SE_DIR,	// SSW_DIR
											SE_DIR,	// SW_DIR
											SE_DIR,	// WSW_DIR
											SE_DIR,	// WEST_DIR
											SE_DIR,	// WNW_DIR
											ESE_DIR,	// NW_DIR
											EAST_DIR},	// NNW_DIR

// dir to target = SSE  (7)

										{	ESE_DIR,	// NORTH_DIR
											ESE_DIR,	// NNE_DIR
											EAST_DIR,	// NE_DIR
											EAST_DIR,	// ENE_DIR
											EAST_DIR,	// EAST_DIR
											ESE_DIR,	// ESE_DIR
											SE_DIR,	// SE_DIR
											SE_DIR,	// SSE_DIR
											SE_DIR,	// SOUTH_DIR
											SSE_DIR,	// SSW_DIR
											SSE_DIR,	// SW_DIR
											SSE_DIR,	// WSW_DIR
											SSE_DIR,	// WEST_DIR
											SE_DIR,	// WNW_DIR
											SE_DIR,	// NW_DIR
											ESE_DIR},	// NNW_DIR

// dir to target = due SOUTH (8)

										{	SSE_DIR,	// NORTH_DIR	try to get in front
											SE_DIR,	// NNE_DIR
											SE_DIR,	// NE_DIR
											ESE_DIR,	// ENE_DIR
											ESE_DIR,	// EAST_DIR
											ESE_DIR,	// ESE_DIR
											SE_DIR,	// SE_DIR
											SE_DIR,	// SSE_DIR
											SSE_DIR,	// SOUTH_DIR
											SOUTH_DIR,	// SSW_DIR
											SSW_DIR,	// SW_DIR
											SSW_DIR,	// WSW_DIR
											SW_DIR,	// WEST_DIR
											SW_DIR,	// WNW_DIR
											SSW_DIR,	// NW_DIR
											SOUTH_DIR},	// NNW_DIR


// dir to target = SSW (9)

										{	SOUTH_DIR,	// NORTH_DIR
											SOUTH_DIR,	// NNE_DIR
											SSE_DIR,	// NE_DIR
											SE_DIR,	// ENE_DIR
											ESE_DIR,	// EAST_DIR
											ESE_DIR,	// ESE_DIR
											SE_DIR,	// SE_DIR
											SSE_DIR,	// SSE_DIR
											SSE_DIR,	// SOUTH_DIR
											SOUTH_DIR,	// SSW_DIR
											SSW_DIR,	// SW_DIR
											SW_DIR,	// WSW_DIR
											SW_DIR,	// WEST_DIR
											SW_DIR,	// WNW_DIR
											SW_DIR,	// NW_DIR
											SSW_DIR},	// NNW_DIR


// dir to target = SW  - (A) index into this table based on dir. target is going

										{	WEST_DIR,	// NORTH_DIR
											WSW_DIR,	// NNE_DIR
											SW_DIR,	// NE_DIR
											SSW_DIR,	// ENE_DIR
											SOUTH_DIR,	// EAST_DIR
											SSE_DIR,	// ESE_DIR
											SE_DIR,	// SE_DIR
											SSE_DIR,	// SSE_DIR
											SOUTH_DIR,	// SOUTH_DIR
											SOUTH_DIR,	// SSW_DIR
//											SOUTH_DIR,	// SW_DIR
											SSW_DIR,	// SW_DIR
											SSW_DIR,	// WSW_DIR
											SSW_DIR,	// WEST_DIR
											WSW_DIR,	// WNW_DIR
											WSW_DIR,	// NW_DIR
											WSW_DIR},	// NNW_DIR
//	reversed - punt wide out -wrong way		NNW_DIR,	// WNW_DIR
//											NORTH_DIR,	// NW_DIR
//											NORTH_DIR},	// NNW_DIR


// dir to target = WSW 	    (B)

										{	WEST_DIR,	// NORTH_DIR
											WEST_DIR,	// NNE_DIR
											WSW_DIR,	// NE_DIR
											WSW_DIR,	// ENE_DIR
											SSW_DIR,	// EAST_DIR
											SOUTH_DIR,	// ESE_DIR
											SOUTH_DIR,	// SE_DIR
											SOUTH_DIR,	// SSE_DIR
											SOUTH_DIR,	// SOUTH_DIR
											SSW_DIR,	// SSW_DIR
											SW_DIR,	// SW_DIR
											WSW_DIR,	// WSW_DIR
											WSW_DIR,	// WEST_DIR
											WSW_DIR,	// WNW_DIR
											WEST_DIR,	// NW_DIR
											WNW_DIR},	// NNW_DIR


// dir to target = DUE WEST  - index into this table based on dir. target is going

										{	NNW_DIR,	// NORTH_DIR
											NNW_DIR,	// NNE_DIR
											NW_DIR,	// NE_DIR
											WNW_DIR,	// ENE_DIR
											WEST_DIR,	// EAST_DIR
											WSW_DIR,	// ESE_DIR
											SW_DIR,	// SE_DIR
											SSW_DIR,	// SSE_DIR
											SSW_DIR,	// SOUTH_DIR
											SSW_DIR,	// SSW_DIR
											SW_DIR,	// SW_DIR
											WSW_DIR,	// WSW_DIR
											WEST_DIR,	// WEST_DIR
											WNW_DIR,	// WNW_DIR
											NW_DIR,	// NW_DIR
											NNW_DIR},	// NNW_DIR


// dir to target = WNW  = 13

										{	NORTH_DIR,	// NORTH_DIR
											NORTH_DIR,	// NNE_DIR
											NORTH_DIR,	// NE_DIR
											NORTH_DIR,	// ENE_DIR
											NNW_DIR,	// EAST_DIR
											NW_DIR,	// ESE_DIR
											WNW_DIR,	// SE_DIR
											WSW_DIR,	// SSE_DIR
											SW_DIR,	// SOUTH_DIR
											SW_DIR,	// SSW_DIR
											WSW_DIR,	// SW_DIR
											WEST_DIR,	// WSW_DIR
											WNW_DIR,	// WEST_DIR
											WNW_DIR,	// WNW_DIR
											NW_DIR,	// NW_DIR
											NNW_DIR},	// NNW_DIR

// dir to target = NW (14) - index into table based on dir. target is going

										{	NORTH_DIR,	// NORTH_DIR
											NNE_DIR,	// NNE_DIR
											NE_DIR,	// NE_DIR
											ENE_DIR,	// ENE_DIR
											NORTH_DIR,	// EAST_DIR
											NNW_DIR,	// ESE_DIR
											NW_DIR,	// SE_DIR
											WNW_DIR,	// SSE_DIR
											WEST_DIR,	// SOUTH_DIR
											WEST_DIR,	// SSW_DIR
											WEST_DIR,	// SW_DIR
											WEST_DIR,	// WSW_DIR
											WNW_DIR,	// WEST_DIR
											NW_DIR,	// WNW_DIR
											NNW_DIR,	// NW_DIR
											NORTH_DIR},	// NNW_DIR

// dir to target = NNW (15)

										{	NNE_DIR,	// NORTH_DIR
											NNE_DIR,	// NNE_DIR
											NE_DIR,	// NE_DIR
											NE_DIR,	// ENE_DIR
											NE_DIR,	// EAST_DIR
											NE_DIR,	// ESE_DIR
											NNE_DIR,	// SE_DIR
											NNW_DIR,	// SSE_DIR
											NW_DIR,	// SOUTH_DIR
											NW_DIR,	// SSW_DIR
											NW_DIR,	// SW_DIR
											NW_DIR,	// WSW_DIR
											NW_DIR,	// WEST_DIR
											NNW_DIR,	// WNW_DIR
											NNW_DIR,	// NW_DIR
											NORTH_DIR}};	// NNW_DIR





/***********************************************************************/

const BYTE BCRunTable[16] =
	{SW_DIR,			// NORTHDir
	WSW_DIR,			// NNEDir
	WSW_DIR,			// NEDir
	WEST_DIR,		// ENEDir

	WEST_DIR,		// EASTDir
	WEST_DIR,		// ESEDir
	WNW_DIR,			// SEDir
	WNW_DIR,			// SSEDir

	NW_DIR,			// SOUTHDir
	NW_DIR,			// SSWDir
	NW_DIR,			// SWDir
	NW_DIR,			// WSWDir

	NNW_DIR,			// WESTDir
	SW_DIR,			// WNWDir
	SW_DIR,			// NWDir
	SW_DIR};			// NNWDir



/********************** not used

//  1 <= str <=15
BYTE BaseStrength[TOTAL_PLAYERS_ON_TEAM] = {	// based on player position

	9,		//default Quarterback strength
	5,		//default right receiver strength
	4,		//default left receiver strength
	11,	//default right tight end strength
	10,	//default full back strength
	8,		//default half back strength
	3,		//default wide receiver 3 strength
	5,		//default wide receiver 4 strength
	4,		//default wide receiver 5 strength
	10,	//default left tight end strength
	8,		//default extra tight end strength
	6,		//default extra running back strength
	7,		//default backup quarter back	strength

	15, 	//default offensive right tackle	strength
	15, 	//default offensive left tackle strength
	13, 	//default offensive right guard strength
	14, 	//default offensive left guard strength	
	13, 	//default offensive center	strength	
	12, 	//default extra offensive lineman strength

	9,		//default PUNTER strength
	5,		//default PUNT_RETURNER	strength
	3,		//default KICK_RETURNER	strength
	9,		//default KICKER strength

	12, 	//default defensive left end strength
	13, 	//default defensive left tackle strength
	13, 	//default defensive right tackle	strength
	12, 	//default defensive right end	strength
	9,		//default free safety strength
	10, 	//default strong safety	strength
	10, 	//default outside left linebacker strength
	11, 	//default inside left linebacker strength
	11, 	//default inside right linebacker strength
	10, 	//default outside right linebacker strength
	8,		//default right cornerback strength
	7,		//default left cornerback strength
	6,		//default nickel back strength
	5,		//default dime back strength
	5,		//default extra defensive back strength
	9,		//default extra linebacker strength
	11, 	//default extra defensive line strength
};

*/

/*  define the macro to only use the "frames per anim" number	*/

#define pgmac(abc,xyz)  abc,
const UWORD NumFramesPerPlayerGraphic[]  = {

	#include "FBPGData.h"

};

#undef pgmac



/**************************************************************

		A I      G R A P H I C S   D A T A

**************************************************************/


	#include	"FBAI2GRP.h"	// THIS HAS TABLE of AI animations





// below is pointer array to above AI animations
//  (AI animations are variable length)

#define aipgmac(abc,xyz)  &abc,		// use macro to define entry as an address

void *AIToGraphicTable[] = {

	#include "FBAIGrap.h"	// this file is also used in "fbgequ.h" as enums

};
#undef aipgmac


#define aipgmac(abc,xyz)  &xyz##Offsets,		// use macro to define entry as an address

void *AIGraphicOffsetTable[] = {

	#include "FBAIGrap.h"	// this file is also used in "fbgequ.h" as enums

};
#undef aipgmac







#define formmacro(positionformnum,PCformnum,string,OfforDef,offseq)	OfforDef,
#define	ALL_FORMS_VALID


BYTE FormationOffOrDef[MAX_FORMATIONS] =  {

	#include	"fbfrmdat.h"
};

#undef	ALL_FORMS_VALID
#undef formmacro


#define formmacro(positionformnum,PCformnum,string,OfforDef,offseq)	offseq,
#define	ALL_FORMS_VALID
WORD SequentialOffFormIndex[MAX_FORMATIONS] = {


	#include	"fbfrmdat.h"
};

#undef	ALL_FORMS_VALID
#undef formmacro


/***********************  End of File: FBAIDATA.C   *******************/
