/**************************************************************


					FBBall.C

	INITIAL AUTHOR: M. Knauer       Start Date: 7/28/95

	Modify Date: 7-31-95 Mike Heilemann - Wrote PassTarget routine
		- which calculates X,Y,Z for passing to a player.

	Modify Date: 9-1-95 Mike Heilemann - Wrote Bouncing Ball stuff

	Modify Date: 9-7-95 Mike Heilemann - Wrote Player select stuff


	Copyright 1995  GameTek, Inc.


	This file contains modules dealing with the movement of the
	actual ball - passing, kicking, pitchouts, etc.


------  below is STANDARD DISCLAIMER ----
	NOTE: All AI routines in the main AI loop refer to the
	"CURRENT PLAYER" (usually the loop counter of the AI loop 0-21 -
	often refered to by variable "gPNum")

	Any references to the players Data Stuctures are "hidden" in
	a macro starting wtih the initials CP. This allows us to
	more freely change the data structure (if needed) from the initial
	non-linked array based DS's (easier to debug) to a structure based
	data structure

	E.g.  CPXPos =  PlayerXPosition[gPNum]     (current as of 5/19/95)
		could be changed to
		 CPXPos =  Player[gPNum].XPosition
		or something like
		 initialize  	CP = Player[gPNum]
		 in macro		CPXPos =  CP->XPosition

	--  end STANDARD DISCLAIMER ----


	NOTE:  All Ball transitions, such as receivers (or defenders)
	catching passes, catching punts, getting fumbles, etc. are
	handled as part of BALL ACTIONs (that is, the ball gives itself to
	a player).
	This must be handled this way because we can't put it in player
	STATES, as human controlled players don't do scripted STATES.

	Here are the only exceptions to this rule:
		A) QB direct handoff to FutureBC
		B) QB or Punter getting long hike



**************************************************************/

	#include  "fbequ.h"
	#include  "fbRAM.h"			/* external VARs needed by this file	*/
	#include  "FBPlays.h"		/* Script enums			*/
	#include  "sound1.h"		/* SFX enums			*/
	#include  "fbbutton.h"
	#include  "fbstats.h"

#define MKPASSING	TRUE
#define MAXPASSES_IS_5 0
#define MAX_PASS_TYPES 11
#define	BALL_BEING_CARRIED_HEIGHT	YARD

#define	NO_WIND		0	// in QuickFigureBallMovement
#define	USE_WIND	1

//#define BALL_ACTION(abcd) void abcd(void)

#ifdef __MSDOS__
#include <conio.h>
#endif
#define BALL_ACTION(xyz) \
	void  (xyz)(void);  \
	void  (xyz)(void)   /* actual function name of the State  */


#define BallGoingUpMacro	(CurrentBallDeltaZ > ZERO)	// ball is still going up


enum kicktypes {
	FIELD_GOAL_KICK_TYPE,
	PUNT_KICK_TYPE,
	KICKOFF_KICK_TYPE
};


/******* Ball Actions	*********************/

void BallPreSnap(void);
void PlayerHasBall(void);
void BallAfterTackle(void);     		/*  a BALL ACTION		   */
void BallBeingPassed(void);
void BallBeingPunted(void);
void BallBeingBounced(void);
//void InitBallBeingPassed(void);
void InitBouncingBall(void);
void BallBeingFGed(void); //... and forever more

/*--- ball action subroutines	----*/
BOOL FigureBallHeight(void);
void MoveBallInAir(void);
void FigureBallSphereOfInfluence(void);
void UpdatePlayerWithBall(void);
//void PassTarget(WORD receiver, WORD BallSpeed, WORD dz);
BOOL HeShouldFumbleGeneric(WORD possiblefumbler);	// in FBMove.c
void CheckKickOOB(void);	// used on punt, KO - set OOBX instant kick goes OOB

BOOL WhereWillPlayerBe(WORD pnum, WORD numframes);	// returns TRUE if aborted
void DoAction(void);


//void DetermineOffensivePassOutCome(WORD poscatcher);
//BOOL TheReceiverShouldCatchPass(WORD possiblecatcher);
//void CheckForInterception(WORD defguy);	/** ball STATE subroutine	*/
void SetInterception(WORD defguy);
void SetPassDeflection(void);
void DeterminePassOutCome(void);		/* in a ball state	*/
UWORD FindDirection(WORD DeltaX, WORD DeltaY);


BOOL KickBlockedOutcome(WORD type);	//Returns true if punt or FG blocked
WORD FieldGoalHitorMiss(UWORD maxdistance);

BOOL AnybodyGetFreeBall(void);
void KeepBallInWorld(void);	 // a BALL subroutine - keep in Stadium

void FigurePassTarget(WORD TheReceiver, WORD passtype);
void QuickFigureBallMovement(WORD startxpos, WORD startzpos, WORD usewind,WORD color);

WORD	DefCanGetBall(void);	// return slot num of guy else NOBODY
WORD 	OffCanGetBall(void);
BOOL	HeCanGetBall(WORD guy);
WORD FindClosestOffPlayer(WORD defguy);
void SetReceiversToLeadBlockers(void);

//PLAYER_ACTION(WaitAfterCatchBall);
PLAYER_ACTION(CycleAnimActionAndEnd);
void SetCPAction(voidFCNPTR action);

void StartGameClock(void); //  when guy touches ball


WORD Distance(WORD x, WORD y);
WORD Distance2Players(WORD p1, WORD p2);
void FigureDefenseKeyDistance(WORD player);

/**  Kicking stuff   ***/
void MoveBallArcCounter(void);
void FigureFinalKickParameters(WORD kicktype);
void FigureKickArc(void);
BOOL AdjustKickTargetX(void);
void TurnOffKickInterface(void);


/**  following in FBMove.c		****/
void  SetNewBallCarrier(WORD NewBallCarrier);
void SetPlayersForDefJustGotBall(void);
void SetPlayersForOffJustGotBall(void);

void InitBallFumble(WORD fumbletype,WORD DirToGo);	// in this file
void InitFumble(WORD fumbletype,WORD DirToGo,WORD NeedSFX);	// in FBMove.c

void ClearReceivers(void);	// sets all pos. receivers to -1, i.e. no graphics
BOOL BallOutOfEntireField(void);	// returns TRUE if Ball outof bounds or over Endlines
WORD Distance2Points(WORD x1, WORD y1, WORD x2, WORD y2);

void InitCatchBallX(void);	// called if def gets ball

BOOL SwapControllers(WORD newplayerslotnum, WORD oldplayerslotnum);
//void ChangeControlledPlayer(WORD jpad_num,WORD incval,WORD toggletype);
//void ChangeControlledPlayer(WORD jpad_num,WORD incval);
void  InitGraphics(WORD AIgraphicsnum);
BOOL  InitGraphicsQ(WORD AIgraphicsnum);	// a Question with an answer

WORD AIPlayEffectCheck(WORD sfxnum);
void SaveIRPBallTarget(void);	//bug # 1987 - show target during IRP
BOOL ItsAFleaFlickerPlay(void);


void AudioCrowdReaction(WORD sfxnum, WORD basevolume);
void SetNewCrowdVolume(WORD crowdchannel, WORD desiredvolume,WORD tickstodesired);
void HomeOrAwayAudioCrowdReaction(WORD homesfx, WORD awaysfx);


#define	RANDOM_PERCENT_MACRO	(rand()%101)

#ifdef __MSDOS__
#include <stdio.h>
#endif

#if	PC_GRAPHIC_MODE
void PlotZ(WORD x, WORD z,WORD color);
void DisplayBallArc(WORD color);
void DrawCross(UWORD x, UWORD y,WORD TARGET_COLOR);	// pc only
#endif

UWORD BallArc[MAX_POINTS_IN_BALL_ARC][3];

UWORD RecX[128];
UWORD RecY[128];
WORD  KickButtonCount;

WORD KickerStrength;	// global to easily keep track off
WORD KickerAccuracy;	// global to easily keep track off

/**********************************************************************

					 B A L L    S T A T E S

		  (Actions of the ball)

	1. First the offense moves
	2. Then the defense moves
	3. then the ball moves (below in "MoveBall()")

	The only exception to this is if a "PlayerHasBall", then the ball
	is moved directly after the ball carriers STATE, as the BC's AI
	may need to use the exact and current Ball X,Y (such as in handing
	off).


**********************************************************************/

void MoveBall(void)
{

	BallAction.todo();		/* do the BALLs current action(STATE)	*/

}


BOOL ReplotBallArcForWind(void)	// different for KO
{	// punt, pass, or FG
BOOL rv = FALSE;	// assume

	if( (BallXPosition > BallArc[PointsInBallArc/3][0]) // 1/3 way in air
	  &&(ReplotBallArcCounter == ZERO)	// none it yet???
	  &&(WindDeltaX | WindDeltaY)	// any wind???
	  )
	{
		ReplotBallArcCounter = 1;
		QuickFigureBallMovement(BallXPosition, BallZPosition, USE_WIND, 200);
		BallTargetX = (ULONG)BTX1/2+(ULONG)BTX2/2;
		BallTargetY = (BTY1+BTY2)/2;
		rv = TRUE;
	}
	return(rv);
}


void SetBallDirToBallTarget(void)
{
	BallDirection = FindDirection2Points(BallXPosition,BallYPosition,BallTargetX,BallTargetY);
}

/*----------------------------------------------------------------
			Ball   Being   Passed

				  a BALL ACTION

	Give players two chances to get ball:
	a) move ball on X,Y
	c) move ball on Z
	d) check for catch by any/all players (in DeterminePassOutCome())

----------------------------------------------------------------*/
BALL_ACTION(BallBeingPassed)		/*  a BALL ACTION		   */
{
WORD divisor = 6;	// default	was 5,4	(bug #1144)

	ReplotBallArcForWind();		// if necessary

	MoveBallInAir(); 	/* updates BallX, BallY, BallSI	*/

	if(BallTargetX > StartBallX+18*YARDS)
		divisor = 4;	//react less on long passes

	if(BallXPosition > BallArc[PointsInBallArc/divisor][0]) // see bug #1144 (/6 on 2/6/96)
	{
		SetBit1(EventFlags,DEF_REACT_TO_PASS_BFLAG);	//DefReactToPassMACRO
	}

	if(CurrentBallDeltaZ < ZERO)	// pass is coming down
	{
		if(! (BallFlags & PASS_COMING_DOWN_BFLAG))
		{	/*  tell receiver to go get ball when ball starts down	*/
			SetBit(BallFlags,PASS_COMING_DOWN_BFLAG);

			if(TheReceiver >= ZERO  && TheReceiver < 11)	// valid receiver???
			{
				OPScriptPtr(TheReceiver) = All_Scripts[RunToBallTargetScriptNUM];
				OPStateInit(TheReceiver) = ZERO;	/* re-init rec's new state	*/
				OPAIState(TheReceiver) = NoAI;	// could be RB looking to lead block

			}
#if PC_ERROR_MODE
			else
			{
				printf("\nError - No Receiver on Ball Being Passed- %d",TheReceiver);
			}
#endif
		}
	}

	if(!FigureBallHeight())     /* updates Ball Z  - false if hit ground	*/
	{

		if( !(PlayFlags & PASS_PLAY_BFLAG))	// set only if forward pass
/*
		if( (BallDeltaX < ZERO)		// backward pass???
		  && (!(BallFlags & PASS_DEFLECTED_BFLAG))
		  && (!(BallFlags & NOBODY_CATCH_PASS_BFLAG))
		  ) */
		{   // ..... is a fumble
			InitFumble(BALL_HIT_GROUND_FUMBLE,BallDirection,NEED_FUMBLE_SFX);
		}
		else
		{
			if(BallTargetX > DEF_END_LINE)	// BTarget cleared if deflection
				CatchBallX = SETTOWHOLEYARD(BallTargetX);	// in case of pass interference

			SetBallAction(InitBouncingBall);	// next frame ball will bounce
			SetBit(PlayOverFlags,(BALL_DEAD_BFLAG|INCOMPLETE_PASS_BFLAG));
			ResetBit(BallFlags,BALL_PASSED_BFLAG);
	//		ClearReceivers();	will be done at PlayOver anyway
		}


	}
	else if( !(BallFlags & NOBODY_CATCH_PASS_BFLAG) )	// set if deflection
		DeterminePassOutCome();		/* in a ball state	*/
}



BALL_ACTION(BallBeingPitched)	/*-------------  a BALL ACTION -------------*/
{
EFFVAR i;

//	SetBit(BallFlags,BALL_PITCHED_BFLAG);

	MoveBallInAir(); 	/* updates BallX, BallY, BallSI, ShowBallBFlag*/

	if(HeCanGetBall(FutureBC))
	{
		if(HeShouldFumbleGeneric(FutureBC) )
		{
			if(PlayFlags & PASS_PLAY_BFLAG)
//			if(BallDeltaX > ZERO)	// forward pitch???
			{	// ..... is not a fumble
				SetBallAction(InitBouncingBall);	// next frame ball will bounce
				SetBit(PlayOverFlags,(BALL_DEAD_BFLAG|INCOMPLETE_PASS_BFLAG));
			}
			else
				InitFumble(BALL_HIT_GUY_FUMBLE,BallDirection,NEED_FUMBLE_SFX);
		}
		else
		{	// NOTE: DON'T force change of SCRIPT to BCAvoid yet
			if(PlayFlags & PASS_PLAY_BFLAG)
//			if(BallDeltaX > ZERO)	// forward pitch???
			{	// ..... is not a fumble
				PassCatcher = FutureBC;
			}

			SetNewBallCarrier(FutureBC);
			if ( (i = GameTeamData[OffensiveSlotNum].ControllerNum[0]) != -1)
				SwapControllers(FutureBC, ControlWhichPlayer[i]);
	//		ClearReceivers();	not for FleaF
		}
	}
	else
	{
		if(!FigureBallHeight())     /* updates Ball Z  - false if hit ground	*/
		{
//			ResetBit(BallFlags,BALL_PITCHED_BFLAG);

			if(PlayFlags & PASS_PLAY_BFLAG)
//			if(BallDeltaX > ZERO)	// forward pitch???
			{   // ..... is not a fumble
				SetBallAction(InitBouncingBall);	// next frame ball will bounce
				SetBit(PlayOverFlags,(BALL_DEAD_BFLAG|INCOMPLETE_PASS_BFLAG));
			}
			else
			{
				InitFumble(BALL_HIT_GROUND_FUMBLE,BallDirection,NEED_FUMBLE_SFX);
			}
		}

	}
}

#ifdef NEW_LONG_HIKE
	/*   new Long hike uses different ball parameters so as not
		 to interfere with Kicking arc stuff
	*/
#define HikeSpeed	150	// for 20htz f.rate

BALL_ACTION(BallLongHikeAction)		/*  a BALL ACTION  - shotgun and punt  */
{
   // unlike NORMAL ball movement, the Plyers SCRIPT must take ball

	BallCarrier = NOBODY;

	HikeXPosition -= HikeSpeed;
	HikeYPosition += HikeDeltaY;	// only for HB snap

	if(HikeXPosition < HikeTargetX) /* updates Ball Z  - false if hit ground	*/
	{	// QB/punter/holder missed it - will this ever happen???
		HikeZPosition -= QTR_YARD/2;
		if(HikeZPosition <= ZERO)
		{
			HikeZPosition = BallZPosition = ZERO;
			InitFumble(BALL_HIT_GROUND_FUMBLE,BallDirection,NEED_FUMBLE_SFX);
							// sets players react time to chase ball
		}
	}
	else
		HikeZPosition += HikeBallDeltaZ;	// CBDZ set up at hike init

	BallXPosition = HikeXPosition;
	BallYPosition = HikeYPosition;
	BallZPosition = HikeZPosition;
	SetBit(BallFlags,(SHOW_BALL_BFLAG |SPIRAL_BALL_BFLAG));


}

#else

BALL_ACTION(BallLongHikeAction)		/*  a BALL ACTION  - shotgun and punt  */
{    // unlike NORMAL ball movement, the Plyers SCRIPT must take ball

	MoveBallInAir(); 	/* updates BallX, BallY, BallSI, ShowBallBFlag*/

	if(BallXPosition < BallTargetX)     /* updates Ball Z  - false if hit ground	*/
	{
		BallZPosition -= QTR_YARD/2;
		if(BallZPosition <= ZERO)
		{
			BallZPosition = ZERO;
			InitFumble(BALL_HIT_GROUND_FUMBLE,BallDirection,NEED_FUMBLE_SFX);
							// sets players react time to chase ball
		}
	}
	else
		BallZPosition += CurrentBallDeltaZ;	// CBDZ set up at hike init
}
#endif


void SetPassPlay(void)		// called during pass or pitch
{
	if(BallDeltaX >= ZERO)	// see if forward "pass"
	{
		Passer = gPNum;	// save for stats
		SetBit(PlayFlags,PASS_PLAY_BFLAG);
	}
	else
		PossibleFumbleX = BallXPosition;	// in case fumble later

}


#define Pitch45DegreeLength	180
void InitPitch(void)		/* a PLAYER STATE  subroutine	*/
{
	BallZPosition  = BallZPosition = 4*FEET;

	// below moved into SetPassPlay
//	PossibleFumbleX = BallXPosition;	// in case fumble later

	if(FutureBC != NOBODY)
	{
		FigurePassTarget(FutureBC, LOB_PASS);
	}
	else	// Future BC had better be set up !!!! - below is actually error correction
	{
		if(CPFacing == SW_DIR)	// QB - CPFacing set in STATE init
		{
			BallDeltaX = -Pitch45DegreeLength;
			BallDeltaY = Pitch45DegreeLength;
		}
		else
			BallDeltaX = BallDeltaY = -Pitch45DegreeLength;		// left
	}
	BallCarrier = NOBODY;
	SetBallAction(BallBeingPitched);

	SetPassPlay();	// only if a forward pass
}



/*-----------------------------------------------------------------*/
/**  following called as part of "InitFumble()", it sets Ball parameters	*/
void InitBallFumble(WORD fumbletype, WORD dirtogo)
{
EFFVAR rnumto100;		// a local var. saves calls (and time)

	rnumto100 = RANDOM_PERCENT_MACRO;

	if(fumbletype == BALL_HIT_GROUND_FUMBLE)
	{	// gravity stuff should have already been set up by pitch,hike etc
		if (rnumto100  < 50)		// 50 % chance to change to a bad bounce
			SetBit1(EventFlags, BAD_BOUNCE_BFLAG);

	// next frame bounce it (it just hit ground this frame)
		SetBallAction(InitBouncingBall);
	}
	else //if(fumbletype == BALL_HIT_GUY_FUMBLE)
	{
	/* set up BallDelta,gravity Parameters here- will be modified in InitBounce*/
		CurrentBallGravity = 2;				// if 1 is bad try 0
		InitialBallGravity = 2;				// if 1 is bad try 0

		ConstantBallDeltaZ = 8;
		CurrentBallDeltaZ  = 32;			// seems the right constant...

		BallDeltaX = 50 + (rand()&63);
		BallDeltaY = 50 + (rand()&63);

		if (rnumto100  < 60)
		{	// 60 % chance to change to a bad bounce
			SetBit1(EventFlags, BAD_BOUNCE_BFLAG);
		}
	  // this frame start the bounce as there just was a player collision
		OldBallZPosition = BallZPosition = 5*FEET;	// start these fumbles up high
		InitBouncingBall();	 // call a BALL ACTION as a subroutine (really-it's okay)

	}
	BallDirection = dirtogo;
}



/*************************************************************************

					INIT BOUNCING BALL

	Takes care of the ball bouncing initialization
	live or dead.  This is kinda recursive because it sets the
	gravity, and BallDeltaX, Y then sets the ball action to bounce.
	When the bouncing is over it calls InitBouncingBall again, etc.

	NOTE: this routine can be called directly if need (e.g. ball was just
	fumbled when a player was tackled) OR it can be a BALL ACTION

	In most cases the ball is on the ground (BAllZPos = ZERO) except
	if BALL_HIT_GUY_FUMBLE,

*************************************************************************/

// the stronger a pass, the less likely to be a bad bounce (AFTER 1st bounce)
#define MAX_BOUNCES 10
//WORD PassBadBouncePercent[MAX_PASS_TYPES] = {0,40,35,30,25,20};
WORD PassBadBouncePercent[MAX_PASS_TYPES] = {0,45,40,37,35,32,30,27,25,20,20};
/* following are % of ball speed lost each bounce	*/
WORD CBDZGrassBounceDeltaLoss[MAX_BOUNCES]= {50,60,70,60,50,40,30,20,10,0};

// bug # 1775 - reduce bouncing on turf
WORD CBDZTurfBounceDeltaLoss[MAX_BOUNCES] = {55,65,70,60,50,40,30,20,10,0};
//WORD CBDZTurfBounceDeltaLoss[MAX_BOUNCES] = {60,70,80,70,50,40,30,20,10,0};

//WORD BounceDeltaLoss[MAX_BOUNCES] = {50,60,70,60,50,40,30,20,10,0};
//WORD BounceDeltaLoss[MAX_BOUNCES] = {70,70,70,70,70,40,30,20,10,0};
//WORD   BounceDeltaLoss[MAX_BOUNCES] = {70,70,70,50,50,40,30,20,10,0};	//dies 4/29/96
WORD   BounceDeltaLoss[MAX_BOUNCES] = {60,60,70,50,50,40,30,20,10,0};


BALL_ACTION(InitBouncingBall)    		/*  a BALL ACTION		   */
{
//WORD temp;
EFFVAR rnumto100;		// a local var. saves calls (and time)


	SetBit(BallFlags,BALL_BOUNCING_BFLAG);

	if(  (PlayFlags & KICK_BLOCKED_BFLAG)
	   && (PlayType >= FIELD_GOAL_PLAY))	// if it got blocked & EX pt
	{ // it's over
		SetBit(PlayOverFlags,KICK_OVER_BFLAG);
	}

	if(NumBallBounces < 4)
		AIPlayEffectCheck(PUNT_SFX);	// final bounces(rolling) didn't sound right

	// Check for ball out of bounds...
	if (BallOutOfEntireField())	// returns TRUE if Ball outof bounds or over Endlines
	{
		SetBit(PlayOverFlags,
			(BALL_DEAD_BFLAG | BALL_OUT_OF_BOUNDS_BFLAG | OKAY_TO_GO_OOB_BFLAG));
		CheckKickOOB();	// used on punt, KO
//		KickOOBX = BallXPosition;	// punted ball taht bounces bug #
	}
	else if( (BallFlags & BALL_PUNTED_BFLAG)  && (BallXPosition >= OFF_GOAL_LINE) )
	{ // KLUDGY, but it works 
		SetBit(PlayOverFlags,(BALL_DEAD_BFLAG | OFF_DOWNED_PUNT_BFLAG));
	}

	// Turn off spinning on ball, so it doesn't look silly.
//	ResetBit(BallFlags, BALL_PASSED_BFLAG);	/* show it, but don't spin it	*/
	SetBit(BallFlags, SHOW_BALL_BFLAG | END_OVER_END_BFLAG);	/* show it [wierdly] */
	ResetBit(BallFlags, SPIRAL_BALL_BFLAG);	/* no more spiral	*/


	if( BallFlags & NOBODY_CATCH_PASS_BFLAG)
	{	//a deflection straight down - 1st time thru set new ballspeed
		BallDeltaX = (rand()&31)+32;
		BallDeltaY = (rand()&31)+32;
		ResetBit(BallFlags,NOBODY_CATCH_PASS_BFLAG);
		SetBit1(EventFlags,BAD_BOUNCE_BFLAG);
	}
	else
	{
		// each bounce has a % decrease of ball speed
		BallDeltaX = (BallDeltaX * BounceDeltaLoss[NumBallBounces]) /100;
		BallDeltaY = (BallDeltaY * BounceDeltaLoss[NumBallBounces]) /100;

		rnumto100 = RANDOM_PERCENT_MACRO;
		if ( NumBallBounces != ZERO
			&& (rnumto100 < PassBadBouncePercent[PassType]) )
			SetBit1(EventFlags,BAD_BOUNCE_BFLAG);
	}

// Do a bad bounce...  (1 of 8 directions) (or 1 of 4 (if x & y are = or x or y = 0)
	if (EventFlags.BAD_BOUNCE_BFLAG)	// could be set elsewhere (in Fumble logic)
	{
#if 1		// rewrote 7/28/96
//		ResetBit(EventFlags,BAD_BOUNCE_BFLAG);	// it was a little too bouncy bug#195
		rnumto100 = rand();	// just a regular random

		if(NumBallBounces < 3)
		{
			if(BallDeltaX == ZERO)	// zero led to weird bounces
				BallDeltaX = (rnumto100&15) +5;

			if(BallDeltaY == ZERO)
				BallDeltaY = (rnumto100&15) +5;
		}

			// swap delta x and/or Y
//		if (rnumto100 & 0x0001 )		// 50%
		if (rnumto100 < 26 )		// 25% (11/10/96 for onside kick)
			BallDeltaX = -BallDeltaX;

		if (rnumto100 & 0x0002 )		// 50%
			BallDeltaY = -BallDeltaY;

//		if (rnumto100 & 0x0004 )		// 50%
//		{
//			temp = BallDeltaX;
//			BallDeltaX = BallDeltaY;
//			BallDeltaY = temp;
//		}



#else
		ResetBit1(EventFlags,BAD_BOUNCE_BFLAG);	// it was a little too bouncy bug#195
		rnumto100 = rand();	// just a regular random

		if(NumBallBounces < 3)
		{
			if(BallDeltaX == ZERO)	// zero led to weird bounces
				BallDeltaX = (rnumto100&15) +5;

			if(BallDeltaY == ZERO)
				BallDeltaY = (rnumto100&15) +5;
		}

			// swap delta x and/or Y
		if (rnumto100 & 0x0001 )		// 50%
			BallDeltaX = -BallDeltaX;

		if (rnumto100 & 0x0002 )		// 50%
			BallDeltaY = -BallDeltaY;

		if (rnumto100 & 0x0004 )		// 50%
		{
			temp = BallDeltaX;
			BallDeltaX = BallDeltaY;
			BallDeltaY = temp;
		}
#endif
	}


	KeepBallInWorld();		// actually the stadium


	if (FieldType == TURF_FIELD)
	{
		ConstantBallDeltaZ =
			(ConstantBallDeltaZ * CBDZTurfBounceDeltaLoss[NumBallBounces]) /100;
	}
	else	// grass field
	{
		ConstantBallDeltaZ =
			(ConstantBallDeltaZ * CBDZGrassBounceDeltaLoss[NumBallBounces]) /100;
	}

	CurrentBallGravity = InitialBallGravity;
//	CurrentBallDeltaZ = ConstantBallDeltaZ / 2;
	CurrentBallDeltaZ = ConstantBallDeltaZ * 2;


	if(++NumBallBounces >= MAX_BOUNCES)
	{
		NumBallBounces = MAX_BOUNCES-1;
		ResetBit(BallFlags, (END_OVER_END_BFLAG | SPIRAL_BALL_BFLAG));

	}

	SetBallAction(BallBeingBounced);	// Bounce it next time...


//	BallBeingBounced();					// ...  and this time !!!!
//  NO-NO - only do a portion of "BallBeingBounced()" - this caused a problem
//	on fumbles as guys would get fumbles in same frame they fumbled
	MoveBallInAir(); 	/* updates BallX, BallY, BallSI	*/
	KeepBallInWorld();			// a BALL subroutine
	FigureBallHeight();	/* updates Ball Z  - false if hit ground */

}


/*************************************************************************

			 BALL BEING BOUNCED

	- Takes care of the ball bouncing live or dead

 *************************************************************************/
BALL_ACTION(BallBeingBounced)		/*  a BALL ACTION		   */
{
//int player;

	MoveBallInAir(); 	/* updates BallX, BallY, BallSI	*/

	KeepBallInWorld();			// a BALL subroutine

	CheckKickOOB();			// used on punt, KO
	if(!AnybodyGetFreeBall())
		if(!FigureBallHeight())	/* updates Ball Z  - false if hit ground */
			SetBallAction(InitBouncingBall);
}





void KeepBallInWorld(void)	 // a BALL subroutine - keep in Stadium
{
//	if(BallXPosition > (UWORD)(230 *YARDS))  // wrap on UWORD ???
	if(BallXPosition < DEF_BACK_LINE)
	{
//		BallXPosition = 1;		// keep in bounds
		if(BallDeltaX < ZERO)
			BallDeltaX = -(BallDeltaX>>1);	// and bounce "positive" and less
	}
	else if(BallXPosition > OFF_BACK_LINE)  // wrap on UWORD ???
	{
		if(BallDeltaX > ZERO)
			BallDeltaX = -(BallDeltaX>>1);	// and bounce "negative"
	}


//	if(BallYPosition > (UWORD)(230 *YARDS))  // wrap on UWORD ???
	if(BallYPosition < NORTH_BACKLINE)
	{
//		BallYPosition = 1;		// keep in world
		if(BallDeltaY < ZERO)
			BallDeltaY = -(BallDeltaY>>1);   // and bounce "positive"
	}
	else if(BallYPosition > SOUTH_BACKLINE)
	{
		if(BallDeltaY > ZERO)
			BallDeltaY = -(BallDeltaY>>1);	// and bounce "negative"
	}

}


//#define MAX_PASS_COUNTER_VALUE (PASS_RELEASE_FRAME_NUM-PASS_START_FRAME_NUM+1)
#define MAX_PASS_COUNTER_VALUE ((PASS_RELEASE_FRAME_NUM-PASS_START_FRAME_NUM+1)*2)
//#define MAX_PASS_COUNTER_VALUE 	(9-4+1)


BALL_ACTION(BallError)		/*  a BALL ACTION		   */
{
//int player;
#if PC_ERROR_MODE
	printf("Ball Error");

#endif
	SetBit(ErrorNumber,1);
	SetBit(BallFlags,BALL_BOUNCING_BFLAG);

	OldBallZPosition = BallZPosition = ZERO;	// not floating in midair
	if(PlayHappening())
		AnybodyGetFreeBall();

}



/*-------------------------------------------------------------------

				Release Pass

	The the instant a QB releases the pass. (subroutine of passing action)

	PassCounter (global) determines type of pass. For a human this counts
	how long the button was held down - it is forced by the CPU QB.

-------------------------------------------------------------------*/
#if MAXPASSES_IS_5
BYTE PassTypeTable[MAX_PASS_COUNTER_VALUE] = {
		BULLET_PASS,
		MEDIUM_BULLET_PASS,
		NORMAL_PASS,
		MEDIUM_LOB_PASS,
		LOB_PASS,
};
#else
BYTE PassTypeTable[MAX_PASS_COUNTER_VALUE] = {
		10,9,8,7,6,5,4,3,2,1

/*		BULLET_PASS,
		BULLET_PASS,
		MEDIUM_BULLET_PASS,
		MEDIUM_BULLET_PASS,
		NORMAL_PASS,
		NORMAL_PASS,
		MEDIUM_LOB_PASS,
		MEDIUM_LOB_PASS,
		LOB_PASS,
		LOB_PASS,
*/
};
#endif

void ReleasePass(BOOL TruePass)	/*  called by QB ACTION	   */
{
//WORD temp;

	if( (gPNum != BallCarrier)		// bug # 1501 ??
		|| (CPYPos < NORTH_SIDELINE)
		|| (CPYPos > SOUTH_SIDELINE)
	  )
		return;		// no passing if OOB or not BC(duh!)

	OldBallZPosition = BallZPosition = GET_BALL_NORMAL_HEIGHT; //2*YARDS;


	if(PassType == NO_PASS)	 // if passtype already inited, CPU has decided what type
	{ // NO_PASS must be human throwing and we need to look at how long button down
		if(PassCounter >= MAX_PASS_COUNTER_VALUE)	// error check
			PassType = LOB_PASS;
		else
			PassType = PassTypeTable[PassCounter];
	}


//	temp = PassType;
//	for(PassType = 1; PassType < MAX_PASS_TYPES; PassType++)
//		FigurePassTarget(TheReceiver, PassType);	// set Ball Delta X,Y

//	PassType=temp;

	if(TruePass)
	{
		FigurePassTarget(TheReceiver, PassType);	// set Ball Delta X,Y
	}
	else	// must be QB down ball
	{
		BallTargetX = CPXPos;
		BallTargetY = CPYPos + 2 * YARD;
		BallDeltaX = 2;		// don't throw forward - guard catches it
		BallDeltaY = 300;   // must be far enough so QB doesn't catch

		CurrentBallDeltaZ = -140;
		CurrentBallGravity = 100;	// high so it never goes to zero

	}

	SaveIRPBallTarget();	//bug # 1987 - show target during IRP

//	StartPassX = SETTOWHOLEYARD(CPXPos);	// used for grounding
//	QuickFigureBallMovement(BallXPosition,BallZPosition,243);

/*  don't check about player getting ball or ball hit ground on this 1st frame */
//	MoveBallInAir(); 	/* updates BallX, BallY, BallSI, BC = NOBODY	*/
//	FigureBallHeight();     /* updates Ball Z  - false if hit ground	*/

	SetBallDirToBallTarget();	// for graphic display
	SetBit(BallFlags,SHOW_BALL_BFLAG|SPIRAL_BALL_BFLAG|BALL_PASSED_BFLAG);
	SetBallAction(BallBeingPassed);	// will do this frame, (ball done AFTER players)

	BallCarrier = NOBODY; //bug #1329,1330
	/* BC =NOBODY now, since when def goes next frame, they still think
	QB has ball, cause BallState (which sets BC=NOBODY) goes after
	 defmove in the main AI loop */


/*  tell receiver to go get ball - moved to after ball comes down	*/
//	OPScriptPtr(TheReceiver) = All_Scripts[RunToBallTargetScriptNUM];
//	OPStateInit(TheReceiver) = ZERO;	/* re-init rec's new state	*/

	SetPassPlay();	// only if a forward pass

}


/******************  is this a feature or a bug ???  ****************

						QuickFigurePass

	called every frame during QB passing ACTION,
	allows human to see where ball will go if he hold button

**********************************************************************/

void QuickFigurePass(WORD pnum)	// allow arc to be seen while button down
{
EFFVAR pt,tempz;

	if(GameOptions[GAME_OPTNS_PASS_ARC+OffensiveSlotNum] == OPTION_ON)
	{
		pt = PassType;
		if(pt == NO_PASS)	 // if passtype already inited, CPU has decided what type
		{ // NO_PASS must be human throwing and we need to look at how long button down
			if(PassCounter >= MAX_PASS_COUNTER_VALUE)	// error check
				pt = LOB_PASS;
			else
				pt = PassTypeTable[PassCounter];
		}

		tempz = BallZPosition;		// save [CARRY BALL] height

		BallZPosition = GET_BALL_NORMAL_HEIGHT;
		FigurePassTarget(pnum, pt);	// set Ball Delta X,Y
		BallZPosition = tempz;

		SetBit(InterfaceFlags,DISPLAY_BALL_ARC_BFLAG);


	//	QuickFigureBallMovement(BallXPosition,GET_BALL_NORMAL_HEIGHT,243);
	}

}


BALL_ACTION(BallPreSnap)       		/*  a BALL ACTION		   */
{
	OldBallXPosition = BallXPosition = StartBallX;
	OldBallYPosition = BallYPosition = StartBallY;
	OldBallZPosition = BallZPosition = ZERO;

	BallFlags = SHOW_BALL_BFLAG;
//	SetBit(BallFlags,SHOW_BALL_BFLAG);	/* show it, but don't spin it	*/

	if(PlayType == KICKOFF_PLAY)
		SetBit(BallFlags,STAND_BALL_BFLAG);	/* it's on a tee	*/

}



BALL_ACTION(BallGPTest)
{
	SetBit(BallFlags,SHOW_BALL_BFLAG);	/* show it, but don't spin it	*/
	OldBallZPosition = BallZPosition = OPZPos(0) + 10*FEET;
	OldBallXPosition = BallXPosition = OFF_END_LINE-FOOT;
	BallYPosition = OPYPos(0);
	if(BallYPosition < LOW_GOALPOST_Y)
		BallYPosition = LOW_GOALPOST_Y;
	else if(BallYPosition > HIGH_GOALPOST_Y)
		BallYPosition = HIGH_GOALPOST_Y;
	OldBallYPosition =BallYPosition;
}


/**********************************************************************/

BALL_ACTION(PlayerHasBall)     		/*  a BALL ACTION		   */
{ //WARNING - don't force BC to NOBODY (e.g. at end of play) without
  // updating  BallAction !!!!

	if(BallCarrier >= ZERO) 	// error checking
	{
		UpdatePlayerWithBall();
	}
	else
		SetBallAction(BallError);
}



BALL_ACTION(BallAfterTackle)     		/*  a BALL ACTION		   */
{
	BallCarrier = NOBODY;
	ResetBit(BallFlags,SHOW_BALL_BFLAG);	/* don't show it	*/
}





/****************** B A L L       S U B R O U T I N E S     *******/


void UpdatePlayerWithBall(void)
	/* called by Ball ACTION or after Player States	*/
{
	BallXPosition = OPXPos(BallCarrier);
	BallYPosition = OPYPos(BallCarrier);

//	OldBallXPosition = BallXPosition = OPXPos(BallCarrier);
//	OldBallYPosition = BallYPosition = OPYPos(BallCarrier);

#ifdef FRAME_RATE_30HERTZ
	OldBallXPosition = OPOldXPos(BallCarrier);
	OldBallYPosition = OPOldYPos(BallCarrier);

	OldBallZPosition =
#endif
		BallZPosition = BALL_BEING_CARRIED_HEIGHT;
	if(OPFlags1(BallCarrier) & IM_HOLDER_BFLAG)
		BallZPosition = 1;


	FigureBallSphereOfInfluence();
	BallFlags = ZERO;
}




/*---------------------------------------------------------------------*/

WORD DefCanGetBall(void) // interceptors, punt returner, OR getting fumble
{
EFFVAR i;

	for(i=11; i<22; i++)
	{
		if(HeCanGetBall(i))
			return(i);
	}
	return(NOBODY);
}

WORD OffCanGetBall(void)
{
EFFVAR i;

	for(i=0; i<11; i++)
	{
		if(HeCanGetBall(i))
			return(i);
	}
	return(NOBODY);
}


void ClearFumbleStats(void)	// called in middle of play after fumble recovery
{

	// move what was generated by Stats update to other area for display
 /*  NO-NO 11/26/96 - this just confuses the issue
	sprintf(PlayInfoText0,"%s",PlayInfoText1);		*/

	PlayInfoText1[0] = PlayInfoText2[0] = '\0';	// clear Text Display buffer
	WordsInAnnouncerPhrase = ZERO;	// don't say anything

//	Passer = PassCatcher = KickReturner = Tackler = NOBODY;

}

void DebugScreen(void);

/*********************************************************************

				GET   FREE   BALL

	if ball is bouncing around, this will give it to somebody

		below used for fumble, onside kick or blocked kick or???

 ***********************************************************************/
#define	GET_FREE_BALL_PERCENT	25

BOOL AnybodyGetFreeBall(void)		// called in "Ballbounce" ACTION
{
EFFVAR i,fbc,temp;

  /* ignore if ball is dead (OutOfBounds or incomplete or missed FG)  */
	if (PlayOverFlags & BALL_DEAD_BFLAG)
		return(FALSE);

	// no one can get fumble for a while after it happened
	if(AIFrameCounter < FumbleFrameCounter)
		return(FALSE);

	fbc = OffCanGetBall();
	if( (fbc != NOBODY) && (RANDOM_PERCENT_MACRO < (PlayType == KICKOFF_PLAY?95:GET_FREE_BALL_PERCENT)) )
	{
		if(BallFlags & BALL_PUNTED_BFLAG)
		{
			SetBit(PlayOverFlags,(OFF_DOWNED_PUNT_BFLAG |BALL_DEAD_BFLAG));
			InitCatchBallX();	// used for stats stuff later - length of punt

			temp = gPNum;	// save our global
			gPNum = fbc;	// okay to use gPNum in a BallSTATE
			CPFacing = FindCPDirectionToBallMACRO;
			InitGraphicsQ(AI_DOWN_PUNT_NUM);
			CPAction = CycleAnimActionAndEnd;
			gPNum = temp;

		}
		else
		{
			if(BallFlags & BALL_KICKEDOFF_BFLAG)
			{
				InitCatchBallX();	// used for stats stuff later
				KickReturner = fbc;
//				SetNewCrowdVolume(HIGH_CROWD_SFX,CHL2_VOL,7*SECONDS);
				SetNewCrowdVolume(HIGH_CROWD_SFX,CHL2_VOL,1*SECONDS);	//11/21/96
			}
			else if(EventFlags.FUMBLE_HAPPENED_BFLAG)
			{	// must be fumble recovery

				if(LastBallCarrier > 10)
				{	//  - if LBC on defense (punt or Int)
#if PC_ALWAYS_MODE
				printf("\nOff. got def. fumble, LBC = %d",LastBallCarrier);
#endif
					UpdatePrelimDefPlayStats(Fumbler, FumbleX);
					ClearFumbleStats();	// erase any text generated by above
		/********************
					StatsData[DefensiveSlotNum].allplyrstats[GS_FUM_LOST]++;
					// give def. credit for their INT or punt return
					UpdateNormalDefPlayStats(LastBallCarrier,FumbleX);
					ClearFumbleStats();	// erase any text generated by above
		*****/
					// off. gets new 1st down no matter what
					SetBit(MoreEventFlags,FORCE_NEW_1STDOWN_BFLAG);
				}
				else if(LastBallCarrier != NOBODY)
				{	// off got back own fumble, give credit to what happened earlier
					UpdatePrelimOffPlayStats(Fumbler, FumbleX,FALSE);
					ClearFumbleStats();	// erase any text generated by above

//					UpdateNormalOffPlayStats(LastBallCarrier,FumbleX,FALSE);
//					ClearFumbleStats();	// erase any text generated by above
				}
				FumbleRecoveryX = SETTOWHOLEYARD(BallXPosition);
				FumbleRecoverer = fbc;

			}
			else
			{
			#if PC_ALWAYS_MODE
				printf("\nError on Off. get free ball. = GUy = %d",fbc);
				getch();
				DebugScreen();
			#endif
			}

			SetNewBallCarrier(fbc);
			SetPlayersForOffJustGotBall();
			if ( (i = GameTeamData[OffensiveSlotNum].ControllerNum[0]) != -1)
				SwapControllers(fbc, ControlWhichPlayer[i]);
			if(PlayType == KICKOFF_PLAY)
				StartGameClock(); //  in case nobody has touched ball yet

		}
		return(TRUE);
	}
	else if( ((fbc = DefCanGetBall()) != NOBODY)  && (RANDOM_PERCENT_MACRO < GET_FREE_BALL_PERCENT))
	{
		if(BallFlags & BALL_PUNTED_BFLAG)
		{
			InitCatchBallX();	// used for stats stuff later
			PuntReturner = fbc;
//			SetNewCrowdVolume(HIGH_CROWD_SFX,CHL2_VOL,7*SECONDS);
			SetNewCrowdVolume(HIGH_CROWD_SFX,CHL2_VOL,4*SECONDS);	//

		}
		else if(BallFlags & BALL_KICKEDOFF_BFLAG)
		{
		// def. get onside?? - EndOfPlay logic will handle
#if PC_ALWAYS_MODE
			printf("\nDef touched kickoff, fBC=%d",fbc);
#endif
		}
		else if(EventFlags.FUMBLE_HAPPENED_BFLAG)
		{ 	// must be fumble recovery
			if(LastBallCarrier >= ZERO && LastBallCarrier <11)
			{	//  - def. got offense fumble (not NOBODY)
				UpdatePrelimOffPlayStats(Fumbler, FumbleX,FALSE);
				ClearFumbleStats();	// erase any text generated by above

/**********
				StatsData[OffensiveSlotNum].allplyrstats[GS_FUM_LOST]++;
				// give credit to what happened earlier
				UpdateNormalOffPlayStats(LastBallCarrier,FumbleX,FALSE);
				ClearFumbleStats();	// erase any text generated by above
****/		}
			else // def. got back own fumble
			{    // give credit to what happened earlier
#if PC_ALWAYS_MODE
				printf("\nDef. got def. fumble, LBC=%d",LastBallCarrier);
#endif
				UpdatePrelimDefPlayStats(Fumbler, FumbleX);
				ClearFumbleStats();	// erase any text generated by above

//				UpdateNormalDefPlayStats(LastBallCarrier,FumbleX);
//				ClearFumbleStats();	// erase any text generated by above
			}
			FumbleRecoveryX = SETTOWHOLEYARD(BallXPosition);
			FumbleRecoverer = fbc;
		}
		else if(PlayFlags & KICK_BLOCKED_BFLAG)
		{
			FumbleRecoveryX = SETTOWHOLEYARD(BallXPosition);
//			InitCatchBallX();	// used for stats stuff later
			BlockedKickRecoverer = fbc;
		}
		else	// what else could it be?????
		{
		#if PC_ALWAYS_MODE
			printf("\nError on Def. get free ball. = GUy = %d",fbc);
			getch();
			DebugScreen();
		#endif
		}

		SetNewBallCarrier(fbc);
		SetPlayersForDefJustGotBall();

		if(PlayType == KICKOFF_PLAY)
			StartGameClock(); //  in case nobody has touched ball yet

		if ( (i = GameTeamData[DefensiveSlotNum].ControllerNum[0]) != -1)
			SwapControllers(fbc, ControlWhichPlayer[i]);


		return(TRUE);
	}
	return(FALSE);
}
/************************************************************************

					 B A L L     H E I G H T

	Figure height based on gravity

	1) Add CurBallDeltaZ(CBDZ) to BallZPos (CBDZ could be negative)
	2) Dec CurrentBall Gravity (counter)
	3) if CurrentBall Gravity = zero, reset to init gravity AND decrease
		CurBDZ by ConBDZ

	if InitG = 2, ConBDZ = 1 , and CurBDZ starts at 4
	This will generate a curve such as:(based on BDZ)
	 4,4,3,3,2,2,1,1,0,0,-1,-1,-2,-2,-3,-3,-4,-4,-5 (should hit ground)

	So for every X,Y the ball moves, the first time it will move 4 up,
	then 4, then 3, etc.

	Following should have already been inited :

	BallZPos: should start at 2*yards for passing
	ConstantBallDeltaZ: amount to subtract from CurrentBallDeltaZ when
			gravity goes to zero.
	CurrentBallDeltaZ: initially set at 4,5 times ConstantBallDeltaZ
	InitialBallGravity: used to reset CurrentBallGravity when it goes to zero
	CurrentBallGravity: actually a decrement counter -when it goes to zero,
		subtract ConBDZ from CurBDZ (see above).


	FORMULA:  #frames = InitGrav + (2 x InitGrav x CurBDeltaZ)/ ConBDeltaZ
		or    CurBDeltaZ = ((#frames - InitGrav) x ConBDeltaZ) / (2 x InitGrav)

	Once Pass target is figured, we know #frames to reach target (based on
	X,Y movement).  InitGrav is a function of #frames, ConBDeltaZ is a
	constant depending on pass type (LOB or Bullet),
	so we solve for CurBDeltaZ so that the pass ellipse (i.e.
	height formula) will hit exactly at the target.


	RETURNS: FALSE if ball hit ground


**************************************************************************/

BOOL FigureBallHeight(void)		// returns FALSE if ball hit ground
{
	BallZPosition += CurrentBallDeltaZ;

	if(BallZPosition > ZERO)
	{
#if MKPASSING
		if(--CurrentBallGravity <= ZERO)
#else
		if(--CurrentBallGravity < ZERO)
#endif
		{
			if(ConstantBallDeltaZ < 1)	// bug #2258? (ballz = Constant)
				ConstantBallDeltaZ = 1;
			CurrentBallGravity = InitialBallGravity;
			CurrentBallDeltaZ -= ConstantBallDeltaZ;
		}
	}
	else
	{
		BallZPosition = ZERO;	/* ball on ground	*/
		return(FALSE);
	}
	return(TRUE);
}

//#define MinWindAffectHeight	(7*FEET)
//#define WindAdjustBallXMacro(ballzpos) ((ballzpos -  MinWindAffectHeight)>>6)
//#define WindAdjustBallYMacro(ballzpos) ((ballzpos -  MinWindAffectHeight)>>6)

#define MinWindAffectHeight	(10*FEET)
#define MaxWindAffectHeight	(20*FEET)
#define WEDIFF (MaxWindAffectHeight-MinWindAffectHeight)

//#define WindAdjustBallXMacro(ballzpos) WindDeltaX
//#define WindAdjustBallYMacro(ballzpos) WindDeltaY


#define WindAffectsBallMacro(ballzpos) (ballzpos > MinWindAffectHeight)

#define WindAdjustBallXMacro(ballz)  \
 (ballz > MaxWindAffectHeight ? WindDeltaX : (((ballz-MinWindAffectHeight)*WindDeltaX)/WEDIFF))

#define WindAdjustBallYMacro(ballz)  \
 (ballz > MaxWindAffectHeight ? WindDeltaY : (((ballz-MinWindAffectHeight)*WindDeltaY)/WEDIFF))


void MoveBallInAir(void) 	/* updates BallX, BallY, BallSI	*/
{
EFFVAR t;
//	SetBit(BallFlags,(SHOW_BALL_BFLAG|SPIN_BALL_BFLAG));	//+BALL_PASSED_BFLAG;
	// Init routine must set spiral OR EndOverEnd flag
	SetBit(BallFlags,(SHOW_BALL_BFLAG));	//+BALL_PASSED_BFLAG;
	BallCarrier = NOBODY;
	BallXPosition += BallDeltaX;
	BallYPosition += BallDeltaY;

	if(WindAffectsBallMacro(BallZPosition))
	{
//		BallXPosition += WindDeltaX;
//		BallYPosition += WindDeltaY;
		t =  WindAdjustBallXMacro(BallZPosition);
		BallXPosition += t;
		t =  WindAdjustBallYMacro(BallZPosition);
		BallYPosition += t;

	}

	FigureBallSphereOfInfluence();
}




void FigureBallSphereOfInfluence(void)
{

//#define	BALL_SI_RANGE	0x0200		/* remember, YARD = 256 (0x0100)	*/
#define	BALL_SI_RANGE		0x01c0		/* reduced 1/3/96)	*/

	BallSphereInfluence.Low.X =	BallXPosition-BALL_SI_RANGE/2;
	BallSphereInfluence.High.X = BallXPosition+BALL_SI_RANGE/2;
	BallSphereInfluence.Low.Y =	BallYPosition-BALL_SI_RANGE/2;
	BallSphereInfluence.High.Y = BallYPosition+BALL_SI_RANGE/2;
}


void ConvertCurrentPlayerAIToActualGraphics(void);	//proto

void CheckForStandingCatch(WORD poscatcher)	// pass OR put
{
EFFVAR temp;

	if(OPAIGraphicState(poscatcher) == AI_STANDINGCATCH_NUM)
	{	// guy waiting for ball does a short ACTION
//			SetCPAction(WaitAfterCatchBall);
//			SetOPAction(CycleAnimActionAndEnd);

		OPAction(poscatcher) = CycleAnimActionAndEnd;

		temp = gPNum;	// temp holder
		gPNum = poscatcher;
		CycleAnimActionAndEnd();	// and do it now!
		ConvertCurrentPlayerAIToActualGraphics();	// and be sure to show This frame!
		gPNum = temp;
	}
	/** below new 4/21/97	***/
	else if(OPAIGraphicState(poscatcher) == AI_RUNNINGCATCH_NUM)
	{
		temp = gPNum;	// temp holder
		gPNum = poscatcher;

		InitGraphics(AI_RUNNINGCAUGHT_NUM);
		OPAction(poscatcher) = CycleAnimActionAndEnd;

		CycleAnimActionAndEnd();	// and do it now!
		ConvertCurrentPlayerAIToActualGraphics();	// and be sure to show This frame!
		gPNum = temp;

	}
	/** end new stuff 4/21/97	*/
}


BOOL HumanEffectValid(WORD guy)
{	// if Human effct option on AND guy is human controlled
	return( (GameOptions[GAME_OPTNS_HUMAN_EFFECT] == 1)
		   && (OPFlags(guy) & IS_HCONTROLLED_BFLAG) );
}


//BOOL PassInterferenceValid(void);

/********************************************************************

				 P A S S I N G    O U T C O M E S

	Passes can either be:

	1) Caught be receiver	(if hit off. player)
	2) Intercepted          (if hit def. player)
	3) Deflected			(if hit off. or def. player)
	4) Incomplete (if hit ground)


	Algorithm uses two % (off. and def.) to see who will catch.
	After % figured, get a random and see if
		a) defense can catch, if not then
		b) offense can catch, if not then
		c) deflected (either straight down or bounce off guys head)


	Numbers changed 10/2/96 to favor offense catching

********************************************************************/

//GAME_OPTNS_WEATHER 	            0-normal 1-rain 2-snow
WORD ExtraWeatherPassCatchPercent[MAX_WEATHER_OPTIONS] = {0,-10,-15};


void DeterminePassOutCome(void)		/* in a ball state	*/
{
EFFVAR posdefguy,posoffguy;
EFFVAR passtypepercent,i;
//WORD passtypepercent,rnum,i;

	InterceptPercent= 0;		// globals (for testing)
	OffCatchPercent0 = OffCatchPercent = 0;
	OffRandomNum = DefRandomNum = 0;

	CatchStatus = 0;	//a testing/debug global variable

		// can't catch if deflection is going up
	if( (CurrentBallDeltaZ > ZERO)
	  && (BallFlags & PASS_DEFLECTED_BFLAG) )
		return;

	posoffguy = OffCanGetBall();
	posdefguy = DefCanGetBall();


	if(posdefguy == NOBODY && posoffguy == NOBODY)
		return;	// NOBODY touched it
//	if(posoffguy > 5)                tackle illegible??
//		return;	// lineman touched it

//	passtypepercent = 10 - PassType*2;	// easier to catch lob(1) than bullet(5)
	// below if 10 passtypes
//	passtypepercent = 10 - PassType;	// easier to catch lob(1) than bullet(10)
	passtypepercent = (10 - PassType)/2; // easier to catch lob(1) than bullet(10)(10/2/96)

/***    Pass Interfer okay on intercept 4/21/07
	if(PassInterferenceValid())
	{
		InterceptPercent = -400; // no way for interception if P.I.
#if 0	//PC_ALWAYS_MODE
//		InterceptPercent= OffCatchPercent = -500;	// test only !!!no chance to catch
#endif
	}
****/
	if(BallOutOfEntireField())	// returns TRUE if Ball outof bounds or over Endlines
	{
		InterceptPercent= OffCatchPercent = -500;	// no chance to catch
	}


	if(ItsAFleaFlickerPlay())	// okay, it's a cheat
	{
		InterceptPercent += 10;
		OffCatchPercent -= 10;	// small chance to catch just cuz
	}

//	if(posoffguy >= ZERO && posoffguy < 6)	// no lineman catching
	if(posoffguy >= ZERO && posoffguy < 8)	// no lineman catching
	{	// off can catch, assume best possible catch

		CatchStatus = 1;

		InterceptPercent -= 40;

	  // base percent = 66% to 66+20+8+4 = 98%  then subtract
//		OffCatchPercent = 66+OPHands(posoffguy)/5 + passtypepercent;
	  // base percent = 74% to 72+19+5+4 = 102%  then subtract (10/2/96)
		OffCatchPercent += (74+OPHands(posoffguy)/5 + passtypepercent);

		if(	FindDirection2Points(OPXPos(posoffguy),OPYPos(posoffguy),BallXPosition,BallYPosition)
			==  OPFacing(posoffguy))
			OffCatchPercent += 4;   // if looking at it better chance

		OffCatchPercent += ExtraWeatherPassCatchPercent[GameOptions[GAME_OPTNS_WEATHER]];

		if(HumanEffectValid(posoffguy))
		{	// human rec. get X extra points
			OffCatchPercent += 5;
			if(OPZPos(posoffguy) > 1)	// human jumping for ball???
				OffCatchPercent += 5;

		}

		OffCatchPercent0 = 	OffCatchPercent; // this is for debugging purposes only - bug # 2009

		FigureDefenseKeyDistance(posoffguy);

		if(DefenseKeyDistance[0].distance < 3*HALFYARD)	//was 1.5
		{ // a defender is really close - lose 20% + defenders "hands"

			PassDefender = DefenseKeyDistance[0].slotnum;

			OffCatchPercent -= 20;
			OffCatchPercent -= OPHands(PassDefender)/4;

			if(HumanEffectValid(posoffguy))
			{	// if defender is human, rec. get X fewer points
				OffCatchPercent -= 5;
			}

			CatchStatus = 3;

			if(DefenseKeyDistance[1].distance < 5*HALFYARD)	//was 2.0
			{ 	// double covered
				OffCatchPercent -= 15;
				CatchStatus = 5;

				if(DefenseKeyDistance[2].distance < 7*HALFYARD)	//was 3.5
				{	// triple covered
					OffCatchPercent -= 15;
					CatchStatus = 6;
				}

			}
			else if(DefenseKeyDistance[1].distance < 7*HALFYARD)	//was 3.5
			{	// second guy sorta close
					CatchStatus = 4;
					OffCatchPercent -= 8;
			}
		}
		else if(DefenseKeyDistance[0].distance < 5*HALFYARD)	//was 2.0
		{ 	// closest guy is sorta close
				CatchStatus = 2;
//				OffCatchPercent -= 10;
				OffCatchPercent -= 14;	//(10/2/96)
				PassDefender = DefenseKeyDistance[0].slotnum;
		}


	}

	if(posdefguy >= ZERO )
	{	// def can catch

		CatchStatus += 10;

		if(BallGoingUpMacro) 	// ball is still going up
		{
			InterceptPercent -= 30;
		}
		else	// ball coming down
		{
//			InterceptPercent += 50;
			InterceptPercent += 45;
			if(HumanEffectValid(posoffguy))
			{	// human rec. get X extra points
				InterceptPercent += 5;
			}
		}

/*		// defense base % = 50% to 50+25+8+4 = 87% (if wide open)
		// (then subtract
		InterceptPercent += (OPHands(posdefguy)/4 + passtypepercent);
*/

	// new 10/9/96 defense base % = 45% to 45+25+5+4 = 79% (if wide open)
		// (then subtract
		InterceptPercent += (OPHands(posdefguy)/4 + passtypepercent);
		if(	FindDirection2Points(OPXPos(posdefguy),OPYPos(posdefguy),BallXPosition,BallYPosition)
			==  OPFacing(posdefguy))
			InterceptPercent += 4;   // if looking at it better chance

		i = Distance2Players(posdefguy,FindClosestOffPlayer(posdefguy));

		if(i < 6*HALF_YARD)
		{	// off. is close to def
			CatchStatus += 20;
//			InterceptPercent /= 3;
			InterceptPercent /= 4;	//10/9/96 - too many ints

		}
//		InterceptPercent = 200;	// test only

	}

#if 0
	//TEST only !!!!
	if(!(BallFlags & PASS_DEFLECTED_BFLAG))
	{
		SetPassDeflection();
		return;
	}
#endif

	DefRandomNum = RANDOM_PERCENT_MACRO;	// get a number 0-100
	if(DefRandomNum < InterceptPercent)
	{
		SetInterception(posdefguy);
//		AIPlayEffectCheck(CATCH);	just said interception
	}
	else if( (OffRandomNum = RANDOM_PERCENT_MACRO) < OffCatchPercent)
	{
		SetNewBallCarrier(posoffguy);

		/* screen blockers used to look for PassCatcher, but then
		they wouldn't block for backward passes - below fixes it	*/
		SetBit(PlayFlags,OK_TO_SCREEN_BLOCK_BFLAG);

		if(PlayFlags & PASS_PLAY_BFLAG)
//		if(BallDeltaX > ZERO)	// must be forward to get credit for catch
		{
			PassCatcher = posoffguy;
		}
//		PassCatcher = posoffguy; // used later for stats

	//	LastBallCarrier = BallCarrier = NewBallCarrier;
//		BallFlags = ZERO;	// don't show	4/1/96
//		BallTargetX = BallTargetY = ZERO;
//		SetBallAction(PlayerHasBall);


		AIPlayEffectCheck(CATCH);

	/*   have receiver who just caught ball to run like hell to End Zone	*/
		OPScriptPtr(posoffguy) = All_Scripts[BallCarrierAvoidScriptNUM];
		OPStateInit(posoffguy) = FALSE;

		SetReceiversToLeadBlockers();		// except BC
		ClearReceivers();	// don't clear receivers if deflected!!!

		CheckForStandingCatch(posoffguy);

		if ( (i = GameTeamData[OffensiveSlotNum].ControllerNum[0]) != -1)
			SwapControllers(posoffguy, ControlWhichPlayer[i]);
	}
	else
	{	//deflect or knock down
		SetPassDeflection();
	}

	TurnOffKickInterface();		//make sure ball arc display off
//	return;

}




void SetInterception(WORD defguy)
{
EFFVAR i;

	AIPlayEffectCheck(INTERCEPT_SFX);
	SetNewBallCarrier(defguy);
	PassInterceptor = defguy;
//	PassCatcher = defguy;

	SetPlayersForDefJustGotBall();
	InitCatchBallX();	// used for stats stuff later
	ClearReceivers();

	/*  team captain gets to control interceptor	*/
/*  SwapControllers won't switch if old isn't controlled OR new is already controlled */
	if ( (i = GameTeamData[DefensiveSlotNum].ControllerNum[0]) != -1)
		SwapControllers(defguy, ControlWhichPlayer[i]);

	HomeOrAwayAudioCrowdReaction(OHHH_SFX, YEAH);

}

/*------------------------------------------------------------*/

WORD StraightDownDeflectionPercent[5] =
//	   { 70,80,95,1000};	// 1000 forces straight down if 4th deflection
//	{ 70,76,85,95,1000};	// bug #2509	- few deflections (0 index not used)
	{ 88,88,92,95,1000};	// 11/15/96 - even fewer deflections

void SetPassDeflection(void)
{
EFFVAR rannum,mask;
BOOL randomdeflect = TRUE;	// assume true deflection(not straight down)


	CatchBallX = BallTargetX;	// in case of pass interference
	BallTargetX = BallTargetY = ZERO;	/* bug # 1379	*/

	if(BallFlags & NOBODY_CATCH_PASS_BFLAG)
		return;

	if(++NumDeflections > 3)
	{
		randomdeflect = FALSE;
	}
	else if ( randper  < StraightDownDeflectionPercent[NumDeflections])
		randomdeflect = FALSE; 	// make deflection go  straight down


//	randomdeflect = TRUE;	// test only
//	randomdeflect = FALSE;	//test only

	if(randomdeflect)
	{
		rannum = rand();
		if( (CurrentBallDeltaZ > ZERO) // still going up
		   ||  (rannum & 0x0100) )		// 50%
			BallDeltaX = - BallDeltaX;		// make it go backwards

		if(rannum & 0x0200)
			BallDeltaY = - BallDeltaY;

		mask = (rannum & 0x0007) + 17-6;	// get numerator
		BallDeltaX = (BallDeltaX*mask)>>5; 	// divide by 32

		mask = ((rannum>>3) & 0x0007) +17-6; // was 17-24 32ths
		BallDeltaY = (BallDeltaY*mask)>>5;	// 2/11/96 11-18 32ths-bug#1420

		mask = (rannum & 0x0400) ? 2:3;

		CurrentBallDeltaZ  = ConstantBallDeltaZ * mask;	// test 11/29/95 was 3
		InitialBallGravity = 2;
	//	BallZPosition += FOOT;

	// when PASS_DEFLECTED_BFLAG set, players run to ball, not balltarget
		SetBit(BallFlags,PASS_DEFLECTED_BFLAG);
	}
	else	// go straight down
	{
		BallDeltaX = BallDeltaY = ZERO;
		CurrentBallDeltaZ = -140;
		CurrentBallGravity = 100;	// high so it never goes to zero

		SetBit(BallFlags,NOBODY_CATCH_PASS_BFLAG);

	}
}







/**********************************************************************

			 H U M A N      K I C K I N G     I N T E R F A C E

	Kicking happens in two parts:

	1) When all players are ready, a kick cursor comes on the field
	which the human can move around the field (X,Y,and Z) based on
	the ratings of his kicker (he can do this as long as the play call
	clock doesn't expire). An arc is drawn from the target to the ball.

	2) When the human has the cursor where he wants it, he presses the
	HIKE button. This starts the play (kicker moving on KO, or ball hiked
	on punt/FG) and then a quick moving cursor moves over the arc. The
	human must press the hike again at the "maximum" height to achieve
	the longest and straightest kick.  Once the button is pressed the
	second time, the true kick parameters will be figured depending on
	the relationship between "BallArcCounter" and "PointsInBallArc"


**********************************************************************/


/*---------------------------------------------------------------------

		I N I T    K I C K I N G   V A R I A B L E S


	Inits all kick (punt, KO, or FG) variables and turns on interface
	if human kicker.


	Uses the actual kicker's rating to determine MAX distance(X only) of
	kick.  Actual distance figured when toe touches ball based on the
	Kick Interface values. (anywhere form 50-100% of max distance based
	on human interface).

	All kicks take the kickers rating and divide by 5 to get a number
	0-20 which is added to a constant to get max kick distance.

	KO goes 55-75 yards.		(kickx = startballx)
	Punt goes 40-60 from LOS.	(kickx = startballx-11.5 yards)
	FG goes 55-75 yards from KickX (kickx = startballx-7 yards)
		(FG assumes target 10 yards behind endline to be good, so a
		min kick (55 yards) is really a 45 yard kick from the 38 yardline.)



	Inits even if it's a fake punt or FG

---------------------------------------------------------------------*/


#if FRAME_RATE_30HERTZ	// these ball speeds are for the new 15HTZ movement rate (x1.33) 4/8/96
	#define INIT_KICK_BALLSPEED		275	//260
	#define INIT_KICKOFF_BALLSPEED	210	// make KOs slow so def. can get there
	#define KICKOFF_BS_VARIANCE		20	// plus/minus limits on KO
	#define MIN_KICK_BALLSPEED		210	//200
	#define MAX_KICK_BALLSPEED		325	//320

	#define	KICK_X_INC			(YARD/2)
	#define	KICK_Y_INC			(YARD/2)
	#define	KICK_Z_INC			1

#else	// old 20 hTZ movement - now new again 7/25/96
	#define INIT_KICK_BALLSPEED		200	//
//	#define INIT_KICKOFF_BALLSPEED	160	//make KOs slow so def. can get there
	#define INIT_KICKOFF_BALLSPEED	150	//make KOs slow so def. can get there
	#define KICKOFF_BS_VARIANCE		12  // plus/minus limits on KO
	#define MIN_KICK_BALLSPEED		150	//
	#define MAX_KICK_BALLSPEED		250	//

	#define	KICK_X_INC			(YARD/2)
	#define	KICK_Y_INC			(YARD/2)
	#define	KICK_Z_INC			1

#endif

/**** reference only - could be wrong - see FBEQU.h
#define	GOAL_POST_WIDTH		(8*YARDS-16*INCH)	// better for rounding error

#define LOW_GOALPOST_Y 		(MID_FIELDY - ((GOAL_POST_WIDTH)/2))
#define HIGH_GOALPOST_Y 	(MID_FIELDY + ((GOAL_POST_WIDTH)/2))
----------------------------------------------------*/

WORD ExtraKickDistance[MAX_WEATHER_OPTIONS] = {0,-3*YARDS,-9*YARDS};
//WORD GPOffsetsForHumanFG[3] = {1*YARDS,5*YARDS,9*YARDS};
//WORD   GPOffsetsForHumanFG[3] = {1*YARDS,3*YARDS,6*YARDS};	// 11/3/96
WORD   GPOffsetsForHumanFG[3] = {-2*YARDS,1*YARDS,5*YARDS};	// 11/4/96


void InitKickInterfaceVars(void)	// called once from Play STATE
{
EFFVAR defplay,ks;
EFFVAR rnum;
EFFVAR exkdist;

	exkdist = ExtraKickDistance[GameOptions[GAME_OPTNS_WEATHER]];
	OnSideKickType = NO_ONSIDE;	//assume

	if(PlayFlags & POS_ANYKICK_PLAY_BFLAG)
	{

		BallSpeed = INIT_KICK_BALLSPEED;	//default

		KickStartZ = 1;	// ball will be kicked 1/7 inch above ground(except punt)

//temp var. - assume kick slightly right so kick arc looks 3D from behind- bug #1567
		defplay = 3*YARD;
		if(StartBallY > MID_FIELDY)
			defplay = -defplay;     // kick slightly left if on right
		KickTargetY = StartBallY + defplay;


		if(PlayFlags & POS_KO_PLAY_BFLAG)
		{		//kicking right to left (logically high to low)

			BallSpeed = INIT_KICKOFF_BALLSPEED;	//override default

//			ConstantBallDeltaZ = 25; //20;
//			ConstantBallDeltaZ = 15; //20;
			ConstantBallDeltaZ = 12;	// 20 hertz @ slow kick speed

			InitialBallGravity = 5;

			KickerSlot = 11;	// a global also used for stats
			KickerStrength = OPKickStrength(KickerSlot);

			KickStartX = StartBallX;

			MaxBallTargetX = StartBallX - 5*YARDS;

			KickTargetX = MinBallTargetX =		// rating = 0-100 -> 0-20 yards
//					StartBallX - 55*YARDS - (KickerStrength* YARDS/5);
					StartBallX - 60*YARDS	// kicker farther 7/15/96 mjk
						- (KickerStrength* YARDS/5) - exkdist;

//		KickTargetX = MinBallTargetX =	DEF_GOAL_LINE-5*YARDS;	//test only

			defplay = GameTeamData[DefensiveSlotNum].PlayNum;

			if(DefenseIsCPUTeamMacro) // CPU kicking??
			{	// CPU uses special flags - not play based

				if(KickOffType == ONSIDE_KO_TYPE)
				{
					if(defplay == KickoffStackLeftNUM)
						OnSideKickType = ONSIDE_LEFT;
					else   // regular form or stack right
						OnSideKickType = ONSIDE_RIGHT;
				}
				else if(KickOffType == SQUIB_KO_TYPE)
				{
					KickTargetX = StartBallX - 45*YARDS;

					ConstantBallDeltaZ = 4;	// lower parameters than normal kick
					InitialBallGravity = 3;
				}
			}
			else // human kicking
			{
				if(defplay == KickoffStackLeftNUM)
					OnSideKickType = ONSIDE_LEFT;
				else if(defplay == KickoffStackRightNUM)
					OnSideKickType = ONSIDE_RIGHT;
			}

			if(OnSideKickType == ONSIDE_LEFT)
			{		// bug # 1538 - set up short kicks for human
				KickTargetY = StartBallY + 14*YARDS + (rand()&0x3ff); // 16-20 yards
				KickTargetX = StartBallX - 9*YARDS - (rand()&0x3ff); // 8-12 yards
			}
			else if(OnSideKickType == ONSIDE_RIGHT)
			{
				KickTargetY = StartBallY-14*YARDS - (rand()&0x3ff);	 // 16-20 yards
				KickTargetX = StartBallX-9*YARDS - (rand()&0x3ff);  // 8-12 yards
			}

		}
		else	// punt or FG go left to right
		{
			MinBallTargetX = StartBallX+9*YARDS; 	// I like 9
			InitialBallGravity = 4;

			if(PlayFlags & POS_PUNT_PLAY_BFLAG)
			{
//				ConstantBallDeltaZ = 8;	//22;	//30;	was 8 for Sony 4/1/96
				ConstantBallDeltaZ = 14;	// made higher as per bug #1762


				KickStartZ = BALL_BEING_CARRIED_HEIGHT;	// ball not kicked off ground
				KickerSlot = 0;		// actually the punter
				KickerStrength = OPKickStrength(KickerSlot);

			// bug # 1660 - punter out of EZ if on 1 YRD
//				KickStartX = StartBallX-21*HALFYARD;	// where ball will be kicked from
//				KickStartX = StartBallX-18*HALFYARD; // where ball will be kicked from
				KickStartX = OPXPos(0)+3*HALFYARD; // this is best-in fornt of punter

		/* punter kicks 40-60 yards from LOS depending on rating 0-100  */
				if(GameTeamData[OffensiveSlotNum].PlayNum == ShotGunPoochKickNUM)
					exkdist -= 13*YARDS;
				KickTargetX = MaxBallTargetX =	// rating = 0-100 -> 0-20 yards
						StartBallX + 40*YARDS + (KickerStrength* YARDS/5 + exkdist);


				if(OffenseIsCPUTeamMacro) // CPU kicking??
				{
					KickerAccuracy = GetPlayerDataPtr(KickerSlot)->ratings[KICKACCURACY_RATE];
					if(StartBallX > MID_FIELDX-1*YARD)
					{	// go for coffin corner
						rnum = rand();
						if(rnum & 1)
						{
							KickTargetY =
								NORTH_SIDELINE - (5*KickerAccuracy) + (rnum & 0x7ff);
						}
						else
						{
							KickTargetY =
								SOUTH_SIDELINE + (5*KickerAccuracy) - (rnum & 0x7ff);
						}
						if(KickTargetX > OFF_GOAL_LINE + 2*YARDS)
							KickTargetX = OFF_GOAL_LINE + 2*YARDS - (rand()&0x7ff);

					}
				}
			}
			else    //-------------- FIELD  GOAL ---------------------
			{
				KickerSlot = 4;
				KickerStrength = OPKickStrength(KickerSlot);
 //				KickerStrength = 100;	// test bug # 2963


				KickStartX = StartBallX-7*YARDS;   	// where ball will be kicked from

		/* FG kicks 55-67 yards from KickX depending on rating 0-100  -
			this assumes target is 10 yards beyond end line
			--------  not anymore 10/7/96 -----  */

		// rating = 0-100 -> 0-20 yards + constant
//				MaxKickDistance = 55*YARDS + (KickerStrength* YARDS/5) + exkdist;
//				MaxKickDistance = 45*YARDS + (KickerStrength* YARDS/5) + exkdist; //bug#2963-too far
				MaxKickDistance = 45*YARDS + (KickerStrength* YARDS/4) + exkdist; //(11/2/96)

				KickTargetX = MaxBallTargetX = KickStartX + MaxKickDistance;
				KickTargetY = MID_FIELDY;	// assume middle
/*
				if( (PlayFlags & POS_HUMAN_KICK_BFLAG)	// flag set in InitAI
				   && GameOptions[GAME_OPTNS_DIFFICULTY] > 0) // not EASY MODE
				{	// goal post is about 8 yards wide
					KickTargetY = LOW_GOALPOST_Y - 2*YARDS + (rand()%(12*YARDS));
				}

*/
				if(PlayFlags & POS_HUMAN_KICK_BFLAG)	// flag set in InitAI
				{	// Y position depends on difficulty
/*					if(PlayType == EX_PT1_PLAY)
						rnum = 1*YARDS;	//2*YARDS
					else
						rnum =  GPOffsetsForHumanFG[GameOptions[GAME_OPTNS_DIFFICULTY]];
  */
					rnum =  GPOffsetsForHumanFG[GameOptions[GAME_OPTNS_DIFFICULTY]];

					rnum += (rand()&511);	// 2 YARDS

					if(PlayType == EX_PT1_PLAY)
						rnum /= 4; 	// closer to center if EX PT

					if(rnum&1)
						KickTargetY = LOW_GOALPOST_Y - rnum;
					else
						KickTargetY = HIGH_GOALPOST_Y + rnum;

				}

//				ConstantBallDeltaZ = 8;	//30;  //50;	//high kick
//				ConstantBallDeltaZ = 18;	//14;	// made higher as per bug #1762
//				ConstantBallDeltaZ = 28; //even higher 7/15/96 2:00pm
//				ConstantBallDeltaZ = 25; //too high 7/15/96  5:15pm
//				if(PlayType == EX_PT1_PLAY)
				if(StartBallX > (OFF_GOAL_LINE - (8*YARDS+1)))	//PlayType == EX_PT1_PLAY)
					ConstantBallDeltaZ = 24; //PAT higher than FG
				else
					ConstantBallDeltaZ = 18; //20HTZ 7/28/96
			}

		}


		KickerAccuracy = GetPlayerDataPtr(KickerSlot)->ratings[KICKACCURACY_RATE];

	  /*  keep all parameters inside the stadium	*/
		MinBallTargetX = max(DEF_BACK_LINE,MinBallTargetX);
		MaxBallTargetX = min((OFF_BACK_LINE+(20*YARDS)),MaxBallTargetX);
		AdjustKickTargetX();


		if(!(PlayFlags & FAKE_KICK_PLAY_BFLAG))	// bug # BK 1 don't screw with fake punt/fg pass
			PassType = LOB_PASS;	// in case of ball bouncing		*/

#if 0 // test only- use when changing ball speed or frame rate or...
		for(defplay= 2; defplay < 25; defplay +=2)
		{
			ConstantBallDeltaZ = defplay;	// test
			FigureKickArc();
		}
#else
		FigureKickArc();
#endif

		if(PlayFlags & POS_HUMAN_KICK_BFLAG)	// flag set in InitAI
		{
			BallArcCounter = ZERO;	// init cursor at bottom

		/* except Kick off in Normal or hard mode (has random init)
			if(PlayFlags & POS_KO_PLAY_BFLAG	// KO
			   && GameOptions[GAME_OPTNS_DIFFICULTY] > 0) // not EASY MODE
			{  // init COunter at random place on ARC
				if(MaxPointsInBallCursor>ZERO) // legal to divide?
					BallArcCounter = rand()%MaxPointsInBallCursor;
			}
				not any more 2/21/97 	***/

//			BallArcCounterDir = 4;	used with ARC DIVISOR =2
//			BallArcCounterDir = 2 + (101 - KickerStrength))/3;
//			BallArcCounterDir = 3;	// test only - uniform

		/* use Strength to get # points to move cursor each frame
			this will be scaled to 1 to X(4tweaked) points per frame	*/
			ks = KickerStrength;
			if (ks < 50)
				ks = 50;
		/* init negative so 1st point is at bottom (if not KO)
		 COunter is scaled by 100 so 200 = 2 points per move(frame)	*/

		// range now 100(good) to X (bad)
//			BallArcCounterDir = -(100 + ((100 - ks) * 10));	// range =100-600
			BallArcCounterDir = -(100 + ((100 - ks) * 7)); //range = 100-450 (11/3/96)
//
//			BallArcCounterDir = 100 + (100 - KickerStrength) * 3;
#ifdef SATURNMODE
//			BallArcCounterDir = 150;
#endif

			SetBit(InterfaceFlags,KICK_INTERFACE_ACTIVE_BFLAG);
		}
		else		// CPU will kick
		{

		}
	}
}


WORD GetTrueBallArcCounter(void)
{
	return(BallArcCounter/100);	// scaled as of 8/6/96
}

WORD GetTrueMaxPointsInBallCursor(void)
{
	return(MaxPointsInBallCursor/100);	// scaled as of 8/6/96
}

BOOL AdjustKickTargetX(void)	// returns TRUE if a max/min was reached
{
	if(KickTargetX < MinBallTargetX)
	{
		KickTargetX = MinBallTargetX;
		return (TRUE);
	}
	else if(KickTargetX > MaxBallTargetX)
	{
		KickTargetX = MaxBallTargetX;
		return (TRUE);
	}

	return (FALSE);

}



#define TEST_KICK_RANGE  0
#define MAGICNUM	1500
/*----------------------------------------------------------------

				H U M A N    K I C K I N G


	Human can move the Ball Target (X,Y) or the
	Height (ConstantBallDeltaZ).

	WARNING -  figuring ball arc interferes with long hikes (i.e.
	on punts, FG). Therefore, human can move the kick target X,Y,Z
	only before the snap. Once the ball is snapped, then the fast
	moving cursor moves over the ball arc - the human is supposed
	to press the button at the highest part of the arc.

	The final kick arc is determined only
	when the ball is actually kicked (in the kickers STATE).

	Note:  MAGICNUM is just a weird number to try to tie in the
	relationship between kick distance and kick height.

----------------------------------------------------------------*/
//#define MAX_KICK_CBDZ	50	//45		// was 60

void HumanKickInterface(void)
{
EFFVAR kickpad, ballspeedinc;
EFFVAR KickXUpButton=KICK_X_UP_BUTTON;
EFFVAR KickXDownButton=KICK_X_DOWN_BUTTON;
EFFVAR KickYUpButton=KICK_Y_UP_BUTTON;
EFFVAR KickYDownButton=KICK_Y_DOWN_BUTTON;
EFFVAR AnyKickButton;
EFFVAR temp,kickxinc,kickyinc;
EFFVAR minkickballspeed = MIN_KICK_BALLSPEED;
EFFVAR maxkickballspeed = MAX_KICK_BALLSPEED;	// defaults

   if( (InterfaceFlags & KICK_INTERFACE_ACTIVE_BFLAG) == ZERO)
		return;

   SetBit(InterfaceFlags,DISPLAY_BALL_ARC_BFLAG);

   kickpad = AIJoyPadData[KickJPadNum];

//   if(kickpad & JPADRup)
//		ConstantBallDeltaZ = ConstantBallDeltaZ;	// put BP here

	AnyKickButton = KickXUpButton | KickYUpButton | KickXDownButton
		| KickYDownButton |  KICK_Z_UP_BUTTON | KICK_Z_DOWN_BUTTON;

	if(BallSnapped)
	{

		if( (PlayFlags & FAKE_KICK_PLAY_BFLAG)
		  && (BallCarrier != NOBODY)  )
		{
			TurnOffKickInterface();
			MaxPointsInBallCursor = BallArcCounter = -1000;	// don't display
		}
		else
		{
	//		if(	((PreviousJoyReading[KickJPadNum] & KICK_BUTTON) == ZERO)
	//		if(	((PrevJpad[KickJPadNum] & KICK_BUTTON) == ZERO)
			if( (TimeIntoPlay > 0)
				&& ((PreviousJoyReading[KickJPadNum] & KICK_BUTTON) == ZERO)
				&& (kickpad & KICK_BUTTON)	// if X pressed again
			  )
			{
// 9/24/96 - now leave arc up even after X presses so they can move UDLR
//				TurnOffKickInterface();	// turn off active & display
				SetBit(MoreEventFlags,HUMAN_KICK_RANGE_DONE_BFLAG);
			}

			if( !(MoreEventFlags & HUMAN_KICK_RANGE_DONE_BFLAG))
//			else
			{
				MoveBallArcCounter();
			  #if PC_GRAPHIC_MODE
				DisplayBallArc(143);	// = color
			  #endif
			}
		}
	}
//	else if( kickpad & AnyKickButton )	removed 7/28/96 - BUg # 2085

	if( kickpad & AnyKickButton )
	{
#ifdef NEW_LONG_HIKE
	// new speed based on kicker strength - range is 130 to 85 (256=YARD)
//		kickxinc = 	kickyinc = 85 + (KickerStrength>>2); 	// fixes bug # 2087
		if( (temp = KickerAccuracy - 30) < ZERO)	//was -50
			temp = ZERO;
//		kickxinc = 	kickyinc = 85 +	temp*4;	//85-285
		kickxinc = 	kickyinc = 65 +	temp*4;	//65-345 bug#3075

#else
		if(++KickButtonCount > 10)	// if button held down a while....
		{
			kickxinc = KICK_X_INC * 3;	// move cursor faster
			kickyinc = KICK_Y_INC * 3;
		}
		else
		{
			kickxinc = KICK_X_INC;
			kickyinc = KICK_Y_INC;
		}
#endif
  // need to know if "X_UP" is moving target closer to or farther from KickStart
		if(KickStartX > KickTargetX)
		{	// must be KO
			ballspeedinc = 2;
			maxkickballspeed = INIT_KICKOFF_BALLSPEED + KICKOFF_BS_VARIANCE;
			minkickballspeed = INIT_KICKOFF_BALLSPEED - KICKOFF_BS_VARIANCE;

#if 1	//__psx__
			temp = KickXUpButton;
			KickXUpButton= KickXDownButton;
			KickXDownButton = temp;
			temp = KickYUpButton;
			KickYUpButton= KickYDownButton;
			KickYDownButton = temp;
#endif
		}
		else
		{	// punt or FG
			ballspeedinc = -2;
		}


		if(kickpad & KickXUpButton)
		{
			KickTargetX += kickxinc;
			if(!AdjustKickTargetX())
			{	// if not at max
				BallSpeed -= ballspeedinc;
				while( ((abs(KickStartX - KickTargetX))/YARD) *ConstantBallDeltaZ > MAGICNUM)
					ConstantBallDeltaZ -= KICK_Z_INC;
			}
		}
		if(kickpad & KickXDownButton)
		{
			KickTargetX -= kickxinc;
			if(!AdjustKickTargetX())
			{	// if not at min
				BallSpeed += ballspeedinc;
				while( ((abs(KickStartX - KickTargetX))/YARD) *ConstantBallDeltaZ > MAGICNUM)
					ConstantBallDeltaZ -= KICK_Z_INC;
			}
		}
		if(kickpad & KickYUpButton)
			KickTargetY += kickyinc;
		if(kickpad & KickYDownButton)
			KickTargetY -= kickyinc;

		/*  check for adjustment to keep in bounds	*/
//		if(KickTargetY < NORTH_SIDELINE)
//			KickTargetY = NORTH_SIDELINE+2*HALFYARD;
//		else if(KickTargetY > SOUTH_SIDELINE)
//			KickTargetY = SOUTH_SIDELINE-2*HALFYARD;

		// bug #1406 - they want to kick OOB
		if(KickTargetY < NORTH_SIDELINE-3*YARDS)
			KickTargetY = NORTH_SIDELINE-3*YARDS;
		else if(KickTargetY > SOUTH_SIDELINE+3*YARDS)
			KickTargetY = SOUTH_SIDELINE+3*YARDS;

#if 0	// removed to "fix" bug # 2209,2086  7/21/96
		if(kickpad & KICK_Z_UP_BUTTON)
		{

		  if( ((abs(KickStartX - KickTargetX))/YARD) *ConstantBallDeltaZ < MAGICNUM)
		  {
			ConstantBallDeltaZ += KICK_Z_INC;
			if(ConstantBallDeltaZ > MAX_KICK_CBDZ)
			{
				ConstantBallDeltaZ = MAX_KICK_CBDZ;
			}
			else
			{
//				BallSpeed -= 5;	// slower speed as height increases

//				while( ((abs(KickStartX - KickTargetX))/YARD) *ConstantBallDeltaZ > MAGICNUM)
//					KickTargetX += YARD;
			}
		  }
		}
		if(kickpad & KICK_Z_DOWN_BUTTON)
		{
			ConstantBallDeltaZ -= KICK_Z_INC;
			if(ConstantBallDeltaZ < 1)
			{
				ConstantBallDeltaZ = 1;
			}
			else
			{
				BallSpeed += 5;	// faster speed as height decreases
				while( ((abs(KickStartX - KickTargetX))/YARD) *ConstantBallDeltaZ > MAGICNUM)
					KickTargetX += YARD;
			}
		}
#endif

		BallSpeed = SetMinMax(BallSpeed,minkickballspeed,maxkickballspeed);
/*		if(BallSpeed < minkickballspeed)
			BallSpeed =  minkickballspeed;
		if(BallSpeed > maxkickballspeed)
			BallSpeed = maxkickballspeed;	*/


		if(ConstantBallDeltaZ < 1)
			ConstantBallDeltaZ = 1;

		AdjustKickTargetX();		// within MIN, MAX values

		FigureKickArc();	// fills BallArc array
	}
	else
		KickButtonCount = ZERO;

//	FigureKickArc();


}



void TurnOffKickInterface(void)
{
	ResetBit(PlayFlags,FAKE_KICK_PLAY_BFLAG);
	ResetBit(InterfaceFlags,(KICK_INTERFACE_ACTIVE_BFLAG | DISPLAY_BALL_ARC_BFLAG));
//	BallArcCounter = -1;	// don't display	NOT HERE - DISRUPTS KICKING

}

/*------------------------------------------------------------------*/

// means curor moves over first 1/ARC_DIVISOR of arc
#define	ARC_DIVISOR		6	//old=5(8/7/96)

void MoveBallArcCounter(void)	// part of human kicking interface
{ // called after BallSnapped as Kicker is running toward ball

// as of 8/6/96 - Counter is scaled by 100
	BallArcCounter += BallArcCounterDir;

	if(BallArcCounterDir < ZERO)
	{
		if(BallArcCounter < ZERO)
		{
			BallArcCounter = ZERO;
			BallArcCounterDir  = -BallArcCounterDir;
		}

	}
	else if(BallArcCounter >= MaxPointsInBallCursor)
	{
		BallArcCounter = MaxPointsInBallCursor;
		BallArcCounterDir  = -BallArcCounterDir;
	}
}



/***********************************************************************/

void FigureKickArc(void)
{
EFFVAR ballframestotarget;

//	BallZPosition = 1;		// some height for QuickFigure

   /*  # frames = distance from QB to possible KickTarget  divided by ballspeed */
	ballframestotarget = Distance2Points(KickStartX,BallYPosition,KickTargetX, KickTargetY)/ BallSpeed;
	ballframestotarget = max(1,ballframestotarget);	// don't call Distance twice in "max" macro

	BallDeltaX = (WORD)(KickTargetX - KickStartX) / ballframestotarget;
	BallDeltaY = (WORD)(KickTargetY - BallYPosition) / ballframestotarget;


//	CurrentBallGravity = InitialBallGravity = ballframestotarget / 16 +1;
//	CurrentBallGravity = InitialBallGravity = 4;
	CurrentBallGravity = InitialBallGravity; 	// now inited depending on kick type

	CurrentBallDeltaZ  = ((ballframestotarget-InitialBallGravity)*
							 ConstantBallDeltaZ)/ (2* InitialBallGravity);

	QuickFigureBallMovement(KickStartX,KickStartZ,NO_WIND,143);

	MaxPointsInBallCursor = (PointsInBallArc/ARC_DIVISOR) * 100; // 100 = scale factor


}


/**********************************************************************

					Figure  Final Kick  Parameters

	When a human presses the kick button the second time (the 1st time
	sets BAllSnapped) OR if the kicker actually kicks ball without the
	button being pressed, then this routine figures where the ball is
	really going (i.e BallTargetX/Y)

**********************************************************************/


void FigureFinalKickParameters(WORD kicktype)
{
EFFVAR xdist,points,percent,p2,p3;
ULONG bignum;
EFFVAR rnum;
UEFFVAR maxdistance;

	points = max(1,	GetTrueMaxPointsInBallCursor());	// no divide by zero
	rnum = rand();

	if(kicktype == FIELD_GOAL_KICK_TYPE)
	{
		if(PlayFlags & POS_HUMAN_KICK_BFLAG)	// flag set in InitAI
		{		// a human is kicking

//			if(InterfaceFlags & KICK_INTERFACE_ACTIVE_BFLAG)
			if( !(MoreEventFlags & HUMAN_KICK_RANGE_DONE_BFLAG))
			{	// kick interface still active, - he didn't press a button

//				percent = (rnum&31) + 40; 	// range 40-71 %
				// modified 1/13/96 due to bug #1243
//				percent = (rnum&31) + 20; 	// range 20-51 %
				// modified 1/20/96 due to bug #1243
///				percent = (rnum&15) + 2; 	// range 2-17 %
//				percent = (rnum&15) + 10; 	// range 10-25 %	(10/21/96)
				percent = (rnum&15) + 15; 	// (11/1/96)

			}
			else // % based on kick meter scaled slightly more than 50-100%
			{
				percent = (((GetTrueBallArcCounter()+1)*50)/points) + 50;
			}
		}
		else	// must be CPU Control
		{
//			percent = (rnum%21) + 80; 	// range = 80 - 100 %
			percent = (rnum&15) + 85; 	// range = 85 - 100 % (10/21/96 - CPU missed too much)

		}

		maxdistance = (MaxKickDistance/100)* percent;
		KickTargetX = KickStartX+maxdistance;
		KickTargetX = min(KickTargetX,OFF_BACK_LINE+10*YARD);

		if( (p2 = FieldGoalHitorMiss(maxdistance)) == MISS_FG_SHORT)
		{
			//make sure it's short - 1-2 past endline
			KickTargetX = min(KickTargetX,OFF_END_LINE+(rand()&511));
			// really don't care much about Y
			KickTargetY = KickTargetY-15*HALFYARD + (rand()&15)*YARD;

		}
		else if (p2 == MISS_FG_WIDE)
		{
//			p3 = 2*YARDS + (rand()&7)*YARDS;
			p3 = -YARD + (rand()&7)*YARDS;		// chance to make it (5/31/96mjk)
			if(StartBallY < MID_FIELDY)
				KickTargetY = LOW_GOALPOST_Y - p3;
			else
				KickTargetY = HIGH_GOALPOST_Y + p3;

		}
		else	// should make field goal (if KtargetY in middle)
		{
			if( KickTargetY > LOW_GOALPOST_Y		// bug # 1505
			 && KickTargetY < HIGH_GOALPOST_Y)
			{
//				KickTargetY = KickTargetY-3*HALFYARD+ (rand()&3)*YARD;
				KickTargetY = KickTargetY-7*HALFYARD+ (rand()&7)*YARD;	// larger chance to miss(5/31/96mjk)
			}
			//else  - leave KickTargetY where it was
		}
	// test only - hit GP?
//		KickTargetY = HIGH_GOALPOST_Y + 6*INCHES;

	}
	else	//------------  punt or KO  ------------------------------
	{
		if(PlayFlags & POS_HUMAN_KICK_BFLAG)	// flag set in InitAI
		{		// a human is kicking
			if( !(MoreEventFlags & HUMAN_KICK_RANGE_DONE_BFLAG))
//			if(InterfaceFlags & KICK_INTERFACE_ACTIVE_BFLAG)
			{	// kick interface still active, - he didn't press a button
//				percent = (rnum&31) + 55;	//range = 55-86%
				// modified 1/13/96 due to bug #1243
//				percent = (rnum&31) + 30;	//range = 30-61%
				// modified 1/20/96 due to bug #1243
//				percent = (rnum&15) + 3;	//range = 3-18%
//				percent = (rnum&15) + 10;	//range = 10-25%	(10/21/96)
				percent = (rnum&15) + 15;	//(11/1/96)
			}
			else	//% based on kick meter scaled to 51-101%
			{
				percent = (((GetTrueBallArcCounter()+1)*50)/points) + 51;
//				percent = (((BallArcCounter+1)*100)/points);
// 				percent /=2;
//				percent +=  50;
			}
		}
		else	// must be CPU Control
		{
//			percent = (rnum%41) + 60; 	// range = 60 - 100 %
//			percent = (rnum&31) + 70; 	// range = 70 - 101 %
			percent = (rnum%21) + 80; 	// range = 80 - 101 % (10/21/96)

			if(PlayType == KICKOFF_PLAY && KickOffType == ONSIDE_KO_TYPE)
				percent = (rnum&7) + 90; // range = 90 - 97 %

		}

	  /* very "straight line" coding for easy debugging	*/

		if( (xdist = KickStartX - KickTargetX) < ZERO)
		{
			xdist = -xdist;		// need abs val.
			bignum = (ULONG)xdist*(ULONG)percent;
			bignum /= (ULONG)100;
			KickTargetX = KickStartX + bignum;
		}
		else	// KO
		{
			bignum = (ULONG)xdist*(ULONG)percent;
			bignum /= (ULONG)100;
			if (bignum < 8*YARDS)
				bignum = 8*YARDS;	// crappy kick goes at least 8 yards
			KickTargetX = KickStartX - bignum;

		}
	/* now figure Y offset - */
		percent = max(0,100-percent);	// 50-100 --> 50-0

/**
//		percent *= YARDS/2;
//		percent += 5*YARDS;	WHAT IS THIS MESS?
		percent *= YARDS/8;
		percent += 3*YARDS;
		p2 = percent/2;

		p3 = randnum(percent)-p2;

		KickTargetY += p3;	//random(percent)-p2;
***/
		percent *= 20;
		if(rnum&0x0010)
			percent = -percent;

		KickTargetY += percent;


	}

	FigureKickArc();		// the final time

	BallTargetX = (ULONG)BTX1/2+(ULONG)BTX2/2;	// tell returner where to go
	BallTargetY = (BTY1+BTY2)/2;
//	BallTargetX = BTX1;	// tell returner where to go
//	BallTargetY = BTY1;


	SaveIRPBallTarget();	// for Instant replay display - bug # 1987

	SetBallDirToBallTarget();	// for graphic display

	TurnOffKickInterface();

	/* start ramp slowly - audio comes down when kick starts to come down
		on punt and KO		*/
	SetNewCrowdVolume(HIGH_CROWD_SFX,CHL2_VOL,2*SECONDS);

	// give human control immediately on onside kicks
//	if(PlayType == KICKOFF_PLAY && KickTargetX > StartBallX-20*YARDS)
//		SetBit1(EventFlags,OK_FOR_HUMAN_OFF_CONTROL_BFLAG);

}



/********************************************************************

			 F I E L D     G O A L     O U T C O M E

	Uses a percentage table based on the kicker's max kick distance
	(i.e. kicker strength). Table index (based on 1 yard per index)
	is adjusted so index 0 is at kickers max kick distance (lowest %).

	Therefore, a kicker with a max distance of 55 yards will miss all
	kicks longer than that, hit kicks at 55 yards FGPercent[0] percent
	of the time, hit kicks at 54 yards FGPercent[1] % of the time.

	(Remember, "good" kicks should go 10 yards pass End line).

	WARNING - this just gives a guideline, calling routine must adjust
	Kick Target X,Y - we only know if it's good when (or if) it goes
	through the goal posts (see BallBeingFGed ballstate).

********************************************************************/

BYTE FGPercentTable[20] =
  //  0, 1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17,18,19,10,
	{10,15,20,25,30,35,40,45,50,55,60,65,70,75,80,85,90,91,92,93};

WORD FieldGoalHitorMiss(UWORD maxdistance)
{
UEFFVAR maxkickx;
EFFVAR diff,percent;
EFFVAR rv;


	maxkickx = OFF_END_LINE+10*YARDS-maxdistance;

	if( (diff = KickStartX - maxkickx) < ZERO)
	{
		rv= MISS_FG_SHORT;	// out of kickers range
	}
	else
	{
		diff /= YARD;		//scale yards to indexes
		if(diff >19)		// if within range...
			percent = 96;	// ... use a high %
		else
			percent = FGPercentTable[diff];	// else use table

	/* adjust percent based on ball Y (on hashmark) ????	*/

		if(randper < percent || (PlayFlags & POS_HUMAN_KICK_BFLAG))
		{	// human can never miss wide (11/4/96)
			rv = MAKE_FG;
		}
		else
		{
			rv = MISS_FG_WIDE;
		}
	}

	return (rv);
}



/*************************************************************************

			 Q U I C K   F I G U R E    B A L L  T A R G E T

  following does ball height (and X,Y) all at once to find correct target
	and sets up arc in BallArc[][]

	These GLOBAL variables MUST have already been set up.
	(but are not modified during this routine)
		BallXPosition;
		BallYPosition;
		BallZPosition;
		CurrentBallDeltaZ;
		CurrentBallGravity;
		InitialBallGravity;
		ConstantBallDeltaZ;


	Sets up these:

		where ball hits ground			BTX2, BTY2;
		where ball below catch height	BTX1, BTY1;
		all points of ball travel		BallArc[][3]

*************************************************************************/

void QuickFigureBallMovement(WORD startxpos, WORD startzpos, WORD usewind, WORD color)
{
register UEFFVAR ballx,bally;
register EFFVAR ballz,curballdeltaz,curballgrav;

	PointsInBallArc = BTX1 = BTX2 = BTY1 = BTY2 = ZERO;	//init globals

	ballx = startxpos;			// not BallXPos on punts, FG
	bally = BallYPosition;
	ballz = startzpos;
	curballdeltaz = CurrentBallDeltaZ;
	curballgrav   = CurrentBallGravity;

	BallArc[0][0] = ballx;	// set inital starting parameters
	BallArc[0][1] = bally;
	BallArc[0][2] = ballz;

	while(ballz > ZERO)
	{
		ballx += BallDeltaX;
		bally += BallDeltaY;

		if(usewind == USE_WIND)
		{
			if(WindAffectsBallMacro(ballz))
			{
				ballx += WindAdjustBallXMacro(ballz);
				bally += WindAdjustBallYMacro(ballz);
			}
		}

		ballz += curballdeltaz;

		if(PointsInBallArc < MAX_POINTS_IN_BALL_ARC-1)
		{
			BallArc[++PointsInBallArc][0] = ballx;
			BallArc[PointsInBallArc][1] = bally;
			BallArc[PointsInBallArc][2] = ballz;
		}


		if(ballz > ZERO)
		{
#if MKPASSING
			if(--curballgrav <= ZERO)
#else
			if(--curballgrav < ZERO)
#endif
			{
				curballgrav = InitialBallGravity;
				curballdeltaz -= ConstantBallDeltaZ;
			}
			if(curballdeltaz < ZERO && BTX1 == ZERO && ballz < GET_BALL_NORMAL_HEIGHT)
			{
				BTX1 = ballx;
				BTY1 = bally;
			}
		}
	}

	BTX2 = ballx;
	BTY2 = bally;

/*	MaxPointsInBallCursor = (PointsInBallArc/ARC_DIVISOR) * 100; // 100 = scale factor
   not here - affects arc cursor during passing		*/

#if PC_GRAPHIC_MODE
	DisplayBallArc(color);

#endif
}



/*************************************************************************

				F I G U R E     P A S S    T A R G E T

	Passes the ball figuring the X, Y, and Z coordinates.  The trick is
	to have the ball and receiver arrive at a spot on the field at the
	same time.  This function is called right when the QB releases the ball
	( frame 10 of the 12 passing animation frames).


	BELOW not TRUE as of 12/27/95
	1) Figure how fast reciever is moving (recdeltax,recdeltay).
	2) Based on ball speed, do binary search to pinpoint  the number of
		frames it will take for the two lines (receiver and ball) to
		intersect.  Since the receiver dir and speed are known, as is
		the BallSpeed, we are more or less solving for Ball direction,
		which is finally determined by BallDeltaX, BallDeltaY.

  ---	12/27/95 ---
	2) Based on ball speed AND receiver script, do binary search to
		pinpoint the number of frames it will take for the two lines
		(receiver and ball) to	intersect.
		We are more or less solving for Ball direction,
		which is finally determined by BallDeltaX, BallDeltaY.


	NOTE: the pass "target" is usually at a Z (height) of 2 YARDS, however
	a pass can be caught anywhere betwen PASS_CATCH_HEIGHT and 0.


	IMportant !!!  - See NOTES on "FigureBallHeight()"


	QB Arm Strength is figured in on both Pass Speed and Max Pass distance
	by taking the rating (0-100) and dividing by a DIVISOR to get a range
	to add to the base.  E.G. divisor = 3 gives a range 0-34 to add to
	whatever base we want.

	Ranges of passes (based on below tables 12/15/95):
					SPEED  		MAX DISTANCE-YARDS (X only)
	NO-PASS			n.a.	    n.a.
	LOB				250-250		42-75
	SEMI-LOB		262-272		37-70
	NORMAL			275-300		36-61
	SEMI-BULLET		287-320		33-55
	BULLET			300-350		30-47


	Used in the following:
		1) instant pitch is released
		2) instant pass is released
		3) In "QuickFigurePass()" - called every frame in passing sequence
			to allow Ball Arc to be display

 *************************************************************************/

#if MAXPASSES_IS_5	// should be false but still here as reference

// NOTE - ALL BallSPeeds slower as of 2/24/96
//WORD PassSpeeds[MAX_PASS_TYPES]    =    {0,250,262,275,287,300};
WORD PassSpeeds[MAX_PASS_TYPES]	       = { 0,200,212,225,237,250};
WORD PassSpeedsDivisor[MAX_PASS_TYPES] = { 1,101, 10,  4,  3,  2};

WORD MaxPassDistance[MAX_PASS_TYPES]
				 = {0,45*YARDS,42*YARDS,38*YARDS,34*YARDS,30*YARDS};
WORD MaxPassDistanceDivisor[MAX_PASS_TYPES]
				 = {1,3,3,4,5,6};

/* bug #1553 - passes need more arc	*/
//WORD	PassConstantBallDeltaZ[MAX_PASS_TYPES] = {1,16,14,12,10, 8};
//WORD	PassConstantBallDeltaZ[MAX_PASS_TYPES] = {1,20,18,16,14,12};
// even more height in arc  3/10/96
WORD PassConstantBallDeltaZ[MAX_PASS_TYPES] = {1,27,24,21,18,15};

#else

// NOTE - ALL BallSPeeds slower as of 2/24/96

#ifdef FRAME_RATE_20HERTZ
#define TK 0	// faster speed 10/16/96 so def. won't double/triple team
/* NOte - relation between Speed and ConBallDZ - if slower, CDBZ must be lower	*/
//WORD PassSpeeds[MAX_PASS_TYPES] = { 0,150,154,159,163,168,172,177,181,186,192};
WORD PassSpeeds[MAX_PASS_TYPES]= { 0,150+TK,154+TK,159+TK,163+TK,168+TK,172+TK,177+TK,181+TK,186+TK,192+TK};
//WORD PassConstantBallDeltaZ[MAX_PASS_TYPES] = {1,18,17,16,14,13,12,11,10,8,6};
WORD   PassConstantBallDeltaZ[MAX_PASS_TYPES] = {1,12,12,11,11,10,10, 9, 8,7,6};

#else	// 15 hertz -

WORD PassSpeeds[MAX_PASS_TYPES]	       = { 0,200,206,212,219,225,231,237,243,250,256};
WORD PassConstantBallDeltaZ[MAX_PASS_TYPES] = {1,27,25,24,22,21,19,18,16,15,13};
#endif

WORD PassSpeedsDivisor[MAX_PASS_TYPES] = { 1,101, 15, 10,  8,  7,  6,  5,  4,  3,  2};

WORD MaxPassDistance[MAX_PASS_TYPES]
 = {0,45*YARDS,43*YARDS,42*YARDS,41*YARDS,40*YARDS,
	  38*YARDS,36*YARDS,34*YARDS,32*YARDS,30*YARDS};
WORD MaxPassDistanceDivisor[MAX_PASS_TYPES]	// used to divide arm strength
				 = {1,3,3,4,4,5,5,6,6,7,7};


#endif

#define TA_OOB (2*YARDS)	// ThrowAwayOutOfBounds max

void FigurePassTarget(WORD receiver, WORD passtype)	// a subroutine of player STATE
{
EFFVAR recdeltax,recdeltay;	/* size of a reciever's steps while ball in air */
EFFVAR speedtablevalue,i;
EFFVAR stepframes, curframes,ballframestointersect;	// used in binary search
UEFFVAR maxpassX;
EFFVAR armstrength,dir;
BOOL Guessing = FALSE;


	passtype = SetMinMax(passtype,1,MAX_PASS_TYPES-1);
/*
	if(passtype >= MAX_PASS_TYPES)		//error correction
		passtype = MAX_PASS_TYPES-1;
	else if(passtype <= ZERO)		//error correction
		passtype = 1;
*/

	armstrength = CPArmStrength;
	BallSpeed = PassSpeeds[passtype] +
							(armstrength / PassSpeedsDivisor[passtype]);

// below is test only !!!!
//	BallSpeed = 250 + (armstrength/10)*25;

//	BallSpeed = PassSpeeds[passtype];

	maxpassX = MaxPassDistance[passtype];

	if(CPMyPNum > QB3)	// HB option or punter
		maxpassX /= 2;	// half as far
	maxpassX += ((armstrength / MaxPassDistanceDivisor[passtype])*YARDS);


  /*  NEW -use following formula to approximate the reciever's speed while
		the ball is in the air	(average of  (current + 2*maxspeed)
				 figure this stuff outside of loop !!
  */

	if(  (OPFlags(receiver) & IS_HCONTROLLED_BFLAG)
		|| ThrowAwayRec > ZERO)
	{	// guess where human will be
		Guessing  = TRUE;
	}
	else // use actual player SCRIPTS/STATE to see where he will be (allows "cutting")
	{	// optimized 4/2/96 - only do this once, not 6-7 times during loop
		if(WhereWillPlayerBe(receiver, 128)) // stores in RecX/Y[]
		{   // above was ABORTED due to weird (blocking) SCRIPT
			Guessing  = TRUE;	// so we guess
		}
	}

	if(Guessing)
	{	// at least have a good guess
		dir = OPDirToGo(receiver); //assume
		if(ThrowAwayRec == 1) // right receiver?
		{
			dir = SOUTH_DIR;
			speedtablevalue = 25;
			receiver = 1;
		}
		else if(ThrowAwayRec == 2) // left receiver?
		{
			dir = NORTH_DIR;
			speedtablevalue = 25;
			receiver = 2;
		}
		else if(OPSpeed(receiver) == ZERO)
		{
			speedtablevalue = ZERO;	// if not running, throw right to him
		}
		else
		{
			speedtablevalue = (	((OPSpeed(receiver) + OPMaxSpeed(receiver)
				 + OPMaxSpeed(receiver)) / 3) >> SPEED_SHIFT );
		}
	/*  use our SpeedTables to get exact values of the reciever's "steps"	*/
		recdeltax =	SpeedTable[speedtablevalue][dir].X;
		recdeltay =	SpeedTable[speedtablevalue][dir].Y;
	}



/* do binary search to find # of frames until pass and receiver intersection */

	stepframes = curframes = 64;	/* was 128, changed 10/17/95 mjk - first
									hit was always too high so changed to 64
									 - could still max at 127	*/

	for (i = 0; i < 11; i++)  // 7 should be all we need in this binary search...
	{
		stepframes /= 2;

		if(Guessing)
		{	// bug #1434
		/*  figure a POSSIBLE Ball Target	*/
			BallTargetX = OPXPos(receiver) + recdeltax * curframes;
			BallTargetY = OPYPos(receiver) + recdeltay * curframes;
		}
		else
		{
	// player needs one extra cause he won't move till next frame
//			WhereWillPlayerBe(receiver, curframes+1);
//			BallTargetX = OPXPos(DUMMY_SLOT_NUM);
//			BallTargetY = OPYPos(DUMMY_SLOT_NUM);

			BallTargetX = RecX[curframes];	// new optimized stuff as of 4/2/96
			BallTargetY = RecY[curframes];	// - RecX/Y figured in WhereWillPlayer Be before loop
//			BallTargetX = RecX[curframes+1];	// new optimized stuff as of 4/2/96
//			BallTargetY = RecY[curframes+1];	// - RecX/Y figured in WhereWillPlayer Be before loop
		}

		/*  check for adjustment to keep in bounds	*/
		if(BallTargetX > OFF_END_LINE)
			BallTargetX = OFF_END_LINE-2*YARD;
		else if(BallTargetX < DEF_END_LINE)	/* beware of UWORD wrap @ 128 YARDS */
			BallTargetX = DEF_END_LINE+2*YARD;

		/*  check for adjustment for max QB arm strength	*/
		if(BallTargetX > BallXPosition + maxpassX)
			BallTargetX = BallXPosition + maxpassX;


	/*  check for adjustment to keep in bounds - NOTE- these are UWORDs	*/
		if(ThrowAwayRec > ZERO) // okay for OOB if throwaway
		{
			if(BallTargetY < NORTH_SIDELINE-TA_OOB || BallTargetY > (UWORD)200*YARDS)
				BallTargetY = NORTH_SIDELINE-TA_OOB;
			else if(BallTargetY > SOUTH_SIDELINE+TA_OOB)
				BallTargetY = SOUTH_SIDELINE+TA_OOB;
		}
		else
		{
			if(BallTargetY < NORTH_SIDELINE || BallTargetY > (UWORD)200*YARDS)
				BallTargetY = NORTH_SIDELINE+CLOSE_TOSL;
			else if(BallTargetY > SOUTH_SIDELINE)
				BallTargetY = SOUTH_SIDELINE-CLOSE_TOSL;
		}


	/* minor logic flaw - since ballframestointersect changes every time
	thru loop based on a changing BallTarget, but curframes changes based
	on binary search algorithm, it's possible that the two miss each other
	(as curframes goes keeps getting smaller (closer to binary search target)
	that target is changing i.e. binary search is valid only to search for a
	constant, "ballframestointersect" is NOT a constant during search
	okay, so we get a few incomplete passes.
	*/
   /*  # frames = distance from QB to possible BallTarget  divided by ballspeed */
		ballframestointersect = Distance(BallTargetX, BallTargetY) / BallSpeed;

		if (ballframestointersect < curframes)
			curframes -= stepframes;
		else if (ballframestointersect > curframes)
			curframes += stepframes;
		else if (ballframestointersect == curframes)
			break;		/*  success !!!	*/
	}

#if 0	//PC_DEBUG_MODE
	printf("\n%d %d %d",i,curframes,ballframestointersect);
	if(i >= 11)	// testing - error BP here
		printf("\nToo many I's in FigurePassTarget");
#endif

//	BallDeltaX = (WORD)(BallTargetX - PlayerXPosition[BallCarrier]) / curframes;
//	BallDeltaY = (WORD)(BallTargetY - PlayerYPosition[BallCarrier]) / curframes;
	BallDeltaX = (WORD)(BallTargetX - CPXPos) / curframes;	// don't use BC (as he might not exist)
	BallDeltaY = (WORD)(BallTargetY - CPYPos) / curframes;


/*  now set up Gravity parameters based on pass type and distance	*/


#if 0	//MKPASSING
	if(passtype == LOB_PASS)
	{
  #if PC_DEBUG_MODE
	 /*** begin test	****/

//		CurrentBallGravity = InitialBallGravity = curframes/12 + 1;
	#if 0
		InitialBallGravity=1;
		i = InitialBallGravity+12;
		do
		{
			CurrentBallGravity = InitialBallGravity;
			ConstantBallDeltaZ = 16;
			CurrentBallDeltaZ  = ((curframes-InitialBallGravity)*
								 ConstantBallDeltaZ)/ (2* InitialBallGravity);
			QuickFigureBallMovement(BallXPosition,BallZPosition,NO_WIND,243);
		}while(++InitialBallGravity < i);
	/*  end test part 1***/
	#endif
	#if 1
		CurrentBallGravity = InitialBallGravity = curframes/12 + 1;
		InitialBallGravity=3;
		i = ConstantBallDeltaZ = 60;
		do
		{
			CurrentBallGravity = InitialBallGravity;
			ConstantBallDeltaZ = i;
			CurrentBallDeltaZ  = ((curframes-InitialBallGravity)*
								 ConstantBallDeltaZ)/ (2* InitialBallGravity);
			QuickFigureBallMovement(BallXPosition,BallZPosition,NO_WIND,143);
			i -= 4;
		}while( i > 2);

	#endif

	/*  end test part 2***/
  #endif



		CurrentBallGravity = InitialBallGravity = curframes/12 + 1;
		ConstantBallDeltaZ = 16;
		CurrentBallDeltaZ  = ((curframes-InitialBallGravity)*
								 ConstantBallDeltaZ)/ (2* InitialBallGravity);
	}
	else	/* BULLET PASS	*/
	{
		CurrentBallGravity = InitialBallGravity = curframes/12 + 1;
		ConstantBallDeltaZ = 8;
		CurrentBallDeltaZ  = ((curframes-InitialBallGravity)*
								 ConstantBallDeltaZ)/ (2* InitialBallGravity);
	}


#endif

	curframes--;	// one less cause ball will move this frame (in BallAction)
	CurrentBallGravity = InitialBallGravity = curframes/12 + 1;
	ConstantBallDeltaZ = PassConstantBallDeltaZ[passtype];
	CurrentBallDeltaZ  = ((curframes-InitialBallGravity)*
								 ConstantBallDeltaZ)/ (2* InitialBallGravity);

/* following used to speed up defense calculations
		 (-1 since AIFcount is after this in loop)		*/
	PosCatchFrame = AIFrameCounter + curframes - 1;

	QuickFigureBallMovement(BallXPosition,BallZPosition,NO_WIND,143+passtype);

#if PC_GRAPHIC_MODE
	DrawCross(BallTargetX,BallTargetY,15);
#endif


}

/**********************************************************************

						K I C K I N G

	Each kick of the 3 kicks has 2 parts 1) initing, 2) in flight

**********************************************************************/
//#define PUNT_RETURNER_SLOT 20

void CheckKickOOB(void)	// used on punt, KO (or bounce)
	//- set OOBX the instant kick goes OOB - even if BallZ != ZERO
{
  // save so we can correctly position ball where it went OOB,
  // not where it hit ground
	if(KickOOBX ==  ZERO)	// already OOB???
	{
		if(BallYPosition < NORTH_SIDELINE+1
		|| BallYPosition > SOUTH_SIDELINE-1
		|| BallXPosition <= DEF_END_LINE
		|| BallXPosition >= OFF_END_LINE	// punt from 1 yardline
		  )
		{
			KickOOBX = BallXPosition;
		}
	}
}


BALL_ACTION(BallBeingPunted)		/*  a BALL ACTION		   */
{
EFFVAR i,guy;

	if(ReplotBallArcForWind())
	{	// tell PRet guy to go get it (if he was already @ target, he wouldn't move)
		for(i=11; i<22; i++)
		{
			if(OPFlags(i) & IM_PUNT_RETURNER_BFLAG)
			{
				OPStateInit(i) = FALSE;	// reset STATE
				OPAIState(i) = NoAI;
			}
		}
	}

	MoveBallInAir(); 	/* updates BallX, BallY, BallSI	*/

	if(!KickBlockedOutcome(PUNT_PLAY))
	{
		if((guy = DefCanGetBall()) != NOBODY)
		{
			InitCatchBallX();	// used for stats stuff later

			if(HeShouldFumbleGeneric(guy) )
			{
				LastBallCarrier = guy;
				InitFumble(BALL_HIT_GUY_FUMBLE,BallDirection,NEED_FUMBLE_SFX);
				ResetBit(BallFlags,BALL_PUNTED_BFLAG);	// no downing punt
			}
			else
			{
				SetNewBallCarrier(guy);
				PuntReturner  = guy;//	KickReturner = guy;
//				SetNewCrowdVolume(HIGH_CROWD_SFX,CHL2_VOL,7*SECONDS);
				SetNewCrowdVolume(HIGH_CROWD_SFX,CHL2_VOL,4*SECONDS);	// 11/21/96

				SetPlayersForDefJustGotBall();

			/* if guy waiting for ball, have a few frames of him catching it*/
				CheckForStandingCatch(guy);

				if ( (i = GameTeamData[DefensiveSlotNum].ControllerNum[0]) != -1)
					SwapControllers(guy, ControlWhichPlayer[i]);
			}
			return;	// do not do below
		}
		else if(!FigureBallHeight())     /* updates Ball Z  - false if hit ground	*/
		{
			SetBallAction(InitBouncingBall);
		}


		if(BallXPosition > OFF_BACK_LINE)	// hit the net?
			BallXPosition = OFF_BACK_LINE;	// hit the net!

		if(BallXPosition > StartBallX  && !(BallFlags & PUNT_PAST_LOS_BFLAG) )
		{
			for(i=11; i<22; i++)
			{
				if( !(OPFlags(i) & IM_PUNT_RETURNER_BFLAG))
				{
					OPScriptPtr(i) = All_Scripts[DefRunToPuntTargetScriptNUM];
					OPStateInit(i) = FALSE;
					OPAIState(i) = NoAI;
				}
			}
//			OPScriptPtr(PUNT_RETURNER_SLOT) = All_Scripts[PuntReturnerNUM];
			SetBit(BallFlags,PUNT_PAST_LOS_BFLAG);
		}

		CheckKickOOB();	// used on punt, KO

	}
}


void SetAudioIfKickComingDown(void)
{
	if(CurrentBallDeltaZ < ZERO)	// kick is coming down
	{
		if(! (PlayFlags & KICK_COMING_DOWN_BFLAG))
		{
			SetBit(PlayFlags,KICK_COMING_DOWN_BFLAG);
			SetNewCrowdVolume(HIGH_CROWD_SFX,CHL1_VOL,5*SECONDS);	// back down
		}
	}
}

BOOL CPUDownKickOff(WORD kr);	// CPU Kick returner


BALL_ACTION(BallBeingKickedoff)		/*  a BALL ACTION		   */
{
EFFVAR i,guy;

//	ReplotBallArcForWind();		// if necessary
	// following "backwards" from ReplotBallArcForWind() due to diff. kick dir.
	if( (BallXPosition < BallArc[PointsInBallArc/3][0]) // 1/3 way in air
	  &&(ReplotBallArcCounter == ZERO)	// none it yet???
	  &&(WindDeltaX | WindDeltaY)	// any wind???
	  )
	{
		ReplotBallArcCounter = 1;
		QuickFigureBallMovement(BallXPosition, BallZPosition, USE_WIND, 200);
		MaxPointsInBallCursor = (PointsInBallArc/ARC_DIVISOR) * 100; // 100 = scale factor
		BallTargetX = (ULONG)BTX1/2+(ULONG)BTX2/2;
		BallTargetY = (BTY1+BTY2)/2;
	}

	MoveBallInAir(); 	/* updates BallX, BallY, BallSI	*/

	SetAudioIfKickComingDown();

	if( (guy = OffCanGetBall()) != NOBODY)
	{
		StartGameClock(); //  when guy touches ball

		InitCatchBallX();	//  so we can figure how long punt was

		if(HeShouldFumbleGeneric(guy) )
		{
			LastBallCarrier = guy;		// bug #1624 - muff kick goes OOB
			InitFumble(BALL_HIT_GUY_FUMBLE,BallDirection,NEED_FUMBLE_SFX);
		}
		else
		{
			SetNewBallCarrier(guy);
			KickReturner = guy;
			SetNewCrowdVolume(HIGH_CROWD_SFX,CHL2_VOL,7*SECONDS);

		/*   have guy who just caught ball to run like hell to End Zone	*/
			OPScriptPtr(guy) = All_Scripts[BallCarrierAvoidScriptNUM];
			OPStateInit(guy) = FALSE;	// bug #1271
			/*  team captain controls the returner */
			if ( (i = GameTeamData[OffensiveSlotNum].ControllerNum[0]) != -1)
				SwapControllers(guy, ControlWhichPlayer[i]);

			CPUDownKickOff(guy);	// CPU Kick returner - if necessary


//		SetPlayersForOffJustGotBall();
		}

	}
	else if(!FigureBallHeight())     /* updates Ball Z  - false if hit ground	*/
	{
		InitFumble(BALL_HIT_GROUND_FUMBLE,BallDirection,NO_FUMBLE_SFX);
		BallTargetX = BallTargetY = ZERO;	// don't display
	}

	CheckKickOOB();	// used on punt, KO

}




/*---  FIELD GOAL  - Check to see if ball goes through up rights	---*/

#define GP_BALL_WIDTH (2*INCHES)	// on each side of GP

BALL_ACTION(BallBeingFGed)		/*  a BALL ACTION - Field Goal		   */
{
UEFFVAR savey;
//UWORD lowgpy = LOW_GOALPOST_Y;
//UWORD highgpy = HIGH_GOALPOST_Y;	// var so can look at in debugger

	ReplotBallArcForWind();		// if necessary

	SetAudioIfKickComingDown();


	savey = BallYPosition;	// might be needed below

	MoveBallInAir(); 	/* updates BallX, BallY, BallSI	*/

	if(!KickBlockedOutcome(FIELD_GOAL_PLAY))
	{
	//	if(BallXPosition > OFF_END_LINE && !(PlayOverFlags & KICK_OVER_BFLAG) )
		if(BallXPosition >= OFF_END_LINE
		  && (!(PlayOverFlags & FG_PAST_GP_BFLAG))
//			 && BallXPosition < OFF_END_LINE+YARD
		  )
		{ // ball just crossed endline
			SetBit(PlayOverFlags,FG_PAST_GP_BFLAG | BALL_DEAD_BFLAG);

			if(		BallYPosition >= LOW_GOALPOST_Y + GP_BALL_WIDTH
				 && BallYPosition <= HIGH_GOALPOST_Y - GP_BALL_WIDTH
				 && BallZPosition >= GOALPOST_Z + GP_BALL_WIDTH
			  )
			{  // it went in
				SetBit1(EventFlags,KICK_GOOD_BFLAG);
				if(OffensiveSlotNum == HOME_TEAM_SLOT)
				{
					AIPlayEffectCheck(AHHH_SFX);
					if(TheQuarter> 4)	// overtime???
					{	// the crowd roars
						SetNewCrowdVolume(MID_CROWD_SFX,CML4_VOL,4*SECONDS);
						SetNewCrowdVolume(HIGH_CROWD_SFX,CHL5_VOL,1*SECONDS);
					}
				}
			}
			else	// missed  - check if hit GP
			{
				if(OffensiveSlotNum == HOME_TEAM_SLOT)
					AIPlayEffectCheck(OHHH_SFX);	// ... too bad

				if( ( BallYPosition < LOW_GOALPOST_Y + GP_BALL_WIDTH
					&& BallYPosition > LOW_GOALPOST_Y-GP_BAR_WIDTH - GP_BALL_WIDTH
					&& BallZPosition > GOALPOST_Z-GP_BAR_WIDTH- GP_BALL_WIDTH
					)
				||
					(  BallYPosition > HIGH_GOALPOST_Y - GP_BALL_WIDTH
					&& BallYPosition < HIGH_GOALPOST_Y+GP_BAR_WIDTH +GP_BALL_WIDTH
					&& BallZPosition > GOALPOST_Z-GP_BAR_WIDTH - GP_BALL_WIDTH
					)
				||		// hit cross bar ??
					(  BallZPosition < GOALPOST_Z + GP_BALL_WIDTH
					&& BallZPosition > GOALPOST_Z-GP_BAR_WIDTH-GP_BALL_WIDTH
					&& BallYPosition > LOW_GOALPOST_Y-GP_BAR_WIDTH-GP_BALL_WIDTH
					&& BallYPosition < HIGH_GOALPOST_Y+GP_BAR_WIDTH + GP_BALL_WIDTH
					)
				  )
				{	// ball hit upright or crossbar
					BallXPosition = OFF_END_LINE;
					BallDeltaX >>= 2;		// reduce speed
					BallDeltaY >>= 2;		// reduce speed

					BallDeltaX = -BallDeltaX;		// bounce back
					if(BallDeltaY < 15 && BallDeltaY > -15)
						BallDeltaY = 15;

					AIPlayEffectCheck(PUNT_SFX);
				}
			}
		}

		if(BallXPosition > OFF_BACK_LINE)	// hit the net
		{
			BallXPosition = OFF_BACK_LINE;	// hit the net
			BallYPosition = savey;	// don't set delta to ZERO!
		}

		if(!FigureBallHeight())     /* updates Ball Z  - false if hit ground	*/
		{
//			BallDeltaX >>= 1;	// bug # 1884  reduce FG bounce
//			BallDeltaY >>= 1;
			BallDeltaX >>= 2;	// bug # 2460  reduce FG bounce even more! 7/19/96
			BallDeltaY >>= 2;

			SetBallAction(InitBouncingBall);
			SetBit(PlayOverFlags,KICK_OVER_BFLAG | BALL_DEAD_BFLAG);

		}

	}
}




void InitPuntedBall(void)		/* a CPU PLAYER STATE  subroutine	*/
{

	if(gPNum != BallCarrier)
		return;

	if(PlayType < EX_PT1_PLAY)	// pouch kick on Ex pt!?!
		PlayType = PUNT_PLAY;

	PassType = LOB_PASS;	// in case of ball bouncing

	FigureFinalKickParameters(PUNT_KICK_TYPE);

	LastBallCarrier = BallCarrier = NOBODY;
	SetBit(BallFlags,SHOW_BALL_BFLAG|SPIRAL_BALL_BFLAG | BALL_PUNTED_BFLAG);

	BallBeingPunted();				// do it now once...
	SetBallAction(BallBeingPunted); //... and forever more

	/*  Offensive can now toggle to closest guy [to BallTarget ] */
	SetBit(GameTeamData[OffensiveSlotNum].TeamFlags,TOGGLE_TO_CLOSEST_VALID_BFLAG);
	AIPlayEffectCheck(PUNT_SFX);

	ClearReceivers();	// no need for scaling ICONS during punt

}




/*-------------------------------------------------------------*/

void InitKickedoffBall(void)	// called from kicker STATE
{		// ball will move WEST
#if TEST_KICK_RANGE
EFFVAR savex,savey,i;
#endif

/* NOTE: All Ball Parameters (i.e. BallSpeed, Target, Gravity, etc)
	MUST have already been set upball in InitKickInterfaceVars
	(Human and CPU).  Human can then also manipulate parameters
	in HumanKickInterface.
*/
	PassType = LOB_PASS;	// in case of ball bouncing

	LastBallCarrier = BallCarrier = NOBODY;

	#if TEST_KICK_RANGE	// test random kick range
	savex = KickTargetX;
	savey = KickTargetY;

	for(BallArcCounter=100; BallArcCounter < MaxPointsInBallCursor/100; BallArcCounter++)
	{
		for(i=0; i<100; i++)
		{
			KickTargetX = savex;
			KickTargetY = savey;
			FigureFinalKickParameters(KICKOFF_KICK_TYPE);
		#if PC_GRAPHIC_MODE
			DrawCross(KickTargetX,KickTargetY,15);
		#endif

		}
	}
	#else
	FigureFinalKickParameters(KICKOFF_KICK_TYPE);
	#endif

	AIPlayEffectCheck(KICK_SFX);

	SetBit(BallFlags,SHOW_BALL_BFLAG|END_OVER_END_BFLAG|BALL_KICKEDOFF_BFLAG);
	BallBeingKickedoff();				// do it now once...
	SetBallAction(BallBeingKickedoff); //... and forever more



}



void InitFieldGoal(void)	//called instant ball is kicked
{
EFFVAR temp;

	if(PlayType == NORMAL_PLAY)
		PlayType =  FIELD_GOAL_PLAY;	//not an Ex Pt

	if( (BallCarrier != HOLDER_SLOT_NUM)
	   || (EventFlags.TACKLE_STARTED_BFLAG)
	  ) // don't kick if holder doesn't have ball (fumble) he's getting tackled
		return;

	PassType = LOB_PASS;	// in case of ball bouncing

	FigureFinalKickParameters(FIELD_GOAL_KICK_TYPE);

	LastBallCarrier = BallCarrier = NOBODY;

//	SetBit(BallFlags,SHOW_BALL_BFLAG|SPIRAL_BALL_BFLAG|BALL_KICKEDFG_BFLAG);
	SetBit(BallFlags,SHOW_BALL_BFLAG|END_OVER_END_BFLAG|BALL_KICKEDFG_BFLAG);

	SetBit(PlayOverFlags,BALL_DEAD_BFLAG);	// nobody pickup if bad kick

	temp = gPNum;
	for(gPNum = 11; gPNum <22; gPNum++)
		SetNewScript(DefAfterFGKickedScriptNUM);	// STATE in FBDefAI.c

	gPNum = temp;

	AIPlayEffectCheck(KICK_SFX);

	BallBeingFGed();				// do it now once...
	SetBallAction(BallBeingFGed); //... and forever more



}







void InitCatchBallX(void)	// called if def gets ball or KickRet
{
	CatchBallX = SETTOWHOLEYARD(BallXPosition);
/*
	CatchBallX = BallXPosition;
	if(CatchBallX <= MID_FIELDX)	// x00_00-x00_ff -> 0x01_00 (i.e 1 YARD)
		CatchBallX += YARD-1;

	CatchBallX &= 0xff00;		// mask low byte	OR x50_00-x50_ff -> x50_00
*/
}


PLAYER_ACTION(JumpUpAction);	// prototypes
PLAYER_ACTION(JumpDownAction);


BOOL KickBlockedOutcome(WORD type)	//Returns true if punt or FG blocked
{
BOOL rv = FALSE;
EFFVAR guy,tempguy;

	if(BallGoingUpMacro) 	// ball is still going up
	{
		if ( (guy = DefCanGetBall()) != NOBODY)
		{
			SetBit(PlayFlags,KICK_BLOCKED_BFLAG);
			BlockedKickBlocker = guy;

			ResetBit(BallFlags,BALL_PUNTED_BFLAG | BALL_KICKEDFG_BFLAG);

			if(PracticeMode)
			{	// end play now - if def. gets ball in Practice we're in troble
				if(type == PUNT_PLAY)
					SetBit(PlayOverFlags,(OFF_DOWNED_PUNT_BFLAG |BALL_DEAD_BFLAG));
				else
					SetBit(PlayOverFlags,KICK_OVER_BFLAG);
			}

			HomeOrAwayAudioCrowdReaction(SWELL1, YEAH);	// away blocked it/home blocked it

			InitFumble(BALL_HIT_GROUND_FUMBLE,BallDirection,NO_FUMBLE_SFX);
			if( OPAction(guy) == JumpUpAction)
//			 || (OPAction(guy) == JumpDownAction) )
			{
			}
			else	//bug #1519 - force a [pre]jump
			{
				tempguy = gPNum;	// save our global
				gPNum = guy;	// NO NO NO okay to use gPNum in a BallSTATE
				CPAction = JumpUpAction;
				InitGraphicsQ(AI_JUMPUP_NUM);

				CPAIGraphicSequence = MAX_FRAMES_MACRO(CPAIGraphicState)/2;
				CPZPos = FOOT;
				gPNum = tempguy;
			}

			/* I didn't want to put stats stuff here - only in
			 CheckForPlayOver()	but with Beta in 1 week... what the heck */
			if(KickerSlot != NOBODY)	// error check
			{
				KickStatsType = GS_PUNT_BLK;
/*
				if(type == PUNT_PLAY)
				{
					UpdateStats(KickerSlot, GS_PUNT,1);
					UpdateStats(KickerSlot, GS_PUNT_BLK,1);
				}
				else if(PlayType == EX_PT1_PLAY || PlayType == EX_PT1_PLAY)
				{
					UpdateStats(KickerSlot, GS_PAT_ATT,1);	// no BlockedPAT stats
				}
				else
				{
					UpdateStats(KickerSlot, GS_FG_ATT,1);
					UpdateStats(KickerSlot, GS_FG_BLK,1);
				}
*/
			}

			rv = TRUE;

		}
	}

	return(rv);
}





/**************************************************************************

			 W H E RE   W I L L   P L A Y E R   B E   ?????

	This routine uses a "dummy" slot to figure out where a player will
	be in X frames if he keeps executing his current script.

	Used by passing AI to see where a receiver will be. Runs normal
	Script but ignores any collisions that could/would/should happen.


	The answer is in PlayerX/YPosition[DUMMY_SLOT_NUM].

	NOTE:
	If the player is human controlled, I don't know what happens, but
	I'm sure one of the testers will tell me in a few weeks. (It shouldn't
	blow up though).

	CAVEATS:
	1) Won't work if ball has not been snapped.
	2) HAs problems if rec. is LeadBlocking

	ref (2) - now a BOOLEAN function that TRUE if "wierdness" happened
	Calling routine will then use "guessing" logic instead of true logic

	"wierdness" defined as throwing to A LeadBlocker(toggle guy) or
	or SaveQB STATE (which is random)

**************************************************************************/
BOOL WhereWillPlayerBe(WORD pnum, WORD numframes)
{
EFFVAR temp,i;
BOOL rv = FALSE;

	temp = gPNum; 	// save "current" player

	gPNum = DUMMY_SLOT_NUM;

/* copy all current relevant info into dummy slot	*/
	CPXPos = OPXPos(pnum);
	CPYPos = OPYPos(pnum);
	CPZPos = OPZPos(pnum);

	CPDirToGo = OPDirToGo(pnum);
	CPFacing = OPFacing(pnum);
	CPTargetX = OPTargetX(pnum);
	CPTargetY = OPTargetY(pnum);

	CPScriptPtr = OPScriptPtr(pnum);
	CPStateInit = OPStateInit(pnum);
	CPAction = OPAction(pnum);


	CPSpeed	 = OPSpeed(pnum);
	CPMaxSpeed	 = OPMaxSpeed(pnum);
	CPAcceleration = OPAcceleration(pnum);
	CPReactTime	 = OPReactTime(pnum);
	CPStrength  = OPStrength(pnum);
	CPControllerNum = NO_CONTROLLER;

	SetAI(NoAI);
	CPBStatePtr = OPBStatePtr(pnum);
	CPState2Count = -1;		// no secondary STATES
	CPTimer = OPTimer(pnum);

	CollisionDetectActive = FALSE;	// in "TryMove()"

	AbortWhereWillPlayerBe = FALSE;  // a global set in blocking routines

	for(i=0; i < numframes; i++)
	{
		DoAction(); 	// will do STATE of current script
		RecX[i] = CPXPos;
		RecY[i] = CPYPos;
		if(AbortWhereWillPlayerBe && i < 40)	//
		{
			rv=TRUE;
			i= i;
			break;
		}
	}

#if PC_GRAPHIC_MODE
//	DrawCross(CPXPos,CPYPos,150);
#endif
	CollisionDetectActive = TRUE;

	gPNum = temp;	// restore "current" player

	return(rv);
}




/*************************  end of  file - FBBall.C  *********************/
