/**************************************************************


					FBBLOCK.C

	INITIAL AUTHOR: M. Heilemann       Start Date: 6/12/95

	Modify Date:


	Copyright 1995  GameTek, Inc.



	Contains blocking routines for football.

	Also "TryManyMovesPush()" - the main movement routine used
	by defenders and HUMANS.


	The main blocking routine is "BlockOrTryManyMoves()".  It is invoked
	by "LeadBlocker" or "BeBlocker"


	Player will block based on the PBlockState, which is the address
	in a structure of Blocking Structure made up of:

	BlockingStructure	struct
		void (*todo)(void); 	Address  of blocking routine
		WORD	parameter;  	(usually speed to move (or push)
		endstruct

	This Structure is repeated depending on the severity of the block,
	with the last BlockingStructure ALWAYS having an address of
	"NoBlockContact", which will reset to a new overall BlockingState once
	contact with another player has been re-established.


 *************************************************************************/
#ifdef __MSDOS__
#include  <stdlib.h>		/* for rand & abs	*/
#include  <stdio.h>		/* for printf	*/
#endif

#include  "fbequ.h"
#include  "fbRAM.h"			/* external VARs needed by this file	*/
#include  "FBPlays.h"		/* Script enums							*/
#include  "FBforms.h"		/* need to know for correct blocking schemes */
#include  "FBpenal.h"		/* Penalty enums & protos				*/


/*****    function prototyes   ***********/

WORD TryManyMoves(UWORD PossibleDirToMove);
WORD TryMove(UWORD PossibleDirToMove,WORD speedtablevalue);
//void  MakeTargetInBounds(void);
WORD IncrementPlayerSpeed(void);
WORD DecrementPlayerSpeed(WORD DecelFactor,WORD minspeed);
//WORD  DecrementPlayerSpeed(WORD DecelFactor);

void  GeneratePlayerSphereOfInfluence(UWORD PNum);
//void  SetDefOffsides(WORD type);	// called if collision before snap OR past LOS at snap

void  Run(void);
BOOL  TurnPlayerOptimal(WORD FinalDirection);
BOOL  ReachedTarget(void);
WORD  CheckForTackleAndFumble(WORD GuyHit);

//void  GoToNextState(WORD  ParamIncValue);
void  PlayerHasBall(void);

void	Init2ndState(WORD tickstostayinstate);
//void	Do2ndState(void);

void DoState(void);
void	NoAction(void);		/*  will do a players STATE	*/
void	FreezeAction(void);	/* a NULL action - no way to exit- does NOT do STATE	*/

void RunInPlace(void);
void ChaseBall(void);
WORD Distance(WORD x, WORD y);
BYTE CanDefGetBC(void);
BYTE QBOutOfPocket(void);
//void DefShuffleRun(void);
//void ShuffleRandom(void);
void Shuffle(WORD shuffletype);
//void RunToTargetState(void);
//BYTE RunToTarget(void);
//BYTE FindClosestOff(void);

void SetGuyToBlock(WORD offslotnum,WORD defslotnum);

void InitGraphics(WORD AIgraphicnum);	/*  in FBMOVE.C	*/

UWORD FindDirection(WORD DeltaX, WORD DeltaY);
UWORD DirectionToClosestDef(WORD pnum);


/****  Blocking functions  ******/
void BlockOrTryManyMoves(void);
void NoBlockContact(void);
void OffensivePushesDef(void);
void DefensivePushesOff(void);
void DefensiveGoAroundOff(void);
void DefensiveGoAroundOff1st(void);

void StunBlockeeInit(void);
void UnStunBlockee(void);

//WORD AIPlayEffectCheck(WORD sfxnum);


//void SetPlayerStunned(void);
//void StunPlayerInit(WORD time);
//void StunPlayer(void);

void SetBlockerGraphics(void);

void FigureBlockFactor(void);
WORD FigureStrengthBlockFactor(void);


WORD TryManyMovesPush(WORD dir);
void PushOtherGuy(WORD guyhit, WORD PushSpeedFactor);

//void BlockerFindClosestOpponent(void);
BOOL BlockerFindClosestOpponent(UWORD lowy,UWORD highy,UWORD highx);	// a box to look for guys in
BOOL BlockerFindClosestUnBlockedOpponent(UWORD lowy,UWORD highy,UWORD highx);	// a box to look for guys in
void DefBlockerFindClosestOpponent(UWORD lowy,UWORD highy,UWORD highx);	// a box to look for guys in
void FaceBallandShuffle(void);
BOOL OnMyTeam(WORD guy);
void FigureWhoToBlock(void);



/******  External Data Structures   **************/
extern  WORD  *All_Scripts[];
extern  struct play ThePlays[];

//#define randper	(rand()%100)

void InsertFaceArray(void);


/*************************************************************************

					LEAD  BLOCKER

 * 	This state holds all the lead blocker code, including
 *		Initialization.  It works like this:  Check to see if someone
 *		has the ball, if so, see if they are past you.  If you can lead
 *		block for them, try to keep their Y, and the endzone X as your
 *		target.
 *
 *		If CP has someone to block, check distance, if close run to him, if
 *		not, run to 4 yards in front of him.
 *
 *		If CP doesn't have anyone to block, find closest opponet who is not
 *		being blocked.  If he is close run at him.  Otherwise just keep
 *		target calculated in the beginning.
 *************************************************************************/

STATEDEF(LeadBlocker)
{
EFFVAR g2b;
BOOL refiguredir = FALSE;

	 // tell QB that this guy (possible receiver) is blocking
	AbortWhereWillPlayerBe = TRUE;	// used in WhereWillPlayerBe (QB passing logic in FBBall.c)

	if (!CPStateInit)		// Setup lead blocker flag
	{
		CPStateInit = 1;
		SetAI(NoAI);   	// kill any possible LeadBlocker AI
		SetBit(CPFlags,LEAD_BLOCKER_BFLAG);
		refiguredir = TRUE;
	}


	if(BallXPosition > CPXPos + 7 * YARD)	// Ball or BC is way past me.
	{
		FaceBallandShuffle();
	}
	else
	{	// then find a target

		if( ( (AIFrameCounter+gPNum) & 3) == ZERO)
			refiguredir = TRUE;

		if (CPGuyToBlock == NOBODY)		// I am already blocking someone.
		{ // if I'm not blocking anybody, see what's available
			if(BlockerFindClosestUnBlockedOpponent(CPYPos-5*YARD,CPYPos+5*YARD,CPXPos+7*YARD))
				refiguredir = TRUE;
		}

		if ((g2b = CPGuyToBlock) == NOBODY)		// I am already blocking someone.
		{   // if still I'm not blocking anybody, head for EZ

			if(CPStateInit == 3)	// BC caught up last frame
			{
				refiguredir = TRUE;
				CPStateInit = 1;
			}

			if(CPStateInit == 1  && CPXPos > BallXPosition + 9*YARD)	// I'm in front of ball
			{
//				FaceBallandShuffle();	// stop if way in front
				CPStateInit = 2;
			}

			if(CPStateInit == 2)
			{
				FaceBallandShuffle();
				if(CPXPos < BallXPosition + 5*YARD)
				{
					CPStateInit = 3;	// bc caught up
//					refiguredir = TRUE;
				}
			}
			else if(CPXPos > OFF_END_LINE-(2*YARD) - (gPNum*YARD)/3)
			{   // allow "spacing" in EZ
				FaceBallandShuffle();
			}
			else
			{
				if(refiguredir)
				{
					CPTargetX = OFF_END_LINE-YARD+1;
					CPTargetY = BallYPosition-3*HALFYARD+(gPNum*YARD)/3;
					CPDirToGo = FindCPDirectionToTargetMACRO;

				}   // else keep old target - reduces TWITCH - bug #2943

				BlockOrTryManyMoves();	// MUST have target setup (but ok if g2b==NOBODY)
			}
		}
		else	// I have somebody to block
		{    	// If he is close, run at him
			if(refiguredir)
			{
				if (DistanceToPlayer(g2b) < 5 * HALF_YARD)
					CPTargetX = OPXPos(g2b);
				else   // else Run a bit in front of him(assume he's running WEST)
					CPTargetX = OPXPos(g2b) - 2 * YARD;

				CPTargetY = OPYPos(g2b);	// In either case, block on same Y
				CPDirToGo = FindCPDirectionToTargetMACRO;
			}
			BlockOrTryManyMoves();	// MUST have target setup (but ok if g2b==NOBODY)
		}

	#if 	PC_DEBUG_MODE
//		InsertFaceArray();		// twitch test
	#endif

	}

}

/*-----------------------------------------------------------------------
						LEAD  Blocker AI

	 An AI (SMARTS) state - the idea is to have a Lead Blocker look for
	 guys to block while he is doing his "normal" script (RunToRel)
	 If we find a blockee (in a smaller "box" than normal, then institute
	a new SCRIPT which only has 1 STATE - LeadBLocker
	NOTE: a lead blocker's SCRIPT normal last STATE is LeadBLocker
-------------------------------------------------------------------*/
STATEDEF(LeadBlockerAI)
{

	BlockerFindClosestUnBlockedOpponent(CPYPos-3*YARD,CPYPos+3*YARD,CPXPos+3*YARD);
	if (CPGuyToBlock != NOBODY)		// I just found somebody to block
	{
		SetNewScript(LeadBlockerScriptNUM);
		SetAI(NoAI);
	}

}


STATEDEF(InitLeadBlockerState)	// 1st STATE of Lead Blocker Script
{
	SetBit(CPFlags1,IM_A_BLOCKER_BFLAG);
	SetAI(LeadBlockerAI);	// AI level will look for somebody to block
	GoToNextState(1);
}


STATEDEF(InitIWillBeABlocker)	// set a flag so we don't pass to these guys
{	// for ALL blockers especially RBs and TEs
	SetBit(CPFlags1,IM_A_BLOCKER_BFLAG);
	GoToNextState(1);
}



#define NEAR_FACTOR		(3*QTRYARD)

BOOL ImNearTarget(void)	// used by blockers
{
	if( (CPXPos > CPTargetX-NEAR_FACTOR)
	  &&(CPXPos < CPTargetX+NEAR_FACTOR)
	  &&(CPYPos > CPTargetY-NEAR_FACTOR)
	  &&(CPYPos < CPTargetY+NEAR_FACTOR)
	  )
		  return(TRUE);

	return(FALSE);
}

/*************************************************************************

							BE BLOCKER  - a STATE

	Used by Offensive Line
	Sets up current player to be a blocker & do blocking stuff

	STATE with no EXIT

 *************************************************************************/

WORD KickWedgeOffsets[11] =

{0*YARD,6*YARD,3*YARD,0*YARD,-3*YARD,-6*YARD,
 -8*QTRYARD,0*QTRYARD,8*QTRYARD,4*QTRYARD,-4*QTRYARD};

STATEDEF(BeBlocker)
{
UEFFVAR highx,ty;
BOOL refiguredir = FALSE;

	 // tell QB that this guy (maybe a possible receiver) is blocking
	AbortWhereWillPlayerBe = TRUE;	// used in WhereWillPlayerBe (QB passing logic in FBBall.c)

	if (!CPStateInit)
	{
		CPStateInit = 1;
		CPDirToGo = EAST_DIR;
		CPTargetX = OFF_GOAL_LINE + 4 * YARD;
		CPTargetY = CPYPos;
		refiguredir = TRUE;
	}

	if ( BallXPosition < CPXPos + 6 * YARD  )
	{ // if not pretty far past me

		if( ( (AIFrameCounter+gPNum) & 3) == ZERO
//			|| (CPFlags1 & IM_BLOCKING_HUMAN_BFLAG)	//if blocking a human
		  )
		{
			refiguredir = TRUE;
		}

		if (CPGuyToBlock == NOBODY)   // We dont have someone to block already
		{
			if( CPFlags & PASS_BLOCKER_BFLAG)
				highx = CPXPos + 3*YARDS;
			else
				highx = CPXPos + 7*YARDS;

			if(PlayType == KICKOFF_PLAY && gPNum > 5)
			{
				ty = 3*HALF_YARD;	// wedge guys  on KO
			}
			else
			{
	//			ty = 4*YARD;
	// if 4 yards, center picked up end 1 sec in to play if blitz
				ty = 3*YARD;
			}
			if(BlockerFindClosestUnBlockedOpponent(CPYPos-ty,CPYPos+ty,highx))
				refiguredir = TRUE;

		}


		if(CPFlags & PASS_BLOCKER_BFLAG)
		{
			if (CPGuyToBlock == NOBODY ||
				 OPXPos(CPGuyToBlock) > StartBallX + 4*HALF_YARD)
			{	// don't block guy if I'm pass blocking and he's not near LOS
				if(CPXPos < OPXPos(0)+5*HALFYARD)
					Shuffle(BLOCKING_SHUFFLE);
				else
				{   // move slowly toward QB (drop into pocket)
					TryMove(FindCPDirectionToPlayer(0),0);	//will move if no collision
					Shuffle(BLOCKING_SHUFFLE);
				}
				if(refiguredir)	// don't twitch
					CPFacing = DirectionToClosestDef(gPNum);

			}
			else
			{
				if(OPFlags(CPGuyToBlock) & IS_HCONTROLLED_BFLAG)
				{	// mustcheck everyframe in case toggled onto/off of
					refiguredir = TRUE;
				}

				if(refiguredir)
				{
					CPTargetX = OPXPos(CPGuyToBlock);
					CPTargetY = OPYPos(CPGuyToBlock);
				}
				BlockOrTryManyMoves();
			}
		}
		else
		{		//run blocker
			if (CPGuyToBlock != NOBODY)		// Did we find someone to block?
			{
				if(refiguredir)
				{
					CPTargetX = OPXPos(CPGuyToBlock);
					CPTargetY = OPYPos(CPGuyToBlock);
				}
				BlockOrTryManyMoves();
			}
			else	// nobody to block
			{
				if(PlayType == KICKOFF_PLAY)
				{
					if(CPXPos > OFF_GOAL_LINE+YARD)
					{
						Shuffle(BLOCKING_SHUFFLE);	// for new blocking shuffle
					}
					else
					{
						if(refiguredir)
						{
							if(BallCarrier != NOBODY)
							{
							  //	tx = OPXPos(BallCarrier);
								ty = OPYPos(BallCarrier);
								CPTargetX = CPXPos + 1*YARDS;
							}
							else
							{
//								tx = KickTargetX;
								ty = KickTargetY;
								CPTargetX = KickTargetX + 5*YARDS;
							}
//							CPTargetX = tx + 5*YARDS;
//							CPTargetY = (CPYPos>>1) + (ty>>1); //don't add then divide cause of overflow
							CPTargetY = ty + KickWedgeOffsets[gPNum];
						}
						BlockOrTryManyMoves();
					}
				}
				else
					Shuffle(BLOCKING_SHUFFLE);
			}
		}
	}
	else
	{
		FaceBallandShuffle();
	}
}



STATEDEF(BePuntBlocker)
{
	BeBlocker();

	if (BallFlags & BALL_PUNTED_BFLAG)
		GoToNextState(1);

}

#if 1
/*************************************************************************
 *	DEF BE BLOCKER - Sets up current player to be a blocker & do blocking
 * 	Used if def has ball - turnovers, punts, etc.
 *************************************************************************/
STATEDEF(DefBeBlocker)
//void DefBeBlocker(void)
{
	if (!CPStateInit)
	{
		CPDirToGo = WEST_DIR;
//		CPTargetX = DEF_GOAL_LINE;
//		CPTargetY = CPYPos;
		CPStateInit = 1;
	}

	if (BallCarrier == NOBODY)	// If I have strength and a Ball Carrier
	{
		FaceBallandShuffle();
	}
	else if (CPXPos - 5 * YARD < OPXPos(BallCarrier)) // not 5 yards past me
	{
		if (CPGuyToBlock == NOBODY)	// We dont have someone to block already
			DefBlockerFindClosestOpponent(CPYPos-6*YARDS,CPYPos+6*YARDS,CPXPos);

		if(CPGuyToBlock != NOBODY)	// Did we find someone to block?
		{
			CPTargetX = OPXPos(CPGuyToBlock);
			CPTargetY = OPYPos(CPGuyToBlock);
			BlockOrTryManyMoves();	// MUST have target setup
		}
		else
		{
//			TryManyMovesPush(WEST_DIR);	this single line was old logic (pre10/31)

		 // below new 10/31/96 to make guys block better
			if(CPStateInit == 1  && CPXPos < BallXPosition -13*YARD)	// I'm in front of ball
			{
//				FaceBallandShuffle();	// stop if way in front
				CPStateInit = 2;
			}

			if(CPStateInit == 2)
			{
				FaceBallandShuffle();
				if(CPXPos > BallXPosition - 9*YARD)
					CPStateInit = 1;	// bc caught up
			}
			else if(CPXPos < DEF_GOAL_LINE-YARD- ((gPNum-11)*YARD)/3)
			{   // allow "spacing" in EZ
				FaceBallandShuffle();
			}
			else // CPInit = 1 and "close" to BC
			{
				TryManyMovesPush(WEST_DIR);
//				CPTargetX = DEF_GOAL_LINE-5*YARDS;
//				CPTargetY = CPYPos;
//				BlockOrTryManyMoves();	// MUST have target setup (but ok if g2b==NOBODY)
			}


		}
	}
	else
		FaceBallandShuffle();
}

#endif



STATEDEF(OLPassBlockDropBack)
{	// drop back on pass blocking depending on position
EFFVAR tx;
//	GoToNextState(1);
//	return;

	if (!CPStateInit)
	{
		CPStateInit = TRUE;

		if(CPMyFormPosition == CENTER)
			tx = -1 * HALF_YARD;	//  center
		else if(CPMyFormPosition > LT)
			tx = -1 *YARDS;	// guards
		else	// tackles or TE
			tx = -2 *YARDS;

		tx = tx/2;
//		tx = tx*2;

		CPDirToGo = WEST_DIR;
		CPTargetX = CPXPos + tx;	// use CPXPos, not StartBallX
		CPTargetY = CPYPos;		// really a DONT_CARE, but just to be safe

		CPAcceleration = BASIC_ACCELERATION;	// do it slowly
	}

#if 0
	CPDirToGo = FindDirection2Points(CPXPos,CPXPos,OPXPos(0)-10*YARDS,OPYPos(0))

	TryManyMoves(CPDirToGo);
	SetBlockerGraphics();
	CPFacing = EAST_DIR;	// must be AFTER  "SetBlockerGraphics()"
	if(ReachedTarget())
	{
		CPAcceleration = MAX_ACCELERATION;
		GoToNextState(1);
	}

#else
	CPDirToGo = FindDirection2Points(CPXPos,CPYPos,OPXPos(0)-9*YARDS,OPYPos(0));
	if (CPGuyToBlock == NOBODY ||
			 OPXPos(CPGuyToBlock) > CPXPos + 3*HALF_YARD)
	{	// don't block guy if I'm pass blocking and he's not near LOS
		TryManyMoves(CPDirToGo);
		SetBlockerGraphics();

		CPFacing = EAST_DIR;	// must be AFTER  "SetBlockerGraphics()"
		if(ReachedTarget())
		{
			CPAcceleration = MAX_ACCELERATION;
			GoToNextState(1);
		}
	}
	else
	{
		CPAcceleration = MAX_ACCELERATION;
		GoToNextState(1);
	}
#endif
}


/*************************************************************************

						BLOCK OR TRY MANY MOVES

	Player will block based on the PBlockState, which is the address
	in a structure of Blocking Structure made up of:

	BlockingStructure	struct
		WORD	Parameter	(usually speed to move (or push)
		WORD	Address  of blocking routine
		endstruct

	This Structure is repeated depending on the severity of the block,
	with the last BlockingStructure ALWAYS having an address of
	"NoBlockContact", which will reset to a new overall BS once
	contact with another player has been re-established.


  *** WARNING ***  The TARGET where the blocker wants to go MUST have been
	set up!!!    Also CPGuyToBlock must have been setup.

	There are currently 4 things that can happen:

	1) OffensivePushesDef,X,	X = how far blocker is pushed

NO-NO	2) SetPlayerStunned,X, 		X = of frames to be stunned
			Defense gets killed
	3) DefensivePushesOff,X,      X = how far blockee is pushed

	4) NoBlockContact,X  	X = don't care
		Used if looking for or moving toward guy to block
		When contact made than do one of the above 3 things

 *************************************************************************/
void BlockOrTryManyMoves(void)
{
	CPBStatePtr->todo();
}




//#define  PUSH_FAR_FACTOR	4
//#define  PUSH_SHORT_FACTOR	2

/* --------------  BLOCKER/BLOCKEE STATE TABLES  ------------------- */

/*************   WARNING  **********************************
		- all PUSH-PULLS must be in order as the higher
	ones "fall through" to the lower ones and only the last one
	(PushPull8)	has a "NoBlockContact"
	this is an old assembly language trick to save space

ABOVE changed 2/13/96 because it's not proper "C"

	PushPull  used strictly for PASS blocking.

	Idea is to set Blocker State to point into this struct (closer to
	beginning is better for offense). At end of the struct are a few
	states which allow the defender to get around the blocker.

----------------------------------------------------------------------*/
struct  BlockerState PushPullTable[] = {	// for PASS blocking
// 0
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
//4
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
// 8
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
// 12
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
//16
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
//20
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
//24
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
//28
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
//32
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
//36
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
//40
	DefensivePushesOff,2,
	OffensivePushesDef,3,
	DefensivePushesOff,2,
	OffensivePushesDef,3,
//44
	DefensivePushesOff,2,
	DefensiveGoAroundOff1st,2,		// allow defense to get by

	DefensiveGoAroundOff,2,
	DefensiveGoAroundOff,2,
	DefensiveGoAroundOff,2,
	DefensiveGoAroundOff,2,

	DefensiveGoAroundOff,2,
	DefensiveGoAroundOff,2,
	DefensiveGoAroundOff,2,
	DefensiveGoAroundOff,2,

	NoBlockContact,3			// allow blocker to re-init blocking
};





#if 1
struct  BlockerState PushTheBlockeeTable[] = {	//RUN blocking
//	StunBlockeeInit,DONT_CARE,	// this 1st blocker state is init for all run blockers
/* 0 */
	OffensivePushesDef,2,
	OffensivePushesDef,2,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/* 4 */
	OffensivePushesDef,2,
	OffensivePushesDef,2,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/* 8 */
	OffensivePushesDef,2,
	OffensivePushesDef,2,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/*12 */
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/*16 */
	OffensivePushesDef,2,
	OffensivePushesDef,2,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/*20 */
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/*24 */
	OffensivePushesDef,2,
	OffensivePushesDef,2,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/*28 */
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/*32 */
	OffensivePushesDef,2,
	OffensivePushesDef,2,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/*36 */
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/*40 */
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
	OffensivePushesDef,1,
/*44 */
	UnStunBlockee,DONT_CARE,

	DefensiveGoAroundOff1st,2,

	DefensiveGoAroundOff,2,
	DefensiveGoAroundOff,2,
	DefensiveGoAroundOff,2,
	DefensiveGoAroundOff,2,

	DefensiveGoAroundOff,2,	// added 10/29/96 - give def. a chance
	DefensiveGoAroundOff,2,	// added 10/29/96 - give def. a chance

	NoBlockContact,3
};

#else
struct  BlockerState KillBlockee[] = {
	SetPlayerStunned,5,		// # of frames to be stunned

	DefensivePushesOff,2,	// def. is stunned so they won't push
	DefensivePushesOff,2,	//  .... and Off. won't move in this case
	DefensivePushesOff,2,
	DefensivePushesOff,2,
	DefensivePushesOff,2,
	DefensivePushesOff,2,

	NoBlockContact,3
};

#endif

struct  BlockerState NormalBlocker[] = {

	NoBlockContact,2
};


/****************  Main   Blocking table  *******************************/

/*   "figureBlockFactor()" will generate num 0-7 which indexes the following
		table	*/
#define PUSHPULLSIZE	44
#define PUSHBLOCKEESIZE	44

//#define	MAX_BLOCK_FACTORS	8

#define PushPull(frames) &PushPullTable[PUSHPULLSIZE-frames]
#define PushBlockee(frames) &PushTheBlockeeTable[PUSHBLOCKEESIZE-frames]

struct BlockerState *RunBlockFactorTable[MAX_BLOCK_FACTORS] = {
#if 1
/* 	NOTE: PushBlockee(20) opens a good hole	*/
/*
	PushBlockee(4),		// worse for offense
	PushBlockee(8),
	PushBlockee(12),
	PushBlockee(16),

	PushBlockee(20),
	PushBlockee(24),
	PushBlockee(28),
	PushBlockee(32),	// best for offense (good block)
*/
/*	// following new 8/28/96 - better blocking to help running game
	PushBlockee(8),		// worse for offense
	PushBlockee(20),
	PushBlockee(24),	// tables should be a lttle "top" heavy - favor off.
	PushBlockee(28),

	PushBlockee(32),
	PushBlockee(36),
	PushBlockee(40),
	PushBlockee(44),	// best for offense (good block)
*/
/*  // new 10/29/96 - run blocking was way too good
	PushBlockee(8),		// worse for offense
	PushBlockee(12),
	PushBlockee(16),	// tables should be a lttle "top" heavy - favor off.
	PushBlockee(20),

	PushBlockee(24),
	PushBlockee(28),
	PushBlockee(36),
	PushBlockee(44),	// best for offense (good block)
*/
 /*  back up a bit - 11/9/96	*/
	PushBlockee(8),		// worse for offense
	PushBlockee(16),
	PushBlockee(20),	// tables should be a lttle "top" heavy - favor off.
	PushBlockee(24),

	PushBlockee(28),
	PushBlockee(32),
	PushBlockee(36),
	PushBlockee(44),	// best for offense (good block)


#else
#define PBTEST	44	//20
	PushBlockee(10),
	PushBlockee(PBTEST),
	PushBlockee(PBTEST),
	PushBlockee(PBTEST),

	PushBlockee(PBTEST),
	PushBlockee(PBTEST),
	PushBlockee(PBTEST),
	PushBlockee(PBTEST),
#endif
};

struct BlockerState *PassBlockFactorTable[MAX_BLOCK_FACTORS] = {
#if 1
	PushPull(8),	// tables should be a lttle "top" heavy - favor off.
	PushPull(20),
	PushPull(24),
	PushPull(28),

	PushPull(32),
	PushPull(36),
	PushPull(40),
	PushPull(44),

#else
#define PBTEST1	44
	PushPull(10),
	PushPull(PBTEST1),
	PushPull(PBTEST1),
	PushPull(PBTEST1),
	PushPull(PBTEST1),
	PushPull(PBTEST1),
	PushPull(PBTEST1),
	PushPull(PBTEST1),
#endif
};

/****   a little subroutine to check if guy I'm blocking is "far" away -
	if so, go back to "NormalBlocker				*/
void IncBlockerStateAndCheckBlockeeFarAway(void)
{
	if (DistanceToPlayer(CPGuyToBlock) > 3 * HALF_YARD)
	{
		CPBStatePtr = NormalBlocker;
	}
	else
		CPBStatePtr++;

}

/*************************************************************************
 *
 *					BLOCKER  STATE  ROUTINES
 *
 *	There are 4 low level blocking states which are strung together in
 *	 "Blocking scripts".
 *
 *	1) NoBlockContact	- basic
 *	2) OffensivePushesDef
 *	3) DefensivePushesOff
 *	4) SetPlayerStunned
 *
 *	These states are put together in tables to form BlockStructures
 *
 *************************************************************************


				No Block Contact	- a BLOCKER STATE

	This is default state which should be at the end of all
  Blocker state tables(structures) - (and also inited to at start of play)
  it will NOT advance to next blocker state

 *** WARNING ***  Target of Blocker MUST be set up before entering here !!!!


	Used by a blocker who is going toward a guy to block but hasn't made
	contact yet.


 WARNING  I_CAN_PUSH_BFLAG in CPFlags is a VERY POWERFUL flag, especially
	when cotrolling defense.  It's only looked at in TryManyMovesPush,
	which is what the defense or offensive blockers use.  Turning this
	OFF helps the defense since they will then try to go around an offensive
	player instead of push through him.

	The flag is Inited to TRUE in InitAI (except for KickOff plays)

 *************************************************************************/
void NoBlockContact(void)
{
EFFVAR guyhit;

/* set up push parameters for both guys in case of contact this frame	*/
	CPPushFactor = CPBStatePtr->parameter;
//	OPPushFactor(CPGuyToBlock) = CPBStatePtr->parameter;

	SetBit(CPFlags,I_CAN_PUSH_BFLAG);	// Set offensive blockers to push

	if(CPGuyToBlock != NOBODY)	// ... then Opponent is not blocked now
	{
//		ResetBit(OPFlags(CPGuyToBlock),(IM_BLOCKED_BFLAG | I_CAN_PUSH_BFLAG));
		ResetBit(OPFlags(CPGuyToBlock),(IM_BLOCKED_BFLAG));
/*
		if(PlayType == KICKOFF_PLAY)	// I hope Bobby's happy
		{
			ResetBit(OPFlags(CPGuyToBlock),(HAVE_BEEN_BLOCKED_BFLAG));
			OPBlockingMe(CPGuyToBlock) = NOBODY;
			CPGuyToBlock = NOBODY;
		}***/
	}


	if(ImNearTarget() || (CPFlags & LEAD_BLOCKER_BFLAG) )
	{    //leave it the same as it was last time - no twitching
		guyhit = 0;	// breakpoint here
	}
	else	// refigure direction
	{
		CPDirToGo = FindCPDirectionToTargetMACRO;
	}

	guyhit = TryManyMovesPush(CPDirToGo);

	if (guyhit >= ZERO && guyhit == CPGuyToBlock)	// he pushed somebody
	{
		SetBit(OPFlags(CPGuyToBlock),IM_BLOCKED_BFLAG | HAVE_BEEN_BLOCKED_BFLAG);
//		SetBit(OPFlags(CPGuyToBlock),IM_BLOCKED_BFLAG); // Set guy being blocked to blocked
		FigureBlockFactor();		// Sets up PBlockState if there is contact
	}
	else if (guyhit == CANT_MOVE)	// due to clipping
		CYCLE_GRAPHICS_MACRO(CPAIGraphicState);

}


/*************************************************************************

				  Offensive Pushes Defense - 	A blocking STATE

	Current Offensive Player (the BLOCKER) will push the
	defensive player ( the BLOCKEE)


 *** WARNING ***  Target of Blocker MUST be set up before entering here !!!!

*************************************************************************/
void OffensivePushesDef(void)
{
	CPPushFactor = OPPushFactor(CPGuyToBlock) = CPBStatePtr->parameter;

	CPDirToGo = FindCPDirectionToTargetMACRO;
	TryManyMovesPush(CPDirToGo);

 // Set guy being blocked to CANT PUSH
//	ResetBit(OPFlags(CPGuyToBlock),I_CAN_PUSH_BFLAG);

	IncBlockerStateAndCheckBlockeeFarAway();

}


/*************************************************************************

				  Defensive Pushes Offense - 	A blocking STATE

	Current Offensive Player (the BLOCKER) does nothing and
	allows the defensive player ( the BLOCKEE) to push him.

 *************************************************************************/
/*  Blocker does NO movement - Defense guy should push him */
void DefensivePushesOff(void)
{
//	SetBlockerGraphics();
// if blocker was far away (running) don't force back into block
// as it causes twitches due to diff. graphic states every other frame
	CYCLE_GRAPHICS_MACRO(CPAIGraphicSequence);

	OPPushFactor(CPGuyToBlock) = CPBStatePtr->parameter;

 // Set guy being blocked so he pushes me
	SetBit(OPFlags(CPGuyToBlock),I_CAN_PUSH_BFLAG);

	// If they are not close anymore, become a normal blocker
	IncBlockerStateAndCheckBlockeeFarAway();
}




/*************************************************************************

			  Defensive GO AROUND  Offense - 	A blocking STATE

	   more effective for DEFENSE than(Defensive Pushes Offense)

	Current Offensive Player (the BLOCKER) does nothing and
	allows the defensive player ( the BLOCKEE) to go around him.
	This is accomplished by turning off blockees push flag.

 *************************************************************************/
/*  Blocker does NO movement - Defense guy should push him */
void DefensiveGoAroundOff(void)
{
	SetBlockerGraphics();	//

//	OPPushFactor(CPGuyToBlock) = CPBStatePtr->parameter;

 // Turn off PUSH FLAG  allows defender to go around blocker
	ResetBit(OPFlags(CPGuyToBlock),I_CAN_PUSH_BFLAG);

	// If they are not close anymore, become a normal blocker
	IncBlockerStateAndCheckBlockeeFarAway();
}



void DefensiveGoAroundOff1st(void)	// identical to above, but allows esy debugging with BPs
{
	SetBlockerGraphics();	//

//	OPPushFactor(CPGuyToBlock) = CPBStatePtr->parameter;

 // Turn off PUSH FLAG  allows defender to go around blocker
	ResetBit(OPFlags(CPGuyToBlock),I_CAN_PUSH_BFLAG);

	// If they are not close anymore, become a normal blocker
	IncBlockerStateAndCheckBlockeeFarAway();
}




#if 0
/*************************************************************************
 * SET PLAYER STUNNED - Stuns player for Parameter cycles.
 *************************************************************************/
void SetPlayerStunned(void)
{
	StunPlayerInit(CPBStatePtr->parameter);

	CPBStatePtr++;
}

#endif

/*************************************************************************
 * SET BLOCKER GRAPHICS - Sets up blocker graphics.
 *************************************************************************/
void SetBlockerGraphics(void)
{

	InitGraphics(AI_BLOCKING_NUM);	/*  if necessary	*/


	CPFacing = CPDirToGo;

#if	1	// sequential
	CYCLE_GRAPHICS_MACRO(AI_BLOCKING_NUM);
#else	// Juan said random looks good
	CPAIGraphicSequence =
	   min(	(random0_99/NumFramesPerPlayerGraphic[PG_BLOCKING_NUM]),
			NumFramesPerPlayerGraphic[PG_BLOCKING_NUM]-1);
#endif

}

/*************************************************************************
 * SET BLOCKER GRAPHICS - Sets up blocker graphics.
 *************************************************************************/
void SetBlockeeGraphics(void)
{

}


/*************************************************************************
 *
 *	F I G U R E    B L O C K I N G    F A C T O R
 *
 *	3 things come into play during a block:
 *
 *	1) TEAM Blocking factor
 *	2) relative strengths of players involved
 *	3) random factor
 *
 *	Will chosse one of the eight possible blocking structure for a
 *	player to execute.
 *
 *	INPUTS: Y--> indexes blocker
 *		X--> indexes blockee
 *
 *************************************************************************/


/* following table is used to obtain final Block Value based on the
	original block factor based on player abilities, etc ( a value 0-7
	used as a main index) and then a random number 0-15 as the second
	index
*/
#if 1
BYTE BlockerBellCurve[MAX_BLOCK_FACTORS/2][16] = {
0,0,0,0, 0,0,0,0, 1,1,1,2, 2,3,4,5,		// 0
0,0,0,1, 1,1,1,1, 1,2,2,2, 3,4,5,6,		// 1
0,0,1,1, 2,2,2,2, 2,2,3,3, 4,5,6,7,		// 2
0,1,1,2, 2,3,3,3, 3,3,3,4, 4,5,6,7		// 3

//7-0,7-1,7-1,7-2, 7-2,7-2,7-3,7-3, 7-3,7-3,7-3,7-4, 7-4,7-5,7-6,7-7,	// 7-3=4
//7-0,7-0,7-1,7-1, 7-1,7-2,7-2,7-2, 7-2,7-2,7-3,7-3, 7-3,7-5,7-6,7-7,	// 7-2=5
//7-0,7-0,7-0,7-1, 7-1,7-1,7-1,7-1, 7-1,7-2,7-2,7-2, 7-3,7-4,7-5,7-6,	// 7-1=6
//7-0,7-0,7-0,7-0, 7-0,7-0,7-0,7-0, 7-1,7-1,7-1,7-2, 7-2,7-3,7-4,7-5,	// 7-0=7

};
#else 		// for TESTing
BYTE BlockerBellCurve[MAX_BLOCK_FACTORS/2][16] = {
0,0,0,0, 0,0,0,0, 0,0,0,0, 0,0,0,0,		// 0
1,1,1,1, 1,1,1,1, 1,1,1,1, 1,1,1,1,		// 1
2,2,2,2, 2,2,2,2, 2,2,2,2, 2,2,2,2,		// 2
3,3,3,3, 3,3,3,3, 3,3,3,3, 3,3,3,3		// 3
};
#endif
#define	RAND_MINBLK	3	// % of time a blocker will always do best or worst

void FigureBlockFactor(void)
{
struct BlockerState **tableptr;	// pointer to pointer of blocker scripts
EFFVAR r,index;


//	if(++CPTimer > 1 )			// no more blocks after first one
//		CPAction = FreezeAction;

/*  we used different blocker scripts depending on if RUN or PASS blocking */
	if( CPFlags & PASS_BLOCKER_BFLAG  || BallCarrier >= START_DEF_SLOT_NUM)
		tableptr = PassBlockFactorTable;	
	else
	{
		StunBlockeeInit();		// table will determine for how long
		tableptr = RunBlockFactorTable;
	}
#if 	PC_DEBUG_MODE
	if (GenValue != ZERO)	// PC testing only !!!
	{
		index = (GenValue-1)&7;
	}
	else
#endif
	if ( (r=randper) < RAND_MINBLK) // 3% of time alway do best
	{
		index  = MAX_BLOCK_FACTORS-1;	// Best for Offense
	}
	else if (r < 2*RAND_MINBLK) //// 3% of time alway do worst
	{
		index  = ZERO;	// Best for Defense
	}
/*	else if ( (CPFlags & LEAD_BLOCKER_BFLAG)	// If I am a Lead Blocker
			|| (PlayType == KICKOFF_PLAY) )
	{ 	// Only one major block for a lead blocker...
		ResetBit(CPFlags,LEAD_BLOCKER_BFLAG);	// It is on, so turn it off...
	// 50 % chance... either kill him or miss...(take into account prev. "r" uses)
		index = (r < (100+2*RAND_MINBLK)/2) ? ZERO: MAX_BLOCK_FACTORS-1;
	}*/
	else if (PlayType == KICKOFF_PLAY)
	{ 	// 50 % chance... either kill him or miss...(take into account prev. "r" uses)
		index = (r < (100+2*RAND_MINBLK)/2) ? ZERO: MAX_BLOCK_FACTORS-1;
	}
	else
	{
		r = rand()&15;	// r is 0 to 15 to be used in "BellCurve" table

		if (BallCarrier >= START_DEF_SLOT_NUM)
		{	/* if defensive has ball then poor blocking	*/
			index  = BlockerBellCurve[0][r];
		}
		else // just a normal block based on opposing players abilities. et. al.
		{
			index = FigureStrengthBlockFactor() + TeamBlockFactor;

			if(    (CPFlags & LEAD_BLOCKER_BFLAG)
//				&& (CPXPos < StartBallX + 8*YARDS)	// only effective near LOS
			  )
			{	// help blocking any way possible
				index += 2;
			}
			index = SetMinMax(index,0,MAX_BLOCK_FACTORS-1);	// keep in valid range

			BlockTypeCount[index]++;	// for debugging - pre bell curve

			if (index > 3)
			{			// top half of table is "opposite" of bottom half
				index = 7-index;
				index = BlockerBellCurve[index][r];
				index = 7-index;
			}
			else
				index = BlockerBellCurve[index][r];

			BlockTypeCountActual[index]++;	// for debugging - post bell curve

		}
	}

	CPBStatePtr = tableptr[index];


}


/*************************************************************************

			FIGURE STRENGTH BLOCK FACTOR

	Return a number from 0 to 7 depending on strength of players.

 MOD. 2/19/96  Strength now based on 0-100 scale (NOT 0-15)

 REWROTE - 10/15/96

 *************************************************************************/
WORD TackleDiffTable[8] = { -40, -20,-10,0,5,10,20,2000};

#if 1
WORD FigureStrengthBlockFactor(void)
{
EFFVAR blockee,mypnum;
EFFVAR mystr,ostr,i;

	mypnum = CPMyPNum;	// use local instead of array entry

	if( (blockee = CPGuyToBlock) < ZERO)
	{
#if PC_ALWAYS_MODE
		printf(" ILLEGAL GUY TO BLOCK ");
//		getch();
#endif
		return(0);	// do a weak block on NOBODY)
	}

	ostr  = OPStrength(blockee);	// assume
	mystr = CPStrength;				// assume

	if(ThePlays[GameTeamData[OffensiveSlotNum].PlayNum].PlayTypeDesignator <= RUN_OUTSIDE
		|| PassCatcher != NOBODY)
	{   // running play or after Pass caught

		if(mypnum >= RR && mypnum <= OL7)	//Off. except QB
			mystr = (CPStrength + CPRunBlockRating)/2;
//		else
//			mystr = CPStrength;

	}
	else	// passing (or other - KO, punt, etc)
	{
		if(mypnum >= RR && mypnum <= FB2)	//RBs and Rec
			mystr = (CPStrength + CPRunBlockRating)/2;
		else if(mypnum >= RT && mypnum <= OL7)	//Off. line
			mystr = (CPStrength + CPPassBlockRating)/2;
//		else
//			mystr = CPStrength;

		if(OPMyPNum(blockee) >= DLE && OPMyPNum(blockee) <= LB6)
			ostr = (OPStrength(blockee) + OPRushRating(blockee))/2;

	}

//	ostr = (mystr - ostr + 100)/24;		// set bp here
	ostr = (mystr - ostr);		// set bp here

	for(i=0; i<7; i++)
	{
		if(ostr < TackleDiffTable[i])
			break;
	}
	return(i);
}
#else  // the following was sooo old it was really starting to reek!
WORD FigureStrengthBlockFactor(void)
{
//	CPStrength - OPStrength + 15 (num from 0 - 30) >> 2 = (from 0 - 7)
//	return((CPStrength - OPStrength(CPGuyToBlock) + 15) >> 2);

// okay to return >7 as index will be checked for extremes
// NOTE: adding 100 always give a positive number - the range will then
// be 0-200 with a value of 100 if guys have equal strength
	return( (CPStrength - OPStrength(CPGuyToBlock) + 100)/24);
}
#endif



#if 0
/*************************************************************************
 * STUN PLAYER INIT -
 *************************************************************************/
void StunPlayerInit(WORD time)
{
EFFVAR blockee;

	blockee = CPGuyToBlock;
	if (OPAction(blockee) != StunPlayer)
	{
		OPSpeed(blockee) = 0;		// If the guy is stunned stop him.
		OPAction(blockee) = StunPlayer;
		// Set Graphics Stuff...
//	OPGraphicSequence(blockee) = time;

		OPAIGraphicSequence(blockee) = max(time,MAX_FRAMES_MACRO(OPAIGraphicState(blockee))-1);

//		SetBit(OPFlags(blockee),IM_BLOCKED_BFLAG | HAVE_BEEN_BLOCKED_BFLAG);
	}
}


/*************************************************************************
 * STUN PLAYER -
 *************************************************************************/
void StunPlayer(void)
{
	if (CPAIGraphicSequence <= 0)	// if we have done all the frames
	{
		CPAction = NoAction;
		ResetBit(CPFlags,IM_BLOCKED_BFLAG);
		DoState();
	}
	else
	{
		CPAIGraphicSequence--;
	}
}

#endif

/*************************************************************************

			STUN BLOCKEE

 *************************************************************************/

PLAYER_ACTION(BlockeeStunned)	// something in blocker state must unstun	*/
{
	InitGraphics(AI_BLOCKING_NUM);	/*  if necessary	*/

	if(CPBlockingMe >= ZERO)
		CPFacing = FindCPDirectionToPlayer(CPBlockingMe);
	else
		CPFacing = CPDirToGo;

	CPZPos /= 2;		// brings jumping guys back to earth

	CYCLE_GRAPHICS_MACRO(AI_BLOCKING_NUM);
}

void DiveAction(void);
void FreezeActionOnGround(void);


void StunBlockeeInit(void)
{ 	// an offensive blocker state NO - now a sub to save table space
EFFVAR blockee;

	blockee = CPGuyToBlock;

	if ( OPAction(blockee) != DiveAction
		&& OPAction(blockee) != FreezeActionOnGround)
	{
		OPSpeed(blockee) = 0;		// If the guy is stunned stop him.
		OPAction(blockee) = BlockeeStunned;
//		OPZPos(blockee) = ZERO;		// put jumping guys back on ground no in action
	}

	SetBlockerGraphics();	//
//	IncBlockerStateAndCheckBlockeeFarAway();

}

void UnStunBlockee(void)		// an offensive blocker state
{
EFFVAR blockee;

	if( (blockee = CPGuyToBlock) >= ZERO)
	{
		OPAction(blockee) = NoAction;
		OPZPos(blockee) = ZERO;		// make sure he's back on ground (if jumping)
	}
	SetBlockerGraphics();	//
	IncBlockerStateAndCheckBlockeeFarAway();

}


/*
WORD BGrunts[16] = {

	GRUNT0_SFX,GRUNT1_SFX,GRUNT2_SFX,GRUNT3_SFX,
	GRUNT4_SFX,GRUNT5_SFX,GRUNT6_SFX,GRUNT7_SFX,
	GRUNT0_SFX,GRUNT1_SFX,GRUNT2_SFX,GRUNT3_SFX,
	GRUNT4_SFX,GRUNT5_SFX,GRUNT6_SFX,GRUNT7_SFX


};
*/

void TriggerBlockingSounds(void)
{	// modified 8/13/96 - don't do sound now - only at end of AI loop
	// keep track of "loudest" sound this frame
	// must init MaxBlockAudioDistance = 32000 at beginning of AI loop

EFFVAR dist;


	if(gPNum > -1 && gPNum < 22)
	{
		dist = CPKeyDistance;
		if(dist < MaxBlockAudioDistance)
		{
			MaxBlockAudioDistance = dist;
			KeyAudioY = CPYPos;		// save for R/L channel later
		}
		SFXCount++;		// test
	}
}


BYTE ClipTable[16] =   {1,1,1,0,0,0,0,0,0, 0, 0, 0, 0, 0, 1, 1}; // leinient table
//BYTE ClipTable[16] =   {1,1,1,1,0,0,0,0,0, 0, 0, 0, 0, 1, 1, 1};


BOOL ImClipping(WORD posdir, WORD guyhit)
{	// returns TRUE if hitting "guyhit" would be a clip
EFFVAR diff;

	diff = posdir - OPFacing(guyhit);	// range is -15 to 15
	if(diff < 0)		// -15 maps to 1, -14 to 2, etc
		diff +=	16;


#ifdef PC_ALWAYS_MODE
{
BOOL clip;
	clip = ClipTable[diff];
	if(clip == 1)
		clip = clip;	// BP here - testing only
	return(clip);
}
#else

	return(ClipTable[diff]);
#endif


}

/*************************************************************************

						TryManyMoves - Push


	Used by blockers, defenders and HUMANS !!!
	Not used by offensive guys in scripts (i.e RunToRelative)

	If contact with guy on same team, then try to run around.

	If contact with guy on other team, check PUSH bit to see if
	other guy should be pushed or gone around.
	(Most everybody PUSHES, except blockees who are being blocked)

	NOTE: If the player wants to hit or block someone, then the "dir"
	had better be toward the player he wants to go to (figured before
	callng this), as when we step thru the 4 possible moves, if the first
	move doesn't hit anybody, that's what the player will do (move and
	not hit anybody)

	WARNING - Doesn't update CPDirToGo - do this outside of this routine

	INPUTS:	Dir to move

	RETURNS:guyhit ( a positive number) if pushed somebody
			NO_COLLISION  (-1 I think) if player moved
			TACKLE_STARTED (-2 I think)
			CANT_MOVE 		(-5 I think) if couldn't move or push in an y of the 4 dirs


 *************************************************************************/


WORD TryManyMovesPush(WORD dir)
{
EFFVAR i,guyhit, sp,action;
EFFVAR posdir;

	if (dir > -1)
		sp = IncrementPlayerSpeed();
	else
	{	/* dir = -16 to -1 mean move 0 to 15 and don't inc speed	*/
		dir += MAX_DIR;
		sp = CPSpeed >> SPEED_SHIFT;
	}

	for (i = 3; i > -1; i--)	// I just converted original table, so use it same way
	{
		posdir = TryManyMovesTable[dir][i];
		guyhit = TryMove(posdir, sp);		// for setting BP

		if (guyhit > TackleGuyTest)	// FOR TESTING DON'T TACKLE QB
		{
			if (!OnMyTeam(guyhit))
			{
//				CPGuyToBlock = guyhit;	// ?? are we sure we want to do this here???
				if( (action = CheckForTackleAndFumble(guyhit)) > ZERO)// Should we tackle him?
				{
					if(action == HIT_BC_BUT_CANT_TACKLE)
					{
						PushOtherGuy(guyhit,CPPushFactor);	// at least push him
						SetBlockerGraphics();	// will set CPFacing
					}
					// action == 2 means I really tackled him
					return(TACKLE_STARTED);
				}

				if (I_CAN_PUSH_MACRO
					&& !ImClipping(posdir,guyhit)
				//don't want human BC in blocking anim, let him go around
					&& gPNum != BallCarrier
					)	// can I push??
				{		// if so, push him; if not, try another move
					if(PlayStateNum < PS_PlayHappeningNUM)
					{
//						TriggerBlockingSounds();

						if(gPNum > 10)
						{	// def hit off before start of play
							InitDefPenaltyParameters(gPNum,ENCROACHMENT_PENALTY,StartBallX);
//							SetDefOffsides(DEF_ENCROACHMENT_TYPE);	//collision before snap
						}
					}
					else
					{
						SetBit1(EventFlags,FIRST_HIT_BFLAG);
						TriggerBlockingSounds();	//BUG #2006
					}

					PushOtherGuy(guyhit,CPPushFactor);
					SetBlockerGraphics();	// will set CPFacing
					return(guyhit);
				}
			}
			else if (PlayStateNum == PS_PlayHappeningNUM)
				// if fixes bug # 2006
				TriggerBlockingSounds();	// test only
		}
		else if (guyhit < -1) // Hit Sidelines (try move should do this, it doesn't)
		{
			if(guyhit == OUT_OF_BOUNDS_EAST)
			{	// stop if hit back endline
				Run();
				break;
			}
			else
				continue;
//			Shuffle();
//			return(CANT_MOVE);
		}
		else
		{
			if( !(CPFlags & IM_SPINNING_BFLAG) )
				TurnPlayerOptimal(dir);	//use direction passed in
			Run();
			return(NO_COLLISION);
		}
	}
//	Init2ndState();		// we couldn't go anywhere...
	return(CANT_MOVE);
}




/**************************************************************************

		PUSH OTHER GUY	- Do a push if there has been contact.

	NOTE: PushSpeedFactor is an index into the speed table, - should
	only be 0,1,2,3,or 4


 !!!!    DOES NOT UPDATE  EITHER PLAYERS  GRAPHICS   !!!!!

 **************************************************************************/

void PushOtherGuy(WORD guyhit, WORD PushSpeedFactor)
{
EFFVAR dir,tempPnum;

#if 0
	dir = CPDirToPush &15;	// mask off flags - okay, only used at init
	// note: DirToPush has flags in upper byte, so use > test
	if ( !(dir == PUSH_RIGHT || dir == PUSH_LEFT))
		if (randper > 50)	// Push Right or Left if no dir
			dir = PUSH_RIGHT;
		else
			dir = PUSH_LEFT;
#else
	if(CPDirToPush == NO_DIR)
	{
		if (randper > 50)	// Push Right or Left if no dir
			dir = PUSH_RIGHT;
		else
			dir = PUSH_LEFT;
		dir = PUSH_STRAIGHT;	// test only - force straight

	}
	else
		dir = CPDirToPush &15;	// mask off flags - okay, only used at init
#endif
	CPDirToPush = dir;	// save and use this (no high bits) from now on

	if(gPNum < 11)
	{	// pusher on offense

		if( (dir == PUSH_RIGHT && CPDirToGo == SE_DIR)
		  ||(dir == PUSH_LEFT && CPDirToGo == NE_DIR)
		  )
		{
			dir = CPDirToGo;	// if going "main" dir, no change
		}
		else	// else change
		{
			dir += CPDirToGo;	// Change current direction by -1 or 1
			dir &= MAX_DIR_MASK;	// make sure it's a legal dir
		}

		tempPnum = gPNum;		// save who I am
		gPNum = guyhit;		// set global to guyhit so MOVE rouitnes work

	/* move(push) both pusher and pushee the same PushSpeedFactor  */
		TryMove(dir,PushSpeedFactor );	// Move opponent where he should go
		DecrementPlayerSpeed(3,2);		// don't decelerate below speed2
	//	CPSpeed = 0;	this might be too much, especially at low speeds

		gPNum = tempPnum;		// restore me

//		TryMove(dir, PushSpeedFactor);	// Move to where I am supposed to go
		TryMove(CPDirToGo, PushSpeedFactor);	// Move to where I am supposed to go

		DecrementPlayerSpeed(2,2);


	}
	else	// player (pusher) on defense
	{

		dir += CPDirToGo;	// Change current direction by -1 or 1
		dir &= MAX_DIR_MASK;	// make sure it's a legal dir

		tempPnum = gPNum;
		gPNum = guyhit;

	/* move(push) both pusher and pushee the same PushSpeedFactor  */
		TryMove(dir,PushSpeedFactor );	// Move opponent where he should go
		DecrementPlayerSpeed(3,2);		// don't decelerate below speed2
	//	CPSpeed = 0;	this might be too much, especially at low speeds

		gPNum = tempPnum;

		TryMove(dir, PushSpeedFactor);	// Move to where I am supposed to go
//	TryMove(CPDirToGo, PushSpeedFactor);	// Move to where I am supposed to go

		DecrementPlayerSpeed(2,2);
	}
}





/*************************************************************************
 * ON MY TEAM - Returns 1 if on same team, 0 if on different teams
 *************************************************************************/
BOOL OnMyTeam(WORD guy)
{
//	BYTE i;

	if (gPNum > 10 && guy < 11)	// I am on Defense, he is on Offense
			return(FALSE);
	if (gPNum < 11 && guy > 10)	// I am on Offense, he is on Defense
			return(FALSE);

	return(TRUE);
}


/*************************************************************************

					BLOCKER FIND CLOSEST OPPONENT

	USED AT PLAY INIT  - use "BlockerFindClosestUnBlockedOpponent()"
	during the play


	Find closest within a certain box and NOT already assigned
	to be blocked.

	The closest guy in the box will be set as the CPGuyToBlock
	If no one in box, CPGuyToBlock will remain the same (NOBODY I assume)

*************************************************************************/

BOOL BlockerFindClosestOpponent(UWORD lowy,UWORD highy,UWORD highx)
{	// returns TRUE if found someone to block
EFFVAR dist,i,j,posguytoblock;
BOOL rv = FALSE;

	dist = 7*YARDS;			// no check guys closer than this
	posguytoblock = NOBODY;

	for (i = 11; i < 22; i++)		// check all defensive guys
	{	// now check if non-blocked and inside box
		if(OPBlockingMe(i) == NOBODY
			&& OPYPos(i) > lowy
			&& OPYPos(i) < highy
			&& OPXPos(i) <= highx
			&& OPXPos(i) >= CPXPos - YARD	// not behind - Sega bug#59
		  )
		{
			j = DistanceToPlayer(i);
			if (j < dist )
			{
				dist = j;
				posguytoblock = i;		   // We are blocking now...
			}
		}
	}
	if(posguytoblock != NOBODY)
	{
		SetGuyToBlock(gPNum,posguytoblock);
//		CPGuyToBlock = posguytoblock;
//		OPBlockingMe(posguytoblock) = gPNum;
		rv = TRUE;
	}

	return(rv);
}


/*-----------------------------------------------------***********/
BOOL BlockerFindClosestUnBlockedOpponent(UWORD lowy,UWORD highy,UWORD highx)
{
EFFVAR dist,i,j,posguytoblock;
BOOL rv = FALSE;

	dist = 6*YARDS;			// don't check guys closer than this
	posguytoblock = NOBODY;

	for (i = 11; i < 22; i++)		// check all defensive guys
	{	// now check if non-blocked and inside box
		if(  (OPBlockingMe(i) == NOBODY)	// this gets blitzing LBs
			  // following gets def. who have lost their blockers
		  || ((OPFlags(i) & (IM_BLOCKED_BFLAG | HAVE_BEEN_BLOCKED_BFLAG))
			== HAVE_BEEN_BLOCKED_BFLAG) )
		{
			if( OPYPos(i) >= lowy
				&& OPYPos(i) <= highy
				&& OPXPos(i) <= highx
				&& OPXPos(i) >= CPXPos - YARD	// not behind - Sega bug#59
			)
			{
				if ((j = DistanceToPlayer(i)) < dist )
				{
					dist = j;
					posguytoblock = i;	   // We are blocking now...
				}
			}
		}
	}

	if(posguytoblock != NOBODY)
	{
		SetGuyToBlock(gPNum,posguytoblock);
		rv = TRUE;
	}

	return(rv);
}



/**    following used if turnover or punt   ***************/
void DefBlockerFindClosestOpponent(UWORD lowy,UWORD highy,UWORD highx)
{
EFFVAR dist,i,j,posguytoblock;

	dist = 7*YARDS;			// no check guys closer than this
	posguytoblock = NOBODY;

	for (i = 0; i < 11; i++)		// check all offensive guys
	{	// now check if non-blocked and inside box
		if(OPBlockingMe(i) == NOBODY
			&& OPYPos(i) >= lowy
			&& OPYPos(i) <= highy
			&& OPXPos(i) <= highx)
		{
			if ((j = DistanceToPlayer(i)) < dist )
			{
				dist = j;
				posguytoblock = i;		   // We are blocking now...
			}
		}
	}

	if(posguytoblock != NOBODY)
	{
		SetGuyToBlock(gPNum,posguytoblock);
//		CPGuyToBlock = posguytoblock;
//		OPBlockingMe(posguytoblock) = gPNum;
	}
}


void SetGuyToBlock(WORD offslotnum,WORD defslotnum)
{
	OPGuyToBlock(offslotnum) = defslotnum;
	OPBlockingMe(defslotnum) = offslotnum;

}
/*************************************************************************

				Figure  Who  To  Block

	This routine is called the moment the ball is hiked to determine
	who the offensive line should block. I Know it's long but it's only
	called once per play!

	Okay to use gPNum

	It's outside of MovePlayers so it's okay to use gPNum - remember gPNum
	must be assigned to allow our "CP.." macros to work.

	The idea is to figure "key" (i.e SLANT, TRAP) blocks first and then
	normalblocks. Don't have guys block defenders who have
	already been assigned guys to block them.

	It's key here that the offensive line is defined in their slots as:
	RT=6, LT=7, RG=8,LG=9, C=10 - therefore the tackles are assigned guys
	to block first, then the guards, etc. based on "CheckBlockersTable"

*************************************************************************/
/** following table is used at moment ball is hiked to check possible
	blockers in that order based on slot num */
#define	MAX_BLOCKERS	10
/*  usual position based on slot#=            RT,LT,RG,LG, C,RTE,LTE,FB,HB,RR	*/
WORD NormalCheckBlockersTable[MAX_BLOCKERS] = {6, 7, 8, 9,10,  3,  2, 4, 5, 1};
/*  usual position based on slot#=               RT,LT, c,RG,LG,RTE,LTE,FB,HB,RR	*/
WORD CheckBlockers3ManLineTable[MAX_BLOCKERS] = {6, 7, 10, 8, 9, 3,  2, 4, 5, 1};


void FigureWhoToBlock(void)

{
EFFVAR i,j,d2push,yoff;
EFFVAR lowy,highy,highx,guy;
EFFVAR defform,dist;
WORD *tableptr;
UWORD blockwidth[2] = {YARD,2*YARD};

	tableptr = NormalCheckBlockersTable;	// assumption
	defform = GameTeamData[DefensiveSlotNum].FormationNumber;
	if(defform == DEF_3_4_FORM_NUM || defform == DEF_DIME_FORM_NUM)
		tableptr = CheckBlockers3ManLineTable;	// assumption was wrong



//	if(GameTeamData[DefensiveSlotNum].PlayNum == 184)
//	{	// test only - center blocks only def line
//		SetGuyToBlock(10,16);
//	}



	/*  human blitzing caused a problem - this fixes it  11/16/96 */

	if(OPFlags(0) & CAN_PASS_BFLAG || OPFlags(5) & CAN_PASS_BFLAG)	// passing play?
	{
		lowy = StartBallY-3*YARDS;
		highy = StartBallY+3*YARDS;
		highx = StartBallX+2*YARDS;

		for (gPNum = 11; gPNum < 22; gPNum++)	// check all defensive guys
		{	// now check if non-blocked and inside box
			if( (OPFlags(gPNum) & IS_HCONTROLLED_BFLAG)
				&& OPYPos(gPNum) >= lowy
				&& OPYPos(gPNum) <= highy
				&& OPXPos(gPNum) <= highx)
			{   // found human controlled guy ready to blitz

				dist = 70*YARDS;	// init max distance
				guy = -1;
				for (i = 10; i > 1; i--) // check all offensive blockers-& TE
				{	// now check if non-blocked and inside box
					if(OPFlags1(i) & IM_A_BLOCKER_BFLAG)
					{

						if ((j = DistanceToPlayer(i)) < dist )
						{
							dist = j;
							guy = i;		   // We are blocking now...
						}
					}
				}
				if(guy > -1)
				{
					SetGuyToBlock(guy,gPNum);	// off. guy now blocks human
					SetBit(OPFlags1(guy),IM_BLOCKING_HUMAN_BFLAG);	//get guy out of 3pt stance NOW!
				}
			}
		}
	}




 /* 1st check all guys doing slant or trap	*/

	for(i = 0; i<MAX_BLOCKERS; i++)	// check blockers in non-standard order
	{  //initing dirtopush should have been 1st STATE of blockers SCRIPT
		gPNum = tableptr[i];
		d2push = CPDirToPush;
		yoff = ZERO;

		// figure how far over to look for guy to block
		if(d2push & TRAP_BLOCK_BFLAG)
			yoff = 3*HALF_YARD;	// only look for defenders this far to right/left
		else if(d2push & SLANT_BLOCK_BFLAG)
			yoff = 1*QTR_YARD;	// only look for defenders this far to right/left

		if(yoff != ZERO)	// ... then find guy to trap or slant block
		{
			if((d2push & PUSH_LEFT) == PUSH_LEFT)
			{
				highy = CPYPos-yoff;	// check guys to left
				lowy = NORTH_SIDE_LINE;
				highx = CPXPos+7*YARD;
			}
			else
			{                      		// check all guys to right
				lowy = CPYPos+yoff;
				highy = SOUTH_SIDE_LINE;
				highx = CPXPos+7*YARD;
			}
			BlockerFindClosestOpponent(lowy,highy,highx);
		}
	}

/*  now check all guys who aren't doing slant or trap	*/
 for(j=0; j<2; j++)	// check guys directly in front, then further out
 {
	for(i = 0; i<MAX_BLOCKERS; i++)	// check blockers in non-standard order
	{  //initing dirtopush should have been 1st STATE of blockers SCRIPT
		gPNum = tableptr[i];

		if(CPGuyToBlock > -1)	// already have somebody??
			continue;

		if(CPDirToPush <= PUSH_LEFT)	// L = 15, R =1
		{
			// build a blocking lane to search for possible blockees
			if( CPFlags & PASS_BLOCKER_BFLAG)
//				highx = CPXPos + 3*YARDS;
				highx = CPXPos + 7*HALF_YARD; 	// lines start 3 yards apart
			else
				highx = CPXPos + 7*YARDS;


			if(CPDirToPush == PUSH_RIGHT)
			{                 // box favors right side
				lowy = CPYPos-1*YARD;
				highy = CPYPos+3*YARD;
			}
			else if(CPDirToPush == PUSH_LEFT)

			{				// box favors left side
				lowy = CPYPos-3*YARD;
				highy = CPYPos+1*YARD;
			}
			else	// should be STRAIGHT
			{				// box favors left side
				lowy = CPYPos-blockwidth[j];
				highy = CPYPos+blockwidth[j];
			}

			BlockerFindClosestOpponent(lowy,highy,highx);
		}
	}
  }
}


/*-----------------------------------------------------------------------
					INIT   PASS  BLOCKING

	This is a STATE for pass Blockers right before WAIT4Snap.  It will
	init a DirToPush so that when the ball is hiked, "FigureWhoToBlock()"
	will assign our pass blockers somebody to hit
------------------------------------------------------------------------*/
STATEDEF(WhoToPassBlock)
{
	SetBit(CPFlags,PASS_BLOCKER_BFLAG);

/*
	if (randper > 50)	// Push Right or Left if no dir
		CPDirToPush = PUSH_RIGHT;
	else
		CPDirToPush = PUSH_LEFT;
*/	CPDirToPush = PUSH_STRAIGHT;	// test only ???

	GoToNextState(1);
}


/*  following is our cheat routine - used to decrease blocking factors
	and improved defense rating if HUMAN runs the same plays (or rather
	the same players) over and over

	RETURNS: index 0-8 (9 possible) of the max number of either
	run or play pass offense has run in the last 8 real plays

	PrevOffPlayData[] has the last ballcarrier of last 8 plays OR
		 -1 if no gain or loss on play
	*/

EFFVAR FigureRunPassDistibutionForHuman(void)
{
EFFVAR i,lbc,index;
EFFVAR runplays=0,passplays=0;

	index = ZERO;

	if(PlayType < EX_PT1_PLAY
	 && Q_HumanTeamMacro(OffensiveSlotNum)
	 )
	{	// only human teams
		for (i = 0; i < MAX_OFF_PLAYDATA; i++)
		{
			if ( (lbc = GameTeamData[OffensiveSlotNum].PrevOffPlayData[i]) >= ZERO)
			{	// lbc = -1 if yards gained < 0
				if(lbc < 6)
					runplays++;
				else
					passplays++;
			}
		}
		index =  max(runplays,passplays);

	}
	return (index);	// values 0 -8
}




WORD TeamBlockFactorTable[MAX_OFF_PLAYDATA+1] = {
   0,0,0,0,0,  0,-1,-3,-7	};	// pre-bell curve


void SetTeamBlockFactor(void)
{	// called in InitAI AFTER both teams have picked play
EFFVAR index;

	index = FigureRunPassDistibutionForHuman();
	TeamBlockFactor =  TeamBlockFactorTable[index];

}



/*************
void SetTeamBlockFactor(void)
{	// called in InitAI AFTER both teams have picked play
EFFVAR i,lbc;
EFFVAR runplays=0,passplays=0;

	TeamBlockFactor = ZERO;		//assume (TBF is a global)

	if(PlayType < EX_PT1_PLAY && Q_HumanTeamMacro(OffensiveSlotNum))
	{	// only human teams
		for (i = 0; i < MAX_OFF_PLAYDATA; i++)
		{
			if ( (lbc = GameTeamData[OffensiveSlotNum].PrevOffPlayData[i]) >= ZERO)
			{	// lbc = -1 if yards gained < 0
				if(lbc < 6)
					runplays++;
				else
					passplays++;
			}
		}
		TeamBlockFactor =  TeamBlockFactorTable[max(runplays,passplays)];

	}

}

******/

/***************************   End of File - FBBlock.c  ********************/
