/**************************************************************


					FBDefAI.C  (FootBall DEFensive A.I.)

	INITIAL AUTHOR: M. Knauer       Start Date: 5/30/95

	ROUTINES WRITTEN BY: Mike Knauer
	Modify Date: 5/31/95


	Copyright 1995  GameTek, Inc.


	This file contains States for Defensive players.


	NOTE: All AI routines in the main AI loop refer to the
	"CURRENT PLAYER" (usually the loop counter of the AI loop 0-21 -
	often refered to by variable "gPNum")

	Any references to the players Data Stuctures are "hidden" in
	a macro starting wtih the initials CP. This allows us to
	more freely change the data structure (if needed) from the initial
	non-linked array based DS's (easier to debug) to a structure based
	data structure

	E.g.  CPXPos =  PlayerXPosition[gPNum]     (current as of 1/19/97)
		could be changed to
		 CPXPos =  Player[gPNum].XPosition
		or something like
		 initialize  	CP = Player[gPNum]
		 in macro		CPXPos =  CP->XPosition

**************************************************************/
	#include  <stdlib.h>		/* for rand & abs	*/

	#include  "fbequ.h"
	#include  "fbRAM.h"			/* external VARs needed by this file	*/
	#include  "FBPlays.h"		/* Script enums					*/

#if PC_ALWAYS_MODE
	#include  <stdio.h>
#endif
/*------------------ MACROS  ------------------------*/

#define STATEDEF(xyz) \
	void  (xyz)(void);  /* prototype  */  \
	void  (xyz)(void)   /* actual function name of the State  */



#define	NORMAL_2ND_STATE_TICKS	4


#define QTR_SECOND QUARTER_SECOND

/*---------------  function prototypes in this file  -----------*/

void CoverPossibleReceiver(void);
void ChaseGuy(WORD who);
void ChaseBall(void);			// a player STATE
void OffenseChaseBall(void);

WORD Distance(WORD x, WORD y);	// from CP to x,y
WORD Distance2Players(WORD p1, WORD p2);
BOOL CanDefGetBC(void);
BOOL CanOffGetBC(void);
BOOL QBOutOfPocket(WORD yards);

void DefShuffleRun(void);
void DefShuffleWait(void);




/*------  function prototypes in other files  -----------*/

WORD TryManyMoves(UWORD PossibleDirToMove);
WORD TryManyMovesPush(WORD dir);	 // to go directly through line to ballcarrier...
void  MakeTargetInBounds(void);

void  Run(void);
void  Walk(void);
BOOL  ReachedTarget(void);
BOOL TurnPlayerOptimal(WORD FinalDirection);
UWORD FindDirection(WORD DeltaX, WORD DeltaY);

void  PlayerHasBall(void);

void Init2ndState(WORD tickstostayinstate);
void SetInitialScriptState(void);	// a STATE, but okay to use as subroutine

void ExitStateWhenBC(void);

void DefLookReady(void);
void Shuffle(WORD shuffletype);			//in FBMOVE
void FaceBallandShuffle(void);


void StartJump(void);	/* a STATE (or Human controlled) subroutine	*/
BOOL OkayForDefLineToJump(void);

void RunToTargetState(void);
WORD FindClosestOff(void);

void StartDive(void);
void  InitGraphics(WORD graphicsnum);
void InitCPUBurst(WORD frames);	// frames to burst (if not currently bursting)

void NoAction(void);		/*  a NULL action	*/


/***    AI routines	*******/

void ReactToChaseBall(void);	//a Defensive AI routine
void SetReactTime(WORD timeuntilreaction);	// true Ticks (seconds) not frames



/**********************************************************************/

#if 1	/* new 10/16/96 - slower times to allow for
		players reaction rating to be added later		*/
WORD NormalDefReactTable[MAX_DIR] = {

	3*QTR_SECOND-10,	//NORTH_DIR		0
	4*QTR_SECOND-10,	//NNE_DIR		1
	6*QTR_SECOND-10,	//NE_DIR		2
	7*QTR_SECOND-10,	//ENE_DIR		3
	8*QTR_SECOND-10,	//EAST_DIR		4
	7*QTR_SECOND-10,	//ESE_DIR		5
	6*QTR_SECOND-10,	//SE_DIR		6
	4*QTR_SECOND-10,	//SSE_DIR		7
	3*QTR_SECOND-10,	//SOUTH_DIR		8
	2*QTR_SECOND-10,	//SSW_DIR		9
	1*QTR_SECOND-10,	//SW_DIR		10
	12-9,	//8,				//WSW_DIR		11
	8-6,  //0,				//WEST_DIR		12
	12-9,	//8,				//WNW_DIR		13
	1*QTR_SECOND-10,	//NW_DIR		14
	2*QTR_SECOND-10	//NNW_DIR		15
};
#else
WORD NormalDefReactTable[MAX_DIR] = {

	3*QTR_SECOND,	//NORTH_DIR		0
	4*QTR_SECOND,	//NNE_DIR		1
	6*QTR_SECOND,	//NE_DIR		2
	7*QTR_SECOND,	//ENE_DIR		3
	8*QTR_SECOND,	//EAST_DIR		4
	7*QTR_SECOND,	//ESE_DIR		5
	6*QTR_SECOND,	//SE_DIR		6
	4*QTR_SECOND,	//SSE_DIR		7
	3*QTR_SECOND,	//SOUTH_DIR		8
	2*QTR_SECOND,	//SSW_DIR		9
	1*QTR_SECOND,	//SW_DIR		10
	12,	//8,				//WSW_DIR		11
	8,  //0,				//WEST_DIR		12
	12,	//8,				//WNW_DIR		13
	1*QTR_SECOND,	//NW_DIR		14
	2*QTR_SECOND	//NNW_DIR		15
};
#endif
// 0 -3 = 1 frame
// CPReactionAbility = 0 -255
//                      0-32->8 ticks = 1/2 second

#if 0
#define BEST_REACT_ABILITY 256
WORD PlayerReactByPosTest[] = {
10*8,		//#define	QB		ZERO    /* Quarterback (duh)		*/
10*8,		//#define	RR		1		/* right receiver			*/
10*8,		//#define LR		2		/* left receiver			*/
10*8,		//#define	RTE		3		/* right tight end			*/
10*8,		//#define	FB		4		/* full back				*/
10*8,		//#define	HB		5		/* half back				*/
10*8,		//#define	WR3		6		/* wide receiver 3 			*/
10*8,		//#define	WR4		7		/* wide receiver 4 			*/
10*8,		//#define	WR5		8		/* wide receiver 5 			*/
10*8,		//#define	LTE		9		/* left tight end			*/
10*8,		//#define	TE3		10		/* extra tight end			*/
10*8,		//#define	RB3		11		/* extra running back		*/
10*8,		//#define	QB2 	12		/* backup quarter back		*/

10*8,		//#define	RT		13		/* offensive right tackle	*/
10*8,		//#define LT		14		/* offensive left tackle	*/
10*8,		//#define	RG		15		/* offensive right guard	*/
10*8,		//#define	LG		16		/* offensive left guard		*/
10*8,		//#define	CENTER	17		/* offensive center			*/
10*8,		//#define	OL6		18		/* extra offensive lineman	*/

10*8,		//#define PUNTER	19
10*8,		//#define	PUNT_RETURNER	20
10*8,		//#define	KICK_RETURNER	21
10*8,		//#define	KICKER	22

10*8,	   	//#define	DLE		23		/* defensive left end		*/
16*8,		//#define	DLT		24		/* defensive left tackle	*/
20*8,		//#define	DRT		25		/* defensive right tackle	*/
26*8,		//#define	DRE		26		/* defensive right end		*/
16*8,		//#define	FS		27		/* free safety				*/
23*8,		//#define	SS		28		/* strong safety			*/
10*8,		//#define	OLLB	29		/* outside left linebacker	*/
16*8,		//#define	ILLB	30	  	/* inside left linebacker	*/
20*8,		//#define	IRLB	31	  	/* inside right linebacker	*/
26*8,		//#define ORLB	32		/* outside right linebacker	*/
255,		//define	RC		33		/* right cornerback			*/
2*8,		//#define	LC		34		/* left cornerback			*/
16*8,		//#define	NB		35		/* nickel back				*/
16*8,		//#define	DB		36		/* dime back				*/
16*8,		//#define	DB7		37		/* extra defensive back		*/
16*8,		//#define	LB5		38		/* extra linebacker			*/
16*8		//#define	DL5		39		/* extra defensive line		*/
};

#endif

/*************************************************************************

			 T H E     D E F E N S I V E    S T A T E S


*************************************************************************/




STATEDEF(DefSnapWAIT)
{

	if(!CPStateInit)
	{   /* init target to run to  */
		CPSpeed = ZERO;		// in case guy was in motion
		CPStateInit = TRUE;
	}

	if(BallSnapped)
	{ // make all linemen down in crouch get up
	  //	- assume they're in "last" sequence when started
		if(CPAIGraphicState == AI_3PTSTANCE_NUM || CPAIGraphicState == AI_4PTSTANCE_NUM)
		{
			if(--CPAIGraphicSequence <= ZERO)
			{
				CPAIGraphicSequence = ZERO; // just to be safe
				GoToNextState(1);
			}
		}
		else
			GoToNextState(1);
	}
/*
	else if (randper == 1)	// Percent chance to jump offsides. Depends on position
	{
//		offsides = 1;
		GoToNextState(1);
	}
*/
}


#if	1
WORD DeflineDirtab[8] = {
//	SW_DIR,
//	NW_DIR,
	WSW_DIR,
	WSW_DIR,
	WNW_DIR,
	WNW_DIR,
	WEST_DIR,
	WEST_DIR,
	WEST_DIR,
	WEST_DIR
};
#else
WORD DeflineDirtab[8] = {
	WEST_DIR,
	WEST_DIR,
	WEST_DIR,
	WEST_DIR,
	WEST_DIR,
	WEST_DIR,
	WEST_DIR,
	WEST_DIR
	};
#endif


/*************************************************************************

						 D E F E N S I V E    L I N E

	The idea here is to have the defensive line stay in their lanes by
	trying to get 1.5 yards pass LOS before CHASING ball

**************************************************************************/

STATEDEF(DefLine)
{
	if (!CPStateInit)
	{
		if( *(CPScriptPtr+1) == DEF_LINE_BLITZER) /* parameter in SCRIPT says if blitzer or not	*/
			CPDirToGo = WEST_DIR;	// blitzers run straight in
		else
			CPDirToGo = DeflineDirtab[(rand() & 7)];	// Get one of 8 random directions

		CPTargetX = StartBallX - 3 * HALFYARD;
		CPTargetY = StartBallY;
		CPStateInit = 1;
// WARNING - CPReactTime MUST be reset every STATE if "SetReactTime()" used in that STATE
		CPReactTime = MAX_REACT_TIME;
	}

	if(BallFlags & BALL_PASSED_BFLAG)
	{
		if(OkayForDefLineToJump())
			StartJump(); 	// def. lineman blocking pass
		else
			GoToNextState(2);

	}
	else if( BallXPosition > CPXPos
		   || BallCarrier > 0)			// Ball Carrier not QB chase it...
	{
		GoToNextState(2);
	}
	else if( (BallFlags &  (BALL_PUNTED_BFLAG | BALL_KICKEDFG_BFLAG))
		   && (randper < 5)	// 10% seemed too high as they do this every frame
		   )
	{
		StartJump();	// try to block kick
	}
	else
	{
		TryManyMovesPush(CPDirToGo);	// Go to target
		if (ReachedTarget())			// If at target...
			GoToNextState(2);			// ... ChaseBall
	}
}



/*******************      BLITZING   STATES    *********************/

STATEDEF(SetDefRunUpTarget)		// used by [possible] blitzers
{
	CPTargetX = StartBallX+YARD;
	CPTargetY = CPYPos;

	GoToNextState(1);    //which better be RuntoTargetPreSnapState
}


STATEDEF(RuntoTargetPreSnapState)
{
	if (BallSnapped)
	{
		GoToNextState(1);
	}
	else
	{
		RunToTargetState();	// could GoToNextState(1);
		CPStateInit = TRUE;	// used by RunToTargetState(), mustbe init after it
	}
}


STATEDEF(BlitzerWait4Snap)
{
	if (BallSnapped)
	{
		if(!CPStateInit)
		{
			CPReactTime = 6 + (rand()&15);
			CPStateInit = TRUE;
		}
		if(TimeIntoPlay > CPReactTime)
			GoToNextState(1);
		else
			Shuffle(NORMAL_SHUFFLE);
	}
	else
	{
		CPFacing = WEST_DIR;
		Shuffle(NORMAL_SHUFFLE);
		CPSpeed = 2<<SPEED_SHIFT;	// give a little inertia
	}
}


/*------------------------------------------------------------------*/


BOOL DefensiveGuyShouldReact(WORD players_position)
{	// used in ZONE, CBRead & FSRead
BOOL rv=FALSE;

//	if(PassComingDownMACRO) //BallInAirMACRO)   // Past Line Of Scrimage or Passing
//	if(DefReactToPassMACRO || BallXPosition > CPXPos-YARD) //BallInAirMACRO)   // Passing - tweaked again!
	if( EventFlags.DEF_REACT_TO_PASS_BFLAG
	   || BallXPosition > CPXPos-YARD)
	{
		rv=TRUE;
	}
	else
	{
		 if (BallCarrier != NOBODY)   // NOBODY means pass/pitch it air
		 {		// Every body except FS & CBs react to nonQB BC
			if( (BallXPosition > StartBallX) )
			{
				rv=TRUE;
			}
			switch (players_position)
			{
				case FS:	// these guys always follow their man
				case RC:
				case LC:
					if(BallTargetX | BallTargetY) // is there a target?
					{
						if( (BallTargetY < NORTH_SIDELINE+6*YARDS
							 && CPYPos < MID_FIELDY)
						   ||
							(BallTargetY > SOUTH_SIDELINE-6*YARDS
							 && CPYPos > MID_FIELDY)
						  )
						{	// "out" pass on my side of field- cheat to get there quick
							if( (rand()&3) == ZERO)
							{  // only cheat X% of time (every frame)
								rv = TRUE;

							}
						}
					}
					break;

				default:

					if (BallCarrier > 0)
						rv=TRUE;
					break;
			}
		 }
	}
	return (rv);
}

/***********************************************************************

					  Z O N E  D E F E N S E

	The Zone Defense consists of two "boxes" for each zone.  The outside
	box is the actual zone that a defender covers.  There is also an
	inside box such that the defender will not go outside this inside
	box.

	The main algorithm is that if no one is in a defenders zone (outside box),
	the defender will go to the center of his zone (center of inside box).
	If someone comes in the defenders [outside] zone, the defender will
	move over toward him, but stay within his inside box. If more than
	two offensive players in the zone, the defender will average the
	distance between the two (or three..).

***********************************************************************/

// NOTE: targets (center of zone) should have been setup in InitPlay
STATEDEF(ZoneDefenseInit)
{
	CPTargetX = CPCenterZone.X;	// Set up Target X & Y
	CPTargetY = CPCenterZone.Y;
	CPDirToGo = FindCPDirectionToTargetMACRO;
// WARNING - CPReactTime MUST be reset every STATE if "SetReactTime()" used in that STATE
	CPReactTime = MAX_REACT_TIME;
	GoToNextState(1);
}


// NOTE: targets (center of zone) should have been setup in InitPlay
STATEDEF(ZoneDefense)
{
EFFVAR i,num = 0;
EFFVAR dir;
UEFFVAR x = 0,
	  x1, y1, mypos;
ULONG y = 0;



/**  Step 0 - Linebackers wait a bit before going to zone	*/
	mypos = CPMyFormPosition;
	if(mypos >= OLLB && mypos <= ORLB && (TimeIntoPlay < 1*SECONDS))	// was 2sec, but that's too long

	{	// then I am a linebacker in the first few moments of the play....
		DefShuffleWait();		// just hang out
		if (  (BallXPosition > StartBallX)	// unless something major happens
		 || PassComingDownMACRO	// was BallInAirMACRO
		 || BallCarrier > 0)	// Past Line Of Scrimage,Passing, or non QB BC
			GoToNextState(1);	// note - no delay

		return;
	}





/**  Step 1 - see if time to break out of zone	*/


	if(DefensiveGuyShouldReact(mypos))	// based on Pass in air, etc
		SetReactTime(NormalDefReactTable[CPFacing]);

/*****
	if (BallCarrier == NOBODY)
	{
		if(PassComingDownMACRO) //BallInAirMACRO)   // Past Line Of Scrimage or Passing
		// when reaction time up, then ChaseBall
			SetReactTime(NormalDefReactTable[CPFacing]);
	}
	else if ( (BallXPosition > StartBallX) || PassComingDownMACRO) //BallInAirMACRO)   // Past Line Of Scrimage or Passing
	{  	// when reaction time up, then ChaseBall
		SetReactTime(NormalDefReactTable[CPFacing]);
	}
	else if (BallCarrier > 0  && mypos != FS)	// Every except FS react to nonQB BC
	{  	// when reaction time up, then ChaseBall
		SetReactTime(NormalDefReactTable[CPFacing]);
	}
*/

/**  Step 2 - set up target to go to	*/
	for (i = 0; i < 6; i++)	// Check for receivers in Zone
	{					// also check QB and treat as rec if not past LOS
		x1 = OPXPos(i);			// Do once so it's in a local var for speed
		y1 = OPYPos(i);

		// Is he in Zone ???
		if (x1 > CPOutsideZone.Low.X && x1 < CPOutsideZone.High.X &&
			 y1 > CPOutsideZone.Low.Y && y1 < CPOutsideZone.High.Y)
		{
			if(i==ZERO)	//QB in my zone??? added 6/19/96
			{
				GoToNextState(1);	// .. if so, go get 'em
				return;
			}

			if (x < x1)
				x = x1 + 2 * YARD;	// Run in front of Opponent closest to endzone
			y += y1;							// Need Y average
			num++;							// Divisor
		}
	}

	if (num)	// If Receiver is in your Zone, stay within "inside" box
	{
		y /= num;						// Average Y

		if (x > CPInsideZone.High.X)	//	Check Inner box & make values inside it...
			x = CPInsideZone.High.X;

		if (x < CPInsideZone.Low.X)
			x = CPInsideZone.Low.X;

		if (y > CPInsideZone.High.Y)
			y = CPInsideZone.High.Y;

		if (y < CPInsideZone.Low.Y)
			y = CPInsideZone.Low.Y;

		CPTargetX = x;						// Set up Target X & Y
		CPTargetY = y;
	}
	else		// no one in zone
	{
#if	0  // if Quarterback is within 3 yards of yer zone
		if (OPXPos(0) + YARD > CPOutsideZone.Low.X)
		{
			GoToNextState(1);
			return;
		}
#endif

		CPTargetX = CPCenterZone.X;	// Set up Target X & Y
		CPTargetY = CPCenterZone.Y;
	}

/**  Step 3 - now that we have a target - go to it	*/
	dir = FindCPDirectionToTargetMACRO;

	if(dir > (EAST_DIR - 3) && dir < (EAST_DIR + 3))
	{  // if my zone goes to endzone, then cover this guy
		if( (CPOutsideZone.High.X > OFF_GOAL_LINE-1)
		   && (num)
		   && (CPXPos < x+2*YARDS) // ...when he's with 2 yards of me
		  )
		{
			No_Back_Pedal_Macro;	// and no run backwards
		}
		else if( CPOutsideZone.High.X > OFF_GOAL_LINE-1)
		{ // nobody in my zone yet
			if(CPXPos < StartBallX+18*YARDS
			   && (gPNum == 17 || gPNum == 12)  //
			  )
			{	// hurry to my zone if a CB and have deep zone coverage
				No_Back_Pedal_Macro;
				SetBit(CPFlags,I_RUN_BACKWARD_BFLAG);	// faster but keep facing WEST
			}
/*			else // Im close to zone and nobody near
			{
				if(CPFacing == WEST_DIR)
				{
					Do_Back_Pedal_Macro;
				}
				else
				{
					CPFacing--;
					CPFacing &= 15;
				}
			}*/
		}
		else
		{
			Do_Back_Pedal_Macro;
		}
	}
	else
	{
		No_Back_Pedal_Macro;
	}



	if(CPTargetX == CPXPos && CPTargetY ==	CPYPos)
	{	// we're at target
		DefShuffleWait();
	}
	else
	{
		CPDirToGo = dir;
		if (TryManyMoves(dir) != CANT_MOVE)
			Run();
		else
			Shuffle(NORMAL_SHUFFLE);

		if (ReachedTarget())
		{
			CPXPos = CPTargetX;	// Make them exact so next time we know if
			CPYPos = CPTargetY; // we're at center or not
//			CPAction = NoAction;	// Kill any action
		}
	}
}




STATEDEF(DefRunToPuntTargetState)
{	// this STATE (SCRIPT) is force to all defenders when ball in punted
	if(BallTargetX < YARD)
	{
		FaceBallandShuffle();
	}
	else
	{
		if(!CPStateInit)
		{
			CPTargetX = BallTargetX - 3*YARDS - (rand()&3)*YARDS;
			CPTargetY = BallTargetY - (WORD)(StartBallY - CPYPos)/2;
//			CPStateInit = TRUE;
		}

		RunToTargetState();
		CPStateInit = TRUE;	// used by RunToTargetState(), must be init after it

	}
}


STATEDEF(CBCheckForPunt)
{
// if (Punt)
//	{
//		if (randper < 26)
//			CPState = DefBeBlockerState;	// 25% chance to be a good blocker...
//	}
//	else
		GoToNextState(1);
}


/*************************************************************************

					CORNER BACK READ - Man To Man

	GuyToCover MUST have already been Inited!!!! (in PlayInit)


*************************************************************************/

STATEDEF(CBRead)
{
	if (CPStateInit == ZERO)
	{
		CPStateInit = 1;  		// MUST use incremental STATEInit if using "CoverPosRec"
//		CPStateInit = TRUE;
// WARNING - CPReactTime MUST be reset every STATE if "SetReactTime()" used in that STATE
		CPReactTime = MAX_REACT_TIME;
	}

//	if (  (BallXPosition > StartBallX)
//		 || PassComingDownMACRO			//BallInAirMACRO
//		 || BallCarrier > 0)	// Past Line Of Scrimage or Passing fix above...
	if(DefensiveGuyShouldReact(RC))
	{
		SetReactTime(NormalDefReactTable[CPFacing]);	// when reaction time up, then ChaseBall
	}

	CoverPossibleReceiver();

	if(   (OPGuyToBlock(CPGuyToCover)  >10) //guy I'm covering blocking someone on def.
	   && (OPGuyToBlock(CPGuyToCover) != gPNum)	// but not me
	   )
	{	// guy I'm covering is blocking someone else (Wishbone sweep vs. man)
		SetNewScript(ChaseBallScriptNUM);
		SetAI(NoAI);
	}


}



void SetReactTime(WORD timeuntilreaction)
{

	timeuntilreaction += ((100-CPReaction)/3);

	if(PassCatcher != NOBODY)	// react right away if somebody caught pass
		timeuntilreaction = ZERO;

	SetAI(ReactToChaseBall);
	// set react time to minimum of "current react time" or value passed
	if(timeuntilreaction < CPReactTime)
		CPReactTime = timeuntilreaction;	// a count down timer based on ticks

}

/*-------------------------------------------------------------------

	When the Defensive man's Reaction time has expired, he
	will then "chaseBall"
-------------------------------------------------------------------*/


void ReactToChaseBall(void)	//a Defensive AI routine
{
	CPReactTime -= TICKS_PER_AI_FRAME;
	if(CPReactTime <= ZERO)
	{
		CPReactTime = 10*SECONDS+1;	//reset to a high value in case used again
		SetNewScript(ChaseBallScriptNUM);
		SetAI(NoAI);
	}
}



/*-----------------------------------------------------------------*/

BYTE LBReadOnQBDIR[16] = {
	NORTH_DIR,
	NORTH_DIR,
	WEST_DIR,
	WEST_DIR,

	WEST_DIR,
	WEST_DIR,
	WEST_DIR,
	SOUTH_DIR,

	SOUTH_DIR,
	SOUTH_DIR,
	SOUTH_DIR,
	SW_DIR,

	WEST_DIR,
	NW_DIR,
	NORTH_DIR,
	NORTH_DIR
};

/*************************************************************************

					LINEBACKER READ - Man To Man

	Also used for SS, NB, DB

	LBRead rewritten 8/24/96  - before was too powerful (too good)
	now takes into account reaction time


 *************************************************************************/

STATEDEF(LBRead)
{
EFFVAR dir;
BOOL react = FALSE;

//	return;		// see if this improves running game
		// If ball past LOS or Pass

	if (CPStateInit == ZERO)
	{
		CPStateInit = 1;  		// MUST use incremental STATEInit if using "CoverPosRec"
// WARNING - CPReactTime MUST be reset every STATE if "SetReactTime()" used in that STATE
		CPReactTime = MAX_REACT_TIME;
	}

	if(CPStateInit > 1)
	{
		ChaseGuy(5);	// chase RB
		CPStateInit--;	// for a few frames

	}
	else if( (PlayFlags & POS_HANDOFF_BFLAG)	// LBs bite on fake Handoff ?
	   && (randper <  (CPReaction>>1))	// here high reaction is a detriment
	  )
	{
//		GoToNextState(1);	 this sent them right to QB
//		CPStateInit = AI_FRAME_RATE + (rand()&15);	// 1 - 1 3/4 seconds
		CPStateInit = AI_FRAME_RATE/2 + (rand()&7)
						+ (101 - CPReaction)/7;		// 1/2  - 1 3/4 seconds

		ChaseGuy(5);	// chase RB

	}
	else
	{
		if ( ( (BallXPosition > StartBallX) && (BallCarrier >=ZERO))
			|| PassComingDownMACRO 	//1/2 half - slower than DBS
			|| CPXPos < StartBallX  // somehow I got past LOS
			|| BallCarrier > ZERO	// RB has ball
		   )
		{
			react = TRUE;		//GoToNextState(1);
		}

		if ( CPGuyToCover != NOBODY
//			&& (!(OPFlags1(CPGuyToCover)&IM_A_BLOCKER_BFLAG))
		   )
		{   // don't cover guy who is blocking
			CoverPossibleReceiver();
		}
		else	// QB still has ball (or I'm waiting to react)
		{
			No_Back_Pedal_Macro;		// get out of backpedal state
			if (QBOutOfPocket(4*YARDS))	// AND I'm on same side of field as QB
			{
				react = TRUE;
//				GoToNextState(1);	// added 2/16/96 - had to be a bug!?!
			}

			if ((OPSpeed(0) < (3<<SPEED_SHIFT)) )	// Not Running
			{
				DefLookReady();	// shuffle without moving
			}
			else
			{
				if (OPFlags(0) & BACK_PEDAL_BFLAG)	// If QB Backpeddling?
					dir = EAST_DIR;	// mimic him back (he'll probably pass)
				else
					dir = LBReadOnQBDIR[OPDirToGo(0)];

				if(dir < SW_DIR || dir >NW_DIR)
				{
					Do_Back_Pedal_Macro;	// Set up Back Pedal
				}

				CPDirToGo = dir;
				TryManyMoves(dir);	// to push or not to push
				Run();
			}
		}

		if(react)
		{
			SetReactTime(3*QTR_SECOND);		// test for now
//			SetReactTime(1);		// test for now
		}
	}
}


/*-----------------------------------------------------------*/

STATEDEF(LBCoverQB)	// similar to LBRead, but a little less
{					// used in QB Spy play or some Dime plays
EFFVAR dir;

		// If ball past LOS or Pass
	if ( BallXPosition > StartBallX
		|| PassComingDownMACRO 	//BallInAirMACRO)
		|| CPXPos < StartBallX  // somehow I got past LOS
	   )
	{
		GoToNextState(1);
	}
	else if (BallCarrier < 1)		// NOBODY or QB has ball
	{
		if (CPGuyToCover != NOBODY)
			CoverPossibleReceiver();
		else
		{
			No_Back_Pedal_Macro;		// get out of backpedal state
			if (QBOutOfPocket(8*YARDS))	// AND I'm on same side of field as QB
			{
				GoToNextState(1);	// added 2/16/96 - had to be a bug!?!
			}
			else if ((OPSpeed(0) < (3<<SPEED_SHIFT)) )	// Not Running
			{
				DefLookReady();

			}
			else
			{
				if (OPFlags(0) & BACK_PEDAL_BFLAG)	// If QB Backpeddling?
					dir = EAST_DIR;
				else
					dir = LBReadOnQBDIR[OPDirToGo(0)];

				if(dir < SW_DIR || dir >NW_DIR)
				{
					Do_Back_Pedal_Macro;	// Set up Back Pedal
				}

				CPDirToGo = dir;
				TryManyMoves(dir);
				Run();
			}
		}
	}
	else							// Player other than QB has ball
	{
		GoToNextState(1);
	}
}




/*************************************************************************

					FSREAD - Free Safety Read State



*************************************************************************/
#define MAX_DEF_COVERX1 (OFF_GOAL_LINE+6*YARDS)
STATEDEF(FSRead)
{
EFFVAR i, x, dir,
	 receiver = 0;
UEFFVAR targetX;

	if (!CPStateInit)
	{
		targetX = (UWORD)CPXPos + (UWORD)(15 * YARD);
		if (targetX > MAX_DEF_COVERX1)	// Check for endzone stuff...
			targetX	= MAX_DEF_COVERX1;
		CPTargetX = targetX;
		CPStateInit = 1;  		// MUST use incremental STATEInit if using "CoverPosRec"
// WARNING - CPReactTime MUST be reset every STATE if "SetReactTime()" used in that STATE
		CPReactTime = MAX_REACT_TIME;
	}

	No_Back_Pedal_Macro;

 //  NOTE: FS does not react to BC unless past LOS
//	if ( (BallXPosition > StartBallX) || PassComingDownMACRO) //BallInAirMACRO)   // Past Line Of Scrimage or Passing
	if(DefensiveGuyShouldReact(FS))
	{  	// when reaction time up, then ChaseBall
		SetReactTime(NormalDefReactTable[CPFacing]);
	}

	if(   (OPGuyToBlock(CPGuyToCover)  >10) //guy I'm covering blocking someone on def.
	   && (OPGuyToBlock(CPGuyToCover) != gPNum)	// but not me
	   )
	{	// guy I'm covering is blocking someone else (Wishbone sweep vs. man)
		CPGuyToCover = NOBODY;		// so don't cover him
	}

	if (CPGuyToCover != NOBODY)		// If there is a guy to cover, do it
	{
		CoverPossibleReceiver();
	}
	else				// Check to see if FS should cover somebody
	{
		x = CPXPos - 5 * YARDS;		// X position - 10 yards.
		for (i = 0; i < 6; i++)		// Check players 0 - 5
			if (OPXPos(i) > x)			// He is within 10 yards
				receiver = i;

		if (receiver)						// Someone got past me...
		{
			CPGuyToCover = receiver; 	// Now I do have a guy to cover
			CoverPossibleReceiver();
		}
		else
		{
			if(!CPStateInit)
			{
				CPTargetX = CPXPos + 15 * YARD;
				if (CPTargetX > MAX_DEF_COVERX)	// Check for endzone stuff...
					CPTargetX = MAX_DEF_COVERX;

				CPStateInit = TRUE;
			}
			if (CPXPos < CPTargetX) // back Ped 15 yards max
				Do_Back_Pedal_Macro;
		///		CPSpeed |= BACK_PEDAL_FLAG;	// Set Back Pedal Flag

		// Check for pitch

		// follow ball carrier left or right
			if (abs(BallYPosition - CPYPos) < 512) // if within 2 yards
				dir = EAST_DIR;
			else if (BallYPosition > CPYPos)
				dir = SE_DIR;
			else
				dir = NE_DIR;

			TryManyMoves(dir);
			Run();
		}
	}
}


/*************************************************************************

					COVER POSSIBLE  RECEIVER

	The idea here is to "cover" the man up to a certain point
	and then "chase" him.

		"CPGuyToCover"  must not be NOBODY	!!!


	_____________________WARNING ___________________

 trickery with CPStateInit (so all STATE which call this routine be AWARE)

	Will set CPSTateInit negative if supposed to chase guy, not
	just cover him

*************************************************************************/
//#define COVER_WIDTH (2*YARDS)
#define COVER_WIDTH (5*HALFYARD)	//10/5/96 (bug#297

void CoverPossibleReceiver(void)
{

EFFVAR guytocover; 	// local var probably saves a few bytes of code
EFFVAR coverwidth;

	guytocover= CPGuyToCover;
	if(guytocover > ZERO && guytocover == BallCarrier)
	{
		ChaseGuy(guytocover);	// last frame for this
		SetNewScript(ChaseBallScriptNUM);	// next frame go for him
		SetAI(NoAI);
	}
//	if(	(CPFlags & PRIORITY_COVER_BFLAG) || (CPStateInit > 1))
	else if(	CPStateInit < ZERO)
	{
		ChaseGuy(guytocover);
	}
	else
	{
//		coverwidth = COVER_WIDTH - CPReaction*2;  //512- 0to200)
	//bug #2997(10/5/96) - lowest reaction is 30
		// 640 +180 = 820 - 600
		coverwidth = COVER_WIDTH+180 - CPReaction*6;  //640-220)

		if( OPYPos(guytocover) < CPYPos-coverwidth
			|| OPYPos(guytocover) > CPYPos+coverwidth
//			|| CPXPos > OFF_END_LINE-2*YARDS)	// i'm near back of EZ
			|| CPXPos > OFF_END_LINE-10*YARDS)	// i'm near back of EZ
		{  // guy is outside of box in front of defender
			ChaseGuy(guytocover);			// chase him now...
			CPStateInit = -2;		//  ... and forever more (must be neg. for LBRead)
		}
		else
		{	// reciever inside of my box
//			if(OPXPos(guytocover) > CPXPos-3*HALFYARD)
			if(OPXPos(guytocover) > CPXPos-4*HALFYARD)
			{	// if getting too close..
				ChaseGuy(guytocover);			// chase him now...
				CPStateInit = -2;		//  ... and forever more
			}
			else if(OPXPos(guytocover) > CPXPos-12*HALFYARD)
			{	// slightly in front of
				Do_Back_Pedal_Macro;
				CPDirToGo = DOWN_FIELD_DIR;
// old			if(TryManyMoves(UP_FIELD_DIR) == NO_COLLISION)
				// keep trying to move in that direction even if contact
				TryManyMoves(DOWN_FIELD_DIR);
				Run();		// will handle backpedaling
				CPFacing = TOWARD_OFFENSE_DIR;  // but always face this way
			}
			else	// wait for him to get close
				DefShuffleWait();
		}
	}
}


/*************************************************************************

			 C H A S E    G U Y   (POSSIBLE RECEIVER)

	This is the second step in a defender covering a possible receiver
	(the first being "cover guy")

	This subroutine sticks with him much closer.

*************************************************************************/

void ChaseGuy(WORD guytocover)
{
EFFVAR dir,dist;

//	guytocover= CPGuyToCover;

	No_Back_Pedal_Macro;

	if(CPXPos > OFF_GOAL_LINE-3*YARDS)
		dist = 2 *HALFYARD;   // don't give leeway if close to EZ
	else
		dist = 6 *HALFYARD;
/*
	if( (OPFlags1(guytocover) & IM_A_BLOCKER_BFLAG)
	   && (OPGuyToBlock(guytocover) != gPNum)
	   )
	{	// guy I'm covering is blocking someone else (Wishbone sweep vs. man)
		DefShuffleWait();
	}
	else		*/

	if(DistanceToPlayer(guytocover) > dist)	//FS didn't cover very close 1/12/96
	{
	 // first index is dir to opponent, 2nd is dir of opponent
		dir = DirVsDirTable[FindCPDirectionToPlayer(guytocover)][OPDirToGo(guytocover)];

	}
	else
	{		// If Behind me & I'm Faster, keep going at his speed...
		if (CPXPos > OPXPos(CPGuyToCover) && CPSpeed > OPSpeed(CPGuyToCover)) // If you are faster stay with him...
			CPSpeed = OPSpeed(CPGuyToCover);

//		dir = FindCPDirectionToPlayer(CPGuyToCover);	// find direction of receiver
		dir = OPDirToGo(guytocover);	// mimic other guys direction
	}
	CPDirToGo = dir;
	if (TryManyMoves(dir) == NO_COLLISION)
		Run();
	else
		DefShuffleWait();

}


BOOL OkayForDefLineToJump(void)
{	// used by FG blockers or trying to block pass

   return(BallXPosition > CPXPos - 8*HALFYARD		//2 *YARDS
		 && BallYPosition > CPYPos - 2 *YARDS
		 && BallYPosition < CPYPos + 2 *YARDS
		 && BallZPosition < 9*FEET
		 && CPFacing > SSW_DIR		// must face QB
		 );

}

STATEDEF(DefAfterFGKickedState)
{	// init when FG is kicked

	if(BallXPosition > CPXPos + 2 *YARDS)
	{
		FaceBallandShuffle();
	}
	else if(OkayForDefLineToJump())
	{
		StartJump();
	}
	else
	{
		CPDirToGo = FindCPDirectionToBallMACRO;
		TryManyMovesPush(CPDirToGo);
	}
	// only way to exit this STATE is if Kick blocked (in BallBeingFGed())
}


void ChaseOffensiveBallCarrier(BOOL refiguredir);	// a subroutine of ChaseBall
void ChaseDefensiveBallCarrier(void);	// a subroutine of ChaseBall

/*************************************************************************

							CHASE BALL

	This is the main defensive AI routine, usually invoked once the
	final Ball Carrier has been determined, such as the ball crossing
	the LineOfScrimmage.


	Can be used by both Defense and Offense(if turnover) !

	This is a complicated piece of code so it is as straightforward
	as possible (even if it wastes code).


*************************************************************************/


/**------------------------------------------------------------**/
WORD DefGetPassTable[3] = {5,45,80};	// based on game difficulty


void DefChaseBall(void)
{
BOOL refiguredir = FALSE;
BOOL watchit = FALSE;
EFFVAR tx,ty,diff;


	if(!CPStateInit)
	{
		refiguredir = TRUE;
		CPStateInit = 1;
	}

	if( ( (AIFrameCounter+gPNum) & 3) == ZERO)
		refiguredir = TRUE;



	if(BallFlags & (BALL_PASSED_BFLAG | BALL_PUNTED_BFLAG
					| BALL_KICKEDOFF_BFLAG |BALL_KICKEDFG_BFLAG) )
	{ // ball in air - pass or any kick (COULD BE BOUNCING!!!)

		tx = BallXPosition;		// assume we run to ball
		ty = BallYPosition;

		if(BallFlags & BALL_PASSED_BFLAG)
		{
			if(BallXPosition > CPXPos + 6*YARDS)
			{  // stay where I'm at, watch it (mainly linemen)
				watchit = TRUE;
			  //	tx = CPXPos;
			 //	ty = CPYPos;
			}
			else if (  (BallZPosition < 2*YARD-1)
			 || (BallFlags & PASS_DEFLECTED_BFLAG)  )
			{	// if low OR deflected go directly toward it
//				CPTargetX =	BallXPosition;
//				CPTargetY = BallYPosition;
			}
			else 	// if high in air run to where it should hit
			{
				tx = BallTargetX;
				ty = BallTargetY;

				// figure if I should burst
				if( (TheReceiver != NOBODY && CPGuyToCover == TheReceiver)
				   || DefenseKeyDistance[0].slotnum == gPNum
				  )
				{	// receiver is my guy or I'm closest to target

					if(CPStateInit == 1)	//1st time in - see if burst or int.
					{   // smarter def. depending on difficulty mode
						CPStateInit = 2;
						diff = DefGetPassTable[GameOptions[GAME_OPTNS_DIFFICULTY]];

						if(   OPDirToGo(TheReceiver) == NORTH_DIR
						   || OPDirToGo(TheReceiver) == SOUTH_DIR)
						{ // rec. running "out" pattern
							diff += 15;	// even bigger chance
						}

						if(randper < diff)
						{
							CPStateInit = 3;	// go for interception
							refiguredir = TRUE;
						}
						if(randper < diff)	// and/or burst
							InitCPUBurst(25 + (rand()&15));
					}
				}
				if(CPStateInit == 3)
				{
					tx -= (3*HALFYARD); // go for interception
				}
			}
		}
		else if(BallFlags & BALL_KICKEDFG_BFLAG)
		{
			if(BallXPosition > CPXPos + 2*YARDS)
			{  // stay where I'm at, watch it
				watchit = TRUE;
			  /*
				tx = CPXPos;
				ty = CPYPos;*/
			}
		//	else go right to it
		}
		else if(BallFlags & BALL_KICKEDOFF_BFLAG)
		{
			if (  BallZPosition > 2*YARD-1)
			{	// if high in air run to where it should hit
				tx = BallTargetX;
				ty = BallTargetY;
			}
			// else go to Ball
		}
		else
		{      // SS does this
			#if PC_ALWAYS_MODE
//			printf("\nDef.(%d) in ChaseBall on Punt",gPNum);
			#endif
		}


		if(watchit)
		{    	// passive
			FaceBallandShuffle();
		}
		else if (Distance(tx, ty) < YARD)	// I'm near target
		{
			if(   (Distance(BallXPosition,BallYPosition) < 12*HALFYARD)	//was 9,3 HY
			   && (CPXPos < tx)	// Im in front of target
			   && (BallXPosition < CPXPos-1*YARD)
			   && (CPFacing > SOUTH_DIR)
			   && ( !(BallFlags & BALL_PUNTED_BFLAG))
			   && (randper < 25)
			  )
			{
				StartJump();	// to catch pass
			}
			else
			{
//				FaceBallandShuffle();
				CPDirToGo = FindCPDirectionToBallMACRO;
				TurnPlayerOptimal(CPDirToGo);
				Shuffle(BLOCKING_SHUFFLE);	// BLOCKING_SHUFFLE is okay 10/23/96
			}
		}
		else
		{
			CPTargetX = tx;
			CPTargetY = ty;
			if(refiguredir)		// else leave it what it was last time
				CPDirToGo = FindCPDirectionToTargetMACRO;

/*			if( BallFlags & PASS_DEFLECTED_BFLAG
				|| BallZPosition < 3
			  )
			{
				if(TryManyMovesPush(CPDirToGo) == NO_COLLISION)  //
					Run();
			}
			else
			{ */
				TryManyMoves(CPDirToGo);	// note: NO PUSHING when Ball in Air
				Run();		// run even if collision
//			}
		}
	}
	else if (BallCarrier == NOBODY)
	{	// must be a fumble or something - Ball Actions should give someone ball
		if ( (LastBallCarrier == NOBODY) || (BallFlags & BALL_FREE_BFLAG) )	// must be punt, FG
			CPDirToGo = FindCPDirectionToBallMACRO;
		else
			CPDirToGo = FindCPDirectionToPlayer(LastBallCarrier);

		TryManyMovesPush(CPDirToGo);
	}
	else if (BallCarrier > 10)		// Defense has ball
	{
		#if PC_ALWAYS_MODE
		printf("\nERROR - def.(%d) chasing Def BC(%d)",gPNum,BallCarrier);
		#endif
	}
	else
	{
		ChaseOffensiveBallCarrier(refiguredir);	// doesn't look at CPStateInit
	}


}


/***********************************************************************/


void ChaseOffensiveBallCarrier(BOOL refiguredir)	// a subroutine of ChaseBall
{
EFFVAR dir2go,rnum;
EFFVAR disttoBC;

BOOL dive = FALSE;


	 /*** Priority 1 - run directly to BC	***/

	rnum = rand();	// use this throughout
	disttoBC = CPKeyDistance;		// DistanceToPlayer(BallCarrier);
	dir2go = CPDirToGo;	// assume I'll end up same way I was going

/* If he is going slow OR if he is within 1.5 yards - go straight at him 	*/
	if ( (OPSpeed(BallCarrier) < (3<<SPEED_SHIFT)+1 )
		 || disttoBC < 3 * HALFYARD)
	{
		dir2go = FindCPDirectionToPlayer(BallCarrier);
//		if (CPXPos < OPXPos(BallCarrier) + HALF_YARD	// bug #2468 - dive if in front
		if (CPXPos < OPXPos(BallCarrier) + QTR_YARD	// 11/5/96 - Sega
		   && disttoBC < 3 * HALFYARD)
//		if (CPXPos < OPXPos(BallCarrier) && disttoBC < 3 * YARD)	//bug #2246
		{
			if(CPFacing == dir2go)	// If he is within X yards and past me...
			{
				dive = TRUE;
			}
		}
	}

/*------ Priority 2 - run to a yard in front of BC	-------*/

//	else if (disttoBC < 3 * YARD || DefenseKeyDistance[0].slotnum == gPNum)
	else if (disttoBC < 7 * HALFYARD || DefenseKeyDistance[0].slotnum == gPNum)
	{
			// If < 3 yards or Closest Defender
//		if ( (CPXPos < OPXPos(BallCarrier) + HALF_YARD)
		if ( (CPXPos < OPXPos(BallCarrier) + QTR_YARD)	// 11/5/96 - Sega
			&& (CPFacing == dir2go)	// If I'm facing him and he's past me...
//			&& (disttoBC < 3 * YARD)	//bug #2246
			&& (disttoBC < 7 * HALFYARD)	//bug #2269 - modified 8/11/96
		   )
		{
			dive = TRUE;
		}
		else
		{
			CPTargetX =	OPXPos(BallCarrier) + YARD;	// assume Target Ball Carrier
			CPTargetY = OPYPos(BallCarrier);
			dir2go = FindCPDirectionToTargetMACRO;
			if(CPXPos < OPXPos(BallCarrier) - 6 *HALFYARD)
			{
				InitCPUBurst(10 + (rnum&7));
			}
		}
	}

/*-------- Priority 3 - make sure I can get him before he gets a TD	-----*/

	else if (!CanDefGetBC())
	{
		if(refiguredir)
		{
			CPTargetX = OFF_GOAL_LINE - HALFYARD;	// Try to get him at 1/2 yard line
			CPTargetY = OPYPos(BallCarrier);
			dir2go = FindCPDirectionToTargetMACRO;
		}
		else	// stop the wiggles
			dir2go = CPDirToGo;


		if(CPXPos < OPXPos(BallCarrier))
		{
			InitCPUBurst(25 + (rnum&15));
		}


	}
/*----------- Priority 4 -  if too far - try to cut down angle	-----*/
	else
	{  	 // first index of DirVsDirTable is dir to opponent, 2nd is dir opponent is going
		if(refiguredir)
			dir2go = DirVsDirTable[FindCPDirectionToPlayer(BallCarrier)][OPDirToGo(BallCarrier)];
		else	// stp the wiggles
			dir2go = CPDirToGo;

		if(CPXPos < OPXPos(BallCarrier) - (rnum&255))	// diff. every frame
		{
			InitCPUBurst(15 + (rnum&15));
		}


	}

	CPDirToGo = dir2go;

	if(dive)
	{
		StartDive();
	}
	else
	{
		TryManyMovesPush(dir2go);
	}
}



/***********************************************************************/


void OffChaseBall(void)
{
BOOL refiguredir = FALSE;
EFFVAR tx,ty;		// local "target"


	if(!CPStateInit)
	{
		CPStateInit = refiguredir = TRUE;
	}

	if( (AIFrameCounter & 3) == ZERO)
			refiguredir = TRUE;



	if(BallFlags & (BALL_PASSED_BFLAG | BALL_PUNTED_BFLAG
					| BALL_KICKEDOFF_BFLAG |BALL_KICKEDFG_BFLAG) )
	{ // ball in air - pass or any kick (COULD BE BOUNCING!!!)

		tx = BallXPosition;		// assume we run to ball
		ty = BallYPosition;

		if(BallFlags & (BALL_PASSED_BFLAG | BALL_KICKEDOFF_BFLAG))
		{	// run to ball

			#if 0	//PC_ALWAYS_MODE
			printf("\nOff.(%d) in ChaseBall on PlayNum %d, Rec=",
				gPNum,GameTeamData[OffensiveSlotNum].PlayNum,TheReceiver);
			#endif
		}
		else if(BallFlags & BALL_KICKEDFG_BFLAG)
		{	// this seems to happen on all FGs
			#if 0 //PC_ALWAYS_MODE
			printf("\nOff.(%d) in ChaseBall on FG",gPNum);
			#endif
			if(BallXPosition > CPXPos + 2*YARDS)
			{  // watch it
				tx = CPXPos;
				ty = CPYPos;
			}
		//	else go right to it
		}
		else // punt - only legitimate off. chase ball in air!?!
		{
			if(BallFlags & BALL_BOUNCING_BFLAG)
			{
				if(BallZPosition < 5
				  || BallDeltaX < 1
				  || BallXPosition > OFF_GOAL_LINE - 3*YARDS)
				{	// on ground or bouncing back - go to it
//					tx = BallXPosition;
//					ty = BallYPosition;
				}
				else	// let it bounce - don't bunch around it
				{
					tx = BallXPosition-2*YARDS;
					ty = BallYPosition-5*YARDS+(gPNum*HALFYARD);

				}
			}
			else if(BallTargetX | BallTargetY)	// is there a BTarget?
			{
				tx = BallTargetX-2*YARDS;	// give PR some room
				ty = BallTargetY;
			}
		}

		if (Distance(tx, ty) < YARD)
		{
			CPDirToGo = FindCPDirectionToBallMACRO;
			TurnPlayerOptimal(CPDirToGo);
			Shuffle(BLOCKING_SHUFFLE);	// BLOCKING_SHUFFLE is okay 10/23/96
		}
		else
		{
			CPTargetX = tx;
			CPTargetY = ty;
			if(refiguredir)		// else leave it what it was last time
				CPDirToGo = FindCPDirectionToTargetMACRO;

  /*			if( BallFlags & PASS_DEFLECTED_BFLAG
				|| BallZPosition < 3
			  )
			{
				if( TryManyMovesPush(CPDirToGo) == NO_COLLISION)  //
					Run();
			}
			else
			{*/
				TryManyMoves(CPDirToGo);	// note: NO PUSHING when Ball in Air
				Run();		// run even if collision
//			}
		}
	}
	else if (BallCarrier == NOBODY)		// if ball not in air or no BC
	{	// must be a fumble or something - Ball Actions should give someone ball
		if ( (LastBallCarrier == NOBODY) || (BallFlags & BALL_FREE_BFLAG) )	// must be punt, FG
			CPDirToGo = FindCPDirectionToBallMACRO;
		else
			CPDirToGo = FindCPDirectionToPlayer(LastBallCarrier);
		TryManyMovesPush(CPDirToGo);
	}
	else if (BallCarrier < 11)
	{
#if PC_ALWAYS_MODE
		printf("\nERROR - off.(%d) chasing Off BC(%d)",gPNum,BallCarrier);
#endif
	}
	else
	{
		ChaseDefensiveBallCarrier();

	}

}



/************************************************************************

			OffenseChaseBallCarrier

	following is subroutine of "ChaseBall - very similar to it but takes
	into account going the other way -
	already screened for ball in air or no BC

************************************************************************/
void ChaseDefensiveBallCarrier(void)
{
EFFVAR disttoBC,rnum;
EFFVAR temp,temp1;	// for debugging

	rnum = rand();	// use this throughout
	disttoBC =  DistanceToPlayer(BallCarrier);


/* If he is going slow OR if he is within 1.5 yards - go straight at him 	*/
	if ( (OPSpeed(BallCarrier) < (3<<SPEED_SHIFT)+1 )
			 || disttoBC < 3 * HALFYARD)
	{
		CPDirToGo = FindCPDirectionToPlayer(BallCarrier);
		if (CPXPos > OPXPos(BallCarrier) && disttoBC < 3 * YARD)
		{
			if(CPFacing == CPDirToGo)	// If he is within 1.5 yards and past me...
			{
				StartDive();
				return;
			}
		}
	}
	else if (disttoBC < 3 * YARD || OffenseKeyDistance[0].slotnum == gPNum)
	{ 	/* Priority 2 - run to a yard in front of BC if with 3 yards	*/
		CPTargetX =	OPXPos(BallCarrier) - YARD;	// assume Target Ball Carrier
		CPTargetY = OPYPos(BallCarrier);
		CPDirToGo = FindCPDirectionToTargetMACRO;
		if(CPXPos > OPXPos(BallCarrier) + 6 *HALFYARD)
		{
			InitCPUBurst(10 + (rnum&7));
		}
	}
/* Priority 3 - make sure I can get him before he gets a TD	*/
	else if (!CanOffGetBC())
	{
		CPTargetX = DEF_GOAL_LINE + HALFYARD;	// Try to get him at 1/2 yard line
		CPTargetY = OPYPos(BallCarrier);
		CPDirToGo = FindCPDirectionToTargetMACRO;

		if(CPXPos > OPXPos(BallCarrier))
		{
			InitCPUBurst(25 + (rnum&15));
		}


	}
/* Priority 4 -  if too far - try to cut down angle	*/
	else
	{  	 // first index of DirVsDirTable is dir to opponent, 2nd is dir opponent is going
		//	take "opposite" normal Defensive oriented DirVSDirTable
//		CPDirToGo = ((DirVsDirTable[(FindCPDirectionToPlayer(BallCarrier)+8)&15][OPDirToGo(BallCarrier)])+8)&15;
		temp = FindCPDirectionToPlayer(BallCarrier);
		temp1= OPDirToGo(BallCarrier);
		CPDirToGo =
			((16 - DirVsDirTable[((16-(temp))&15)][temp1])&15);


		if(CPXPos > OPXPos(BallCarrier) + (rnum&255))	// diff. every frame
		{
			InitCPUBurst(15 + (rnum&15));
		}


/****	testing code below

		if(gPNum ==2)	///test punt guy
		{
			temp = temp;	// bp here
			if(CPDirToGo < 4)
				temp = temp;	// bp here
		}
  end test code******/
	}

	TryManyMovesPush(CPDirToGo);
}




/*************************************************************************

	CAN DEF GET BC ??? - Can the Defense reach Ball Carrier before he Scores.

 *************************************************************************/
#define GL_DEFEND_YARD        (OFF_GOAL_LINE-HALF_YARD)
BOOL CanDefGetBC(void)
{
// defender must be in his end of field ( > his 20 Yard line)
	if (CPXPos > MID_FIELDX + 30 * YARD) // if close enough to matter
	{
/* key point is [near] where BC would cross EZ if BC was running due EAST,
	see who is closer to that point - me(TRUE) or BC(FALSE)			*/
			// "Distance" is from CP to poin X,Y
		if (Distance(GL_DEFEND_YARD, OPYPos(BallCarrier)) > GL_DEFEND_YARD - OPXPos(BallCarrier))
			return(FALSE); // BC is closer to goal than defender...
	}
	return(TRUE);
}


BOOL CanOffGetBC(void)
{
// defender must be in his end of field ( > his 20 Yard line)
	if (CPXPos < MID_FIELDX - 30 * YARD) // if close enough to matter
	{
/*	see who is closer to that point - me(TRUE) or BC(FALSE)			*/

		if (Distance(DEF_GOAL_LINE, OPYPos(BallCarrier)) > OPXPos(BallCarrier)-DEF_GOAL_LINE)
			return(FALSE); // BC is closer to goal than defender...

	}
	return(TRUE);
}

/*************************************************************************/

STATEDEF(ChaseBall)
{

	No_Back_Pedal_Macro;

	if(gPNum > 10)
	{
		DefChaseBall();
	}
	else
	{
		OffChaseBall();
	}
}


/*************************************************************************
						 * DEF SHUFFLE RUN -

 *************************************************************************/

/**  following used by defensive guys who are waiting for something to happen	*/
void DefShuffleWait(void)
{
	TurnPlayerOptimal(TOWARD_OFFENSE_DIR);
	CPSpeed = SHUFFLE_SPEED;
//	Shuffle(NORMAL_SHUFFLE);
  // not really blocking, but more "active" than normal shuffle (ok 10/26/96)
	Shuffle(BLOCKING_SHUFFLE);

}


/*************************************************************************
 *	SHUFFLE RANDOM - Move randomly but DONT change direction facing.
 *

 *************************************************************************/
#if 1
void DefLookReady(void)		// do backpedal without moving - used by LBs
{
//	InitGraphics(AI_SIDESTEP_NUM);
	InitGraphics(AI_BACKPEDAL_NUM);
	CYCLE_GRAPHICS_MACRO(CPAIGraphicState);
}

#else
void ShuffleRandom(void)
{
	BYTE dir;

//	CPSpeed = 0;	// Stop him...
	CPSpeed = SHUFFLE_SPEED;

	if ((dir = randper) < 16)	// One percent to go in any dir...
		TryManyMoves(dir);
	else
		Shuffle(NORMAL_SHUFFLE);
}

#endif


/*************************************************************************

						 DISTANCE   ROUTINES

	 Instead of a*a + b*b = c*c we use an approximation:

	 Calculated by using formula Distance = distance of longest leg
		+ 0.375 * distance of shortest leg.

		Optimal is: D = d(L) + d(S) * 0.336358 (Standard Dev 5.5%)

	 RETURNS: distance from Current Player to point X,Y

 *************************************************************************/
WORD Distance(WORD x, WORD y)
{
register WORD dx,dy;	// MUST be WORD not EFFVAR

	dx = CPXPos - x;
	if (dx < 0)			// Get absolute val... (abs() don't work right on PSX)
		dx = -dx;

	dy = CPYPos - y;
	if (dy < 0)
		dy = -dy;

	if (dx > dy)							// X is greater
	{
		dx += (dy >> 2) + (dy >> 3);	// D = dx + dy / 4  + dy / 8
//		dx += (dy*3)/8);	// D = dx + dy / 4  + dy / 8
		return(dx);
	}
	else										// Y is greater
	{
		dy += (dx >> 2) + (dx >> 3);	// D = dy + dx / 4  + dx / 8
//		dy += (dx*3)/8);	// D = dy + dx / 4  + dx / 8
		return(dy);
	}
}


/*---------------------------------------------------------------*/

WORD Distance2Points(WORD x, WORD y, WORD x2, WORD y2)
{
//register EFFVAR dx,dy;
register WORD dx,dy;	// MUST be WORD not EFFVAR

	dx = x2 - x;
	if (dx < 0)			// Get absolute val... (abs() don't work right on PSX)
		dx = -dx;

	dy = y2 - y;
	if (dy < 0)
		dy = -dy;

	if (dx > dy)							// X is greater
	{
		dx += (dy >> 2) + (dy >> 3);	// D = dx + dy / 4  + dy / 8
//		dx += (dy*3)/8);	// D = dx + dy / 4  + dy / 8
		return(dx);
	}
	else										// Y is greater
	{
		dy += (dx >> 2) + (dx >> 3);	// D = dy + dx / 4  + dx / 8
//		dy += (dx*3)/8);	// D = dy + dx / 4  + dx / 8
		return(dy);
	}
}

/*---------------------------------------------------------------*/

WORD Distance2Players(WORD p1, WORD p2)
{
register EFFVAR dx,dy;

	dx = OPXPos(p1) - OPXPos(p2);
	if (dx < 0)			// Get absolute val... (abs() don't work right on PSX)
		dx = -dx;

	dy = OPYPos(p1) - OPYPos(p2);
	if (dy < 0)
		dy = -dy;

	if (dx > dy)							// X is greater
	{
		dx += (dy >> 2) + (dy >> 3);	// D = dx + dy / 4  + dy / 8
		return(dx);
	}
	else										// Y is greater
	{
		dy += (dx >> 2) + (dx >> 3);	// D = dy + dx / 4  + dx / 8
		return(dy);
	}
}




/*************************************************************************
 * QB OUT OF POCKET - Checks to see if the Quarterback is out of the
 * 	Pocket.
 *************************************************************************/
BOOL QBOutOfPocket(WORD yards)
{

	return( (   (BallYPosition < StartBallY - yards)	// North of Pocket
			 && (CPYPos < StartBallY)
			)
		||
			(  (BallYPosition > StartBallY + yards)	// South of Pocket
			&& (CPYPos > StartBallY)	// I am on South Side of Field...
			)
		  );
}


/*************************************************************************

						 RUN TO TARGET STATE

			Run to XY then Go To Next State once there.

 *************************************************************************/

void RunToTargetState(void)
{
	if(!CPStateInit || ((AIFrameCounter & 3) == ZERO))
	{	// not every time to stop twitching
		CPDirToGo = FindCPDirectionToTargetMACRO;
	}


	if (TryManyMoves(CPDirToGo) != CANT_MOVE)
	{
		Run();
	}

	if (ReachedTarget())
	{
		CPTargetX = CPXPos;	// Make them exact
		CPTargetY =	CPYPos;

		CPAction = NoAction;	// Kill any action
		GoToNextState(1);
	}
}





WORD FindClosestOffPlayer(WORD defguy)	// used in pass catching(interception)
{
EFFVAR i,pnum = 0;
UEFFVAR dist,lowest;

	lowest = Distance2Players(defguy,0);

	for (i = 1; i < 11; i++)
	{
		dist = Distance2Players(defguy,i);
		if (dist < lowest)
		{
			pnum = i;
			lowest = dist;
		}
	}
	return(pnum);
}

/***------------------------------------------------------------------*/

void TryManyMovesOrInit2nd(void)
{
EFFVAR i;

	if((i = TryManyMoves(CPDirToGo)) == NO_COLLISION)
	{
		Run();
	}
	else if (i == CANT_MOVE)  /* couldn't move in one of 4 primary directions   */
	{
		Init2ndState(NORMAL_2ND_STATE_TICKS);

	}

}

/*************************************************************************


*************************************************************************/


STATEDEF(CoverManInMotionState)		// initiated by OFF. script
{
EFFVAR guytocover,guysdir;

	if(BallSnapped)
	{
		SetInitialScriptState();	// restart what he was intended to do

	}
	else
	{
		guytocover = CPGuyToCover;
		guysdir = OPDirToGo(guytocover);

		if( (guysdir == NORTH_DIR && CPYPos > OPYPos(guytocover))
		  ||(guysdir == SOUTH_DIR && CPYPos < OPYPos(guytocover))
//		if( (OPSpeed(guytocover) > (3<<SPEED_SHIFT))
//			|| (CPYPos < (OPYPos(guytocover) - 3*QTR_YARD))
//			|| (CPYPos > (OPYPos(guytocover) + 3*QTR_YARD))
		  )
		{
			CPDirToGo = guysdir;
			TryManyMovesOrInit2nd();
		}
		else
		{
			CPFacing = WEST_DIR;
			Shuffle(NORMAL_SHUFFLE);
			CPSpeed = 2<<SPEED_SHIFT;	// give a little inertia

		}
	}

}


//NORTH_DIR		0
//NNE_DIR		1
//NE_DIR		2
//ENE_DIR		3
//EAST_DIR		4
//ESE_DIR		5
//SE_DIR		6
//SSE_DIR		7
//SOUTH_DIR		8
//SSW_DIR		9
//SW_DIR		10
//WSW_DIR		11
//WEST_DIR		12
//WNW_DIR		13
//NW_DIR		14
//NNW_DIR		15

/*************************  end of file - FBDEFAI.C  *********************/


