/**************************************************************


							FBEQU.H

	AUTHOR: M. Knauer						Start Date: 3/8/95
	Modify Date:


	Copyright 1995,1996,1997  GameTek, Inc.


	This file contains all equates for Football AI.

	It is an INCLUDED file.

	Also see FB_GEQU.h (global equates) which has equates shared
	by the AI and the graphics engine.


	Some common abbreviations you'll see through out the code:

 -- programming stuff ---
	CPxxx           a "Current Player" data structure
	OPxxx(x)        an "Other Player" data structure
	BFLAG           bit flag

 -- football stuff ---
	BC                      Ball Carrier
	LOS                     Line Of Scrimmage
	OOB                     Out Of Bounds
	COP                     Change of Possession
	EOP                     End Of Play
	EZ                      End Zone
	KO                      Kick Off
	TD                      Touch Down
	TO                      Turn Over (maybe Time Out)

**************************************************************/


#ifndef _FBEQU_H

#define _FBEQU_H


/**  figure out who is compiling us	*************************/

#ifdef __MSDOS__
	#include  <stdlib.h>	/* for rand     */
	#include  <stdio.h>		/* for printf    */
	#include  <conio.h>     /* for getch    */

	#include  "fbmodes.h"   /* tells if test, graphic test, or PSX  */
	#ifndef TESTPLAYMODE    /* this is defined (or not) in FBMODES.h  */
		#define DEBUGMODE
	#endif

	#define	PC_ALWAYS_MODE	1
	#define	PC_GRAPHIC_MODE 1
	#define	PC_DEBUG_MODE   1
	#define	PC_ERROR_MODE	1
	#define	PC_SIM_DEBUG	1
#else
	#ifndef max
		#define min(aaa,bbb)  ((aaa) < (bbb) ? (aaa) : (bbb))
		#define max(aaa,bbb)  ((aaa) > (bbb) ? (aaa) : (bbb))
	#endif

	#ifdef __psx__
		#define PSXMODE
	#else
		#define SATURNMODE
	#endif

#endif




/*---------------------- CONSTANTS      -------------*/


#include        "prjtypes.h"

#define ZERO			0
#define DONT_CARE		ZERO
typedef unsigned int   UEFFVAR;	// don't care variable - make most EFFicient
typedef signed int   	EFFVAR;

typedef void (*voidFCNPTR)(void);	/* function pointer */



#define FRAME_RATE_20HERTZ

#ifndef FRAME_RATE_20HERTZ
	#define FRAME_RATE_30HERTZ
#endif


#include        "FBGEQU.H" /* global equate - shared by AI,Graphic,Shell */





#define 	NEWPLAYS1	1	// now 15 plays per off. formation


#define TEAM0	0		// slightly better documentation
#define TEAM1	1

#define	HOME_TEAM_SLOT	0
#define	AWAY_TEAM_SLOT	1

#define INVALID_DISTANCE	250



/************************************************************************

				 Football  Field  Constants

	Field is layed out with "handle" in upper left corner.
	This is point (0,0) - all X,Y positions will be positive.

	The field runs west to east where the Goalline is in the East.
	X goes left to right (west to east).
	Y goes top to bottom (north to south)

	There are 16 different directions staring at North (zero) going
	clockwise to east(4), south (8), west (12) and ending at NNW(15).


	Any field positioning is based a UWORD, where the high byte is
	the actual yard, ands the low byte is 1/256 of a yard (1/7 of
	an inch).

--------------------------------------------------------------------*/
#define	YARD_SHIFT		8	// YARD = 256

#define YARD			256 /* high byte is YARDS, low is 1/256 of yards */
#define HALFYARD		128
#define QTRYARD			64
#define EIGHTHYARD		32
#define SIXTEENTHYARD	16

#define YARDS			YARD
#define HALF_YARD		HALFYARD
#define QTR_YARD		QTRYARD
#define EIGHTH_YARD		EIGHTHYARD
#define SIXTEENTH_YARD	SIXTEENTHYARD
#define QTR_YARDS		QTRYARD
#define FEET			(YARD/3) // NOTE: 3*FEET != YARD (due to rounding)
#define FOOT			FEET
#define INCH			(YARD/36) // NOTE: 36*INCH != YARD(due to rounding)
#define INCHES			INCH

#define BORDER_WIDTH	(UWORD)(60*YARDS)
#define SIDELINE_WIDTH	120
#define FIELD_WIDTH		(UWORD)((45*YARDS) - (SIDELINE_WIDTH * 2))
#define DEF_END_LINE	(BORDER_WIDTH)
#define DEF_GOAL_LINE	(UWORD)(DEF_END_LINE+(10*YARDS))
#define OFF_GOAL_LINE	(UWORD)(DEF_GOAL_LINE+(100*YARDS))
#define OFF_END_LINE	(UWORD)(DEF_GOAL_LINE+(110*YARDS))
#define MID_FIELDX		(UWORD)(DEF_GOAL_LINE+(50*YARDS))
#define NORTH_SIDELINE	(BORDER_WIDTH + SIDELINE_WIDTH)
#define SOUTH_SIDELINE	(UWORD)(NORTH_SIDELINE+FIELD_WIDTH)
#define MID_FIELDY		(UWORD)(NORTH_SIDELINE+FIELD_WIDTH/2)

/*   parameters to keep ball in "stadium"       */
#define DEF_BACK_LINE	(UWORD)(DEF_END_LINE-10*YARDS)
#define OFF_BACK_LINE	(UWORD)(OFF_END_LINE+(10*YARDS))
#define NORTH_BACKLINE	(UWORD)(NORTH_SIDELINE-10*YARDS)
#define SOUTH_BACKLINE	(UWORD)(SOUTH_SIDELINE + (10*YARDS))


/*  below is pro hash marks	*/
//#define NORTH_HASH_MARK	(UWORD)(MID_FIELDY-(3*YARDS))
//#define SOUTH_HASH_MARK	(UWORD)(MID_FIELDY+(3*YARDS))
#ifdef SATURNMODE
	#define NORTH_HASH_MARK	(UWORD)(MID_FIELDY-(4*YARDS))	// matches field as of 3/28/96
	#define SOUTH_HASH_MARK	(UWORD)(MID_FIELDY+(4*YARDS))
#else
	#define NORTH_HASH_MARK	(UWORD)(MID_FIELDY-(7*HALFYARD)) // matches field as of 3/28/96-wrong(10/2/96)
	#define SOUTH_HASH_MARK	(UWORD)(MID_FIELDY+(7*HALFYARD))
#endif
/*  these are college hash marks
//#define NORTH_HASH_MARK	(UWORD)(MID_FIELDY-(15*HALFYARD)) //was 6Y-too wide
//#define SOUTH_HASH_MARK	(UWORD)(MID_FIELDY+(15*HALFYARD))

#define NORTH_HASH_MARK	(UWORD)(MID_FIELDY-(11*HALFYARD)) //7.5 didn't work with graphics
#define SOUTH_HASH_MARK	(UWORD)(MID_FIELDY+(11*HALFYARD))
*/
#define NORTH_SIDE_LINE NORTH_SIDELINE
#define SOUTH_SIDE_LINE SOUTH_SIDELINE



// GP width is 23ft 4in in college, 18ft 6inch in pro

#if 1	// below are actual values of PSX graphical GP - 10/8/96
//#define	GOAL_POST_WIDTH		(8*YARDS-8*INCH)	// better for rounding error
#define	GOAL_POST_WIDTH		(8*YARDS-16*INCH)	// better for rounding error

#define LOW_GOALPOST_Y 		(MID_FIELDY - ((GOAL_POST_WIDTH)/2))
#define HIGH_GOALPOST_Y 	(MID_FIELDY + ((GOAL_POST_WIDTH)/2))
//#define GOALPOST_Z			(10*FEET+6*INCHES)	// so it matches graphics
#define GOALPOST_Z			(11*FEET+10*INCHES)	// so it matches graphics
#define GP_BAR_WIDTH		(8*INCHES)	//(3*FEET)

#else // add 4 inches to bar width (2" each side) to account for ball width
	  no - Field Goal routine handles ball width
#define	GOAL_POST_WIDTH		(8*YARDS-16*INCH)	// better for rounding error

#define LOW_GOALPOST_Y 		(MID_FIELDY - ((GOAL_POST_WIDTH)/2) - 2*INCH)
#define HIGH_GOALPOST_Y 	(MID_FIELDY + ((GOAL_POST_WIDTH)/2) + 2*INCH)
#define GOALPOST_Z			(11*FEET+12*INCHES)	// so it matches graphics
#define GP_BAR_WIDTH		(12*INCHES)	//(3*FEET)
#endif


#define MAX_DEF_COVERX  (UWORD)(OFF_GOAL_LINE+9*YARDS)
#define MAX_COVER_NORTH (NORTH_SIDE_LINE)
#define MAX_COVER_SOUTH (SOUTH_SIDE_LINE)

#define CLOSE_TOSL	(3*HALFYARD) /* how close players get to SideLines   */
#define CLOSE_TOEL	(3*HALFYARD) /* how close players get to EndLines    */

//#define GET_BALL_NORMAL_HEIGHT  (6*FEET+3*INCHES)
#define GET_BALL_NORMAL_HEIGHT  (5*FEET+10*INCHES)

#define NEUTRAL_ZONE_LENGTH	 128

#define NORTH_DIR		0
#define NNE_DIR			1
#define NE_DIR			2
#define ENE_DIR			3
#define EAST_DIR		4
#define ESE_DIR			5
#define SE_DIR			6
#define SSE_DIR			7
#define SOUTH_DIR		8
#define SSW_DIR			9
#define SW_DIR			10
#define WSW_DIR			11
#define WEST_DIR		12
#define WNW_DIR			13
#define NW_DIR			14
#define NNW_DIR			15
#define MAX_DIR			16
#define MAX_DIRECTIONS  MAX_DIR /* number of directions inwhich to move */
#define MAX_DIR_MASK    (16-1)  /* for use of ANDing over/under flow of dir */

//#define       UP_FIELD_DIR    EAST_DIR
#define DOWN_FIELD_DIR  		EAST_DIR
#define TOWARD_OFFENSE_DIR      WEST_DIR


/****************   Timing  Stuff   **********************************/

#define SECOND			60
#define HALF_SECOND		(SECOND/2)
#define QUARTER_SECOND  (SECOND/4)
#define QUARTERSEC		QUARTER_SECOND
#define QTR_SECOND		QUARTER_SECOND
#define SECONDS			SECOND

#define SECS_PER_MIN	60

#ifdef FRAME_RATE_20HERTZ
	#define TICKS_PER_AI_FRAME 3	/* we run at 60 / 20 = 3	*/
	#define AI_FRAME_RATE 	   20	/* we run at 60 / 20 = 3	*/

#endif

#ifdef FRAME_RATE_30HERTZ
	#define TICKS_PER_AI_FRAME 4	/* 30 htz move is actually 15 HTZ / 2 	*/
	#define AI_FRAME_RATE 	   15	/* we run at 60 / 20 = 3	*/
#endif



#define	AI_TEXTINFO_CHARS	60	// size of text char arrays


#define MAX_USERS_PER_TEAM	6	/* human players on a team	*/

/*  following values used as return values for collision detect routines
	if number is positive then that is the number of the guy collided
	with, else there will be one of the following negative numbers.

	NO_COLLISION means that the players X,Y have been changed and it's
	up to the calling routine to do the grahics.
	CANT_MOVE means that the player couldn't move in one of 4 directions
	 (if "TryManyMoves()", so the player most likely should be put
	 into a "secondary STATE".
	Anything else (e.g. "TACKLE_STARTED" means the graphics and
	movement have been taken care of so do nothing.
	NO_COLLISION means that the players X,Y have been changed and it's
	up to the calling routine to do the grahics.
	CANT_MOVE means that the player couldn't move in one of 4 directions
	 (if "TryManyMoves()", so the player most likely should be put
	 into a "secondary STATE".
	Anything else (e.g. "TACKLE_STARTED" means the graphics and
	movement have been taken care of so do nothing.
*/
#define NO_COLLISION    -1		/* return value for collision detect    */
#define TACKLE_STARTED  -2		/* return value for collision detect    */
//#define  other cases where "movement actions" or new graphics have started here
#define CONTACTMADE		-3		/* used by TMM-PUSH						*/
#define OUT_OF_BOUNDS   -4		/* very low level - in TryMove()		*/

#define CANT_MOVE		-5		/* return value for collision detect    */
#define HE_MOVED        NO_COLLISION	/* return value for collision detect */

#define OUT_OF_BOUNDS_EAST   -6		/* new - 11/16/96	*/

/*  following used in "action = CheckForTackleAndFumble(guyhit)"	*/
#define NO_HIT_BC_SO_NO_TACKLE	0
#define HIT_BC_BUT_CANT_TACKLE	1		// tell calling routine I'm tackling
#define HIT_BC_AND_CAN_TACKLE	2		// tell calling routine I'm tackling



/*********** PLAYER FLAGS (bits in PlayerFlag)          *************/


#define IM_LYING_ON_GROUND_BFLAG 0x0001  /* bit in PlayerFlags		*/
#define LEAD_BLOCKER_BFLAG		0x0002  /* bit in PlayerFlags		*/
#define IM_PUNT_RETURNER_BFLAG	0x0004  /* bit in PlayerFlags		*/
#define BACK_PEDAL_BFLAG		0x0008  /* bit in PlayerFlags		*/

#define IS_HCONTROLLED_BFLAG    0x0010  /* bit in PlayerFlags		*/
#define CAN_HANDOFF_BFLAG		0x0020  /* bit in PlayerFlags		*/
#define CAN_PASS_BFLAG			0x0040  /* bit in PlayerFlags		*/
#define	I_RUN_BACKWARD_BFLAG	0x0080  /* bit in PlayerFlags		*/

#define FORCE_WALK_BFLAG		0x0100  /* bit in PlayerFlags		*/
#define IM_BEING_TACKLED_BFLAG	0x0200  /* bit in PlayerFlags		*/
#define IM_BLOCKED_BFLAG		0x0400  /* bit in PlayerFlags		*/
#define HAVE_BEEN_BLOCKED_BFLAG	0x0800  /* bit in PlayerFlags		*/

#define I_CAN_PUSH_BFLAG		0x1000  /* bit in PlayerFlags		*/
#define	IM_DIVING_BFLAG			0x2000  /* bit in PlayerFlags		*/
#define	IM_SPINNING_BFLAG		0x4000  /* bit in PlayerFlags		*/
#define PASS_BLOCKER_BFLAG		0x8000  /* bit in PlayerFlags
								-set if run blocker else pass blocker */
#define	REF_ACTIVE_BFLAG		0x4000  /* bit in PlayerFlags-double duty for ref */
#define	REF_RUNNING_OUT_BFLAG	0x8000  /* bit in PlayerFlags-double duty for ref */




#define	I_HAVE_BEEN_HCONTROLLED_BFLAG	0x0001  /* bit in PlayerFlags1 */
#define	MUST_LIFT_OFF_CONTROLLER_BFLAG	0x0002  /* bit in PlayerFlags1 */
#define	QB_CAN_BPEDAL_BFLAG			0x0004  /* bit in PlayerFlags1 */
#define	IM_HOLDER_BFLAG				0x0008  /* bit in PlayerFlags1 */

#define	OKAY_FOR_RCATCH_ANIM_BFLAG	0x0010  /* bit in PlayerFlags1 */
#define	BURST_ACTIVE_BFLAG			0x0020  /* bit in PlayerFlags1 */
#define	IM_A_BLOCKER_BFLAG			0x0040  /* bit in PlayerFlags1 */
#define	IM_REVERSE_GUY_BFLAG		0x0080  /* bit in PlayerFlags1 - for def. audio	*/

#define I_SCORED_CONV_BFLAG			0x0100  /* bit in PlayerFlags1 	*/
#define	I_SCORED_A_TD_BFLAG			0x0200  /* bit in PlayerFlags1	*/
#define	IM_BLOCKING_HUMAN_BFLAG		0x0400  /* used by Off. Line -bit in PlayerFlags1 */
#define	I_JUST_CAUGHT_PASS_BFLAG	0x0800  /* used to trigger graphics(PlayerFlags1) */



#define IM_BLOCKED_MACRO		(CPFlags & IM_BLOCKED_BFLAG)
#define HAVE_BEEN_BLOCKED_MACRO	(CPFlags & HAVE_BEEN_BLOCKED_BFLAG)
#define I_CAN_PUSH_MACRO		(CPFlags & I_CAN_PUSH_BFLAG)
#define I_SCORED_TD_MACRO		(CPFlags & I_SCORED_TD_BFLAG)

#define OP_BLOCKED_MACRO(xxx)		(OPFlags(xxx) & IM_BLOCKED_BFLAG)
#define OP_BEEN_BLOCKED_MACRO(xxx)	(OPFlags(xxx) & HAVE_BEEN_BLOCKED_BFLAG)
#define OP_CAN_PUSH_MACRO(xxx)		(OPFlags(xxx) & I_CAN_PUSH_BFLAG)



#if 1	/* see if EFFVAR structure is more efficient on 32bit RISC complier
		   result - very slight  - Eventflags saved about 100 bytes	*/
struct evflags {
EFFVAR TACKLE_STARTED_BFLAG	;//0x0001  /* bit in EventFlags		*/
EFFVAR TACKLE_ENDED_BFLAG		;//0x0002  /* bit in EventFlags		*/
EFFVAR BAD_BOUNCE_BFLAG		;//0x0004  /* bit in EventFlags		*/
EFFVAR FUMBLE_HAPPENED_BFLAG	;//0x0008	/* bit in EventFlags		*/

EFFVAR DEF_REACT_TO_PASS_BFLAG ;//0x0010  /* in EventFlags - set at about 1/3 way into pass */
EFFVAR KICK_GOOD_BFLAG			;//0x0020  /* set when FG or ExPt was good */
EFFVAR BC_OUT_OF_EZ_BFLAG		;//0x0040  /* set if BC went out of EZ*/
EFFVAR DEF_BC_OUT_OF_EZ_BFLAG	;//0x0080  /* set if Def BC went out of his EZ*/

EFFVAR SAFETY_HAPPENED_BFLAG	;//0x0100  /* set if Def BC went out of his EZ*/
EFFVAR BC_PASSED_LOS_BFLAG		;//0x0200	/* set if BC passed LOS			*/
EFFVAR TD_HAPPENED_BFLAG		;//0x0400	/* bit in EventFlags			*/
EFFVAR FIRST_HIT_BFLAG			;//0x0800  /* in EventFlags - for audio	*/

EFFVAR IRP_INITED_BFLAG		;//0x1000  /* set when Instant Replay Started */
//EFFVAR AUDIBLE_ACTIVE_BFLAG 	;//0x2000  /* bit in EventFlags		*/
EFFVAR OK_FOR_HUMAN_OFF_CONTROL_BFLAG  ;//0x4000  /* bit in EventFlags	*/
EFFVAR OK_FOR_HUMAN_DEF_CONTROL_BFLAG;  //0x8000  /* bit in EventFlags	*/
};
#else
/*****************    Bit Flags in "EventFlags"  **************/

#define TACKLE_STARTED_BFLAG	0x0001  /* bit in EventFlags		*/
#define TACKLE_ENDED_BFLAG		0x0002  /* bit in EventFlags		*/
#define BAD_BOUNCE_BFLAG		0x0004  /* bit in EventFlags		*/
#define FUMBLE_HAPPENED_BFLAG	0x0008	/* bit in EventFlags		*/

//#define PASS_PLAY_BFLAG 		0x0010  /* bit in EventFlags-set when ball passed */
#define DEF_REACT_TO_PASS_BFLAG 0x0010  /* in EventFlags - set at about 1/3 way into pass */
#define KICK_GOOD_BFLAG			0x0020  /* set when FG or ExPt was good */
#define BC_OUT_OF_EZ_BFLAG		0x0040  /* set if BC went out of EZ*/
#define DEF_BC_OUT_OF_EZ_BFLAG	0x0080  /* set if Def BC went out of his EZ*/

#define SAFETY_HAPPENED_BFLAG	0x0100  /* set if Def BC went out of his EZ*/
#define BC_PASSED_LOS_BFLAG		0x0200	/* set if BC passed LOS			*/
#define TD_HAPPENED_BFLAG		0x0400	/* bit in EventFlags			*/
#define FIRST_HIT_BFLAG			0x0800  /* in EventFlags - for audio	*/

#define IRP_INITED_BFLAG		0x1000  /* set when Instant Replay Started */
#define AUDIBLE_ACTIVE_BFLAG 	0x2000  /* bit in EventFlags		*/
#define OK_FOR_HUMAN_OFF_CONTROL_BFLAG  0x4000  /* bit in EventFlags	*/
#define OK_FOR_HUMAN_DEF_CONTROL_BFLAG  0x8000  /* bit in EventFlags	*/
#endif
/*****************    Bit Flags in "MoreEventFlags"  **************/

#define FALSE_START_HAPPENED_BFLAG	0x0001  /* bit in MoreEventFlags	*/
#define FALSE_START_PENDING_BFLAG	0x0002  /* bit in MoreEventFlags	*/
//#define DEF_ONSIDE_RECOVER_BFLAG	0x0002  /* bit in MoreEventFlags	*/
#define KO_WENT_10YARDS_BFLAG		0x0004  /* bit in MoreEventFlags	*/
#define HUMAN_KICK_RANGE_DONE_BFLAG	0x0008  /* bit in MoreEventFlags	*/

#define HIT_BC_THIS_FRAME			0x0010  /* bit in MoreEventFlags	*/
#define	FORCE_NEW_1STDOWN_BFLAG		0x0020  /* bit in MoreEventFlags	*/
#define	NEED_FORCED_TIMEOUT_BFLAG	0x0040  /* bit in MoreEventFlags	*/
#define ILLEGAL_KICK_BFLAG			0x0080  /* bit in MoreEventFlags	*/

#define NEED_NO_HUDDLE_BFLAG		0x0100  /* bit in MoreEventFlags	*/
#define	OFF_AUDIBLE_ACTIVE_BFLAG  	0x0200  /* bit in MoreEventFlags	*/
#define	DEF_AUDIBLE_ACTIVE_BFLAG  	0x0400  /* bit in MoreEventFlags	*/
#define	OKAY_FOR_STATS_BFLAG  		0x0800  /* bit in MoreEventFlags	*/


/*-------------------- Bit Flags in "PlayOverFlags" -------------------*/
#define NEED_COP_BFLAG			0x0001  /* Need Change of Possession in PlayOverFlags   */
#define KICK_OVER_BFLAG			0x0002  /* set when FG or ExPt over	*/
#define INCOMPLETE_PASS_BFLAG	0x0004  /* bit in PlayOverFlags		*/
#define BALL_DEAD_BFLAG			0x0008  /* bit in PlayOverFlags		*/

#define BALL_OUT_OF_BOUNDS_BFLAG 0x0010  /* bit in PlayOverFlags	*/
#define OFF_DOWNED_PUNT_BFLAG	0x0020  /* bit in PlayOverFlags		*/
#define	OKAY_TO_GO_OOB_BFLAG	0x0040  /* gloabl for all players (in PlayOverFlags)*/
#define	FG_PAST_GP_BFLAG		0x0080  /* gloabl for all players (in PlayOverFlags)*/

/*------ Bit Flags in "GameFlags" (GameFlags ONLY clear on game init)-----*/

#define GAME_TIMER_ON_BFLAG		0x0001  /* bit in GameFlags     */
#define PLAY_TIMER_ON_BFLAG		0x0002  /* bit in GameFlags     */
#define	PAUSE_ACTIVATED_BFLAG	0x0004	/* bit in GameFlags     */
#define	FREEZE_ACTIVATED_BFLAG	0x0008	/* bit in GameFlags     */

#define	QUARTER_OVER_BFLAG		0x0010	/* bit in GameFlags     */
#define	GAME_OVER_BFLAG			0x0020	/* bit in GameFlags     */
#define	AI_STALLED_BFLAG 		0x0040	/* bit in GameFlags     */
#define BIG_HOME_TD_BFLAG		0x0080	/* bit in GameFlags		*/


#define EX_PT_PENDING_BFLAG		0x0100	/* bit in GameFlags     */
#define NEED_2MIN_WARNING_BFLAG	0x0200	/* bit in GameFlags     */
#define GAVE_2MIN_WARNING_BFLAG	0x0400	/* bit in GameFlags     */
#define LEAVE_GAME_BFLAG		0x0800	/* bit in GameFlags-after game over */

#define PLAY_CALL_INITED		0x1000	/* bit in GameFlags- set
						by PlayCall Screen - cleared by AI */
#define FREE_KICK_BFLAG			0x2000	/* bit in GameFlags- set Safety KO so no onsides*/
#define DEF_PENALTY_PENDING_BFLAG 0x4000 /* bit in GameFlags- set if Pass Int. */
//#define NEED_INIT_OF_PLAYCALL_BFLAG 0x8000 /* bit in GameFlags */
#define INSTANT_REPLAY_ACTIVE	0x8000 /* bit in GameFlags */


/**------------------  BITS OF PLAY FLAGS  -----------------------------*/
		  // PlayFlags cleared every InitAI
#define POS_PUNT_PLAY_BFLAG		0x0001  /* bit in PlayFlags     */
#define POS_FG_PLAY_BFLAG		0x0002  /* bit in PlayFlags     */
#define POS_KO_PLAY_BFLAG		0x0004  /* bit in PlayFlags     */
#define POS_ANYKICK_PLAY_BFLAG	0x0008  /* set if any of the other 3 kick bits are set  */


#define POS_HUMAN_KICK_BFLAG	0x0010  /* in PlayFlags, set if human will [possibbly] kick */
#define PASS_PLAY_BFLAG			0x0020  /* bit in PlayFlags-set when ball passed */
#define	FAKE_KICK_PLAY_BFLAG	0x0040	/*in PlaysFlags, used to turn off Kick interface */

//#define DEF_PASS_INTERFERENCE_BFLAG	0x0080	/*in PlaysFlags */
#define OFF_PASS_INTERFERENCE_BFLAG	0x0100	/*in PlaysFlags */
#define POS_PUNT_NOPOOCH_PLAY_BFLAG	0x0200  /* bit in PlayFlags     */
#define POS_HANDOFF_BFLAG			0x0400  /* bit in PlayFlags     */
#define KICK_BLOCKED_BFLAG			0x0800  /* bit in PlayFlags     */

#define OK_TO_DISPLAY_AITEXT_BFLAG	0x1000  /* bit in PlayFlags     */
#define KICK_COMING_DOWN_BFLAG		0x2000  /* any kick (in PlayFlags)  */
#define OK_TO_SCREEN_BLOCK_BFLAG	0x4000  /* bit in PlayFlags     */
#define SACK_ON_PLAY_BFLAG			0x8000  /* bit in PlayFlags     */


/*************-------- INTERFACE  FLAGS  - AI & Graphic Engine  *******/
#define	AMINS_LOADED_BFLAG			0x0001	/* in InterfaceFlags */
#define	FIELD_NOT_VIEWABLE_BFLAG	0x0002	/* in InterfaceFlags */
#define	DISPLAY_BALL_ARC_BFLAG   	0x0004	/* in InterfaceFlags */
#define KICK_INTERFACE_ACTIVE_BFLAG 0x0008	/* in InterfaceFlags */

#define DID_SCOREBOARD_AI_STUFF		0x0010	/* in InterfaceFlags */



#define DISPLAY_EOP_INFO	0x0001	/* bit in DisplayLevel	*/
#define DISPLAY_EOG_STATS	0x0002	/* bit in DisplayLevel	*/

/***  Zone Coverage flags       */
#define CZN -1  /* cover zone north     */
#define CZS -2	// unsigned -2
#define CZD -3	// unsigned -3



#define MAX_CONTROLLERS		8       /* maximum number of human players	*/
#define NO_CONTROLLER		-1


/*---------------------------------------------------------------------

			S L O T (INDEX)  D A T A   E Q U A T E S


	There are 22 data structures(SLOTs) for players on the field,

	11 "slots" per team  - these represent the 11 players
	on the field at that time. This corresponds to the array index of
	the player  - offense is 0-10, defense is 11-21

	Referee is player # 22
	A "hidden" player (DUMMY_SLOT_NUM) exists to assist in QB passing.

	Some positions (i.e. offensive line) always occupy the same slot numbers.
	Other slots have different positions depending on what
	formation is being used.

	These SLOTs are references in the AI loop by the global
	variable "gPNum" - global Player Number.

----------------------------------------------------------------------*/

#include	"fbpdata.h"		// the player positions(equates QB = 0,etc)


#define NUM_PLAYERS_ON_FIELD_PER_TEAM   11

#define NOBODY  				-1

#define START_OFF_SLOT_NUM      0
#define LAST_OFF_SLOT_NUM       10
#define START_DEF_SLOT_NUM      11
#define START_DEF_INDEX 		START_DEF_SLOT_NUM
#define LAST_DEF_SLOT_NUM       21

#define HOLDER_SLOT_NUM			ZERO
#define QB_SLOT_NUM		0	// Quarterback(punter, kickret) is always in this slot
								//
#define RT_SLOT_NUM		6	// right tackle is always in this slot
#define LT_SLOT_NUM		7	// Left tackle is always in this slot
#define RG_SLOT_NUM		8	// right guard is always in this slot
#define LG_SLOT_NUM		9	// Left guard is always in this slot
#define CENTER_SLOT_NUM 10	// the Center is always in this slot
#define DEF_KICKER_SLOT_NUM 11 // on kickoff kicker is in this slot


#define	REF_SLOT_NUM			22
#define	DUMMY_SLOT_NUM			23

#define	NUM_PLAYER_SLOTS		(22+2)		// 0 to 23
// last slot is "dummy" slot used in "WhereWillPlayerBe()"


#define	MAX_RECEIVERS			5	// possible(elligible) receivers take slots 0-5
#define	MAX_RECEIVER_SLOTS		6	// possible(elligible) receivers take slots 0-5


/*  "hide" REF only variables in unused rating stuff	*/
#define	RefSFXPending 		PlayerHands[REF_SLOT_NUM]
#define	RefAIAnimPending	PlayerArmStrength[REF_SLOT_NUM]


/*-----------------------------------------------*/

#define MAX_PLAYS_PER_FORMATION         12
#define ON_OFFENSE                      ZERO
#define ON_DEFENSE                      1



/*******************   Player Speeds    *****************************/

#define MAX_NUM_SPEEDS			26      /* 0 -19 (actually 21 in table for DIVE) */
#define	HIGHEST_MAX_SPEED    	24
#define	HIGHEST_NORMAL_SPEED    19
#define	LOWEST_NORMAL_SPEED    	6		// everybody at least this fast
#define SPEED_SHIFT				8		/* speeds are times 256 */
#define SHUFFLE_SPEED   (3 <<SPEED_SHIFT)

// NOTE: changing the Speed table values greatly effects these speeds

#define REF_RUN_SPEED			(MAX_NUM_SPEEDS-2)
//#define REF_RUN_SPEED			14
#define RETURN_TO_HUDDLE_SPEED 	(MAX_NUM_SPEEDS-1)
#define TO_LOS_KO_SPEED			(MAX_NUM_SPEEDS-2)	//kickoff - off. only
#define TO_LOS_FAST_SPEED		(MAX_NUM_SPEEDS/2)
#define TO_LOS_SLOW_SPEED		7
#define DIVE_SPEED				(MAX_NUM_SPEEDS-1) //special provision in SpeedTable for this "extra Speed"


/* as of 8/23/96 all players(CPU & human) have instant acceleration
	unless they all blocked or collide while running patterns
	Once players retrun to max speed, then their accel is set to max_accel
*/
#define	BASIC_ACCELERATION	84		// 1/3 of 256 - slow accel to max speed
#define	MAX_ACCELERATION	0x1000	// instant accel to max speed (normal)
#define	HUMAN_ACCEL_DEC     0x0111	// passed parameter to DecrementPlayerSpeed
									// .. to override normal stuff




#define NO_FUMBLE_SFX	0
#define NEED_FUMBLE_SFX	1




/******************    AUDIO  VOLUME   STUFF -----------------------*/
#define LARGE_DISTANCE	(120*YARDS)

#define MAX_VOLUME			127		// this is HardWare max

//#define	CVOL_SHIFT			4	// resolution of 1/16 - too small
//  CVOL_SHIFT=8 overflowed WORD is short ramp time
#define	CVOL_SHIFT				7	// resolution of 1/256
#define AI_MAX_CROWD_VOLUME		(MAX_VOLUME<<CVOL_SHIFT)

/**  only 3 crowd sounds use SCALED_VOL_PERCENT
		 to give better ramping resolution				*/
//#define	SCALED_VOL_PERCENT(x) 	((AI_MAX_CROWD_VOLUME*x)/100)
#define	SCALED_VOL_PERCENT(x) 	((AI_MAX_CROWD_VOLUME/100)*x)
	//divide 1st is better if MAX _VOl is high
#define	TRUE_VOL_PERCENT(x) 	((MAX_VOLUME*x)/100)

#if 0	//below used if no ramping

#define	CML1_VOL	SCALED_VOL_PERCENT(20)	// in huddle
#define	CML2_VOL	SCALED_VOL_PERCENT(40)	// at line
#define	CML3_VOL	SCALED_VOL_PERCENT(60)	// during play
#define	CML4_VOL	SCALED_VOL_PERCENT(80)	// big  play

#else	// use these with MINOR MID ramping

#define	CML1_VOL	SCALED_VOL_PERCENT(18)	// in huddle
#define	CML2_VOL	SCALED_VOL_PERCENT(38)	// at line
#define	CML3_VOL	SCALED_VOL_PERCENT(58)	// during play
#define	CML4_VOL	SCALED_VOL_PERCENT(78)	// big  play
/*
#define	CHL1_VOL	SCALED_VOL_PERCENT(30)	// HIGH (right after snap)
#define	CHL2_VOL	SCALED_VOL_PERCENT(70)	// HIGH (ramp to this slowly as ball passed
#define	CHL3_VOL	SCALED_VOL_PERCENT(70)	// HIGH (ramp to this slowly as ball passed
#define	CHL4_VOL	SCALED_VOL_PERCENT(85)	// home TD
#define	CHL5_VOL	SCALED_VOL_PERCENT(98)	// big home TD
*/
// higher 10/4/96 - bug #2972
#define	CHL1_VOL	SCALED_VOL_PERCENT(50)	// HIGH (right after snap)
//#define	CHL2_VOL	SCALED_VOL_PERCENT(90)	// HIGH (ramp to this slowly as ball passed
#define	CHL2_VOL	SCALED_VOL_PERCENT(65)	// 11/21/96 - save something for reserve
#define	CHL3_VOL	SCALED_VOL_PERCENT(90)	// HIGH
#define	CHL4_VOL	SCALED_VOL_PERCENT(95)	// home TD
#define	CHL5_VOL	SCALED_VOL_PERCENT(98)	// big home TD

//#define	MINOR_CML_RANGE	SCALED_VOL_PERCENT(5)	// minor Mid crowd ramp range (%) -way too low 8/26/96
#define	MINOR_CML_RANGE	SCALED_VOL_PERCENT(25)	// minor Mid crowd ramp range (%)

#endif

// higher 10/4/96 - bug #2972
#define CROWD_REACT_VOL_LOW	 TRUE_VOL_PERCENT(60)	// range = 0-127
#define CROWD_REACT_VOL_MED	 TRUE_VOL_PERCENT(85)	// range = 0-127
#define CROWD_REACT_VOL_HIGH TRUE_VOL_PERCENT(95)	// range = 0-127

//#define CROWD_REACT_VOL_LOW	 TRUE_VOL_PERCENT(40)	// range = 0-127
//#define CROWD_REACT_VOL_MED	 TRUE_VOL_PERCENT(60)	// range = 0-127
//#define CROWD_REACT_VOL_HIGH TRUE_VOL_PERCENT(80)	// range = 0-127




/****************    Structures   *********************************/


struct  u8XY     {	// unsigned little Box (unused ??)
	UBYTE    X;
	UBYTE    Y;
};

struct  s8XY     {	// signed little Box !!! used in Speed table (11/6/96)
	BYTE    X;
	BYTE    Y;
};


struct  sXY     {	// signed Box !!! used in zones
	WORD    X;
	WORD    Y;
};

struct  uXY      {
	UWORD    X;
	UWORD    Y;
};

struct  Point3D {
	WORD    X;
	WORD    Y;
	WORD    Z;
};


struct  uBox     {
	struct  uXY      Low;
	struct  uXY      High;
};


struct  sBox     {
	struct  sXY  Low;
	struct  sXY  High;

};

/*****************************************************************

		FOOTBALL   A.I.   NAMING CONVENTIONS

	A "PLAY" is made up of 11 "SCRIPT"s (each player has a SCRIPT)
	A "SCRIPT" is made up of various "STATE"s e.g. RunTo, Wait,
		LookForReceiver,

	The levels of Player A.I. are:
	1) Lowest level - do this first- "ACTION"s -
		a move that once started must be completed
		e.g. falling down, getting up, turning

	2) 2nd level - to be done if no ACTIONs pending  - "STATE"

	3) 3rd level - usually done at end of STATE - "SMARTS"
		this will allow a player to change SCRIPTS


	NOTE: Human controlled players don't do level 2 or 3.


*****************************************************************/

struct  State   {               /* a pointer to a function      */
	void (*todo)(void);
};

struct  BlockerState    {		/* a pointer to a function      */
	void (*todo)(void);
	WORD    parameter;
};


struct  script {
//      WORD    PositionNum;     /* now part of formation        */
	WORD    ScriptNumToDo;
};

struct  play    {
	WORD	PlayTypeDesignator;		/* inside/outside run, short/med/long pass etc */
	WORD    PlayInitRoutineNumber;  /* routine to do during play init	*/
	WORD    FormationNumber;		/* positional formation of play		*/
	struct  script  Script[11];		/* what the 11 guys will do			*/
};


/* following "Formation" Data structures tells what players are included
 in the formation and where they start relative to the ball.    */

struct  formationstart  {
	UWORD   WhoAmI;		/* position they play (0 to #men on team)       */
	WORD    XStart;		/* START is relative to Starting Ball Pos       */
	WORD    YStart;
};

struct  formation       {
	struct  formationstart  FormationStart[11];
};


struct  slotdist {
	WORD slotnum;
	UWORD distance;		// distances are UNSIGNED 1/3/96 mjk !!!
};


struct BKzonearea {	/* the new "Bobby King" (not burger king) zones */
WORD    PlayerPos;            /* FS, MLB, ...    -1 if end    */
struct  sBox     outsidezone;
struct  sBox     insidezone;
};

#define ZBM     0x8000          /* Zone Behind Me Flag in zonedata      */
#define CLZ     0x4000          /* Cover Left Zone Flag in zonedata     */
#define CRZ     0x2000          /* Cover Right Zone Flag in zonedata    */
#define ZONE_FLAG_MASK  0xF000  /* to mask out those high bits  */



struct defshiftDS {
	BYTE slotnum;
	WORD targetx;   // targets relative to StartBallX/Y
	WORD targety;   //  YARDS multiplied in during init process
};




//#define SETTOWHOLEYARD(xxx)	((xxx) <= MID_FIELDX) ? (((xxx) + YARD-1)&0xff00) : ((xxx) & 0xff00)
//UWORD SetToWholeYard(UWORD Xyardline);	// use MACRO or function - your choice
UWORD RoundToNearestWholeYard(UWORD Xyardline);
#define SETTOWHOLEYARD(xxx) RoundToNearestWholeYard(xxx)

#define	TURNTORIGHT		1
#define	TURNTOLEFT	   -1




/*********** BALL FLAGS (bits in BallFlags)             *************/
// NOTE: BallFlag = ZERO when player has ball
/*  Ball Flags reordered 10/14/96 to pack more into IRP - lower 4 bits
in IRP (not needed in IRP) will be ball direction 			*/

#define BALL_PASSED_BFLAG      	0x0001  /* BIT in BallFlags     */
#define PASS_DEFLECTED_BFLAG	0x0002    /* BIT in BallFlags        */
#define PASS_COMING_DOWN_BFLAG	0x0004    /* BIT in BallFlags        */
#define NOBODY_CATCH_PASS_BFLAG 0x0008	/* in BallFlags - set sometimes in pass deflection */

#define BALL_PUNTED_BFLAG       0x0010    /* BIT in BallFlags   */
#define PUNT_PAST_LOS_BFLAG     0x0020    /* BIT in BallFlags   */
#define BALL_KICKEDOFF_BFLAG	0x0040    /* BIT in BallFlags   */
#define BALL_KICKEDFG_BFLAG     0x0080    /* field goal BIT in BallFlags */

#define BALL_FREE_BFLAG       	0x0100 	/* BIT in BallFlags bouncing during fum. KO.punt     */
#define BALL_BOUNCING_BFLAG     0x0200  /* BIT in BallFlags     */
#define BALL_IS_FUMBLED_BFLAG   0x0400  /* BIT in BallFlags     */
//#define BALL_PITCHED_BFLAG      0x0800  /* BIT in BallFlags     */

#define END_OVER_END_BFLAG		0x1000  /* BIT in BallFlags     */
#define STAND_BALL_BFLAG        0x2000  /* BIT in BallFlags-on Pre-kickoff     */
#define SHOW_BALL_BFLAG         0x4000  /* BIT in BallFlags     */
#define SPIRAL_BALL_BFLAG       0x8000  /* BIT in BallFlags     */


/*********** BLOCKING FLAGS (bits in CPBeingBlocked)    *************/

//#define       IMBLOCKED      0       // bitnum in PBeingBlocked
//#define       HAVEBEENBLOCKED 1       // bitnum in PBeingBlocked
//#define       ICANPUSH        2       // bitnum in PBeingBlocked
#define	MAX_BLOCK_FACTORS	8		// this many "levels" of blocking


// Directions to push  (in CPDirToPush
#define PUSH_STRAIGHT		0
#define PUSH_LEFT			15   // 15 = -1 (mod 16)
#define PUSH_RIGHT			1
#define NO_DIR  			0x0080  /* No (or undetermined)  Direction      */
#define SLANT_BLOCK_BFLAG	0x0100	// bit which ends up in CPDirToPush
#define TRAP_BLOCK_BFLAG	0x0200



/*   parameters for initing fumbles     */
#define BALL_HIT_GROUND_FUMBLE	1
#define BALL_HIT_GUY_FUMBLE		2



/* 	team_types mainly used for CPU play calling.
	note: specific teams are never "super" teams - they only become "super"
	depending on the game situation		*/
enum teamtypes {
	SUPER_RUNNING_TEAM,
	RUNNING_TEAM,
	BALANCED_TEAM,
	PASSING_TEAM,
	SUPER_PASSING_TEAM,
	MAX_TEAM_TYPES
};




enum fieldtypes {
	GRASS_FIELD,
	TWO_TONE_GRASS_FIELD,	// mowed/painted differently
	TURF_FIELD,
	DOME_TURF_FIELD
};

enum fieldconditions {
	NORMAL_FIELD_CONDITION,
	SNOWY_FIELD_CONDITION,
	RAINY_FIELD_CONDITION,
	MUDDY_FIELD_CONDITION,
	WINDY_FIELD_CONDITION
};
#define MAX_NUM_WIND_SPEEDS	2

enum weatherconditions {
	NORMAL_WEATHER_CONDITION,
	RAINY_WEATHER_CONDITION,
	SNOWY_WEATHER_CONDITION,
};



enum FGoutcomes {
	MISS_FG_SHORT,
	MISS_FG_WIDE,
	MAKE_FG
};


enum offsidetypes {
	DEF_OFFSIDES_TYPE,
	DEF_ENCROACHMENT_TYPE,
};



struct actualgraphics {
	BYTE GraphicNum;
	BYTE GraphicSeq;
};


enum gametypes {
	EXHIBITION_GAME,
	PRACTICE_GAME,
	SEASON_GAME,
	PLAYOFF_GAME,
	CHAMPIONSHIP_GAME
};



enum crowdsfxs {
	LOW_CROWD_SFX,
	MID_CROWD_SFX,
	HIGH_CROWD_SFX,
	LOOP_SFX,
	MINOR_LOW_CROWD_SFX, // minors are added to majors so crowd never steady volume
	MINOR_MID_CROWD_SFX,
	MINOR_HIGH_CROWD_SFX,
	MINOR_LOOP_SFX,
	MAX_CROWD_SFXS	// low(normal), med, high(cheering)
};

#define NUM_MAJOR_CROWD_SFXS	(MAX_CROWD_SFXS/2)


/**********************  Play  State Equates   **********************/


#define psmacro(xyz)  xyz##NUM,

enum PlayStates {


	#include	"FBpsdata.h"	// as handles (intergers)
	 MAX_PLAY_STATES
};

#undef psmacro

#define OKAY_TO_HIKE_PS         PS_Say1stHutNUM
#define OKAY_FOR_MOTION_PS      PS_SaySetNUM




/**  types of passes		*/
enum passtypes {
	NO_PASS,
	LOB_PASS,			//1
	LOB_PASS1,			//2
	MEDIUM_LOB_PASS, 	//3
	MEDIUM_LOB_PASS1, 	//4
	NORMAL_PASS, 		//5
	NORMAL_PASS1, 		//6
	MEDIUM_BULLET_PASS,	//7
	MEDIUM_BULLET_PASS1,//8
	BULLET_PASS,		//9
	BULLET_PASS1,		//10
	MAX_PASS_TYPES		//11
};

// for QB AI Animations  (0-6  = 7 frames 2/12/96
#define	PASS_START_FRAME_NUM	1
#define	PASS_RELEASE_FRAME_NUM	5
#define	PASS_END_FRAME_NUM		6


enum shuffletypes {
	NORMAL_SHUFFLE,
	BLOCKING_SHUFFLE
};

//#define	PASSTIMER10			10

enum playtypes {        // stored in var "PlayType"
	NORMAL_PLAY,    // starts as a run
	PASS_PLAY,   	// never set (used in playcountarray if pass_bflag set)
	PUNT_PLAY,
	FIELD_GOAL_PLAY,
	ASK_FOR_EXPT,   // not really a play but it's useful here
	EX_PT1_PLAY,    // Play call will only offer 3 choices
	EX_PT2_PLAY,    // Play Call will offer all non-special team plays
	KICKOFF_PLAY,    // Play call will only offer 3 choices
	MAX_PLAY_TYPES
};

enum kickofftypes {
	DONT_CARE_KO_TYPE,
	NORMAL_KO_TYPE,
	ONSIDE_KO_TYPE,
	SQUIB_KO_TYPE
};

enum onsidekicks {
	NO_ONSIDE,
	ONSIDE_LEFT,
	ONSIDE_RIGHT
};

enum scoretypes {
	NO_SCORE,
	EXPT_SCORE,
	CONV_SCORE,
	SAFETY_SCORE,
	FG_SCORE,
	TD_SCORE,
	MAX_SCORE_TYPES
};



#define RUN_PLAY	NORMAL_PLAY


enum kickreturnertypes {
	KR_ALL,
	KR_LEFT,	// if spread formation
	KR_RIGHT,	// if spread formation
};

/**************  Player / Team  Data Structures   *****************/

#define MAX_OFF_PLAYDATA	8


#if 0

struct playerabilities{
	UBYTE   Strength;		//now 0-15 (soon to be 0-100 ???)
	UBYTE   Speed;          // 0-15	(6-13)
	UBYTE   Acceleration;   // 64-255
	UBYTE   Reaction;       // 0 -100
	UBYTE	Hands;          // 0 -100
	UBYTE	ArmStrength; 	// 0 -100
	UBYTE	FootStrength;   // 0 -100
	UBYTE	Misc;
};


#define FIRST_NAME_LENGTH       10
#define LAST_NAME_LENGTH        11

struct specificplayerdata {
	UBYTE FirstName[FIRST_NAME_LENGTH];
	UBYTE LastName[LAST_NAME_LENGTH];
	UBYTE SkinColor;
	UBYTE JerseyNum;
	UBYTE PhysSize; // low nibble is width, high nibble is height offset
//	UBYTE Misc[3];          // WORD boundaries, please
	struct playerabilities Ratings;
};
#endif

struct specificteamdata {	// see fbtmData.c

	BYTE    TeamNumber;     /* 0 to num teams in entire game        */
	BYTE    BaseTeamNumber; /* original team (if team of past)      */
	BYTE	TeamType;		// run, pass, or normal
	BYTE	reserved;		// future use

	BYTE    *TeamCity;
	BYTE    *TeamNickName;

	BYTE    Formations[2][7];	//[0] = offenive forms, [1] = def. forms
	BYTE    Stats[8];				/* added by Nick*/
	WORD	PenaltyFactor;

};

struct  gameteamdata    {
	WORD    TeamNumber;				/* 0 to num teams in entire game	*/
	WORD    TimeOuts;
	WORD    TeamFlags;				// these cleared every play
	WORD    GameTeamFlags;			// these never cleared except at gameinit
	WORD    ScorePerQuarter[5];
	WORD 	DownsForFirst;			// why always 4??
	WORD 	YardsForFirst;          // why always 10??

	WORD    TotalScore;				/* can't you add?       */
	WORD    OffDefFlag;             /* */
	WORD    NumHumanControllers;    /* 0 means computer controlled  */
	WORD    ControllerNum[MAX_USERS_PER_TEAM]; /* -1 means computer controlled */
	WORD    PrevDefControlledPlayers[MAX_USERS_PER_TEAM]; /* -1 means computer controlled */
	WORD    PrevOffControlledPlayers[MAX_USERS_PER_TEAM]; /* -1 means computer controlled,
										otherwise which RECEIVER they were */

	WORD    PlayNum;
	BOOL    UseEditedPlays[2];		/* True if using edited plays [off/def] */
	BOOL    PlayFlipped;			/* True if play flipped				*/
	WORD    FormationNumber;		/* actual formation number			*/
	struct  specificteamdata STD;	/* "ROM" data will be put here		*/
	struct  pdata_t PlayerData[TOTAL_PLAYERS_ON_TEAM];   /* "ROM" data will be put here  */
	WORD	allplyrstats[MAX_NUM_STATS_1TEAM];		// no "compression" here
	WORD	alloppplyrstats[MAX_NUM_STATS_1TEAM];	// opposition stats

	WORD	PrevOffPlayNum[MAX_OFF_PLAYDATA];  // play num of last X plays
	WORD	OffPlayNumCounter;		// index into above
	WORD	PrevOffPlayData[MAX_OFF_PLAYDATA];	// BC of last X plays
	WORD	OffPlayDataCounter;		// index into above
	struct  pdata_t *createdteamptr;	// init this after loading edited team
};



#define TOGGLE_TO_CLOSEST_VALID_BFLAG	0x0001	/* in TeamFlags	*/
#define	OKAY_TO_SAY_HUT_BFLAG			0x0002	/* in TeamFlags	*/
#define NO_MORE_CPU_SAY_HUT_BFLAG		0x0004	/* in TeamFlags	*/

#define PDATA_LOADED_BFLAG				0x0008	/* in GameTeamFlags	*/


/**********  Instant Replay Data  Structures   ********************/


/*  above moved to see  FBirp.h 11/5/96	---------------------*/


/*  NOTE: COnsole (RISC) compilers make all structure entries the same
	size - the size of the largest. Therefore, size of struct with
	1 WORD and 5 bytes is 12, not 7  -
	  PLEASE VERIFY THIS, it may NOTbe true
	oct. 96 - this does not appear to be true
*/

#define NEW_IRP_STRUCT	1
#if NEW_IRP_STRUCT
struct IRPData
{             		// 6 bytes per player
	UWORD XPos:12;
	UWORD Facing:4;			// bit fields

	UWORD YPos:12;
	UWORD GraphicSeqNum:4;

	UWORD ZPos:10;
	UWORD GraphicNum:6;

//	BYTE Facing;
//	BYTE GraphicNum;
//	BYTE GraphicSeqNum;
};
#else
/*
struct IRPData
{             		// 8 bytes per player
	UWORD XPos;
	UWORD YPos;
	UWORD ZPos;

	UWORD Facing:4;			// bit fields
	UWORD GraphicSeqNum:4;
	UWORD GraphicNum:8;

//	BYTE Facing;
//	BYTE GraphicNum;
//	BYTE GraphicSeqNum;
}; */
#endif



struct frameIRPData	// Even-Odd frame counter
{

	UWORD  PlayerIRPData[22][3];

	UWORD BallFlags;
	UWORD BallXPos;
	UWORD BallYPos;
	UWORD BallZPos;

	UWORD Controller0:5;
	UWORD Controller1:5;
	UWORD Controller2:5;
	UWORD ExtraBit1:1;

	UWORD Controller3:5;
	UWORD Controller4:5;
	UWORD Controller5:5;
	UWORD ExtraBit2:1;

	UWORD Controller6:5;
	UWORD Controller7:5;
	WORD  BallCarrier:6;

	UWORD OddFrameData[22];
	UWORD OddBallFlags;
	WORD  OddBallCarrier;

};

/*
struct frameIRPData
{
//	struct IRPData  PlayerIRPData[22];
	UWORD  PlayerIRPData[22][3];
//      struct IRPData  BallIRPD;
	WORD BallFlags;
	UWORD BallXPos;
	UWORD BallYPos;
	UWORD BallZPos;

	UWORD Controller0:5;
	UWORD Controller1:5;
	UWORD Controller2:5;
	UWORD ExtraBit1:1;

	UWORD Controller3:5;
	UWORD Controller4:5;
	UWORD Controller5:5;
	UWORD ExtraBit2:1;

	UWORD Controller6:5;
	UWORD Controller7:5;
	WORD BallCarrier:6;
//	BYTE BallCarrier;
};
*/

struct fullIRPData	// used once to save at IRP init to save true data
{					// had bug when return to huddle if not exactly restored
// 8 bytes per player
	UWORD XPos;
	UWORD YPos;
	UWORD ZPos;

	UWORD Facing:4;			// bit fields
	UWORD GraphicSeqNum:4;
	UWORD GraphicNum:8;

};
struct fullframeIRPData // used once to save at IRP init to save true data
{
	struct fullIRPData  PlayerIRPData[22];
	WORD BallFlags;
	UWORD BallXPos;
	UWORD BallYPos;
	UWORD BallZPos;

	UWORD Controller0:5;
	UWORD Controller1:5;
	UWORD Controller2:5;
	UWORD ExtraBit1:1;

	UWORD Controller3:5;
	UWORD Controller4:5;
	UWORD Controller5:5;
	UWORD ExtraBit2:1;

	UWORD Controller6:5;
	UWORD Controller7:5;
	WORD BallCarrier:6;
};



struct onetimeIRPdata
{
	UWORD RefX;
	UWORD RefY;
	WORD  Receivers[3];
	WORD  BeginningofPlayOffTeamSlotNum;
	WORD  SavedOffTeamSlotNum;
	WORD  BeginningofPlayOffenseDir;
	WORD  SavedOffenseDir;
	UWORD BallTargetX;
	UWORD BallTargetY;
	UWORD SaveRefMarkerX[2];
	UWORD StartPlayRefMarkerX[2];
	UWORD SaveFirstDownX;
	UWORD SaveStartBallX;
	UWORD StartFirstDownX;
	UWORD StartStartBallX;
	WORD  SaveBallFlags;
	WORD  SaveBallDirection;
	specificplayerdata *IRPPlayerDataPtr[22];
};





#ifdef  DEBUGMODE
#define TRACE(xyz) printf(#xyz "=%4x ",xyz)
#define DEBUG(xyz) printf("\n"#xyz"  ")
#else
#define TRACE(xyz)
#define DEBUG(xyz)
#endif

#include        "fbaidata.h"

#endif          /*      _FBEQU_H        */



/******       function prototypes    *******************/

void    NoAI(void);                     /*  a Do Nothing AIaction       */
void    NoAction(void);         /*  a NULL action       */


WORD FindCPDirectionToTargetSub(void);
WORD FindCPDirectionToBallSub(void);
WORD FindCPDirectionToPlayer(WORD opnum);

WORD SetMinMax(WORD value, WORD min, WORD max);
void SetMemory(void *area, BYTE setto, UWORD n);	// in fbstats.c


void  GoToNextState(UEFFVAR  ParamIncValue);



/******        M A C R O S    *******************/
#define PLAYER_ACTION(functionname) 	void functionname(void)


#define random0_99      (rand()%100)
#define Random1000		(rand()%1001)
#define Random10000		(rand()%10001)

#define FindCPDirectionToTargetMACRO	FindCPDirectionToTargetSub()
#define FindCPDirectionToBallMACRO		FindCPDirectionToBallSub()

#define FindDirection2Points(x1,y1,x2,y2) FindDirection( ((x2)-(x1)),((y2)-(y1)))

#define DistanceToPlayer(opnum) Distance(OPXPos(opnum),OPYPos(opnum))
#define CPDistanceToTargetMACRO Distance(CPTargetX,CPTargetY)
#define CPDistanceToBallMACRO Distance(BallXPosition,BallYPosition)


#define DEFENSIVE_SLOT_NUM_MACRO (GameTeamData[0].OffDefFlag == ON_DEFENSE ? 0:1)
#define OFFENSIVE_SLOT_NUM_MACRO (GameTeamData[0].OffDefFlag == ON_OFFENSE ? 0:1)

#define HOME_TEAM_HAS_BALL (GameTeamData[0].OffDefFlag == ON_OFFENSE)
#define AWAY_TEAM_HAS_BALL (GameTeamData[1].OffDefFlag == ON_OFFENSE)

#define   SetBallAction(xyz)  BallAction.todo = (xyz)



#define SetBit(xxx,yyy)		xxx |= (yyy)
#define ResetBit(xxx,yyy)	xxx &= ~(yyy)
// following for structure based flags
#define SetBit1(xxx,yyy)		xxx.yyy = 1
#define ResetBit1(xxx,yyy)	xxx.yyy = 0


#define Do_Back_Pedal_Macro     SetBit(CPFlags,BACK_PEDAL_BFLAG)
#define No_Back_Pedal_Macro		ResetBit(CPFlags,(BACK_PEDAL_BFLAG | I_RUN_BACKWARD_BFLAG))



// following also used for passes,punts, kicks - any "BallInAir"
//      except long hike or pitches or FGs

#define NEW_LONG_HIKE
/*   new Long hike uses different ball parameters so as not
		 to interfere with Kicking arc stuff	*/
#define HikeSpeed	150	// for 20htz f.rate


#define BallInAirMACRO  (BallFlags & (BALL_PASSED_BFLAG|BALL_PUNTED_BFLAG|BALL_KICKEDOFF_BFLAG) )
#define	PassComingDownMACRO	(BallFlags & PASS_COMING_DOWN_BFLAG)
//#define	DefReactToPassMACRO	(EventFlags.DEF_REACT_TO_PASS_BFLAG)

#define MAX_REACT_TIME		(30*SECONDS)


#define ImBallCarrierMACRO      (gPNum == BallCarrier)


// NOTE: if #frame in anim = 12 then MaxAIFrame=11  (0 to 11)

#define  MAX_FRAMES_MACRO(aigraphnum)	(MaxAIFrames[aigraphnum])

/***
#define CYCLE_GRAPHICS_MACRO(graphnum)  \
	if(++CPAIGraphicSequence > MaxAIFrames[graphnum])\
		CPAIGraphicSequence = ZERO		***/

void CycleGraphics(UWORD graphnum);		// routine saves code 11/7/96
#define CYCLE_GRAPHICS_MACRO(graphnum)  CycleGraphics(graphnum)

/*#define CYCLE_GRAPHICS_MACRO(graphnum)  CycleGraphic(graphnum)\
	if(++CPAIGraphicSequence > MaxAIFrames[graphnum])\
		CPAIGraphicSequence = ZERO	*/

#define	PutCameraOnRefMacro		SetBit(PlayerFlags[REF_SLOT_NUM],I_CAN_PUSH_BFLAG)
#define	PutCameraOffRefMacro	ResetBit(PlayerFlags[REF_SLOT_NUM],I_CAN_PUSH_BFLAG)
#define	CameraOnRefMacro	(PlayerFlags[REF_SLOT_NUM] & I_CAN_PUSH_BFLAG)




#define SetNewScript(xxx)       \
		CPScriptPtr = All_Scripts[xxx]; CPStateInit = FALSE


#define SetOPNewScript(xxx,yyy)       \
		OPScriptPtr(yyy) = All_Scripts[xxx]; OPStateInit(yyy) = FALSE

#define STATEDEF(xyz) \
	void  (xyz)(void);   \
	void  (xyz)(void)   /* actual function name of the State  */

/* randper returns value 1-100	*/
#define randper ( (rand()%100)+1)
#define randnum(xyz) (rand()%xyz)


#define SetAI(xyz)  CPAIState = (xyz)


#define	Q_CPUTeamMacro(teamnum)	(GameTeamData[teamnum].NumHumanControllers == ZERO)
#define	Q_HumanTeamMacro(teamnum)	(GameTeamData[teamnum].NumHumanControllers > ZERO)
#define	OffenseIsCPUTeamMacro	(GameTeamData[OffensiveSlotNum].NumHumanControllers == ZERO)
#define	DefenseIsCPUTeamMacro	(GameTeamData[DefensiveSlotNum].NumHumanControllers == ZERO)

#define ANY_EXPT_PLAY ((PlayType == EX_PT1_PLAY) || (PlayType == EX_PT2_PLAY))


/*---------------------------------------------------------------------*/

extern lgplyrPTR LeaguePlayersPtr;	// MUST use typedef
extern struct teamstats *ROMTeamStatsPtr;	//ROM stats - display only
extern struct colleges *CollegePtr;
extern struct allcreatedplayers  *CreatedPlayerPtr;



/********************** End of File  - FBEQU.H   **********************/
