/**************************************************************


					FBFRMDAT.H

	AUTHOR: Bobby King			Start Date: 11/28/95
	Modify Date:


	Copyright 1995  GameTek, Inc.



	This file contains formation data for Football AI.

	It uses our redefinable MACROs to group all formation data
	here in this file and then change the MACRO depending on
	what data we need, such as enums, array data elements, or strings.



	It is an INCLUDED file.

**************************************************************/


/**********************************************************************

			F O R M A T I O N S

	Formations are refered to in two ways:

	"General Formations" - used in Play Calling Screen - this includes
	ProSet, Special teams, etc.

	"Positional Formations" - used to position the players at the start
	of the play - this [Formation Number] is included in the play data
	structure. E.G. "special teams" is NOT a positional formation, but
	punt formation and field goal formation are positional formations.

	On Offensive, teams have a choice of 5 of the 13 normal formations -
	all teams have 2 default formations, then 1 from column A and 1 from
	column B, and 1 from column C.

	A provision has been made for "editable formations" (General only)
	on both offense and defense.


	The macro [currently] has 5 parameters:
	1) General formation name - use in enum table and in pointer table
	2) True Play calling formation - used to map positional formations
		to play call formations - used by CPU_Play_Calling to see if
		a selected play is valid for that team's formations
	3) String (actual name) of formation used in Play Calling
	4) Used in an array to tell if formation (and play) is OFF or DEF
	5) Offense sequential numbering (used by defense AI table)

	All General (used in Play Call) Formations are listed first, then
	positional and

-----------------------------------------------------------------------*/

/*  0 */ formmacro(GOAL_LINE_FORM,GOAL_LINE_FORM_NUM,GOAL LINE,OFF_FORM,0)
/*  1 */ formmacro(WISH_BONE_FORM,WISH_BONE_FORM_NUM,WISHBONE,OFF_FORM,1)
/*  2 */ formmacro(POWER_I_FORM,POWER_I_FORM_NUM,POWER I,OFF_FORM,2)
#ifdef SATURNMODE
/*  3 */ formmacro(NEAR_FAR_FORM,NEAR_FAR_FORM_NUM,VEER,OFF_FORM,3)
#else
/*  3 */ formmacro(NEAR_FAR_FORM,NEAR_FAR_FORM_NUM,SHIFTED I,OFF_FORM,3)
#endif
/*  4 */ formmacro(I_SLOT_FORM,I_SLOT_FORM_NUM,I SLOT,OFF_FORM,4)

/*  5 */ formmacro(SINGLE_BACK_FORM,SINGLE_BACK_FORM_NUM,ONE BACK,OFF_FORM,5)
/*  6 */ formmacro(H_BACK_FORM,H_BACK_FORM_NUM,H BACK,OFF_FORM,6)
/*  7 */ formmacro(I_FORM_FORM,I_FORM_FORM_NUM,I FORM,OFF_FORM,7)
/*  8 */ formmacro(PROSET_FORM,PROSET_FORM_NUM,PRO SET,OFF_FORM,8)
/*  9 */ formmacro(WEST_COAST_FORM,WEST_COAST_FORM_NUM,W COAST,OFF_FORM,9)

/* 10 */ formmacro(SHOTGUN_FORM,SHOTGUN_FORM_NUM,SHOTGUN,OFF_FORM,10)
/* 11 */ formmacro(TRIPS_FORM,TRIPS_FORM_NUM,TRIPS,OFF_FORM,11)
/* 12 */ formmacro(RUN_SHOOT_FORM,RUN_SHOOT_FORM_NUM,RUN SHOOT,OFF_FORM,12)
/* 13 */ formmacro(OFF_SPECIAL_TEAMS_FORM,KICKOFF_KICKING_FORM_NUM,SPECIAL,NO_FORM,-1)    /* Special Teams */


/* 14 */ formmacro(DEF_GOALINE_FORM,DEF_GOALINE_FORM_NUM,GOAL LINE,DEF_FORM,-1)
/* 15 */ formmacro(DEF_4_3_FORM,DEF_4_3_FORM_NUM,4-3,DEF_FORM,-1)			/* 4-3 */
/* 16 */ formmacro(DEF_3_4_FORM,DEF_3_4_FORM_NUM,3-4,DEF_FORM,-1)        /* 3-4 */
/* 17 */ formmacro(DEF_NICKEL_FORM,DEF_NICKEL_FORM_NUM,NICKEL,DEF_FORM,-1)
/* 18 */ formmacro(DEF_DIME_FORM,DEF_DIME_FORM_NUM,DIME,DEF_FORM,-1)

/* 19 */ formmacro(DEF_SPECIAL_TEAMS_FORM,DEF_SPECIAL_TEAMS_FORM_NUM,SPECIAL,NO_FORM,-1)


/* 20 */ formmacro(OFF_EASTER_FORM,KICKOFF_KICKING_FORM_NUM,OFF TEST,NO_FORM,-1)    /* 14 offensive edit */
/* 21 */ formmacro(DEF_EASTER_FORM,KICKOFF_KICKING_FORM_NUM,DEF TEST,NO_FORM,-1)    /* 14 offensive edit */
/* 22 */ formmacro(TEAM0_EDIT_FORM,DEF_EASTER_FORM_NUM,EDITED,NO_FORM,-1)
/* 23 */ formmacro(TEAM1_EDIT_FORM,DEF_EASTER_FORM_NUM,EDITED,NO_FORM,-1)

/*   below are "positional" formations  */
#ifdef	ALL_FORMS_VALID

/* 24 */ formmacro(GOAL_LINE_NO_WR_FORM,GOAL_LINE_FORM_NUM,G.L. NO WR,OFF_FORM,13)
/* 25 */ formmacro(GOAL_LINE_T_FORM,GOAL_LINE_FORM_NUM,GOAL LINE T,OFF_FORM,14)
/* 26 */ formmacro(POWER_I_WEAK_FORM,POWER_I_FORM_NUM, POWER I WEAK,OFF_FORM,15)
/* 27 */ formmacro(FAR_FORM,NEAR_FAR_FORM_NUM,SHIFTED I 2,OFF_FORM,16)
/* 28 */ formmacro(SHOTGUN_TE_FORM,SHOTGUN_FORM_NUM, SHOT GUN TE,OFF_FORM,17)
/* 29 */ formmacro(QB_KNEEL_FORM,KICKOFF_KICKING_FORM_NUM,X,OFF_FORM,18)
/* 30 */ formmacro(PUNT_FORM,KICKOFF_KICKING_FORM_NUM,X,OFF_FORM,19)
/* 31 */ formmacro(FIELD_GOAL_FORM,KICKOFF_KICKING_FORM_NUM,X,OFF_FORM,20)

/* 32 */ formmacro(DEF_PUNT_FORM,KICKOFF_KICKING_FORM_NUM,X,DEF_FORM,-1)
/* 33 */ formmacro(DEF_PUNT_DOUBLE_COVER_FORM,KICKOFF_KICKING_FORM_NUM,X,DEF_FORM,-1)
/* 34 */ formmacro(DEF_PUNT_BLOCK_FORM,KICKOFF_KICKING_FORM_NUM,X,DEF_FORM,-1)
/* 35 */ formmacro(DEF_FG_FORM,KICKOFF_KICKING_FORM_NUM,X,DEF_FORM,-1)
/* 36 */ formmacro(DEF_ALL_OUT_BLOCK_FORM,KICKOFF_KICKING_FORM_NUM,X,DEF_FORM,-1)
/* 37 */ formmacro(DEF_DIME_4MANLINE_FORM,KICKOFF_KICKING_FORM_NUM,X,DEF_FORM,-1)

/* 38 */ formmacro(DEF_DUMMY_FORM,DEF_DUMMY_FORM_NUM,X,DEF_FORM,-1)
/* 39 */ formmacro(DEF_ONE_MAN_FORM,DEF_ONE_MAN_FORM_NUM,X,DEF_FORM,-1)
/* 40 */ formmacro(OFF_ONE_MAN_FORM,OFF_ONE_MAN_FORM_NUM,X,OFF_FORM,-1)
/* 41 */ formmacro(KICKOFF_KICKING_FORM,KICKOFF_KICKING_FORM_NUM,KICK OFF,DEF_FORM,-1)
/* 42 */ formmacro(KICKOFF_RETURN_FORM,KICKOFF_KICKING_FORM_NUM,KICK RET,OFF_FORM,-1)
/* 43 */ formmacro(OFF_HUDDLE_FORM,OFF_HUDDLE_FORM_NUM,X,OFF_FORM,-1)
/* 44 */ formmacro(DEF_HUDDLE_FORM,DEF_HUDDLE_FORM_NUM,X,DEF_FORM,-1)
/* 45 */ formmacro(OFF_RACE_FORM,OFF_RACE_FORM_NUM,X,OFF_FORM,-1)
/* 46 */ formmacro(KICKOFF_LEFT_KICKING_FORM,KICKOFF_KICKING_FORM_NUM,KICK OFF,DEF_FORM,-1)
/* 47 */ formmacro(KICKOFF_RETURN_SPREAD_FORM,KICKOFF_KICKING_FORM_NUM,KICK RETURN,OFF_FORM,-1)
/* 48 */ formmacro(KICKOFF_RIGHT_KICKING_FORM,KICKOFF_KICKING_FORM_NUM,KICK OFF,DEF_FORM,-1)
/* 49 */ formmacro(KICKOFF_RETURN_HANDS_FORM,KICKOFF_KICKING_FORM_NUM,KICK RETURN,OFF_FORM,-1)
#endif


/*********************   End of File:  FBFRMDAT.h  ********************/
