/**************************************************************


					FBINIT.C

	INITIAL AUTHOR: M. Knauer       Start Date: 3/7/97

	Modify Date:


	Copyright 1997  GameTek, Inc.



	This file contains Power ON Init stuff of the Football AI.
	OR stuff that should be called BEFORE InitGameData

	Mainly deals with init Rosters, options, and
	loading player data into GameteamData[x].??? before a game starts.
	The best time to load player data is right after the two teams have
	been determined.

	To use overlays, this file was broken out of FBAIMain.c




 ---  Disclaimer ------------------------------------------
 (i.e.-you better know the following if you are	working on the code)

	NOTE: All AI routines in the main AI loop refer to the
	"CURRENT PLAYER" (usually the loop counter of the AI loop 0-21 -
	often refered to by variable "gPNum")

	Any references to the players Data Stuctures are "hidden" in
	a macro starting wtih the initials CP. This allows us to
	more freely change the data structure (if needed) from the initial
	non-linked array based DS's (easier to debug) to a structure based
	data structure

	E.g.  CPXPos =  PlayerXPosition[gPNum]     (current as of 5/19/95)
		could be changed to
		 CPXPos =  Player[gPNum].XPosition
		or something like
		 initialize  	CP = Player[gPNum]
		 in macro		CPXPos =  CP->XPosition

**************************************************************/

	#include  "fbequ.h"
	#include  "fbRAM.h"			/* external VARs needed by this file	*/

	#include  "misc.h"

#if __MSDOS__
	#include  "stdio.h"
	#include  "conio.h"

#endif

void FreeMem(void *ptr);



extern BYTE BaseSpeed[TOTAL_PLAYERS_ON_TEAM];	// based on player position
//extern BYTE BaseStrength[TOTAL_PLAYERS_ON_TEAM];	// based on player position

void InitNumOfRAMStatsPerSet(void);	// proto - module in FBStats.c

EFFVAR BuildSeasonTeam(UEFFVAR teamnum, struct pdata_t *putplyrptr);

void * LoadCDFile(BYTE *fname);
void FreeFileData(void *mem);


/*************  Data *************************************/

struct teamstats *ROMTeamStatsPtr = NULL;	//ROM stats - display only
BYTE *CollegeFilePtr = NULL;
struct colleges *CollegePtr = NULL;
lgplyrPTR LeaguePlayersPtr = NULL;
struct allcreatedplayers  *CreatedPlayerPtr = NULL;



#ifdef __MSDOS__
BYTE *datafilepath = "";
#else	// PSX
BYTE *datafilepath = "/PLAYERS/";
#endif


#define	gameoptmacro(enum,initvalue) initvalue,

WORD GameOptionsInit[MAX_NUM_GAME_OPTNS] = {
	gameoptions_stuff		// a huge define
};

#undef gameoptmacro



/*--------------------------------------------------------------*/

void SetCPUvsCPUGameParameters(void)
{
EFFVAR t,i;


	for(t=0; t <2; t++)
	{
		GameTeamData[t].UseEditedPlays[0] =		// no EDITed or E EGG plays
			GameTeamData[t].UseEditedPlays[1] = ZERO;

//		OverrideEditedSetting(t,2);		// test only - use E EGG plays

		for(i=0; i<MAX_USERS_PER_TEAM; i++)		// clear all controllers - CPU game
			GameTeamData[t].ControllerNum[i]=NO_CONTROLLER;

		GameTeamData[t].GameTeamFlags = ZERO;
	}
}




/*********************************************************************

		 I N I T     P L A Y E R   D A T A / R A T I N G S

	As of 3/1/96, called at in InitAI()(beginiing of every play)
	so that players ratings can	be changed every play by testers.

	(This won't be called in final version- only at game init).

	Now (8/22/96) only called at game beginnning - sets all player
	ratings to a default so things won't blow up later if external
	player data files won't load.

	void InitDefaultPlayerData(WORD teamslotnum)

*********************************************************************/
void InitDefaultTeamRosters(void)	// NOT at game init - could override pre-game substitutions
{ 	// init roster to default (changed by depth chart)
EFFVAR t,i;

	for (t=0; t<2; t++)
		for(i=0; i<TOTAL_PLAYERS_ON_TEAM; i++)
			TeamRoster[t][i] = i;

}


void InitDefaultPlayerData(WORD teamslotnum)	// only at game init
{	// put something in data structure in case "LoadPlayerData" from disc fails
struct pdata_t *pdata;
EFFVAR i,j;

	GameTeamData[teamslotnum].createdteamptr = NULL;


	for(i=0; i<TOTAL_PLAYERS_ON_TEAM; i++)
	{
//		TeamRoster[teamslotnum][i] = i;	// init roster to default (changed by depth chart)

		pdata = &GameTeamData[teamslotnum].PlayerData[i];		//where it goes

		SetMemory(&GameTeamData[teamslotnum].PlayerData[i], ZERO, sizeof(struct pdata_t));

		pdata->pname[0] = 'M';
		pdata->pname[1] = 'I';
		pdata->pname[2] = 'K';
		pdata->pname[3] = 'E';
		pdata->position = i;   		//6 0-47
		pdata->jerseynum = i+1;
		pdata->skincolor= i&3;      //2 0-3
		pdata->weight = i*2; 	//8 0-255(150-405)  **** add offset ***
		pdata->height = i&15;   //8 0-15(66-81)  **** add offset ***

		for(j=0; j < MAX_NUM_RATINGS_PER_PLAYER; j++)
			pdata->ratings[j] = 50;		//assume

		pdata->ratings[SPEED_RATE] = 6 * (BaseSpeed[i]+4);

	}

//	TeamRoster[teamslotnum][0] = 1;	// test only - sub QB
//	TeamRoster[teamslotnum][1] = 0;

}


void InitLeagueRoster(void)
{
WORD t,p;

	for(t=0; t<NUM_TEAMS_IN_LEAGUE; t++)
		for(p=0; p < TOTAL_PLAYERS_ON_TEAM; p++)
			LeagueRoster[t][p] = (t*TOTAL_PLAYERS_ON_TEAM) + p;
}



/******************************************************************

				void LoadLeaguePlayerData(void)	// encapusaltion

	Loads files from disc s and sets valid global pointers for:

		lgplyrPTR LeaguePlayersPtr; // all players in league (teams 0-30)

		struct teamstats *ROMTeamStatsPtr	//ROM stats - display only

		BYTE *CollegeFilePtr
		struct colleges *CollegePtr	// use this pointer & college index

	Loads from biggest to smallest

******************************************************************/


void LoadLeaguePlayerData(void)	// encapusaltion
{

BYTE fname[40];

	if(LeaguePlayersPtr ==  NULL)	// don't load if already loaded
	{	// LoadCDFile will alloc memory and load from disc

		sprintf(fname,"%sallplyr.dat",datafilepath);

		LeaguePlayersPtr = (lgplyrPTR)LoadCDFile(fname);

	}

	if(ROMTeamStatsPtr ==  NULL)	// don't load if already loaded
	{	// LoadCDFile will alloc memory and load from disc

		sprintf(fname,"%sallteam.dat",datafilepath);

		ROMTeamStatsPtr	= (struct teamstats *)LoadCDFile(fname);

	}

	if(CollegeFilePtr ==  NULL)	// don't load if already loaded
	{	// LoadCDFile will alloc memory and load from disc

		sprintf(fname,"%scolleges.dat",datafilepath);

		CollegeFilePtr	= (BYTE *)LoadCDFile(fname);

		NumColleges = *((LONG *)CollegeFilePtr);

		CollegePtr	= (struct colleges *)(CollegeFilePtr + 4);
	}
}


void FreeLeaguePlayerData(void)	// encapusaltion
{   // free in reverse order of alloc

	if(CollegeFilePtr != NULL)
	{
		FreeMem(CollegeFilePtr);
		CollegeFilePtr = NULL;
		CollegePtr = NULL;
	}
	NumColleges = ZERO;

	if(ROMTeamStatsPtr != NULL)
	{
		FreeMem(ROMTeamStatsPtr);
		ROMTeamStatsPtr = NULL;
	}

	if(LeaguePlayersPtr != NULL)
	{
		FreeMem(LeaguePlayersPtr);
		LeaguePlayersPtr = NULL;
	}
}

/********************************************************************/


void PowerOnAIInitialization(void)
{
EFFVAR i;

/*  assume we go right into a game without any other init,
	such as team nums, controller options, OPTIONS, etc
*/

	GameType = 	EXHIBITION_GAME;
	PracticeMode = 0;	// Set by external shell, cleared at game end

	for(i=0; i<MAX_NUM_GAME_OPTNS; i++)	// "ROM" options to RAM options
		GameOptions[i] = GameOptionsInit[i];


	CPURatingCheat = TRUE;	// normal - allows CPU to cheat ratings if
							// users calling same plays over and over
	ForcePenalties = FALSE;	// debug variable

#if PC_ALWAYS_MODE
	CPURatingCheat = FALSE;	// test only!!!
	ForcePenalties = TRUE;	// debug variable
#endif


//	GameOptions[GAME_OPTNS_GAME_SPEED] = 2;	// test only

	SetCPUvsCPUGameParameters();

	InitNumOfRAMStatsPerSet();

	NumColleges = ZERO;
	PhraseFlag = FALSE;		// no announcer phrases are being said

	/*  set default rosters and player data (name, ratings, etc.) in
		case later CD loads fail			*/

	InitLeagueRoster();
	InitDefaultTeamRosters();	// NOT at game init
	InitDefaultPlayerData(0);	// name, etc.
	InitDefaultPlayerData(1);	// name, etc.

	GameTeamData[0].ControllerNum[0]=0;	// controller 0 on Home team - 1 player game

#if 0	//test only
	GameTeamData[0].ControllerNum[1]=1;	// controller 0 on Home team - 1 player game
	GameTeamData[0].ControllerNum[2]=2;	// controller 0 on Home team - 1 player game
	GameTeamData[0].ControllerNum[3]=3;	// controller 0 on Home team - 1 player game

	GameTeamData[1].ControllerNum[0]=4;	// controller 0 on Home team - 1 player game

	GameTeamData[1].ControllerNum[1]=5;	// controller 0 on Home team - 1 player game
	GameTeamData[1].ControllerNum[2]=6;	// controller 0 on Home team - 1 player game
	GameTeamData[1].ControllerNum[3]=7;	// controller 0 on Home team - 1 player game
#endif
//	GameTeamData[0].JoyPadConfig = GameTeamData[1].JoyPadConfig = ZERO;

 /*----- below  are testing variables	-----*/

#if	PC_DEBUG_MODE
//	GameOptions[GAME_OPTNS_DELAY_OF_GAME] = OPTION_OFF;	// no delay of game
#endif

	GameTeamData[0].TeamNumber=12;	// Dolphins
	GameTeamData[1].TeamNumber=25;	// Cowboys

	GenValue = 0;		 //for PC tester only

//	ShowHuddle = TRUE;		// debug variables
//	ShowHuddle = OkToTackle = TRUE;		// debug variables

//	TeamSpeed0 = TeamSpeed1 = 4; 		// ratings test variable
//	TeamHands0 = TeamHands1 = 50; 		// ratings test variable
//	TeamOtherRating0 = TeamOtherRating1 = 50;	// ratings test variable



#ifndef	SATURNMODE
/* MemoryCardMimic allocated on Saturn - however, everything and
	everybody (all machines) should	use MemoryCardMimicPtr			*/
	MemoryCardMimicPtr = &MemoryCardMimic;
#endif

}


void InitExhibitionGame(void)	// called on transistion from Main menu to exhib.
{
	GameType = 	EXHIBITION_GAME;
	PracticeMode = 0;	// Set by external shell, cleared at game end

	InitLeagueRoster();		// sets to default

	InitDefaultTeamRosters();	// NOT at game init - could override pre-game substitutions

	// don't use created/edited teams
	GameTeamData[0].createdteamptr = GameTeamData[1].createdteamptr = NULL;

}



/**
#ifdef __MSDOS__
	sprintf(s, "PDATA%d.DAT",GameTeamData[slotnum].TeamNumber);
#else
	// WARNING - only 30 files per directory on PSX
	sprintf(s, "%s%d/PDATA%d.DAT", path, GameTeamData[slotnum].TeamNumber / 30,
					 GameTeamData[slotnum].TeamNumber);
#endif
**/
/*************************************************************************
 *
 * LOAD PLAYER DATA - Loads player data to proper slots for both teams.
 *
	"old" style method which loads 1 teams data (47 players) from disc

	Returns: 0 if okay else error code

 *************************************************************************/
WORD LoadPlayerDataFor1Team(WORD slotnum)
{
register specificplayerdata *pdata;
char fname[40];
WORD errorcode = 1;	//assume we fail


	//      WARNING -     only 30 files per directory on PSX

	sprintf(fname,"%sPDATA%d.DAT",datafilepath,GameTeamData[slotnum].TeamNumber);

	if ((pdata = (specificplayerdata *)LoadCDFile(fname)) != NULL)
	{
		CopyBytes(	(UBYTE *)pdata,								// from
					(UBYTE *)GameTeamData[slotnum].PlayerData,	//to
					sizeof(specificplayerdata) * TOTAL_PLAYERS_ON_TEAM);

/**  no more teams stats in data file 4/15/97
		ptr = (UBYTE *)pdata + sizeof(specificplayerdata) * TOTAL_PLAYERS_ON_TEAM;
		CopyBytes(ptr, (UBYTE *)GameTeamData[slotnum].allplyrstats,
					 MAX_NUM_STATS_1TEAM * sizeof(WORD));

		ptr += MAX_NUM_STATS_1TEAM * sizeof(WORD);
		CopyBytes(ptr, (UBYTE *)GameTeamData[slotnum].alloppplyrstats,
				 MAX_NUM_STATS_1TEAM * sizeof(WORD));
****/
		FreeFileData(pdata);

		errorcode = ZERO;	// no errors
	}

	return(errorcode);

}


void BuildROMTeamStats(WORD slotnum)	// revised 4/11/97 for new season data base
{
EFFVAR teamnum;
UBYTE *ptr;

	teamnum = GameTeamData[slotnum].TeamNumber;

	if(ROMTeamStatsPtr != NULL
	 && teamnum >= ZERO
	 && teamnum < MAX_NUM_TEAMS)
	{	// valid

		ptr = (UBYTE *)(ROMTeamStatsPtr+teamnum);

		CopyBytes(ptr, (UBYTE *)GameTeamData[slotnum].allplyrstats,
				 MAX_NUM_STATS_1TEAM * sizeof(WORD) * 2);
/*
		ptr += MAX_NUM_STATS_1TEAM * sizeof(WORD);
		CopyBytes(ptr, (UBYTE *)GameTeamData[slotnum].alloppplyrstats,
				 MAX_NUM_STATS_1TEAM * sizeof(WORD));
*/
	}
	else	// clear
	{
		SetMemory(&GameTeamData[slotnum].allplyrstats, ZERO,
				 MAX_NUM_STATS_1TEAM * sizeof(WORD) * 2);
	}
}

/*************************************************************************

					void BuildPlayerDataForGame(void)

	There are 3 types of loading data into GameTeamData[x].PlayerData
	based on the team:

	1) Normal team (0-29)  - use AllPlyr.dat file to load players
		 1 at a time (in case of trading/cloning)
	2) Greatest teams (31-56) - load each team separately (these teams
		can not be modified)
	3) Created/Edited team - move data from created area to final area

	GameTeamData[x].TeamNumber	MUST have already been set up

	RETURNS: 0 if okay else error code

*************************************************************************/

EFFVAR BuildPlayerDataForGame(WORD slotnum)	// revised 4/11/97 for new season data base
{
EFFVAR errorcode,teamnum;

	errorcode = 10;	// assume we fail (error code)

	ResetBit(GameTeamData[slotnum].GameTeamFlags, PDATA_LOADED_BFLAG);

	teamnum = GameTeamData[slotnum].TeamNumber;	// this MUST have been inited

	if(teamnum < 0)
	{
		errorcode = 6;		// error - invalid team
	}
	else if(teamnum <= FREE_AGENTS)
	{	// assume AllPlayer data file is already opened

		errorcode = BuildSeasonTeam(teamnum,GameTeamData[slotnum].PlayerData);

	}
	else if(teamnum < MAX_NUM_TEAMS)
	{
		errorcode = LoadPlayerDataFor1Team(slotnum);	// team file (all 47 players)
	}
	else if(GameTeamData[slotnum].createdteamptr != NULL)
	{   // must be edited/created team
		CopyBytes(
					(UBYTE *)GameTeamData[slotnum].createdteamptr,	// from
					(UBYTE *)GameTeamData[slotnum].PlayerData,		// to
					  sizeof(specificplayerdata) * TOTAL_PLAYERS_ON_TEAM);

		errorcode = ZERO;
	}

	if(errorcode == ZERO)	// data is now okay
	{
		SetBit(GameTeamData[slotnum].GameTeamFlags, PDATA_LOADED_BFLAG);
	}

	return(errorcode);
}




void LoadPlayerDataForGame(void)	// revised 4/11/97 for new season data base
{
	BuildPlayerDataForGame(0);	// revised 4/11/97 for new season data base
	BuildPlayerDataForGame(1);	// revised 4/11/97 for new season data base


	BuildROMTeamStats(0);	// revised 4/11/97 for new season data base
	BuildROMTeamStats(1);	// revised 4/11/97 for new season data base

}




/**********************************************************************

	Build a team based on LeagueRoster -

	Can be used in Season mode OR Exhibition mode

	Assumes all "ROM" (or created) player data is in RAM and global
	pointers point to data areas.  Saves having to seek and load multiple
	(old style) team data files from disk in the case of an edited team
	having traded players from all 30 original teams. (i.e. the trade-off
	is to do 1 seek and load in a big file once as opposed to at least
	2 seek/loads or as many as 60 seek/loads if the league roster is
	totally convoluted due to major trading


	Input: Team Number (0-29) (to index League Roster)
		   PutPtr - where to put(build) this teams player data
			(usually GameTeamData[x].PlayerData)


	Uses:   LeagueRoster  (as to who's on this team)
			CreatedPlayerPtr (points to all current Created players)
			LeaguePlayersPtr (points to all 1400 NFLPA players)


	Returns: 0 if success
				else error num

**********************************************************************/

EFFVAR BuildSeasonTeam(UEFFVAR teamnum, struct pdata_t *putplyrptr)
{
EFFVAR i,plyrnum;
lgplyrPTR getplyrptr;	// MUST use typedef

	if(teamnum >= NUM_TEAMS_IN_LEAGUE)
	{
		return(1);		// ERROR - illegal input team
	}

	if(LeaguePlayersPtr == NULL)
	{
		return(2);		// ERROR - non-inited global pointer
	}


	for(i=0; i< TOTAL_PLAYERS_ON_TEAM; i++)
	{
		plyrnum = LeagueRoster[teamnum][i];

		if(plyrnum >= CREATED_PLAYER_START_INDEX)
		{	// this is a created player

			plyrnum -= CREATED_PLAYER_START_INDEX;
			if(plyrnum >= CreatedPlayerPtr->NumCreatedPlayers
			   || CreatedPlayerPtr == NULL
			  )
			{
				return(3); 	// illegal Created Player Num

			}
			else
			{
				getplyrptr = &(CreatedPlayerPtr->CreatedPlayers[plyrnum]);
			}

		}
		else	// normal NFLPA player
		{
			if(plyrnum >= TOTAL_PLAYERS_ON_TEAM * (NUM_TEAMS_IN_LEAGUE+1))	// include Free Agent team
			{
				return(4);	// illegal player number
			}
			else
			{
				getplyrptr =  LeaguePlayersPtr + plyrnum;
			}

		}

		CopyBytes((UBYTE *)getplyrptr, (UBYTE *)putplyrptr,
											 sizeof(struct pdata_t));

		putplyrptr +=  1;		// where to put next player
	}

	return(0);	// success
}


/****** following tables used for testing to init a players rating **/

// mjk test table - try to keep positions constant -all rec, DB same speed

#define SA	3	//not changed 2/6/96 when speedtable was decreased
//#define SA	0

BYTE BaseSpeed[TOTAL_PLAYERS_ON_TEAM] = {	// based on player position

// NEW for new 47 man roster
	7-SA,		//#define QB		ZERO	/* Quarterback (duh)	*/
	7-SA,		//#define QB2		1	/* backup quarter back		*/
	7-SA,		//#define QB3 	2

	11-SA,		//#define RR		3	/* right receiver  			*/
	11-SA,		//#define LR		4	/* left receiver			*/
	11-SA,		//#define WR3		5	/* wide receiver 3          */
	11-SA,		//#define WR4		6	/* wide receiver 4        	*/
	11-SA,		//#define WR5		7	/* wide receiver 5          */

	9-SA,		//#define RTE		8	/* right tight end		    */
	9-SA,		//#define LTE		9	/* left tight end           */
	9-SA,		//#define TE3		10	/* extra tight end          */

	9-SA,		//#define FB		11	/* full back				*/
	10-SA,		//#define HB		12	/* half back                */
	10-SA,		//#define RB3		13	/* HB2 extra running back		*/
	9-SA,		//#define FB2 	14

	7-SA,		//#define RT		15	/* offensive right tackle	*/
	7-SA,		//#define LT		16	/* offensive left tackle	*/
	7-SA,		//#define RG		17	/* offensive right guard	*/
	7-SA,		//#define LG		18	/* offensive left guard		*/
	7-SA,		//#define CENTER	19	/* offensive center			*/
	7-SA,		//#define OL6		20	/* extra offensive lineman	*/
	7-SA,		//#define	OL7		21

   11-SA,		//#define PUNT_RETURNER   22
   11-SA,		//#define KICK_RETURNER   23
   11-SA,		//#define KR3 	24	// back up KR & PR

	7-SA,		//#define PUNTER  25
	7-SA,		//#define KICKER  26	// kicker on both off.(FG) and def(KO)

	7-SA,		//#define DLE		27	/* defensive left end		*/
	7-SA,		//#define DLT		28	/* defensive left tackle	*/
	7-SA,		//#define DRT		29	/* defensive right tackle	*/
	7-SA,		//#define DRE		30	/* defensive right end      */
	7-SA,		//#define DL5		31	/* extra defensive line     */
	7-SA,		//#define DL6 	32	/* extra,extra defensive line     */

	9-SA,		//#define OLLB	33	/* outside left linebacker  */
	9-SA,		//#define ILLB	34	/* inside left linebacker   */
	9-SA,		//#define IRLB	35	/* inside right linebacker  */
	9-SA,		//#define ORLB	36	/* outside right linebacker */
	9-SA,		//#define LB5		37	/* extra linebacker			*/
	9-SA,		//#define LB6 	38	/* extra,extra linebacker			*/

	12-SA,		//#define FS		39	/* free safety				*/
	11-SA,		//#define SS		40	/* strong safety			*/

	11-SA,		//#define RC		41	/* right cornerback			*/
	11-SA,		//#define LC		42	/* left cornerback			*/
	11-SA,		//#define NB		43	/* nickel back				*/
	11-SA,		//#define DB		44	/* dime back				*/
	11-SA,		//#define DB7		45	/* extra defensive back		*/
	11-SA,		//#define DB8 	46	/* extra,extra defensive back		*/



};



/***************************  End of file - FBInit.c  *****************/