/*********************************************************************


					FBIRP.C

	INITIAL AUTHOR: M. Knauer       Start Date: 11/19/96

	Modify Date:


	Copyright 1995  GameTek, Inc.



	This file contains AI related Instant   Replay.



	MAJOR MOD.  11/19/96
	IRP now uses interpolation on the odd frames
	Odd frame data has only graphic attributes, not X,Y,Z
    Average X,Y,Z based on in between to even frames.

***********************************************************************/

	#include  "fbequ.h"
	#include  "fbRAM.h"		/* external VARs needed by this file	*/

	#include  "fbirp.h"		// size of IRP buffer


#ifdef __MSDOS__
	#include  <stdio.h>		/* for printf    */
#endif

extern struct frameIRPData *IRPptr;	// MUST be 1st var for SATURN!!!!


void * AllocMem(long bytes);
void FreeMem(void *addr);

void CopyBytes (BYTE *src , BYTE *dest , int count );	//in fbstats.c

/***********************************************************************

					I N S T A N T    R E P L A Y

	The Instant replay works by saving all pertinent player GRAPHIC
	data (X,Y,Z, Graphic State, Graphic sequence, etc) each time
	thru the loop.

	We can then "replay" this info either forward or backwards at
	varying speeds.

	Drawback is we need a big chuck of memory (approx. 30-40K for 15
	seconds @ 15 HZ).


	Note: Instant Replay will start recording at "BallSnapped" and
	continue recording until either:
		A) !PlayHappening (if Play took longer than max time)

//		a) reached MAX IRP frames	   		OR
		b) returned to Play Call Screen		OR
		c) IRP started (from PAUSE screen)

************************************************************************/


/*  following call in "replay loop" to restore RP data
	based on "framestoinc", it can run forwards(=1), backwards(-1)
	or fast forward (=2,3) or fast backward
	NOTE: no AI is done in the replay loop - only this			*/

//void Save1IRPFrame(struct frameIRPData *tptr);
void Save1IRPFrame(UWORD framenumber);
void Restore1IRPFrame(struct frameIRPData *tptr);

void Restore1EvenIRPFrame(struct frameIRPData *tptr0);
void Restore1OddIRPFrame(struct frameIRPData *tptr0,struct frameIRPData *tptr1);

void Save1FullIRPFrame(struct fullframeIRPData *tptr);
void Restore1FullIRPFrame(struct fullframeIRPData *tptr);



void DoInstantReplay(WORD framestoinc)
{
struct frameIRPData *tptr,*nptr;
EFFVAR lastirpframe,nextframe;

	if(IRPptr == NULL)
		return;

	if ( (lastirpframe = EndIRPFrameCounter-1) < ZERO)
	{ // if happen to be at beginning/end of buffer
		lastirpframe = MAX_IRP_FRAMES-1;
	}

	if(framestoinc < ZERO)	// want to go backwards
	{
		do
		{
			if(CurIRPFrameCounter != StartIRPFrameCounter)
			{
				if(--CurIRPFrameCounter < ZERO)
					CurIRPFrameCounter = MAX_IRP_FRAMES-1;
			}
		}while(++framestoinc != ZERO);
	}
	else	// forward or none
	{

		while(framestoinc-- > ZERO)
		{
			if(CurIRPFrameCounter != lastirpframe)
			{
				if(++CurIRPFrameCounter >= MAX_IRP_FRAMES)
					CurIRPFrameCounter = ZERO;
			}
		 }

	}


//	tptr = IRPptr+CurIRPFrameCounter;
//	Restore1IRPFrame(tptr);

	if(CurIRPFrameCounter >= MAX_IRP_FRAMES)	// extra check
		CurIRPFrameCounter = ZERO;


	tptr = IRPptr + (CurIRPFrameCounter/2);
#if 0
	Restore1EvenIRPFrame(tptr);

#else
	if(CurIRPFrameCounter&1)
	{	// odd frame

		// if at end, use current frame for both frames
		if(CurIRPFrameCounter == lastirpframe)
			nextframe = CurIRPFrameCounter;
		else
			nextframe = CurIRPFrameCounter + 1;

		if(nextframe >= MAX_IRP_FRAMES)
			nextframe = ZERO;

		nptr = IRPptr + (nextframe/2);
		Restore1OddIRPFrame(tptr,nptr);

	}
	else	// even frame
	{
		Restore1EvenIRPFrame(tptr);
	}

#endif


}


BOOL AtEndOfInstantReplay(void)
{
EFFVAR lastirpframe;
	// has EndIRPFrameCounter stopped at ZERO??
	if ( (lastirpframe = EndIRPFrameCounter-1) < ZERO)
				lastirpframe = MAX_IRP_FRAMES-1;

	return(CurIRPFrameCounter == lastirpframe);
}

/*-----------------------------------------------------------------
	following called (by graphic engine) before "replay loop" to init RePlay
	data- need to save where players currently are and then retore when IRP
	over, if not, then players start at whereever they were when IRP ended
-------------------------------------------------------------------------*/
void InitInstantReplay(void)
{

	ResetBit(InterfaceFlags,DISPLAY_BALL_ARC_BFLAG); // could have been up for a frame


	CurIRPFrameCounter = StartIRPFrameCounter;
	SetBit1(EventFlags,IRP_INITED_BFLAG);  // means don't save anymore

	SetBit(GameFlags,INSTANT_REPLAY_ACTIVE);	// some player data routines need to know

	Save1FullIRPFrame(&CurrentIRPData);	// save where guys currently are

	OneTimeIRP.RefX = PlayerXPosition[REF_SLOT_NUM];
	OneTimeIRP.RefY = PlayerYPosition[REF_SLOT_NUM];
	// ref at 0,0 won't be displayed
	PlayerXPosition[REF_SLOT_NUM] = PlayerYPosition[REF_SLOT_NUM] = ZERO;

/**  leftovers from old toggled receiver method
	Receivers[0] = OneTimeIRP.Receivers[0];
	Receivers[1] = OneTimeIRP.Receivers[1];
	Receivers[2] = OneTimeIRP.Receivers[2];
***********/
	OneTimeIRP.SavedOffTeamSlotNum = OFFENSIVE_SLOT_NUM_MACRO;	// who's on off now

	GameTeamData[OneTimeIRP.BeginningofPlayOffTeamSlotNum].OffDefFlag = ON_OFFENSE;
	GameTeamData[OneTimeIRP.BeginningofPlayOffTeamSlotNum^1].OffDefFlag = ON_DEFENSE;

	OneTimeIRP.SavedOffenseDir = OffenseDir;	// save in global "temp"
	OffenseDir = OneTimeIRP.BeginningofPlayOffenseDir;

	BallTargetX = OneTimeIRP.BallTargetX;
	BallTargetY = OneTimeIRP.BallTargetY;

	OneTimeIRP.SaveRefMarkerX[0] = RefMarkerX[0];	// where refs are at now
	OneTimeIRP.SaveRefMarkerX[1] = RefMarkerX[1];

	RefMarkerX[0] = OneTimeIRP.StartPlayRefMarkerX[0];	// where refs started play at
	RefMarkerX[1] = OneTimeIRP.StartPlayRefMarkerX[1];

	OneTimeIRP.SaveFirstDownX = FirstDownX;	// where refs are at now
	OneTimeIRP.SaveStartBallX = StartBallX;

	FirstDownX = OneTimeIRP.StartFirstDownX;
	StartBallX = OneTimeIRP.StartStartBallX;

	OneTimeIRP.SaveBallFlags = BallFlags;
	OneTimeIRP.SaveBallDirection = BallDirection;


}

void CleanUpInstantReplay(void)
{
//	ResetBit(EventFlags,IRP_INITED_BFLAG);
//	ResetBit(EventFlags,IRP_INITED_BFLAG);	// uncommented 6/4/96


	Restore1FullIRPFrame(&CurrentIRPData);	// restore where guys were before IRP

	// following allows another replay later in same frame
	CurIRPFrameCounter = StartIRPFrameCounter+1;

	PlayerXPosition[REF_SLOT_NUM] = OneTimeIRP.RefX;
	PlayerYPosition[REF_SLOT_NUM] = OneTimeIRP.RefY;
	Receivers[0] = Receivers[1] = Receivers[2] = NOBODY;

	GameTeamData[OneTimeIRP.SavedOffTeamSlotNum].OffDefFlag = ON_OFFENSE;
	GameTeamData[OneTimeIRP.SavedOffTeamSlotNum^1].OffDefFlag = ON_DEFENSE;

	BallTargetX = BallTargetY = ZERO;	// clear IRP Target of the prev. play

	RefMarkerX[0] = OneTimeIRP.SaveRefMarkerX[0];	// restore ref current X
	RefMarkerX[1] = OneTimeIRP.SaveRefMarkerX[1];

	FirstDownX = OneTimeIRP.SaveFirstDownX;
	StartBallX = OneTimeIRP.SaveStartBallX;

	BallFlags =	OneTimeIRP.SaveBallFlags;
	BallDirection = OneTimeIRP.SaveBallDirection;

	OffenseDir= OneTimeIRP.SavedOffenseDir;	// restore camera from gloabl "temp"

	ResetBit(GameFlags,INSTANT_REPLAY_ACTIVE);
}




BOOL OKForInstantReplay(void)
{ 								// set to 0 at game and play init
	if(  (IRPptr == NULL)
	  || (PlayHappening() )
	  || (CurIRPFrameCounter == StartIRPFrameCounter)
	  )
		return(FALSE);

	return(TRUE);		// if MAX frame counter non zero okay to show,
}



/**---------------------------------------------------------------

NOTE: some RISC compilers always align structures on 4 byte
	boundaries, - therefore our [former] 6 byte structure IRPData
	was not efficient in overall "frameIRPData", especially because
	very bit counts in IRP.

	The new frameIRPData uses a [22][3] WORD doubly dimensioned array
	to hold player data, thereby insuring that only 6 bytes are used.

	The struct IRPData is now used only as a local structure
------------------------------------------------------------------*/

#if 1	// new interpolated structure


/*
struct frameIRPData	// Even-Odd frame counter
{

	UWORD  PlayerIRPData[22][3];

	UWORD BallFlags;
	UWORD BallXPos;
	UWORD BallYPos;
	UWORD BallZPos;

	UWORD Controller0:5;
	UWORD Controller1:5;
	UWORD Controller2:5;
	UWORD ExtraBit1:1;

	UWORD Controller3:5;
	UWORD Controller4:5;
	UWORD Controller5:5;
	UWORD ExtraBit2:1;

	UWORD Controller6:5;
	UWORD Controller7:5;
	WORD  BallCarrier:6;

	UWORD OddFrameData[22];
	UWORD OddBallFlags;
	WORD  OddBallCarrier;

};
*/



void Save1EvenIRPFrame(struct frameIRPData *tptr)
{
struct IRPData tempIRP;
void *sptr, *dptr;

	for(gPNum=0; gPNum <22; gPNum++)
	{
		tempIRP.XPos = CPXPos>>4;
		tempIRP.YPos = CPYPos>>4;
		tempIRP.ZPos = CPZPos>>3;
		tempIRP.Facing = CPFacing;
		tempIRP.GraphicNum = CPGraphicState;
		tempIRP.GraphicSeqNum = CPGraphicSequence;

		sptr = &tempIRP;
		dptr = &(tptr->PlayerIRPData[gPNum][0]);

		CopyBytes(sptr, dptr, 6); // they're all WORDS

	}

/*  new 10/14/96 lower 4 bits of BallFlag is "don't care" for graphic engine
	put BallDir in those 4 bits		*/
	tptr->BallFlags = (BallFlags&0xfff0) | (BallDirection&0x000f);
//	tptr->BallFlags = BallFlags;

	tptr->BallXPos = BallXPosition;
	tptr->BallYPos = BallYPosition;
	tptr->BallZPos = BallZPosition;

	tptr->BallCarrier = BallCarrier;


/** adding 1 makes NOBODY somebody		*/
	tptr->Controller0 = ControlWhichPlayer[0]+1;
	tptr->Controller1 = ControlWhichPlayer[1]+1;
	tptr->Controller2 = ControlWhichPlayer[2]+1;
	tptr->Controller3 = ControlWhichPlayer[3]+1;
	tptr->Controller4 = ControlWhichPlayer[4]+1;
	tptr->Controller5 = ControlWhichPlayer[5]+1;
	tptr->Controller6 = ControlWhichPlayer[6]+1;
	tptr->Controller7 = ControlWhichPlayer[7]+1;

}



struct oddfIRPframedata	// for 22 players
{
	UWORD Facing:4;			// bit fields
	UWORD GraphicSeqNum:4;
	UWORD GraphicNum:6;
	UWORD extrabits:2;

//	UWORD wholething;
};


void Save1OddIRPFrame(struct frameIRPData *tptr)
{
struct oddfIRPframedata tempIRP;
UWORD *wptr = (UWORD *)&tempIRP;

	for(gPNum=0; gPNum <22; gPNum++)
	{	// save only graphic attributes
		tempIRP.Facing = CPFacing;
		tempIRP.GraphicNum = CPGraphicState;
		tempIRP.GraphicSeqNum = CPGraphicSequence;


//		tptr->OddFrameData[gPNum] = tempIRP.wholething;
		tptr->OddFrameData[gPNum] = *wptr;

	}

/*  new 10/14/96 lower 4 bits of BallFlag is "don't care" for graphic engine
	put BallDir in those 4 bits		*/
	tptr->OddBallFlags = (BallFlags&0xfff0) | (BallDirection&0x000f);
	tptr->OddBallCarrier = BallCarrier;

}







void Save1IRPFrame(UWORD framenumber)
{	// savings the easy part, restorings a bitch
struct frameIRPData *tptr;



	tptr = IRPptr + (framenumber/2);

	if(framenumber&1)
	{
		Save1OddIRPFrame(tptr);

	}
	else	// even frame
	{
		Save1EvenIRPFrame(tptr);
	}
}



#else // old non-interpolated structure


void Save1IRPFrame(struct frameIRPData *tptr)
{
struct IRPData tempIRP;
void *sptr, *dptr;

	for(gPNum=0; gPNum <22; gPNum++)
	{
		tempIRP.XPos = CPXPos>>4;
		tempIRP.YPos = CPYPos>>4;
		tempIRP.ZPos = CPZPos>>3;
		tempIRP.Facing = CPFacing;
		tempIRP.GraphicNum = CPGraphicState;
		tempIRP.GraphicSeqNum = CPGraphicSequence;

		sptr = &tempIRP;
		dptr = &(tptr->PlayerIRPData[gPNum][0]);

		CopyBytes(sptr, dptr, 6); // they're all WORDS

	}

/*  new 10/14/96 lower 4 bits of BallFlag is "don't care" for graphic engine
	put BallDir in those 4 bits		*/
	tptr->BallFlags = (BallFlags&0xfff0) | (BallDirection&0x000f);
//	tptr->BallFlags = BallFlags;

	tptr->BallXPos = BallXPosition;
	tptr->BallYPos = BallYPosition;
	tptr->BallZPos = BallZPosition;

	tptr->BallCarrier = BallCarrier;


/** adding 1 makes NOBODY somebody		*/
	tptr->Controller0 = ControlWhichPlayer[0]+1;
	tptr->Controller1 = ControlWhichPlayer[1]+1;
	tptr->Controller2 = ControlWhichPlayer[2]+1;
	tptr->Controller3 = ControlWhichPlayer[3]+1;
	tptr->Controller4 = ControlWhichPlayer[4]+1;
	tptr->Controller5 = ControlWhichPlayer[5]+1;
	tptr->Controller6 = ControlWhichPlayer[6]+1;
	tptr->Controller7 = ControlWhichPlayer[7]+1;

}


#endif



/*--------------------------------------------------------------------*/

void Restore1EvenIRPFrame(struct frameIRPData *tptr)
{
struct IRPData tempIRP;
void *sptr, *dptr;

	for(gPNum=0; gPNum <22; gPNum++)
	{
		sptr = &(tptr->PlayerIRPData[gPNum][0]);
		dptr = &tempIRP;

		CopyBytes(sptr, dptr, 6); // they're all WORDS

		CPXPos = tempIRP.XPos<<4;
		CPYPos = tempIRP.YPos<<4;
		CPZPos = tempIRP.ZPos<<3;
		CPFacing = tempIRP.Facing;
		CPGraphicState = tempIRP.GraphicNum;
		CPGraphicSequence = tempIRP.GraphicSeqNum;
	}





  /* restore player stuff	*/
#if 0	//NEW_IRP_STRUCT
	for(gPNum=0; gPNum <22; gPNum++)
	{
		CPXPos = tptr->PlayerIRPData[gPNum].XPos<<4;
		CPYPos = tptr->PlayerIRPData[gPNum].YPos<<4;
		CPZPos = tptr->PlayerIRPData[gPNum].ZPos<<3;
		CPFacing = tptr->PlayerIRPData[gPNum].Facing;
		CPGraphicState = tptr->PlayerIRPData[gPNum].GraphicNum;
		CPGraphicSequence = tptr->PlayerIRPData[gPNum].GraphicSeqNum;
	}
//#else
	for(gPNum=0; gPNum <22; gPNum++)
	{
		CPXPos = tptr->PlayerIRPData[gPNum].XPos;
		CPYPos = tptr->PlayerIRPData[gPNum].YPos;
		CPZPos = tptr->PlayerIRPData[gPNum].ZPos;
		CPFacing = tptr->PlayerIRPData[gPNum].Facing;
		CPGraphicState = tptr->PlayerIRPData[gPNum].GraphicNum;
		CPGraphicSequence = tptr->PlayerIRPData[gPNum].GraphicSeqNum;
	}
#endif
  /* restore Ball, other stuff	*/
	BallCarrier = tptr->BallCarrier;

/*  new 10/14/96 lower 4 bits of BallFlag is "don't care" for graphic engine
	put BallDir in those 4 bits		*/
	BallFlags = ((tptr->BallFlags)&0xfff0);
	BallDirection  =  ((tptr->BallFlags)&0x000f);

	BallXPosition = tptr->BallXPos;
	BallYPosition = tptr->BallYPos;
	BallZPosition = tptr->BallZPos;

#if 0	// debug only
	if(BallCarrier != -1)
	{
		if( (OPXPos(BallCarrier) != BallXPosition)
		 || (OPYPos(BallCarrier) != BallYPosition) )
		{
	#if		PC_DEBUG_MODE
			printf("\n IRP Error BCx = %x, BallX = %x", OPXPos(BallCarrier),BallXPosition);
	#endif
			BallXPosition = BallXPosition;	///bp here if PSX
		}
	}
#endif

/*  need to save evey bit in IRP data, so we need a little extra code here
	to handle players 0-22 AND player -1 (NOBODY)
*/
//#define conmacro(num)	cwp = tptr->Controller##num;	if(cwp != -1) cwp = (UWORD)cwp;	ControlWhichPlayer[num] = cwp
//#define conmacro(num)	cwp = tptr->Controller##num;	if(cwp == 31) cwp = -1;	ControlWhichPlayer[num] = cwp
#define conmacro(num)	ControlWhichPlayer[num] = tptr->Controller##num -1
	conmacro(0);
	conmacro(1);
	conmacro(2);
	conmacro(3);
	conmacro(4);
	conmacro(5);
	conmacro(6);
	conmacro(7);


}


void Restore1OddIRPFrame(struct frameIRPData *tptr0,struct frameIRPData *tptr1)
{
struct IRPData irp0,irp1;
LONG t0,t1;
void *sptr, *dptr;

struct oddfIRPframedata tempIRP;
UWORD *wptr = (UWORD *)&tempIRP;

	for(gPNum=0; gPNum <22; gPNum++)
	{
		// graphics in this frame - straight method - no averaging

//		tempIRP.wholething = tptr0->OddFrameData[gPNum]; union didn't work
		*wptr = tptr0->OddFrameData[gPNum];	// wptr points to 1 UWORD of bitfields

		CPFacing = tempIRP.Facing;
		CPGraphicState = tempIRP.GraphicNum;
		CPGraphicSequence =	tempIRP.GraphicSeqNum;


		sptr = &(tptr0->PlayerIRPData[gPNum][0]);
		dptr = &irp0;
		CopyBytes(sptr, dptr, 6); // they're all WORDS

		sptr = &(tptr1->PlayerIRPData[gPNum][0]);
		dptr = &irp1;
		CopyBytes(sptr, dptr, 6); // they're all WORDS

		t0 = irp0.XPos<<4;
		t1 = irp1.XPos<<4;
		CPXPos = (t0+t1)/2;

		t0 = irp0.YPos<<4;
		t1 = irp1.YPos<<4;
		CPYPos = (t0+t1)/2;

		t0 = irp0.ZPos<<3;
		t1 = irp1.ZPos<<3;
		CPZPos = (t0+t1)/2;
	}

  /* restore Ball, other stuff	*/
	BallCarrier = tptr0->OddBallCarrier;

/*  new 10/14/96 lower 4 bits of BallFlag is "don't care" for graphic engine
	put BallDir in those 4 bits		*/
	BallFlags = ((tptr0->OddBallFlags)&0xfff0);
	BallDirection  =  ((tptr0->OddBallFlags)&0x000f);


// average the 2 ball parameters
	t0 = tptr0->BallXPos;
	t1 = tptr1->BallXPos;
	BallXPosition = (t0+t1)/2;

	t0 = tptr0->BallYPos;
	t1 = tptr1->BallYPos;
	BallYPosition = (t0+t1)/2;

	t0 = tptr0->BallZPos;
	t1 = tptr1->BallZPos;
	BallZPosition = (t0+t1)/2;



}











/*-------------------------------------------------------------------*/

void Save1FullIRPFrame(struct fullframeIRPData *tptr)
{	// called at init of IRP only to save all (not shifted) true data
	for(gPNum=0; gPNum <22; gPNum++)
	{
		tptr->PlayerIRPData[gPNum].XPos = CPXPos;
		tptr->PlayerIRPData[gPNum].YPos = CPYPos;
		tptr->PlayerIRPData[gPNum].ZPos = CPZPos;
		tptr->PlayerIRPData[gPNum].Facing = CPFacing;
		tptr->PlayerIRPData[gPNum].GraphicNum = CPGraphicState;
		tptr->PlayerIRPData[gPNum].GraphicSeqNum = CPGraphicSequence;
	}

/*  new 10/14/96 lower 4 bits of BallFlag is "don't care" for graphic engine
	put BallDir in those 4 bits		*/
	tptr->BallFlags = (BallFlags&0xfff0) | (BallDirection&0x000f);
//	tptr->BallFlags = BallFlags;

	tptr->BallXPos = BallXPosition;
	tptr->BallYPos = BallYPosition;
	tptr->BallZPos = BallZPosition;

	tptr->BallCarrier = BallCarrier;


/** adding 1 makes NOBODY somebody		*/
	tptr->Controller0 = ControlWhichPlayer[0]+1;
	tptr->Controller1 = ControlWhichPlayer[1]+1;
	tptr->Controller2 = ControlWhichPlayer[2]+1;
	tptr->Controller3 = ControlWhichPlayer[3]+1;
	tptr->Controller4 = ControlWhichPlayer[4]+1;
	tptr->Controller5 = ControlWhichPlayer[5]+1;
	tptr->Controller6 = ControlWhichPlayer[6]+1;
	tptr->Controller7 = ControlWhichPlayer[7]+1;

}




void Restore1FullIRPFrame(struct fullframeIRPData *tptr)
{
	for(gPNum=0; gPNum <22; gPNum++)
	{
		CPXPos = tptr->PlayerIRPData[gPNum].XPos;
		CPYPos = tptr->PlayerIRPData[gPNum].YPos;
		CPZPos = tptr->PlayerIRPData[gPNum].ZPos;
		CPFacing = tptr->PlayerIRPData[gPNum].Facing;
		CPGraphicState = tptr->PlayerIRPData[gPNum].GraphicNum;
		CPGraphicSequence = tptr->PlayerIRPData[gPNum].GraphicSeqNum;
	}
  /* restore Ball, other stuff	*/
	BallCarrier = tptr->BallCarrier;

/*  new 10/14/96 lower 4 bits of BallFlag is "don't care" for graphic engine
	put BallDir in those 4 bits		*/
	BallFlags = ((tptr->BallFlags)&0xfff0);
	BallDirection  =  ((tptr->BallFlags)&0x000f);

	BallXPosition = tptr->BallXPos;
	BallYPosition = tptr->BallYPos;
	BallZPosition = tptr->BallZPos;


/*  need to save evey bit in IRP data, so we need a little extra code here
	to handle players 0-22 AND player -1 (NOBODY)
*/
	conmacro(0);
	conmacro(1);
	conmacro(2);
	conmacro(3);
	conmacro(4);
	conmacro(5);
	conmacro(6);
	conmacro(7);


}



/*---  following called in normal game loop to save RePlay data ----*/

void SaveInstantReplayData(void)
{
//struct frameIRPData *tptr;

	if(IRPptr == NULL)
		return;

	// start saving only after ball hiked
	if(BallSnapped)
	{
		if( ( (!PlayHappening()) && (EndIRPFrameCounter < StartIRPFrameCounter))
		  || (EventFlags.IRP_INITED_BFLAG)  )
		{	// stop if playover and it took a long time
			// or if IRP was already run
		}
		else
		{
			if(CurIRPFrameCounter >= MAX_IRP_FRAMES)
			{  //extra safety check
				CurIRPFrameCounter = ZERO;
			}


//			tptr = IRPptr+CurIRPFrameCounter;
//			Save1IRPFrame(tptr);
			Save1IRPFrame(CurIRPFrameCounter);


			if(++CurIRPFrameCounter >= MAX_IRP_FRAMES)
			{  //we went over max allowable time, wrap around
				CurIRPFrameCounter = ZERO;
			}

			EndIRPFrameCounter = CurIRPFrameCounter;

//			if(EndIRPFrameCounter <= StartIRPFrameCounter)
			// 11/19/96 if caught up - reset start (IRP won't show beginning of play)
			if(EndIRPFrameCounter == StartIRPFrameCounter)
			{
				StartIRPFrameCounter = EndIRPFrameCounter + 1;
				if(StartIRPFrameCounter >= MAX_IRP_FRAMES)
					StartIRPFrameCounter = ZERO;
			}
		}
	}
}




void SaveIRPBallTarget(void)
{	// called at play init or when ever BT is valid
	OneTimeIRP.BallTargetX = BallTargetX;
	OneTimeIRP.BallTargetY = BallTargetY;
}


#if	0	//PC_DEBUG_MODE
void TestIRP(void)
{
WORD i;
long tsize;


	ControlWhichPlayer[0] = 1;
	ControlWhichPlayer[1] = NOBODY;
	ControlWhichPlayer[2] = 16;
	ControlWhichPlayer[3] = -2;
	ControlWhichPlayer[4] = 21;
	ControlWhichPlayer[5] = 25;
	ControlWhichPlayer[6] = 4;
	ControlWhichPlayer[7] = 31;
	BallCarrier = -1;

//	Save1IRPFrame(IRPptr);
	Save1IRPFrame(0);
	Save1IRPFrame(1);
//	Restore1IRPFrame(IRPptr);

	printf("\nsize of IRP = %d",sizeof(struct frameIRPData));
//	tsize = sizeof(struct frameIRPData)*15*15;
//	printf("\nTotal size of IRP buffer if 15 sec = %ld",tsize);
//	tsize = sizeof(struct frameIRPData)*15*10;

	tsize = IRP_ALLOC_SIZE;
	printf("\nCurrent size  IRP buffer = %ld",tsize);

	tsize = (IRP_ALLOC_SIZE/sizeof(struct frameIRPData));
	printf("\nactual 'double' frames of IRP = %ld",tsize);

	tsize = (IRP_ALLOC_SIZE/sizeof(struct frameIRPData)*2);
	printf("\ntrue # of even/odd frames of IRP = %ld",tsize);

	printf("\nseconds of IRP = %ld",tsize/20);

	printf("\n");

	for(i=0; i<8; i++)
		printf("\nControlWhichPlayer[%d] = %d",i,ControlWhichPlayer[i]);
	printf("\nBallCarrier = %d",BallCarrier);


}

#endif


void SetMemory(void *area, BYTE setto, UWORD n);	// in fbstats.c

/*--------------------------------------------------------------------*/

/*   set aside memory for the Instant replay stuff	*/

void AllocAIMemory(void)
{

	/* set aside memory for saving Instant Replay Data	*/
	/*  as a precaution - allocate a little extra		*/
	if ( IRPptr ==  NULL)
	{
		IRPptr = (struct frameIRPData *) AllocMem(IRP_ALLOC_SIZE+200);
	}

	if(IRPptr != NULL)	// clear just to be safe
	{
		SetMemory(IRPptr,0,IRP_ALLOC_SIZE);
	}
#if PC_ALWAYS_MODE
	else
	{
		printf("\nError on IRP allocation");
//		getch();
	}
#endif
}

void FreeAIMemory(void)
{
	if ( IRPptr !=  NULL)
	{
		FreeMem(IRPptr);
		IRPptr = NULL;
	}
}

/****************************  End of file: FBIrp.c  *********************/


