/*************************************************************


					FBRAMDAT.H

	AUTHOR: M. Knauer				Start Date: 3/21/95
	Modify Date:


	Copyright 1995  GameTek, Inc.



	This file contains all A LIST of global variables.


	It is used [at least] 2 places.


	In FBRAM.C as actual variables.

	As an INCLUDE file FBRAM.H as "extern" variables.


	The exact file can be used because of the use of a conditionally
	defined macro(s) VAR, VAREQU, and VARSTRUCT which differ
	depending on the equate REALRAM_C.  (See FBEQU.H)

	If REALRAM_C is defined, the variable is a "true" variable (in
	FBRAM.C).  If REALRAM_C is undefined, the variable is an
	external (in FBRAM.H)

	The reason for this is to only have to type in the variable
	once and it will be used in both places.


	Major Variables which are initialized (such as play names) are
	usually in their own files.

	Also see "FB_GRAM.H"  - an include file with all of these
	variables, plus others, as EXTERN.

**************************************************************/

//	#include	"fbequ.h"


//#define	NUM_PLAYER_SLOTS	(NUM_PLAYERS_ON_FIELD + 2)


// last slot is "dummy" slot used in "WhereWillPlayerBe()"
// next to last slot is REFEREE_SLOT

/*******************************************************************

			R A M   DEFINE  MACRO

	The following MACRO allows the programmer to define a RAM
	variable once and then use it in two places
	 1) the actual .c file
	 2) an included file (in a code file) used for external references


**********************************************************************/



#ifdef	REALRAM_C		/* treat as actual variable	*/

	#define	VAR(type,name) type name
/* new 8/8/96  don't let comiler init anything - this makes too many
   assumptions about AI varibales
*/
	#define	VAREQU(type,name,equ) type name = equ
//	#define	VAREQU(type,name,equ) type name
	#define	VARSTRUCT(aaa,bbb) struct	aaa	bbb
	#define	VARSTRUCTEQU(aaa,bbb,ccc) struct	aaa	bbb= {ccc};
#else					/* treat as external variable	*/

#define	VAR(type, name) extern	type name
#define	VAREQU(type, name, equ) extern	type name
#define	VARSTRUCT(aaa,bbb) extern	struct	aaa	bbb
#define	VARSTRUCTEQU(aaa,bbb,ccc) extern	struct	aaa	bbb

#endif






/*---------------  Player Data Structures  -----------*/

	/*--- Player D.S. inited and used during play	----*/

VAR	(UWORD,PlayerXPosition[NUM_PLAYER_SLOTS]);
VAR	(UWORD,PlayerYPosition[NUM_PLAYER_SLOTS]);
VAR	(UWORD,PlayerZPosition[NUM_PLAYER_SLOTS]);

#ifdef FRAME_RATE_30HERTZ

VAR	(UWORD,PlayerOldXPosition[NUM_PLAYER_SLOTS]);
VAR	(UWORD,PlayerOldYPosition[NUM_PLAYER_SLOTS]);
VAR	(UWORD,PlayerOldZPosition[NUM_PLAYER_SLOTS]);
#endif

VAR	(WORD,PlayerCurrentSpeed[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerBurstCounter[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerCPUBurstTimer[NUM_PLAYER_SLOTS]);
//VAR	(WORD,PBackPedalSpeed[NUM_PLAYER_SLOTS]);

VAR	(WORD,PlayerMyFormPosition[NUM_PLAYER_SLOTS]);	/* position based on formation	*/
VAR	(WORD,PlayerMyPlayerNum[NUM_PLAYER_SLOTS]);	/* team player number 0-47	*/

VAR	(UWORD,PlayerTargetX[NUM_PLAYER_SLOTS]);
VAR	(UWORD,PlayerTargetY[NUM_PLAYER_SLOTS]);

VAR	(WORD,PlayerDeltaX[NUM_PLAYER_SLOTS]);	// used for 360 degree movement (dives)
VAR	(WORD,PlayerDeltaY[NUM_PLAYER_SLOTS]);	// used for 360 degree movement (dives)

VAR	(WORD,PlayerDirToGo[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerFacing[NUM_PLAYER_SLOTS]);

VAR	(WORD,PlayerFlags[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerFlags1[NUM_PLAYER_SLOTS]);	// need a lot of PFlags
VAR	(WORD,PlayerTimer[NUM_PLAYER_SLOTS]);

VAR	(WORD,PlayerStateInit[NUM_PLAYER_SLOTS]); //EFFVAR didn't help
VAR	(WORD,PlayerDirToPush[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerState2Count[NUM_PLAYER_SLOTS]);

VAR	(WORD,PlayerBlockingMe[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerPushFactor[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerGuyToBlock[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerGuyToCover[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerKeyDistance[NUM_PLAYER_SLOTS]);

/* following receiver/passing parameters inited in InitRecParametersSTATE */
VAR	(WORD,PlayerOptimalPassTime[MAX_RECEIVER_SLOTS]);	// only need MAX RECEIVERS
VAR	(WORD,CPUPassType[MAX_RECEIVER_SLOTS]);	// type of pass CPU QB should throw to each rec.


VAR	(WORD,PlayerGraphicState[NUM_PLAYER_SLOTS]);	/* i.e. RUNNING, THROWING */
VAR	(WORD,PlayerGraphicSequence[NUM_PLAYER_SLOTS]);	/* frame number	*/
VAR	(WORD,PlayerGraphicCounter[NUM_PLAYER_SLOTS]);	/* counter to tell when to inc PGSeq	*/
VAR	(WORD,PlayerAIGraphicState[NUM_PLAYER_SLOTS]);	/* i.e. RUNNING, THROWING */
VAR	(WORD,PlayerAIGraphicSequence[NUM_PLAYER_SLOTS]);	/* frame number	*/

VAR	(WORD,PlayerControllerNum[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerNumSpinMoves[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerNumSpeedBursts[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerTackleCount[NUM_PLAYER_SLOTS]);

VAR	(WORD,*PlayerScriptPtr[NUM_PLAYER_SLOTS]);
VARSTRUCT	(State,PlayerAction[NUM_PLAYER_SLOTS]);
VARSTRUCT	(State,PSmarts[NUM_PLAYER_SLOTS]);
VARSTRUCT	(BlockerState,*PlayerBlockState[NUM_PLAYER_SLOTS]);


// keep track of jersey nums during play as opposed to figuing out each time
VAR	(BYTE,PlayerJerseyNum[NUM_PLAYER_SLOTS]);


//WORD	PlayerSphereInfluenceXLow[NUM_PLAYER_SLOTS];
//WORD	PlayerSphereInfluenceXHigh[NUM_PLAYER_SLOTS];
//WORD	PlayerSphereInfluenceYLow[NUM_PLAYER_SLOTS];
//WORD	PlayerSphereInfluenceYHigh[NUM_PLAYER_SLOTS];


VARSTRUCT	(uBox,PlayerSphereInfluence[NUM_PLAYER_SLOTS]);
VARSTRUCT	(uXY,PlayerPosition[NUM_PLAYER_SLOTS]);

VARSTRUCT	(uBox,PlayerInsideZone[NUM_PLAYER_SLOTS]);
VARSTRUCT	(uBox,PlayerOutsideZone[NUM_PLAYER_SLOTS]);
VARSTRUCT	(uXY,PlayerCenterZone[NUM_PLAYER_SLOTS]);	/* of Inside zone	*/


VARSTRUCT	(BKzonearea, *gZonePtr);
VAR	(WORD,PlayerReactTime[NUM_PLAYER_SLOTS]);


/*---- Player Abilities (inited at play start and never changed --------*/

VAR	(WORD,PlayerMaxSpeed[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerMinMaxSpeed[NUM_PLAYER_SLOTS]);	// used when BC gets tired
VAR	(WORD,PlayerAcceleration[NUM_PLAYER_SLOTS]);

VAR	(WORD,PlayerStrength[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerReaction[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerHands[NUM_PLAYER_SLOTS]);          // 0 -100


VAR	(WORD,PlayerArmStrength[NUM_PLAYER_SLOTS]); 	// 0 -100
VAR	(WORD,PlayerFootStrength[NUM_PLAYER_SLOTS]);   // 0 -100
//VAR	(WORD,PlayerMiscRating[NUM_PLAYER_SLOTS]);	//different depending on position
VAR	(WORD,PlayerAgility[NUM_PLAYER_SLOTS]);	//different depending on position
//	struct playerabilities Ratings;

VAR	(WORD,PlayerRunBlockRating[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerPassBlockRating[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerTackleRating[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerRushRating[NUM_PLAYER_SLOTS]);

/* following used to help optimize collision detect -all players are sorted
	on Y, then we start from current backwards until guy out of range
	and then forward until guy out of range, hopefully we only check
	2-3 guys instead of all other 21
****/

VAR	(WORD,PlayersYSortList[NUM_PLAYER_SLOTS]);
VAR	(WORD,PlayerYSortPos[NUM_PLAYER_SLOTS]);  // where a player is in sort list

#define	OPYSortPos(opnum) 	(PlayerYSortPos[opnum])


/********************************************************************

	P L A Y E R    D A T A   S T R U C T U R E   E Q U A T E S

	The FB Player Data Structures are generally defined so as to
	be able to modify the specific data structure if needed
	(E.G. changing from an array based DS to a player structure DS)

	In most cases in the AI, the datastructure CPxxx is used where
	"CP" means CURRENT PLAYER - the CURRENT portion is assumed to have
	been set-up in the main AI loop.

********************************************************************/

#ifndef _PLAYER_DS_EQUS_
#define	CPXPos	(PlayerXPosition[gPNum])
#define	CPYPos	(PlayerYPosition[gPNum])
#define	CPZPos	(PlayerZPosition[gPNum])

#ifdef FRAME_RATE_30HERTZ
#define	CPOldXPos	(PlayerOldXPosition[gPNum])
#define	CPOldYPos	(PlayerOldYPosition[gPNum])
#define	CPOldZPos	(PlayerOldZPosition[gPNum])
#else
#define	CPOldXPos	(PlayerXPosition[gPNum])
#define	CPOldYPos	(PlayerYPosition[gPNum])
#define	CPOldZPos	(PlayerZPosition[gPNum])
#endif

#define	CPSpeed			(PlayerCurrentSpeed[gPNum])
#define	CPBurstCounter	(PlayerBurstCounter[gPNum])
#define	CPCPUBurstTimer (PlayerCPUBurstTimer[gPNum])

/*  constant ratings	*/
#define	CPStrength		(PlayerStrength[gPNum])
#define	CPMaxSpeed		(PlayerMaxSpeed[gPNum])
#define	CPMinMaxSpeed	(PlayerMinMaxSpeed[gPNum])
#define	CPAcceleration	(PlayerAcceleration[gPNum])
#define	CPReaction		(PlayerReaction[gPNum])
#define	CPHands			(PlayerHands[gPNum])          	// 0 -100
#define	CPArmStrength	(PlayerArmStrength[gPNum])	// 0 -100
#define	CPFootStrength	(PlayerFootStrength[gPNum]) // 0 -100
//#define	CPMiscRating	(PlayerMiscRating[gPNum])
#define	CPAgility	(PlayerAgility[gPNum]) // 0 -100

#define	CPRunBlockRating	(PlayerRunBlockRating[gPNum])
#define	CPPassBlockRating	(PlayerPassBlockRating[gPNum])
#define	CPTackleRating		(PlayerTackleRating[gPNum])
#define	CPRushRating		(PlayerRushRating[gPNum])



#define	CPJerseyNum		(PlayerJerseyNum[gPNum])


#define	CPPassStrength	CPArmStrength

#define	CPMyPNum 	(PlayerMyPlayerNum[gPNum])	/* playernum ie. QB, HB	*/
#define	CPMyFormPosition (PlayerMyFormPosition[gPNum])	/* formation ie. QB, HB	*/
#define	CPReactTime		(PlayerReactTime[gPNum])


#define	CPTargetX	(PlayerTargetX[gPNum])
#define	CPTargetY	(PlayerTargetY[gPNum])

#define	CPDeltaX	(PlayerDeltaX[gPNum])	// used for 360 degree movement (dives)
#define	CPDeltaY	(PlayerDeltaY[gPNum])	// used for 360 degree movement (dives)

#define	CPDirToGo	(PlayerDirToGo[gPNum])
#define	CPFacing	(PlayerFacing[gPNum])

#define	CPFlags	(PlayerFlags[gPNum])
#define	CPFlags1	(PlayerFlags1[gPNum])
#define	CPTimer	(PlayerTimer[gPNum])

#define	CPStateInit	(PlayerStateInit[gPNum])
#define	CPDirToPush	(PlayerDirToPush[gPNum])
#define	CPState2Count	(PlayerState2Count[gPNum])

#define	CPBlockingMe	(PlayerBlockingMe[gPNum])
#define	CPPushFactor	(PlayerPushFactor[gPNum])
#define	CPGuyToBlock	(PlayerGuyToBlock[gPNum])
#define	CPGuyToCover	(PlayerGuyToCover[gPNum])
#define	CPKeyDistance	(PlayerKeyDistance[gPNum])

#define	CPTackleCount	(PlayerTackleCount[gPNum])

#define	CPGraphicState	(PlayerGraphicState[gPNum])	/* i.e. RUNNING, THROWING */
#define	CPGraphicSequence	(PlayerGraphicSequence[gPNum])	/* frame number	*/
#define	CPAIGraphicCounter	(PlayerGraphicCounter[gPNum])	/* counter to tell when to inc PGSeq	*/
#define	CPAIGraphicState	(PlayerAIGraphicState[gPNum])	/* i.e. RUNNING, THROWING */
#define	CPAIGraphicSequence	(PlayerAIGraphicSequence[gPNum])	/* frame number	*/

#define	CPAction	(PlayerAction[gPNum].todo)
#define	CPScriptPtr	(PlayerScriptPtr[gPNum])
#define	CPAIState	(PSmarts[gPNum].todo)
#define	CPBStatePtr	(PlayerBlockState[gPNum])
#define	CPControllerNum	(PlayerControllerNum[gPNum])
#define	CPNumSpinMoves 	(PlayerNumSpinMoves[gPNum])
#define	CPNumSpeedBursts (PlayerNumSpeedBursts[gPNum])

#define	CPOutsideZone  PlayerOutsideZone[gPNum]
#define	CPInsideZone	PlayerInsideZone[gPNum]
#define	CPCenterZone	PlayerCenterZone[gPNum]	/* of Inside zone	*/


#define	OPReactTime(opnum)		(PlayerReactTime[opnum])




/*  "other" player data structure equates ( any of the players)	*/

#define	OPXPos(opnum)	(PlayerXPosition[opnum])
#define	OPYPos(opnum)	(PlayerYPosition[opnum])
#define	OPZPos(opnum)	(PlayerZPosition[opnum])

#ifdef FRAME_RATE_30HERTZ
#define	OPOldXPos(opnum)	(PlayerOldXPosition[opnum])
#define	OPOldYPos(opnum)	(PlayerOldYPosition[opnum])
#define	OPOldZPos(opnum)	(PlayerOldZPosition[opnum])
#else
#define	OPOldXPos(opnum)	(PlayerXPosition[opnum])
#define	OPOldYPos(opnum)	(PlayerYPosition[opnum])
#define	OPOldZPos(opnum)	(PlayerZPosition[opnum])
#endif

#define	OPSpeed(opnum)	(PlayerCurrentSpeed[opnum])

#define	OPStrength(opnum)		(PlayerStrength[opnum])
#define	OPMaxSpeed(opnum)		(PlayerMaxSpeed[opnum])
#define	OPAcceleration(opnum)	(PlayerAcceleration[opnum])
#define	OPReaction(opnum)		(PlayerReaction[opnum])
#define	OPHands(opnum) 			(PlayerHands[opnum])
#define	OPArmStrength(opnum) 	(PlayerArmStrength[opnum])
#define	OPKickStrength(opnum) 	(PlayerFootStrength[opnum])
//#define	OPMiscRating(opnum) 	(PlayerMiscRating[opnum])
#define	OPAgility(opnum)		(PlayerAgility[opnum])


#define	OPRunBlockRating(opnum)		(PlayerRunBlockRating[opnum])
#define	OPPassBlockRating(opnum)	(PlayerPassBlockRating[opnum])
#define	OPTackleRating(opnum)		(PlayerTackleRating[opnum])
#define	OPRushRating(opnum)			(PlayerRushRating[opnum])


#define	OPMyFormPosition(opnum) (PlayerMyFormPosition[opnum])	/* position, ie. QB, HB	*/
#define	OPMyPNum(opnum) 		(PlayerMyPlayerNum[opnum])	/* position, ie. QB, HB	*/


#define	OPTargetX(opnum)	(PlayerTargetX[opnum])
#define	OPTargetY(opnum)	(PlayerTargetY[opnum])

#define	OPDirToGo(opnum)	(PlayerDirToGo[opnum])
#define	OPFacing(opnum)	(PlayerFacing[opnum])

#define	OPFlags(opnum)	(PlayerFlags[opnum])
#define	OPFlags1(opnum)	(PlayerFlags1[opnum])
#define	OPTimer(opnum)	(PlayerTimer[opnum])

#define	OPStateInit(opnum)	(PlayerStateInit[opnum])


#define	OPDirToPush(opnum)	(PlayerDirToPush[opnum])
#define	OPState2Count(opnum)	(PlayerState2Count[opnum])
#define	OPControllerNum(opnum)	(PlayerControllerNum[opnum])

#define	OPBlockingMe(opnum)	(PlayerBlockingMe[opnum])
#define	OPPushFactor(opnum)	(PlayerPushFactor[opnum])
#define	OPGuyToBlock(opnum)	(PlayerGuyToBlock[opnum])
#define	OPGuyToCover(opnum)	(PlayerGuyToCover[opnum])
#define	OPKeyDistance(opnum)	(PlayerKeyDistance[opnum])
#define	OPOptimalPassTime(opnum) (PlayerOptimalPassTime[opnum]);	// only need MAX RECEIVERS


#define	OPGraphicState(opnum)	(PlayerGraphicState[opnum])	/* i.e. RUNNING, THROWING */
#define	OPGraphicSequence(opnum)	(PlayerGraphicSequence[opnum])	/* frame number	*/
#define	OPAIGraphicState(opnum)	(PlayerAIGraphicState[opnum])	/* i.e. RUNNING, THROWING */
#define	OPAIGraphicSequence(opnum)	(PlayerAIGraphicSequence[opnum])	/* frame number	*/


#define	OPAction(opnum)	(PlayerAction[opnum].todo)
#define	OPScriptPtr(opnum)	(PlayerScriptPtr[opnum])
#define	OPAIState(opnum)	(PSmarts[opnum].todo)
#define	OPBStatePtr(opnum)	(PlayerBlockState[opnum])


#define	OPJerseyNum(opnum)	(PlayerJerseyNum[opnum])


#define _PLAYER_DS_EQUS_
#endif




/***********************  BALL   VARIABLES	*************************/

VAR	(UWORD,PracticeStartBallX);
VAR	(UWORD,StartBallX);
VAR	(UWORD,StartBallY);

VAR	(UWORD,CatchBallX);	// where def (or K.Returner) got ball, used for stats
VAR	(UWORD,FumbleX);	// where ball was fumbled, used for stats
VAR	(UWORD,PossibleFumbleX);	// set when ball pitched, used if RB doesn't get it
VAR	(UWORD,FumbleRecoveryX);	// where ball was recovered, used for stats
VAR (WORD,NumFumblesOnPlay);	// fumble counter

VAR	(UWORD,BallXPosition);
VAR	(UWORD,BallYPosition);
VAR (WORD,BallZPosition);

//#ifdef FRAME_RATE_30HERTZ
VAR	(UWORD,OldBallXPosition);
VAR	(UWORD,OldBallYPosition);
VAR (WORD,OldBallZPosition);
//#endif

VAR (WORD,BallDirection);	// used mainly for graphic display
VAR (WORD,BallSpeed);	// used in QuickFigureBallMovement


// where the ball is going if it is thrown...
VAR	(UWORD, BallTargetX);
VAR	(UWORD, BallTargetY);

VAR	(WORD,BallDeltaX);
VAR (WORD,BallDeltaY);

VAR	(WORD,PassType); 	// actual pass type - affects BallSpeed
VAR	(WORD,PassCounter);		/* used to track how long receiver button is pressed	*/
VAR	(WORD,BallFlags);
VAR (WORD,NumDeflections);	// per plays
VAR (WORD,NumBallBounces);	// per plays

VAR (WORD,PointsInBallArc);	// used in QuickFigureBallMovement
VAR	(WORD, BallArcCounter);
VAR	(WORD, BallArcCounterDir);
VAR (WORD, 	MaxPointsInBallCursor);

VAR (WORD,ReplotBallArcCounter);

VAR (WORD,CurrentBallGravity);
VAR (WORD,InitialBallGravity);
VAR (WORD,CurrentBallDeltaZ);
VAR (WORD,ConstantBallDeltaZ);
VAR (WORD,GenValue);

#if	0	//intial stuff
VAREQU (WORD,BV1,4);
VAREQU (WORD,BV2,1);
VAREQU (WORD,BV3,3*QTR_YARD);
VAREQU (WORD,BV4,1*QTR_YARD);
//#else	// use with 8/15/95 passing
VAREQU (WORD,BV1,1);
VAREQU (WORD,BV2,1);
VAREQU (WORD,BV3,96);
VAREQU (WORD,BV4,16);
#endif

VAR	(UWORD,BTX1);	// ball target testings vars
VAR	(UWORD,BTX2);	// BT1 is highest point at which ball can be caught
VAR	(UWORD,BTY1);	// BT2 is where ball will hit ground
VAR	(UWORD,BTY2);


VAR	(WORD,KickJPadNum);	// for human kicking interface
VAR	(UWORD,MinBallTargetX);
VAR	(UWORD,MaxBallTargetX);
VAR	(UWORD,KickTargetX);	// can't use BallTarget as long hikes blow up
VAR	(UWORD,KickTargetY);	// can't use BallTarget as long hikes blow up
VAR (UWORD,KickStartX);
VAR (UWORD,KickStartZ);		// punt starts higher than other kicks
VAR	(UWORD,MaxKickDistance);
VAR (UWORD,KickOOBX);		// where a punt goes OOB


VAR (UWORD,HikeXPosition);	// for long hikes so we can do concurrent kick arc
VAR (UWORD,HikeYPosition);	// only for HB snap
VAR (UWORD,HikeDeltaY);	// only for HB snap
VAR (UWORD,HikeZPosition);
VAR (WORD,HikeBallDeltaZ);
VAR (UWORD,HikeTargetX);	// ball will begin dropping after reaching this target



VARSTRUCT	(State,BallAction);

VARSTRUCT	(uBox,BallSphereInfluence);

/*-------------  General Global Variables  ---------------------*/

VAR	(WORD,TheDown);
VAR	(WORD,TheQuarter);      // only up to 5
VAR	(WORD,TheTrueQuarter);	// > 5 if overtime in Playoffs
VAR	(UWORD,FirstDownX);
VAR	(UWORD,NextFirstDownX);	// where it could be (if no penalty)

//VAR	(WORD,HandOffX);
//VAR	(WORD,HandOffY);

VAR	(WORD,PlayType);	// run, pass, kickoff , field goal etc
VAR	(WORD,NextPlayType); //set moment play is over, next ->P.T. at play cleanup
VAR	(WORD,LastPlayType); // used for no huddle info

VAR	(WORD,PlayCallStatus); //0 = okay to start play
VAR (WORD,ExtraPointStatus);	// 0- 3 for how human wants extra point (or -1)
VAR	(WORD,KickOffType);	// for CPU-normal, onside, squib (independent of formation)
VAR	(WORD,OnSideKickType);
VAR	(WORD,IllegalKickCount);

//VAR (WORD,EOPSFXPending);		// End OfPlay SFX to play (after whistle)


VAR	(UWORD,AIFrameCounter);
/* actual 60 HZ "timer" inc'ed by S/W at end of main loop
	it uses SECONDS and is inc'ed by TicksPerAIFrame		*/
VAR	(UWORD,AIPlayTimer);
/* below two "timerwaits" used as a compare value to
			 see if AIPlayTimer has reached a certain value */
VAR	(UWORD,AIPlayTimerWait);
VAR	(UWORD,CPUPlayTimerWait);	// CPU team timer (when to hike)
VAR	(UWORD,PosCatchFrame);	// saved during ReleasePass() to help speed def. calcs


VAR (ULONG,TimeOfPoss[2]);

VAR (UWORD,RefMarkerX[2]);	// where the marker guys go

/*  AITickCount  keeps track of internal timing to see which part of AI
	to update - AI run s a 15 Htz, graphics run at 30 Htz	*/

VAR	(UWORD,AITickCount);

VAR	(UWORD,TimeIntoPlay);	// only active after ball hiked
VAR	(UWORD,FumbleFrameCounter);

VAR (WORD,FieldType);
//VAREQU (WORD,FieldCondition,NORMAL_FIELD_CONDITION);
//VAREQU (WORD,WeatherCondition,NORMAL_WEATHER_CONDITION);

VAR (WORD,WindSpeed);
VAR (WORD,WindDeltaX);
VAR (WORD,WindDeltaY);
VAR (WORD,ShowWindFlag);	// set by AI, tells graphic engine to display wind vane
VAR (WORD,WindDirection);		// constant during game
VAR (WORD,AIWindDirection);	//modified for AI depending on FieldDir

VAR	(WORD,OffenseDir);	// direction offense going


VAR	(UWORD,PlayStateNum);
VAR	(UWORD,PlayStateFlags);

/* see FBEQU.H for bit equates of these flags	*/
//VAR	(UWORD,EventFlags);
VARSTRUCT (evflags,EventFlags);
VAR	(UWORD,MoreEventFlags);
VAR	(UWORD,GameFlags);
VAR	(UWORD,PlayOverFlags);
VAR	(UWORD,PlayFlags);
VAR (UWORD,InterfaceFlags);

VAR (BOOL,AutoReplayFlag);
VAR	(WORD,YardsGainedThisPlay);


VAR	(EFFVAR,gPNum);		/* global Player Number	*/

VAR	(EFFVAR,BallCarrier);
VAR	(WORD,LastBallCarrier);	// the "last" guy to have possession of ball
VAR	(WORD,PenaltyBallCarrier);	// used after play for personal fouls
VAR	(WORD,FinalBallCarrier);	// used after play for stats
VAR	(WORD,FutureBC);
VAR	(WORD,CameraGuy);		// if positive, put camera on him
VAR (WORD,Tackler);			// the guy who did the tackling
VAR (WORD,PenaltyGuy);		// the guy the penalty is on
VAR (WORD,PenaltyGuy);		// the guy the penalty is on
VAR (WORD,PersonalFouler);	// the guy the foul
VAR (WORD,BCHitter);		// hopefully human who get p. foul

VAR (WORD,OffPenaltyGuy);		// the guy the penalty is on(offense)
VAR (WORD,DefPenaltyGuy);		// the guy the penalty is on(Defense)
VAR (WORD,OffPenaltyPending);	// type of penalty on offense
VAR (WORD,DefPenaltyPending);	// type of penalty on defense

VAR (WORD,ActivePenalty);		// 1 means accept, 2 means decline
VAR (WORD,PenaltyChooser);		// team slot num if human chosing(-1 if CPU)
VAR (BOOL,PenaltyResolved);		// graphic engine sets true



VAR	(WORD,TheReceiver);		// the "final" receiver
VAR	(WORD,IntendedReceiver); // if incomplete
VAR	(WORD,ThrowAwayRec);	// if trying to throw ball away
VAR	(WORD,Receivers[MAX_RECEIVERS]);	// the 3 current receivers
//VAR	(WORD,PosReceivers[MAX_RECEIVERS]);	// the 5 elligible receiviers
//VAR	(WORD,ToggledReceiver);	// index (2, 3, or 4)
VAR (WORD,PrimaryReceiver);
VAR	(WORD,RecDistToClosestDef[MAX_RECEIVERS]);	// used for icon scaling
VAR (WORD,CPURecIndex[MAX_RECEIVERS]);		// order in which CPU QB throws to recs
VAR (BOOL,ShowReceiverIcons);	/* usually on unless KickOff or play over	*/

VAR	(WORD,DefPassInterferenceCounter);
//VAR	(WORD,DefPassInterferenceGuy);
VAR	(WORD,OffPassInterferenceCounter);
//VAR	(WORD,OffPassInterferenceGuy);	it's "TheReceiver"
VAR  (WORD,QBHandoffFreezeTime);	// var. time to stay on HO anim 


VAR	(WORD,QBEndOfPlayScriptNum);	// what QB does after handing off..WatchEOP or Flea

VAR	(WORD,CurTeamSlot);		/* 0 or 1 depending on who gPNum belongs to 	*/
VAR	(EFFVAR,OffensiveSlotNum);
VAR	(EFFVAR,DefensiveSlotNum);
VAR	(WORD,LastTeamToScoreSlot); //helps with choosing Ex Pt. Question

VAR	(WORD,StartIRPFrameCounter);
VAR	(WORD,CurIRPFrameCounter);
VAR	(WORD,EndIRPFrameCounter);

VAR (WORD,TeamBlockFactor);

VAR (WORD,DefStrutter);	// defensive guy who shows off after play

VAR	(BOOL,BallSnapped);

VAR	(WORD,ControlWhichPlayer[MAX_CONTROLLERS]);
//VAR	(WORD,ControlWhichPlayerPrevPlay[MAX_CONTROLLERS]);
VAR (WORD,DefToggleIndex[MAX_CONTROLLERS]); 		/* -1 means on original guy			 */

VAR	(WORD,AIJoyPadData[MAX_CONTROLLERS]);

VAR	(WORD,PreviousJoyReading[MAX_CONTROLLERS]);
VAR	(BOOL,CollisionDetectActive);

/* Field Dir for JoyToDir Table	 - upfield (to Off. EZ) */
VAR	(WORD,FieldDir);		// PC goes L to R

VAR (WORD, HitBCFrameCount);
VAR (UWORD, ForwardProgress);

VAR (WORD,ExtraTackleX);    // used if BC falls forward
VAR (WORD,TempExtraTackleX);

VAR (WORD,TDAnimLoadedNum);	// actual physical anim loaded for TD celebration

VAR	(BYTE,PlayInfoText1[AI_TEXTINFO_CHARS]);	// main end of play info
VAR	(BYTE,PlayInfoText2[AI_TEXTINFO_CHARS]);   // secondary end of play info
VAR	(BYTE,PlayInfoText0[AI_TEXTINFO_CHARS]);	// only used if fumble

VAR	(WORD,ExtraCPURatings);	// used in playoff games or hard difficulty


VAR	(WORD,GameType);	// Season, exhibition, etc
VAR	(WORD,PracticeMode);	// Set by external shell, cleared at game end

VAR (BOOL,AbortWhereWillPlayerBe);  // set in blocking routines


VARSTRUCT (uBox,BCStuck);
VAR (WORD,BCStuckTimer);
VAR (BOOL,BCStopClock);		// set if offense should stop clock

VAR (WORD,AccumulatedTacklePercent);

/*--------------------  STATS    VARIABLES  -----------------*/


VAR	(WORD,Passer);
VAR	(WORD,PassCatcher);		// for stats
VAR	(WORD,PassInterceptor);	// for stats
VAR	(WORD,PassDefender);	// for stats

VAR	(WORD,KickerSlot);
VAR	(WORD,KickReturner);	// whoever gets kick(offense)
VAR	(WORD,PuntReturner);	// whoever gets punt(defense)
VAR	(WORD,PosPuntReturner);	// whoever might get punt

VAR	(WORD,Fumbler);			// whoever fumbled
VAR	(WORD,FumbleRecoverer);	// whoever gets fumble
VAR	(WORD,BlockedKickBlocker);
VAR	(WORD,BlockedKickRecoverer);

VAR (WORD,NumPlaysInDrive);
VAR (UWORD,StartDriveX);
VAR (ULONG,DriveTime);
VAR (WORD,NextOffTeam);


  // below new as of 3/25/97 for penalty pending stuff
  // i.e. NO stats at EOP - only a bit afterward
VAR (EFFVAR,NextDown);
VAR (BOOL,StartNewDrive);
VAR (BOOL,NeedTeam1stDownStats);
VAR (WORD,PuntYards);
VAR (WORD,PassYards);
VAR (WORD,RushingYards);
VAR (WORD,ReturnYards);
VAR (WORD,SackYards);

VAR (WORD,KickStatsType);
VAR (WORD,KickLength);
VAR	(WORD,PointsThisPlay);
VAR	(WORD,ScoringTeamThisPlay);
VAR (WORD,ScoreStat);	// type of score stats - GS_CONV, etc.
VAR (WORD,ScoreType);	// type of score TD, FG, etc
VAR (WORD,Scorer);
VAR (WORD,SackAllower);
VAR (WORD,Rusher);

VAR	(BOOL,KickEndsPlay);	// helps with possible stats errors

VAR (UWORD,OriginalStartBallX);	// where "last" play started from
							// StartBallX changes at EOP
VAR (UWORD,PenaltyX);	// where penalty happened at (default = StartBallX)


/*------------------------  TESTING    VARIABLES  -----------------*/

VAR (WORD,ShowHuddle);		// debug variable
VAR (WORD,OkToTackle); 		// debug variable
VAR	(WORD,TackleGuyTest); //TEST only - a PNum below which a player won't be tackled

VAR	(WORD,InterceptPercent);
VAR	(WORD,OffCatchPercent);
VAR	(WORD,OffCatchPercent0);
VAR (WORD,CatchStatus);
VAR (WORD,OffRandomNum);
VAR (WORD,DefRandomNum);
VAR	(WORD,PassState);
VAR	(WORD,ErrorNumber);
VAR	(WORD,ForcePenalties);
VAR	(WORD,MissTackleCount);


VAR (WORD,BaseTacklePercent);

VAR (WORD,TeamSpeed[2]); 		// ratings test variable
//VAR (WORD,TeamSpeed1); 		// ratings test variable
VAR (WORD,TeamHands[2]); 		// ratings test variable
//VAR (WORD,TeamHands1); 		// ratings test variable

VAR (WORD,Team0EditStatus); 		// ratings test variable
VAR (WORD,Team1EditStatus); 		// ratings test variable

VAR (WORD,BlockTypeCount[MAX_BLOCK_FACTORS]);	// for debugging
VAR (WORD,BlockTypeCountActual[MAX_BLOCK_FACTORS]);	// for debugging



VAR	(WORD,GameOptions[MAX_NUM_GAME_OPTNS]);


/****************  AUDIO   GLOBAL   VARs   ***********************/

/*  NOTE: the last MAX_CROWD_SFXS/2 are minor volumes so major
	 volumes are never steady - these are added to the above major
	 volumes each time thru loop
	- max variance is 5% of max volume						*/

VAR	(WORD,QBAudioPitch[2]);
VAR	(WORD,CurCrowdVolume[MAX_CROWD_SFXS]);		// this is scaled
VAR	(WORD,DesiredCrowdVolume[MAX_CROWD_SFXS]);  // this is scaled
VAR	(WORD,CrowdVolumeDelta[MAX_CROWD_SFXS]);    // this is scaled
VAR	(WORD,TrueCrowdVolume[NUM_MAJOR_CROWD_SFXS]); 	// only 1/2 - to H/W channels (not scaled)


VAR	(WORD,CurLoopSFXNum);		// variable SFX of crowd[LOOP_SFX]
VAR	(WORD,LeftChannelPercent);
VAR	(WORD,DesiredLeftChannelPercent);

//10/21/96 AIAnnouncerPhrase lengthened due to SILENCE sfx
VAR	(WORD,AIAnnouncerPhrase[32]); /* > 0 --> effect#, <=0 --> number(inverted)	*/
VAR	(WORD,WordsInAnnouncerPhrase);

VAR	(WORD,AIRefPhrase[15]);	/* > 0 --> effect#, <=0 --> number(inverted)	*/
VAR	(WORD,WordsInRefPhrase);

VAR	(WORD,AudibleSFX1);
VAR	(WORD,AudibleSFX2);
VAR	(WORD,AudibleVoice);
VAR	(WORD,SideLineAudioTimer);

VAR (WORD,SFXCount);
VAR (BOOL,PhraseFlag);

VAR	(UWORD,MaxBlockAudioDistance);
VAR	(UWORD,KeyAudioY);		// save for R/L channel later
VAR	(WORD,BlockAudioCount);


/*-------------  Team Data Structures ---------------------*/


VARSTRUCT	(gameteamdata,GameTeamData[2]);
VAR 		(UBYTE, TeamRoster[2][TOTAL_PLAYERS_ON_TEAM]);


VAR (UWORD,BurstButtonMask[2]);	// will have 1 bit set ..
VAR (UWORD,DiveButtonMask[2]);	// .. depending on JPConfiguration
VAR (UWORD,JumpButtonMask[2]);
VAR (UWORD,MiscButtonMask[2]);


VARSTRUCT	(slotdist,OffenseKeyDistance[11]);
VARSTRUCT	(slotdist,DefenseKeyDistance[11]);
VARSTRUCT	(slotdist,DefenseToggleArray[11]);	// mod. of Key Dist.


VARSTRUCT	(fullframeIRPData,CurrentIRPData);
VARSTRUCT	(onetimeIRPdata,OneTimeIRP);


VAR	(UBYTE,MaxAIFrames[MAX_AI_GRAPHICS]);


VAR (WORD, LeagueRoster[NUM_TEAMS_IN_LEAGUE][TOTAL_PLAYERS_ON_TEAM]);

VAR (UBYTE, TeamOffensiveAudible[2][MAX_OFFFORMS_FOR_AUDIBLES][MAX_PLAYS_PER_AUDIBLEFORM]);
VAR (UWORD, TeamDefensiveAudible[2][MAX_DEFFORMS_FOR_AUDIBLES][MAX_PLAYS_PER_AUDIBLEFORM]);

//#undef	REALRAM_C		/* treat as externals from now on actual variable	*/

VAR (WORD,PlayTypeCount[2][MAX_PLAY_TYPES]);


VAR (EFFVAR,PlayerNameShow[8]);	//1st 4 on off, 2nd 4 on def


VAR	(BOOL, CPURatingCheat);
VAR	(LONG, NumColleges);

#ifdef	SATURNMODE
#else
/* MemoryCardMimic allocated on Saturn - however, everything and
	everybody (all machines) should	use MemoryCardMimicPtr			*/
VARSTRUCT	(memorycardmimic,MemoryCardMimic);	// 24K ????

#endif


/***********************  end of  file - FBRAMDAT.H  *********************/
