/*************************************************************************
 * FBREPLAY.C - Has routines for keeping track of instant replays.
 *
 *		Uses a circular buffer...
 *
 * Written by: Mike Heilemann
 *
 * Started 6-20-95
 *************************************************************************/

 
 
#define MAX_REPLAY_BUFFER 150		// 150 frame max (about 10 seconds)


typedef struct {
	WORD PlayersX[22];
	WORD PlayersY[22];
	BYTE PlayersFacing[22];
	BYTE PlayersAnim[22];		// What Animation are we running
	BYTE PlayersAnimFrame[22];	// What frame within Animation
	BYTE PlayersControl[22];

	WORD BallX;
	WORD BallY;
	WORD BallZ;
} frame; 
 
WORD frame_num;	// for easy calculation for end of buffer.

frame replay_buffer[MAX_REPLAY_BUFFER];
frame *replay_start;
frame *replay_ptr;


/*************************************************************************
 * REPLAY INIT - Called every AI loop to collect players positions,
 *		ball position, and update pointers.
 *************************************************************************/
void ReplayInit(void)
{
 	replay_start = replay_buffer;
 	replay_ptr = replay_buffer;		// Will point to the last frame + 1.
}


/*************************************************************************
 * GET REPLAY DATA - Called every AI loop to collect players positions,
 *		ball position, and update pointers.
 *************************************************************************/
void GetReplayData(void)
{
	BYTE i;

	if (frame_num % MAX_REPLAY_BUFFER && frame_num)	// if it is > 0 && no remander
	{
		replay_ptr = replay_buffer;
		replay_start = replay_buffer;
		replay_start++;
	}

	for (i = 0; i < 22; i++)
	{
		replay_ptr->PlayersX[i] = OPXPos(i);
		replay_ptr->PlayersY[i] = OPYPos(i);
		replay_ptr->PlayersFacing[i] = OPFacing(i);
		replay_ptr->PlayersAnim[i] = OPGraphicState(i);
		replay_ptr->PlayersAnimFrame[i] = OPGraphicSequence(i);
		replay_ptr->PlayersControl[i] = OPControllerNum(i);
	}
	replay_ptr->BallX = BallPositionX;
	replay_ptr->BallY = BallPositionY;
	replay_ptr->BallZ = BallPositionZ;
	replay_ptr++;
	frame_num++;
	if (frame_num > MAX_REPLAY_BUFFER)
		replay_start++;
}
