/**************************************************************



					FBStates.h

	PROGRAMMER AUTHOR: M. Knauer       Start Date: 7/22/95
	PLAY DESIGNER: Bobby "The" King

	Modify Date: All the time


	Copyright 1995  GameTek, Inc.



	This INCLUDE file containing the names of the STATES is used
	3 places in FBPLAYS.C with a different macro definition each time
	to define:

	1) the STATEs (functions) as EXTERNALS
	2) to initialize the AllStates array (ptr to function)
	3) enumed with "NUM" tagged to end used to index(handle) the
	   AllStates array

	The purpose is to only have to type in the STATE name once and also
	for to NUM to be in the exact order as their cooresponding pointers.

**************************************************************/



/*****************************************************************

		FOOTBALL   A.I.   NAMING CONVENTIONS

	A "PLAY" is made up of 11 "SCRIPT"s (each player has a SCRIPT)
	A "SCRIPT" is made up of various "STATE"s e.g. RunTo, Wait,
		LookForReceiver,

	The levels of Player A.I. are:
	1) Lowest level - do this first- "ACTION"s -
		a move that once started must be completed
		e.g. falling down, getting up, turning

	2) 2nd level - to be done if no ACTIONs pending  - "STATE"

	3) 3rd level - usually done at end of STATE - "SMARTS"
		this will allow a player to change SCRIPTS


	NOTE: Human controlled players don't do level 2 or 3.


*****************************************************************/


/**********************************************************************



				 S T A T E S   O F   S C R I P T

	These are the STATEs that make up the different scripts.
	The scripts are an array of WORDS referenced by a pseudo
	instruction pointer for each Player on the field (PScriptPtr).

	The first entry is a State Number which is then used as an
	index into the AllStates array. This is an array of pointers
	to the actual functions - all functions take and return no
	parameters - all parameters are referenced by the functions
	themselves using the pseudo instruction pointer.  These
	functions will increase the PScriptPtr to the next State
	when the current State is complete.

--------------------------------------------------------------*/


/* State ZERO is error as many ZEROS float around in 32 bit uP
	as they are used for filler if non 4 byte boundary	*/
	fsp(ErrorState)
//	fsp(PuntState)		// now an ACTION
	fsp(PRWait4Punt)
//	fsp(KickState)
//        fsp(SetPitchPoint)
//        fsp(SetHandOffPoint)
	fsp(DefKickerKickOff)
	fsp(OffKickerKickBall)
//  fsp(OffKickerKickPunt)
	fsp(KickHold)
	fsp(DefAfterFGKickedState)	// init when FG is kicked
	fsp(SetHolderGraphicsState)
	fsp(Wait4Snap)
	fsp(Wait4SnapShuffle)
	fsp(WaitForReady)
	fsp(QBSnap)
	fsp(QBInitFleaFlicker)
	fsp(QBFleaFlickerAfterHandOff)
	fsp(QBLookToPassState)
	fsp(RunToRelRandomX)
	fsp(RunToRel)
	fsp(RunToRelSB)
//  fsp(RunForGoal)
//	fsp(TakePitch)
//	fsp(TakeHandOff)
//	fsp(PitchToHB)
	fsp(SetFutureBCHandoffState)
	fsp(SetFutureBCReverseState)
	fsp(SetFutureBCPitchState)
//	fsp(HandOffToHB)
	fsp(LeadBlocker)
	fsp(InitLeadBlockerState)
//	fsp(PitchToFB)
//	fsp(HandOffToFB)
//	fsp(RBTakeHandOff)
//	fsp(RBTakePitch)
	fsp(BCAvoid)
	fsp(DefBCAvoid)
	fsp(PShuffleWait)
	fsp(ShuffleWaitTilIGetBall)	// used by RBs on Draw
	fsp(InitQBDrawPlayState)
//  fsp(GoUpField)
	fsp(SaveQBShort)
	fsp(SaveQBMedium)
	fsp(SaveQBDeep)
	fsp(QBSGSnap)
	fsp(QBGetSGSnap)
	fsp(PunterGetSGSnap)	// punter must immediately go to punt ACTION
	fsp(LookForReceiver)
	fsp(QBWatchEOP)
	fsp(InitRecParameters)
	fsp(TurnRightSW)
	fsp(TurnLeftNW)
	fsp(TurnPlayerState)
	fsp(Wait4CatchShuffle)
//	fsp(HandOffBall)
	fsp(DoNothing)
	fsp(ChaseBall)
	fsp(ThrowDown)
	fsp(KneelDown)
	fsp(FakeThrowDown)
	fsp(FakeHandoff)
	fsp(DefSnapWAIT)
	fsp(DefLine)
	fsp(SetDefRunUpTarget)
	fsp(RuntoTargetPreSnapState)
	fsp(BlitzerWait4Snap)
	fsp(ZoneDefenseInit)
	fsp(ZoneDefense)
//  fsp(TESTZONE)
	fsp(CBCheckForPunt)
	fsp(CBRead)
	fsp(StandStraight)
	fsp(DoNothingState)
	fsp(DoNothingUpdateGraphics)
	fsp(LBRead)
	fsp(LBCoverQB)
	fsp(FSRead)
	fsp(Wait4Kicker)
	fsp(Wait4BallInAir)
//	fsp(InitOffKO)
//	fsp(RunToTargetSub)
//  fsp(Initpopup2State)
//  fsp(Removepopup2State)
//	fsp(DefBCAvoidState)
///	fsp(InitKickReturner)
	fsp(KickReturnerGetKickState)	/* used by receiver to chase ball being passed	*/
//	fsp(SetToBlockStance)
	fsp(WhoToPassBlock)
	fsp(RunBackwardsState)
	fsp(BePuntBlocker)
	fsp(BeBlocker)
	fsp(DefBeBlocker)
	fsp(InitRunBlock)

	fsp(QBRunToFutureBC)
	fsp(QBSnapThenHandOff)
	fsp(QBSGSnapThenHandOff)
	fsp(PitchBallRightState)
	fsp(PitchBallLeftState)
	fsp(RunToBallTargetState)
	fsp(DefRunToPuntTargetState)


	fsp(FreezePlayerWait)
	fsp(HuddleRunToLOSState)
	fsp(DefAtLOS)
	fsp(OffAtLOS)
	fsp(SetInitialScriptState)
	fsp(CheckForDefBackPedalState)

	fsp(WaitToGoInMotion)
	fsp(GoInMotionBeforeMotion)
	fsp(DecelerateState)
	fsp(EverybodyWatchEOP)
	fsp(ReturnToHuddle)


	fsp(GoInMotion)
	fsp(TestState1)
	fsp(TestState2)
	fsp(TwitchState)
	fsp(Twitch2State)

	fsp(RunToSideLinesState)
	fsp(ReachedSideLinesState)
	fsp(CelebrateState)
	fsp(CoverManInMotionState)
	fsp(InitIWillBeABlocker)

	fsp(SetMaxSpeedState)		// used by PReturner,
	fsp(RefRunToTargetState)
	fsp(RefDoHisThingState)	// do anim an/or SFX
	fsp(RefDoNothingState)

	fsp(OLPassBlockDropBack)
	fsp(FalseStartState)
	fsp(CheckForOnsideState)
	fsp(Celebrate1State)
	fsp(SetQBAsFBCState)	// for flea flicker
	fsp(InHuddleState)
	fsp(QBFleaFlickerWaitForPitch)
	fsp(FalseStartStatePart2)
	fsp(RefPenaltyOutcomeState)

/************************  End of File: FBSTATES.H   *******************/
