#ifndef _FB_STATS_H
#define _FB_STATS_H
/**********************************************************************


					FBSTATS.H

	INITIAL AUTHOR: M. Knauer       Start Date: 6/3/96

	Modify Date:


	Copyright 1995,1996  GameTek, Inc.


	This file contains Stats stuff of the Football AI.


	NOTE:  Each Player uses a subset of "All Stats" - this subset is
	based on the position of the player and is indexed by
	"BYTE PositionToStatSetTable[TOTAL_PLAYERS_ON_TEAM]".  This index
	is then used in a ptr table "void * StatSetPtrTable[MAX_STATS_SETS]"
	where each ptr points to the valid stats for that player/position
	(-1 at end of table).


	Each StatsSet is ordered based on the ROM stats for that
	position.  These ROM stats are a subset of the RAM stats and are
	always in the first "NumOfROMStats[stat_set_num]" indexes.

	ROM stats = non-modifyable data from 1995 season
	RAM stats = kept track of during each game

	MC stats (MC == Memory Card)
	->all ROM/RAM stats are WORDS based on [double-dimensioned] arrays
	with all players having the same number of slots in the array
	whether they are used or not. (it's easier to keep track of this way)
	->MC stats are based on specific structures which only account for
	the stats used for a specific player's position and also whether the
	total of that stat during the season word be a WORD or BYTE.

*************************************************************************/


#define	ILLEGAL_STAT	-32000


// QB,Punter & Kicker have the most stats
#define	MAX_STATS_PER_1PLAYER	12

#define SACK_FACTOR 1	// was 10 (divide by due to ROM sacks = 2.5)

/*---------------------------------------------------------------------

				  S T A T S      I N F O

	In keeping with our construct that all related data be in one place
	by using redefinable macros, here's all stats related stuff.
	Since stats are referenced in an array, the order below determines
	the index for ALL stats stuff - therefore if an emun is reordered,
	the string is also reordered automatically.

	In the StatsMacro, the first entry is used for enum,
	the second is the string.


	StatsMacro(enum,string,divsor,type)

	values for "type" are:
		CUME	// cumulative only
		CUMEL	// cumualtive and next index is longest
		LNG		// longest - only change if value is greater that what's
				   already there
		FORM		// Formula based
---- -----------------------------------------------------------------*/
//#define statsmacro(Enum,string,divsor,type)

#define	statsinfo	\
\
/* 0*/ statsmacro(GS_PASS_ATT,PAtt,1,	CUME) \
/* 1*/ statsmacro(GS_PASS_COMP,PComp,1,CUME) \
/* 2*/ statsmacro(GS_PASS_YARDS,PYds,1,CUMEL) \
/* 3*/ statsmacro(GS_PASS_LONG,PLg,1,	LNG) \
/* 4*/ statsmacro(GS_PASS_TD,PTd,1,	CUME) \
/* 5*/ statsmacro(GS_PASS_INT,Int,1,	CUME) \
/* 6*/ statsmacro(GS_PASS_RATE,Rating,10,FORM)/* figured real time*/ \
\
/* 7*/ statsmacro(GS_RUSH_ATT,RAtt,1,	CUME) \
/* 8*/ statsmacro(GS_RUSH_YARDS,RYds,1,CUMEL) \
/* 9*/ statsmacro(GS_RUSH_LONG,RLg,1,	LNG) \
/*10*/ statsmacro(GS_RUSH_TD,RTd,1,	CUME) \
\
/*11*/ statsmacro(GS_CONV_NUM,Conv,1,	CUME) \
\
/*12*/ statsmacro(GS_RECP_NUM,Recp,1,	CUME) \
/*13*/ statsmacro(GS_RECP_YARDS,RecYds,1,CUMEL) \
/*14*/ statsmacro(GS_RECP_LONG,RecLg,1,LNG) \
/*15*/ statsmacro(GS_RECP_TD,RecTd,1,	CUME) \
\
/*16*/ statsmacro(GS_SACK_ALLOWED,SackAllow,SACK_FACTOR,CUME)\
\
/*17*/ statsmacro(GS_TACK_NUM,Tack,1,	CUME) \
/*18*/ statsmacro(GS_SACK_NUM,Sacks,SACK_FACTOR,CUME) \
/*19*/ statsmacro(GS_FUMFORCE,FumFor,1,CUME) \
/*20*/ statsmacro(GS_FUMRECOV,FumRec,1,CUME) \
/*21*/ statsmacro(GS_FUM_TD,FRecTd,1,	CUME) \
\
/*22*/ statsmacro(GS_INT_NUM,IntNo,1,	CUME) \
/*23*/ statsmacro(GS_INT_YARDS,IntYds,1,CUMEL) \
/*24*/ statsmacro(GS_INT_LONG,IntLg,1,	LNG) \
/*25*/ statsmacro(GS_INT_TD,IntTd,1,	CUME) \
\
/*26*/ statsmacro(GS_PAT,Pat,1,	CUME) \
/*27*/ statsmacro(GS_PAT_ATT,PatAtt,1,	CUME) \
/*28*/ statsmacro(GS_FG,Fg,1,	CUME) \
/*29*/ statsmacro(GS_FG_ATT,FgAtt,1,	CUME) \
/*30*/ statsmacro(GS_FG_LONG,FgLg,1,	LNG) \
/*31*/ statsmacro(GS_FG_BLK,FgBlk,1,	CUME) \
\
/*32*/ statsmacro(GS_PUNT,Punt,1,	CUME) \
/*33*/ statsmacro(GS_PUNT_YARDS,PuntYds,1,	CUMEL) \
/*34*/ statsmacro(GS_PUNT_LONG,PuntLg,1,	LNG) \
/*35*/ statsmacro(GS_PUNT_IN20,In20,1,	CUME)\
/*36*/ statsmacro(GS_PUNT_TB,Tb,1,	CUME)\
/*37*/ statsmacro(GS_PUNT_BLK,PuntBlk,1,	CUME) \
\
/*38*/ statsmacro(GS_KRET_NUM,KRet,1,	CUME) \
/*39*/ statsmacro(GS_KRET_YARDS,KRetYds,1,	CUMEL) \
/*40*/ statsmacro(GS_KRET_LONG,KRetLg,1,	LNG) \
/*41*/ statsmacro(GS_KRET_TD,KRetTd,1,	CUME) \
\
/*42*/ statsmacro(GS_PRET_NUM,PRet,1,	CUME) \
/*43*/ statsmacro(GS_PRET_YARDS,PRetYds,1,	CUMEL) \
/*44*/ statsmacro(GS_PRET_LONG,PRetLg,1,	LNG) \
/*45*/ statsmacro(GS_PRET_TD,PRetTd,1,	CUME) \
/* following are valid only for overall team (not player) stats)*/\
/*46*/ statsmacro(GS_WIN,Wins,1,	CUME) \
/*47*/ statsmacro(GS_LOSS,Loss,1,	CUME) \
/*48*/ statsmacro(GS_TIE,Ties,1,	CUME) \
/*49*/ statsmacro(GS_1STDOWNS,1stDowns,1,	CUME) \
/*50*/ statsmacro(GS_3RDDOWNS_ATT,3rdDownAtt,1,	CUME) \
/*51*/ statsmacro(GS_3RDDOWNS_MADE,3rdDownMade,1,	CUME) \
/*52*/ statsmacro(GS_4THDOWNS_ATT,4thDownAtt,1,	CUME) \
/*53*/ statsmacro(GS_4THDOWNS_MADE,4thDownMade,1,	CUME) \
/*54*/ statsmacro(GS_SCOREQ1,Q1Score,1,	CUME) \
/*55*/ statsmacro(GS_SCOREQ2,Q2Score,1,	CUME) \
/*56*/ statsmacro(GS_SCOREQ3,Q3Score,1,	CUME) \
/*57*/ statsmacro(GS_SCOREQ4,Q4Score,1,	CUME) \
/*58*/ statsmacro(GS_SCOREQ5,Q5Score,1,	CUME) \
/*59*/ statsmacro(GS_TOTAL_SCORE,Score,1,	CUME) \
/*60*/ statsmacro(GS_TOP,TimeOfPoss,1,		CUME) \
/* below added 8/25/96 */	\
/*61*/ statsmacro(GS_SACK_YARDS,SackYards,1,CUME)	\
/*62*/ statsmacro(GS_PENALTY_NUM,Penalties,1,CUME) \
/*63*/ statsmacro(GS_PENALTY_YARDS,PenYards,1,CUME)\
\
/*64*/ statsmacro(GS_NUM_RUN_PLAYS,RunPlays,1,CUME)	\
/*65*/ statsmacro(GS_NUM_PASS_PLAYS,PassPlays,1,CUME)  \
/*66*/ statsmacro(GS_FUM_LOST,FumLost,1,CUME)  \
//statsmacro(GS_NUM_PUNT_PLAYS,PuntPlays,1,CUME)  \
//statsmacro(GS_NUM_FG_PLAYS,FGPlays,1,CUME)      \
//statsmacro(GS_NUM_KO_PLAYS,KOPlays,1,CUME)


/*  following are all of the stats we keep for a team and/or a player.
	Other tables use a subset of these stats to form mini-tables
	depending on a player position.
*/

#define statsmacro(Enum,string,divsor,type)	Enum,
enum gamestatsoffsets {
	statsinfo

	MAX_NUM_STATS_1TEAM	// MUST be last - used as array length

};

#undef statsmacro


enum statstypes {
	CUME,		// cumulative
	CUMEL,		// cumulative longest i.e next index is LNG for this stat
	LNG,		// "longest" i.e Longest Pass (always byte in MC)
	FORM		// formula based
};


/*********************** MEMORY   CARD  STRUCTURES   ************/

struct MCQBstats {
	WORD	GS_PASS_ATT;
	WORD	GS_PASS_COMP;
	WORD	GS_PASS_YARDS;

	WORD	GS_RUSH_ATT;
	WORD	GS_RUSH_YARDS;

	BYTE	GS_PASS_LONG;
	BYTE	GS_PASS_TD;

	BYTE	GS_PASS_INT;
	//GS_PASS_RATE;

	BYTE	GS_RUSH_TD;

	BYTE	GS_RUSH_LONG;		//RAM stats only
	BYTE	GS_CONV_NUM;        //RAM stats only
};


struct MCRBstats { // these are valid stats for all Running backs

	WORD	GS_RUSH_ATT;
	WORD	GS_RUSH_YARDS;
	BYTE	GS_RUSH_LONG;
	BYTE	GS_RUSH_TD;

	WORD	GS_RECP_YARDS;
	BYTE	GS_RECP_NUM;
	BYTE	GS_RECP_LONG;
	BYTE	GS_RECP_TD;

	BYTE	GS_CONV_NUM;
};


struct MCRecstats {	// these are valid stats for Receivers
						// same stats as RBs, except in different order
	WORD	GS_RECP_YARDS;
	WORD	GS_RUSH_YARDS;

	BYTE	GS_RECP_NUM;
	BYTE	GS_RECP_LONG;
	BYTE	GS_RECP_TD;
	BYTE	GS_CONV_NUM;

	BYTE	GS_RUSH_ATT;		// here and below are strictly RAM stats
	BYTE	GS_RUSH_LONG;
	BYTE	GS_RUSH_TD;
};


struct MCOLstats {		// these are valid stats Offensive Line

	BYTE	GS_SACK_ALLOWED;
};


struct MCDefstats {		// these are valid stats for all defensive guys
	WORD	GS_TACK_NUM;
	WORD	GS_INT_YARDS;

	BYTE	GS_SACK_NUM;
	BYTE	GS_FUMFORCE;
	BYTE	GS_FUMRECOV;

	BYTE	GS_INT_NUM;
	BYTE	GS_INT_TD;

	BYTE	GS_FUM_TD;		   //RAM stats only
	BYTE	GS_INT_LONG;       //RAM stats only
};


struct MCKickerstats {	// these are valid stats for Kicker OR punter
	WORD	GS_PUNT_YARDS;

	BYTE	GS_PAT;
	BYTE	GS_PAT_ATT;
	BYTE	GS_FG;
	BYTE	GS_FG_ATT;
	BYTE	GS_FG_LONG;

	BYTE	GS_FG_BLK;			// here and below are strictly RAM stats

	BYTE	GS_PUNT;
	BYTE	GS_PUNT_LONG;
	BYTE	GS_PUNT_IN20;
	BYTE	GS_PUNT_TB;
	BYTE	GS_PUNT_BLK;
};

struct MCPunterstats {	// these are valid stats for punter
							// same as kicker, but different order
	WORD	GS_PUNT_YARDS;
	BYTE	GS_PUNT;
	BYTE	GS_PUNT_LONG;
	BYTE	GS_PUNT_IN20;
	BYTE	GS_PUNT_TB;
	BYTE	GS_PUNT_BLK;

	BYTE	GS_PAT;					// here and below are strictly RAM stats
	BYTE	GS_PAT_ATT;
	BYTE	GS_FG;
	BYTE	GS_FG_ATT;
	BYTE	GS_FG_LONG;
	BYTE	GS_FG_BLK;
};

struct MCKRstats {	// these are valid stats for Kick OR punt Returner
	BYTE	GS_KRET_NUM;
	BYTE	GS_KRET_TD;

	WORD	GS_KRET_YARDS;
	WORD	GS_PRET_YARDS;

	BYTE	GS_PRET_NUM;
	BYTE	GS_PRET_TD;

	BYTE	GS_KRET_LONG;	// here and below are strictly RAM stats
	BYTE	GS_PRET_LONG;
};


struct MCteamstats {
/*  fewer people on team ???
	struct MCQBStats	MCQBStats[2];
	struct MCRBStats	MCRBStats[3];  // valid stats for all Running backs
	struct MCRecStats 	MCRecStats[8]; //  valid stats for Receivers
	struct MCOLStats 	MCOLStats[6];	// valid stats Offensive Line
	struct MCDefStats 	MCDefStats[17];	// valid stats for all defensive guys
	struct MCKickerStats MCKickerStats[1];	// valid stats for Kicker OR punter
	struct MCPunterStats MCPunterStats[1];	// valid stats for punter
	struct MCKRStats 	MCKRStats[2];// valid stats for Kick OR punt Returner
	WORD allplyrstats[MAX_NUM_STATS_1TEAM];
	WORD alloppplyrstats[MAX_NUM_STATS_1TEAM];	// opposition stats
*/
	struct MCQBstats	MCQBStats[NUM_QBs];
	struct MCRBstats	MCRBStats[NUM_RBs];  // valid stats for all Running backs
	struct MCRecstats 	MCRecStats[NUM_RECs]; // valid stats for Receivers
	struct MCOLstats 	MCOLStats[NUM_OLs];	//  valid stats Offensive Line
	struct MCDefstats 	MCDefStats[NUM_DEFs];	// valid stats for all defensive guys
	struct MCKickerstats MCKickerStats[NUM_Ks];	// valid stats for Kicker OR punter
	struct MCPunterstats MCPunterStats[NUM_Ps];	// valid stats for punter
	struct MCKRstats 	MCKRStats[NUM_KRs];// valid stats for Kick OR punt Returner
	WORD	allplyrstats[MAX_NUM_STATS_1TEAM];		// no "compression" here
	WORD	alloppplyrstats[MAX_NUM_STATS_1TEAM];	// opposition stats

};


typedef struct teamRAMstats {	// used by StatsData[2] & SeasonStatsPtr
	WORD allplyrstats[MAX_NUM_STATS_1TEAM];
	WORD alloppplyrstats[MAX_NUM_STATS_1TEAM];	// opposition stats
	WORD posstats[TOTAL_PLAYERS_ON_TEAM][MAX_STATS_PER_1PLAYER];	// MUST BE last FOR SHELL
} TEAMRAMStats;



struct teamstats {		//ROM stats
	WORD	allplyrstats[MAX_NUM_STATS_1TEAM];		// no "compression" here
	WORD	alloppplyrstats[MAX_NUM_STATS_1TEAM];	// opposition stats
};



#define NUM_REG_SEASON_GAMES 	240
#define NUM_POST_SEASON_GAMES	11
#define NUM_ALL_SEASON_GAMES (NUM_REG_SEASON_GAMES + NUM_POST_SEASON_GAMES)


/**  this structure is the size of 3 mem blocks on the PSX (excluding
internal checksums of 888 bytes)  = 23688	 (total 3blocks = 24K= 24576) */
typedef struct memorycardmimic { /**** WARNING **** not over 23688 bytes*/
	BYTE password[16];
	struct MCteamstats MCTeamStats[NUM_TEAMS_IN_LEAGUE];
	BYTE			GameScores[NUM_ALL_SEASON_GAMES][2];	// home/away
	WORD   UseTradedTeam;

	WORD   ReservedWord0;	// following reserved for future use
	WORD   ReservedWord1;	// .. in future products
	WORD   ReservedWord2;	//
	BYTE   ReservedByte0;
	BYTE   ReservedByte1;   /**** WARNING **** not over 23688 bytes*/
} memorycardMIMIC;


struct allcreatedplayers {
	WORD	NumCreatedPlayers;		// how many of the MAX are used
	struct pdata_t CreatedPlayers[MAX_CREATED_PLAYERS];
};



extern struct memorycardmimic *MemoryCardMimicPtr;
extern struct teamRAMstats *SeasonStatsPtr;






/*---- Here's some more redefinable macros relating to Stats Sets  --*/

//  statssetmacro(enum, ptrtoRAMindexs, numROMStats)

#define statssetinfo							\
statssetmacro(SS_QB,QBStatsSet,			10)		\
statssetmacro(SS_RB,RBStatsSet,			9) 		\
statssetmacro(SS_REC,RecStatsSet,		5)		\
statssetmacro(SS_OL,OLStatsSet,			1)		\
statssetmacro(SS_DEF,DefStatsSet,		7)		\
statssetmacro(SS_KICKER,KickerStatsSet,	5)		\
statssetmacro(SS_PUNTER,PunterStatsSet,	6)		\
statssetmacro(SS_KR,KRStatsSet,			6)


/*  following are all of the stats sets(SS_xxx) we use in the game	*/

#define  statssetmacro(enum, ptrtoRAMindexs, numROMStats) enum,

enum statssetsbypos {
	statssetinfo
	MAX_STATS_SETS		// used to define table size
};
#undef	statssetmacro





/*---------- prototypes of functions found in FBStats.c	 -----------*/

WORD *GetStatsSetPtr(WORD pos);
WORD ValidStatForPos(WORD pos,WORD statnum);

WORD UpdateStats(WORD pnum0_21, WORD statnum, WORD value);
void UpdateStatsMax(WORD pnum0_21, WORD statnum, WORD value,WORD attempts);
void UpdateTeamStats(WORD teamslotnum, WORD statnum, WORD value);
void UpdateTeamStatsMax(WORD teamslotnum, WORD statnum, WORD value);


WORD GetPlayerGameStats(WORD teamslotnum, WORD pos0_46, WORD statnum);

void ClearAllStats(void);

BYTE *GetStatsString(UWORD statnum);



specificplayerdata *GetPlayerDataPtr(WORD plyrslotnum);	// slot based (aracde) could be IRP

specificplayerdata *GetPlayerDataPtr0_21(WORD plyrslotnum);	// slot based (aracde)
specificplayerdata *GetPlayerPosDataPtr(WORD teamnum0_1,WORD posnum); // position/team based(shell)


void UpdateMiscStats(void);	// QB ratings, score per quarter
void UpdateFinalGameStats(void);
WORD GameStatsToSeasonStats(WORD teamnum,struct teamRAMstats *fromptr);

// following MUST be called AFTER "GameStatsToSeasonStats(WORD teamnum, struct teamRAMstats *fromptr)"
WORD UpdateSeasonGameScores(WORD seasongamenum, struct teamRAMstats *fromptrhometeam);


void TeamSeasonStatsToMemoryCardMimic(WORD tnum0_29);
void TeamMemoryCardMimicToSeasonStats(WORD tnum0_29);



void ClearAllGameStats(void);

void UpdatePrelimOffPlayStats(WORD lastbc, UWORD endx, BOOL endofplay);
void UpdatePrelimDefPlayStats(WORD lastbc, UWORD endx);

void UpdateEOPPlayerStats(void);
void UpdateEOPStats(void);

void OfficiallyAddScore(void);	// and handle individual score stats


/**--------------------- global tables in FBStats.c	  ---------------*/

extern WORD NumOfROMStatsPerSet[MAX_STATS_SETS];
extern struct teamRAMstats StatsData[2];	// RAM stuff in fbstats.c
extern BYTE *StatsNames[MAX_NUM_STATS_1TEAM];
extern BYTE *PositionStrings[TOTAL_PLAYERS_ON_TEAM];
extern BYTE *DepthChart43Strings[TOTAL_PLAYERS_ON_TEAM];
extern BYTE *DepthChart34Strings[TOTAL_PLAYERS_ON_TEAM];
extern BYTE PositionToStatSetTable[TOTAL_PLAYERS_ON_TEAM];


/***************************  End of File - FBStats.H  *******************/
#endif	//ifndef _FB_STATS_H
