/*************************************************************************
	FIELD.C - Field Model handling
 
    Written by: Mike Heilemann
	Updated by Christopher Deel 
			
		Copyright (C) 1995 Gametek Inc.
		All rights Reserved
*************************************************************************/

#define USE_PMD 1	// Use PMD type model

//#define TEST_STADIUM_MODEL 1		// for debug

#include <sys/types.h>

#include <libetc.h>			   /* For Controller routines */
#include <libgte.h>			   /* For LIBGS */
#include <libgpu.h>			   /* For LIBGS */
#include <libgs.h>			   /* Graphics Library */
#include <libsn.h>

#include "prjtypes.h"
#include "prjhdrs.h"    /* Project header files */
#include "debug.h"
#include "arcade.h"

#include "fbequ.h"
#include "fbram.h"

#define DEBUG_THIS 1

#if USE_PMD
//#define MODEL_BYTES  247810		// Max Bytes for PMD data (stad 6)
//#define MODEL_BYTES  243720		// Max Bytes for PMD data
#define MODEL_BYTES  125000		// Max Bytes for PMD data

//#define PACKET_BYTES 10    // Max Bytes for TMD packet dat
#else  // TMD
#define MODEL_BYTES  111000		// Max Bytes for TMD data
#define PACKET_BYTES 211000    // Max Bytes for TMD packet dat
#endif

char *gTmdFname = "/stadium/test/a.tmd ...................";
//char *gTmdReversedFname = "/stadium/test/a.tmd .............";
char *gTimFname = "/stadium/test/astnorm1.mfl\0............";
char *gTmpTimFname = "/stadium/test/astnorm1.mfl\0............";

char *gStadTimNames[] = { 	
			"tex01.tim",
			"tex02.tim",
			"tex03.tim",
			"tex04.tim",
			"tex05.tim",
			"tex06.tim",
			"tex07.tim",
			"tex08.tim",
			"tex09.tim",
			"tex10.tim",
			"dummy11.tim",
			"*",			// Tex12
			"tex08b.tim",

//			"tex11.tim",
//			"tex12.tim",
			NULL
		};

//vram_t *gStadTpage[16];

static char gStadPermTex[16] = {1,1,1,1, 0,0,0,1, 1,1,1,1, 1,1,1,1}; // Keep these in VRAM
int gPermTexLoaded = False;

SubTexture_t gStadTexture[16];
vram_t gStadClut[16];			// STtadium Clut information		
vram_t *gTex10BlackClut;

//Point2DType gStadTimPos[16];
		
ULONG gStadTmdSector[2] = {0, 0};
ULONG gStadTimSector = 0;
char *gStadTmdPtr = NULL, *gStadTimPtr = NULL;
int gStadTimsLoaded = False;

#if USE_PMD
GsDOBJ3		object[OBJECTMAX]; /* Array of Objects */
#else // TMD
GsDOBJ5		object[OBJECTMAX]; /* Array of Objects */
ULONG *gObjPacket[OBJECTMAX];
#endif

ULONG          Objnum;			/* number of objects */
GsCOORDINATE2   DWorld;        /* World Coordinate system */
SVECTOR         PWorld;        /* */

extern MATRIX GsIDMATRIX;		/* Unit Matrix */

GsRVIEW2  view;				   /* View Point */
GsF_LIGHT pslt[3];			   /* Flat light source stuff*/
GsFOGPARAM gFog;					/* Fog */
int gStadiumNum = 1;
int gSunlight = 0xD0;  // Sunlight Brightness
int gDomeStadium = FALSE;

ULONG padd;					  /* Controller */

//ULONG PacketArea[PACKET_BYTES / 4];    /* GPU Packets area */
ULONG *gStadPacketPtr = NULL;

/*-------------------------------------------------------------
  DrawField ~ Draw the field to the frame
-------------------------------------------------------------*/
void DrawField(void)
{
	int     i;
//	register GsDOBJ5 *op;		 /* pointer to Object Handler */
	register GsDOBJ3 *op;		 /* pointer to Object Handler */
	MATRIX  tmpls;
   
//	do_controls();			  /* Process controller button presses */
	set_coordinate(&PWorld,&DWorld);
	GsSetRefView2(&view);
  
  /* Set flag so it recalculates coordinates */
	DWorld.flg = 0;

	op = object;
//	for(i = 0, op = object; i < Objnum; i++)
//	{
		GsGetLs(op->coord2,&tmpls);  /* Calculate Local-Screen Matrix */
		GsSetLightMatrix(&tmpls);  // Set LWMATRIX to GTE Lighting Registers */
		GsSetLsMatrix(&tmpls);    /* Set LSMATRIX to GTE Registers */

      /* Perspective Translate Object and Set OT */
#if USE_PMD      
		GsSortObject3(op, gPtrGsOt, 14-OT_LENGTH);
#else	// TMD
		GsSortObject5(op, gPtrGsOt, 14-OT_LENGTH, getScratchAddr(0) );
#endif
//		op++;
//	}
}

/*-------------------------------------------------------------
  LoadStadiumModel ~ 
-------------------------------------------------------------*/
void LoadStadiumModel(void)
{
	ULONG sector, bytes, length;
	static int dirLoaded = 0;
	 
	 Cmarker();
//	 dirLoaded = OffenseDir;  // no need for 2 stadiums?
//	if ((gStadTmdPtr == NULL) || (dirLoaded != OffenseDir) ) 
	if ((gStadTmdPtr == NULL) ) 
	{
		if (gStadTmdSector[0] == 0)
		{
			gStadTmdSector[0] = GetFileSector(gTmdFname);
//			gStadTmdSector[1] = GetFileSector(gTmdReversedFname);
		}
//		gStadTmdPtr = LoadFileSectors(gStadTmdSector);
//		sector = gStadTmdSector[OffenseDir];
		sector = gStadTmdSector[0];
		length = MGetSectorIDSize(sector);
		bytes = length * SIZE_1SECTOR;
		 DumpAllocLoc();
//		if (gStadTmdPtr == NULL)
//		{
//			gStadTmdPtr = (char*)AllocMem(bytes + 8000);
			gStadTmdPtr = (char*)AllocMem(MODEL_BYTES);
//		}
#if PC_DEBUG
		if (bytes > MODEL_BYTES)
			Cerror();
#endif				
		LoadFileSectorsTo(sector, gStadTmdPtr);
//		dirLoaded = OffenseDir;
		 DumpAllocLoc();
	}
}

/*-------------------------------------------------------------
  PreSeekStadiumTims ~ 
-------------------------------------------------------------*/
void PreSeekStadiumTims(void)
{
	if (gStadTimSector) 
	{
		SeekSector(gStadTimSector, False);
	}
}

/*-------------------------------------------------------------
  LoadScoreboardScreenPic ~ 
-------------------------------------------------------------*/
void LoadScoreboardScreenPic(int wait)
{
	int loaded;
	
	ClearLoadList();
	gScoreboardScreenMfl = AllocAddLoadList(gScoreboardScreenSectorUse);
	StartLoadList();		
	if (wait)
	{
		loaded = False;
		while(!loaded)
		{
			loaded = CheckLoadList();
		}
//		Cmarker();	
	}
}  /* End LoadScoreboardScreenPic() */

/*-------------------------------------------------------------
  LoadStadiumTims ~ 
-------------------------------------------------------------*/
void LoadStadiumTims(int wait)
{
	int loaded;
	
	if (gStadTimSector == 0)  // Stadium TIMs for Now
	{
		gStadTimSector = GetFileSector(gTimFname);
	}
	
//	gStadTimPtr = (ULONG*)LoadFileSectors(gStadTimSector);
//	 Cmarker();
	ClearLoadList();
	gStadTimPtr = AllocAddLoadList(gStadTimSector);
//	gScoreboardScreenMfl = AllocAddLoadList(gScoreboardScreenSectorUse);
	StartLoadList();		
	if (wait)
	{
		loaded = False;
		while(!loaded)
		{
			loaded = CheckLoadList();
		}
//		Cmarker();	
	}
	gStadTimsLoaded = True;
}

/*-------------------------------------------------------------
  FreeScoreboardScreenPic ~ 
-------------------------------------------------------------*/
void FreeScoreboardScreenMfl(void)
{
	if (gScoreboardScreenMfl)
	{
		FreeMem(gScoreboardScreenMfl);
		gScoreboardScreenMfl = NULL;
	}
}  /* End FreeScoreboardScreenPic() */

/*-------------------------------------------------------------
   FreeStadiumTims ~ Free Stadium TIMs once they are in VRAM
-------------------------------------------------------------*/
void FreeStadiumTims(void)
{
	if (gStadTimPtr)
	{
		FreeMem(gStadTimPtr);
		gStadTimPtr = NULL;
	}
//	FreeScoreboardScreenPic();
}  /* End */

/*----------------------------------------------------------------------
 InitStadiumTextures ~ 
 Load up all field textures first so they get marked as used by MMVRAM 
 this is because they must be loaded at specific locations indicated by
 the TIM file for the TMD data to put them where they go... MRH 
-----------------------------------------------------------------------*/
void InitStadiumTextures(void)
{
	ULONG *ptr, *filePtr, *name;
	int i;
	GsIMAGE tim;
	vram_t *texture, *clut;

	 Cmarker();
	filePtr = (ULONG*)gStadTimPtr;

	i = 0;
	do
	{
		if (gStadPermTex[i] && gPermTexLoaded)
		{
			ReAllocVram(gStadTexture[i].texture, 0, 0);
			ReAllocVram(gStadTexture[i].clut, 0, 0);
		}
		else
		{	
			ptr = NULL;
			if (*gStadTimNames[i] != '*')
				ptr = (ULONG*)GetMultiFilePtr(filePtr, gStadTimNames[i]);
			
			if (ptr != NULL)
			{
				LoadTimExact(ptr);
				GetTimInfo2((u_long *)ptr,&tim);
				texture = FindVramAt(tim.px, tim.py);
				clut = FindVramAt(tim.cx, tim.cy);
				if (clut == NULL)
				{
//					 DebugPrintf (0, "Clut not found %d,%d\n", tim.cx, tim.cy);
					clut = &gStadClut[i];
					clut->x = tim.cx;
					clut->y = tim.cy;
				}
				else
				{
					clut = (vram_t*)AllocVramAt(tim.cx, tim.cy);
				}
				clut->CLUT_ID = GetClut(tim.cx, tim.cy);
				
				if (i == 9)
				{
//					*(USHORT*)tim.clut = 0;
					SetClutTransparent((USHORT*)tim.clut, 16, FALSE);
//					clut = register_CLUT( (ULONG*)tim.clut, VRAM_CLUT256);
					Reload4BitClut((ULONG*)tim.clut, clut);
					Set0Transparent(clut); 
					DrawSync(0);
										
					*(USHORT*)tim.clut = 1;
					gTex10BlackClut = register_CLUT( (ULONG*)tim.clut, VRAM_CLUT256);
				}
				InitSubTexture(&gStadTexture[i], texture, clut, 0,0, 256,256, 0,0);
				 
				 DebugPrintf(0, "%s #%d  %d,%d  %d,%d  %dc\t", 
		 				gStadTimNames[i], i,tim.px,tim.py,tim.cx,tim.cy,tim.cw);
				 DebugPrintf(0, "  %d,%d %d\n", 
				 		gStadTexture[i].clut->x, gStadTexture[i].clut->y,
				 		gStadTexture[i].clut->used);
			}
		}
		i++;
			
	} while (gStadTimNames[i] != NULL);
	 Cmarker();
		
//	if (!gPermTexLoaded)
//	{
		ptr = (ULONG*)GetMultiFilePtr(filePtr, "sky.clt");
		CopyShorts((USHORT*)ptr, &gSkyClutData[0], 16);

//		texture = alloc_VRam(VRAM_TEXTURE, 0); // Tex12
		texture = (vram_t*)AllocVramAt(640, 256); // Tex12
//		clut = alloc_VRam(VRAM_CLUT16, 0);
		clut = (vram_t*)AllocVramAt(0, 482);
		InitSubTexture(&gStadTexture[11], texture, clut, 0,0, 256,256, 0,0);
//	}
//	else
//	{
//		ReAllocVram(gStadTexture[i].texture, 0, 0);
//		ReAllocVram(gStadTexture[i].clut, 0, 0);
//	}

	InitExtraStadiumTextures(gStadTimPtr);
	gPermTexLoaded = True;
				
//	VSync(0);
	 DebugPrintf(0, "Tims Loaded: ");
	 DumpAllocLoc();
	
//	SetScoreboardScreenState(SCOREBOARD_SCREEN_PICTURE);
	 Cmarker();
//	 DebugPrintf(0, "Stadtex addr: %X %X\n", gStadTimPtr, gScoreboardScreenMfl);
	DrawSync(0);
	FreeStadiumTims();
//	InitLogos();
	InitLogoTextures();
	 Cmarker();
}

/*-------------------------------------------------------------
  FreeStaduimVram ~ Free Staduim VRAM for Play call
-------------------------------------------------------------*/
void FreeStadiumVram(void)
{
	int i;
	
	 Cmarker();
	i = 0;
	for (i = 0; i < 12; i++)
	{
		if (!gStadPermTex[i])
			free_VRam( gStadTexture[i].texture );
	}
	
	gStadTimsLoaded = False;
	FreeExtraStadiumTextures();
//	 DumpVramAlloc();
}

/*-------------------------------------------------------------
  TranslateField ~ TranslateField in XYZ
-------------------------------------------------------------*/
void TranslateFieldTo(SVECTOR *vec)
{
	DWorld.coord.t[0] = vec->vx;
	DWorld.coord.t[1] = vec->vy;
	DWorld.coord.t[2] = vec->vz;
}

/*-------------------------------------------------------------
  RotateFieldTo ~ 
-------------------------------------------------------------*/
void RotateFieldTo(SVECTOR *vec)
{
     PWorld.vx = vec->vx;    
     PWorld.vy = vec->vy;    
     PWorld.vz = vec->vz;    
}
    
/*************************************************************************
 *
 *************************************************************************/
view_init()
{
  /*---- Set projection,view ----*/
//	GsSetProjection(1000);

	/* Set view point location */
//	view.vpx = 0;	  /* LEFT/RIGHT */
//	view.vpy = -45000; /* UP/DOWN */
//	view.vpy = -1000; /* UP/DOWN */
//	view.vpz = -2500;	  /* FORWARD/BACK */

	/* Set focus point location */
//	view.vrx = 5200;
//	view.vry = 0;
//	view.vrz = -2500; /* */

	/* Setting bank of SCREEN */
	view.rz=0;

  /* Set parent of viewing coordinate */
	view.super = WORLD;
  
  /* Calculate World-Screen Matrix from viewing paramter */
	GsSetRefView2(&view);

	GsSetNearClip(-1000);


  /* Rotate X  */
/*     PWorld.vx -= 20*ONE/360;    /* Move Object */
/*     view.vpy -=1*ONE/360;;     /* Move Camera */

  /* Zoom in */
/*     DWorld.coord.t[2] -= 500;   /* Move Object */
/*     view.vpz -= 50;            /* Move Camera */

/*     DWorld.coord.t[0] -= 1000;  /* Translate X */
/*     DWorld.coord.t[1] += 500;   /* Translate Y */
}

/*-------------------------------------------------------------
  SetLightColor ~ Set color of light
-------------------------------------------------------------*/
void SetLightColor(char red, char green, char blue, int num)
{
	pslt[num].r = red;
	pslt[num].g = green;
	pslt[num].b = blue;
}

/*-------------------------------------------------------------
  SetLightDir ~ Set light direction
-------------------------------------------------------------*/
void SetLightDir(long vx, long vy, long vz, int num)
{
	pslt[num].vx = vx;
	pslt[num].vy = vy;
	pslt[num].vz = vz;
}

/*-------------------------------------------------------------
  SetLightGs ~ Send light data to LibGs
-------------------------------------------------------------*/
void SetLightGs(int num)
{
	GsSetFlatLight(num, &pslt[num]);
}

/*-------------------------------------------------------------
  InitLights ~ 
-------------------------------------------------------------*/
void InitLights(void)
{
	SetLightColor(gSunlight,gSunlight,gSunlight, 0);
	SetLightDir(0x2000, 0x2000, 0x1000, 0);
	SetLightGs(0);

	SetLightColor(gSunlight - 20, gSunlight - 20, gSunlight - 20, 1);
	SetLightDir(0x1000, 0x2000, 0x600, 1);
	SetLightGs(1);

	SetLightColor(gSunlight - 30, gSunlight - 30, gSunlight - 30, 2);
	SetLightDir(-0x2000, 0, 0x2000, 2);
	SetLightGs(2);

	GsSetAmbient(0xF0,0xF0,0xF0);

	/*---- Set Fog parm --*/
//	gFog.dqa = -1000*ONE/64/1000;
//	gFog.dqb = 5/4*ONE*ONE;
//	gFog.dqb = 3*ONE*ONE;

//	gFog.dqa = -600;
//	gFog.dqb = 5120*4096;

//	gFog.rfc = gFog.gfc = gFog.bfc = 0x00;    /*##*/
//	gFog.rfc = 0x80;
//	gFog.rfc = gFog.gfc = gFog.bfc = 0x80;    /*CGD*/
//	GsSetFogParam(&gFog);

	GsSetLightMode(0);
//	GsSetLightMode(1);	// enable fog
}

/*************************************************************************
 *
 *************************************************************************/
static coord_init()
{
  /* Set hierarchy of Coordinate */
	GsInitCoordinate2(WORLD,&DWorld);

  /* Init work vector */
	PWorld.vx = PWorld.vy = PWorld.vz = 0;

  /* the org point of DWold is set to Z = -4000 */
	DWorld.coord.t[2] = 0;
}

/*************************************************************************
 *
 *************************************************************************/
set_coordinate(SVECTOR *pos, GsCOORDINATE2 *coor)
{
	MATRIX tmp1;
	SVECTOR v1;

	tmp1   = GsIDMATRIX;			/* start from unit matrix */

  /* Set translation  */
	tmp1.t[0] = coor->coord.t[0];
	tmp1.t[1] = coor->coord.t[1];
	tmp1.t[2] = coor->coord.t[2];

	v1 = *pos;
  
  /* Rotate Matrix */
	RotMatrix(&v1,&tmp1);
  
  /* Set Matrix to Coordinate */
	coor->coord = tmp1;
  
  /* Clear flag becuase of changing parameter */
	coor->flg = 0;
}    

#if USE_PMD
/*-------------------------------------------------------------
  InitPmdModel ~ 
-------------------------------------------------------------*/
void InitPmdModel(void)
{
	ULONG *dop;
	register GsDOBJ3 *objp;				  /* Object Handle */
	int i;

	LoadStadiumModel();
	dop = (ULONG*)gStadTmdPtr;
	
	Objnum = GsLinkObject3((ULONG)dop, object);

	for(i=0,objp=object; i < Objnum; i++)
	{
		/* Set Coordinate of Object Handler */
		objp->coord2 = &DWorld;

//		objp->attribute = 0;	/* attribute */
		objp->attribute |= GsLOFF;	/* attribute */
		objp->attribute |= GsALON;	/* Semi Transparent */
		objp->attribute |= GsATWO;	/* Semi Transparent */
//		objp->attribute |= GsDIV1;   
//		objp->attribute |= GsFOG;    
//		objp->attribute |= GsLDIM3;  
//		objp->attribute |= GsMATE;   
//		 DebugPrintf(0, "Atb:%x \r", objp->attribute);
		objp++;
	}
}

#else // TMD
/*-------------------------------------------------------------
   ~  Init TMD model
-------------------------------------------------------------*/
InitTmdModel()
{
	ULONG *dop;
	register GsDOBJ5 *objp;				  /* Object Handle */
	int i, packetSize;
	ULONG *packet, *packet2;

	LoadStadiumModel();
	dop = (ULONG*)gStadTmdPtr;
	
	if (gStadPacketPtr == NULL)
	{
		gStadPacketPtr = AllocMem(PACKET_BYTES);
	}
	packet = gStadPacketPtr;			   /* packet Alloced */

//	packet = PacketArea;			   /* packet Array*/
	
	dop++;							/* header skip */

	GsMapModelingData(dop);		/* */
	dop++;
	Objnum = *dop;				  /* Get actual number of objects */
	dop++;							/* Adjust for GsLinkObject5? */

/* Link ObjectHandler and TMD FORMAT MODELING DATA */
	for(i = 0; i < Objnum; i++)
		GsLinkObject5((ULONG)dop, &object[i], i);
	
	for(i=0,objp=object; i < Objnum; i++)
	{
		/* Set Coordinate of Object Handler */
		objp->coord2 = &DWorld;

//			objp->attribute = 0;	/* attribute */
//		objp->attribute |= GsLOFF;	/* attribute */
//		objp->attribute |= GsALON;	/* Semi Transparent */
//		objp->attribute |= GsDIV1;   
//		objp->attribute |= GsFOG;    
//		objp->attribute |= GsLDIM3;  
//		objp->attribute |= GsMATE;   
//		 DebugPrintf(0, "Atb:%x \r", objp->attribute);

		packet2 = GsPresetObject(objp, packet);
#if PC_DEBUG
		 packetSize = (ULONG)packet2 - (ULONG)packet;
		 DebugPrintf(0, "Packet %X %X %d\n", packet, packet2, packetSize);
		 if (packetSize > PACKET_BYTES)
		 	DebugPrintf(0, "Packet Overflow error!\n");
#endif //PC_DEBUG		 	
		packet = packet2;
		objp++;
	}
}
#endif // USE_PMD



/*-------------------------------------------------------------
  model_init ~ 
-------------------------------------------------------------*/
void model_init(void)
{
#if USE_PMD
	InitPmdModel();
#else		
	InitTmdModel();				  /* Load up Model */
#endif	
	
}

/*************************************************************************
 * INIT FIELD - Initialize field textures and model data...
 *************************************************************************/
void InitField(void)
{
//	SetScreenMode(VMODE512); 
//	initVRamHandling();

//	InitStadiumTextures();

	coord_init();				  /* Initialize Coordinant system */
	model_init();				  /* Load up Model */
	view_init();				  /* Initialize View */
//	light_init();				  /* Initialize Light source */
	InitLights();
}

/*-------------------------------------------------------------
  FreeModel ~ 
-------------------------------------------------------------*/
void FreeModel(void)
{
	FreeMem(gStadTmdPtr);
	gStadTmdPtr = NULL;
#if !USE_PMD	
	FreeMem(gStadPacketPtr);
	gStadPacketPtr = NULL;
#endif
	gPermTexLoaded = False;
	gStadTimSector = 0;
	gStadTmdSector[0] = 0;
	gStadTmdSector[1] = 0;
}

/*-------------------------------------------------------------
  SetStadiumConditions ~ 
-------------------------------------------------------------*/
void SetStadiumConditions(int textures, int stadium, int turf,
						int weather, int sunlight, int dome)
{
//	char *turfNames[] = {"drk", "ttn", "ast", "ast" };
	char *turfNames[] = {"ttn", "ttn", "ast", "ast" };
//	char *weatherNames[] = {"suny", "rain", "snow", "dome"};
	char *weatherNames[] = {"suny", "rain", "snow", "suny"};

//	 stadium = 20; //debug
//	 textures = 1;  //debug
	turf &= 3;
	weather &= 3;
//	stadium &= 7;
//	if (stadium < 1) 
//		stadium = 1;
	gStadiumNum = stadium;
	gDomeStadium = dome;
	if (dome)	
	{
//		gDomeStadium = True;
		weather = 3;
	}
//	if (stadium >= 6)  // for testing
//	{
//		stadium = 6;
//		weather = 0;
//		turf = 0;
//	}
	
	
#if USE_PMD
//	sprintf(gTmdFname, "/stadium/%d/stad%d.pmd", stadium, stadium);
	sprintf(gTmdFname, "/stadium/pmd/stad%02d.pmd", stadium);
#else	
	sprintf(gTmdFname, "/stadium/%d/stad%d.tmd", stadium, stadium);
#endif	
	
	sprintf(gTimFname, "/stadium/ts%d/t%02d/%s%s.mfl", 
			textures/30, textures, turfNames[turf], weatherNames[weather]);
//	sprintf(gTmpTimFname, "/stadium/%d/tmpstad.mfl", stadium);
	
	#if TEST_STADIUM_MODEL	//debug
		sprintf(gTmdFname, "/stadium/test/stadtest.pmd");
		sprintf(gTimFname, "/stadium/test/stadtest.mfl");
	#endif //TEST_STADIUM_MODEL
	
	gSunlight = sunlight;
	DebugPrintf(0, "Stad: Sun:%X  %s %s\n", gSunlight, gTmdFname, gTimFname);
	gStadTimSector = 0;
}


#if DEBUG_THIS
//**************************** Test Routines ***********************

/*-------------------------------------------------------------
  DumpFieldData ~ 
-------------------------------------------------------------*/
void DumpFieldData(void)
{
	DebugPrintf(0, "V:%d,%d,%d  ", view.vpx, view.vpy, view.vpz);
	DebugPrintf(0, "R:%d,%d,%d  ", view.vrx, view.vry,view.vrz);

	DebugPrintf(0, "P:%d,%d,%d  ", PWorld.vx, PWorld.vy, PWorld.vz);
}

#define DbViewMove 1    /*##*/
/*#define DbRefMove 1    /*##*/
/*#define DbObjMove 1    /*##*/

/*************************************************************************
 * DO CONTROLS - Looks at control states and acts on them.
 *************************************************************************/
do_controls()
{
	SVECTOR v1;
	MATRIX  tmp1;
	register ULONG padd;

	padd = JpadData[0];
#if DbObjMove  
  /* Rotate Y  */
	if((padd & PADRleft)>0)
		PWorld.vy -=ONE/360;    /* Move Object */
	if((padd & PADRright)>0)
		PWorld.vy +=ONE/360;    /* Move Object */

  /* Rotate X  */
	if((padd & PADRup)>0)
		PWorld.vx +=ONE/360;    /* Move Object */
	if((padd & PADRdown)>0)
		PWorld.vx -=ONE/360;    /* Move Object */
       
  /* Zoom */
	if((padd & PADm)>0)
	{
		DWorld.coord.t[2]-=100;   /* Move Object */
//		DebugPrintf(0, "Z: %d\n", DWorld.coord.t[2]);
	}
	if((padd & PADl)>0)
	{
		DWorld.coord.t[2]+=100;   /* Move Object */
//		DebugPrintf(0, "Z: %d\n", DWorld.coord.t[2]);
	}


#endif

#if DbViewMove  
  /* Rotate Y  */
	if((padd & PADRleft)>0) view.vpx -= 30;     /* Move Camera */
	if((padd & PADRright)>0) view.vpx += 30;     /* Move Camera */

  /* Rotate X  */
	if((padd & PADRup)>0)  view.vpy += 30;     /* Move Camera */
	if((padd & PADRdown)>0) view.vpy -= 30;      /* Move Camera */
       
  /* Zoom */
	if((padd & PADm)>0)
	{
		view.vpz += 30;            /* Move Camera */
//		DebugPrintf(0, "Z: %d\n", view.vpz);
	}
	if((padd & PADl)>0)
	{
		view.vpz -= 30;            /* Move Camera */
//		DebugPrintf(0, "Z: %d\n", view.vpz);
	}

#endif

#if DbRefMove  
  /* Rotate Y  */
	if((padd & PADRleft)>0)  view.vrx -= 5*ONE/360;     /* Move Camera */
	if((padd & PADRright)>0) view.vrx += 5*ONE/360;     /* Move Camera */

  /* Rotate X  */
	if((padd & PADRup)>0)   view.vry += 5*ONE/360;     /* Move Camera */
	if((padd & PADRdown)>0)  view.vry -= 5*ONE/360;      /* Move Camera */
       
  /* Zoom */
	if((padd & PADm)>0)
	{
		view.vrz += 50;            /* Move Camera */
//		DebugPrintf(0, "Z: %d\n", view.vrz);
	}
	if((padd & PADl)>0)
	{
		view.vrz -= 50;            /* Move Camera */
//		DebugPrintf(0, "Z: %d\n", view.vrz);
	}

#endif

  /* Translate Z */
	if((padd & PADLleft)>0) DWorld.coord.t[2] +=20;
	if((padd & PADLright)>0) DWorld.coord.t[2] -=20;
  
  /* Translate Y */
	if((padd & PADLup)>0) DWorld.coord.t[1] -=20;
	if((padd & PADLdown)>0) DWorld.coord.t[1] +=20;  /* */

  /* Translate X */
	if((padd & PADn)>0) DWorld.coord.t[0] -=20;
	if((padd & PADo)>0)	DWorld.coord.t[0] +=20;  /* */
		
		 
  /* Reset Coordinats */
	if((padd & PADh)>0)
	{
		coord_init();				   /* Initialize Coordinant system */
		view_init();				  /* Initialize View */
	}

  /* Set up new coordinats */
	set_coordinate(&PWorld,&DWorld);
	GsSetRefView2(&view);
  
  /* Set flag so it recalculates coordinates */
	DWorld.flg = 0;
}

/*************************************************************************
 * FIELD FRAME - Callback function to setup the frame stuff...
 *************************************************************************/
int FieldFrame(void)
{
	do_controls();			  /* Process controller button presses */

/*	PollPC(); /* */
//	UpdateBackground();
	DrawField();
//	padd = PadRead(1);			/* Get controller states */
	return(1);
}

/*************************************************************************
 *
 *************************************************************************/
void TestField()
{
	ResetCallback();
	InitField();
	DoFrameLoop(0, FieldFrame);
}

/*-------------------------------------------------------------
  field ~ 
-------------------------------------------------------------*/
void field(void)
{
	TestField();
}

#endif
