/****************************************************************************
 *	Fonts.h Data Structures
 ****************************************************************************/

#define CHARS_PER_FONT 80
#define NUM_STRINGS 160

typedef struct _char_poly_t {
  char 	font_num;					/* Number of font in tpage to use (0-2) */
  char	depth;						/* Depth of the polygons in string. (Z) */
  POLY_FT4	poly;						/* polygons for display */
  struct _char_poly_t *next;		/* pointer to next character */
  struct _char_poly_t *prev;		/* pointer to previous character */
} char_poly_t;


typedef struct {
  vram_t *vram; 						/* TPAGE data for fonts */
  char_poly_t *string[NUM_STRINGS];	/* Start of linked list */
  vram_t *clut[3];					/* TPAGE data for fonts */
//  char	action[NUM_STRINGS];		/* What mods are we doing to string */
//  short	act_1[NUM_STRINGS];		/* What mods are we doing to string */
//  short	act_2[NUM_STRINGS];		/* What mods are we doing to string */
  char	*spacing[NUM_STRINGS];	/* What spacing table to use */
} strings_t;


extern strings_t strings;			/* Strings definition is in font.c */

extern char	fontdpc0[];
extern char	fontdpc1[];
extern char	fontdpc2[];
extern char	fontdpc3[];
extern char	fontdpc4[];
extern char	fontdpc6[];
extern char	fontdpc7[];
extern char	fontdpc9[];
extern RECT ClipString;

/**************************************************************************
 *	Current String Data
 **************************************************************************
 *  	String states
 **************************************************************************/

#define FONT0 			0x00	/* FONT 0 */
#define FONT1 			0x01	/* FONT 1 */
#define FONT2 			0x02	/* FONT 2 */

#define TIM_FONT0 	0x03	/* FONT 0 */
#define TIM_FONT1 	0x04	/* FONT 1 */
#define TIM_FONT2 	0x05	/* FONT 2 */

#define PROPORTIONAL	0x04	/* Proportional Spacing ON */

// Not supported currently
#define UPPERCASE		0x08	/* Upper Case Chars ONLY */
#define SMALL			0x10	/* SMALL FONTS */


#define EXPAND			0x01	// ACT_1
#define FADE			0x02	// ACT_2
#define WAVE			0x04	// ACT_1

#define FONT_NUM_CLUTS 20 /*maximum font colors 0-19*/

extern vram_t *FontClutIDs[FONT_NUM_CLUTS];
extern u_long	FONT_CLUTS[][8];


/************
 * 1. Setup
 ************/
void InitFonts(void);			/* Sets up TPAGE */
void FreeFonts(void);			/* */
void LoadFont(char *file, char pos); 	/* Loads font NUM to pos (0-2) */

/************************
 * 2. Called every loop
 ************************/
void DisplayAllStrings(void);

/*********************
 * 3. String related
 *********************/
void ResetStrings(void);		/* Resets all strings to null * frees them */
void FreeString(char index);	/* Frees a single string */
void PopString(char index);		/* Frees a string & decrements string buffer ptr (arcade only) */

short CreateString(char *text, char *spacing_table, int x, int y, char flags);
short GetStrLen(char *string, char *spacing);
void CenterString(char string_num, char *string, short center);

/***********
 * 4. Misc
 ***********/
void ChangeString(char num, char *new_string, char flags);
void ChangeStringCLUT(char_poly_t *string, Ushort CLUT_ID);
void ChangeCharCLUT(char_poly_t *ch, Ushort CLUT_ID);
void ChangeStringABSXY(char_poly_t *string, int num_x, int num_y);
void ChangeStringXY(char_poly_t *string, int num_x, int num_y);
void ChangeCharXY(char_poly_t *ch, int num_x, int num_y);
void ChangeStringSize(char_poly_t *string, int size_x, int size_y);
void ChangeCharSize(char_poly_t *ch, int size_x, int size_y);
void ChangeCharFont(char_poly_t *ch, char new_font); // Note does not change spacing
short ClearString(short x, short y, short w, short h);
short CreateStringClip(char *text, char *spacing_table, int x, int y, char flags);

/*************************
 * 5. High Level Actions
 *************************/
void ExpandString(char string_num, short wide, char init);
void FadeString(char string_num, char fade_in, char init);
void WaveString(char string_num, char height, char init);

/*********
 * Local
 *********/
void DisplayString(char_poly_t *string);



