/*************************************************************
    Graphics Interface routines for PSX

 	By Christopher Deel

    Copyright 1994,97 GameTek
    All rights reserved
*************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include "prjhdrs.h"          /* Project header files */
#include "profile.h"

extern char Rewind_Switch;

DspBuf_t gDspBuf[2];
char gCurrDB = 0;		/* Current Display Buffer # */
DspBuf_t *gPtrCurrDB;	/* Pointer to Current Display Buffer Struct */
GsOT *gPtrGsOt;			/* Pointer to Current Gs Ordering Table */
ULONG *gPtrGpuOt;		/* Pointer to Current GPU Ordering Table */

char     gVidMode;	/* current video mode 640 = 640 X 480, 320 = 320 X 240 */

StdFrameFlags_t gStdFrameFlags = { True };

CVECTOR gBgColor = {0,0,0,0};

enum
{
	FADE_OFF,
	FADE_IN,
	FADE_OUT,
};

int FadeMode;
int FadeCount;
int FadeBuffer;
int FadeInterleave;
int FadeSpeed;

/*-------------------------------------------------------------
  SetBackgroundColor ~ Set The background color of the screen
-------------------------------------------------------------*/
void SetBackgroundColor(UBYTE red, UBYTE green, UBYTE blue)
{
	gBgColor.r = red;
	gBgColor.g = green;
	gBgColor.b = blue;
}

/*************************************************************************
 * SET SCREEN MODE - Set Screen Resolution
 *    Mode = 0 VMODE320 (320x240)
 *    mode = 1 VMODE512 (512x240)
 *    mode = 2 VMODE640 (640x480)
 *************************************************************************/
void SetScreenMode(char mode)
{
	int i;

	gVidMode = mode;

	FreeBackgrounds();  /* Be sure to free up previous picture space */

	VSync(0);
	ResetGraph(0);      /* reset graphic subsystem */
	switch (mode)
	{
		case VMODE640:
			GsInitGraph(640,480,2,1,0); /* */
			GsDefDispBuff(0,0,0,0); /* Single buffer setting */
			break;
		case VMODE512:
			GsInitGraph(512,240,0,0,0);
			GsDefDispBuff(0,0,0,240);  /* Double buffer setting */
			break;
		case VMODE320:
			GsInitGraph(320,240,0,0,0);
			GsDefDispBuff(0,0,0,240);
			break;
		default:
			Cerror();
			break;
	}
	GsInit3D();			/* Init 3D System */
	initVRamHandling();
}

/*----------------------------------------
    Init PS-X graphics system.
    See VRAM map for screen useage.
----------------------------------------*/
void InitGraph(void)
{
	int i;
	RECT rec;

 /* SYSTEM INITIALISE */
	ResetGraph(0);      /* reset graphic subsystem (0:cold,1:warm) */
	SetGraphDebug(0);   /* set debug mode (0:off, 1:monitor, 2:dump) */

	VSync(0);		// Clear VRAM without flashing
	setRECT(&rec, 0,0, 512, 512);
	ClearImage(&rec, 0,0,0);
	setRECT(&rec, 512,0, 512, 512);
	ClearImage(&rec, 0,0,0);
	DrawSync(0);

 	ProfileInit(2);

//	SetScreenMode(VMODE320); /* Initial screen mode = 320 X 240 */
	SetScreenMode(VMODE512);
	GsInitVcount();
	GsInit3D();

	for (i = 0; i < 2; i++)
	{
		gDspBuf[i].draw.isbg = 1;
		setRGB0(&gDspBuf[i].draw, 0, 0, 0);

		GsClearOt(0,0,&gDspBuf[i].gsot);      /* clear OT */
		gDspBuf[i].gsot.length = OT_LENGTH;
		gDspBuf[i].gsot.point  = 0;
		gDspBuf[i].gsot.offset = 0;
		gDspBuf[i].gsot.org    = (GsOT_TAG *)gDspBuf[i].ot;
		gDspBuf[i].gsot.tag    = (GsOT_TAG *)gDspBuf[i].ot + OTSIZE - 1;
	}
	DebugPrintf(0, "gDspBuf %d\n", sizeof(DspBuf_t) * 2);
}


/*-------------------------------
-------------------------------*/
void SwitchScreenOff(void)
{
	SetDispMask(0);     /* enable to display (0:inhibit, 1:enable) */
}

/*-------------------------------
-------------------------------*/
void SwitchScreenOn(void)
{
	SetDispMask(1);     /* enable to display (0:inhibit, 1:enable) */
}


/*************************************************************************

/*------------------------------------------
 Init a frame
    Set drawing to appropriate DB struct.
------------------------------------------*/
void InitFrame(void)
{
	int     i;
	GsDOBJ5 *op;					 /* pointer to Object Handler */
	MATRIX  tmpls;

/*	double_buffer_id = GsGetActiveBuff(); /* get active buffer */

	gCurrDB ^= 1;
	gPtrCurrDB = &gDspBuf[gCurrDB];	  /* Set up Global Vars */
	gPtrGsOt = &gDspBuf[gCurrDB].gsot;
	gPtrGpuOt = (ULONG*)&gDspBuf[gCurrDB].ot;
	GsSetWorkBase(gPtrCurrDB->gsprim);
	GsClearOt(0,0,gPtrGsOt);      /* clear OT */
/*	ClearOTagR(gPtrCurrDB->ot, OTSIZE);    /*##*/
}

/*-------------------------------------------
    Show frame which was being generated.
------------------------------------------*/
void ShowFrame(void)
{
	PollPC();    /*##*/

//	ResetGraph(1);           /* Reset GPU */
//	SetDispMask(1);          /* Make sure display is on */

//	GsSwapDispBuff();
	ReadProfileValue(2);
	if(SyncFlag)
	    VSync(0); 	// for Shell
	DrawSync(0);
//	VSync(0);
	GsSwapDispBuff();
	ReadProfileValue(1);
	DrawProfileBars(gPtrGpuOt, gCurrDB);
//	VSync(0);                /* Wait */
//	GsSwapDispBuff();
//	GsSortClear(0x0,0x0,0x0,gPtrGsOt); /* Clear Sceen 1st cmd */
	GsSortClear(gBgColor.r, gBgColor.g, gBgColor.b, gPtrGsOt);
	DrawOTag(gPtrGpuOt+OTSIZE-1);
	if(!SyncFlag)
	{
//	    VSync(3); //for arcade
	    VSync(0); //for arcade
	}
	ReadProfileValue(0);
	ProfileStartCount();

/*	GsDrawOt(gPtrGsOt);    /*##*/
	CheckSpinDown();
}



/*~~-------------------------
    Clear Screen
---------------------------*/
void ClearScreen(void)
{
	int i;
//	RECT Rect;

//	VSync(0);
//	Rect.x = 0;
//	Rect.y = 0;
//	Rect.w = 512;
//	Rect.h = 480;
//	ClearImage(&Rect, 0, 0, 0);    // Clear with black

	for (i=0; i<2; i++)
	{
		InitFrame();    /*##*/
//		VSync(0);
		GsSortClear(gBgColor.r, gBgColor.g, gBgColor.b, gPtrGsOt);
		ClearOTagR(gPtrCurrDB->ot, OTSIZE);    /*##*/
		VSync(0);
		ShowFrame();
	}
}

/*-------------------------------------------------------------
  ClearScreen2 ~
-------------------------------------------------------------*/
void ClearScreen2(void)
{
	RECT rect;

	VSync(0);
	setRECT(&rect, 0,0, 512,480);
	ClearImage(&rect, 0,0,0);
	DrawSync(0);
}

/*-------------------------------------------------------------
  StdFrameSub ~ Standard Frame routine
-------------------------------------------------------------*/
int StdFrameSub(void)
{
//	if (gStdFrameFlags.background)
		UpdateBackground();
	return True;
}

/*~~---------------------------------------------------------
    Loop on showing frames CGD
-----------------------------------------------------------*/
void DoFrameLoop(char num, FCNPTR func)
{
	BOOL working, finite;

	working = True;
	if (num)
		finite = True;
	else
		finite = False;
	while (working)
	{
		InitFrame();
		fade_in(0, 0);
		fade_out(0, 0);
		DisplayDialogBox();
		DisplayDialogBox();
		CheckPhraseLoad();
		ReadPads();
//		PollPC();
		if (! (*func)() )
			working = False;
		ShowFrame();
		if (finite)
		{
			num--;
			if (num < 1)
				working = False;
		}
	}
}

/*-------------------------------------------------------------
  DupDisplayBuffer ~
-------------------------------------------------------------*/
void DupDisplayBuffer(void)
{
	RECT src;
	int destX, destY;

	DISPENV dispEnv;
	DRAWENV drawEnv;
	USHORT data[512];

	GetDispEnv(&dispEnv);
	GetDrawEnv(&drawEnv);
	setRECT(&src, drawEnv.clip.x, drawEnv.clip.y,
					dispEnv.disp.w, dispEnv.disp.w);
	destX = dispEnv.disp.x;
	destY = dispEnv.disp.y;
	MoveImage(&src, destX, destY);
}  /* End DupDisplayBuffer() */

/*-------------------------------------------------------------
  StartFadeOut ~
-------------------------------------------------------------*/
void StartFadeOut(int speed, int interleave, BOOL wait)
{

	DupDisplayBuffer();
	FadeMode = FADE_OUT;
//	FadeBuffer = GsGetActiveBuff();
	FadeInterleave = interleave;
	FadeSpeed = speed;
	FadeCount = (31  / FadeSpeed);
	if (wait)
	{
		while (UpdateFade());
	}

}  /* End StartFadeOut() */



/*-------------------------------------------------------------
  FadeOutLine ~ Fade Out display line
-------------------------------------------------------------*/
void FadeOutLineData(USHORT *data, int width, int speed)
{
	register USHORT *ptr;
	register int color;
//	int red, green, blue;
	register int redMask, greenMask, blueMask;
	register int greenSub, blueSub;
	int i, spd;
//	int bitMask = 0x1E | (0x1E << 5) | (0x1E << 10);

	redMask = 0x1F;
	greenMask = 0x1F << 5;
	blueMask = 0x1F << 10;

//	redSub = amount;
	greenSub = (1 << 5) * speed;
	blueSub = (1 << 10) * speed;

	for (i = 0; i < width; i++)
	{
		color = *ptr;
//		color = color & bitMask;
//		color >>= 1;
		if ((color & redMask) >= speed)
			color -= speed;
		if ((color & greenMask) >= greenSub)
			color -= greenSub;
		if ((color & blueMask) >= blueSub)
			color -= blueSub;
		*ptr++ = color;
	}
}  /* End FadeOutLine() */

/*-------------------------------------------------------------
  UpdateFadeOut ~
-------------------------------------------------------------*/
void UpdateFadeOut(void)
{
	int y, vramX, vramY, width, height;
	int ileave, spd;
	RECT rect;
	DISPENV dispEnv;
	DRAWENV drawEnv;
	USHORT data[512];

	GetDispEnv(&dispEnv);
//	GetDrawEnv(&drawEnv);
	vramX = dispEnv.disp.x;
	vramY = dispEnv.disp.y;
//	vramX = drawEnv.clip.x;
//	vramY = drawEnv.clip.y;
	width = dispEnv.disp.w;
	height = dispEnv.disp.h;
	for (ileave = 0; ileave < FadeInterleave; ileave++)
	{
		for (y = ileave; y < height; y+= FadeInterleave)
		{
			setRECT(&rect, 0, vramY + y, width, 1);
			StoreImage(&rect, (ULONG*)&data);
			DrawSync(0);
			if (FadeCount > 2)
			{
				FadeOutLineData((USHORT*)&data, width, FadeSpeed);
			}
			else
			{
				memset(&data, 0, width * 2); // zero line
			}
//			setRECT(&rect, 0, y, width, 1);
			LoadImage(&rect, (ULONG*)&data);
//			setRECT(&rect, 0, y+240, width, 1);
//			LoadImage(&rect, &data);
		}  //for y
	}// for ileave
	if (FadeCount)
	{
		FadeCount--;
	}
	else
	{
		FadeMode = FADE_OFF;
	}
}  /* End UpdateFadeOut() */

/*-------------------------------------------------------------
  UpdateFade ~
    Return:  FadeMode  (0 = fade done)
-------------------------------------------------------------*/
int UpdateFade(void)
{
	switch (FadeMode)
	{
		case FADE_OUT:
			UpdateFadeOut();
			break;
		case FADE_IN:
//			UpdateFadeIn();
			break;
	}
	return FadeMode;
}  /* End UpdateFade() */




