/***************************************************************
	Handle New Jersey number method

	Copyright 1995,97 GameTek
	All rights reserved.

	By Christopher G. Deel
****************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include <kernel.h>

#include "prjhdrs.h"	/* Project header files */

#include "MultiFil.h"
#include "PlyrAnim.h"
#include "gameloop.h"
#include "arcade.h"
#include "Jnumber.h"
#include "debugshl.h"

#include "fbequ.h"
#include "fbram.h"
#include "fbramdat.h"

#define JNUM_BASE_U 128
#define JNUM_BASE_V 200

struct 
{
	char u, v;
} gJnumUV[22] = {0,0};

vram_t *gJnumClut[2];

#include "jnumpos.ci"	// Jersey number position tables
                            	
                              // A, B, C, D, E
JnumPos_t *JnumPosDirRUN[5] = {(void*)&JnumPosRUNA, 0,0,0, (void*)&JnumPosRUNE };
JnumPos_t *JnumPosDirTHR[8] = {(void*)JnumPosTHRA, 0,0,0, (void*)JnumPosTHRE ,0,0,0};
JnumPos_t *JnumPosDirBAK[8] =	{(void*)JnumPosBAKA ,0,0,0, (void*)JnumPosBAKE, 0,0,0 };
//JnumPos_t *JnumPosDirHFG[5] =	{(void*)JnumPosHFGA ,0,0,0, (void*)JnumPosHFGE };
JnumPos_t *JnumPosDirHFG[8] =	{(void*)JnumPosHFGA ,0,0,0, 0, 0,0,0 };

JnumPos_t *JnumPosDir4PS[5] =	{(void*)JnumPos4PSA, 0,0,0, (void*)JnumPos4PSA};
JnumPos_t *JnumPosDirBAD[5] =	{(void*)JnumPosBADA ,0,0,0, (void*)JnumPosBADE };
JnumPos_t *JnumPosDirBLK[5] =	{(void*)JnumPosBLKA ,0,0,0, (void*)JnumPosBLKE };
//JnumPos_t *JnumPosDirBRS[5] =	{(void*)JnumPosBRSA ,0,0,0, (void*)JnumPosBRSE };
//JnumPos_t *JnumPosDirDIV[5] =	{(void*)JnumPosDIVA ,0,0,0, (void*)JnumPosDIVE };
//JnumPos_t *JnumPosDirDM1[5] =	{(void*)JnumPosDM1A ,0,0,0, (void*)JnumPosDM1E };
//JnumPos_t *JnumPosDirDM2[5] =	{(void*)JnumPosDM2A ,0,0,0, (void*)JnumPosDM2E };
//JnumPos_t *JnumPosDirDM3[5] =	{(void*)JnumPosDM3A ,0,0,0, (void*)JnumPosDM3E };
//JnumPos_t *JnumPosDirDM4[5] =	{(void*)JnumPosDM4A ,0,0,0, (void*)JnumPosDM4E };
//JnumPos_t *JnumPosDirDM5[5] =	{(void*)JnumPosDM5A ,0,0,0, (void*)JnumPosDM5E };
//JnumPos_t *JnumPosDirDSS[5] =	{(void*)JnumPosDSSA, 0,0,0, (void*)JnumPosDSSE };
//JnumPos_t *JnumPosDirDT1[5] =	{(void*)JnumPosDT1A ,0,0,0, (void*)JnumPosDT1E };
//JnumPos_t *JnumPosDirDT2[5] =	{(void*)JnumPosDT2A ,0,0,0, (void*)JnumPosDT2E };
//JnumPos_t *JnumPosDirDT3[5] =	{(void*)JnumPosDT3A ,0,0,0, (void*)JnumPosDT3E };
JnumPos_t *JnumPosDirFBB[5] =	{(void*)JnumPosFBBA ,0,0,0, (void*)JnumPosFBBE };
//JnumPos_t *JnumPosDirFBF[5] =	{(void*)JnumPosFBFA ,0,0,0, (void*)JnumPosFBFE };
//JnumPos_t *JnumPosDirFBR[5] =	{(void*)JnumPosFBRA ,0,0,0, (void*)JnumPosFBRE };
//JnumPos_t *JnumPosDirFFF[5] =	{(void*)JnumPosFFFA ,0,0,0, (void*)JnumPosFFFE };
//JnumPos_t *JnumPosDirFFR[5] =	{(void*)JnumPosFFRA ,0,0,0, (void*)JnumPosFFRE };
//JnumPos_t *JnumPosDirFFS[5] =	{(void*)JnumPosFFSA ,0,0,0, (void*)JnumPosFFSE };
//JnumPos_t *JnumPosDirHND[5] =	{(void*)JnumPosHNDA ,0,0,0, (void*)JnumPosHNDE };
//JnumPos_t *JnumPosDirHRD[5] =	{(void*)JnumPosHRDA ,0,0,0, (void*)JnumPosHRDE };
JnumPos_t *JnumPosDirJBK[5] =	{(void*)JnumPosJBKA ,0,0,0, (void*)JnumPosJBKE };
//JnumPos_t *JnumPosDirJCT[5] =	{(void*)JnumPosJCTA ,0,0,0, (void*)JnumPosJCTE };
JnumPos_t *JnumPosDirKCK[5] =	{(void*)JnumPosKCKA ,0,0,0, (void*)JnumPosKCKE };
//JnumPos_t *JnumPosDirPIT[5] =	{(void*)JnumPosPITA ,0,0,0, (void*)JnumPosPITE };
JnumPos_t *JnumPosDirPNT[5] =	{(void*)JnumPosPNTA ,0,0,0, (void*)JnumPosPNTE };
//JnumPos_t *JnumPosDirQBC[5] =	{(void*)JnumPosQBCA ,0,0,0, (void*)JnumPosQBCE };
//JnumPos_t *JnumPosDirQBS[5] =	{(void*)JnumPosQBSA ,0,0,0, (void*)JnumPosQBSE };
//JnumPos_t *JnumPosDirRHT[5] =	{(void*)JnumPosRHTA ,0,0,0, (void*)JnumPosRHTE };
//JnumPos_t *JnumPosDirROL[5] =	{(void*)JnumPosROLA ,0,0,0, (void*)JnumPosROLE };
//JnumPos_t *JnumPosDirRUB[5] =	{(void*)JnumPosRUBA ,0,0,0, (void*)JnumPosRUBE };
JnumPos_t *JnumPosDirRUC[5] =	{(void*)JnumPosRUCA ,0,0,0, (void*)JnumPosRUCE };
//JnumPos_t *JnumPosDirSTB[5] =	{(void*)JnumPosSTBA ,0,0,0, (void*)JnumPosSTBE };
//JnumPos_t *JnumPosDirSTC[5] =	{(void*)JnumPosSTCA ,0,0,0, (void*)JnumPosSTCE };
JnumPos_t *JnumPosDirSTD[5] =	{(void*)JnumPosSTDA ,0,0,0, (void*)JnumPosSTDE };
JnumPos_t *JnumPosDirTKL[5] =	{(void*)JnumPosTKLA ,0,0,0, (void*)JnumPosTKLE };
//JnumPos_t *JnumPosDirWRS[5] =	{(void*)JnumPosWRSA ,0,0,0, (void*)JnumPosWRSE };

/*-------------------------------------------------------------
  SetJerseyNumberFrame ~ 
-------------------------------------------------------------*/
void SetJerseyNumberFrame(int plyrNum, int anim, int dir, int flip, int frame)
{
	int jnumOn, jnumX, jnumY, dir2;
	ULONG **posDir;
	ULONG **posAnim;
	JnumPos_t *pos;
	
// 	 Dprintf("JnumPos: %X  %X\n", JnumPosDirRUN, JnumPosRUNA);
 	dir2 = gAnimInfo[anim].dirMult[dir];
//	 dir2 = 0;  //debug
 	jnumOn = FALSE;
 	posDir = (void**)gAnimInfo[anim].jnumPosDir;
 	if (posDir)
 	{
	 	posAnim = (void**)(posDir)[dir2];
	 	if (posAnim)
	 	{
	 		pos = (JnumPos_t*)posAnim + frame ;
	 		
	 		 if (!CheckValidAddress(pos, 2))
	 		 {
		 		#if PC_DEBUG
	 			Dprintf("anim: %d, dir:%d,%d, frame:%d pos: %X ",
	 						anim,dir,dir2,frame, pos);
	 			Cerror();
		 		#endif
		 		return;   //major error
	 		 }
	 		
	 		gPlayerSprite[plyrNum].jnumX = jnumX = pos->x;
	 		gPlayerSprite[plyrNum].jnumY = jnumY = pos->y;
 		
	 		if (jnumX | jnumY)
	 		{
	 			jnumOn = TRUE;
	 		}
	 	}
	}
 	
	if (DebugShellVar[DEBUG_SHELL_JNUM].Value == JNUM_DEBUG_ALL_OFF)
		jnumOn = FALSE;  // debug
	
	SetPolySpriteOnOff(jnumOn, gPlayerSprite[plyrNum].jnumSprite);
}  /* End SetJerseyNumberFrame() */


/*-------------------------------------------------------------
  SetJerseyNumberPosition ~ 
-------------------------------------------------------------*/
void SetJerseyNumberPosition(int x, int y, int z, int plyrNum)
{
	int width, height;
	int addY, addX;
	int jnumX, jnumY;
	
	width = gPlayerSprite[plyrNum].sprite->width;
	height = gPlayerSprite[plyrNum].sprite->height;
	jnumY = gPlayerSprite[plyrNum].jnumY;
	jnumX = gPlayerSprite[plyrNum].jnumX;
	addY = ((jnumY * height) / 64) - 32;  // -32??
//	addY -= (60 + 16);
	addX = ((jnumX * width) / 64) - width/2; 

//	 addY = 0; //debug	
//	SetPolySpritePos(0, SPRITE_GROUND_Y - 60, 0, sprite);
	SetPolySpritePos(addX, -addY, 0, gPlayerSprite[plyrNum].jnumSprite);
	MovePolySpriteTo(x, y, z, gPlayerSprite[plyrNum].jnumSprite);
}  /* End SetJerseyNumberPosition() */

/*-------------------------------------------------------------
  InitJnumUVTable ~ 
-------------------------------------------------------------*/
void InitJnumUVTable(void)
{
	int i;
	
	for (i = 0; i < NUM_PLAYERS_ON_FIELD -1; i++)
    {
		gJnumUV[i].u = JNUM_BASE_U + ((i % 7) * 16);
		gJnumUV[i].v = JNUM_BASE_V + ((i / 7) * 16);
	}
	gJnumUV[21].u = JNUM_BASE_U + 64;
	gJnumUV[21].v = JNUM_BASE_V - 16;
	
}  /* End InitJnumUVTable() */

/*-------------------------------------------------------------
  SetJerseyNumberCluts ~ 
-------------------------------------------------------------*/
void SetJerseyNumberCluts(void)
{
	int i, team;
	int swapTeams = FALSE;
	USHORT id[2];
	
	if (!IsSpriteUsed(gPlayerSprite[0].jnumSprite))	// safety net
		return;
		
	if (GameTeamData[0].OffDefFlag != ON_OFFENSE)
			swapTeams = TRUE;  // Swap team colors
	
	id[0] = gJnumClut[0]->CLUT_ID;
	id[1] = gJnumClut[1]->CLUT_ID;
	for (i = 0; i < NUM_PLAYERS_ON_FIELD; i++)
	{
		team = i / 11;
		if (swapTeams)
			team ^= 1;
		SetPolySpriteClut(id[team], gPlayerSprite[i].jnumSprite);
	}
}  

/*-------------------------------------------------------------
  SetJerseyNumberValues ~ 
-------------------------------------------------------------*/
void SetJerseyNumberValues(void)
{
	int i, pageX, pageY, u, v, val;
	char *buff, *cmp;
	RECT rect;
	
	if (gAnimBuff)
		buff = gAnimBuff;
	else
		buff = AllocMem(16 * 9);
	pageX = gAnimPage[2]->x;
	pageY = gAnimPage[2]->y;
	if (!gJnumUV[0].u)
		InitJnumUVTable();
	for (i = 0; i < NUM_PLAYERS_ON_FIELD; i++)
    {
		u = gJnumUV[i].u;
		v = gJnumUV[i].v;
		val = GetJerseyNum(i);
		 if (DebugShellVar[DEBUG_SHELL_JNUM].Value == JNUM_DEBUG_PLAYER_NUM)
		 	val = i;  // debug
		val %= 100;
		cmp = GetMultiFileByNum(gJnumPicPtr, val);
		if (cmp)
		{
			DecompressFrameBytes(buff, cmp);
			setRECT(&rect, pageX + u/4, pageY + v, 16/4, 16);
			LoadImage(&rect, buff);
		}
	}
	if (!gAnimBuff)
		FreeMem(buff);
}  /* End SetJerseyNumberValues() */

/*-------------------------------------------------------------
  InitJerseyNumberSprites ~ 
-------------------------------------------------------------*/
void InitJerseyNumberSprites(void)
{
	PolySprite_t *sprite;
	SubTexture_t subtex;
	int i; 
	int u, v, texW = 16, texH = 16; 
	int spriteW , spriteH;
	int pageX, pageY;
	int team;
	USHORT tpageId, clutId, color;
	vram_t *clut;
	static USHORT clutData[16];
	
//	 DebugShellVar[DEBUG_SHELL_JNUM].Value = JNUM_DEBUG_ALL_OFF;	//debug
	for (team = 0; team < 2; team++)
	{
		if (team == 0)
			color = gTeamClutData1[CLUT_JNUM];
		else
			color = gTeamClutData2[CLUT_JNUM];
			
		for (i = 0; i < 16; i++)
		{
			clutData[i] = color;
		}
		clutData[0] = 0;
//		 clutData[0] = GenPSXColor(40, 255, 40, 0);  //Debug  green
		clut = alloc_VRam(VRAM_CLUT16, 0);
		Reload4BitClut(clutData, clut);
		gJnumClut[team] = clut;		
	}
	pageX = gAnimPage[2]->x;
	pageY = gAnimPage[2]->y;
	tpageId = GetTPage(0, 0, pageX, pageY);
	spriteW = (DefaultPlayerWidth / 7) + 3;
	spriteH = (DefaultPlayerHeight / 7) + 2;
	clutId = clut->CLUT_ID;
	InitJnumUVTable();
	for (i = 0; i < NUM_PLAYERS_ON_FIELD; i++)	/* Create sprites */
	{
		u = gJnumUV[i].u;
		v = gJnumUV[i].v;
		sprite = CreatePolySprite(spriteW, spriteH);
		TexturePolySprite(tpageId, u,v,	texW, texH,	clutId, sprite);
		SetPolySpriteFlags(PSPRITE_JNUM, TRUE, sprite);
		SetPolySpritePos(0, SPRITE_GROUND_Y - 60, 0, sprite);
		gPlayerSprite[i].jnumSprite = sprite;
		 if (DebugShellVar[DEBUG_SHELL_JNUM].Value == JNUM_DEBUG_ALL_OFF)
			SetPolySpriteOnOff(FALSE, gPlayerSprite[i].jnumSprite);
	}
	SetJerseyNumberCluts();
		
//		u = 128;	//debug
//		v = 128;
//		sprite = CreatePolySprite(200, 100);
//		TexturePolySprite(tpageId, u,v,	127, 127,	clutId, sprite);
//		SetPolySpriteFlags(PSPRITE_2D, TRUE, sprite);
//		MovePolySpriteTo(0, 0, 0, sprite);
}  /* End InitJerseyNumberSprites() */
