/*************************************************************************
 *	LOWSHELL.C for the PSX.
 *
 *	Copyright 1995 GAMETEK
 *	All rights reserved.
 *
 * Written by Mike Heilemann
 * Started: 3-31-95
 *
 *	Notes: Low level shell polygon support & texture routines.
 *
 *************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include <strings.h>
#include <memory.h>

#include "prjhdrs.h"

char in_arcade = 0;	// 1 = in arcade portion of shell, 0 = not

char num_templates = 0;
template_t template[NUM_TEMPLATES];		// 0 - 8 tpages... should be enough...
//spin_t spin;
unsigned long *multifile;					// Multifile in use
char *TemplateBuff = NULL;
char FlipKludge = 1;

RECT clearRect;

/*************************************************************************
 * INIT TEMPLATE - Inits all templates...
 *************************************************************************/
void InitTemplates(void)
{
	int i;

	in_arcade = 0;

	for (i = 0; i < NUM_TEMPLATES; i++)
	{
		free_VRam(template[i].vram);
		template[i].vram = NULL;
	}
	num_templates = 0;

//	for (i = 0; i < MAX_SPIN; i++)
//		spin.used[i] = 0;
}

/*************************************************************************
 * LOAD TEMPLATE TO - Sets up one Tpage and polygons for one template...
 *
 *	Note: For 256 colors width must be evenly divisible by 4 and 16 color
 *		pictures must be divisible by 8, due to compression.
 *************************************************************************/
void LoadTemplateTo(char *file, int num)
{
	int i = 0,
		 mul = 2;
	register int u,
					 v,
					 w,
					 h;
	char *buffer;
	char s[80];									// for temporary file name manipulation

	RECT rect;
	unsigned long *clut_ptr;
	vram_t *clut;
	t_entry_t  *t_entry;						// Template file, XYUVWH, Clut Info...


	template[num].vram = alloc_VRam(VRAM_TEXTURE, 0); /* Get template TPAGE */

	(unsigned long *)t_entry = GetMultiFilePtr(multifile, file);
	if (TemplateBuff)
		buffer = TemplateBuff;
	else
		buffer = (char *)AllocMem(128 * 256);

	while (t_entry->filename[0] != 0xFF)// 0xFF marks End Of File
	{
		template[num].clut_size[i] = t_entry->clut_size;
		if (t_entry->clut_size == 1)	// if it is 16 colors (4 bit)...
		{
			mul = 4;
		}

		u = t_entry->u;
		v = t_entry->v;
		w = t_entry->w;
		h = t_entry->h;

		setPolyFT4(&template[num].poly[i]);
		template[num].poly[i].tpage = GetTPage(t_entry->clut_size-1, 0, template[num].vram->x, template[num].vram->y);

		// SET TRANSPARENCY MODE
		setRGB0(&template[num].poly[i], 0x80, 0x80, 0x80);
		setSemiTrans(&template[num].poly[i], 0);
//		setShadeTex(&template[num].poly[i], 1);

	   setXYWH(&template[num].poly[i], t_entry->x - 256, t_entry->y - 120, w, h);	// -256 & -120 to put at 0, 0 in upper left corner
		if (v + h == 256)	// Crashes under -O3 compile optimization
			if ((u + w == 256 && t_entry->clut_size == 1) || (u + w == 128 && t_entry->clut_size != 1))
				setUVWH(&template[num].poly[i], u, v, w-1, h-1); /* Point to texture... */
			else
				setUVWH(&template[num].poly[i], u, v, w, h-1); /* Point to texture... */
		else
			if ((u + w == 256 && t_entry->clut_size == 1) || (u + w == 128 && t_entry->clut_size != 1))
				setUVWH(&template[num].poly[i], u, v, w-1, h); /* Point to texture... */
			else
				setUVWH(&template[num].poly[i], u, v, w, h); /* Point to texture... */

		template[num].action[i] = DISPLAY;		// Default is display it...
		template[num].depth[i] = t_entry->depth;		// Default is display it...

		rect.x = template[num].vram->x + (u/mul); 	// Use our info...
		rect.y = template[num].vram->y + v;
		rect.w = w/mul;						// For LoadImage
		rect.h = h;

		strcpy(s, t_entry->filename);		// Compressed file name
		strcat(s, ".CMP");

		DecompressFrameBytes((unsigned long *)buffer, GetMultiFilePtr(multifile, s));
		LoadImage(&rect, (unsigned long *)buffer);

// CLUT STUFF
		template[num].clut[i] = alloc_VRam(t_entry->clut_size, 0);	// Allocate a CLUT
  		strcpy(s, t_entry->filename);
  		strcat(s, ".CLT");
  		clut_ptr = (unsigned long *)GetMultiFilePtr(multifile, s);

		if (template[num].clut_size[i] == 1)	// if it is 16 colors (4 bit)...
		{
			rect.x = template[num].clut[i]->x + 16;
			rect.y = template[num].clut[i]->y;
			rect.w = 240;
			rect.h = 1;
			StoreImage(&rect, (unsigned long *)buffer);		// Load it, cos LOAD CLUT ONLY LOADS 256 even in 4 bit mode
		}

		template[num].clut[i]->CLUT_ID = LoadClut(clut_ptr, template[num].clut[i]->x, template[num].clut[i]->y);
		template[num].poly[i].clut = template[num].clut[i]->CLUT_ID;

		if (template[num].clut_size[i] == 1)	// if it is 16 colors (4 bit)...
			LoadImage(&rect, (unsigned long *)buffer);		// Store it back

		t_entry++;
		i++;
	}

	DrawSync(0);
	if (!TemplateBuff)
		FreeMem(buffer);
	template[num].action[i] = END_OF_POLYS;
}


/*************************************************************************
 * LOAD TEMPLATE - Sets up one Tpage and polygons for one template...
 *
 *	Note: For 256 colors width must be evenly divisible by 4 and 16 color
 *		pictures must be divisible by 8, due to compression.
 *************************************************************************/
void LoadTemplate(char *file)
{
	int num = 0;


	while(template[num].vram)				// Find first template.
		num++;

	if (num == NUM_TEMPLATES)
		return;

	LoadTemplateTo(file, num);
}


/*************************************************************************
 * FREE TEMPLATE - Frees a template from VRAM.
 *************************************************************************/
void FreeTemplate(char num)
{
	int i;

	if (template[num].vram)
	{
		free_VRam(template[num].vram);
		for (i = 0; template[num].action[i] != END_OF_POLYS; i++)
			free_VRam(template[num].clut[i]);
		template[num].vram = NULL;				// Mark it unused...
		num_templates--;
	}
}


/*************************************************************************
 * FREE ALL TEMPLATES - Frees a template from VRAM.
 *************************************************************************/
void FreeAllTemplates(void)
{
	int num;

	for (num = 0; num < NUM_TEMPLATES; num++)
		FreeTemplate(num);
}


/*************************************************************************
 * RESTORE TEMPLATE - Marks template tpage as used.
 *************************************************************************/
void RestoreTemplate(char num)
{
	template[num].vram->used = 1;
}


/*************************************************************************
 * DISPLAY SHELL POLYS - Displays template polygons for all templates.
 *************************************************************************/
void DisplayShellPolys(void)
{
	register int i,
		 			 j;
	register template_t *ptr;

	for (i = 0; i < NUM_TEMPLATES; i++)
		if(template[i].vram)	// If there is a template here...
		{
			ptr = &template[i];
			j = 0;
			while(!(ptr->action[j] & END_OF_POLYS))	// We at the end yet???
			{
				if (ptr->action[j] & DISPLAY)	// If we wanna display it
				{

// CHECK OTHER ACTIONS HERE	// Disable ones not used so we stay in I-CACHE
//					if (ptr->action[j] & FADE)	// If we are fading in/out
//						FadePoly(i, j, 0, 0);

//					if (ptr->action[j] & SPIN && !(ptr->action[j] & FLIP))	// Set it up if it has stopped
//						FlipPoly(i, j, 0, 0, 0, 1);

//					if (ptr->action[j] & FLIP)	// If we are Spinning
//						FlipPoly(i, j, 0, 0, 0, 0);

					GsSortPoly(&ptr->poly[j], gPtrGsOt, ptr->depth[j]); /* Poly is first entry */
				}
				j++;
			}
		}
}


/*************************************************************************
 * CHANGE POLY TEX - Changes texture of the polygon
 *************************************************************************/
void ChangePolyTex(short num, short index, char *buffer)
{
	RECT rect;
	char *buf;

	if (TemplateBuff)
		buf = TemplateBuff;
	else
		buf = (char *)AllocMem(128 * 256);

	DecompressFrameBytes((unsigned long *)buf, (unsigned long *)buffer);

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 4;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 4;
	}
	else
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 2;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 2;
	}
	rect.y = template[num].vram->y + template[num].poly[index].v0;
	rect.h = template[num].poly[index].v2 - template[num].poly[index].v0;

	LoadImage(&rect, (unsigned long *)buf);
	DrawSync(0);
	if (!TemplateBuff)
		FreeMem(buf);
}


/*************************************************************************
 * CHANGE POLY TEXTURE - Changes texture of the polygon
 *************************************************************************/
void ChangePolyTexture(short num, short index, char *buffer)
{
	RECT rect;
	char *buf;

	if (TemplateBuff)
		buf = TemplateBuff;
	else
		buf = (char *)AllocMem(128 * 256);

	DecompressFrameBytes((unsigned long *)buf, GetMultiFilePtr(multifile, buffer));

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 4;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 4;
	}
	else
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 2;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 2;
	}
	rect.y = template[num].vram->y + template[num].poly[index].v0;
	rect.h = template[num].poly[index].v2 - template[num].poly[index].v0;

	LoadImage(&rect, (unsigned long *)buf);
	DrawSync(0);
	if (!TemplateBuff)
		FreeMem(buf);
}


/*************************************************************************
 * CHANGE POLY CMP TEXTURE - Changes texture of the polygon
 *************************************************************************/
void ChangePolyCmpTexture(short num, short index, char *file)
{
	RECT rect;
	char *buffer,
        *buf;

	if (TemplateBuff)
		buffer = TemplateBuff;
	else
		buffer = (char *)AllocMem(128 * 256);

	*strchr(file, '.') = NULL;	// Take extension off...
	strcat(file, ".CMP");
	DecompressFrameBytes((unsigned long *)buffer, GetMultiFilePtr(multifile, file));

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 4;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 4;	// +1 to take care of rounding errors
	}
	else
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 2;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 2;
	}
	rect.y = template[num].vram->y + template[num].poly[index].v0;
	rect.h = template[num].poly[index].v2 - template[num].poly[index].v0;

	LoadImage(&rect, (unsigned long *)buffer);

// DO CLUT
	*strchr(file, '.') = NULL;	// Take extension off...
	strcat(file, ".CLT");
	(unsigned long *)buf = GetMultiFilePtr(multifile, file);

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
	{
		rect.x = template[num].clut[index]->x + 16;
		rect.y = template[num].clut[index]->y;
		rect.w = 240;
		rect.h = 1;
		StoreImage(&rect, (unsigned long *)buffer);
	}

	template[num].clut[index]->CLUT_ID = LoadClut((unsigned long *)buf, template[num].clut[index]->x, template[num].clut[index]->y);
	template[num].poly[index].clut = template[num].clut[index]->CLUT_ID;

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
		LoadImage(&rect, (unsigned long *)buffer);

	DrawSync(0);
	if (!TemplateBuff)
		FreeMem(buffer);
}



/*************************************************************************
 * CHANGE POLY CMP TEXTURE BUFF - Changes texture of the polygon using a
 *		passed in buffer.
 *************************************************************************/
void ChangePolyTextureBuff(short num, short index, char *buffer)
{
	RECT rect;
   char *buf;

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 4;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 4;	// +1 to take care of rounding errors
	}
	else
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 2;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 2;
	}
	rect.y = template[num].vram->y + template[num].poly[index].v0;
	rect.h = template[num].poly[index].v2 - template[num].poly[index].v0;

	LoadImage(&rect, (unsigned long *)buffer);
}



/*************************************************************************
 * CHANGE POLY TIM TEXTURE - Changes texture of the polygon
 *************************************************************************/
void ChangePolyTimTexture(short num, short index, unsigned long *texture)
{
	RECT rect;
	GsIMAGE image;
	char *buffer;


	buffer = (char *)AllocMem(256);
	texture++;
	GsGetTimInfo(texture, &image);

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 4;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 4;	// +1 to take care of rounding errors
	}
	else
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 2;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 2;
	}
	rect.y = template[num].vram->y + template[num].poly[index].v0;
	rect.h = template[num].poly[index].v2 - template[num].poly[index].v0;

	LoadImage(&rect, image.pixel);

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
	{
		rect.x = template[num].clut[index]->x + 16;
		rect.y = template[num].clut[index]->y;
		rect.w = 240;
		rect.h = 1;
		StoreImage(&rect, (unsigned long *)buffer);
	}

	template[num].clut[index]->CLUT_ID = LoadClut(image.clut, template[num].clut[index]->x, template[num].clut[index]->y);
	template[num].poly[index].clut = template[num].clut[index]->CLUT_ID;

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
		LoadImage(&rect, (unsigned long *)buffer);

	DrawSync(0);
	FreeMem(buffer);
}


/*************************************************************************
 * SET POLY W H - change polygon W, H
 *************************************************************************/
void ChangePolyWH(POLY_FT4 *poly, short num_w, short num_h)
{
	int x,
		 y,
		 w,
		 h;

/* Get poly X, Y, W & H */
	x = poly->x0;
	y = poly->y0;
	w = poly->x1 - poly->x0;
	h = poly->y2 - poly->y0;

   setXYWH(poly, x, y, w + num_w, h + num_h);
}


// not really even needed... but I put it in for consistancy.
/*************************************************************************
 * SET POLY X Y W H - change polygon X, Y, W, H
 *************************************************************************/
void SetPolyXYWH(POLY_FT4 *poly, short num_x, short num_y, short num_w, short num_h)
{
   setXYWH(poly, num_x, num_y, num_w, num_h);
}



/*************************************************************************
 * MOVE POLY TO X Y
 *************************************************************************/
void MovePolyToXY(POLY_FT4 *poly, short num_x, short num_y)
{
	int w,
		 h;

/* Get poly W & H */
	w = poly->x1 - poly->x0;
	h = poly->y2 - poly->y0;

   setXYWH(poly, num_x, num_y, w, h);
}


/*************************************************************************
 * MOVE POLY X Y
 *************************************************************************/
void MovePolyXY(POLY_FT4 *poly, short num_x, short num_y)
{
	int x,
		 y,
		 w,
		 h;

/* Get poly X, Y, W & H */
	x = poly->x0;
	y = poly->y0;
	w = poly->x1 - poly->x0;
	h = poly->y2 - poly->y0;

   setXYWH(poly, x + num_x, y + num_y, w, h);
}


/*************************************************************************
 * FLIP POLY X - Flips x coordinants to display reversed on x axis
 *************************************************************************/
void FlipPolyX(POLY_FT4 *poly)
{
	int x,
		 w,
		 uw;

	uw = poly->u1 - poly->u0;
	if (poly->u0 == 0 && FlipKludge)
	{
		poly->u0++;
		uw -= 2;
		setUVWH(poly, poly->u0, poly->v0, uw, poly->v2 - poly->v0);
	}

	x = poly->x1;
	w = poly->x0 - poly->x1;

	setXYWH(poly, x, poly->y0, w, poly->y2 - poly->y0);
}


/*************************************************************************
 * CHANGE POLY SIZE - Increments or decrements current size
 *************************************************************************/
void ChangePolySize(POLY_FT4 *poly, short size_x, short size_y)
{
	int w,
		 h;

/* Get poly X, Y, W & H */
	w = poly->x1 - poly->x0;
	h = poly->y2 - poly->y0;

/* Add to X... for both polys */
   setXYWH(poly, poly->x0, poly->y0, w + size_x, h + size_y);
}


/*************************************************************************
 * CHANGE POLY 4 CLUT - Changes CLUT pointer to a new VRAM_T CLUT entry.
 *************************************************************************/
void ChangePoly4CLUT(short num, short index, unsigned long *clut)
{
	template[num].clut[index]->CLUT_ID = Load4BitClut(clut, template[num].clut[index]->x, template[num].clut[index]->y);
	template[num].poly[index].clut = template[num].clut[index]->CLUT_ID;
}


/*************************************************************************
 * CHANGE POLY CLUT - Changes CLUT pointer to a new VRAM_T CLUT entry.
 *************************************************************************/
void ChangePolyCLUT(short num, short index, unsigned long *clut)
{
	template[num].clut[index]->CLUT_ID = LoadClut(clut, template[num].clut[index]->x, template[num].clut[index]->y);
	template[num].poly[index].clut = template[num].clut[index]->CLUT_ID;
}


#if 0
/*************************************************************************
 * FADE POLY - Fades a polygon in or out.  This function uses the polygon
 *		RGB values for the fade.
 *
 *		NOTE: This function uses act_2.
 *				DO NOT USE WITH FLIP...
 *************************************************************************/
void FadePoly(short num, short index, char fade_in, char init)
{
   int			i;
	char_poly_t *ptr;

	if (init)					// Set fade up
	{
		template[num].action[index] ^= FADE;
		template[num].act_2[index] = fade_in;

		setShadeTex(&ptr->poly, 0);
		if (fade_in)
		{
			template[num].poly[index].r0 = 0;
			template[num].poly[index].g0 = 0;
			template[num].poly[index].b0 = 0;
		}
		return;
	}

	if (template[num].act_2[index]) /* if we are fading in from black */
	{
		i = 5;
		if (template[num].poly[index].r0 > 0x80)
		{
			template[num].action[index] |= FADE; /* stop fading in by turning FADE action off*/
			setShadeTex(&template[num].poly[index], 1); /* Once faded in, turn off shading */
			return;
		}
	} else
	{
		i = -5;
		if (template[num].poly[index].r0 + i < 0)
		{
			template[num].action[index] |= FADE; /* stop fading out by turning FADE action off */
			return;
		}
	}

	template[num].poly[index].r0 += i;
	template[num].poly[index].g0 += i;
	template[num].poly[index].b0 += i;
}


/*************************************************************************
 * FLIP POLY - Flips a polygon over...
 *
 *	Note: don't us fading with this function because it uses act_2.
 *
 *		dir = -1 - clockwise
 *		dir =  0 - use previous direction
 *		dir =  1 - counter clockwise
 *************************************************************************/
void FlipPoly(short num, short index, char dir, char *front, char *back, char init)
{
	POLY_FT4 *ptr;
	signed int  dir1,
			dir2,
			y;
	static unsigned long *texture[2];
	static unsigned long *clut[2];
	int x = 0;

	if (init)
	{
		template[num].action[index] ^= FLIP;
		if (dir)	 // Keep the old one if multiple spin
		{
			x = 0;
			while(spin.used[x])
				x++;

			template[num].act_1[index] = (dir << 8) + dir;
			template[num].act_2[index] = x;
			spin.used[x]++;
			spin.w[x] = template[num].poly[index].x1 - template[num].poly[index].x0;
			spin.h[x] = template[num].poly[index].y2 - template[num].poly[index].y0;

			*strchr(front, '.') = NULL;	// Take extension off...
			strcat(front, ".CMP");
			spin.texture[x][0] = (unsigned long *)GetMultiFilePtr(multifile, front);
			*strchr(back, '.') = NULL;	// Take extension off...
			strcat(back, ".CMP");
			spin.texture[x][1] = (unsigned long *)GetMultiFilePtr(multifile, back);

			*strchr(front, '.') = NULL;	// Take extension off...
			strcat(front, ".CLT");
			spin.clut[x][0] = (unsigned long *)GetMultiFilePtr(multifile, front);
			*strchr(back, '.') = NULL;	// Take extension off...
			strcat(back, ".CLT");
			spin.clut[x][1] = (unsigned long *)GetMultiFilePtr(multifile, back);
		}

		return;
	}

	ptr = &template[num].poly[index];

	dir1 = template[num].act_1[index] >> 8;
	dir2 = template[num].act_1[index] & 0x00FF;
	x = template[num].act_2[index];
	y = spin.h[x];
//	if (ptr->x1 - ptr->x0 > -template[num].act_2[index] >> 8 || ptr->x1 - ptr->x0 < template[num].act_2[index] >> 8)
	if (ptr->x1 - ptr->x0 >= spin.w[x] || ptr->x1 - ptr->x0 <= -spin.w[x])
	{

		while (template[num].poly[index].y2 - template[num].poly[index].y0 < y) // Be sure to go back to original height
		{
			ptr->y0 += dir2;
			ptr->y1 -= dir2;
			ptr->y2 -= dir2;
			ptr->y3 += dir2;
		}
		while(template[num].poly[index].y2 - template[num].poly[index].y0 > y)
		{
			ptr->y0 -= dir2;
			ptr->y1 += dir2;
			ptr->y2 += dir2;
			ptr->y3 -= dir2;
		}

		dir1 *= -1;
		template[num].act_1[index] = (dir1 << 8) + (template[num].act_1[index] & 0x00FF);
		template[num].action[index] ^= FLIP;
	}

	ptr->x0 += 8 * dir1;
	ptr->x1 -= 8 * dir1;
	ptr->x2 += 8 * dir1;
	ptr->x3 -= 8 * dir1;

	if (ptr->x1 - ptr->x0 > 0)	// if it is > 0
	{
		ptr->y0 += dir2;
		ptr->y1 -= dir2;
		ptr->y2 -= dir2;
		ptr->y3 += dir2;
	}
	else
	if (ptr->x1 - ptr->x0 < 4 && ptr->x1 - ptr->x0 > -4)	// if it is == 0
	{
		ChangePolyTexture(num, index, (char *)spin.texture[x][spin.dir[x]]);
		ChangePolyCLUT(num, index, spin.clut[x][spin.dir[x]]);
		spin.dir[x] ^= 1;
	}
	else
	{
		ptr->y0 -= dir2;
		ptr->y1 += dir2;
		ptr->y2 += dir2;
		ptr->y3 -= dir2;
	}
}
#endif


/*************************************************************************
 * COPY POLY - Copys one Polygon info to another.
 *************************************************************************/
//void CopyPoly(POLY_FT4 *new, POLY_FT4 *old)
short CopyPoly(short num, short index)
{
	int i;

	// NOTE: Don't exceed 32 polys or this will blow
	for (i = 0; i < NUM_POLYS; i++)
		if (template[num].action[i] & END_OF_POLYS)
			break;

	template[num].action[i+1] = END_OF_POLYS;
	template[num].action[i] = DISPLAY;

	setPolyFT4(&template[num].poly[i]);
	template[num].poly[i].tpage = template[num].poly[index].tpage;

	setRGB0(&template[num].poly[i], 0xF0, 0xF0, 0xF0); // For Fading...
	setSemiTrans(&template[num].poly[i], &template[num].poly[index]);
	setShadeTex(&template[num].poly[i], &template[num].poly[index]);

   setXYWH(&template[num].poly[i], template[num].poly[index].x0, template[num].poly[index].y0, template[num].poly[index].x1 - template[num].poly[index].x0, template[num].poly[index].y2 - template[num].poly[index].y0);
	setUVWH(&template[num].poly[i], template[num].poly[index].u0, template[num].poly[index].v0, template[num].poly[index].u1 - template[num].poly[index].u0, template[num].poly[index].v1 - template[num].poly[index].v0); /* Point to texture... */
	template[num].depth[i] = template[num].depth[index];		// Default is display it...

// CLUT STUFF
	template[num].poly[i].clut = template[num].poly[index].clut;

	return(index);
}


/*************************************************************************
 * CHANGE ABS POLY WH - Streches polygon to an absolute Width & Height
 *************************************************************************/
void ChangeABSPolyWH(POLY_FT4 *poly, int num_w, int num_h)
{
	int x,
		 y;

/* Get poly X, Y, W & H */
	x = poly->x0;
	y = poly->y0;

   setXYWH(poly, x, y, num_w - 1, num_h - 1);
}


/*************************************************************************
 * Scale POLY WH - Streches polygon by X & Y fixed point scale factors
 *************************************************************************/

void ScalePolyWH(POLY_FT4 *poly, int scalex, int scaley)
{
	int x,
		 y,
		 w,
	    h;

/* Get poly X, Y, W & H */
	x = poly->x0;
	y = poly->y0;

	w = (poly->x1 - poly->x0)-1;
	h = (poly->y2 - poly->y0)-1;


   setXYWH(poly, x, y, (scalex * w) >> 16 , (scaley * h) >> 16);
}


/*************************************************************************
 * Change x,y size to allow loading of the smaller bitmap
 *************************************************************************/
void ChangePolyUV(POLY_FT4 *poly, int num_w, int num_h)
{
	setUVWH(poly, poly->u0, poly->v0, num_w, num_h);
   setXYWH(poly, poly->x0, poly->y0, num_w, num_h);
}

/*************************************************************************
 * makes one template polygon point to anothe one's clut
 *************************************************************************/
void ReassignTemplateCLUT(short target_num, short target_index, short source_num, short source_index)
{
  /*free the clut*/
  free_clut(template[target_num].clut[target_index]);

  /*point the template's clut pointer to another one's*/
  template[target_num].poly[target_index].clut = template[source_num].clut[source_index]->CLUT_ID;

  template[target_num].clut[target_index] = template[source_num].clut[source_index];
}

/*************************************************************************
 * CHANGE POLY CMP TEXTURE - Changes texture of the polygon
 *************************************************************************/
void ChangePolyCmpTextureNOCLT(short num, short index, char *file)
{
	RECT rect;
	char *buffer,
        *buf;

	if (TemplateBuff)
		buffer = TemplateBuff;
	else
		buffer = (char *)AllocMem(128 * 256);

	*strchr(file, '.') = NULL;	// Take extension off...
	strcat(file, ".CMP");
//	DecompressFrame((unsigned long *)buffer, GetMultiFilePtr(multifile, file));
	DecompressFrameBytes((unsigned long *)buffer, GetMultiFilePtr(multifile, file));

	if (template[num].clut_size[index] == 1)	// if it is 16 colors (4 bit)...
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 4;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 4;
	}
	else
	{
		rect.x = template[num].vram->x + template[num].poly[index].u0 / 2;
		rect.w = (template[num].poly[index].u1 - template[num].poly[index].u0+1) / 2;
	}
	rect.y = template[num].vram->y + template[num].poly[index].v0;
//	rect.h = template[num].poly[index].v2 - template[num].poly[index].v0+1;
	rect.h = template[num].poly[index].v2 - template[num].poly[index].v0;

	LoadImage(&rect, (unsigned long *)buffer);
	DrawSync(0);
	if (!TemplateBuff)
		FreeMem(buffer);
}


/*************************************************************************
 * SET TRANS POLY - Sets the passed in polygon to semi-Transparent mode.
 *
 *	Note: CLUT for polygon has to have high bits set on transparent colors.
 *************************************************************************/
void SetTransPoly(POLY_FT4 *poly)
{
 	setSemiTrans(poly, 1);
}



/*************************************************************************
 * CLEAR MOVIE - Clears a space the size of the last movie played. taking
 *		into account 16 or 24 bit.
 *************************************************************************/


void ClearMovie(void)
{
	VSync(0);

	SetDispMask(0);     /* disable display */

	ClearImage(&clearRect, 0, 0, 0);    // Clear with black
	clearRect.y += 240;
	ClearImage(&clearRect, 0, 0, 0);    // Clear with black

//	ClearScreen();
}


