/****************************************************************************
 *	LOWSHELL.H -  Data Structures
 ****************************************************************************/
#define NUM_TEMPLATES 12
#define NUM_POLYS 32
#define MAX_SPIN 10			// Maximum number of polygons that can be spinning

// TPAGE FLAGS

#define USE_TIM_CLUT 0x01
#define KEEP_DATA		0x02
//#define DISPLAY_OFF 0x04
//#define DISPLAY_OFF 0x08
//#define DISPLAY_OFF 0x10
//#define DISPLAY_OFF 0x20
//#define DISPLAY_OFF 0x40
//#define DISPLAY_OFF 0x80

// ACTIONS

#define DISPLAY	0x01	 // Display polygon...
#define FADE		0x02
#define FLIP		0x04
#define SPIN		0x08
//#define DISPLAY_OFF 0x10
//#define DISPLAY_OFF 0x20
//#define DISPLAY_OFF 0x40
#define END_OF_POLYS 0x80


typedef struct {
	char filename[14];
	char clut_size,
		  depth;
  short filenum,
		  x,
		  y;
unsigned char  w,
		  h;
  short u,
		  v;
} t_entry_t;



typedef struct {
  vram_t		*vram;					/* TPAGE data for polys */
  POLY_FT4	poly[NUM_POLYS];		/* Polygons for objects */
  vram_t		*clut[NUM_POLYS];		/* A seperate clut per polygon */
  char		clut_size[NUM_POLYS];/* Size of CLUT 0 = 4 bit, 1 = 8 bit */
  char		action[NUM_POLYS]; 	/* What mods are we doing to poly */
//  short		act_1[NUM_POLYS]; 	/* What mods are we doing to poly */
//  short		act_2[NUM_POLYS]; 	/* What mods are we doing to poly */
  char		depth[NUM_POLYS]; 	/* What Depth to display the object at */
} template_t;


typedef struct {
	char used[MAX_SPIN];
	unsigned long *texture[MAX_SPIN][2];
	unsigned long *clut[MAX_SPIN][2];
	char w[MAX_SPIN],
		  h[MAX_SPIN],
		  dir[MAX_SPIN];
} spin_t;



//extern spin_t spin;									/* definition is in template.c */
extern template_t template[NUM_TEMPLATES];	/* definition is in template.c */
extern unsigned long *multifile;					/* definition is in template.c */
extern char FlipKludge;


// NOTE: Each template has
//			TIM file name, X, Y, U, V, W, H, *clut | 0 | 1 per polygon


// LOW LEVEL FUNCTIONS
void InitTemplates(void);
void LoadTemplate(char *file);
void FreeTemplate(char num);
void FreeAllTemplate(void);
void RestoreTemplate(char num);
void DisplayShellPolys(void);
void ChangePolyTex(short num, short index, char *buffer);		// Load texture
void ChangePolyTexture(short num, short index, char *buffer);		// Load texture
void ChangePolyCmpTexture(short num, short index, char *file);		// Load CMP
void ChangePolyTextureBuff(short num, short index, char *buffer);
void ChangePolyTimTexture(short num, short index, unsigned long *texture);
void ChangePolyWH(POLY_FT4 *poly, short num_w, short num_h);
void SetPolyXYWH(POLY_FT4 *poly, short num_x, short num_y, short num_w, short num_h);
void MovePolyToXY(POLY_FT4 *poly, short num_x, short num_y);		// Absolute
void MovePolyXY(POLY_FT4 *poly, short num_x, short num_y);			// Relative
void ChangePolySize(POLY_FT4 *poly, short size_x, short size_y);	// Relative
void ChangePoly4CLUT(short num, short index, unsigned long *clut);	// Need number of Template & index to poly
void ChangePolyCLUT(short num, short index, unsigned long *clut);	// Need number of Template & index to poly

short CopyPoly(short num, short index);
void ChangeABSPolyWH(POLY_FT4 *poly, int num_w, int num_h);			// Streches polygon to absolute WH. Created by Oick
void ChangePolyUV(POLY_FT4 *poly, int num_w, int num_h);	  			// Changes uv to allow loading smaller bitmaqs
void ChangePolyCmpTextureNOCLT(short num, short index, char *file);
void ReassignTemplateCLUT(short target_num, short target_index, short source_num, short source_index);
void SetTransPoly(POLY_FT4 *poly);

// HIGH LEVEL FUNCTIONS
void FadePoly(short num, short index, char fade_in, char init);
void FlipPoly(short num, short index, char dir, char *front, char *back, char init);
//ZoomPoly();

void ScalePolyWH(POLY_FT4 *poly, int scalex, int scaley);

void ClearMovie(void);

