//
// GAMETEK 1996
//
// Menu code for machine independant menu system
//
// Author : Nigel Spencer 08/01/96
//
//


#include <stdio.h>
#include <sys/types.h>

#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include "prjhdrs.h"

#include "fbequ.h"
#include "fbram.h"
#include "shell.h"


//**************************************************************************
// Data structures & defines for menu system


#include "menu.h"			// Non machine specific Menu defines & structures

#define FLASH_ON	8

extern RECT ClipString;		// Viewport rectangle
BYTE CtrlPad[2];		// tom-used by controller stuff for some shell routines


WORD MenuTxt_Xoffset;	// Global X offset
WORD MenuTxt_Yoffset;	// Global Y offset
WORD MenuCursorX;
WORD MenuCursorY;
WORD MenuItemNum;
WORD MenuSubItemNum;

WORD ScrollYLimit;
WORD AnimPic, AnimCount;		// General Purpose anim vars
WORD FlashMsg;
BYTE *Message;
BYTE TempStr[MAXPATH];
BYTE SpacingFlag = PROPORTIONAL;
WORD Scrolling, ScrollCount;
BYTE *CurFont = fontdpc2;

//**************************************************************************

extern dialog_t dialog[];

// Array of Menus indexed by no. passed to Display_menu

//extern MenuObj GameOptMenu;		// 0
//extern MenuObj PlayerOptMenu;	// 1
extern MenuObj TeamViewMenu;	// 2
extern MenuObj PlayerViewMenu;	// 3
extern MenuObj ScheduleMenu;	// 4
extern MenuObj AudiblesMenu;	// 5
extern MenuObj NewSeasonMenu;	// 6
extern MenuObj ContSeasonMenu;	// 7
//extern MenuObj SeasOptMenu;		// 8
extern MenuObj TmStatsMenu;		// 9
extern MenuObj TmStandingsMenu;	//10
extern MenuObj LgLeadersMenu;	//11
extern MenuObj PlyrLeaderMenu;	//12
extern MenuObj LeadersMenu;		//13
extern MenuObj ContOptSeasonMenu;//14
extern MenuObj SubstitutionMenu;//15
extern MenuObj PlayoffsMenu;	//16
extern MenuObj TradingMenu;		//17
//extern MenuObj ChangeControlMenu;	//18


MenuObj *MenuList[] = {
		NULL,	//&GameOptMenu,			// 0
		NULL,	//&PlayerOptMenu,			// 1
		&TeamViewMenu,			// 2
		&PlayerViewMenu,		// 3
		&ScheduleMenu,			// 4
		&AudiblesMenu,			// 5
		NULL,	//&NewSeasonMenu,			// 6
		NULL,	//&ContSeasonMenu,		// 7
		NULL,	//&SeasOptMenu,			// 8
		&TmStatsMenu,			// 9
		&TmStandingsMenu,		//10
		&LgLeadersMenu, 		//11
		&PlyrLeaderMenu,		//12
		&LeadersMenu,			//13
		NULL,	//&ContOptSeasonMenu,		//14
		&SubstitutionMenu,		//15
		&PlayoffsMenu,			//16
		&TradingMenu,			//17
		NULL,	//&ChangeControlMenu,		//18
};


//**************************************************************************
// Prototypes for menu functions

INT Display_Menu(INT menu_num);
INT DrawMenu(INT menu_num);

INT Std_Menu_Frame(void);
INT Std_Menu_Init(void);
INT Std_Menu_CleanUp(void);

void PrintStringXY(BYTE* string, WORD x, WORD y, WORD color);
void PrintStringXYAbs(BYTE* string, WORD x, WORD y, WORD color);
INT PrintStringXYCenter(BYTE* string, WORD x, WORD y, WORD color);
void SetClipping(RECT *cliprect);
void SetHighlight(void);
INT InputHandler(INT cursory_min, INT cursory_max);
void FadeDown(LONG fadespeed);
INT GetAllPads(INT repeatrate);

void DoMenuStrip(void);


//**************************************************************************

WORD CurMenu;
INT MaxPlayers = 8;			// Whatever var holds MaxPlayers used here
INT VBCount;				// Standard frame counter

WORD Flash;
WORD MenuDirY;
WORD MenuDirX;
LONG MenuFadeFlag = TRUE;

//**************************************************************************

INT Display_Menu(INT menu_num)
{
  /*local variables*/

	INT i,j;
	INT pad;
	INT retcode = 0;
	INT num_items, cur_item;

	DumpHeap();

	CurMenu = menu_num;										// Store for reference

	CurFont = fontdpc2;		// std menu font spacing
	MenuItemNum =
	MenuCursorY =
	MenuTxt_Xoffset =
	MenuTxt_Yoffset = 0;	// Standard init

	FreeAllDialogBoxes();

	FlushPads();

	MenuList[menu_num]->init_menu();						// Do any required initialisation

//	DoMenuStrip();

//	if(MenuFadeFlag)
//		fade_in(10 , 1);

	JpadRepeatFreq = 20;

	SetClipping(&MenuList[menu_num]->rect_menu);

	while(!retcode)											// Standard loop (any specific requirements in the menu's main_menu function
	{
	  	VBCount = VSync(-1);

		Flash++;

		if(!Scrolling)
		{

			ResetStrings();

			num_items = DrawMenu(menu_num);

			retcode = MenuList[menu_num]->main_menu();

			if(retcode)			// Ensure highlighted is lit up before we leave (cosmetic appeal)
			{
				Flash = FLASH_ON;
			}
		}
		else retcode = MenuList[menu_num]->main_menu();

		DoFrameLoop(1, Std_Menu_Frame);

		while ((VBCount + MENU_FRAMERATE) > VSync(-1));

	}

//	if(MenuFadeFlag)
//	{
//		FadeDown(10);
//	}
//	else
//	{
//		LoadingMessage();
//	}

//	LoadingMessage();

	MenuList[menu_num]->term_menu();		// And cleanup before we exit

//	ShowLoadingScreen();	// CGD

//	FreeAllDialogBoxes();
//	LoadingMessage();
//	DoFrameLoop(1, Std_Menu_Frame);
//	DoFrameLoop(1, Std_Menu_Frame);

	return (retcode);
}


//**************************************************************************

// Draws menu

INT DrawMenu(menu_num)
{
	MenuItem *itemptr;
	WORD itemnum = -1;
	WORD color;

	itemptr = MenuList[menu_num]->menu_items;

	itemptr--;

	while((++itemptr)->item_flags != END_MENU)
	{
		color = itemptr->item_flags & 0x0000000F;			// Lower 4 bits contain color index

		if(itemptr->item_flags & MENU_ITEM) itemnum++;	// Count menu items

		if(!(itemptr->item_flags & TEXT_ITEM))
		{
			if(!(itemptr->item_flags & ACTIVE))
				color = (itemptr->item_flags & 0x00000F00) >> 8;
			else
			if(itemptr->item_flags & HIGHLIGHT)					// If highlight get bits4-7
			{
				if(MenuItemNum != itemnum || (Flash & 0x08))
				{
					color = (itemptr->item_flags & 0x000000F0) >> 4;
				}
				else
					color = (itemptr->item_flags & 0x00000F00) >> 8;
			}
		}

		PrintStringXY(itemptr->item_text, itemptr->item_x, itemptr->item_y, color);

		if(itemptr->item_funct) (*(itemptr->item_funct))(itemptr, itemnum);
	}

	return(0);
}


//**************************************************************************

// Sample display update routine

INT Std_Menu_Frame(void)
{
//	DisplayDialogBox();
	UpdateBackground();
	DisplayAllStrings();
	DisplayShellPolys();

	return(0);
}

//**************************************************************************

// Sample terminate routine. Put whatever your machine requires in here

INT Std_Menu_CleanUp(void)
{
//	ClearString(0,0,512,240);

	LoadingMessage();

	ResetStrings();					// required for PSX
	FreeAllTemplates();
	FreeFileData((BYTE *)Shell_MFile);
	Shell_MFile = NULL;				// Clear for Chris
	FreeBackgrounds();

	MenuFadeFlag = TRUE;			// Turn menu fade back on as default

	FlushPads();

	DumpHeap();

	return(0);
}


//**************************************************************************


void PrintStringXY(BYTE* string, WORD x, WORD y, WORD color)
{
	INT handle;


	x+= MenuTxt_Xoffset;
	y+= MenuTxt_Yoffset;

	if(string)		// Check for valid string
	{
		handle = CreateStringClip(string, CurFont, x, y, FONT0 | SpacingFlag);
		if(handle != -1) ChangeStringCLUT(strings.string[handle], FontClutIDs[color]->CLUT_ID);
	}

	return;
}


//**************************************************************************

void PrintStringXYAbs(BYTE* string, WORD x, WORD y, WORD color)
{
	INT handle;

	if(string)		// Check for valid string
	{
		handle = CreateString(string, CurFont, x, y, FONT0 | SpacingFlag);
		ChangeStringCLUT(strings.string[handle], FontClutIDs[color]->CLUT_ID);
	}

	return;
}


//**************************************************************************

INT PrintStringXYCenter(BYTE* string, WORD x, WORD y, WORD color)
{
	INT handle;

	if(string)		// Check for valid string
	{
//		handle = CreateStringClip(string, CurFont, x, y, FONT0 | SpacingFlag);

		if(y >= ClipString.y && y <= ClipString.y+ClipString.h)
		{
			handle = CreateString(string, CurFont, x, y, FONT0 | SpacingFlag);

			if(handle != -1)
			{
				CenterString(handle, string, x);
				ChangeString(handle, string, FONT0 | SpacingFlag);
				ChangeStringCLUT(strings.string[handle], FontClutIDs[color]->CLUT_ID);
			}
		}
	}

	return(handle);
}



//**************************************************************************

void SetClipping(RECT *cliprect)
{
	memcpy(&ClipString,cliprect,sizeof(RECT));
}

//**************************************************************************


void SetHighlight(void)
{
	MenuItem *itemptr;
	WORD itemnum = 0;

	itemptr = MenuList[CurMenu]->menu_items;
	itemptr--;

	while((++itemptr)->item_flags != END_MENU)
	{
		if(itemptr->item_flags & MENU_ITEM);	// Count menu items
		{
			if(itemnum == MenuItemNum)
			{
				itemptr->item_flags |= HIGHLIGHT;
			}
			else
			{
				itemptr->item_flags &= HIGHLIGHT^0xFFFFFFFF;
			}

			itemnum++;
		}
	}
}


//**************************************************************************

INT InputHandler(INT cursory_min, INT cursory_max)
{
	INT retcode = 0;
	INT i;
	BYTE pad;


	MenuDirY = MenuDirX = 0;


	for (i = 0; i < 2; i++)
	{
		pad = CtrlPad[i];
		if ((i==1) && (pad==CtrlPad[1])) continue;		// eliminate duped controllers

	if(pad != -1)
	{

		switch(JpadTrig[pad])
		{
			case JPADLdown:

							if(MenuCursorY < cursory_max)
							{
								MenuCursorY+=15;
								MenuItemNum++;
								PlayEffect(CURSOR_SFX);
							}
							else
							if(MenuTxt_Yoffset > ScrollYLimit)
							{
								MenuTxt_Yoffset-=15;
								MenuItemNum++;
								PlayEffect(CURSOR_SFX);
							}
							else MenuDirY = 1;

							break;

	 		case JPADLup:

							if(MenuCursorY > cursory_min)
							{
								MenuCursorY-=15;
								MenuItemNum--;
								PlayEffect(CURSOR_SFX);
							}
							else
							if(MenuTxt_Yoffset < 0)
							{
								MenuTxt_Yoffset+=15;
								MenuItemNum--;
								PlayEffect(CURSOR_SFX);
							}
							else MenuDirY = -1;
							break;

			case JPADLright:
							MenuDirX = 1;
							break;
			case JPADLleft:
							MenuDirX = -1;
							break;

			case JPADRup:
							PlayEffect(PRESS_SOUND);
							retcode = TRIANGLE;
							break;

			case JPADRdown:
							PlayEffect(PRESS_SOUND);
							retcode = CROSS;
							break;

			case JPADRleft:
							PlayEffect(PRESS_SOUND);
							retcode = SQUARE;	// Go to alternate options screen
							break;

			case JPADRright:
							PlayEffect(PRESS_SOUND);
							retcode = CIRCLE;	// Go to alternate options screen
							break;
		}
	}
	}

	return(retcode);
}


/***************************************************************************/


INT GetAllPads(INT repeatrate)
{
	INT i,j=0;

	JpadRepeatFreq = repeatrate;

	for(i=0; i<8; i++)
	{
		j|=JpadTrig[i];
	}

	return(j);
}

/***************************************************************************/

void FadeDown(LONG fadespeed)
{
	LONG i;

	fade_out(fadespeed , 1);

	for(i=0; i<fadespeed+1; i++)
	{
		DoFrameLoop(1, Std_Menu_Frame);
	}
}


//***************************************************************************
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/
/***************************************************************************/

LONG LoadNickname(LONG teamnum, LONG template_num, LONG template_item)
{
	LONG i;
	BYTE *ptr;

	for(i=0; i<2; i++)
	{
		sprintf(TempStr,"/shells/names/nick%c%d/nick%c%02d.tim",'a'+i, teamnum/30, 'a'+i,teamnum);

		if(!(ptr = LoadCDFile(TempStr)))
			return (FALSE);

		ChangePolyTimTexture(template_num, template_item+i ,(ULONG *) ptr);
		free(ptr);
	}

	return(TRUE);
}

/***************************************************************************/

LONG LoadTeamPic(LONG teamno)
{
		sprintf(TempStr,"/shells/team%d/team%02d.tim",teamno/30,teamno);
		return(LoadBackgroundCD(0,TempStr));	// Load & free from memory
}

/***************************************************************************/

LONG LoadHalfPic(LONG teamno)
{
		sprintf(TempStr,"/shells/halfpics/half%02d.tim",teamno);
		return(LoadBackgroundCD(0,TempStr));	// Load & free from memory
}

/***************************************************************************/
/***************************************************************************/
/***************************************************************************/


//***************************************************************************

void LoadingMessage(void)
{
	LONG boxnum;

	DialogBoxMessage(256, 52, "LOADING...Please Wait");
}

//***************************************************************************

void SetupFont(BYTE *fontname)
{
	InitFonts();
	Register_Font_Cluts();
	LoadFont(fontname, 3);
}

//***************************************************************************

void DoMenuStrip(void)
{
	INT boxnum;

	boxnum = PopupDialogBox(0, 20, 512 , 20, 2, 0x10, 0x10, 0x10);
	ChangeDialogDepth(boxnum,100);
	ChangeDialogLayers(boxnum,1);
}

//***************************************************************************

void EndSeasonMessage(INT win)
{
	if(win)
		TimedMessage(4, "You already won the Superbowl");
	else
		TimedMessage(4, "You lost in the playoffs");
}

//***************************************************************************

// Temporary

// shared shell routines (tom)

//**************************************************************************

#define PREGAME	    0x0

extern	WORD CardSlot;

INT LoadOptions(void)
{
	INT retval;
	INT tries = 3;

	if(GetCardBank())
	{
		do {
			retval = ReadFile((UWORD *)GameOptions, 0, CardSlot, OPTIONFILE);
		}while(retval == FILE_ERROR && --tries);
	}
	return(-1);
}

INT InitControl(INT pad)
{
	INT i=0, j=0;

	if(GameMode == PREGAME)
	{
		if((pad = GameTeamData[0].ControllerNum[0])<0)
			pad = GameTeamData[1].ControllerNum[0];
	}

	WhichTeam = ControllerToTeam(pad);

	if(WhichTeam<0) WhichTeam = ControllerToCpuTeam(pad);

	TeamNo = GameTeamData[WhichTeam].TeamNumber;

	if(pad < 0)
	{
		CtrlPad[0] = CtrlPad[1] = -1;

		while(i<8 && j<2)
		{
			if(JpadID[i])
			{
				CtrlPad[j]=i;
				j++;
			}

			i++;
		}
	}
	else
	{
		CtrlPad[WhichTeam] = pad;
		CtrlPad[WhichTeam^1] = GameTeamData[WhichTeam^1].ControllerNum[0];
	}

//	TeamNo = GameTeamData[WhichTeam].TeamNumber;

}



