/***************************************************************
	MISC.C	Misc PSX routines
	
	By Christopher Deel
****************************************************************/

#include <stdio.h>
#include <stdarg.h>
#include <rand.h>

#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include <kernel.h>

#include "prjhdrs.h"	/* Project header files */
#include "fbequ.h"

#if PC_DEBUG
//	#define DEBUG_TIMERS 1
#endif // PC_DEBUG

ULONG gVBlankEvent;

typedef struct
{
	volatile ULONG Count;
	volatile ULONG Timers[MAX_VB_TIMERS];
	volatile int TimerRunning[MAX_VB_TIMERS];
} VBlankVars_t;

volatile VBlankVars_t gVBlank;


volatile ULONG gVBlankCount = 0;
volatile ULONG gVBlankTimers[MAX_VB_TIMERS] = {0,0,0,0,0};
volatile int gVBlankTimerRunning[MAX_VB_TIMERS];
volatile int gInVBlank = FALSE;

/******************* Vertical blank routines **************/

#if __psx__
/*-------------------------------------------------------------
  HandleVBlank ~ Handle Vertical Blank interrupt
-------------------------------------------------------------*/
long HandleVBlank(void)
{
	register int i;
	register VBlankVars_t *ptr;
	
		
//	gInVBlank = TRUE;		
	ptr = &gVBlank;
	ptr->Count++;	
	for (i = 0; i < MAX_VB_TIMERS; i++)
	{
		if (gVBlank.TimerRunning[i])
		{
		    gVBlank.Timers[i]--;
		    if (gVBlank.Timers[i] == 0 || (gVBlank.Timers[i] > 0x8FFF0000))
		    {
		    	gVBlank.Timers[i] = 0;
		    	gVBlank.TimerRunning[i] = False;
		    }
		}
	}
	PlayPhrase();
//	FadeMusic();
//	gInVBlank = FALSE;		
}

/*-------------------------------------------------------------
  InitVBlankCount ~ Start Vertical blank interrupt 
-------------------------------------------------------------*/
void InitVBlankCount(void)
{
	ULONG vb;
	
	EnterCriticalSection();
	gVBlankEvent = OpenEvent(RCntCNT3, EvSpINT, EvMdINTR, HandleVBlank);
	EnableEvent(gVBlankEvent);
	SetRCnt(RCntCNT3, 1, RCntMdINTR);
	StartRCnt(RCntCNT3);
//	gInVBlank = FALSE;
	ExitCriticalSection();
	
	
#if PC_DEBUG	
	vb = gVBlank.Count;
	VSync(0);
	VSync(0);
//	if (gVBlankCount == 0)
	if (gVBlank.Count == vb)
		DebugPrintf(0, "VBlank Error\n");
#endif		
}

/*-------------------------------------------------------------
  GetVBlankCount ~ Get Vertical Blank Count
-------------------------------------------------------------*/
ULONG GetVBlankCount(void)
{
//	ULONG cnt;
	
//	EnterCriticalSection();
//	cnt = gVBlankCount;
//	ExitCriticalSection();
//	return cnt;
//	return gVBlankCount;

	return VSync(-1);	// This works better  why???
}

/*-------------------------------------------------------------
  IsVBlankAfter ~ 
-------------------------------------------------------------*/
int IsVBlankAfter(ULONG count)
{
	if (GetVBlankCount() < count)
		return FALSE;
	else 
		return TRUE;
}

/*-------------------------------------------------------------
  StartVBlankTimer ~ 
-------------------------------------------------------------*/
void StartVBlankTimer(int num)
{
	gVBlank.TimerRunning[num] = True;

	#if DEBUG_TIMERS
		DebugPrintf(0, "StartTimer #%d = %d\n", num, gVBlankTimers[num]);
	#endif	
}

/*-------------------------------------------------------------
  StopVBlankTimer ~ 
-------------------------------------------------------------*/
void StopVBlankTimer(int num)
{
	gVBlank.TimerRunning[num] = False;
	
	#if DEBUG_TIMERS
		DebugPrintf(0, "StopTimer #%d = %d\n", num, gVBlankTimers[num]);
	#endif	
}

/*-------------------------------------------------------------
  SetVBlankTimer ~ 
-------------------------------------------------------------*/
void SetVBlankTimer(int num, ULONG val)
{
	EnterCriticalSection();
	gVBlank.Timers[num] = val;
//	StopVBlankTimer(num);
	
	#if DEBUG_TIMERS
		DebugPrintf(0, "SetTimer #%d = %d\n", num, gVBlankTimers[num]);
	#endif	
	ExitCriticalSection();
}

/*-------------------------------------------------------------
  GetVBlankTimer ~ 
-------------------------------------------------------------*/
ULONG GetVBlankTimer(int num)
{
	return gVBlank.Timers[num];
}

/*-------------------------------------------------------------
  VBlankToTime ~ Convert VBlank to HH:MM:SS.hh
-------------------------------------------------------------*/
Time_t VBlankToTime(ULONG vb)
{
	Time_t tm;
	ULONG vb2;
	
//	tm.hundreths = ((vb % 60) * 100) / VBlanksPerSec;
//	vb2 = (vb + VBlanksPerSec -1)  / VBlanksPerSec;
	
	tm.hundreths = ((vb * 100) / CLOCKS_TICKS_PER_SECOND) % 100;
	vb2 = (vb + CLOCKS_TICKS_PER_SECOND -1)  / CLOCKS_TICKS_PER_SECOND;
	tm.seconds = (vb2 % 60);
	vb2 /= 60;
	tm.minutes = (vb2 % 60);
	tm.hours = vb2 / 60;
//	tm.hours = (vb2 % 60);		// hours not needed this game
	return tm;
}

/*~~------------------------------------------------------------------
    Delay for n Vertial Blanks
--------------------------------------------------------------------*/
void DelayVB(long n)
{
	Ulong v1,v2;

	v1 = GetVBlankCount();
	while (n > 0)
	{
		v2 = GetVBlankCount();
		n = n - (v2 - v1);
		v1 = v2;
		PollPC();
	}
}

/*-------------------------------------------------------------
  GenPSXColor ~ range 0 - 256
-------------------------------------------------------------*/
USHORT GenPSXColor(int red, int green, int blue, int transparent)
{
	USHORT ret;
	
	red >>= 3;
	green >>= 3;
	blue  >>= 3;
	ret = (blue << 10 | green << 5 | red);
	if (transparent)
		ret |= 0x8000;
	return ret;
}

/*-------------------------------------------------------------
  Randomize ~ Seed random number generator with random value
-------------------------------------------------------------*/
void Randomize(void)
{
	unsigned int seed;
	
	seed = GetVBlankCount() + GetRCnt(RCntCNT1) + rand() + VSync(1);
	srand(seed);
	 DebugPrintf(0, "Randomize:%d %d\n", seed, rand());
}

/*~~-------------------------
---------------------------*/
char getch(void)
{
	char c;
	
	c = 0;
	WaitForButton();
	return c;
}

/*-------------------------------------------------------------
  SetShortVector ~ Set SVECTOR function
-------------------------------------------------------------*/
void SetShortVector(SVECTOR *vec, int x, int y, int z)
{
	vec->vx = x;
	vec->vy = y;
	vec->vz = z;
}  /* End SetShortVector() */

/*-------------------------------------------------------------
  CopyVectorFunc ~ Copy Vector Function
-------------------------------------------------------------*/
void CopyShortVector(SVECTOR *source, SVECTOR *dest)
{
	dest->vx = source->vx;
	dest->vy = source->vy;
	dest->vz = source->vz;
}  /* End CopyVectorFunc() */

/*~~-------------------------
---------------------------*/
int strlenI(char *s)
{
	int n=0;
	for(;*s;n++,s++);
	return n;
}

/*~~-------------------------
---------------------------*/
char *strupr(char *s)
{
	char *o=s;
	
	while(*s)
	{	
		if(*s >= 'a' & *s <= 'z')
			*s=toupper(*s);
		s++;
	}
	return(o);
}  /* strupr */

/*~~-------------------------
 String to Lower Case
---------------------------*/
char *strlwr(char *s)
{
	char *o=s;
	
	while(*s)
	{	
		if(*s >= 'A' & *s <= 'Z')
			*s=tolower(*s);
		s++;
	}
	return(o);
}  /* strupr */


/*~~-------------------------
---------------------------*/
int Int2Str(int num, char *str, int radix)
{
	char buff[10], *start;
	int i;
	ULONG unum;

	start = str;
	if(num<0 && radix < 11)
	{
		unum = (ULONG)-num;
		*str++ = '-';
	}
	else
		unum = (ULONG)num;
	for(i=0; (unum || !i) && i<10; i++)
	{
		buff[i]=(unum%radix)+'0';
		if(buff[i]>'9')
			buff[i]+='A'-('9'+1);
		unum=unum/radix;
	}
	if(i)
	    i--;
	for( ; i>=0; i--)
		*str++=buff[i];
	*str++=0;
	return (str - start) -1;
}

/*~~-------------------------
---------------------------*/
BOOL strincmp(char *arg1,char*arg2,int n)
{
	int i;

	for(i=0;i<n&&*arg1&&*arg2;i++)
	{
		if(*arg1!=*arg2)
			return(*arg2 - *arg1);
		arg1++;
		arg2++;
	}
	return 0;
}

/*~~--------------------------------------------------------------
  Variable Arg SPRINTF (simple version)       Chris Deel
---------------------------------------------------------------*/
void vsprintf2(char *buf, char *format, va_list ap)
{
	char *str,*dest=buf;
	int num, i, len;
	char c;
	
	while(*format)
	{
		if (*format=='%')
		{
			format++;
			switch (*format)
			{
				case 'c':
					c=(char)va_arg(ap,char*);
					*dest++=c;
					break;
				case 's':
					str=(char*)va_arg(ap,char*);
					while(*str)
						*dest++ = *str++;
					break;
				case 'd':
					num=(int)va_arg(ap,int);
					len = Int2Str(num,dest,10);
					dest += len;
					break;
				case 'X':
				case 'x':
					num=(int)va_arg(ap,int);
					len = Int2Str(num,dest,16);
					dest += len;
					break;
				default :
					printf("vsprintf error: %s!\n",*format);
					break;
			}
			format++;
		}
		else
			*dest++=*format++;
	}
	*dest='\0';
}

/*~~-------------------------
---------------------------*/
char* sprintf2(char *buf,char *format, ...)
{
	char *str,*dest=buf;
	int num;
	char c;
	va_list ap;
	
	va_start(ap, format);
	vsprintf2(buf, format, ap);
	va_end(ap);
	return buf;
}


/*~~-------------------------
---------------------------*/
BOOL stricmp(char *s1,char *s2)
{
	while(*s1==*s2)
	{
		if(!*s1)return(0);
		s1++;
		s2++;
	}
	return(*s1-*s2);
}


/*~~-------------------------
---------------------------*/
BOOL isgraph(char c)
{
	return((c!=' ')&&(c)&&(c!='\n'));
}

/**************** Memory allocation routines ******************/
#if DEBUG_MALLOC_FILE
#undef AllocMem
#undef FreeMem
#endif 

/*-------------------------------------------------------------
  AllocMem ~ Allocate memory
-------------------------------------------------------------*/
void * AllocMem(long bytes)
{
	register void *ptr;
	
	ptr = (void *)malloc(bytes);

//#if PC_DEBUG
//	if ( (((ULONG)ptr + (ULONG)bytes) > 0x80200000) && 
//	    						((ULONG)ptr < 0x80200100) )
//		 DebugPrintf(0, "Alloc Overflow: %X %X %X  -%dk\n", ptr, bytes, 
//	 				(ULONG)ptr + (ULONG)bytes, 
//	 				(((ULONG)ptr + (ULONG)bytes) - 0x80200000)/1024 );
//#if DEBUG_MALLOC
//	 DebugPrintf(0, "AllocMem: %X %X %X\n", ptr, bytes, 
//	 				(ULONG)ptr + (ULONG)bytes);	
//#endif	 
//#endif PC_DEBUG
	
	return ptr;
}

/*-------------------------------------------------------------
  FreeMem ~ Free memory
-------------------------------------------------------------*/
void FreeMem(void *ptr)
{
	free(ptr);
#if DEBUG_MALLOC
	 DebugPrintf(0, "FreeMem: %X\n", ptr);	
#endif	 
//#if PC_DEBUG	
//	Check2MegOverflow();
//	if ((ULONG)ptr > 0x80300000 || (ULONG)ptr < 0x8000FFFF)
//		DebugPrintf(0, "Invalid Free: %x\n", ptr);
//#endif	

}

#endif //__psx__


/*********** These routines should compile on a PC **********/

/*-------------------------------------------------------------
  Modulo ~ 
-------------------------------------------------------------*/
long Modulo(long value, long mod)
{
	
}  /* End Modulo() */


/*-------------------------------------------------------------
  GetRandom ~ 
-------------------------------------------------------------*/
int GetRandom(int num)
{
	register long val;
	
	val = (((long)rand()*(num))/(RAND_MAX+1));
	val %= num;  // for safety
	return (int)val;
}


/*-------------------------------------------------------------
  AbsInt ~ Absolute Value of int
-------------------------------------------------------------*/
int AbsInt(int val)
{
	if (val < 0)
		return (-val);
	else
		return val;
}  /* End AbsInt() */

/*-------------------------------------------------------------
  CopyBytes ~ 
-------------------------------------------------------------*/
void CopyBytes(UBYTE *src, UBYTE *dest, int count)
{
	while(count--)
	{
		*dest++ = *src++;
	}
}

/*-------------------------------------------------------------
  CopyShorts ~ 
-------------------------------------------------------------*/
void CopyShorts(USHORT *src, USHORT *dest, int count)
{
	while(count--)
	{
		*dest++ = *src++;
	}
}

/*-------------------------------------------------------------
  CopyLongs ~ 
-------------------------------------------------------------*/
void CopyLongs(ULONG *src, ULONG *dest, int count)
{
	while(count--)
	{
		*dest++ = *src++;
	}
}

