/*************************************************************************
 *	MMVRAM.C
 *
 *	Copyright 1994 GameTek Inc.
 *	All rights reserved.
 *
 *	 Written by Mike Heilemann
 *
 *************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include "prjhdrs.h"	/* Project header files */


extern BYTE fnt[90*8*16];

vram_t texture[NUM_TEXTURE_PAGES];    /* 320 # of textures is the largest */
vram_t clut256[BIG_NUM_CLUT256_PAGES];/* 640 # of cluts is the largest */
vram_t clut16[BIG_NUM_CLUT16_PAGES];  /* 640 # of cluts is the largest */

BYTE FadeIn = 0;
BYTE FadeOut = 0;

int num_textures  = NUM_TEXTURE_PAGES; /* default 320 X 240 */
int num_256_cluts = NUM_CLUT256_PAGES; /* default 320 X 240 */
int num_16_cluts  = NUM_CLUT16_PAGES;  /* default 320 X 240 */


/*************************************************************************
 * INIT VRAM HANDLING - setup VRAM pages for use.
 *************************************************************************/
void initVRamHandling(void)
{
	int i, j,
//	   num_textures  = NUM_TEXTURE_PAGES, /* default 320 X 240 */
//	   num_256_cluts = NUM_CLUT256_PAGES, /* default 320 X 240 */
//	   num_16_cluts  = NUM_CLUT16_PAGES,  /* default 320 X 240 */
	   x = 0,
		 tmp;


	FreeFonts();	// Free Font Vram
	if (gVidMode == VMODE640)
	{
		num_textures  = BIG_NUM_TEXTURE_PAGES; /* 640 X 480 */
		num_256_cluts = BIG_NUM_CLUT256_PAGES; /* 640 X 480 */
		num_16_cluts  = BIG_NUM_CLUT16_PAGES;  /* 640 X 480 */
	}
	else if (gVidMode == VMODE512)
	{
		num_textures  = MED_NUM_TEXTURE_PAGES; /* 512 X 240 */
		num_256_cluts = MED_NUM_CLUT256_PAGES; /* 512 X 240 */
		num_16_cluts  = MED_NUM_CLUT16_PAGES;  /* 512 X 240 */
	}

	/* setup texture pages... */
	for (j = 0; j < 2; j++)
	{
		for (i = 0; i < num_textures / 2; i++) /* 2 rows of tpages */
		{
			tmp = i + (j * (num_textures / 2));
			texture[tmp].used = VRAM_UNUSED;
			texture[tmp].num = 0;
			if (gVidMode == VMODE640) // gVidMode VMODE640 = 640 X 480, VMODE320 = 320 X 240
				texture[tmp].x = 640 + i * 64;
			else if (gVidMode == VMODE512)
				texture[tmp].x = 512 + i * 64;
			else
				texture[tmp].x = 320 + i * 64;
			texture[tmp].y = 256 * j;
		}
	}

	/* ---- setup CLUT 256 pages... ---- */
	for (i = 0, j = 0, tmp = 0; i < num_256_cluts; i++, tmp++)
	{
		clut256[i].used = 0;
		clut256[i].num = 0;
		if (i == 32) /* Extend out if in 640 * 480 mode */
		{
			j = 256;
			tmp = 0;
		}
		clut256[i].x = j;
		clut256[i].y = 480 + tmp;
		clut256[i].w = 256;
		clut256[i].h = 1;
	}
	tmp += 480;

	if (j == 256) /* get to the right place in VRAM */
		x = 256;

	for (j = 0; j + tmp < 512; j++)	/* setup the rest as CLUT 16 pages... */
	{
		for (i = 0; i < 16; i++)		/* Number in 256 byte chunk */
		{
			clut16[(j << 4) + i].used = 0;	// J * 16 + i for index...
			clut16[(j << 4) + i].num = 0;
			clut16[(j << 4) + i].x = (i << 4) + x;
			clut16[(j << 4) + i].y = j + tmp;
			clut16[(j << 4) + i].w = 16;
			clut16[(j << 4) + i].h = 1;
		}
	}
	 Dprintf("Clut256[0]=%d,%d  Clut16[0]=%d,%d\n",
		 	clut256[0].x,clut256[0].y, clut16[0].x,clut16[0].y);
}

/*************************************************************************
 * FREE VRAM - Mark a VRAM space as being free
 *************************************************************************/
void free_VRam(vram_t *vram)
{
	int i;

//   for (i = 0; i <= vram->num; i++)
//      vram += sizeof(vram_t);   /* jump to next texture */
	if (vram)
		vram->used = VRAM_UNUSED;
}


/*************************************************************************
 * ALLOC VRAM - Find a space for type
 *
 *  TYPE 1 = screen			(320 X 240)
 *  TYPE 2 = texture map	( 64 X 256)
 *  TYPE 3 = CLUT 16			(  1 X  16)
 *  TYPE 4 = CLUT 256		(  1 X 256)
 *
 *************************************************************************/
vram_t *alloc_VRam(int type, int num)
{
	int i,
//	   num_textures  = NUM_TEXTURE_PAGES, /* default 320 X 240 */
//	   num_256_cluts = NUM_CLUT256_PAGES, /* default 320 X 240 */
//	   num_16_cluts  = NUM_CLUT16_PAGES,  /* default 320 X 240 */
	   count;

#if 0
	if (gVidMode == VMODE640)
	{
		num_textures  = BIG_NUM_TEXTURE_PAGES; /* 640 X 480 */
		num_256_cluts = BIG_NUM_CLUT256_PAGES; /* 640 X 480 */
		num_16_cluts  = BIG_NUM_CLUT16_PAGES;  /* 640 X 480 */
	}
	else if (gVidMode == VMODE512)
	{
		num_textures  = MED_NUM_TEXTURE_PAGES; /* 512 X 240 */
		num_256_cluts = MED_NUM_CLUT256_PAGES; /* 512 X 240 */
		num_16_cluts  = MED_NUM_CLUT16_PAGES;  /* 512 X 240 */
	}
#endif

	switch(type)
	{
		case VRAM_SCREEN:
			break;

		case VRAM_TEXTURE:
			for (i = 0; i < num_textures; i++)
			{
				if (texture[i].used == VRAM_UNUSED)  /* if it isn't used */
				{
					texture[i].used = 1;   /* Mark it used */
					texture[i].num = 0;    /* Only one Texture page */
					return(&texture[i]);   /* Return a pointer to it */
				}
			}
			Cerror();  /* do error stuff cos it's all full */
			break;

		case VRAM_MULT_PAGE:
			for (i = 0; i < num_textures; i++)
			{
				count = num;
				while(texture[i++].used == VRAM_UNUSED && i < num_textures)
					count--;
				if (count <= 0) /* there were enough consecutive... */
				{
					for (count = 0; count < num; count++)
					{
						texture[i--].used = 1;   /* Mark it used */
					}
					i++; /* be sure to point to the correct one... */
					texture[i].num = num-1;	// So we can free it right...
					return(&texture[i]);   /* Return a pointer to it */
				}
			}
			Cerror();  /* do error stuff cos it's all full */
			break;

		case VRAM_CLUT16:
			for (i = 0; i < num_16_cluts; i++)
				if (!clut16[i].used)  /* if it isn't used */
				{
					clut16[i].used = 1;	  /* Mark it used */
					return(&clut16[i]);	  /* Return a pointer to it */
				}
			Cerror();  /* do error stuff cos it's all full */
			break;

		case VRAM_CLUT256:
			for (i = 0; i < num_256_cluts; i++)
				if (!clut256[i].used)  /* if it isn't used */
				{
					clut256[i].used = 1;   /* Mark it used */
					return(&clut256[i]);   /* Return a pointer to it */
				}
			Cerror();  /* do error stuff cos it's all full */
			break;
	}
	return NULL;
}

/*-------------------------------------------------------------
  SetVramTPage ~ Set tpage of vram_t*	  CGD
		tp:0 = 4 bit,  1 = 8bit, 2 = 16bit  color
		abr: Semi Transparency rate (see GetTPage)
-------------------------------------------------------------*/
USHORT SetVramTPage(vram_t *vram, long tp, long abr)
{
	vram->tpage = GetTPage(tp, abr, vram->x, vram->y);
	return vram->tpage;
}

/*************************************************************************
 *	CLUT.C
 *
 *	Copyright 1994 GameTek Inc.  All rights reserved.
 *
 *	Written: by Mike Heilemann
 * Started:	08/08/94
 ********************************************************************/


/*************************************************************************
 * LOAD 4 BIT CLUT - Load 16 color clut properly
 *************************************************************************/
USHORT Load4BitClut(u_long *clut, int x, int y)
{
	ULONG buffer[128];
	RECT rect;
	int id, i;

	rect.x = x;
	rect.y = y;
	rect.w = 256;
	rect.h = 1;
	StoreImage(&rect, (unsigned long *)buffer);	// LOAD CLUT ONLY LOADS 256 even in 4 bit mode
	id = LoadClut(clut, x, y);

	DrawSync(0);

	for (i = 0; i < 8; i++)
		buffer[i] = clut[i];
	LoadImage(&rect, (unsigned long *)buffer);	// Store it back

	return(id);
}


/*************************************************************************
 * REGISTER CLUT - Allocs and loads a CLUT into VRAM memory
 *
 *    TYPE - Accepts either Enumerated type or Number of colors to define
 *           type of CLUT being registered.
 *************************************************************************/
vram_t *register_CLUT(unsigned long *clut, int type)
{
	vram_t *vram_ptr;

   // accept both passing conventions
	if (type == VRAM_CLUT16 || type == 16)
	{
		vram_ptr = alloc_VRam(VRAM_CLUT16, 0);
		vram_ptr->CLUT_ID = Load4BitClut(clut, vram_ptr->x, vram_ptr->y);
	}

	if (type == VRAM_CLUT256 || type == 256)
	{
		vram_ptr = alloc_VRam(VRAM_CLUT256, 0);
		vram_ptr->CLUT_ID = LoadClut(clut, vram_ptr->x, vram_ptr->y);
	}
	return(vram_ptr);
}


/*************************************************************************
 * RELOAD CLUT - Load data into already allocated CLUT.
 *************************************************************************/
unsigned short ReloadCLUT(Ulong *data, vram_t *vram_ptr)
{
	vram_ptr->CLUT_ID = LoadClut(data, vram_ptr->x, vram_ptr->y);
	return (vram_ptr->CLUT_ID);
}


/*************************************************************************
 * RELOAD 4 BIT CLUT - Load data into already allocated CLUT.
 *************************************************************************/
void Reload4BitClut(Ulong *data, vram_t *vram_ptr)
{
	vram_ptr->CLUT_ID = Load4BitClut(data, vram_ptr->x, vram_ptr->y);
}


/*************************************************************************
 * FREE CLUT - Marks a clut as unused.
 *************************************************************************/
void free_clut(vram_t *vram_ptr)
{
	if (vram_ptr)
		vram_ptr->used = VRAM_UNUSED;	 /* 0	*/
}


typedef struct {
	BYTE rgb[3];
} delta_t;

short *palette256[MED_NUM_CLUT256_PAGES+MED_NUM_CLUT16_PAGES / 16];
delta_t *delta[MED_NUM_CLUT256_PAGES+MED_NUM_CLUT16_PAGES / 16];	// Malloc this later
delta_t *curdelta[MED_NUM_CLUT256_PAGES+MED_NUM_CLUT16_PAGES / 16];	// Malloc this later
BYTE NumFadeFrames;
BYTE FadeCounter;
unsigned char FadeDelta;


/*************************************************************************
 * FADE IN - Fades in a screen after it is set up, by storing the palettes
 *		in DRAM on the init call, then on subsequent calls it increments the
 *		colors by delta, until they are all at their max.
 *
 *		NOTE: First ready page for display, then Call fade_in with init,
 *		then display to screen and in a loop call fade_in.
 *
 *		This function take care of 16 color cluts as 256 and does them all
 *		16 paletts at a time... (Thus the MED_NUM_CLUT16_PAGES / 16)
 *************************************************************************/
void fade_in(INT num_frames, BYTE init)
{
	RECT rect;
	register int i,
					 j,
					 num = 0;
	int			 tmp;
	unsigned short buff[512];

	rect.h = 1;	// Height never changes...
	rect.w = 256;

	if (init)
	{
		FadeIn = 1;
		FadeCounter = 0;
		NumFadeFrames = num_frames;
		memset(buff, 0, 512);	// 0 buffer
		FadeDelta = 0x80 / num_frames;
		for (i = 0; i < N_BG; i++)
			if (picture[i].clutType == VRAM_16BIT)	// 16 BIT
			{
				for (j = 0; j < 8; j++)
				{
					setShadeTex(&picture[i].poly[j], 0);
					picture[i].poly[j].r0 = 0;
					picture[i].poly[j].g0 = 0;
					picture[i].poly[j].b0 = 0;
				}
			}

		for (num = 0; num < MED_NUM_CLUT256_PAGES+MED_NUM_CLUT16_PAGES / 16; num++)
		{
			palette256[num] = 0;
			if ((num >= MED_NUM_CLUT16_PAGES / 16 && clut256[num-MED_NUM_CLUT16_PAGES / 16].used) || num < MED_NUM_CLUT16_PAGES / 16)
			{
				palette256[num] = (short *)AllocMem(512); // 512 bytes
				delta[num] = (delta_t *)AllocMem(256 * 3);
				curdelta[num] = (delta_t *)AllocMem(256 * 3);

				if (num < MED_NUM_CLUT16_PAGES / 16)	// Do 16 bit cluts
				{
					// SAVE PALETTE
					rect.x = clut16[num*16].x;
					rect.y = clut16[num*16].y;
				}
				else
				{
					// SAVE PALETTE
					rect.x = clut256[num-MED_NUM_CLUT16_PAGES / 16].x;
					rect.y = clut256[num-MED_NUM_CLUT16_PAGES / 16].y;
				}
				StoreImage(&rect, (long *)palette256[num]);

				// ZERO PALETTE IN VRAM
				LoadImage(&rect, (long *)buff);

				for (i = 0; i < 256; i++)
				{
					tmp = palette256[num][i];
					for (j = 0; j < 3; j++)
					{
						delta[num][i].rgb[j] = ((tmp & 0x001F) << 3) / NumFadeFrames;
						if (tmp & 0x001F)
							curdelta[num][i].rgb[j] = 8;
						else
							curdelta[num][i].rgb[j] = 0;

						tmp = tmp >> 5;
					}
				}
			}
		}
	}
	else
	if (FadeIn)
	{
		FadeCounter++;

		for (num = 0; num < MED_NUM_CLUT256_PAGES + MED_NUM_CLUT16_PAGES / 16; num++)
			if ((num >= MED_NUM_CLUT16_PAGES / 16 && clut256[num-MED_NUM_CLUT16_PAGES / 16].used) || num < MED_NUM_CLUT16_PAGES / 16)
			{
				if (num < MED_NUM_CLUT16_PAGES / 16)	// Do 16 bit cluts
				{
					// PALETTE X Y
					rect.x = clut16[num*16].x;
					rect.y = clut16[num*16].y;
				}
				else
				{
					// PALETTE X Y
					rect.x = clut256[num-MED_NUM_CLUT16_PAGES / 16].x;
					rect.y = clut256[num-MED_NUM_CLUT16_PAGES / 16].y;
				}

				for (i = 0; i < 256; i++)
				{
					buff[i] = 0;
					for (j = 0; j < 3; j++)
					{
						curdelta[num][i].rgb[j] += delta[num][i].rgb[j];
						buff[i] += ((curdelta[num][i].rgb[j] >> 3) << (5 * j));
					}
				}

				LoadImage(&rect, (long *)buff);
				DrawSync(0);
			}

			for (i = 0; i < N_BG; i++)
				if (picture[i].clutType == VRAM_16BIT)	// 16 BIT
				{
					for (j = 0; j < 8; j++)
					{
						picture[i].poly[j].r0 += FadeDelta;
						picture[i].poly[j].g0 += FadeDelta;
						picture[i].poly[j].b0 += FadeDelta;
					}
				}
	}

	if (FadeIn && FadeCounter >= NumFadeFrames)		// free up memory we allocated for fade in...
	{
		for (i = 0; i < MED_NUM_CLUT256_PAGES+MED_NUM_CLUT16_PAGES / 16; i++)
			if (palette256[i])
			{
				if (i < MED_NUM_CLUT16_PAGES / 16)	// Do 16 bit cluts
				{
					// PALETTE X Y
					rect.x = clut16[i*16].x;
					rect.y = clut16[i*16].y;
				}
				else
				{
					rect.x = clut256[i-MED_NUM_CLUT16_PAGES / 16].x;
					rect.y = clut256[i-MED_NUM_CLUT16_PAGES / 16].y;
				}
				LoadImage(&rect, (long *)palette256[i]);
				DrawSync(0);
				FreeMem(delta[i]);
				FreeMem(curdelta[i]);
				FreeMem(palette256[i]);
				palette256[i] = 0;
				delta[i] = 0;
				curdelta[i] = 0;
			}
		if (picture[i].clutType == VRAM_16BIT)	// 16 BIT
			setShadeTex(&picture[i].poly[j], 1);
		FadeIn = 0;
	}
}


/*************************************************************************
 * FADE OUT - Fades out a screen after it is set up, by storing the deltas
 *		in DRAM on the init call, then on subsequent calls it decrements the
 *		colors by delta, until they are zero
 *
 *		RETURNS: done = 0 not done
 *					done = 1 when finished
 *
 *		NOTE: First ready page for display, then Call fade_in with init,
 *		then display to screen and in a loop call fade_in.
 *
 *		buff[i] += ((palette16[num][i] & 0x7C000000) >> 26) / num_frames * 0x04000000;
 *						real number of color / number of frames to fade in * 1 for that 5 bit color.
 *************************************************************************/
void fade_out(INT num_frames, BYTE init)
{
	RECT rect;
	register int i,
					 j,
					 x,
					 num = 0;
	int			 tmp;
	unsigned short buff[512];
	unsigned char FadeDeltaR,
					  FadeDeltaG,
					  FadeDeltaB;


	rect.h = 1;	// Height never changes...
	rect.w = 256;

	if (init)
	{
	 	if (FadeIn)		// free up memory we allocated for fade in...
		{
			FadeCounter = NumFadeFrames;
			fade_in(0, 0);	// End fade in
		}

		FadeOut = 1;
		FadeCounter = 0;
		NumFadeFrames = num_frames;
		FadeDelta = 0x80 / num_frames;
	  	for (i = 0; i < N_BG; i++)
	  		if (picture[i].clutType == VRAM_16BIT)	// 16 BIT
	  			for (j = 0; j < 8; j++)
				{
	  				setShadeTex(&picture[i].poly[j], 0);
					picture[i].poly[j].r0 = 0x80;
					picture[i].poly[j].g0 = 0x80;
					picture[i].poly[j].b0 = 0x80;
				}

		for (num = 0; num < MED_NUM_CLUT256_PAGES+MED_NUM_CLUT16_PAGES / 16; num++)
			if ((num >= MED_NUM_CLUT16_PAGES / 16 && clut256[num-MED_NUM_CLUT16_PAGES / 16].used) || num < MED_NUM_CLUT16_PAGES / 16)
			{
				palette256[num] = (short *)AllocMem(512); // 512 bytes
				delta[num] = (delta_t *)AllocMem(256 * 3);
				curdelta[num] = (delta_t *)AllocMem(256 * 3);

				if (num < MED_NUM_CLUT16_PAGES / 16)	// Do 16 bit cluts
				{
					// SAVE PALETTE
					rect.x = clut16[num*16].x;
					rect.y = clut16[num*16].y;
				}
				else
				{
					// SAVE PALETTE
					rect.x = clut256[num-MED_NUM_CLUT16_PAGES / 16].x;
					rect.y = clut256[num-MED_NUM_CLUT16_PAGES / 16].y;
				}
				StoreImage(&rect, (unsigned long *)palette256[num]);
				DrawSync(0);

				for (i = 0; i < 256; i++)
				{
					tmp = palette256[num][i];
					for (j = 0; j < 3; j++)
					{
						x = ((tmp & 0x001F) << 3);
						curdelta[num][i].rgb[j] = x;
						delta[num][i].rgb[j] = x / NumFadeFrames;
						tmp = tmp >> 5;
					}
				}
			}
	}
	else
	if (FadeOut)
	{
		FadeCounter++;
		for (num = 0; num < MED_NUM_CLUT256_PAGES + MED_NUM_CLUT16_PAGES / 16; num++)
			if ((num >= MED_NUM_CLUT16_PAGES / 16 && clut256[num-MED_NUM_CLUT16_PAGES / 16].used) || num < MED_NUM_CLUT16_PAGES / 16)
			{
				if (num < MED_NUM_CLUT16_PAGES / 16)	// Do 16 bit cluts
				{
					// PALETTE X Y
					rect.x = clut16[num*16].x;
					rect.y = clut16[num*16].y;
				}
				else
				{
					// PALETTE X Y
					rect.x = clut256[num-MED_NUM_CLUT16_PAGES / 16].x;
					rect.y = clut256[num-MED_NUM_CLUT16_PAGES / 16].y;
				}

				for (i = 0; i < 256; i++)
				{
					buff[i] = 0;
					for (j = 0; j < 3; j++)
					{
						curdelta[num][i].rgb[j] -= delta[num][i].rgb[j];
						buff[i] += ((curdelta[num][i].rgb[j] >> 3) << (5 * j));
					}
				}

				LoadImage(&rect, (long *)buff);
				DrawSync(0);
			}

			for (i = 0; i < N_BG; i++)
				if (picture[i].clutType == VRAM_16BIT)	// 16 BIT
				{
					for (j = 0; j < 8; j++)
					{
						picture[i].poly[j].r0 -= FadeDelta;
						picture[i].poly[j].g0 -= FadeDelta;
						picture[i].poly[j].b0 -= FadeDelta;
					}
				}

			for (i = 0; i < num_dialogs; i++)
			{
				FadeDeltaR = dialog[i].r;
				FadeDeltaG = dialog[i].g;
				FadeDeltaB = dialog[i].b;

				ChangeDialogColor(i, dialog[i].r - dialog[i].r / NumFadeFrames * FadeCounter, dialog[i].g - dialog[i].g / NumFadeFrames * FadeCounter, dialog[i].b - dialog[i].b / NumFadeFrames * FadeCounter);

				dialog[i].r = FadeDeltaR;
				dialog[i].g = FadeDeltaG;
				dialog[i].b = FadeDeltaB;
			}
	}
	if (FadeOut && FadeCounter >= NumFadeFrames)		// free up memory we allocated for fade in...
	{
		for (i = 0; i < MED_NUM_CLUT256_PAGES+MED_NUM_CLUT16_PAGES / 16; i++)
			if (palette256[i])
			{
				if (i < MED_NUM_CLUT16_PAGES / 16)	// Do 16 bit cluts
				{
					// PALETTE X Y
					rect.x = clut16[i*16].x;
					rect.y = clut16[i*16].y;
				}
				else
				{
					rect.x = clut256[i-MED_NUM_CLUT16_PAGES / 16].x;
					rect.y = clut256[i-MED_NUM_CLUT16_PAGES / 16].y;
				}
				memset(palette256[i], 0, 512);	// blank it out for last one...
				LoadImage(&rect, (long *)palette256[i]);
				DrawSync(0);
				FreeMem(delta[i]);
				FreeMem(curdelta[i]);
				FreeMem(palette256[i]);
				palette256[i] = 0;
				delta[i] = 0;
				curdelta[i] = 0;
			}
		for (i = 0; i < N_BG; i++)	// Set background to 0
			if (picture[i].clutType == VRAM_16BIT)	// 16 BIT
			{
				for (j = 0; j < 8; j++)
				{
					picture[i].poly[j].r0 = 0;
					picture[i].poly[j].g0 = 0;
					picture[i].poly[j].b0 = 0;
				}
			}
		FadeOut = 0;
	}
}


/*-------------------------------------------------------------
  Set0Transparent ~ Set color 0 Transparent in CLUT
-------------------------------------------------------------*/
void Set0Transparent(vram_t *clut)
{
	RECT r;
	ULONG data = 0x0000;

	r.x = clut->x;
	r.y = clut->y;
	r.w = 1;
	r.h = 1;
	LoadImage(&r, &data);
}

/*-------------------------------------------------------------
  ReAllocVram ~
-------------------------------------------------------------*/
vram_t * ReAllocVram(vram_t *vram, int type, int num)
{
	if (vram == NULL)
	{
		vram = alloc_VRam(type, num);
	}
	else
	{
		vram->used = 1;
	}
	return vram;
}

/*-------------------------------------------------------------
  FindVramAt ~ Find VRAM at X, Y
-------------------------------------------------------------*/
vram_t *FindVramAt(int x, int y)
{
	vram_t *vram = NULL;
	int i, working;

	int 
//		num_textures  = NUM_TEXTURE_PAGES, /* default 320 X 240 */
//	   num_256_cluts = NUM_CLUT256_PAGES, /* default 320 X 240 */
//	   num_16_cluts  = NUM_CLUT16_PAGES,  /* default 320 X 240 */
	   count;

#if 0	   
	if (gVidMode == VMODE640)
	{
		num_textures  = BIG_NUM_TEXTURE_PAGES; /* 640 X 480 */
		num_256_cluts = BIG_NUM_CLUT256_PAGES; /* 640 X 480 */
		num_16_cluts  = BIG_NUM_CLUT16_PAGES;  /* 640 X 480 */
	}
	else if (gVidMode == VMODE512)
	{
		num_textures  = MED_NUM_TEXTURE_PAGES; /* 512 X 240 */
		num_256_cluts = MED_NUM_CLUT256_PAGES; /* 512 X 240 */
		num_16_cluts  = MED_NUM_CLUT16_PAGES;  /* 512 X 240 */
	}
#endif

	working = True;
	for (i = 0; (i < num_textures) && working; i++)
	{
		if (texture[i].x == x && texture[i].y == y)
		{
			vram = &texture[i];
			working = False;
		}
	}
	for (i = 0; (i < num_256_cluts) && working; i++)
	{
		if (clut256[i].x == x && clut256[i].y == y)
		{
			vram = &clut256[i];
			vram->CLUT_ID = GetClut(x,y);
			working = False;
		}
	}
	for (i = 0; (i < num_16_cluts) && working; i++)
	{
		if (clut16[i].x == x && clut16[i].y == y)
		{
			vram = &clut16[i];
			vram->CLUT_ID = GetClut(x,y);
			working = False;
		}
	}
	return vram;
}

/*-------------------------------------------------------------
  AllocVramAt ~
-------------------------------------------------------------*/
vram_t * AllocVramAt(int x, int y)
{
	vram_t *vram;

	vram = FindVramAt(x, y);
	vram->used = 1;
	return vram;
}

/*-------------------------------------------------------------
  FreeTextureAt ~ Free Texture page at X, Y
-------------------------------------------------------------*/
void FreeVramAt(int x, int y)
{
	vram_t *vram;
	
	vram = FindVramAt(x, y);
	free_VRam(vram);
	return;
}

/*-------------------------------------------------------------
  AllocBottomTpages ~ 
-------------------------------------------------------------*/
vram_t * AllocBottomTpages(int num)
{
	int i, x, y;
	vram_t *vram;
	
	for (i = 0; i < num; i++)
	{
		vram = AllocVramAt(512 + (64 * i), 256);
	}
	vram = FindVramAt(512, 256);
	vram->num = num;
	return vram;	
}  /* End AllocBottomTpages() */
