/*************************************************************************
	 MMVRAM.H

    Written by: Mike Heilemann
    Updated by: Christopher Deel

	Copyright 1994 GameTek
	All rights reserved.
 *************************************************************************/

//#define DEBUG_ALLOC_TEXTURE 1
//#define DEBUG_ALLOC_CLUT 1
//#define DEBUG_FREE_VRAM 1

/* 320 X 240 */
#define NUM_TEXTURE_PAGES   22
#define NUM_CLUT16_PAGES    32
#define NUM_CLUT256_PAGES   30

/* 512 X 240 */
#define MED_NUM_TEXTURE_PAGES   16
#define MED_NUM_CLUT16_PAGES    80	// was 32
#define MED_NUM_CLUT256_PAGES   59 	// was 62

/* 640 X 480 */
#define BIG_NUM_TEXTURE_PAGES   12
#define BIG_NUM_CLUT16_PAGES    80	// was 32
#define BIG_NUM_CLUT256_PAGES   59	// was 62

typedef enum
{
	VRAM_UNUSED=0,
	VRAM_CLUT16 = 1, 
	VRAM_CLUT256, 
	VRAM_16BIT,  /* Used for CLUT type */
	VRAM_SCREEN, 
	VRAM_TEXTURE,	
	VRAM_MULT_PAGE
} VRamDesc;

#define VRamDescToTpMode(desc) (desc - 1)   /* Convert VRamDesc to Texture type */

/*---- Texture Data ----*/
typedef struct tpageData		/* Per object in page          */
{
	short desc;						/* what is area used for       */
	Ushort tpage;					/* TPage ID    */
	int u;							/* x in vram relative to tpage */
	int v;							/* y in vram relative to tpage */
} tpageData;


/*--- Vram Type --- */
typedef struct			/* Per page                    */
{
	short  	used;		/* What is this page used for    */
	Ushort	tpage;  	/* Texture ID                    */
	Ushort  num;     /* Number of consecutive tpages  */
	Ushort 	CLUT_ID; /* CLUT ID	                     */
	int		x,			/* Page X	                     */
   		  	y,			/* Page Y	                     */
   			w,			/* Page Width                    */
	   		h;			/* Page Height                   */
} vram_t;


void initVRamHandling(void);
void free_VRam(vram_t *vram);
int  loadVRam(char *name);
vram_t *alloc_VRam(int type, int num);


/*************************************************************************
 * CLUT STUFF
 *************************************************************************/

typedef struct col {
	unsigned short	red : 5;
	unsigned short	green : 5;
	unsigned short	blue : 5;
	unsigned short	stp : 1;
} COL_STRUCT;

/*NB the CLUTS have not yet been confirmed as below*/
typedef enum {BLACK=0,BLUE,RED,MAGENTA,GREEN,CYAN,YELLOW,WHITE} inkType;
//#define EGA_CLUT 32

vram_t *register_CLUT(unsigned long *clut, int type);

extern vram_t clut256[BIG_NUM_CLUT256_PAGES]; /* structs with CLUT info */
extern vram_t clut16[BIG_NUM_CLUT16_PAGES];

USHORT SetVramTPage(vram_t *vram, long tp, long abr);
void fade_in(INT num_frames, BYTE init);
void fade_out(INT num_frames, BYTE init);
void Set0Transparent(vram_t *clut);
vram_t * FindVramAt(int x, int y);
void FreeVramAt(int x, int y);
vram_t * AllocBottomTpages(int num);

