/*************************************************************************
 *          Movie Sample Program
 *
 *      Copyright (C) 1994,5 by Sony Corporation
 *          All rights Reserved
 *
 *  Sample program of streaming from CD-ROM
 *
 *      1) This sample plays movie even if it's width or height are not
 *         multiple of 16.
 *         But still there is a restriction as follows.
 *         Width of image and horizontal position of image in VRAM (X)
 *         must be even when we play movie with 24 bit mode.
 *         There is no restriction like above when it plays movie
 *         with 16 bit mode.
 *
 *      2) This sample can play multiple movie sequentialy.
 *
 *   Version    Date
 *  ------------------------------------------------------------------------
 *  1.10        Jul.20 1995 ume
 *                  (Changes from tuto0.c ver1.50)
 *                  It supports the movie whose width or height are
 *                  not multiple of 16.
 *                  It plays multiple movie sequentialy.
 *************************************************************************/
/*************************************************************************
 * Modified by Mike Heilemann -
 *		Only clears portion where movie sits.
 *		Allocates buffers as needed then frees them.
 *		Allows movies to play in a frame loop.
 *		Various other bug fixes & optimizations.
 *************************************************************************/

#include <sys/types.h>
#include <stdio.h>
#include <libgte.h>
#include <libgpu.h>
#include <libcd.h>
#include <libgs.h>
#include "prjhdrs.h"          /* Project header files */
#include "fbgequ.h"
#include "fbequ.h"
#include "fbram.h"
#include "fbstats.h"
#include "movie.h"

extern DspBuf_t gDspBuf[2];
extern char gCurrDB;		/* Current Display Buffer # */
extern signed char WeekNum;
extern signed char HumanTeamNum;
extern char CurSong;
extern UBYTE MyGameType;
extern RECT clearRect;

int  id;             /* Buffer ID */
char MovieClear = 1;

#define Cerror()  CmarkerSub("Error", __FILE__, __LINE__)
#define scrWidth	512
//#define scrWidth	320
#define scrHeight	240


void CmarkerSub ( char *msg , char *file , int line );


/* Parameters for MDEC movie */
typedef struct {
    char *fileName;
    char is24bit;
    short endFrame;
    short posX;
    short posY;
    short width;
    short height;
} MovieInfo;

/*
 * MDEC movie data to be played
 *
 *     caution:
 *         posX and width of image must be even with 24 bit mode.
 *         There is no restriction like that with 16 bit mode.
 */
MovieInfo movieInfo[] = {

    /* file name		24bit,  end,    x,    y,	wid, height */
    /* 24 bit */
    {"INTRO",	  		TRUE,	  209,    0,    0,	512,	240},
    {"FBINTRO",	 	TRUE,   631,    0,    0,	512,	240}, // 633

// 2
    {"B0\\B0",			 TRUE,  300,    0,    0,	512,	240},
    {"B0\\B1",			 TRUE,  259,    0,    0,	512,	240},
    {"B0\\B2",			 TRUE,  267,    0,    0,	512,	240},
    {"B0\\B3",			 TRUE,  244,    0,    0,	512,	240},
    {"B0\\B4",			 TRUE,  280,    0,    0,	512,	240},
    {"B0\\B5",			 TRUE,  226,    0,    0,	512,	240},
    {"B0\\B6",			 TRUE,  240,    0,    0,	512,	240},
    {"B0\\B7",			 TRUE,  237,    0,    0,	512,	240},
    {"B0\\B8",			 TRUE,  258,    0,    0,	512,	240},
    {"B0\\B9",			 TRUE,  241,    0,    0,	512,	240},
    {"B0\\B10", 		 TRUE,  293,    0,    0,	512,	240},
    {"B0\\B11", 		 TRUE,  228,    0,    0,	512,	240},
    {"B0\\B12", 		 TRUE,  285,    0,    0,	512,	240},
    {"B0\\B13", 		 TRUE,  228,    0,    0,	512,	240},
    {"B0\\B14", 		 TRUE,  241,    0,    0,	512,	240}, // 244
    {"B0\\B15", 		 TRUE,  189,    0,    0,	512,	240},
    {"B0\\B16", 		 TRUE,  260,    0,    0,	512,	240},
    {"B0\\B17", 		 TRUE,  255,    0,    0,	512,	240},
    {"B0\\B18", 		 TRUE,  210,    0,    0,	512,	240},
    {"B0\\B19", 		 TRUE,  228,    0,    0,	512,	240},
    {"B1\\B20", 		 TRUE,  246,    0,    0,	512,	240},
    {"B1\\B21", 		 TRUE,  233,    0,    0,	512,	240},	// 234
    {"B1\\B22", 		 TRUE,  260,    0,    0,	512,	240},
    {"B1\\B23", 		 TRUE,  255,    0,    0,	512,	240},
    {"B1\\B24", 		 TRUE,  249,    0,    0,	512,	240},
    {"B1\\B25", 		 TRUE,  262,    0,    0,	512,	240},
    {"B1\\B26", 		 TRUE,  225,    0,    0,	512,	240},
    {"B1\\B27", 		 TRUE,  241,    0,    0,	512,	240},
    {"B1\\B28", 		 TRUE,  254,    0,    0,	512,	240},
    {"B1\\B29", 		 TRUE,  187,    0,    0,	512,	240},
    {"B1\\B30", 		 TRUE,  237,    0,    0,	512,	240},
    {"B1\\B31", 		 TRUE,  417,    0,    0,	512,	240},

	// SPECIFIC POSTGAME - 66
    {"P0\\PG0",	  	FALSE,   86,  106,   64,	300,	150},
    {"P0\\PG1",	  	FALSE,  105,  106,   64,	300,	150},
    {"P0\\PG2",	  	FALSE,   94,  106,   64,	300,	150},
    {"P0\\PG3",	  	FALSE,  120,  106,   64,	300,	150},
    {"P0\\PG4",	  	FALSE,   55,  106,   64,	300,	150},
    {"P0\\PG5",	  	FALSE,   72,  106,   64,	300,	150},
    {"P0\\PG6",	  	FALSE,   78,  106,   64,	300,	150},
    {"P0\\PG7",	  	FALSE,   79,  106,   64,	300,	150},
    {"P0\\PG8",	  	FALSE,  100,  106,   64,	300,	150},
    {"P0\\PG9",	  	FALSE,   83,  106,   64,	300,	150},
    {"P0\\PG10",		FALSE,   92,  106,   64,	300,	150},
    {"P0\\PG11",		FALSE,  102,  106,   64,	300,	150},
    {"P0\\PG12",		FALSE,   81,  106,   64,	300,	150},
    {"P0\\PG13",		FALSE,  107,  106,   64,	300,	150},
    {"P0\\PG14",		FALSE,   89,  106,   64,	300,	150},
    {"P0\\PG15",		FALSE,   96,  106,   64,	300,	150},
    {"P0\\PG16",		FALSE,   99,  106,   64,	300,	150},
    {"P0\\PG17",		FALSE,   86,  106,   64,	300,	150},
    {"P0\\PG18",		FALSE,   80,  106,   64,	300,	150},
    {"P0\\PG19",		FALSE,  107,  106,   64,	300,	150},

	// GENERAL POSTGAME
    {"P0\\PG20",		FALSE,   46,  106,   64,	300,	150},
    {"P0\\PG21",		FALSE,   46,  106,   64,	300,	150},
    {"P0\\PG22",		FALSE,   58,  106,   64,	300,	150},
    {"P0\\PG23",		FALSE,   63,  106,   64,	300,	150},
    {"P0\\PG25",		FALSE,   39,  106,   64,	300,	150},
    {"P0\\PG26",		FALSE,   31,  106,   64,	300,	150},
    {"P0\\PG27",		FALSE,   45,  106,   64,	300,	150},
    {"P0\\PG28",		FALSE,   46,  106,   64,	300,	150},
	 {"P1\\PG30",	  	FALSE,   78,  106,   64,	300,	150},
    {"P1\\PG31",	  	FALSE,   82,  106,   64,	300,	150},
    {"P1\\PG32",	  	FALSE,   48,  106,   64,	300,	150},
    {"P1\\PG33",	  	FALSE,   47,  106,   64,	300,	150},
    {"P1\\PG34",	  	FALSE,   62,  106,   64,	300,	150},

	// PLAYOFF MOVIES
    {"P1\\PG37",	  	FALSE,   84,  106,   64,	300,	150},
    {"P1\\PG38",	  	FALSE,   80,  106,   64,	300,	150},
    {"P1\\PG39",	  	FALSE,   90,  106,   64,	300,	150},
    {"P1\\PG40",		FALSE,   80,  106,   64,	300,	150},
    {"P1\\PG41",		FALSE,  110,  106,   64,	300,	150},
    {"P1\\PG42",		FALSE,   96,  106,   64,	300,	150},
    {"P1\\PG44",		FALSE,   68,  106,   64,	300,	150},

	// HINTS
    {"H0\\H0",			 TRUE,  142,    0,    0,	512,	240},
    {"H0\\H1",			 TRUE,   83,    0,    0,	512,	240},
    {"H0\\H2",			 TRUE,  116,    0,    0,	512,	240},
    {"H0\\H3",			 TRUE,  143,    0,    0,	512,	240},
    {"H0\\H4",			 TRUE,  105,    0,    0,	512,	240},
    {"H0\\H5",			 TRUE,  111,    0,    0,	512,	240},
    {"H0\\H6",			 TRUE,   98,    0,    0,	512,	240},
    {"H0\\H7",			 TRUE,  139,    0,    0,	512,	240},
    {"H0\\H8",			 TRUE,  124,    0,    0,	512,	240},
    {"H0\\H9",			 TRUE,  132,    0,    0,	512,	240},
    {"H0\\H10",		 TRUE,  142,    0,    0,	512,	240},
    {"H0\\H11",		 TRUE,  132,    0,    0,	512,	240},
    {"H0\\H12",		 TRUE,  127,    0,    0,	512,	240},
    {"H0\\H13",		 TRUE,   45,    0,    0,	512,	240},
    {"H0\\H14",		 TRUE,   78,    0,    0,	512,	240},
    {"H0\\H15",		 TRUE,  123,    0,    0,	512,	240},
    {"H0\\H16",		 TRUE,  127,    0,    0,	512,	240},
    {"H0\\H17",		 TRUE,  104,    0,    0,	512,	240},
    {"H0\\H19",		 TRUE,   97,    0,    0,	512,	240},
    {"H0\\H20",		 TRUE,  124,    0,    0,	512,	240},
    {"H0\\H21",		 TRUE,  135,    0,    0,	512,	240},
    {"H0\\H22",		 TRUE,  100,    0,    0,	512,	240},
    {"H0\\H23",		 TRUE,  103,    0,    0,	512,	240},
    {"H0\\H24",		 TRUE,  115,    0,    0,	512,	240},
    {"H0\\H25",		 TRUE,   96,    0,    0,	512,	240},
    {"H0\\H26",		 TRUE,   91,    0,    0,	512,	240},
    {"H0\\H27",		 TRUE,  127,    0,    0,	512,	240},
    {"H0\\H28",		 TRUE,   93,    0,    0,	512,	240},
    {"H0\\H29",		 TRUE,   71,    0,    0,	512,	240},
    {"H1\\H30",		 TRUE,   87,    0,    0,	512,	240}, //186
    {"H1\\H31",		 TRUE,  190,    0,    0,	512,	240},	//125
    {"H1\\H32",		 TRUE,  125,    0,    0,	512,	240}, //125

	//	SPEECHES
    {"S\\S0",			 TRUE,  187,    0,    0,	512,	240},	// 151
    {"S\\S1",			 TRUE,  163,    0,    0,	512,	240},	//  84
    {"S\\S2",			 TRUE,  134,    0,    0,	512,	240},	//  96
    {"S\\S3",			 TRUE,  128,    0,    0,	512,	240},	// 108

    {"S\\S4",			 TRUE,  135,    0,    0,	512,	240},	// 129
    {"S\\S5",			 TRUE,   87,    0,    0,	512,	240},	// 144
    {"S\\S6",			 TRUE,  106,    0,    0,	512,	240},	// 103
    {"S\\S10",			 TRUE,   88,    0,    0,	512,	240},	// 139
    {"S\\S7",			 TRUE,  127,    0,    0,	512,	240},	// 144
    {"S\\S11",			 TRUE,   99,    0,    0,	512,	240},	// 195
    {"S\\S8",			 TRUE,  140,    0,    0,	512,	240},	// 110
    {"S\\S9",			 TRUE,  133,    0,    0,	512,	240},	// 157
    {"S\\S14",			 TRUE,  151,    0,    0,	512,	240},	// 134
    {"S\\S12",			 TRUE,  140,    0,    0,	512,	240},	// 180
    {"S\\S13",			 TRUE,  110,    0,    0,	512,	240},	// 131

    {"MISC\\CHAMP",	 TRUE,  275,    0,    0,	512,	240},
    {"MISC\\KONAMI",	 TRUE,  140,    0,    0,	320,	240},
};
#define NMOVIE (sizeof(movieInfo) / sizeof(MovieInfo))

/* Macros related with decoding mode */
#define VRAMPIX(pixels, is24bit) ((is24bit) ? ((pixels) * 3) / 2 : (pixels))
#define DCT_MODE(is24bit) ((is24bit) ? 3 : 2)

/* Decoding environment */
typedef struct {
    u_long  *vlcbuf[2]; /* VLC double buffer for encoded image */
    char vlcid;         /* Current buffer id: 0 or 1 */
    u_short *imgbuf[2]; /* Double buffer for decoded image */
    char imgid;         /* Current image buffer id */
    RECT rect[2];			/* Double buffer on VRAM */
    char rectid;        /* Current VRAM double buffer id */
    RECT slice;      	/* Image strip for DecDCTout */
    char isdone;        /* 0 = incomplete DEC decode, 1=complete or timeout */
    char is24bit;       /* 0 = 16 bit mode, 1 = 24 bit mode */
} DECENV;
static DECENV   dec;    /* Decoding environment */

/*
 *  Ring buffer for streaming.
 *  It contains frame data from CD-ROM.
 *  The size of this buffer should be more than 2 frames.
 */
//#define RING_SIZE   32      /* Size of ring buffer (unit: sector) */
#define RING_SIZE   22      /* Size of ring buffer (unit: sector) */
u_long *Ring_Buff;

/*
 *  VLC buffer (double buffer).
 *  It contains intermediate data after VLC decoding.
 */
#define VLC_BUFF_SIZE (320/2*256)     // Big enough
u_long *vlcbuf0;
u_long *vlcbuf1;

/*
 *  Image buffer (double buffer)
 *  It contains 16 bit wide image strip after DCT decoding.
 */
#define MAX_SCR_HEIGHT 240
#define SLICE_IMG_SIZE (VRAMPIX(16, TRUE) * MAX_SCR_HEIGHT)
u_short *imgbuf0;
u_short *imgbuf1;

/* Macros for the move data whose width or height are not multiple of 16 */
#define bound(val, n) ((((val) - 1) / (n) + 1) * (n))
#define bound16(val)  (bound((val),16))
char isFirstSlice;    /* Flag to tell the first slice of the image */
char movie512 = 1;
char movie_key_press = 1;

/*
 * Flag for rewinding.
 * It comes to be 1 after playing the end frame
 * specifed by user
 */
char Rewind_Switch;
MovieInfo *movie;

static int BPress;

/*
 *  Proto type declarations for static funcions
 */
static void anim(void);
static void strSetDefDecEnv(DECENV *dec, int x0, int y0, int x1, int y1);
static void strInit(CdlLOC *loc, void (*callback)());
static void strCallback();
static void strNextVlc(DECENV *dec);
static void strSync(DECENV *dec, int mode);
static u_long *strNext(DECENV *dec);
static void strKickCD(CdlLOC *loc);

/*************************************************************************
 * void PlayMovie(char movie_num)
 *************************************************************************/
int PlayMovie(short movie_num)
{

#ifndef NO_MOVIES

	if(!GameOptions[GAME_OPTNS_VIEW_MOVIES] || movie_num == CHAMP_MOV)
	{
	ResetSpinDown();
	StopSequence();
	CloseMusVab();

	movie = movieInfo + movie_num;	// Global so we can do a frame in a loop

	Ring_Buff = (u_long *)AllocMem(RING_SIZE * SECTOR_SIZE * sizeof(u_long));
	vlcbuf0 = (u_long *)AllocMem((movie->width / 10 + 40) * 256 * sizeof(u_long));
	vlcbuf1 = (u_long *)AllocMem((movie->width / 10 + 40) * 256 * sizeof(u_long));
	imgbuf0 = (u_short *)AllocMem(VRAMPIX(16, movie->is24bit) * movie->width * sizeof(u_short));
	imgbuf1 = (u_short *)AllocMem(VRAMPIX(16, movie->is24bit) * movie->width * sizeof(u_short));

   ResetCallback();
   CdInit();			// Without it, sound don't play

	if (movie_num == KONAMI_MOV)
		movie512 = 0;
	else
		movie512 = 1;

	if (movie_num == CHAMP_MOV)
		movie_key_press = 0;
	else
		movie_key_press = 1;

	anim();     /* Function to play movie */

	FreeMem(imgbuf1);
	FreeMem(imgbuf0);
	FreeMem(vlcbuf1);
	FreeMem(vlcbuf0);
	FreeMem(Ring_Buff);

	}
	else
#endif
	{
		BPress = GetPadButton(0xFFFF);
		if(BPress) PlayEffect(PRESS_SOUND);
	}

	MovieClear = 1;
	return(BPress);
}


/*************************************************************************
 * Function to play movie
 *************************************************************************/
static void anim(void)
{
	char s[50];
	CdlFILE file;

	/* Set the first slice flag */
	isFirstSlice = 1;

	sprintf(s, "\\MOV\\%s.STR;1", movie->fileName);
	/* Search for the file */
	if (CdSearchFile(&file, s) == 0)
	{
		DebugPrintf(0, "MOVIE FILE: %s NOT FOUND\n", s);
//		StopCallback();
		return;
	}

	/* Set coordinate value on VRAM */
	strSetDefDecEnv(&dec, VRAMPIX(movie->posX, movie->is24bit), movie->posY,
		             VRAMPIX(movie->posX, movie->is24bit), movie->posY+scrHeight);

	/* Initiailize streaming and play data */
	strInit(&file.pos, strCallback);

	/* Get the first frame and VLC decode */
	strNextVlc(&dec);

	Rewind_Switch = 0;

	SetDispMask(0);		/* Disable display */
	/* Clear display */
	// MRH - FIXED BUG (Clear was always from 0, 0)
//    setRECT(&clearRect, 0, 0, VRAMPIX(scrWidth, movie->is24bit), scrHeight*2);
	setRECT(&clearRect, VRAMPIX(movie->posX, movie->is24bit), movie->posY, VRAMPIX(movie->width, movie->is24bit), movie->height);
	ClearImage(&clearRect, 0, 0, 0);     /* Clear with black */

	while (1) {
		ResetSpinDown();

		/* Start sending vlc decoded data to MDEC */
		DecDCTin(dec.vlcbuf[dec.vlcid], DCT_MODE(movie->is24bit));

		/* Prepare to receive data */
		/* Following process is going to be done by the callback function */
		DecDCTout(dec.imgbuf[dec.imgid], dec.slice.w * bound16(dec.slice.h) / 2);

		/* Get next frame and VLC decode */
		strNextVlc(&dec);

		/* Wait for MDEC to decode DCT data */
		strSync(&dec, 0);

		/* Wait for v-BLNK */
		VSync(0);

      /*
       * Swap buffer.
       * Displayed buffer is a different buffer (the other buffer)
       * from the buffer which is used in decoding.
       */
		id = dec.rectid ^ 1;
		if (movie512)
			SetDefDispEnv(&gDspBuf[gCurrDB].disp, dec.rect[id].x - VRAMPIX(movie->posX, movie->is24bit),
							  dec.rect[id].y - movie->posY, VRAMPIX(scrWidth, movie->is24bit), scrHeight);
		else
			SetDefDispEnv(&gDspBuf[gCurrDB].disp, dec.rect[id].x - VRAMPIX(movie->posX, movie->is24bit),
							  dec.rect[id].y - movie->posY, VRAMPIX(320, movie->is24bit), scrHeight);

		if (movie->is24bit)
		{
			gDspBuf[gCurrDB].disp.isrgb24 = movie->is24bit;
			gDspBuf[gCurrDB].disp.disp.w = gDspBuf[gCurrDB].disp.disp.w * 2/3;
		}
		PutDispEnv(&gDspBuf[gCurrDB].disp);
		/* PutDrawEnv(&draw); */
		SetDispMask(1);     /* Enable display */

		if (movie_key_press && (BPress = GetPadButton(0xFFFF)))
		{
			Rewind_Switch = 1;
			MovieClear = 1;
		}

		// Clear Screen so pallet flash does not occur
		if(Rewind_Switch == 1)
		{
			if (MovieClear)
			{
				ClearMovie();
			}
			break;
		}
    }

	/* Clean up */
	DecDCToutCallback(0);
	StUnSetRing();
	CdControlB(CdlPause,0,0);
}


/*************************************************************************
 * Initialization of decoding environment
 *  x0,y0 Destination position for decoded image (buffer 0)
 *  x1,y1 Destination position for decoded image (buffer 1)
 *************************************************************************/
static void strSetDefDecEnv(DECENV  *dec, int x0, int y0, int x1, int y1)
{
	dec->vlcbuf[0] = vlcbuf0;
	dec->vlcbuf[1] = vlcbuf1;
	dec->vlcid     = gCurrDB ^ 1;
//	dec->vlcid     = 0;

	dec->imgbuf[0] = imgbuf0;
	dec->imgbuf[1] = imgbuf1;
	dec->imgid     = gCurrDB ^ 1;
//	dec->imgid     = 0;
//	dec->rectid    = gCurrDB ^ 1;
	dec->rectid    = gCurrDB;
//	dec->rectid    = 1;
//	dec->rectid    = 0;
	dec->isdone = 0;

  /* Width and height of rect[] are set to the value in STR data */
	dec->rect[0].x = x0;
	dec->rect[0].y = y0;
	dec->rect[1].x = x1;
	dec->rect[1].y = y1;
	dec->slice.w = VRAMPIX(16, movie->is24bit);
	dec->is24bit = movie->is24bit;

	if(dec->rectid == 0)
	{
		dec->slice.x = x0;
		dec->slice.y = y0;
	}
	else
	{
		dec->slice.x = x1;
		dec->slice.y = y1;
	}
}

/*************************************************************************
 * Initialization of streaming
 *************************************************************************/
static void strInit(CdlLOC  *loc, void (*callback)())
{
	/* Reset MDEC */
    DecDCTReset(0);

	/* Set callback function for finishing DecDCTout */
    DecDCToutCallback(callback);

	/* Set up ring buffer */
    StSetRing(Ring_Buff, RING_SIZE);

	/* Set up process of streaming */
	/* Set end frame to infinity */
    StSetStream(movie->is24bit, 1, 0xffffffff, 0, 0);

	/* Start streaming */
    strKickCD(loc);
}


/*************************************************************************
 *  Callback function
 *  which is called when DecDCTout() is finished
 *************************************************************************/
static void strCallback()
{
    int mod;

    if (dec.is24bit) {
        extern StCdIntrFlag;
        if(StCdIntrFlag) {
            StCdInterrupt();
            StCdIntrFlag = 0;
        }
    }

	/* Transfer data to VRAM */
    LoadImage(&dec.slice, (u_long *)dec.imgbuf[dec.imgid]);

	/* Switch buffers */
    dec.imgid ^= 1;

	/* Go to next image strip */
    if (isFirstSlice && (mod = dec.rect[dec.rectid].w % dec.slice.w)) {
        dec.slice.x += mod;
        isFirstSlice = 0;
    } else {
        dec.slice.x += dec.slice.w;
    }

	/* Check if there are image strips to be processed or not */
    if (dec.slice.x < dec.rect[dec.rectid].x + dec.rect[dec.rectid].w) {
	     /* Start decoding for the next image strip */
        DecDCTout(dec.imgbuf[dec.imgid], dec.slice.w * bound16(dec.slice.h)/2);
    }
	/* The end of image */
    else {
		/* Tell end of processing */
      dec.isdone = 1;
      isFirstSlice = 1;

		/* Update destination position for the next image */
	     dec.rectid ^= 1;
        dec.slice.x = dec.rect[dec.rectid].x;
        dec.slice.y = dec.rect[dec.rectid].y;
    }
}


/*************************************************************************
 * Get next BS data and decode it
 *************************************************************************/
static void strNextVlc(DECENV  *dec)
{
    int cnt = WAIT_TIME;
    u_long  *next;

	/* Get one image */
    while ((next = strNext(dec)) == 0) {
        if (--cnt == 0)
            return;
    }

	/* Switch buffers */
    dec->vlcid ^= 1;

	/* VLC decoding */
    DecDCTvlc(next, dec->vlcbuf[dec->vlcid]);

	/* Free buffer area for one frame */
    StFreeRing(next);
}

/*************************************************************************
 *  Get one frame from ring buffer
 *  returned value:
 *      no error: top address of frame data
 *      error: NULL
 *************************************************************************/
static u_long *strNext(DECENV *dec)
{
    static int  strWidth  = 0;  /* Width of previous image data */
    static int  strHeight = 0;  /* Height of previous image data */
    u_long      *addr;
    StHEADER    *sector;
    int     cnt = WAIT_TIME;

	/* Get one frame from the ring buffer */
    while(StGetNext((u_long **) &addr, (u_long **) &sector)) {
        if (--cnt == 0)
            return(0);
    }

    /*
     * Set rewind switch to 1 if current frame number is the same (or greater)
     * as the number which is specified in MovieInfo structure.
     */
    if(sector->frameCount >= movie->endFrame) {
        Rewind_Switch = 1;
    }

    /*
     * Clear display if the size of current image is different
     * from the size of previous image
     */
    if(strWidth != sector->width || strHeight != sector->height) {
// MRH - No need to clear screen, this should speed stuff up?
//        if (movie->is24bit) {
//            ClearImage(&clearRect, 0, 0, 0);     /* Clear with black (24 bit mode) */
//        } else {
//            ClearImage(&clearRect, 64, 64, 64);  /* Clear with gray (16 bit mode) */
//        }

        strWidth  = sector->width;
        strHeight = sector->height;
    }

	/* Update decoding environment according to header of STR data */
    dec->rect[0].w = dec->rect[1].w = VRAMPIX(strWidth, movie->is24bit);
    dec->rect[0].h = dec->rect[1].h = strHeight;
    dec->slice.h   = strHeight;

    return(addr);
}


/*************************************************************************
 *  Wait for the finish of decoding for one frame
 *  Check time out
 *************************************************************************/
static void strSync(DECENV *dec, int mode /* VOID */)
{
    volatile u_long cnt = WAIT_TIME;

	/* Wait till dec-> isdone is set to 0 */
    while (dec->isdone == 0) {
        if (--cnt == 0) {
			/* Timeout force to terminate processing */
            DebugPrintf(0,"time out in decoding !\n");
            dec->isdone = 1;
            dec->rectid ^= 1;
            dec->slice.x = dec->rect[dec->rectid].x;
            dec->slice.y = dec->rect[dec->rectid].y;
        }
    }
    dec->isdone = 0;
}

/*************************************************************************
 *  Start streaming from specified address of CD-ROM
 *************************************************************************/
static void strKickCD(CdlLOC *loc)
{
 loop:
  /* Seek to the specified address */
    while (CdControl(CdlSetloc, (u_char *)loc, 0) == 0);

	/* Start streaming */
  if(CdRead2(CdlModeStream|CdlModeSpeed|CdlModeRT) == 0)
    goto loop;
}
#if 0
#define XXX 40
/*************************************************************************
 * void PlayMovie(char movie_num)
 *************************************************************************/
void MovieSetup(char movie_num)
{
	char s[40];
 	CdlFILE file;
	int ring;

	movie = movieInfo + movie_num;

	Ring_Buff = (u_long *)AllocMem(RING_SIZE * SECTOR_SIZE * sizeof(u_long));
	vlcbuf0 = (u_long *)AllocMem(((movie->width / 10) + XXX) * 256 * sizeof(u_long));
	vlcbuf1 = (u_long *)AllocMem(((movie->width / 10) + XXX) * 256 * sizeof(u_long));
	imgbuf0 = (u_short *)AllocMem(VRAMPIX(16, movie->is24bit) * movie->width * sizeof(u_short));
	imgbuf1 = (u_short *)AllocMem(VRAMPIX(16, movie->is24bit) * movie->width * sizeof(u_short));

	DebugPrintf(0,"RING: %d  VLC: %d  IMG: %d\n", RING_SIZE * SECTOR_SIZE * sizeof(u_long), ((movie->width / 10) + XXX) * 256 * sizeof(u_long), VRAMPIX(16, TRUE) * movie->width * sizeof(u_short));

   ResetCallback();
   CdInit();			// Without it, sound don't play

	/* Set the first slice flag */
   isFirstSlice = 1;

	/* Search for the file */
	sprintf(s, "\\MOV\\%s.STR;1", movie->fileName);
	if (CdSearchFile(&file, s) == 0) {
		DebugPrintf(0, "MOVIE FILE: %s NOT FOUND\n", s);
      StopCallback();
		// FREE UP MOVIE...
   	DecDCToutCallback(0);
	   StUnSetRing();
	   CdControlB(CdlPause,0,0);

		FreeMem(Ring_Buff);
		FreeMem(vlcbuf0);
		FreeMem(vlcbuf1);
		FreeMem(imgbuf0);
		FreeMem(imgbuf1);
		return;
	}

//	SetDispMask(0);		/* Disable display */
	/* Set coordinate value on VRAM */
	strSetDefDecEnv(&dec, VRAMPIX(movie->posX, movie->is24bit), movie->posY,
						VRAMPIX(movie->posX, movie->is24bit), movie->posY+scrHeight);

	/* Initiailize streaming and play data */
	strInit(&file.pos, strCallback);

	/* Get the first frame and VLC decode */
	strNextVlc(&dec);

    Rewind_Switch = 0;
	DebugPrintf(0,"1> %d - %d\n", gCurrDB, dec.rectid);

	/* Clear display */
// MRH - FIXED BUG (Clear was always from 0, 0)
//	setRECT(&clearRect, VRAMPIX(movie->posX, movie->is24bit), movie->posY, VRAMPIX(movie->width, movie->is24bit), movie->height);
//	if (movie->is24bit) {
//		ClearImage(&clearRect, 0, 0, 0);     /* Clear with black (24 bit mode) */
//	} else {
//		ClearImage(&clearRect, 64, 64, 64); 	/* Clear with gray (16 bit mode) */
//	}
}


/*************************************************************************
 * MOVIE FRAME - Gets a frame per call (call in a frame loop)
 *************************************************************************/
char MovieFrame(void)
{
	ResetSpinDown();

	dec.rectid = gCurrDB ^ 1;
	DebugPrintf(0,"2> %d - %d\n", gCurrDB, dec.rectid);

	/* Start sending vlc decoded data to MDEC */
	DecDCTin(dec.vlcbuf[dec.vlcid], DCT_MODE(movie->is24bit));

	/* Prepare to receive data */
	/* Following process is going to be done by the callback function */
	DecDCTout(dec.imgbuf[dec.imgid], dec.slice.w * bound16(dec.slice.h) / 2);

	/* Get next frame and VLC decode */
	strNextVlc(&dec);

	/* Wait for MDEC to decode DCT data */
	strSync(&dec, 0);

	/* Wait for v-BLNK */
	VSync(0);

  /*
   * Swap buffer.
   * Displayed buffer is a different buffer (the other buffer)
   * from the buffer which is used in decoding.
   */
//	id = dec.rectid ^ 1;
//	SetDefDispEnv(&gDspBuf[gCurrDB].disp, dec.rect[id].x - VRAMPIX(movie->posX, movie->is24bit),
	SetDefDispEnv(&gDspBuf[gCurrDB].disp, dec.rect[gCurrDB].x - VRAMPIX(movie->posX, movie->is24bit),
				dec.rect[gCurrDB].y - movie->posY,
				VRAMPIX(scrWidth, movie->is24bit), scrHeight);
	/* SetDefDrawEnv(&draw, dec.rect[id].x, dec.rect[id].y, scrWidth*PPW, scrHeight); */

	if (movie->is24bit) {
		gDspBuf[gCurrDB].disp.isrgb24 = movie->is24bit;
		gDspBuf[gCurrDB].disp.disp.w = gDspBuf[gCurrDB].disp.disp.w * 2/3;
	}
	PutDispEnv(&gDspBuf[gCurrDB].disp);
	/* PutDrawEnv(&draw); */
//	SetDispMask(1);     /* Enable display */

//	if (IsAnyButton())
//		Rewind_Switch = 1;

	if(Rewind_Switch == 1)
	{
		// FREE UP MOVIE...
   	DecDCToutCallback(0);
	   StUnSetRing();
   	CdControlB(CdlPause,0,0);

		FreeMem(Ring_Buff);
		FreeMem(vlcbuf0);
		FreeMem(vlcbuf1);
		FreeMem(imgbuf0);
		FreeMem(imgbuf1);
//		gCurrDB ^= 1;
		return(1);
   }
	return(0);
}
#endif

extern struct teamRAMstats StatsData[];
/*************************************************************************
 * PLAY POST GAME MOVIE - Figure out who wins and play their movie.
 *************************************************************************/
void PlayPostGameMovie(void)
{
	register int  i,
					  j;
	int 			  mul,
					  winner,
		  			  possible_movies_high,
		  			  possible_movies;
	signed short  score;
	short			  movie_high[20],
					  movie[20];
	unsigned long *ptr;


	score = GameTeamData[0].TotalScore - GameTeamData[1].TotalScore;
	ClearMovie();
	initVRamHandling();

// BOTH TEAMS CPU RETURN
	if (GameTeamData[0].ControllerNum[0] == -1 && GameTeamData[1].ControllerNum[0] == -1)
		return;

// PLAY WIN SUPER BOWL MOVIE
	if (GameType == CHAMPIONSHIP_GAME)
		if ((GameTeamData[0].TeamNumber == HumanTeamNum && GameTeamData[0].TotalScore > GameTeamData[1].TotalScore) ||
			 (GameTeamData[1].TeamNumber == HumanTeamNum && GameTeamData[1].TotalScore > GameTeamData[0].TotalScore))
		{
			PlayMovie(CHAMP_MOV);
//			ShowLoadingScreen();
			return;
		}

// If MOVIE VIEW Option off don't do any of this...
	if(GameOptions[GAME_OPTNS_VIEW_MOVIES])
		return;

// DO HUMAN TEAMS
	for (i = 0; i < 2; i++)
	{
		if (GameTeamData[i].ControllerNum[0] == -1)
			continue;	// Don't do CPU teams

// LOAD UP BACKGROUND FOR POST GAME MOVIES
		ptr = (unsigned long *)LoadCDFile("/tmp/scrbrd.tim");
		initTexture(0, ptr);
		FreeFileData((char *)ptr);
		MovieClear = 0;

		multifile = (unsigned long *)LoadCDFile("/shells/names/nickname.mfl");
		LoadTemplate("nickname.llx");
		FreeFileData((char *)multifile);
		MovePolyToXY(&template[0].poly[0],-92, -101);
		MovePolyToXY(&template[0].poly[1], 36, -101);

		LoadNickname(GameTeamData[i].TeamNumber, 0, 0);
		for (j = 0; j < 4; j++)
		{
		  	InitFrame();
			UpdateBackground();
			DisplayShellPolys();
			ShowFrame();
		}
		FreeTemplate(0);

// PLAYOFFS & SUPERBOWL MOVIES
		if (WeekNum > 17 && MyGameType == SEASON_GAME)
		{
			if ((score > 0 && GameTeamData[0].ControllerNum[0] == -1) || (score < 0 && GameTeamData[1].ControllerNum[0] == -1))
				if (WeekNum == 21) // Super Bowl LOSS
					PlayMovie(LOSE_SUPER_MOV);
				else
					PlayMovie(LOSE_PLAYOFFS_MOV);
			else
				switch(WeekNum - 18)
				{
					case 0: PlayMovie(WIN_WILD_0_MOV + GetRandom(3));
							  break;
					case 1: PlayMovie(WIN_2ND_MOV);
							  break;
					case 2: PlayMovie(WIN_CONF_MOV);
							  break;
					case 3: PlayMovie(CHAMP_MOV);
							  break;
				}
			continue;
		}

// STANDARD SEASON & EXHIBITION SPECIFIC MOVIES
		if (randper < 80)	// 80% chance to play a specific movie
		{
			possible_movies_high = 0;
			possible_movies = 0;

			mul = GameOptions[GAME_OPTNS_QTR_LENGTH] * 2;
			if (!mul)
				mul = 1;

			if ((score > 0 && i == 0) || (score < 0 && i == 1))
			{
			// GOOD POINTS
				if (GameTeamData[i].TotalScore > 25 * mul)
					movie_high[possible_movies_high++] = GOOD_POINTS_MOV;
				else
				if (GameTeamData[i].TotalScore > 13 * mul)
					movie[possible_movies++] = GOOD_POINTS_MOV;
				else

			// GOOD FIRST DOWNS
				if (StatsData[i].allplyrstats[GS_1STDOWNS] > 9 * mul)
					movie_high[possible_movies_high++] = GOOD_FIRST_DOWNS_MOV;
				else
				if (StatsData[i].allplyrstats[GS_1STDOWNS] > 6 * mul)
					movie[possible_movies++] = GOOD_FIRST_DOWNS_MOV;
				else

			// GOOD 4TH DOWNS
				if (StatsData[i].allplyrstats[GS_4THDOWNS_ATT])
					j = (StatsData[i].allplyrstats[GS_4THDOWNS_MADE] * 100) / StatsData[i].allplyrstats[GS_4THDOWNS_ATT];
				else
					j = 0;

				if (StatsData[i].allplyrstats[GS_4THDOWNS_ATT] > 2 * mul && j > 99)
					movie_high[possible_movies_high++] = GOOD_FOURTH_DOWNS_MOV;
				else
				if (StatsData[i].allplyrstats[GS_4THDOWNS_ATT] > (3 * mul) / 2 && j > 74)
					movie[possible_movies++] = GOOD_FOURTH_DOWNS_MOV;
				else

			// GOOD TOTAL YARDS
				j = StatsData[i].allplyrstats[GS_PASS_YARDS] + StatsData[i].allplyrstats[GS_RUSH_YARDS];
				if (j > 200 * mul)
					movie_high[possible_movies_high++] = GOOD_TOTAL_YARDS_MOV;
				else
				if (j > 150 * mul)
					movie[possible_movies++] = GOOD_TOTAL_YARDS_MOV;
				else

			// GOOD PASSING YARDS
				if (StatsData[i].allplyrstats[GS_PASS_YARDS] > 125 * mul)
					movie_high[possible_movies_high++] = GOOD_PASS_YARDS_MOV;
				else
				if (StatsData[i].allplyrstats[GS_PASS_YARDS] > 80 * mul)
					movie[possible_movies++] = GOOD_PASS_YARDS_MOV;
				else

			// GOOD PASS COMPLETE
				if (StatsData[i].allplyrstats[GS_PASS_ATT])
					j = (StatsData[i].allplyrstats[GS_PASS_COMP] * 100) / StatsData[i].allplyrstats[GS_PASS_ATT];
				else
					j = 0;

				if (StatsData[i].allplyrstats[GS_PASS_ATT] > 5 * mul && j > 80)
					movie_high[possible_movies_high++] = GOOD_PASS_PERCENT_MOV;
				else
				if (StatsData[i].allplyrstats[GS_PASS_ATT] > 5 * mul && j > 65)
					movie[possible_movies++] = GOOD_PASS_PERCENT_MOV;
				else

			// GOOD RUSHING YARDS
				if (StatsData[i].allplyrstats[GS_RUSH_YARDS] > 50 * mul)
					movie_high[possible_movies_high++] = GOOD_RUSH_YARDS_MOV;
				else
				if (StatsData[i].allplyrstats[GS_RUSH_YARDS] > 25 * mul)
					movie[possible_movies++] = GOOD_RUSH_YARDS_MOV;
				else

			// GOOD PENALTYS - Only in normal mode, and never for 2 or 5 minute quarter
				if (StatsData[i].allplyrstats[GS_PENALTY_NUM] == 0 && GameOptions[GAME_OPTNS_QTR_LENGTH] > 1)
					movie[possible_movies++] = GOOD_PENALTY_MOV;
				else

			// GOOD SACKS
				if (StatsData[i].allplyrstats[GS_SACK_NUM] > 4 * mul)
					movie_high[possible_movies_high++] = GOOD_SACKS_MOV;
				else
				if (StatsData[i].allplyrstats[GS_SACK_NUM] > 2 * mul)
					movie[possible_movies++] = GOOD_SACKS_MOV;
				else

			// GOOD TURNOVER - Only do in normal mode, and never for 2 or 5 minute quarter
				j = StatsData[i].allplyrstats[GS_PASS_INT] + StatsData[i].allplyrstats[GS_FUM_LOST];
				if (j > 2 * mul && GameOptions[GAME_OPTNS_QTR_LENGTH] > 1)
					movie_high[possible_movies_high++] = GOOD_TURNOVER_MOV;
				else
				if (j > mul && GameOptions[GAME_OPTNS_QTR_LENGTH] > 1)
					movie[possible_movies++] = GOOD_TURNOVER_MOV;
			}
			else
			{
			// BAD POINTS
				if (GameTeamData[i].TotalScore < mul / 2)
					movie_high[possible_movies_high++] = BAD_POINTS_MOV;
				else
				if (GameTeamData[i].TotalScore < (7 * mul) / 2)
					movie[possible_movies++] = BAD_POINTS_MOV;

			// BAD FIRST DOWNS
				if (StatsData[i].allplyrstats[GS_1STDOWNS] < mul)
					movie_high[possible_movies_high++] = BAD_FIRST_DOWNS_MOV;
				else
				if (StatsData[i].allplyrstats[GS_1STDOWNS] < 2 * mul)
					movie[possible_movies++] = BAD_FIRST_DOWNS_MOV;

			// BAD 4TH DOWNS
				if (StatsData[i].allplyrstats[GS_4THDOWNS_ATT])
					j = (StatsData[i].allplyrstats[GS_4THDOWNS_MADE] * 100) / StatsData[i].allplyrstats[GS_4THDOWNS_ATT];
				else
					j = 0;

				if (StatsData[i].allplyrstats[GS_4THDOWNS_ATT] > (7 * mul) / 2 && j < 10)
					movie_high[possible_movies_high++] = BAD_FOURTH_DOWNS_MOV;
				else
				if (StatsData[i].allplyrstats[GS_4THDOWNS_ATT] > (5 * mul) / 2 && j < 34)
					movie[possible_movies++] = BAD_FOURTH_DOWNS_MOV;

			// BAD TOTAL YARDS
				j = StatsData[i].allplyrstats[GS_PASS_YARDS] + StatsData[i].allplyrstats[GS_RUSH_YARDS];
				if (j < 7 * mul)
					movie_high[possible_movies_high++] = BAD_TOTAL_YARDS_MOV;
				else
				if (j < 15 * mul)
					movie[possible_movies++] = BAD_TOTAL_YARDS_MOV;

			// BAD PASSING YARDS
				if (StatsData[i].allplyrstats[GS_PASS_YARDS] < 5 * mul)
					movie_high[possible_movies_high++] = BAD_PASS_YARDS_MOV;
				else
				if (StatsData[i].allplyrstats[GS_PASS_YARDS] < 20 * mul)
					movie[possible_movies++] = BAD_PASS_YARDS_MOV;

			// BAD PASS COMPLETE
				if (StatsData[i].allplyrstats[GS_PASS_ATT])
					j = (StatsData[i].allplyrstats[GS_PASS_COMP] * 100) / StatsData[i].allplyrstats[GS_PASS_ATT];
				else
					j = 0;

				if (StatsData[i].allplyrstats[GS_PASS_ATT] > 5 * mul && j < 10)
					movie_high[possible_movies_high++] = BAD_PASS_PERCENT_MOV;
				else
				if (StatsData[i].allplyrstats[GS_PASS_ATT] > 5 * mul && j < 20)
					movie[possible_movies++] = BAD_PASS_PERCENT_MOV;

			// BAD RUSHING YARDS
				if (StatsData[i].allplyrstats[GS_RUSH_YARDS] < 5 * mul && StatsData[i].allplyrstats[GS_RUSH_ATT] < 15)
					movie_high[possible_movies_high++] = BAD_RUSH_YARDS_MOV;
				else
				if (StatsData[i].allplyrstats[GS_RUSH_YARDS] < 10 * mul && StatsData[i].allplyrstats[GS_RUSH_ATT] < 15)
					movie[possible_movies++] = BAD_RUSH_YARDS_MOV;

			// BAD PENALTYS
				if (StatsData[i].allplyrstats[GS_PENALTY_NUM] > 7 * mul)
					movie_high[possible_movies_high++] = BAD_PENALTY_MOV;
				else
				if (StatsData[i].allplyrstats[GS_PENALTY_NUM] > 4 * mul)
					movie[possible_movies++] = BAD_PENALTY_MOV;

			// BAD SACKS - Only do in normal mode, and never for 2 minute quarter
				if (StatsData[i].allplyrstats[GS_SACK_NUM] == 0 && GameOptions[GAME_OPTNS_QTR_LENGTH])
					movie[possible_movies++] = BAD_SACKS_MOV;

			// BAD TURNOVER
				j = StatsData[i].allplyrstats[GS_PASS_INT] + StatsData[i].allplyrstats[GS_FUM_LOST];
				if (j == 0 && GameOptions[GAME_OPTNS_QTR_LENGTH])
					movie[possible_movies++] = BAD_TURNOVER_MOV;
			}


			if (possible_movies_high)	// If we get a real strong hit...
			{
				PlayMovie(movie_high[GetRandom(possible_movies_high)]);
//				ShowLoadingScreen();
				return;
			}
			if (possible_movies)	// If we get a real strong hit...
			{
				PlayMovie(movie[GetRandom(possible_movies)]);
//				ShowLoadingScreen();
				return;
			}

		}

		// GENERAL WIN/LOSS 20% or if NO relevant specific movies
		if ((score > 0 && score < 7 && i == 0)	||	(score < 0 && score > -7 && i == 1))
			PlayMovie(POST_GAME_WIN_CLOSE_0_MOV + GetRandom(3));
		else
		if ((score > 6 && score < 14 && i == 0) || (score < -6 && score > -14 && i == 1))
			PlayMovie(POST_GAME_WIN_GENERIC_0_MOV);
		else
		if ((score > 13 && i == 0) || (score < -13 && i == 1))
			PlayMovie(POST_GAME_WIN_BLOWOUT_0_MOV + GetRandom(2));
		else
		if ((score > 0 && score < 4 && i == 1)	||	(score < 0 && score > -4 && i == 0))
			PlayMovie(POST_GAME_LOSE_CLOSE_0_MOV + GetRandom(2));
		else
		if ((score > 3 && score < 15 && i == 1) || (score < -3 && score > -15 && i == 0))
			PlayMovie(POST_GAME_LOSE_GENERIC_0_MOV + GetRandom(4));
		else
		if ((score > 14 && i == 1) || (score < -14 && i == 0))
			PlayMovie(POST_GAME_LOSE_BLOWOUT_0_MOV);
		else
		if (score == 0)
			PlayMovie(POST_GAME_WIN_CLOSE_0_MOV + GetRandom(2));
	}
	MovieClear = 1;
//	ShowLoadingScreen();
}


/*************************************************************************
 * PLAY TEAM BIO MOVIE - Figure out who wins and play their movie.
 *************************************************************************/
int PlayTeamBioMovie(char team)
{
	int i,
		 song;

//	FadeMusic(1, 120);
	song = CurSong;
	CloseVabBody(1);		// Close SHELL so no SRAM fragmentation.
	StopSequence();		// Kills music
	CloseMusVab();

	i = PlayMovie(TEAM_BIO_0_MOV + team);

   PlaySong(song);

//	ShowLoadingScreen();

	if(i==JPADtriangle)
		return(-1);
	else
		return(1);
}


/*************************************************************************
 * PLAY HALFTIME MOVIE - Figure out who wins and play their movie.
 *************************************************************************/
void PlayHalftimeMovie(void)
{
	register signed short score;
//	RECT rect;

// CPU VS CPU - Never play a movie
	if (GameTeamData[0].ControllerNum[0] == -1 && GameTeamData[1].ControllerNum[0] == -1)
		return;

	StopCrowd();	// Stop crowd
	StopPhrase();	// Stop announcer

// HUMAN VS CPU - 50/50 Play HINT/SPEECH

	if (GameTeamData[0].ControllerNum[0] != -1 && GameTeamData[1].ControllerNum[0] != -1)
	{
		PlayMovie(HINT_0_MOV + GetRandom(32));		// Always a hint if 2 player human game
	}
	else
	if (randper < 50 || WeekNum > 17)
	{
// DO SPEECH
		score = GameTeamData[0].TotalScore - GameTeamData[1].TotalScore;
		if (score > -4 && score < 4)	// Close Game
			PlayMovie(SPEECH_CLOSE_0_MOV + GetRandom(2));
		else
		if ((score > -8 && score < -3 && GameTeamData[0].ControllerNum[0] == -1) ||
			(score < 8 && score > 3 && GameTeamData[1].ControllerNum[0] == -1))
			PlayMovie(SPEECH_CLOSE_2_MOV + GetRandom(2));
		else
		if ((score < 8 && score > 3 && GameTeamData[0].ControllerNum[0] == -1) ||
			(score > -8 && score < -3 && GameTeamData[1].ControllerNum[0] == -1))
			PlayMovie(SPEECH_CLOSE_4_MOV);
		else
		if ((score > -14 && score < -7 && GameTeamData[0].ControllerNum[0] == -1) ||
			(score < 14 && score > 7 && GameTeamData[1].ControllerNum[0] == -1))
			PlayMovie(SPEECH_WIN_MID_MOV);
		else
		if ((score < 14 && score > 7 && GameTeamData[0].ControllerNum[0] == -1) ||
			(score > -14 && score < -7 && GameTeamData[1].ControllerNum[0] == -1))
			PlayMovie(SPEECH_LOSE_MID_MOV);
		else
		if ((score < -13 && GameTeamData[0].ControllerNum[0] == -1) ||
			(score > 13 && GameTeamData[1].ControllerNum[0] == -1))
			PlayMovie(SPEECH_WIN_HIGH_0_MOV + GetRandom(2));
		else
		if ((score > 13 && GameTeamData[0].ControllerNum[0] == -1) ||
			(score < -13 && GameTeamData[1].ControllerNum[0] == -1))
			PlayMovie(SPEECH_LOSE_HIGH_0_MOV + GetRandom(2));
	}
	else
	{
		// HUMAN VS HUMAN - Always show a hint
		PlayMovie(HINT_0_MOV + GetRandom(32));
	}

	InitCrowd();
	initVRamHandling();
//	ShowLoadingScreen();

//	PlaySong(7);
}


/*************************************************************************
 * CLEAR MOVIE - Clears a space the size of the last movie played. taking
 *		into account 16 or 24 bit.
 *************************************************************************/


//void ClearMovie(void)
//{
//	VSync(0);
//
//	SetDispMask(0);     /* disable display */
//
//	ClearImage(&clearRect, 0, 0, 0);    // Clear with black
//	clearRect.y += 240;
//	ClearImage(&clearRect, 0, 0, 0);    // Clear with black
//
//}
