/**************************************************************************
	Multi file
	by Christopher Deel 
			
***************************************************************************/


#include <stdio.h>
#include <stdlib.h>

#include "version.h"
#include "prjtypes.h"
#include "cdhandlr.h"
#include "Multifil.h"
#include "debug.h"


//#define DEBUG_THIS 1  	// Debug this module
#define SECTOR_FILES 1

//static MultiFile_t *dir;

/*-------------------------------------------------------------
  GetMultiFileType ~ 
-------------------------------------------------------------*/
int GetMultiFileType(MultiFile_t *base)
{
	char *ptr;
	ULONG val;
	int rval;
	
	val = *(Ulong*)base;
	if (val == 0x544C554D)
		rval = 1;
	else if (val == 0x324c464d)
		rval = 2;
	else
		rval = 0;

#if TESTING
	printf("MultiFileType %d  %lX\n", rval, val);		
#endif

	return rval;
}  /* End CheckMultiFileType() */

/*~~---------------------------------------
    Get Pointer to file in MultiFile
-----------------------------------------*/
char *GetMultiFilePtr(MultiFile_t *base, char *name)
{
	MultiFile_t *dir;
	int i, working = True;
	Ulong ptr = 0;
	char tstr[20];

	if (base == NULL)
	{
		Cerror();		// base is NULL
		return NULL;
	}

	i = 1;
	dir = (MultiFile_t *)base;
	strcpy(tstr, name);
	strupr(tstr);
#if DEBUG_THIS
	DebugPrintf(0,"|%s| ", tstr);
#endif
	while (working)
	{
		if (strncmp(tstr, dir[i].name, 12) == 0)
		{
			working = False;
			ptr = (ULONG)dir[i].offset;
			ptr = ptr + (ULONG)base;
		}
		else if (*(dir[i].name) == 0)
		{
			working = False;
			ptr = NULL;
			DebugPrintf(0, "%x  %s: ", base, tstr);
			Cerror();
		}
		i++;
	}
	return (void *)ptr;
}

/*~~---------------------------------------
    Get number of file in MultiFile
-----------------------------------------*/
int GetNumberInMultiFile(MultiFile_t *base, char *name)
{
	MultiFile_t *dir;
	int i, number, working = True;
	Ulong ptr = 0;
	char tstr[20];

	i = 1;
	number = -1;
	dir = (MultiFile_t *)base;
	strcpy(tstr, name);
	strupr(tstr);
#if DEBUG_THIS
	DebugPrintf(0,"|%s| ", tstr);
#endif
	while (working)
	{
		if (strncmp(tstr, dir[i].name, 12) == 0)
		{
			working = False;
			number = i-1;
		}
		else if (*(dir[i].name) == 0)
		{
			working = False;
			number = 0;
			DebugPrintf(0, "%x  %s: ", base, tstr);
			Cerror();
		}
		i++;
	}
	return number;
}

/*-------------------------------------------------------------
  GetMultiFile2ByNum ~ 
-------------------------------------------------------------*/
char * GetMultiFile2ByNum(MultiFile_t *base, int num)
{
	register ULONG ptr, data, offset;
	
	ptr = (ULONG)base + ((num +1) * 4);
	offset = *(Ulong*)ptr;
	data = (ULONG)base + offset;
	return (void*)data;
}  /* End GetMultiFile2ByNum() */

/*-------------------------------------------------------------
  GetMultiFileByNum ~ Get MultiFile Pointer by file number
-------------------------------------------------------------*/
char * GetMultiFileByNum(MultiFile_t *base, int num)
{
	MultiFile_t *dir;
	register ULONG ptr, offset;
	ULONG header;
	
	header = *(ULONG*)base;
	if (header == MULTIFILE2_HEADER)
		ptr = (ULONG)GetMultiFile2ByNum(base, num);
	else if (header == MULTIFILE1_HEADER)
	{
		dir = (MultiFile_t *)base;
		offset = (ULONG)dir[num+1].offset;
		ptr = offset + (ULONG)base;
		if (offset > (ULONG)dir[0].offset)
			ptr = 0;
	}
	else
		ptr = 0;	// error		

//	#if (PC_DEBUG && 1)
//	 if (!CheckValidAddress(ptr, 4))
//	 {
//	 	DebugPrintf(0, "Invalid MFL addr: base:%X num:%d addr%X\n",
//	 				base, num, ptr);
//	 }
//	#endif		
	return (char*)ptr;
}

#if SECTOR_FILES
/**************** Sector file routines;; ******************/
//#define SFL_DIRECORY_SECTORS 2;

//ULONG gTheSectorFile = 0;  // sector of current sector file
//char *gTheSectorFilePtr = NULL;

/*-------------------------------------------------------------
  LoadSectorFileDirectory ~  Load Sector file directory
  	if name = NULL sector will be used
	if dest = NULL Directory will be allocated
-------------------------------------------------------------*/
SectorFile_t* 
LoadSectorFileDirectory(char *name, ULONG sector, SectorFile_t *dest)  
{
	SectorFile_t *ptr;
	SectorFileHeader_t *header;
	long files, bytes, numSectors;	
	
	
	if (name)
	{
		sector = GetFileSector(name);
	}
	if (dest)
	{
		ptr = dest;
		header = (SectorFileHeader_t*)dest;
	}
	else	
	{
		header = (SectorFileHeader_t*)AllocMem(SIZE_1SECTOR);
	}	
		
	ReadCDSectors(sector, 1, (ULONG*)header);
	files = header->files;
	bytes = (files +1) * 16;
	numSectors = (bytes + (SIZE_1SECTOR -1)) / SIZE_1SECTOR;
	
	if (!dest)
	{
		FreeMem(header);
		ptr = (SectorFile_t*)AllocMem(numSectors * SIZE_1SECTOR); 
	}
		
	ReadCDSectors(sector, numSectors, (ULONG*) ptr);
	header = (SectorFileHeader_t *)ptr;
	header->sector = sector;
	return ptr;
}  /* End LoadSectorFileDirectory() */

/*----------------------------------------------------------------------
  GetSectorFileSector ~ Get sector of file inside sector file
----------------------------------------------------------------------*/
ULONG GetSectorFileSector(SectorFile_t *dir, char *fname)
{
//	MultiFile_t *dir;
	SectorFileHeader_t *header;
	int i, working = True;
	ULONG baseSector, offset, sector, result, size;
	char tstr[20];

//	if (!gTheSectorFile)
//	{
//		Cerror();		// base is NULL
//		return 0;
//	}

	i = 1;
	header = (SectorFileHeader_t*)dir;
//	baseSector = gTheSectorFile & SECTOR_MASK;
	baseSector = header->sector & SECTOR_MASK;
//	dir = (MultiFile_t *)gTheSectorFilePtr;
	strcpy(tstr, fname);
	strupr(tstr);
#if DEBUG_THIS
	DebugPrintf(0,"|%s| ", tstr);
#endif
 
	 DumpMemory(dir, 10, 1);
	while (working)
	{
//		 DebugPrintf(0, "%s  %s\n", tstr, dir[i].name);  // debug
		if (strncmp(tstr, dir[i].name, 12) == 0)
		{
			working = False;
			offset = (ULONG)dir[i].offset;
			size = MGetSectorIDSize(offset);
			sector = baseSector + (offset & SECTOR_MASK);
			sector = CalcSectorID(sector, size);
		}
		else if (*(dir[i].name) == 0)
		{
			working = False;
			sector = 0;
			Dprintf("%x  %s: ", dir, tstr);
			Cerror();
		}
		i++;
	}
	return sector;
}  /* End GetSectorFileSector() */

/*-------------------------------------------------------------
  GetSectorFileByNum ~ 
-------------------------------------------------------------*/
ULONG GetSectorFileByNum(SectorFile_t *dir, int num)
{
	SectorFileHeader_t *header;
	ULONG baseSector, offset, sector, size;

	header = (SectorFileHeader_t*)dir;
	baseSector = header->sector & SECTOR_MASK;
		
	offset = (ULONG)dir[num + 1].offset;
	size = MGetSectorIDSize(offset);
	sector = baseSector + (offset & SECTOR_MASK);
	sector = CalcSectorID(sector, size);
	
	return sector;
}

#endif SECTOR_FILES
