#include <sys/types.h>
#include <sys/file.h>
#include <kernel.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libcd.h>
#include "prjtypes.h"

/*************************************************** extern global vars. */

/* load-address for child process. it's defined in ADDRESS.S ::
	qvZX̃[hAhXBADDRESS.S Œ`Ă */
extern void *LoadAddress;

/**************************************************** overlay files */

static INT CurOverlay = -1;
static INT file_read();
static INT _read1();


typedef struct ovinfo {
  char *fn;			/* bin-file name */
} OVL;

#define MENU_MAX 3

static OVL overlays[MENU_MAX] = {
  { "\\OVERLAYS\\MOVIE.BIN;1"},
  { "\\OVERLAYS\\FBAI.BIN;1"},
  { "\\OVERLAYS\\PLEDIT.BIN;1"},
};

//********************************************************************
// INPUT  : Overlay number (See overlay name defines in overlay.h)
// OUTPUT : 0 = Success
//        :-1 = Error condition
// PURPOSE: Load code overlay into fixed overlay memory buffer
//********************************************************************


INT LoadOverlay(INT ovnum)
{
	if(CurOverlay != ovnum)	// Avoid reloading if overlay is already in the buffer
	{
		file_read(overlays[ovnum].fn,LoadAddress);	/* read child */
	}

	return(0);
}


//********************************************************************
//********************************************************************


static INT file_read(UBYTE *fname,void *address)
{
	CdlFILE	fp;		/* file's location & size */

	INT	mode = CdlModeSpeed;
	INT cnt,i;

	for (i = 0; i < 10; i++)	/* 10 times retry */
	{
		if (CdSearchFile(&fp, fname) == 0)
			continue;

	    /* set target position */

	    CdControl(CdlSetloc, (u_char *)&(fp.pos), 0);
	    cnt = _read1(fp.size,address,mode);

	    /* if success, break */

	    if (cnt == 0)
			return(0);		/* success */
	}
	return(-1);				/* fail */
}


//********************************************************************
//********************************************************************


static INT _read1(LONG byte,void *sectbuf, INT mode)
{
	void *buffer;
	INT nsector,cnt;

	nsector = (byte+2047) / 2048;

	buffer = (char *)AllocMem(nsector * 2048);

//  CdRead(nsector, sectbuf, mode);

	CdRead(nsector, buffer, mode);

	while ((cnt = CdReadSync(1, 0)) > 0 )
	{
		VSync(0);
	}

	memcpy(sectbuf, buffer, byte);

	FreeMem(buffer);

	return(cnt);
}


