/*************************************************************************
     Pictures and Backgrounds for the PSX.

     Written by Mike Heilemann  Started: 8-16-94
     Updated by Christopher Deel

     Copyright 1994 GameTek
     All rights reserved.

*************************************************************************/

#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include "prjhdrs.h"	/* Project header files */
#include "movie.h"

int		numBG = 0;
pict_t	picture[N_BG];


void initTexture(int num, u_long *timP);


/*~~---------------------------------------------------
    Good version of GetTimInfo
------------------------------------------------------*/
ULONG *GetTimInfo2(ULONG *ptr, GsIMAGE *tim)
{
	if (*ptr == TIM_HEADER)
	{
		ptr++;
		GsGetTimInfo(ptr, tim);
	}
	else
	{
		Cerror();
		ptr = NULL;
	}
	return ptr;
}


/*************************************************************************
 * UPDATE BACKGROUND - puts the backgrounds into a list.
 *************************************************************************/
void UpdateBackground(void)
{
	register int i,
					 j;

	for (i = 0; i < numBG; i++)
	{
		for (j = 0; j < picture[i].blocks; j++)
			GsSortPoly(&picture[i].poly[j], gPtrGsOt, (OTSIZE -4) - i); /* Poly is first entry */
	}
}


/*************************************************************************
 * FREE PICT - Frees memory from Picture
 *************************************************************************/
void free_pict(char num)
{
	int i;

	if (picture[num].tpage[0])
	{
		for (i = 0; i < picture[num].blocks; i++)
			free_VRam(picture[num].tpage[i]);	// Mark TPAGES as free
		if (picture[num].CLUT) 		// Mark Clut as Free
			free_VRam(picture[num].CLUT); 		// Mark Clut as Free
		picture[num].tpage[0] = NULL;	/* be sure to set it to NULL */
		if (numBG > 0)
			numBG--;
	}
}


/*************************************************************************
 * LOAD PICT - Cut a TIM up and place it in tpages to be displayed as a
 *		background.
 *************************************************************************/
void load_pict(int bkg, GsIMAGE pic)
{
	int    i, j, num;
	char   *buff,
			 *j1,
			 *buff_ptr,
			 *tmp;


	buff = (char *)malloc(128 * pic.ph);	 /* Get enough mem for 1 TPAGE strip */
	if (buff == NULL)
		Cerror();

	picture[bkg].blocks = pic.pw / 64; // pic.pw is in words (2 bytes)

//	SetDispMask(0);     /* Disable display */

/* Load big blocks (blocks of 128 bytes) */
	for (i = 0; i < picture[bkg].blocks; i++)
	{
		buff_ptr = buff;
		for (j = 0; j < pic.ph; j++)
		{       /* BASE         HEIGHT             WIDTH (in words) */
			j1 = (char *)(pic.pixel + (j * pic.pw / 2) + (i * 32));  /* for speed */
			for (num = 0; num < 128; num++) /* from 0 - 128 bytes... */
			{
				*buff_ptr = *(j1 + num);
				buff_ptr++;
			}
		}                                                  /* BYTES */
		picture[bkg].tpage[i] = alloc_VRam(VRAM_TEXTURE, 0);
		picture[bkg].tpage[i]->tpage = LoadTPage((unsigned long *)buff, 0, 0, picture[bkg].tpage[i]->x, picture[bkg].tpage[i]->y, 256, pic.ph);
	}

#if 0	// Only for 640X480 or 320X200 X 256
/* Load last block (anything not divisible by 128 bytes) */
	num = pic.pw % 64;

	if (num)				/* Do leftover part */
	{
		buff_ptr = buff;
		for (j = 0; j < pic.ph; j++)
		{
			j1 = (char *)(pic.pixel + (j * pic.pw / 2) + (picture[bkg].blocks * 32));  /* for speed */
			for (i = 0; i < num * 2; i++)
			{
				*buff_ptr = *(j1 + i);
				buff_ptr++;
			}
		}
		picture[bkg].tpage[picture[bkg].blocks] = alloc_VRam(VRAM_TEXTURE, 0);
		picture[bkg].tpage[picture[bkg].blocks]->tpage = LoadTPage((unsigned long *)buff, 0, 0, picture[bkg].tpage[picture[bkg].blocks]->x, picture[bkg].tpage[picture[bkg].blocks]->y, num*4, pic.ph);
		picture[bkg].blocks++;
	}
#endif

#if 0	// Only for 640X480
/* Load lower part */
	if (pic.ph > 240)
	{
		for (i = 0; i < picture[bkg].blocks; i++)
		{
			buff_ptr = buff;
			for (j = 240; j < pic.ph; j++)
			{             /* BASE         HEIGHT             WIDTH (in words) */
				j1 = (char *)(pic.pixel + (j * pic.pw / 2) + (i * 32));  /* for speed */
				for (num = 0; num < 128; num++) /* Transfer 128 bytes many times */
				{
					*buff_ptr = *(j1 + num);
					buff_ptr++;
				}
			}
			picture[bkg].tpage[i+picture[bkg].blocks] = alloc_VRam(VRAM_TEXTURE, 0);
			picture[bkg].tpage[i+picture[bkg].blocks]->tpage = LoadTPage((unsigned long *)buff, 0, 0, picture[bkg].tpage[i+picture[bkg].blocks]->x, picture[bkg].tpage[i+picture[bkg].blocks]->y, 256, 240);
		}
		picture[bkg].blocks *= 2;

		num = pic.pw % 64;  /* 16 */

		if (num)				/* Do leftover part */
		{
			buff_ptr = buff;
			for (j = 0; j < num; j++)
			{
				j1 = (char *)(pic.pixel + (j * pic.pw / 2) + (picture[bkg].blocks * 32));  /* for speed */
				for (i = 0; i < num * 2; i++)
				{
					*buff_ptr = *(j1 + i);
					buff_ptr++;
				}
			}
			picture[bkg].tpage[picture[bkg].blocks] = alloc_VRam(VRAM_TEXTURE, 0);
			picture[bkg].tpage[picture[bkg].blocks]->tpage = LoadTPage((unsigned long *)buff, 0, 0, picture[bkg].tpage[picture[bkg].blocks]->x, picture[bkg].tpage[picture[bkg].blocks]->y, num*4, pic.ph);
			picture[bkg].blocks++;
		}
	}
#endif

/* CLUT STUFF */
	if (pic.cw == 16)
	{
		picture[bkg].clutType = VRAM_CLUT16;
		picture[bkg].CLUT = register_CLUT(pic.clut, VRAM_CLUT16);
	}
	else if (pic.cw == 256)
	{
		picture[bkg].clutType = VRAM_CLUT256;
		picture[bkg].CLUT = register_CLUT(pic.clut, VRAM_CLUT256);
	}
	else
	{
		picture[bkg].clutType = VRAM_16BIT;
		picture[num].CLUT = NULL;
	}

	DrawSync(0);	/* Be sure CLUT data is uploaded to VRAM */
	free(buff);
}


/*************************************************************************
 * LoadBackgroundCD - Load background TIM from CD
 *************************************************************************/
void LoadBackgroundCD(int num, char *fname)
{
	char *ptr;

	ptr = LoadCDFile(fname);
	if (ptr)
	{
#if 0		// tom!!
		if (!MovieClear)	// To make black time small as possible after a movie plays
		{
			ClearMovie();
			MovieClear = 1;
		}
#endif
		initTexture(num, (ULONG*)ptr);
		FreeFileData(ptr);
	}
}


/*************************************************************************
 * FREE BACKGROUND - Frees all backgound picture space.
 *************************************************************************/
void FreeBackgrounds(void)
{
	int i;

	for (i = 0; i < N_BG; i++)
		free_pict(i);	 /* be sure the backgrounds are freed */
	numBG = 0;
}


/*************************************************************************
 * INIT TEXTURE - Loads a background.  Supports TIM, 8 & 16 BIT colors
 *		tested, should work with 4 BIT as well.
 *************************************************************************/
void initTexture(int num, u_long *timP)
{
	register int i,
					 pmode,
					 width;
	GsIMAGE TimInfo;

	free_pict(num);			  /* if overwriting, free old memory */

	if (*timP != TIM_HEADER)
	{
		Cerror();
		return;     // error Exit
	}

	numBG++;

	while(*timP == TIM_HEADER)
	{
		timP++;						/*##*/

		GsGetTimInfo(timP, &TimInfo);

		/* PICTURE  texture & CLUT */
		load_pict(num, TimInfo);
	}


// NOTE TimInfo.pmode for 256 color comes out to 9, so make it ourselves.
	if (picture[num].clutType == VRAM_16BIT)	// 16 BIT
	{
		width = 1;
		pmode = 2;
	}
	else
	if (picture[num].clutType == VRAM_CLUT256)	//  8 BIT
	{
		width = 2;
		pmode = 1;
	}
	else							//  4 BIT
	{
		width = 4;
		pmode = 0;
	}

	for (i = 0; i < picture[num].blocks; i++)
	{
		setPolyFT4(&picture[num].poly[i]);
		setSemiTrans(&picture[num].poly[i], 0);	// Background not semi transparent.
		setShadeTex(&picture[num].poly[i], 1);
		setRGB0(&picture[num].poly[i], 0x80, 0x80, 0x80);

		picture[num].poly[i].tpage = SetVramTPage(picture[num].tpage[i], pmode, 0);
		if (picture[num].clutType != VRAM_16BIT)	// 16 BIT
			picture[num].poly[i].clut = picture[num].CLUT->CLUT_ID;
		setUVWH(&picture[num].poly[i], 0, 0, 64 * width, 240); // Point to texture...
		setXYWH(&picture[num].poly[i], i * width * 64 -256, -120, 64 * width, 240); /* Point to texture... */
	}
//	SetDispMask(1);     /* Enable display */
//	gCurrDB ^= 1;
}


/*************************************************************************
 *	TEXTURE INIT - Used in Field.c to load a tim to a specific location for
 *		model data to use...
 *************************************************************************/
void LoadTimExact(u_long addr)
{
	int i, x;
	RECT rect;
	GsIMAGE tim;
	vram_t *vram;

	if ((addr == NULL) | (*(ULONG*)addr != TIM_HEADER))
	{
		Cerror();
		return;
	}
	GsGetTimInfo((u_long *)(addr+4),&tim); /* Load TIM to VRAM */
	rect.x=tim.px;
	rect.y=tim.py;
	rect.w=tim.pw;
	rect.h=tim.ph;
	LoadImage(&rect,tim.pixel);

/* MRH - Figure out what tpage is used and mark it used in VRAM */
	i = rect.x / 64; /* all cases */
	if (gVidMode == VMODE640)	  /* 640 */
	{
		i -= 10;
		if (rect.y)
			i += 6;		  /* lower row of TPAGES */
	}
	else if (gVidMode == VMODE512)	  /* 512 */
	{
		i -= 8;
		if (rect.y)
			i += 8;		  /* lower row of TPAGES */
	}
	else
	{					  /* 320 */
		i -= 5;
		if (rect.y)
			i += 11;   /* lower row of TPAGES */
	}
	texture[i].used = 1; /* mark it as used */
#if DEBUG_ALLOC_TEXTURE
	 DebugPrintf(0, "TIM Alloc Tex#%d, %d,%d\n", i, texture[i].x, texture[i].y);
#endif

	if((tim.pmode>>3)&0x01)		/* Check to see if CLUT is loaded */
	{
		rect.x=tim.cx;			/* Load CLUT from TIM to VRAM */
		rect.y=tim.cy;
		rect.w=tim.cw;
		rect.h=tim.ch;
		LoadImage(&rect,tim.clut);
	}

/* mark CLUT as used in VRAM */
	if (gVidMode == VMODE640)
		x = BIG_NUM_CLUT256_PAGES;
	else if (gVidMode == VMODE512)
		x = MED_NUM_CLUT256_PAGES;
	else
		x = NUM_CLUT256_PAGES;

	if ((rect.y - 480) < x)
		clut256[rect.y - 480].used = 1; /* mark one of the 256 cluts used... */
	else
		clut16[rect.y - 480 + x + rect.x / 16].used = 1; /* mark one of the 256 cluts used... */
}
