//**************************************************************************
//
// PLVIEW.C
//
// Player statistics display code modules
//
//	Bulk of this code ported from Andy Greens Saturn SKU and modified to make
//	it more modular to improve overall structure and reusability
//
//**************************************************************************

#include <stdio.h>
#include <sys/types.h>

#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include "prjhdrs.h"

#include "fbgequ.h"
#include "fbequ.h"
#include "fbram.h"
#include "fbstats.h"
#include "shell.h"
#include "movie.h"
#include "plview.h"

//**************************************************************************
// Data structures & defines for menu system

#include "menu.h"
#include "fbmenu.h"

//**************************************************************************

#define	ITEMS_X	68
#define	ITEMS_Y	70

#define	STATS_X	(ITEMS_X+100)
#define	INFO_X (ITEMS_X+120)

#define STAT_SPACING	90

#define TMVIEW_TEMPLATE 0
#define NAME_TEMPLATE	1
#define	RATINGS_BAR		0
#define	RATINGS_BAR2	8

#define CURSORY_MIN 20
#define CURSORY_MAX 200

#define PLLDRS_X	20
#define PLLDRS_Y	75

//**************************************************************************

extern void DisplayRatings(UWORD player);
extern void DisplaySubstRatings(UWORD player);
extern void InitPlayStats(void);
extern void UpdateNameLines(void);
extern void UpdateNames(void);
extern void UpdateInfo(UWORD player);

extern struct teamRAMstats *SeasonStatsPtr;
extern BYTE CtrlPad[2];
extern BYTE gSkipCoinTossFlag;
extern BYTE WeekNum;
extern WORD CardSlot;
extern SeasonMode;
extern BYTE *GameDataBuf0;

extern WORD LLDivision;
extern WORD NumTeams;
extern WORD DivBase;

//**************************************************************************


static BYTE NameLines[10][14];
static LONG sort_table[30][2];

BYTE TradeMode;
BYTE InfoColor = PEN_WHITE;
BYTE SaveOpp;

unsigned long *This_MFile;

WORD PlayerOffset = 0;
WORD InfoX;
WORD TempTeam;
WORD Ratings_Column;
WORD Ratings_Change;
WORD Mode_YOffset;
WORD StatMode[3]={1,0,0};			// ROM - GAME - SEASON display flags
WORD ModeFlag;
WORD Boxnum;

LONG UseTradedTeam;

LONG Column_Color[3]={PEN_BLUE,PEN_WHITE,PEN_WHITE};

//**************************************************************************
//**************************************************************************

MenuItem PlViewTeamItemList[] = {
	ACTIVE	| MENU_ITEM | HI_YELLOW | PEN_GREY | DIM_YELLOW, ITEMS_X+4, ITEMS_Y+0   , NULL,NULL,
	ACTIVE	| MENU_ITEM | HI_YELLOW | PEN_GREY | DIM_YELLOW, ITEMS_X+4, ITEMS_Y+15  , NULL,NULL,
	ACTIVE	| MENU_ITEM | HI_YELLOW | PEN_GREY | DIM_YELLOW, ITEMS_X+4, ITEMS_Y+30  , NULL,NULL,
	ACTIVE	| MENU_ITEM | HI_YELLOW | PEN_GREY | DIM_YELLOW, ITEMS_X+4, ITEMS_Y+45  , NULL,NULL,
	ACTIVE	| MENU_ITEM | HI_YELLOW | PEN_GREY | DIM_YELLOW, ITEMS_X+4, ITEMS_Y+60  , NULL,NULL,
	ACTIVE	| MENU_ITEM | HI_YELLOW | PEN_GREY | DIM_YELLOW, ITEMS_X+4, ITEMS_Y+75  , NULL,NULL,
	ACTIVE	| MENU_ITEM | HI_YELLOW | PEN_GREY | DIM_YELLOW, ITEMS_X+4, ITEMS_Y+90  , NULL,NULL,
	ACTIVE	| MENU_ITEM | HI_YELLOW | PEN_GREY | DIM_YELLOW, ITEMS_X+4, ITEMS_Y+105 , NULL,NULL,

	END_MENU,
};

MenuObj	PlayerViewMenu = {
	(MenuItem *)PlViewTeamItemList,		// Menu object
	InitViewPlayer,						// Menu Init
	TermViewPlayer,						// Menu termination
	MainViewPlayer,						// Menu proc (called each frame)
	0, ITEMS_Y, 0x98, ITEMS_Y+(9*15),
};


WORD Ratings_Lines;
WORD Season_Line;
WORD Season_Top;
WORD Season_Flash;

BYTE *RATINGS_TXT[][8] = {
	"speed",
	"strength",
	0,
	0,
	"scrambling",
	"arm strength",
	"accuracy",
	"intelligence",

	"speed",
	"strength",
	0,
	"hands",
	"agility",
	"blocking",
	0,0,

	"speed",
	"strength",
	0,
	"hands",
	"agility",
	"blocking",
	0,0,

	"speed",
	"strength",
	0,
	0,
	"pass blocking",
	"run blocking",
	0,0,

	"speed",
	"strength",
	"reaction",
	"hands",
	"rushing",
	"tackling",
	0,0,

	0,
	0,
	0,
	0,
	"leg strength",
	"accuracy",
	0,0,

	0,
	0,
	0,
	0,
	"leg strength",
	"accuracy",
	0,0,

	"speed",
	"strength",
	0,
	"hands",
	"agility",
	0,0,0
};


BYTE RatingTab[8];

UWORD STATS_NUM[] = {
	15,
	12,
	12,
	4,
	10,
	14,
	14,
	11
};

BYTE Rom_Stats[]={
		10,
		15,
		15,
		 5,
		 7,
		 5,
		 6,
		 8,		// was 6
};

WORD STATS_VALUE[][15] = {
	-0x100,	0,	1,	-0x200,	2,	3,	4,	5,	-0x306,	7,	8, -0x400,	10,  9,  11,
	-0x100,	0,	1,	-0x400,	2,	3,	4,	5,	-0x400,	6,	7,	8,	0,0,0,
	-0x100,	0,	1,	-0x400,	2,	3,	5,	6,	-0x400,	7,	8,	4,	0,0,0,
	-0x100,	-0x900,	-1,	0,	0,0,0,0,0,0,0,0,0,0,0,
//	-0x100,	1,	2,	-1,0,0,0,0,0,0,0,0,0,0,0,
//	-0x100,	-0x300,	1,	2,	0,0,0,0,0,0,0,0,0,0,0,
	-0x100,	-0x700,	-0x801,	-0x702,	-0x703,	6,	4,	5,	8,	7,	0,0,0,0,0,
	-0x100,	0,	1,	2,	3,	4,	-0x500,	6,	7, -0x400,	8,	9, 10,	11,	0,
	-0x100,	0,	1,	-0x400,	2,	3,	4,	5,	6,	7,	8,	9, 10, -0x500,	0,
	-0x100,	0,	1,	-0x400,	6,	2,	3,	4,	-0x400,	7,	5,	0,0,0,0,
};


BYTE *STATS_TXT[][2][15] = {
	"",			"Passing",	"Passing",	"Passing",
	"Passing",	"Passing",	"Passing",	"Passing",
	"Passing",	"Rushing",	"Rushing",	"Rushing",
	"Rushing",	"Rushing",	"Rushing",

	"Year",			"Att",			"Comp",			"%",
	"Yds",			"Lg",			"TD",			"Int",
	"Rat",			"Att",			"Yds",			"Avg",
	"Lg",			"TD",			"Con",


	"",				"Rushing",		"Rushing",		"Rushing",
	"Rushing",		"Rushing",		"Receiving",	"Receiving",
	"Receiving",	"Receiving",	"Receiving",	"",
	"",				"",				"",

	"Year",			"Att",			"Yds",			"Avg",
	"Lg",			"TD",			"No.",			"Yds",
	"Avg",			"Lg",			"TD",			"Con",
	"",				"",				"",

	"",				"Receiving",	"Receiving",	"Receiving",
	"Receiving",	"Receiving",	"Rushing",		"Rushing",
	"Rushing",		"Rushing",		"Rushing",		"",
	0,0,0,

	"Year",			"No.",			"Yds",			"Avg",
	"Lg",			"TD",			"Att",			"Yds",
	"Avg",			"Lg",			"TD",			"Con",
	0,0,0,


	"",				"Sacks",			0,				0,
//	"",				"Sacks",			"Team Sks",		"Team",
//	"",				"Team Sks",			"Team",			0,
	0,0,0,0,
	0,0,0,0,
	0,0,0,

	"Year",			"Allowed",		0,					0,
//	"Year",			"Allowed",		"Allowed",			"Rush Yds",
//	"Year",			"Allowed",		"Rush Yds",			0,
	0,0,0,0,
	0,0,0,0,
	0,0,0,

	"",				"",				"",				"Fumbles",
	"Fumbles",		"Fumbles",		"Int",			"Int",
	"Int",			"Int",			0,				0,
	0,0,0,


	"Year",			"Tk",			"Sacks",		"Forced",
	"Received",		"TD",			"No.",			"Yds",
	"Lg",			"TD",			0,				0,
	0,0,0,


	"",				"Pat",			"Pat",			"Fg",
	"Fg",			"Fg",			"Fg",			"Punt",
	"Punt",			"Punt",			"Punt",			"Punt",
	"Punt",			"Punt",			0,

	"Year",			"Pat",			"Att",			"Fg",
	"Att",			"Lg",			"Tp",			"No",
	"Yds",			"Avg",			"Lg",			"In 20",
	"Tb",			"Blk",			0,


	"",				"Punt",			"Punt",			"Punt",
	"Punt",			"Punt",			"Punt",			"Punt",
	"Pat",			"Pat",			"Fg",			"Fg",
	"Fg",			"Fg",			0,


	"Year",			"No.",			"Yds",			"Avg",
	"Lg",			"In 20",		"Tb",			"Blk",
	"Pat",			"Att",			"Fg",			"Att",
	"Lg",			"Tp",			0,


//	"",				"Kick",			"Kick",			"Kick",
//	"Kick",			"Kick",			"Punt",			"Punt",
//	"Punt",			"Punt",			"Punt",			0,
//	0,0,0,
	"",				"Kick Ret",			"Kick Ret",			"Kick Ret",
	"Kick Ret",			"Kick Ret",			"Punt Ret",			"Punt Ret",
	"Punt Ret",			"Punt Ret",			"Punt Ret",			0,
	0,0,0,

	"Year",			"No.",			"Yds",			"Avg",
	"Lg",			"TD",			"No.",			"Yds",
	"Avg",			"Lg",			"TD",
	0,0,0
};


/***************************************************************************/
/***************************************************************************/
/** View Player Menu (InitViewPlayer used to init all player stat menus)
/***************************************************************************/
/***************************************************************************/



INT InitViewPlayer(void)
{
	BYTE *ptr;
	BYTE fname[64];
	WORD i;

	SaveOpp = GameTeamData[1].TeamNumber;

  /*Set graphics mode*/

	initVRamHandling();

	if(CurMenu == SUBSTITUTION_MENU)
		LoadBackgroundCD(0,"/shells/bkg/substitu.tim");	// Load & free from memory
	else
		LoadTeamPic(GameTeamData[WhichTeam].TeamNumber);

  /*Init the multifile pointers*/

	Shell_MFile = (unsigned long *)LoadCDFile("/shells/stats/tmview.mfl");
	DebugPrintf(0,"tmview.mfl loaded into %x\n", Shell_MFile);

	SetupFont("fontdpc2.tim");

  /*load the shell graphics template files*/

	InitTemplates();

	multifile = Shell_MFile;
	LoadTemplate("tmview.llx");

	MenuCursorX = ITEMS_X-40;
	MenuCursorY	= ITEMS_Y;

	switch(CurMenu)
	{
		case SUBSTITUTION_MENU :
					MenuCursorX = ITEMS_X-4;
					MenuCursorY	= ITEMS_Y+8;
					Boxnum = PopupDialogBox(MenuCursorX, MenuCursorY, 165, 12, 2, 0x30, 0x00, 0x30);

					Column_Color[0]=PEN_BLUE;

					This_MFile = (unsigned long *)LoadCDFile("/shells/names/nickname.mfl");
					DebugPrintf(0,"nickname.mfl loaded into %x\n", This_MFile);
					multifile = This_MFile;
					LoadTemplate("nickname.llx");
					LoadNickname(TeamNo, NAME_TEMPLATE, 0);
					FreeFileData((BYTE *)This_MFile);

					This_MFile = (unsigned long *)LoadCDFile("/shells/stats/subst.mfl");
					DebugPrintf(0,"subst.mfl loaded into %x\n", This_MFile);
					multifile = This_MFile;
					LoadTemplate("subst.llx");
					FreeFileData((BYTE *)This_MFile);
					break;

		case TRADING_MENU :
					This_MFile = (unsigned long *)LoadCDFile("/shells/stats/trade.mfl");
					DebugPrintf(0,"trade.mfl loaded into %x\n", This_MFile);
					multifile = This_MFile;
					LoadTemplate("trade.llx");
					FreeFileData((BYTE *)This_MFile);
					break;

		default:
					Column_Color[0]=PEN_WHITE;
					Boxnum = PopupDialogBox(MenuCursorX, MenuCursorY, 130, 12, 2, 0x30, 0x00, 0x30);
					This_MFile = (unsigned long *)LoadCDFile("/shells/names/nickname.mfl");
					DebugPrintf(0,"nickname.mfl loaded into %x\n", This_MFile);
					multifile = This_MFile;
					LoadTemplate("nickname.llx");
					LoadNickname(TeamNo, NAME_TEMPLATE, 0);
					FreeFileData((BYTE *)This_MFile);
					MovePolyToXY(&template[NAME_TEMPLATE].poly[0],160-256, 20-120);
					MovePolyToXY(&template[NAME_TEMPLATE].poly[1],288-256, 20-120);
					break;
	}

	multifile = Shell_MFile;

	/* Init general purpose counters */

	ScrollYLimit = 1000;

	InitPlayStats();

	InfoX = TradeMode = TempTeam = 0;	// Only used by trade menu

	JpadRepeatFreq = 10;

	return(0);

}


//**************************************************************************



INT MainViewPlayer(void)
{
	INT retcode,i,line;
	WORD *statptr, oldcolm;
	BYTE stat[10];
	specificplayerdata *pdptr;

//	SetHighlight();

	FreeAllDialogBoxes();

	Mode_YOffset=0;

	oldcolm = Ratings_Column;

	MoveStatColumns(MenuItemNum+PlayerOffset,3);

	line = MenuItemNum + PlayerOffset;

	for(i = 0; i < 16; i++)
		MovePolyToXY(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR+i],512,0);	// Put unused bars offscreen

	if(Ratings_Column)
	{

		PrintStatHeader(MenuItemNum+PlayerOffset, STATS_X, ITEMS_Y+45);

		for(i=0; i<3; i++)
		{
			ModeFlag=i;

			if(StatMode[i])
			{

				if(Ratings_Column != 0)
				{
					PrintStringXYAbs("Year", STATS_X+20, ITEMS_Y+60, PEN_YELLOW);

					switch(ModeFlag)
					{
						case 0:	strcpy(stat, "1996"); break;
						case 1:	strcpy(stat, "Seas."); break;
						case 2:	strcpy(stat, "Game"); break;
					}

					PrintStringXYAbs(stat, STATS_X+20, ITEMS_Y+75+Mode_YOffset, PEN_BLUE);

				}

				switch(i)
				{
					case 0:
							pdptr = GetPlayerPosDataPtr(WhichTeam, line);
							statptr = (WORD *) pdptr->mypstats;
							break;

					case 1: statptr = (WORD *) (SeasonStatsPtr+TeamNo)->posstats[line];
							break;

					case 2: statptr = (WORD *) StatsData[WhichTeam].posstats[line];
							break;

				}


				DrawStatColumns(STATS_X, ITEMS_Y+75+Mode_YOffset, line, statptr, TRUE);


				Mode_YOffset += 15;
			}
		}
	}
	else
	{
		UpdateRatings(MenuItemNum + PlayerOffset);
		DisplayRatings(MenuItemNum + PlayerOffset);
	}

	UpdateInfo(MenuItemNum+PlayerOffset);
	UpdateNameLines();

	switch(retcode = InputHandler(ITEMS_Y, ITEMS_Y+(15*7)))
	{
		case TRIANGLE:	retcode = -1;
						break;
		case CROSS:		retcode = 1;
						break;
		default:		retcode = 0;
	}

	if(MenuDirY < 0)
	{
		if(PlayerOffset > 0)
		{
			PlayEffect(CURSOR_SFX);
			PlayerOffset--;
		}
	}
	else if(MenuDirY > 0)
	{
		if(PlayerOffset < TOTAL_PLAYERS_ON_TEAM-8)
		{
			PlayEffect(CURSOR_SFX);
			PlayerOffset++;
		}
	}

	SetHighlight();

	if(PlayerOffset)
	{
		PrintStringXYAbs(UP_ARROW,80,58,PEN_RED);
	}

	if(PlayerOffset < 39)
	{
		PrintStringXYAbs(DOWN_ARROW,80,190,PEN_RED);
	}

	if(oldcolm != Ratings_Column) PlayEffect(CURSOR_SFX);

	if(Ratings_Column+3 != STATS_NUM[PositionToStatSetTable[line]])
	{
		PrintStringXYAbs(RT_ARROW,340,200,PEN_RED);
	}

	if(Ratings_Column)
	{
		PrintStringXYAbs(LT_ARROW,320,200,PEN_RED);
	}

	return(retcode);
}


//**************************************************************************


INT TermViewPlayer(void)
{
	GameTeamData[1].TeamNumber = SaveOpp;
	if(SeasonStatsPtr)
	{
		FreeMem(SeasonStatsPtr);
		SeasonStatsPtr = NULL;
	}

	Std_Menu_CleanUp();
	return(0);
}


//**************************************************************************


void ViewPlayer(WORD team)
{
	WhichTeam = team;						// Store whether home or away
	TeamNo = GameTeamData[team].TeamNumber;	// Convert Home-Away value to actual Team No.

	Display_Menu(PLAYER_VIEW_MENU);
}


//**************************************************************************


/* INITIALIZE PLAYER STATS */

void InitPlayStats(void)
{
	UWORD count;

	specificplayerdata *pdptr;

	SeasonStatsPtr = NULL;

	StatMode[1] = StatMode[2] = 0;			// Clear while we determine game status

	if(GameMode) StatMode[2]= 1;			// If not (PREGAME) then is during or post game & show game stats

	if(GameType == SEASON_GAME && WeekNum > 1 && SeasonMode)
	{
		StatMode[1]= 1;						// If Season then always show Season stats

		SeasonStatsPtr	= (TEAMRAMStats *)AllocMem(NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats)+128);
		memset(SeasonStatsPtr, 0, NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats));
		AllTeamMemoryCardMimicToSeasonStats();	// returns 0 if okay
	}

	for(count = 0; count < 8; count++)
	{
		BYTE *name;

		pdptr = GetPlayerPosDataPtr(WhichTeam, count+PlayerOffset);

		name = (BYTE *)pdptr->pname;	while(*name++ != 0x20);

		PlViewTeamItemList[count].item_text = name;
	}

	Season_Line = 0;

	Ratings_Column = 0;
	Ratings_Change = 0;


	UpdateNameLines();
	UpdateInfo(MenuItemNum+PlayerOffset);
	UpdateRatings(MenuItemNum + PlayerOffset);
	DisplayRatings(MenuItemNum + PlayerOffset);
}


//**************************************************************************

/* DISPLAY PLAYER RATINGS */

void DisplayRatings(UWORD line)
{
	UWORD count, item;
	specificplayerdata *pdptr;


	pdptr = GetPlayerPosDataPtr(WhichTeam, line);

	for(count = 0; count < Ratings_Lines; count++)
	{
		if(RatingTab[count])
		{
			ChangeABSPolyWH(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR+count],
								 (pdptr->ratings[count]*2)+2, 5);
		}
	}
}


//**************************************************************************



void UpdateNameLines(void)
{
	INT count,color;
	specificplayerdata *pdptr;
	BYTE tmpstr1[4], tmpstr2[4];

	for(count = 0; count < 8; count++)
	{
		BYTE *name;

		pdptr = GetPlayerPosDataPtr(WhichTeam, count+PlayerOffset);

		name = (BYTE *)pdptr->pname;	while(*name++ != 0x20);

		if(count == MenuItemNum)
		{
			if(Flash & 8)
				color = PEN_YELLOW;
			else
				color = PEN_YELLOW+1;
		}
		else
			color = PEN_GREY;

		GetPositionString(tmpstr1,count+PlayerOffset);
		GetPositionString(tmpstr2,(PlayerOffset+count)-1);

		if(count == MenuItemNum || count==0 || strcmp(tmpstr1,tmpstr2))
				PrintStringXY(tmpstr1,PlViewTeamItemList[count].item_x-32, PlViewTeamItemList[count].item_y,color);

		PlViewTeamItemList[count].item_text = name;
	}
}


/***************************************************************************/

/* UPDATE PLAYER INFO */

void UpdateInfo(UWORD line)
{
	specificplayerdata *pdptr;
	BYTE years[5];
	UWORD feet, inches;

	BYTE info[40], tmpstr[4];

	pdptr = GetPlayerPosDataPtr(WhichTeam, line);

	GetPositionString(tmpstr,line);

	sprintf(info, "%s %d %s", tmpstr,
		pdptr->jerseynum, pdptr->pname);

	PrintStringXYAbs(info, INFO_X+InfoX, ITEMS_Y, InfoColor);

	inches = pdptr->height + PD_HEIGHT_OFFSET;
	feet = inches / 12;	inches -= feet * 12;

	sprintf(years, "%d", pdptr->yearsinleague);

	if(!strcmp(years, "0")) strcpy(years, "R");

	sprintf(info, "Ht: %d'%d%c Wt: %d Seas: %s", feet, inches, '"',
		pdptr->weight + PD_WEIGHT_OFFSET, years);

	PrintStringXYAbs(info, INFO_X+InfoX, ITEMS_Y+15, InfoColor);

	if(!TradeMode)
	{
		sprintf(info, "Dob: %d/%d/%d", pdptr->birthmonth,
	 		pdptr->birthday, pdptr->birthyear + PD_YEAR_OFFSET);

		PrintStringXYAbs(info, INFO_X+InfoX, ITEMS_Y+30, InfoColor);

//		sprintf(info, "%s", pdptr->collegename);

		PrintStringXYAbs(info, INFO_X+InfoX+130, ITEMS_Y+30, InfoColor);
	}
}


/***************************************************************************/

/* UPDATE PLAYER RATINGS */


void UpdateRatings(UWORD line)
{
	UWORD count,item;
	BYTE *text;

//	UWORD line = MenuItemNum+PlayerOffset;

	item = 0;

	for(count = 0; count < 8; count++)
	{
		if(line >= FS && count == 4)
		{
			text = RATINGS_TXT[PositionToStatSetTable[line]][count+1];
		}
		else
		if(line >= FS && count == 5)
		{
			text = NULL;
		}
		else
		{
			text = RATINGS_TXT[PositionToStatSetTable[line]][count];
		}


		if(!text)
		{
			MovePolyToXY(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR+count],512,0);	// Put unused bars offscreen
			RatingTab[count]=NULL;
		}
		else
		{
			PrintStringXYAbs(text, ITEMS_X+120, ITEMS_Y+45+(item*15), PEN_YELLOW);
			MovePolyToXY(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR+count],(ITEMS_X+244)-256, (ITEMS_Y+(item*15)+50)-120);
			RatingTab[count]=TRUE;
			item++;
		}
	}

	Ratings_Lines = count;
};


/***************************************************************************/

/* PRINT A DECIMAL NUMBER */

void PrintDecimal(BYTE *text, LONG number)
{
	LONG ten = number / 10;

	if(number < 0)
		sprintf(text, "-%d.%d", -ten, (-number)%10);
	else
		sprintf(text, "%d.%d", ten, number%10);
}



/***************************************************************************/

/* UPDATE STATS TEXT COLUMNS */

void DrawStatColumns(UWORD xpos, UWORD ypos, UWORD player, WORD *statptr, BYTE absflag)
{
	BYTE stat[10];
	UWORD count;
	UWORD statset = PositionToStatSetTable[player];

	for(count = 1; count < 4; count++)
	{
		BYTE *text;
		WORD *value_ptr, value;

		value_ptr = &STATS_VALUE[statset][(Ratings_Column - 1) + count];

		value = *value_ptr;

		if((Ratings_Column-1)+count == STATS_NUM[PositionToStatSetTable[player]]) return; // don't overun end of stat table

		if(ModeFlag == 0 && (Rom_Stats[statset] <= value)) value = -0x600;

		if(value == -1) return;


		if(value < 0)
		{
			LONG decimal;

			switch(value / 0x100)
			{
				case -2:
							decimal = statptr[*--value_ptr] * 1000;
							value = statptr[*--value_ptr];

							if(value) decimal /= value;
							else decimal = 0;

							PrintDecimal(stat, decimal);
							break;

				case -3:
							decimal = statptr[-(value + 0x300)];
							PrintDecimal(stat, decimal);
							break;

				case -4:
							decimal = statptr[*--value_ptr] * 10;
							value = statptr[*--value_ptr];

							if(value) decimal /= value;
							else decimal = 0;

							PrintDecimal(stat, decimal);
							break;

				case -5:
							value_ptr -= 3;

							decimal = statptr[*value_ptr] * 3;

							value_ptr -= 2;

							decimal += statptr[*value_ptr];

							sprintf(stat, "%d", decimal);
							break;

				case -6:	sprintf(stat,"--");
							break;

				case -7:
							decimal = statptr[-(value + 0x800)];
							if(ModeFlag)
								sprintf(stat, "%d", decimal);
							else
								sprintf(stat,"--");
							break;

				case -8:
							decimal = statptr[-(value + 0x800)];

							if(ModeFlag)
							{
								if(decimal < 0) sprintf(stat, "-%d", -decimal);
								else sprintf(stat, "%d", decimal);
							}
							else
							{
								PrintDecimal(stat, decimal);
							}
							break;
				case -9:
							decimal = statptr[-(value + 0x900)];

							if(ModeFlag)
							{
								decimal *= 10;	// For RAM Sacks allowed !!!
								PrintDecimal(stat, decimal);
							}
							else sprintf(stat,"--");

							break;
			}
		}
		else sprintf(stat, "%d", statptr[value]);

		if(absflag) PrintStringXYAbs(stat, xpos+(count*STAT_SPACING), ypos, Column_Color[count-1]);
		else PrintStringXY(stat, xpos+(count*STAT_SPACING), ypos,  Column_Color[count-1]);

	}

}


//***************************************************************************/
//
//***************************************************************************/

void PrintStatHeader(INT player, INT x , INT y)
{
	INT count, oldy;
	BYTE *text_last = (BYTE *)"";
	BYTE *text;


	UWORD statset = PositionToStatSetTable[player];

	for(count = 1; count < 4; count++)
	{
		text = STATS_TXT[statset][0][(Ratings_Column - 1) + count];

		if((Ratings_Column-1)+count == STATS_NUM[PositionToStatSetTable[player]]) return; // don't overun end of stat table

		if(statset == SS_OL) // || statset == SS_KICKER)
		{
			PrintStringXYAbs(text, x+(count * STAT_SPACING), y, PEN_YELLOW);
		}
		else
		{
			if(text != (BYTE *)"" && text == text_last)
			{
				text = "======";
				PrintStringXYAbs(text, x+((count-1) * STAT_SPACING), y, PEN_YELLOW);
			}
			else
			{
				text_last = text;
				PrintStringXYAbs(text, x+(count * STAT_SPACING), y, PEN_YELLOW);
			}
		}

		text = STATS_TXT[statset][1][(Ratings_Column - 1) + count];
		PrintStringXYAbs(text, x+(count*STAT_SPACING), y+15, PEN_YELLOW);
	}

}


/***************************************************************************/
/* MOVE STATS POINTER */
/***************************************************************************/


WORD MoveStatColumns(UWORD position, UWORD rlimit)
{
	WORD change = 0;
	WORD pad = JpadTrig[CtrlPad[0]] | JpadTrig[CtrlPad[1]];


	WORD column = Ratings_Column;

	if(pad & JPADLleft)
	{
		if(column)
		{
			change = -1;	column--;
		}
	}
	else
	if(pad & JPADLright)
	{
			change = 1;	column++;
	}

	if(column > (STATS_NUM[PositionToStatSetTable[position]] - rlimit))
			column = STATS_NUM[PositionToStatSetTable[position]] - rlimit;

	Ratings_Column = column;

	return change;
}




/*****************************************************************************/

void HalftimeShell(void)
{
	DumpHeap();

	PlayHalftimeMovie();
	DumpHeap();

	DisplayTmStats();
	DumpHeap();
	ViewPlayer(0);
	DumpHeap();
	ViewPlayer(1);

	DumpHeap();
}

/*****************************************************************************/

void EndGameShell(BYTE mode)
{
	SyncFlag = TRUE;
	PlaySong(0x4);

	DisplayTmStats();
	ViewPlayer(0);
	ViewPlayer(1);
}


/*****************************************************************************/
/*****************************************************************************/
// League Leaders Routines
/*****************************************************************************/
/*****************************************************************************/

LONG Ldr_Table[15][4];
static WORD LastSort = -1;

BYTE LdrIndex;
UBYTE *Pnames;


MenuItem PlyrLeaderItemList[] = {
	END_MENU
};


MenuObj	PlyrLeaderMenu = {
	(MenuItem *)PlyrLeaderItemList,		// Menu object
	InitPlyrLeader,						// Menu Init
	TermPlyrLeader,						// Menu termination
	MainPlyrLeader,						// Menu proc (called each frame)
	0, PLLDRS_Y-1, 512, (9*15),
};


enum LeaderCategories {

	PASSING,
	RECEIVING,
	RUSHING,
	KICKRETURNS,
	PUNTS,
	KICKING,

};


BYTE *LdrScreens[]={
		"/shells/bkg/lpassing.tim",
		"/shells/bkg/lrushing.tim",
		"/shells/bkg/lreceivi.tim",
		"/shells/bkg/lkicking.tim",
		"/shells/bkg/lpunts.tim",
		"/shells/bkg/lkickret.tim",
};

BYTE *LdrTemplates[]={
		"passing.llx",
		"rushing.llx",
		"receive.llx",
		"kicking.llx",
		"punts.llx",
		"kickret.llx",
};

BYTE *LdrMfl[]={
		"/shells/stats/passing.mfl",
		"/shells/stats/rushing.mfl",
		"/shells/stats/receive.mfl",
		"/shells/stats/kicking.mfl",
		"/shells/stats/punts.mfl",
		"/shells/stats/kickret.mfl",
};


BYTE PositionTab[][2] = {
							0,3,
							11,15,
							3,15,
							26,27,
							25,26,
							22,25,
};


//**************************************************************************


void SortLeaders(void)
{
	WORD i,j,k;
	WORD *statptr, data;
	WORD tmp[3];
	BYTE flag = 1;
	for(i=0; i<20; i++)
		Ldr_Table[i][2] = -32000;

	for(i=DivBase; i<DivBase+NumTeams; i++)
	{
		for(j=PositionTab[LdrIndex][0]; j<PositionTab[LdrIndex][1]; j++)
		{
			statptr = (WORD *) (SeasonStatsPtr+i)->posstats[j];

			data = GetPlyrStat(j, statptr);

			tmp[0]= 0;
			tmp[1]= Ldr_Table[0][2];

			for(k=1; k < 15; k++)			// Find lowest stat in list
			{
				if(tmp[1] > Ldr_Table[k][2])
				{
					tmp[0] = k;
					tmp[1] = Ldr_Table[k][2];
				}
			}

			if(data > tmp[1])
			{
				Ldr_Table[tmp[0]][0] = i;
				Ldr_Table[tmp[0]][1] = j;
				Ldr_Table[tmp[0]][2] = data;
			}
		}
	}

	while(flag)
	{
		flag = 0;

		for(i=19; i>0; i--)
		{
			if(Ldr_Table[i][2] > Ldr_Table[i-1][2])
			{
				flag++;

				for(j=0; j<3; j++)
				{
					tmp[j] = Ldr_Table[i][j];
					Ldr_Table[i][j] = Ldr_Table[i-1][j];
					Ldr_Table[i-1][j] = tmp[j];
				}
			}
		}
	}
}


//**************************************************************************


void DisplayPlLeaders(void)
{
	INT i;
	WORD *statptr;
	BYTE *nameptr;
	BYTE name[22];
	BYTE abrev[2]={0,0};
	BYTE posstr[4];
	specificplayerdata *pdptr;
	BYTE pencol;

	PrintStatHeader(PositionTab[LdrIndex][0], PLLDRS_X+240, PLLDRS_Y-30);

	for(i=0; i<15; i++)
	{
		statptr = (WORD *) (SeasonStatsPtr+Ldr_Table[i][0])->posstats[Ldr_Table[i][1]];

		sprintf(name,"%2d.",i+1);
		PrintStringXY(name, PLLDRS_X, PLLDRS_Y+(15*i), PEN_YELLOW);		// Rank

		if(Ldr_Table[i][0]==HumanTeamNum && MemoryCardMimicPtr->UseTradedTeam)
		{
			pdptr = GetPlayerPosDataPtr(WhichTeam, Ldr_Table[i][1]);
			nameptr = pdptr->pname;
		}
		else
		{
			 nameptr = Pnames+(Ldr_Table[i][0]*(47*21))+(Ldr_Table[i][1]*21);
		}

		abrev[0] = *nameptr;
		while(*(nameptr++) != 0x20);

		sprintf(name,"%s.%s",abrev,nameptr);

		name[14]=0;	//Terminate at character no. 15

		if(Ldr_Table[i][0] == HumanTeamNum)
			pencol = PEN_BLUE;
		else
			pencol = PEN_WHITE;

		PrintStringXY(name, PLLDRS_X+40, PLLDRS_Y+(15*i), pencol);									// Player name (1st name abreviated)

		GetPositionString(posstr,Ldr_Table[i][1]);

		PrintStringXY(posstr,PLLDRS_X+190, PLLDRS_Y+(15*i), PEN_YELLOW);			// Position

		if(Ldr_Table[i][0] == HumanTeamNum)
			pencol = PEN_BLUE;
		else
			pencol = PEN_RED;

		PrintStringXY(SpecificTeamData[Ldr_Table[i][0]].TeamNickName,PLLDRS_X+220, PLLDRS_Y+(15*i), pencol);	// Team name

		DrawStatColumns(PLLDRS_X+240, PLLDRS_Y+(15*i), Ldr_Table[i][1], statptr, FALSE);
	}
}


//**************************************************************************


WORD GetPlyrStat(UWORD player, WORD *statptr)
{
	BYTE stat[10];
	UWORD count;
	UWORD statset = PositionToStatSetTable[player];

	WORD *value_ptr, value;

	value_ptr = &STATS_VALUE[statset][Ratings_Column];

	value = *value_ptr;

	if(value < 0)
	{
		LONG decimal;

		switch(value / 0x100)
		{
			case -2:
						decimal = statptr[*--value_ptr] * 1000;
						value = statptr[*--value_ptr];

						if(value) decimal /= value;
						else decimal = 0;

						return(decimal);
						break;

			case -3:
						decimal = statptr[-(value + 0x300)];
						return(decimal);
						break;

			case -4:
						decimal = statptr[*--value_ptr] * 10;
						value = statptr[*--value_ptr];

						if(value) decimal /= value;
						else decimal = 0;

						return(decimal);
						break;

			case -5:
						value_ptr -= 3;

						decimal = statptr[*value_ptr] * 3;

						value_ptr -= 2;

						decimal += statptr[*value_ptr];

						return(decimal);
						break;

			case -8:
						decimal = statptr[-(value + 0x800)];

						if(ModeFlag)
						{
							if(decimal < 0) sprintf(stat, "-%d", -decimal);
							else sprintf(stat, "%d", decimal);
						}
						else
						{
							PrintDecimal(stat, decimal);
						}
						break;

		}
	}
	else return(statptr[value]);

}


//**************************************************************************


INT InitPlyrLeader(void)
{
	BYTE *ptr;
	WORD i;

  //Set graphics mode

	initVRamHandling();

  //initialize the background for the shell

	LoadBackgroundCD(0,LdrScreens[LdrIndex]);	// Load & free from memory

	Shell_MFile = (unsigned long *)LoadCDFile(LdrMfl[LdrIndex]);
	DebugPrintf(0,"Leaders MFL No. %d loaded into %x\n", LdrIndex,Shell_MFile);

	This_MFile = (unsigned long *)LoadCDFile("/shells/stats/logos.mfl");
	DebugPrintf(0,"Logos loaded\n");

	SetupFont("fontdpc4.tim");
	CurFont = fontdpc4;

  //load the shell graphics template files

	InitTemplates();

	multifile = Shell_MFile;
	LoadTemplate(LdrTemplates[LdrIndex]);

	multifile = This_MFile;
	LoadTemplate("logos.llx");

	for(i=0; i<3; i++)
	{
		if(i!=LLDivision) template[1].action[i] &=DISPLAY^0xFFFF;
	}

	FreeMem(This_MFile);

	multifile = Shell_MFile;

	MenuCursorX = PLLDRS_X;
	MenuCursorY	= PLLDRS_Y;

//	InitPlayStats();

	JpadRepeatFreq = 8;

	SeasonStatsPtr	= (TEAMRAMStats *)AllocMem(NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats)+128);
	memset(SeasonStatsPtr, 0, NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats));
	AllTeamMemoryCardMimicToSeasonStats();	// returns 0 if okay

	ModeFlag=1;	// Set to display season stats

	switch(LLDivision)
	{
		case 0: NumTeams = 15; DivBase = 0; break;
		case 1: NumTeams = 15; DivBase = 15; break;
		case 2: NumTeams = 30; DivBase = 0; break;
	}

	ScrollYLimit = -((15-9)*15);

	Ratings_Column = 1;

	for(i=0; i<15; i++)
		Ldr_Table[0][1]=0;


	if(HumanTeamNum == GameTeamData[0].TeamNumber)
		WhichTeam = 0;
//		InitControl(GameTeamData[0].ControllerNum[0]);
	else
		WhichTeam = 1;
//		InitControl(GameTeamData[1].ControllerNum[0]);

	CtrlPad[0] = 0;
	CtrlPad[1] = 1;

	LastSort = -1;

	Pnames	= (BYTE *)LoadCDFile("/shells/stats/pnames.dat");

	return(0);

}


//**************************************************************************


INT TermPlyrLeader(void)
{
	if(SeasonStatsPtr)
	{
		FreeMem(SeasonStatsPtr);
		SeasonStatsPtr = NULL;
	}
	FreeMem(Pnames);
	FreeMem(Shell_MFile);

	ResetStrings();
//	FreeTemplate(0);
	FreeAllTemplates();
	FreeBackgrounds();
	return(0);
}


//**************************************************************************


INT	MainPlyrLeader(void)
{
	INT retcode,oldcolm;

	oldcolm = Ratings_Column;

	switch(retcode = InputHandler(1000, -1))
	{
		case TRIANGLE:	retcode = -1;
						break;
		case CROSS:		retcode = 1;
						break;
		default:		retcode = 0;
	}

	MoveStatColumns(PositionTab[LdrIndex][0],1);

	if(!Ratings_Column) Ratings_Column++;

	if(LastSort != Ratings_Column)
	{
		LastSort = Ratings_Column;
		SortLeaders();
	}

	DisplayPlLeaders();

	if(oldcolm != Ratings_Column) PlayEffect(CURSOR_SFX);

	if(Ratings_Column+1 != STATS_NUM[PositionToStatSetTable[Ldr_Table[0][1]]])
	{
		PrintStringXYAbs(RT_ARROW,100,62,PEN_RED);
	}

	if(Ratings_Column > 1)
	{
		PrintStringXYAbs(LT_ARROW,80,62,PEN_RED);
	}

	if(MenuTxt_Yoffset)
		PrintStringXYAbs(UP_ARROW,26,62,PEN_RED);

	if(MenuTxt_Yoffset != -(6*15))
		PrintStringXYAbs(DOWN_ARROW,26,210,PEN_RED);

	return(retcode);
}


//**************************************************************************
//**************************************************************************
// Calls up Player Leaders menu
//**************************************************************************
//**************************************************************************


INT DisplayPlyrLeaders(void)
{
	return(Display_Menu(PLYRLDRS_MENU));
}


//**************************************************************************
//**************************************************************************
// Sets a string to describe playing position
//**************************************************************************
//**************************************************************************


void GetPositionString(BYTE *string, INT position)
{
	sprintf(string, "%s", PositionStrings[position]);
}


//**************************************************************************
//**************************************************************************
// Sets a string to describe playing position but accounts for special case
//**************************************************************************
//**************************************************************************

void GetPositionLString(BYTE *string, INT position)
{
	switch(TeamNo)
	{
		case 5:
		case 10:
		case 18:
				sprintf(string, "%s", DepthChart34Strings[position]);
				break;
		default:
				sprintf(string, "%s", DepthChart43Strings[position]);
				break;
	}
}

//**************************************************************************
//**************************************************************************
// Substitution Menu (based on player stats display)
//**************************************************************************
//**************************************************************************


BYTE *PosStrings[]={
					"Quarterbacks",
					"Receivers",
					"Runningbacks",
					"Offensive Line",
					"Punt Returner",
					"Kick Returner",
					"Defensive Line",
					"Linebackers",
					"Defensive Backs",
					NULL,
};

BYTE SubstPosTab[][2] = {
							0,3,
							3,11,
							11,15,
							15,22,
							22,25,
							22,25,
							27,33,
							33,39,
							39,47,
};



MenuObj	SubstitutionMenu = {
	(MenuItem *)PlViewTeamItemList,		// Menu object
	InitViewPlayer,						// Menu Init
	TermViewPlayer,						// Menu termination
	MainSubstitution,					// Menu proc (called each frame)
	0, ITEMS_Y, 0xB0, ITEMS_Y+(9*15),	// Clips the name list (all other strings overide clip rect)
};


ULONG PosGroup;
ULONG SubstNum;
ULONG SubstMode = 0;
BYTE SubstFlag = 0;
LONG SubstX[]={172, ITEMS_X-48};

INT MainSubstitution(void)
{
	INT retcode,i, temp;
	WORD *statptr;
	BYTE stat[10];
	specificplayerdata *pdptr;

	SetHighlight();

	Mode_YOffset=0;				// Doesn't apply but is used in routines called from here
	JpadRepeatFreq = 10;

	for(i=0; i<8; i++)
	{
		PlViewTeamItemList[i].item_flags &=0xFFFFFFF0;	// Reset menu string colors
		PlViewTeamItemList[i].item_flags |=PEN_GREY;	// to grey
	}

	for(i = 0; i < 16; i++)
		MovePolyToXY(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR+i],512,0);	// Ensure any unused bars are offscreen

	if(SubstFlag)
	{
		UpdateRatings(TeamRoster[WhichTeam][MenuItemNum + SubstPosTab[PosGroup][0]]);
		DisplayRatings(TeamRoster[WhichTeam][MenuItemNum + SubstPosTab[PosGroup][0]]);
		UpdateInfo(TeamRoster[WhichTeam][MenuItemNum + SubstPosTab[PosGroup][0]]);
		UpdateSubstNameLines(PosGroup);

		if(SubstMode) DisplaySubstRatings(TeamRoster[WhichTeam][SubstNum]);

	}
	else
	{
		DoSubstituteMenu();
	}

	switch(retcode = InputHandler(ITEMS_Y+8, ITEMS_Y+8+(15*8)))
	{
		case TRIANGLE:
						if(!SubstFlag)
							retcode = -1;
						else
						{
							FlushPads();
							FreeAllDialogBoxes();
							Boxnum = PopupDialogBox(MenuCursorX, MenuCursorY, 170, 12, 2, 0x30, 0x00, 0x30);
							DoMenuStrip();
							SubstFlag = 0;
							MenuItemNum = PosGroup;
							MenuCursorY	= ITEMS_Y+8+(MenuItemNum*15);
							retcode = 0;
						}
						break;

		case CROSS:		if(!SubstFlag)
						{
							FlushPads();
							FreeAllDialogBoxes();
							MenuCursorY	= ITEMS_Y;
							Boxnum = PopupDialogBox(MenuCursorX, MenuCursorY, 165, 12, 2, 0x30, 0x00, 0x30);
							DoMenuStrip();
							SubstFlag = 1;
							SubstMode = 0;
							PosGroup = MenuItemNum;
							MenuItemNum = 0;
						}
						else
						{
							if(SubstMode)
							{
								SubstMode = 0;
								MenuCursorY	= ITEMS_Y+(MenuItemNum*15);
								temp = TeamRoster[WhichTeam][SubstPosTab[PosGroup][0] + MenuItemNum];
								TeamRoster[WhichTeam][SubstPosTab[PosGroup][0] + MenuItemNum] = TeamRoster[WhichTeam][SubstNum];
								TeamRoster[WhichTeam][SubstNum] = temp;
							}
							else
							{
								SubstNum = SubstPosTab[PosGroup][0] + MenuItemNum;
								SubstMode = 1;
							}
						}
						retcode = 0;
						break;

		default:		retcode = 0;
						break;
	}

	if(SubstFlag)
	{
		if(MenuItemNum == SubstPosTab[PosGroup][1]-SubstPosTab[PosGroup][0])
		{
			MenuItemNum--;
			MenuCursorY	-= 15;
		}
	}
	else
	{
		if(MenuItemNum > 9)
		{
			MenuItemNum--;
			MenuCursorY	-= 15;
		}
	}

	MoveDialog(0, SubstX[SubstFlag], MenuCursorY+1);				// Move to X, Y

	return(retcode);
}

//**************************************************************************
//**************************************************************************

void DoSubstituteMenu(void)
{
	INT i = 0, j;
	INT color;

	for(j= 0; j < 8; j++)
		PlViewTeamItemList[j].item_text = NULL;

	if(MenuItemNum > 8)
	{
		MenuItemNum--;
		MenuCursorY	-= 15;
	}

	while(PosStrings[i] != NULL)
	{
		if(i == MenuItemNum)
		{
			if(Flash & 0x08)
				color = PEN_YELLOW;
			else
				color = PEN_DKYELLOW;
		}
		else color = PEN_WHITE;

		PrintStringXYCenter(PosStrings[i],CENTER_SCREEN, ITEMS_Y+8+(i*15),color);

		i++;
	}
}


//**************************************************************************
//**************************************************************************


void UpdateSubstNameLines(UWORD group)
{
	INT count,color, i, oldw;
	specificplayerdata *pdptr;
	BYTE tmpstr1[4], tmpstr2[4];

	i = 0;

	for(count = SubstPosTab[group][0]; count < SubstPosTab[group][1]; count++)
	{
		BYTE *name;

		pdptr = GetPlayerPosDataPtr(WhichTeam, TeamRoster[WhichTeam][count]);

		name = (BYTE *)pdptr->pname;	while(*name++ != 0x20);

		color = PEN_GREY;

		if(i == MenuItemNum)
		{
			if(Flash & 8)
				color = PEN_YELLOW;
			else
				color = PEN_YELLOW+1;
		}
		else
		{
			if(SubstMode && count == SubstNum)
				color = PEN_RED;
			else
				color = PEN_GREY;
		}

		GetPositionLString(tmpstr1,count);
		PrintStringXYAbs(tmpstr1,PlViewTeamItemList[i].item_x-52, PlViewTeamItemList[i].item_y,color);

		PlViewTeamItemList[i].item_text = name;
		PlViewTeamItemList[i].item_flags &=0xFFFFFFF0;
		PlViewTeamItemList[i].item_flags |=color;

		i++;
	}

}


//**************************************************************************
/* DISPLAY SUBSTITUTING PLAYER RATINGS */
//**************************************************************************

void DisplaySubstRatings(UWORD line)
{
	UWORD count, item;
	BYTE *text;
	specificplayerdata *pdptr;


	pdptr = GetPlayerPosDataPtr(WhichTeam, line);

	item = 0;

	for(count = 0; count < Ratings_Lines; count++)
	{
		if(line >= FS && count == 4)
		{
			text = RATINGS_TXT[PositionToStatSetTable[line]][count+1];
		}
		else
		if(line >= FS && count == 5)
		{
			text = NULL;
		}
		else
		{
			text = RATINGS_TXT[PositionToStatSetTable[line]][count];
		}


		if(!text)
		{
			MovePolyToXY(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR2+count],512,0);	// Put unused bars offscreen
		}
		else
		{
			MovePolyToXY(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR2+count],(ITEMS_X+244)-256, (ITEMS_Y+(item*15)+56)-120);
			ChangeABSPolyWH(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR2+count],
							 (pdptr->ratings[count]*2)+2, 5);
			item++;
		}
	}
}


//**************************************************************************
//**************************************************************************
// Player trading menu (based on player stats display)
//**************************************************************************
//**************************************************************************

BYTE TradePosTab[] = { 0,3,11,15,22,25,26,27,33,39,47 };
BYTE TradePosIndex;
BYTE MyPlayer;
WORD OldItem, OldOffset, OldCursorY;

//**************************************************************************

MenuObj	TradingMenu = {
	(MenuItem *)PlViewTeamItemList,		// Menu object
	InitTradePlayer,						// Menu Init
	TermTradePlayer,						// Menu termination
	MainTradePlayer,					// Menu proc (called each frame)
	0, ITEMS_Y, 0xB0, ITEMS_Y+(9*15),
};

//**************************************************************************
//**************************************************************************

void DisplayTrading(INT team)
{
	WhichTeam = team;
	TeamNo = GameTeamData[team].TeamNumber;

	Display_Menu(TRADING_MENU);
}
//**************************************************************************
//**************************************************************************


INT InitTradePlayer(void)
{
	InitViewPlayer();

	if(GameDataBuf0	= (BYTE *)AllocMem(sizeof(struct gameteamdata)+128))
	{
		memcpy(GameDataBuf0,&GameTeamData[WhichTeam^1],sizeof(struct gameteamdata));
	}
}


//**************************************************************************
//**************************************************************************


INT TermTradePlayer(void)
{

	if(GameDataBuf0)
	{
		memcpy(&GameTeamData[WhichTeam^1],GameDataBuf0,sizeof(struct gameteamdata));
		FreeMem(GameDataBuf0);
	}

	TermViewPlayer();
}

//**************************************************************************
//**************************************************************************


INT MainTradePlayer(void)
{
	specificplayerdata *srcptr, *dstptr;

	INT retcode,i,line;
	WORD *statptr;
	BYTE stat[10];
	specificplayerdata *pdptr;

	Mode_YOffset=0;

	PrintStringXYAbs("Load Existing Team",70,208,PEN_YELLOW);
	PrintStringXYAbs("Save Team",290,208,PEN_YELLOW);

	for(i = 0; i < 16; i++)
		MovePolyToXY(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR+i],512,0);	// Put unused bars offscreen

	PrintStringXYCenter(SpecificTeamData[TeamNo].TeamNickName,100, 40, PEN_YELLOW);	// Team name

	if(!TradeMode)
	{
		JpadRepeatFreq = 10;

		UpdateRatings(MenuItemNum + PlayerOffset);
		DisplayRatings(MenuItemNum + PlayerOffset);
		UpdateInfo(MenuItemNum+PlayerOffset);

		UpdateNameLines();

		if(MenuDirY < 0)
		{
			if(PlayerOffset > 0)
			{
				PlayerOffset--;
				PlayEffect(CURSOR_SFX);
			}
		}
		else if(MenuDirY > 0)
		{
			if(PlayerOffset < TOTAL_PLAYERS_ON_TEAM-8)
			{
				PlayerOffset++;
				PlayEffect(CURSOR_SFX);
			}
		}
	}
	else
	{
		JpadRepeatFreq = 20;

		if(MenuDirX)
		{
			PlayEffect(CURSOR_SFX);
			GameTeamData[WhichTeam^1].TeamNumber +=MenuDirX;
			if(GameTeamData[WhichTeam^1].TeamNumber < 0) GameTeamData[WhichTeam^1].TeamNumber = 29;
			GameTeamData[WhichTeam^1].TeamNumber %=30;
			LoadPlayerDataFor1Team(WhichTeam^1);
		}

		if(MenuItemNum == (TradePosTab[TradePosIndex+1]-TradePosTab[TradePosIndex]))
		{
			MenuCursorY-= 15;
			MenuItemNum--;
		}

		PrintStringXYCenter(SpecificTeamData[GameTeamData[WhichTeam^1].TeamNumber].TeamNickName,380, 40, PEN_YELLOW);	// Team name
		WhichTeam^=1;
		InfoX = 80;
		InfoColor = PEN_RED;
		UpdateInfo(TradePosTab[TradePosIndex]+MenuItemNum);
		DisplaySubstRatings(TradePosTab[TradePosIndex]+MenuItemNum);
		WhichTeam^=1;
		InfoX = -160;
		InfoColor = PEN_WHITE;
		UpdateRatings(MyPlayer);
		DisplayRatings(MyPlayer);
		UpdateInfo(MyPlayer);

	}


	if(TradeMode)
	{
		retcode = InputHandler(ITEMS_Y, ITEMS_Y+((TradePosTab[TradePosIndex+1]-TradePosTab[TradePosIndex]-1)*15));
		if(MenuItemNum)
			PrintStringXYAbs(UP_ARROW,380,58,PEN_GREEN);
		if(MenuItemNum < (TradePosTab[TradePosIndex+1]-TradePosTab[TradePosIndex])-1)
			PrintStringXYAbs(DOWN_ARROW,380,100,PEN_GREEN);

		PrintStringXYAbs(LT_ARROW,280,40,PEN_GREEN);
		PrintStringXYAbs(RT_ARROW,480,40,PEN_GREEN);

	}
	else
		retcode = InputHandler(ITEMS_Y, ITEMS_Y+(15*7));

	switch(retcode)
	{
		case TRIANGLE:	if(!TradeMode) retcode = -1;
						else
						{
							ResetTradeDisplay();
							retcode = 0;
						}
						break;

		case CROSS:
						if(!TradeMode)
						{
							TradeMode = 1;
							for(i=0; i<8; i++) PlViewTeamItemList[i].item_text = NULL;
							TradingMenu.rect_menu.y = 0;
							SetClipping(&TradingMenu.rect_menu);
							InfoX = -180;

							OldOffset = PlayerOffset;
							OldItem = MenuItemNum;
							OldCursorY = MenuCursorY;

							MyPlayer = MenuItemNum + PlayerOffset;

							for(i=0; i<10; i++)
							{
								if((MyPlayer) >= TradePosTab[i]
									&& (MyPlayer) < TradePosTab[i+1])
								{
									TradePosIndex = i;
								}
							}

							MenuItemNum = PlayerOffset = 0;
							MenuCursorY	= ITEMS_Y;
							Boxnum = PopupDialogBox(20, 60, 472, 50, 2, 0x30, 0x30, 0x30);
						}
						else
						{
							dstptr = GetPlayerPosDataPtr(WhichTeam, MyPlayer);
							srcptr = GetPlayerPosDataPtr(WhichTeam^1, TradePosTab[TradePosIndex]+MenuItemNum);

							memcpy(dstptr, srcptr, sizeof(specificplayerdata));

							ResetTradeDisplay();
						}
						retcode = 0;
						break;

		case CIRCLE:	SaveCreatedTeam(WhichTeam);
						retcode = 0;
						break;

		case SQUARE:	LoadCreatedTeam(WhichTeam, 1);
						retcode = 0;
						break;

		default:		retcode = 0;
	}

	SetHighlight();

	if(!TradeMode)
	{
		if(PlayerOffset)
		{
			PrintStringXYAbs(UP_ARROW,80,58,PEN_RED);
		}

		if(PlayerOffset < 39)
		{
			PrintStringXYAbs(DOWN_ARROW,80,190,PEN_RED);
		}
	}

	return(retcode);
}


//**************************************************************************
//**************************************************************************


void ResetTradeDisplay(void)
{
	TradeMode = 0;
	TradingMenu.rect_menu.y = ITEMS_Y;
	SetClipping(&TradingMenu.rect_menu);
	InfoX = 0;

	PlayerOffset = OldOffset;
	MenuItemNum = OldItem;
	MenuCursorY = OldCursorY;
	FreeDialogBox(Boxnum);
}


//**************************************************************************
//**************************************************************************


INT SaveCreatedTeam(LONG team)
{
	UBYTE sjisstr[60];

	BYTE *ptr, *fname, *fcomment;
	LONG retcode = -1;
	UWORD bank, port;

	if(GameType == SEASON_GAME)
	{
		fname = TRADE_SEASONFILE;
		fcomment = TRADE_SEASONSTR;
	}
	else
	{
		fname = TRADE_EXHIBFILE;
		fcomment = TRADE_EXHIBSTR;
	}

	String2SJIS(fcomment, (unsigned short *)sjisstr);

	UseTradedTeam=TRUE;

	// Alloc for team data, 1 long, + the roster

	if(ptr=(BYTE *)AllocMem((TOTAL_PLAYERS_ON_TEAM * sizeof(specificplayerdata))+4+TOTAL_PLAYERS_ON_TEAM+128))
	{
		*ptr = GameTeamData[team].TeamNumber;

		memcpy(ptr+4, &GameTeamData[team].PlayerData[0],TOTAL_PLAYERS_ON_TEAM * sizeof(specificplayerdata));
		memcpy(ptr+4+(TOTAL_PLAYERS_ON_TEAM * sizeof(specificplayerdata)),&TeamRoster[team][0],TOTAL_PLAYERS_ON_TEAM);

		bank = GetPadPortBank(GameTeamData[team].ControllerNum[0]);
		port = bank >> 8;
		bank &= 0xFF;

//		if(GetCardBank())
		if(CheckCard(port, bank))
		{
			if (CheckFile(port, bank, fname))
			{
				if (DialogBoxYN(256, 100, "OVERWRITE current team file?"))
				{
					retcode = OpenFile(ptr, (TOTAL_PLAYERS_ON_TEAM * sizeof(specificplayerdata))+4+TOTAL_PLAYERS_ON_TEAM, port, bank, fname, fcomment);
				}
			}
			else retcode = OpenFile(ptr, (TOTAL_PLAYERS_ON_TEAM * sizeof(specificplayerdata))+4+TOTAL_PLAYERS_ON_TEAM, port, bank, fname, fcomment);
		}

		FreeMem(ptr);
	}

	return(retcode);
}



//**************************************************************************
//**************************************************************************


INT LoadCreatedTeam(LONG team, LONG msgflg)
{
	BYTE *ptr, *fname, *fcomment;
	LONG retcode = -1;
	UWORD bank, port;

	if(GameType == SEASON_GAME)
	{
		fname = TRADE_SEASONFILE;
	}
	else
	{
		fname = TRADE_EXHIBFILE;
	}

	// Alloc for team data, 1 long, + the roster

	if(ptr = (BYTE *)AllocMem((TOTAL_PLAYERS_ON_TEAM * sizeof(specificplayerdata))+4+TOTAL_PLAYERS_ON_TEAM+128))
	{
		bank = GetPadPortBank(GameTeamData[team].ControllerNum[0]);
		port = bank >> 8;
		bank &= 0xFF;

//		if(GetCardBank())
		if(CheckCard(port ,bank))
		{
			if (CheckFile(port, bank, fname))
				retcode = ReadFile((unsigned short *)ptr, port, bank, fname);
			else
				TimedMessage(4, "No Created Team on Memory Card");

			if(!retcode)
			{
				if (!msgflg || DialogBoxYN(256, 112, "Do you wish to load over your current team?"))
				{
					memcpy(&GameTeamData[team].PlayerData[0],ptr+4,TOTAL_PLAYERS_ON_TEAM * sizeof(specificplayerdata));
					memcpy(&TeamRoster[team][0],ptr+4+(TOTAL_PLAYERS_ON_TEAM * sizeof(specificplayerdata)),TOTAL_PLAYERS_ON_TEAM);
				}
			}
		}

		FreeMem(ptr);
	}

	return(retcode);
}

//**************************************************************************

void DisplaySubstitution(void)
{
	Display_Menu(SUBSTITUTION_MENU);
}

//**************************************************************************

