/***************************************************************
	Player Animation code

	Copyright 1995 GameTek
	All rights reserved.

	By Christopher G. Deel
****************************************************************/

#define PLYRANIM_C 1

//#define DEBUG_THIS 1	//---- Debug This module ----
#if DEBUG_THIS
	#define FORCE_ANIM_SET 1	// Force this animation set to load.
#endif //--DEBUG_THIS--


#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include <kernel.h>

#include "prjhdrs.h"	/* Project header files */

#include "MultiFil.h"
#include "PlyrAnim.h"
#include "gameloop.h"
#include "arcade.h"
#include "camera.h"
#include "coinlog.h"
#include "coingraf.h"
//#include "refanim.h"
#include "jnumber.h"

#include "fbequ.h"
#include "fbram.h"
#include "fbramdat.h"
#include "debugshl.h"

#define MAKE_SHADOWS 1
//#define MAX_PLAYERS 22
#define FRAME_WIDTH		64
#define FRAME_HEIGHT	64
#define FRAME_BYTES (FRAME_WIDTH * FRAME_HEIGHT)

PlayerAnim_t	gPlayerSprite[NUM_PLAYER_SLOTS];	// 22 + Refs

#define REFEREE_ANIM_BYTES  26630  // largest referee anim file
char *gRefereeFilePtr = NULL;
ULONG gRefereeIsometricSector, gRefereeSideViewSector, gRefereeNormalSector;

#define MAX_REF_ANIMS (MAX_PG_GRAPHICS - FIRST_REF_ANIM)
#define MAX_PLAYER_ANIMS (FIRST_REF_ANIM)

struct
{
	vram_t *clut;
	char *filePtr;
	USHORT tpage, dummy;
	short animStart[MAX_REF_ANIMS][16];
} gRefGraphic;

int gArcadePermLoaded = False, gArcadeTempLoaded = False;

vram_t *gTeamClut[2];
vram_t *gAnimPage[3];
vram_t *gShadowClut = NULL, *gPlayerMarkerClut = NULL;
//		*gRedClut = NULL, *gGreenClut = NULL,
//		*gPinkClut = NULL, *gBrownClut = NULL;
USHORT *gBaseClutPtr[2], *gTmpClutPtr, *gTmpClutPtr2;
USHORT gTeamClutData1[256], gTeamClutData2[256], gSkinColorsBuff[256];
char gTeamColorFname[2][40];

char *gAnimBasePtr;	/* pointer to begining of Anim data */
char *gAnimBuff = NULL;


int gAnimSetNum = 0;
MultiFile_t *gAnimationsFile[2] = {NULL, NULL};
ULONG gAnimSetSector[3] = {0,0,0};
ULONG gReplayAnimSetSector[3][3] = {0,0,0, 0,0,0, 0,0,0};
ULONG gAnimationSector[2] = {0, 0};
ULONG gSideViewAnimSector[3];
ULONG gIsometricAnimSector[3];
char *gJnumPicPtr;

#define ANIM_NOREPLAY 1
#define ANIM_COIN_TOSS (1<<1)

static struct
{
	short fileNum;
	short pgNum;    	// Player Graphic Num
	char *name;
	void *jnumPosDir;
	ULONG flags;
} gAnimNames[] =
	{
		{0, PG_RUNNING_NUM, 		"runanim.frx", JnumPosDirRUN, 0,},
//		0, PG_WALKING_NUM, 			"rucanim.frx", JnumPosDirRUN, 0,
		{0, PG_SHUFFLE_NUM, 			"stdanim.frx", JnumPosDirSTD, 0},

////		1, PG_3PTSTANCE_NUM, 		"3psanim.frx", NULL,0,
		{0, PG_4PTSTANCE_NUM, 		"4psanim.frx", JnumPosDir4PS,0},

//		{1, PG_QBPLAYCALL_NUM, 		"qbcanim.frx", JnumPosDirQBC,0},
//		{1, PG_SHUFFLEWITHBALL_NUM,	"stbanim.frx", JnumPosDirSTB,0},
		{1, PG_PASSING_NUM, 		"thranim.frx", JnumPosDirTHR,0},
		{1, PG_BACKPEDAL_NUM, 		"badanim.frx", JnumPosDirBAD,0},
		{1, PG_BACKPEDAL_WBALL_NUM,	"bakanim.frx", JnumPosDirBAK,0},
		{1, PG_HANDSUP_NUM, 		"jbkanim.frx", JnumPosDirJBK,0},
		{1, PG_KICKING_NUM, 		"kckanim.frx", JnumPosDirKCK,0},
		{1, PG_PUNTING_NUM, 		"pntanim.frx", JnumPosDirPNT,0},
//		{1, PG_RUNNINGWITHBALL_NUM, "rubanim.frx", JnumPosDirRUB,0},
//		{1, PG_FALLONBELLY_NUM, 	"falanim.frx", NULL, 0},
//		{1, PG_FALLONBELLY2_NUM, 	"ffxanim.frx", NULL,0},
		{1, PG_FALLONBACK_NUM, 		"fbxanim.frx", NULL,0},
		{1, PG_DIVE_NUM, 			"tklanim.frx", JnumPosDirTKL,0},
//		{1, PG_DIVEBALL_NUM, 		"divanim.frx", JnumPosDirDIV,0},
		{1, PG_BLOCKING_NUM, 		"blkanim.frx", JnumPosDirBLK,0},
//		{1, PG_QBPITCH_NUM, 		"pitanim.frx", JnumPosDirPIT,0},
		{1, PG_FGHOLDER_NUM, 		"hfganim.frx", JnumPosDirHFG,0},
		{1, PG_RUNNINGCATCH_NUM, 	"rucanim.frx", JnumPosDirRUC, 0},
//		{1, PG_RUNNINGHEADTURN_NUM, "rhtanim.frx", JnumPosDirRHT, 0},
//		{1, PG_HANDOFF_NUM, 		"hndanim.frx", JnumPosDirHND,0},
//		{1, PG_QBSLIDE_NUM, 		"qbsanim.frx", JnumPosDirQBS, 0},
//		{1, PG_ROLL_NUM, 			"rolanim.frx", NULL,0},
//		{1, PG_BURST_NUM, 			"brsanim.frx", JnumPosDirBRS,	0},
//		{1, PG_SIDESTEP_NUM, 		"dssanim.frx", JnumPosDirDSS, 0},
//		{1, PG_STANDINGCATCH_NUM, 	"stcanim.frx", JnumPosDirSTC, 0},
//		{1, PG_RECEIVERSTANCE_NUM, 	"wrsanim.frx", JnumPosDirWRS, 0},
//		{1, PG_JUMPCATCH_NUM, 		"jctanim.frx", JnumPosDirJCT, 0},

//		{1, PG_DEFENSE_PRESNAP1_NUM, "dmxanim.frx", NULL, 0},
//		{1, PG_DEFENSE_PRESNAP2_NUM, "dmyanim.frx", NULL, 0},
//		{1, PG_DEFENSE_POSTTACKLE_NUM, "dtxanim.frx", NULL, 0},
//		{1, PG_SPIKEBALL_NUM, 		"tdnanim.frx", NULL, 0},
		{0, -1, "\0", },
	};


/* Direction Flip    0  1  2  3  4  5  6  7  8  9 10 11 12 13 14 15 */
char gDirFlip[16] = {0, 1, 1, 1, 1, 1, 1, 1, 0,	0, 0, 0, 0, 0, 0, 0 };

//#define MAX_ANIMS 26  // should be 24

AnimInfo_t gAnimInfo[MAX_PLAYER_ANIMS + 2];

//--- Ball graphics ---
#define BALL_TEX_WIDTH  32
#define BALL_TEX_HEIGHT	32

struct
{
	PolySpritePtr sprite;
	PolySpritePtr shadow;
	vram_t *tpage;
	vram_t *clut;
	USHORT tpageID;
	USHORT shadowID;
	RECT vrec;
	char *base;
	int KickedAnim[3];
	int PassedAnim[3];
	short frame;
} gBallGraphic;

// ---- Player Marker Info ----
#define PAD_NUMBER_HEIGHT 40
ULONG gPlayerMarkerSector = 0;
char *gPlayerMarkerData = NULL;

PlayerMarker_t gPlayerMarker[MAX_PLAYER_MARKERS] =
	{
		NULL, 0, 0,32,32, 0,0,0,  //0 The ball (don't use)
		NULL,32, 0,32,32, 0,0,0,  //1 not used
		NULL,64, 0,32,32, 200,200 ,0,  //2 Ball Carrier
		NULL,96, 0,32,32, 0,0,0,   //3 not used
		
		NULL,   0,32, 16,16, PAD_NUMBER_HEIGHT,PAD_NUMBER_HEIGHT,0, //4 #1
		NULL,  16,32, 16,16, PAD_NUMBER_HEIGHT,PAD_NUMBER_HEIGHT,0, //5
		NULL,2*16,32, 16,16, PAD_NUMBER_HEIGHT,PAD_NUMBER_HEIGHT,0, //6
		NULL,3*16,32, 16,16, PAD_NUMBER_HEIGHT,PAD_NUMBER_HEIGHT,0, //7
		NULL,   0,48, 16,16, PAD_NUMBER_HEIGHT,PAD_NUMBER_HEIGHT,0,  //8
		NULL,1*16,48, 16,16, PAD_NUMBER_HEIGHT,PAD_NUMBER_HEIGHT,0,  //9
		NULL,2*16,48, 16,16, PAD_NUMBER_HEIGHT,PAD_NUMBER_HEIGHT,0,  //10
		NULL,3*16,48, 16,16, PAD_NUMBER_HEIGHT,PAD_NUMBER_HEIGHT,0,  //11
		
		NULL, 0,96,32,32, 200,200,0,  //12 Square
		NULL,32,96,32,32, 200,200,0,  //13 Triangle
		NULL,64,96,32,32, 200,200,0,  //14 Circle
		NULL,96,96,32,32, 200,200,0,  //15 Target
		NULL,96,96,32,32, 150,150,0,   //16 Target replay
		NULL,0,64, 32,32,200,200,0,  //17 Receiver L
		NULL,32,64,32,32, 200,200,0,  //17 Receiver R
	};

//PolySpritePtr gPlayerMarkerSprite[MAX_PLAYER_MARKERS];


// -- Digit masks for 17 segment numbers  (Bit encoded) ---
static unsigned long gSingleDigitMasks[] =
{
	(1<<6 | 1<<7 | 1<<9 | 1<<10 | 1<<11 | 1<<12) >> 1,	//0
	(1<<11 | 1<<12 ) >> 1,								//1
	( 1<<6 | 1<<8 | 1<<9 | 1<<10 | 1<<11 ) >> 1,		//2
	( 1<<6 | 1<<8 | 1<<10 | 1<<11 | 1<<12 ) >> 1,		//3
	( 1<<7 | 1<<8 | 1<<11 | 1<<12 ) >> 1,		//4
	( 1<<6 | 1<<7 | 1<<8 | 1<<10 | 1<<12 ) >> 1,		//5
	( 1<<6 | 1<<7 | 1<<8 | 1<<9 | 1<<10 | 1<<12 ) >> 1,	//6
	( 1<<6 | 1<<11 | 1<<12 ) >> 1,						//7
	( 1<<6 | 1<<7 | 1<<8 | 1<<9 | 1<<10 | 1<<11 | 1<<12 ) >> 1,	//8
	( 1<<6 | 1<<7 | 1<<8 | 1<<11 | 1<<12 ) >> 1			//9
};

static unsigned long gTensDigitMasks[] =
{
	( 1<<1 | 1<<2 | 1<<4 | 1<<5 | 1<<7 | 1<<9 ) >> 1,	//0
	( 1<<7 | 1<<9 ) >> 1,								//1
	( 1<<1 | 1<<3 | 1<<4 | 1<<5 | 1<<7 ) >> 1,			//2
	( 1<<1 | 1<<3 | 1<<5 | 1<<7 | 1<<9 ) >> 1,			//3
	( 1<<2 | 1<<3 | 1<<7 | 1<<9 ) >> 1,					//4
	( 1<<1 | 1<<2 | 1<<3 | 1<<5 | 1<<9 ) >> 1,			//5
	( 1<<1 | 1<<2 | 1<<3 | 1<<4 | 1<<5 | 1<<9 ) >> 1,	//6
	( 1<<1 | 1<<7 | 1<<9 ) >> 1,						//7
	( 1<<1 | 1<<2 | 1<<3 | 1<<4 | 1<<5 | 1<<7 | 1<<9 ) >> 1,	//8
	( 1<<1 | 1<<2 | 1<<3 | 1<<7 | 1<<9 ) >> 1			//9
};

static unsigned long gOnesDigitMasks[] =
{
	( 1<<11 | 1<<12 | 1<<13 | 1<<14 | 1<<16 | 1<<17 ) >> 1,	//0
	( 1<<14 | 1<<16 ) >> 1,									//1
	( 1<<12 | 1<<13 | 1<<14 | 1<<15 | 1<<17 ) >> 1,			//2
	( 1<<13 | 1<<14 | 1<<15 | 1<<16 | 1<<17 ) >> 1,			//3
	( 1<<11 | 1<<14 | 1<<15 | 1<<16 ) >> 1,					//4
	( 1<<11 | 1<<13 | 1<<15 | 1<<16 | 1<<17 ) >> 1,			//5
	( 1<<11 | 1<<12 | 1<<13 | 1<<15 | 1<<16 | 1<<17 ) >> 1,	//6
	( 1<<13 | 1<<14 | 1<<16 ) >> 1,							//7
	( 1<<11 | 1<<12 | 1<<13 | 1<<14 | 1<<15 | 1<<16 | 1<<17 ) >> 1,	//8
	( 1<<11 | 1<<13 | 1<<14 | 1<<15 | 1<<16 ) >> 1			//9
};

static unsigned long gSingleDigitMasksFlip[] =
{
	( 1<<6 | 1<<7 | 1<<9 | 1<<10 | 1<<11 | 1<<12 ) >> 1,	//0
	( 1<<7 | 1<<9 ) >> 1,	//1
	( 1<<6 | 1<<7 | 1<<8 | 1<<10 | 1<<12 ) >> 1,	//2
	( 1<<6 | 1<<7 | 1<<8 | 1<<9 | 1<<10 ) >> 1,	//3
	( 1<<7 | 1<<8 | 1<<9 | 1<<11 ) >> 1,	//4
	( 1<<6 | 1<<8 | 1<<9 | 1<<10 | 1<<11 ) >> 1,	//5
	( 1<<6 | 1<<8 | 1<<9 | 1<<10 | 1<<11 | 1<<12 ) >> 1,	//6
	( 1<<6 | 1<<7 | 1<<9 ) >> 1,	//7
	( 1<<6 | 1<<7 | 1<<8 | 1<<9 | 1<<10 | 1<<11 | 1<<12 ) >> 1,	//8
	( 1<<6 | 1<<7 | 1<<8 | 1<<9 | 1<<11 ) >> 1	//9

};

static unsigned long gTensDigitMasksFlip[] =
{
	( 1<<11 | 1<<12 | 1<<13 | 1<<14 | 1<<16 | 1<<17 ) >> 1,	//0
	( 1<<11 | 1<<12 ) >> 1,	//1
	( 1<<11 | 1<<13 | 1<<15 | 1<<16 | 1<<17 ) >> 1,	//2
	( 1<<11 | 1<<12 | 1<<13 | 1<<15 | 1<<17 ) >> 1,	//3
	( 1<<11 | 1<<12 | 1<<14 | 1<<15 ) >> 1,	//4
	( 1<<12 | 1<<13 | 1<<14 | 1<<15 | 1<<17 ) >> 1,	//5
	( 1<<12 | 1<<13 | 1<<14 | 1<<15 | 1<<16 | 1<<17 ) >> 1,	//6
	( 1<<11 | 1<<12 | 1<<13 ) >> 1,	//7
	( 1<<11 | 1<<12 | 1<<13 | 1<<14 | 1<<15 | 1<<16 | 1<<17 ) >> 1,	//8
	( 1<<11 | 1<<12 | 1<<13 | 1<<14 | 1<<15 ) >> 1	//9
};

static unsigned long gOnesDigitMasksFlip[] =
{
	( 1<<1 | 1<<2 | 1<<4 | 1<<5 | 1<<7 | 1<<9 ) >> 1,	//0
	( 1<<2 | 1<<4 ) >> 1,	//1
	( 1<<1 | 1<<2 | 1<<3 | 1<<5 | 1<<9 ) >> 1,	//2
	( 1<<1 | 1<<2 | 1<<3 | 1<<4 | 1<<5 ) >> 1,	//3
	( 1<<2 | 1<<3 | 1<<4 | 1<<7 ) >> 1,	//4
	( 1<<1 | 1<<3 | 1<<4 | 1<<5 | 1<<7 ) >> 1,	//5
	( 1<<3 | 1<<4 | 1<<5 | 1<<7 | 1<<9 ) >> 1,	//6
	( 1<<1 | 1<<2 | 1<<4 ) >> 1,	//7
	( 1<<1 | 1<<2 | 1<<3 | 1<<4 | 1<<5 | 1<<7 | 1<<9 ) >> 1,	//8
	( 1<<1 | 1<<2 | 1<<3 | 1<<4 | 1<<7 ) >> 1	//9
};

#define WEIGHT_SUBTRACT 150

UBYTE gWeightTable[16][10] =
{
// 0    1    2    3    4    5    6    7    8    9   
   5,  15,  25,  36,  47,  59,  71,  84,  97, 111,   // 5' 8   68'' 
  10,  20,  31,  42,  54,  66,  79,  92, 106, 120,   // 5' 9   69'' 
  17,  27,  38,  49,  61,  73,  86,  99, 113, 127,   // 5'10   70'' 
  24,  35,  46,  58,  70,  83,  96, 110, 124, 139,   // 5'11   71'' 
  31,  42,  54,  66,  79,  92, 106, 120, 135, 150,   // 6' 0   72'' 
  37,  48,  60,  72,  85,  98, 112, 126, 141, 157,   // 6' 1   73'' 
  43,  55,  67,  80,  93, 107, 121, 136, 151, 167,   // 6' 2   74'' 
  50,  62,  74,  87, 100, 114, 129, 144, 160, 176,   // 6' 3   75'' 
  56,  68,  81,  94, 108, 122, 137, 152, 168, 185,   // 6' 4   76'' 
  63,  75,  88, 101, 115, 130, 145, 161, 177, 194,   // 6' 5   77'' 
  70,  83,  96, 110, 124, 139, 155, 171, 188, 206,   // 6' 6   78'' 
  77,  90, 104, 118, 133, 148, 164, 181, 198, 216,   // 6' 7   79'' 
  84,  97, 111, 125, 140, 156, 172, 189, 207, 225,   // 6' 8   80'' 
  91, 105, 119, 134, 149, 165, 182, 199, 217, 236,   // 6' 9   81'' 
  98, 112, 126, 141, 157, 173, 190, 208, 226, 245,   // 6'10   82'' 
 105, 119, 134, 149, 165, 182, 199, 217, 236, 255,   // 6'11   83'' 
};	
	
#if 0  // for testing
/*-------------------------------------------------------------------------
 * DECOMPRESS - Run Length Encoded data is be stored as follows:
 *	 First BYTE MSB = 0 - number of bytes to copy.
 *	 First BYTE MSB = 1 - && with 0x80 for times to duplicate next byte.
 -------------------------------------------------------------------------*/
static void DecompressFrameBytesC(char *dest, char *data)
{
	register unsigned char cnt, val;
	register unsigned char *current,  *ptr;    /*##*/
	register unsigned char	mask80 = 0x80,
							mask7f = 0x7F;

	ptr = dest;
	current = data;
	cnt = *current++;
	while (cnt)
	{
		if (cnt & mask80) /* if MSB = 1 it's a duplicate */
		{
			cnt &= mask7f;
			cnt++;
			val = *current++;
			while(cnt--)
			{
				*ptr++ = val;
			}
		}
		else	/* if it's a copy... */
		{
			while(cnt--)
			{
				*ptr++ = *current++;
			}
		}
		cnt = *current++;
	}
}

#endif

/*-------------------------------------------------------------
  GetScreenPosFrom3d ~ 
-------------------------------------------------------------*/
int GetScreenPosFrom3d(SVECTOR *vec1, SVECTOR *vec2)
{
	long flag, p;
	int otz;

	otz = RotTransPers(vec1, (long*)vec2, &p, &flag);
	if (GsGetActiveBuff() != 1)
		vec2->vy -= 240;
	return otz;
}  /* End GetScreenPosFrom3d() */


/*-------------------------------------------------------------
  MakeShadowClut ~ Make shadow CLUT and put into VRAM
	return vram_t *clut
-------------------------------------------------------------*/
vram_t *MakeShadowClut(void)
{
	vram_t *clut;
	register USHORT shadow, *colors;
	register int i;

//	shadow = 0x8000 | (1 << 10) | (1 << 5) | 1;
	shadow =  0x8000 | 1;
	colors = (USHORT*)AllocMem(512);
	*colors = 0;
	for (i = 1; i < 256; i++)
	{
		*(colors + i) = shadow;
	}
	clut = register_CLUT( (ULONG*)colors, VRAM_CLUT256);
	FreeMem(colors);
	return clut;
}

/*-------------------------------------------------------------
  GenClutFname ~ Generate Clut file name
-------------------------------------------------------------*/
static char *GenClutFname(int team, short away, char *fname)
{
//	 team %= 35;   // debug

	if (away)
	{
		sprintf(fname, "/colors/away%d/a%03d.clt", team/30, team);
	}
	else
	{
		sprintf(fname, "/colors/home%d/h%03d.clt", team/30, team);
	}
	 DebugPrintf(0, "%s\n", fname);
	return fname;
}

/*-------------------------------------------------------------
  GetPlayerClutPtr ~ Get Pointer to player clut in RAM
    Not needed
-------------------------------------------------------------*/
USHORT * GetPlayerClutPtr(int team, int player)
{
	USHORT *ptr;

	ptr = (USHORT*) gBaseClutPtr[team];
	ptr += (player * 0x100);
	return ptr;
}

/*-------------------------------------------------------------
  SetClutSection ~ Set Section of CLUT
-------------------------------------------------------------*/
void SetClutSection(USHORT *new, USHORT *clut, int start, int count)
{
	register USHORT *ptr;

	ptr = clut + start;
	while(count--)
	{
		*ptr++ = *new++;
	}
}

/*-------------------------------------------------------------
  SetClutToNumber ~
-------------------------------------------------------------*/
void SetClutToNumber(USHORT *clut, int team, int number, int flip)
{
	USHORT *on, *off, *data;
	int start, seg;
	register ULONG mask;

	if (number > 99 || number < 0)
	{
		Cerror();
		number = 99;
	}
	if (!flip)
	{
		if (number < 10)
		{
			mask = gSingleDigitMasks[number];
		}
		else
		{
			mask = gOnesDigitMasks[number % 10];
			mask |= gTensDigitMasks[number/10];
		}
	}
	else
	{
		if (number < 10)
		{
			mask = gSingleDigitMasksFlip[number];
		}
		else
		{
			mask = gOnesDigitMasksFlip[number % 10];
			mask |= gTensDigitMasksFlip[number/10];
		}
	}

//	 DebugPrintf(0, "Jnum: %d %d %X\n", number, flip, mask);
	on =  gBaseClutPtr[team] + CLUT_SEG_ON;
	off = gBaseClutPtr[team] + CLUT_SEG_OFF;
	for (seg = 0; seg < 17; seg++)
	{
		start = CLUT_SEGMENTS + (seg * CLUT_SEG_SIZE);
		if (mask & 1)
			data = on;
		else
			data = off;
		SetClutSection(data, clut, start, CLUT_SEG_SIZE);
		mask = mask >> 1;
	}
}

/*-------------------------------------------------------------
  SetPlayerColors ~ Set skin color Etc
-------------------------------------------------------------*/
static void SetPlayerColors(int skin, int gloves, int sleeves, USHORT *clut)
{
	int i, val;
	USHORT *ptr1, *ptr2;
	USHORT buff[8];

	limitRange(skin, 0, 6);
//	ptr1 = &gSkinColorsBuff[(skin * 8) + 1];  //old
	ptr1 = &gSkinColorsBuff[(skin * 32) + 32];
//	SetClutSection(ptr1, clut, CLUT_HEAD, 8);
	SetClutSection(ptr1, clut, CLUT_HEAD, 32);
	
#if 0  // old	
	if (sleeves)
	{
		for (i = 0; i < 8; i++)
		{
			val = 0x40 + i * 0x18;
			buff[i] = GenPSXColor(val, val, val, 0);
		}
		SetClutSection(&buff[0], clut, CLUT_ARMS, 8);
	}
	else
		SetClutSection(ptr1, clut, CLUT_ARMS, 8);
	if (!gloves)
		SetClutSection(ptr1, clut, CLUT_HANDS, 8);
#endif //old
}

/*-------------------------------------------------------------
  SetPlayerLocation ~
-------------------------------------------------------------*/
void MovePlayerSpriteTo(int x, int y, int z, int plyrNum)
{
	MovePolySpriteTo(x, y, z, gPlayerSprite[plyrNum].sprite);
#if MAKE_SHADOWS
	MovePolySpriteTo(x, 0, z - y/2 -10, gPlayerSprite[plyrNum].shadow);
#endif
//	MovePolySpriteTo(x, y-110, z, gPlayerSprite[plyrNum].jnumSprite);
//	MovePolySpriteTo(x, y, z, gPlayerSprite[plyrNum].jnumSprite);
	SetJerseyNumberPosition(x, y, z, plyrNum);
}

/*-------------------------------------------------------------
  GetPlayerSpriteLocation ~
-------------------------------------------------------------*/
void GetPlayerSpriteLocation(int pnum, SVECTOR *vec)
{
	register PolySpritePtr sprite;

	sprite = gPlayerSprite[pnum].sprite;
	vec->vx = sprite->trans.vx;
	vec->vy = sprite->trans.vy;
	vec->vz = sprite->trans.vz;
}

/*-------------------------------------------------------------
  GetFramePtr ~
-------------------------------------------------------------*/
char * GetFramePtr(int anim, int dir, int frameNum)
{
	int fnum2;
	char *base, *ptr;

	base = gAnimInfo[anim].base;
	fnum2 = gAnimInfo[anim].frames[dir] + frameNum;
	return GetMultiFileByNum((MultiFile_t *)base, fnum2);
}

/*-------------------------------------------------------------
  SetPlayerFrame ~
-------------------------------------------------------------*/
void SetPlayerFrame(int plyrNum, int anim, int dir, int frameNum)
{
	int fnum2, flip, shadowFlip, shadowFlipY = FALSE, shadowOn = TRUE;
	int userReplay, lefty, jnumOn;
	char *base;
	PolySpritePtr shadow;
//	char passFlipDir[] = {15, 14, 13, 12, 11, 10, 9, 8,

	if (plyrNum >= REF_SLOT_NUM )
	{
		SetRefereeFrame(plyrNum, anim, dir, frameNum);
		return;
	}

	if (gInstantReplayMode && !gGameLoopFlags2.DoingAutoReplay)
		userReplay = TRUE;
	else
		userReplay = FALSE;
			
//	 anim = PG_RUNNING_NUM;		// For Debug
	if (anim >= MAX_PLAYER_ANIMS || dir > 15 || dir < 0 || frameNum < 0
	    		|| frameNum >= NumFramesPerPlayerGraphic[anim])
	{
		DebugPrintf(0, "Anim error: Plyr:%d anm:%d dir:%d frm:%d\n",
									plyrNum, anim,dir,frameNum);
		return;  // error
	}
//	dir = (dir - ( (gCamera.dir & 0xE) - 4)) & 15;
//	dir = ((dir & 0xE) - ( gCamera.dir - 4)) & 15;
	dir = (dir - ( gCamera.dir - 4)) & 15;
	base = gAnimInfo[anim].base;
	flip = gDirFlip[dir];
	if (base == NULL)
	{
//		 DebugPrintf(0, "NULL Anim Plyr:%d anm:%d\n", plyrNum, anim);
//		anim = PG_RUNNING_NUM;
		switch(anim)
		{
			case PG_DEFENSE_PRESNAP1_NUM:
			case PG_DEFENSE_PRESNAP2_NUM:
			case PG_DEFENSE_POSTTACKLE_NUM:
				anim = PG_STAND_NUM;
				frameNum = 0;
				break;
			default:
				anim = PG_RUNNING_NUM;
				break;
		}
		base = gAnimInfo[anim].base;
	}
	frameNum = min(NumFramesPerPlayerGraphic[anim], frameNum);

//	if ( (anim == PG_PASSING_NUM) || (anim == PG_BACKPEDAL_WBALL_NUM))
	if ( (anim == PG_PASSING_NUM))
	{
	    gReplayAnimFlags = 0;	// No punting or kicking
		if (PlayerLeftHanded(0) && plyrNum == 0 // if Left Handed QB
	    	&& (gInstantReplayMode || gGameCameraStyle==NORMAL_CAMERA_STYLE)
		    )
		{
			flip = True;
			dir = 16 - dir;
		}
		else
			flip = False;
	
		if (gGameLoopFlags2.DoingAutoReplay 
		    	&& gGameCameraStyle != NORMAL_CAMERA_STYLE)
    	{
	    	flip=False;
    	}
	}
	
	switch (anim)
	{
		case PG_PUNTING_NUM:
			SetBit(gReplayAnimFlags, USED_PUNTING_ANIM);
			break;
		
		case PG_KICKING_NUM:
			SetBit(gReplayAnimFlags, USED_KICKING_ANIM);
			break;
		
		case PG_BACKPEDAL_WBALL_NUM:
	    	if (gGameCameraStyle == NORMAL_CAMERA_STYLE && !userReplay
	    	    	&& PlayerLeftHanded(0) && plyrNum == 0 )
   	    	{
   	    		flip = TRUE;
   	    	}
		case PG_BACKPEDAL_NUM:  // same for with ball
	    	if (gGameCameraStyle!= NORMAL_CAMERA_STYLE && !userReplay)
	    	{
				flip = FALSE;
			}
			break;
		
		case PG_QBPLAYCALL_NUM:
		    if (gGameCameraStyle != NORMAL_CAMERA_STYLE)
			{
				flip = True;
			}
			break;
		
		case PG_FALLONBELLY_NUM:
		case PG_FALLONBELLY2_NUM:
		case PG_FALLONBACK_NUM:
		case PG_DIVE_NUM:
		case PG_DIVEBALL_NUM:
			if ((frameNum > 2) && (gCamera.dir < 8))
				shadowFlipY = TRUE;
			if (frameNum > 2)
				shadowOn = FALSE;
			break;
		case PG_FGHOLDER_NUM:
			flip = False;
			break;
	}
		
	if (flip)
		SetPolySpriteClut(gPlayerSprite[plyrNum].clutFlipped->CLUT_ID,
									gPlayerSprite[plyrNum].sprite);
 	else
		SetPolySpriteClut(gPlayerSprite[plyrNum].clut->CLUT_ID,
									gPlayerSprite[plyrNum].sprite);

	if ( gCamera.dir < 8 )
		shadowFlip = flip;
	else
		shadowFlip = flip ^ TRUE;

	fnum2 = gAnimInfo[anim].frames[dir] + frameNum;
	FlipPolySpriteX(flip, gPlayerSprite[plyrNum].sprite);
	
	shadow = gPlayerSprite[plyrNum].shadow;
	SetPolySpriteOnOff(shadowOn, shadow);
	FlipPolySpriteX(shadowFlip, shadow);
//	FlipPolySpriteY(shadowFlipY, shadow);

	gPlayerSprite[plyrNum].compression = gAnimInfo[anim].compression;
	gPlayerSprite[plyrNum].pixels
				= GetMultiFileByNum((MultiFile_t *)base, fnum2);
	
	if ((ULONG)gPlayerSprite[plyrNum].pixels & 3)
	{
		#if (PC_DEBUG)
 		DebugPrintf(0,  // Anim pixels error
			"Anim pixels error anim.%d frm.%d dir.%d base.%X addr.%X\n",
 			anim,frameNum,dir,base,gPlayerSprite[plyrNum].pixels);
		#endif
 		gPlayerSprite[plyrNum].pixels = NULL;
 	}
// 	if (dir == 0 || dir == 8)
//		jnumOn = TRUE;
//	else
//		jnumOn = FALSE;
	
//	SetPolySpriteOnOff(jnumOn, gPlayerSprite[plyrNum].jnumSprite);
	SetJerseyNumberFrame(plyrNum, anim, dir, flip, frameNum);
}

/*-------------------------------------------------------------
  CalcWide ~ 
-------------------------------------------------------------*/
int CalcWide(int height, int weight)
{
	register int wide, y, x, working;
	
	limitRange(height, 68, (6*12) + 11);
	y = height - 68;
	wide = 0;
	working = 1;
	for (x = 0; x < 10 && working; x++)
	{
		if (weight < (gWeightTable[y][x] + WEIGHT_SUBTRACT))
		{
			wide = x;
			working = 0;
		}
	}
	if (working)
		wide = 9;
	return wide;
}

/*--------------------------------------------------------------------
  ResizePlayer ~ Set Player sprite size according to measurements
	tall = height in inches;
	wide = 0 - 10
--------------------------------------------------------------------*/
void ResizePlayer(int plyrNum, int wide, int tall)
{
	int width, height, numberHeight;

	 // debug
//	 if (plyrNum == 3)
//		tall = (5 * 12) + 5;
//	 if (plyrNum == 6)
//		tall = (6 * 12) + 11;

	height = DefaultPlayerHeight + (( (tall - 74) * 8)/4 );
//	width = DefaultPlayerWidth + ((wide - 5) * 8);
	width = DefaultPlayerWidth + ((wide - 5) * 10) + 30;
//	 width = DefaultPlayerWidth; //debug
//	 height = DefaultPlayerHeight; //debug
//	 width = 220; //debug
//	 height = 128; //debug
	numberHeight = SPRITE_GROUND_Y - (height / 2);
	ResizePolySprite(width, height, gPlayerSprite[plyrNum].sprite);
	SetPolySpritePos(0, SPRITE_GROUND_Y, 0, gPlayerSprite[plyrNum].sprite);
	SetPolySpritePos(0, numberHeight, 0, gPlayerSprite[plyrNum].jnumSprite);
//	 DebugPrintf(0, "Size #%d, %d %d\"  %d,%d\n", plyrNum, wide, tall, width, height);
}

/*-------------------------------------------------------------
  InitAllPlayerSizes ~
-------------------------------------------------------------*/
void InitAllPlayerSizes(void)
{
	register int i, weight, wide, tall;

	for (i = 0; i < NUM_PLAYERS_ON_FIELD; i++)
	{
		tall = GetPlayerHeight(i);
		weight = GetPlayerWeight(i);   
		wide = CalcWide(tall, weight);
//		 wide = dbPlayerSizes[i].width;  //debug
//		 tall = dbPlayerSizes[i].height; //debug
		if (tall == 0)
		{
			wide = (rand() % 4) + 5;
			tall = 5*12 + rand() % 24;
		}
		ResizePlayer(i, wide, tall);
	}
}

// ************** Player Marker Code *****************
/*-------------------------------------------------------------
  MovePlayerMarkerTo ~ Set Player marker position and display
-------------------------------------------------------------*/
void MovePlayerMarkerTo(int x, int y, int z, int on, int size, int num)
{
	SVECTOR vec1, vec2;
//	long p, flag, 
	int otz;
	int onScreen, drawX, drawY, wiggle, digit, shape, asymetric, vFlip;
//	static short sx[16], sy[16];
//	static int prevTriangleFlip = 0;
	PolySpritePtr sprite;

	if (gPlayerMarker[num].width)
	{
		sprite = gPlayerMarker[num].sprite;
//		SetPolySpriteOnOff(on, gPlayerMarker[num].sprite);
		SetPolySpriteOnOff(on, sprite);
		if (on)
		{
			digit = FALSE;
			shape = FALSE;
			asymetric = FALSE;
			onScreen = True;
			if (num >= PLAYER_MARKER_1 && num <= PLAYER_MARKER_8)
				digit = TRUE;
			switch (num)	
			{
			    case PLAYER_MARKER_TRIANGLE:
			    case PLAYER_MARKER_L:
			    case PLAYER_MARKER_R:
			    	asymetric = TRUE;  //Needs verital flip
				
				case PLAYER_MARKER_SQUARE:
			    case PLAYER_MARKER_CIRCLE:
			    	shape = TRUE;
			    	break;
			}
			
			if (digit || shape)
			{
				wiggle = abs(20 - ((GetVBlankCount() + (num << 3)) % 40));
				vec1.vx = x;
				vec1.vy = y;
				vec1.vz = z;
//					otz = RotTransPers(&vec1, (long*)&vec2, &p, &flag);
				otz = GetScreenPosFrom3d(&vec1, &vec2);
				drawX = vec2.vx;
				drawY = vec2.vy;
				if (drawX > 480)
				{
					drawX = 480;
					onScreen = False;
				}
				if (drawX < 10)
				{
					drawX = 10;
					onScreen = False;
				}
				if (drawY < 25)
				{
					drawY = 15;
					onScreen = False;
				}
				if (digit)
				{			
					if (drawY > 210)
					{
						drawY = 205;
						onScreen = False;
					}
				}
				else // shape
				{
					if (drawY > 250)
					{
						drawY = 205;
						onScreen = False;
					}
				}
				
//				if (num==PLAYER_MARKER_TRIANGLE /* && GsGetActiveBuff()==1 */)
				if (asymetric)
				{
			 		if ((gCamera.flags & CAMERA_FLIPZ_FLAG) && onScreen)
			 			vFlip = TRUE;
		 			else
		 				vFlip = FALSE;
	 				FlipPolySpriteX(vFlip, sprite);
	 				FlipPolySpriteY(vFlip, sprite);
		 		}
				
				if (onScreen)
				{
					SetPolySpriteFlags(PSPRITE_2D,False,sprite);
					if (shape)
					{
						if (size)
						{
							ResizePolySprite(size,size,sprite);
						}
						MakePolySpriteFloor(gPlayerMarker[num].sprite);
					}
					if (digit)								
						MovePolySpriteTo(x, y - wiggle, z, gPlayerMarker[num].sprite);
					else
						MovePolySpriteTo(x, y, z, gPlayerMarker[num].sprite);
				}
				else  // Not onScreen
				{
					SetPolySpriteFlags(PSPRITE_2D,True,gPlayerMarker[num].sprite);
					ResizePolySprite(PAD_NUMBER_HEIGHT /2,
							PAD_NUMBER_HEIGHT /2, gPlayerMarker[num].sprite);
					MovePolySpriteTo(drawX - 255, drawY - 120, 20,
									gPlayerMarker[num].sprite);
				}
			}
			else // if not Pad #
				MovePolySpriteTo(x, y, z, gPlayerMarker[num].sprite);
		} // end if on
	}
}

/*-------------------------------------------------------------
  ClearPlayerMarkers ~ Turn off All player markers
-------------------------------------------------------------*/
void ClearPlayerMarkers(void)
{
	register int i;

	for (i= 0; i < 16; i++)
		SetPolySpriteOnOff(0, gPlayerMarker[i].sprite);
}

/*-------------------------------------------------------------
  InitPlayerMarkers ~
-------------------------------------------------------------*/
void InitPlayerMarkers(void)
{
	GsIMAGE tim;
	char *ptr, *base;
	int i, baseU, baseV, u,v;
	RECT rec;
	vram_t *vram, *clut;
	USHORT tpageID;

	vram = gStadTexture[9].texture;
	clut = gStadTexture[9].clut;
	baseU = 128;
	baseV = 0;
//	rec.x = (baseU / 4) + vram->x;
//	rec.y = baseV + vram->y;
//	rec.w = tim.pw;
//	rec.h = tim.ph;
//	LoadImage(&rec, tim.pixel);
	// Set ID for 4 bit color
	tpageID = GetTPage(0, 0, vram->x,	vram->y);
//	gPlayerMarkerClut = register_CLUT( (ULONG*)tim.clut, VRAM_CLUT16);
	gPlayerMarkerClut = clut;

	Set0Transparent(gPlayerMarkerClut);
	for (i = 0; i < MAX_PLAYER_MARKERS; i++)
	{
		if (gPlayerMarker[i].width)
		{
			gPlayerMarker[i].sprite = CreatePolySprite(
					gPlayerMarker[i].width, gPlayerMarker[i].height );
			TexturePolySprite(tpageID, 
					baseU + gPlayerMarker[i].u,	baseV + gPlayerMarker[i].v, 
					gPlayerMarker[i].texWidth-1,gPlayerMarker[i].texHeight-1,
					gPlayerMarkerClut->CLUT_ID, gPlayerMarker[i].sprite);
			MovePlayerMarkerTo(0,0,0, 0, 0, i);
		}
	}
	MakePolySpriteFloor(gPlayerMarker[PLAYER_MARKER_CIRCLE].sprite);
	MakePolySpriteFloor(gPlayerMarker[PLAYER_MARKER_TRIANGLE].sprite);
	MakePolySpriteFloor(gPlayerMarker[PLAYER_MARKER_SQUARE].sprite);
	MakePolySpriteFloor(gPlayerMarker[PLAYER_MARKER_X].sprite);
	MakePolySpriteFloor(gPlayerMarker[PLAYER_MARKER_BALL_CARRIER].sprite);
	MakePolySpriteFloor(gPlayerMarker[PLAYER_MARKER_REPLAY].sprite);
	MakePolySpriteFloor(gPlayerMarker[PLAYER_MARKER_L].sprite);
	MakePolySpriteFloor(gPlayerMarker[PLAYER_MARKER_R].sprite);

	SetPolySpriteShade(70, 55, 8, 0, 0, 
					gPlayerMarker[PLAYER_MARKER_BALL_CARRIER].sprite);

//	FreeMem(gPlayerMarkerData);
//	gPlayerMarkerData = NULL;
//	DrawSync(0);
}

// ************** Ball Graphic Code *****************
/*-------------------------------------------------------------
  SetBallSpriteOnOff ~ Turn Ball Sprite on or off
-------------------------------------------------------------*/
void SetBallSpriteOnOff(int on)
{
	SetPolySpriteOnOff(on, gBallGraphic.sprite);
	SetPolySpriteOnOff(on, gBallGraphic.shadow);
}

/*-------------------------------------------------------------
  MoveBallSpriteTo ~ Set position of football
-------------------------------------------------------------*/
void MoveBallSpriteTo(int x, int y, int z)
{
	MovePolySpriteTo(x, y, z, gBallGraphic.sprite);
	MovePolySpriteTo(x, 10, z, gBallGraphic.shadow);
//	if (y < -1)
//		SetPolySpriteOnOff(TRUE, gBallGraphic.shadow);
//	else
//		SetPolySpriteOnOff(FALSE, gBallGraphic.shadow);
}

/*-------------------------------------------------------------
  SetBallNoPriority ~ Don't change ball priority on field goals
-------------------------------------------------------------*/
void SetBallNoPriority(BOOL on)
{
	SetPolySpriteFlags(PSPRITE_NO_PRIORITY, on, gBallGraphic.sprite);
}  /* End SetBallPriority() */


/*-------------------------------------------------------------
  SetBallFrame ~
-------------------------------------------------------------*/
void SetBallFrame(int advance)
{
	register USHORT flags;
	register int dir, flip, kicked, frame;
	char *ptr;
	GsIMAGE tim;
	ULONG anim;
//	static char dirs[8] = {1,1, 0, 0x11, 0x11, 0, 0, 0};
//	static char dirs[8] = {2, 0x11, 0, 0x1, 2, 0x11, 0, 1};
	static char dirs[8] = {2, 1, 0, 0x11, 2, 1, 0, 0x11};

	flags = BallFlags;
//	dir = dirs[BallDirection / 2] & 0xF;
	dir = dirs[((BallDirection - gCamera.dir + 16 + 12) / 2) & 0x7];
	if (dir & 0x10)
		flip = True;
	else
		flip = False;
	dir &= 3;

	if ( (flags & (SPIRAL_BALL_BFLAG | END_OVER_END_BFLAG)) 
		    	 && (BallZPosition > 1) )
	{
		if ( flip && (flags & BALL_PASSED_BFLAG) )
			gBallGraphic.frame -= (advance * 2);
		else
			gBallGraphic.frame += (advance * 2);
	}
	else
	{
		gBallGraphic.frame = 0;
	}
	if (flags & SPIRAL_BALL_BFLAG)
	{
		anim = (ULONG)gBallGraphic.PassedAnim[dir];
	}
	else if (flags & END_OVER_END_BFLAG)
	{
		anim = (ULONG)gBallGraphic.KickedAnim[dir];
	}
	else if (flags & STAND_BALL_BFLAG)
	{
		anim = (ULONG)gBallGraphic.KickedAnim[dir];
		gBallGraphic.frame = 3;
	}
	else
	{
		anim = (ULONG)gBallGraphic.PassedAnim[dir];
	}
	FlipPolySpriteX(flip, gBallGraphic.sprite);
	if (gBallGraphic.frame < 0)
		gBallGraphic.frame = (5*3)-1;
	gBallGraphic.frame %= 5*3;
	frame = gBallGraphic.frame/3;
	ptr = GetMultiFileByNum((MultiFile_t *)gBallGraphic.base, anim + frame);
	DecompressFrame(gAnimBuff, ptr);
	LoadImage(&gBallGraphic.vrec, (ULONG *)gAnimBuff);
//	GetTimInfo2(ptr, &tim);
//	LoadImage(&gBallGraphic.vrec, tim.pixel);
}

/*-------------------------------------------------------------
  InitBallGraphic ~
-------------------------------------------------------------*/
void InitBallGraphic(void)
{
	GsIMAGE tim;
	char *ptr, *base;
	int u,v;
	RECT rec;

	gBallGraphic.base = GetMultiFilePtr((MultiFile_t *)gAnimationsFile[0],"ballanim.frx");
	gBallGraphic.KickedAnim[0] = GetNumberInMultiFile((MultiFile_t *)gBallGraphic.base, "blea00.cmp");
	gBallGraphic.KickedAnim[1] = GetNumberInMultiFile((MultiFile_t *)gBallGraphic.base, "bleb00.cmp");
	gBallGraphic.KickedAnim[2] = GetNumberInMultiFile((MultiFile_t *)gBallGraphic.base, "blec00.cmp");
	gBallGraphic.PassedAnim[0] = GetNumberInMultiFile((MultiFile_t *)gBallGraphic.base, "blsa00.cmp");
	gBallGraphic.PassedAnim[1] = GetNumberInMultiFile((MultiFile_t *)gBallGraphic.base, "blsb00.cmp");
	gBallGraphic.PassedAnim[2] = GetNumberInMultiFile((MultiFile_t *)gBallGraphic.base, "blsc00.cmp");

//	gBallGraphic.KickedAnim =GetMultiFilePtr((MultiFile_t *)gAnimationsFile[0],"ballka.frx");
//	gBallGraphic.PassedAnim[0] =GetMultiFilePtr((MultiFile_t *)gAnimationsFile[0],"ballspa.frx");
//	gBallGraphic.PassedAnim[1] =GetMultiFilePtr((MultiFile_t *)gAnimationsFile[0],"ballspb.frx");
//	gBallGraphic.PassedAnim[2] =GetMultiFilePtr((MultiFile_t *)gAnimationsFile[0],"ballspc.frx");
//	ptr = GetMultiFileByNum((MultiFile_t *)gBallGraphic.KickedAnim, 1);
//	GetTimInfo2(ptr, &tim);
//	gBallGraphic.tpage = gAnimPage[2];
	gBallGraphic.tpage = gStadTexture[9].texture;

	// Set ID for 4 bit color
	gBallGraphic.tpageID = GetTPage(0, 0, gBallGraphic.tpage->x,
												gBallGraphic.tpage->y);
	u = 128;
	v = 0;
	gBallGraphic.vrec.x = (u / 4) + gBallGraphic.tpage->x;
	gBallGraphic.vrec.y = v + gBallGraphic.tpage->y;
	gBallGraphic.vrec.w = BALL_TEX_WIDTH /4;
	gBallGraphic.vrec.h = BALL_TEX_HEIGHT;
//	LoadImage(&gBallGraphic.vrec, tim.pixel);
	ptr = GetMultiFilePtr((MultiFile_t *)gBallGraphic.base, "ball.clt");
//	gBallGraphic.clut = register_CLUT( (ULONG*)tim.clut, VRAM_CLUT16);
//	gBallGraphic.clut = register_CLUT( (ULONG*)tim.clut, VRAM_CLUT256);
	gBallGraphic.clut = register_CLUT( (ULONG*)ptr, VRAM_CLUT256);
	Set0Transparent(gBallGraphic.clut);

	gBallGraphic.sprite = CreatePolySprite(DefaultPlayerWidth /4 ,
									DefaultPlayerHeight /4 );
	TexturePolySprite(gBallGraphic.tpageID, u,v,
				BALL_TEX_WIDTH -1, BALL_TEX_HEIGHT -1,
				gBallGraphic.clut->CLUT_ID, gBallGraphic.sprite);
	SetPolySpriteFlags(PSPRITE_HIGH_PRIORITY, True, gBallGraphic.sprite);
	SetPolySpriteFlags(PSPRITE_PLAYER, True, gBallGraphic.sprite);

	// -- Shadow --
	ptr = GetMultiFilePtr((MultiFile_t *)gAnimationsFile[0], "ballshad.tim");
	GetTimInfo2(ptr, &tim);
	gBallGraphic.shadowID = GetTPage(0, 0, gBallGraphic.tpage->x,
												gBallGraphic.tpage->y);
	u = 128+32;
	v = 0;
	rec.x = (u / 4) + gBallGraphic.tpage->x;
	rec.y = v + gBallGraphic.tpage->y;
	rec.w = BALL_TEX_WIDTH /4;
	rec.h = BALL_TEX_HEIGHT;
	LoadImage(&rec, tim.pixel);
//	gBallGraphic.clut = register_CLUT( (ULONG*)tim.clut, VRAM_CLUT16);
//	Set0Transparent(gBallGraphic.clut);

//	gBallGraphic.shadow = CreatePolySprite(DefaultPlayerWidth /4 ,
//									DefaultPlayerHeight /4 );
	gBallGraphic.shadow = CreatePolySprite(35, 40);
	TexturePolySprite(gBallGraphic.shadowID, u,v,
				BALL_TEX_WIDTH -1, BALL_TEX_HEIGHT -1,
//				gBallGraphic.clut->CLUT_ID, gBallGraphic.shadow);
				gShadowClut->CLUT_ID, gBallGraphic.shadow);
	MakePolySpriteShadow(0,100, gBallGraphic.shadow);

	MoveBallSpriteTo(0,0,0);
}

/*-------------------------------------------------------------
  IsHomeJerseyWhite ~ 
-------------------------------------------------------------*/
int IsHomeJerseyWhite(int homeNum, int awayNum)
{
	int white;
	
	white = FALSE;
		
//	if (homeNum == COWBOYS_NUM)
//		white = TRUE;
//	else if (awayNum == COWBOYS_NUM)	
//	{
//		switch (homeNum)		
//		{
//			case DOLPHINS_NUM:
//			case REDSKINS_NUM:
//				white = TRUE;
//		}
//	}

	switch(homeNum)
	{
		case COWBOYS_NUM:
		case DOLPHINS_NUM:
		case REDSKINS_NUM:
		case PANTHERS_NUM:
			white = TRUE;
			break;
	}
	
	return white;
}  /* End () */

/*-------------------------------------------------------------
  InitPlayerColors ~
-------------------------------------------------------------*/
void InitPlayerColors(int mode)
{
	int i, jnum, homeNum, awayNum, sleeves, gloves;
	USHORT u,v;
	int texNum, page;
	int team;
	char *ptr;
	vram_t *clut;

	gTmpClutPtr = (USHORT*)AllocMem(512);
	gTmpClutPtr2 = (USHORT*)AllocMem(512);

	if (GameOptions[GAME_OPTNS_WEATHER] == SNOWY_WEATHER_CONDITION)
		sleeves = True;
	else
		sleeves = False;
	if (gBaseClutPtr[0] == NULL) // If not loaded
	{
		homeNum = GameTeamData[0].TeamNumber;
		awayNum = GameTeamData[1].TeamNumber;
//		 homeNum = awayNum = 0;  //debug
		
		if (IsHomeJerseyWhite(homeNum, awayNum) )
		{
			GenClutFname(homeNum, 1, gTeamColorFname[0]);
			GenClutFname(awayNum, 0, gTeamColorFname[1]);
		}
		else
		{
			GenClutFname(homeNum, 0, gTeamColorFname[0]);
			GenClutFname(awayNum, 1, gTeamColorFname[1]);
		}

		ptr = LoadCDFile( gTeamColorFname[0]);
		CopyShorts((USHORT*)ptr, gTeamClutData1, 256);
		gBaseClutPtr[0] = (USHORT*)gTeamClutData1;
		*(USHORT*)gTeamClutData1 = 0;
		FreeMem(ptr);

		ptr = LoadCDFile( gTeamColorFname[1]);
		CopyShorts((USHORT*)ptr, gTeamClutData2, 256);
		gBaseClutPtr[1] = (USHORT*)gTeamClutData2;
		*(USHORT*)gTeamClutData2 = 0;
		FreeMem(ptr);

		ptr = LoadCDFile( "/colors/skin.clt" );
		CopyShorts((USHORT*)ptr, gSkinColorsBuff, 256);
		FreeMem(ptr);
	} // Not loaded

	for (i = 0; i < NUM_PLAYERS_ON_FIELD; i++)
	{
//		texNum = i % 8;
//		page = i / 8;
		team = i / 11;

		if (GameTeamData[0].OffDefFlag != ON_OFFENSE)
			team ^= 1;  // Swap team colors

		// Make CLUTs
		CopyShorts(gBaseClutPtr[team], gTmpClutPtr, 256);
		jnum = GetJerseyNum(i);

		 if (DebugShellVar[DEBUG_SHELL_JNUM].Value == 1)
		 	jnum = i;  // debug
//		 if (i == 0) jnum = 34; // debug

//		SetClutToNumber(gTmpClutPtr, team, jnum, False);
		SetPlayerColors(GetPlayerSkinColor(i), PlayerWearingGloves(i),
						sleeves, gTmpClutPtr);
		
		if (mode == 1)
		{
			if (gPlayerSprite[i].clut)
				free_VRam(gPlayerSprite[i].clut);
			gPlayerSprite[i].clut = alloc_VRam(VRAM_CLUT256, 0);
		}
		ReloadCLUT((ULONG*)gTmpClutPtr, gPlayerSprite[i].clut);
//		Set0Transparent(gPlayerSprite[i].clut);
		// Flipped CLUT
		CopyShorts(gTmpClutPtr, gTmpClutPtr2, 256);

//		SetClutToNumber(gTmpClutPtr2, team, jnum, True);  // flipped clut
		
		if (mode == 1)
		{
			if (gPlayerSprite[i].clutFlipped)
				free_VRam(gPlayerSprite[i].clutFlipped);
			gPlayerSprite[i].clutFlipped = alloc_VRam(VRAM_CLUT256, 0);
		}
		ReloadCLUT((ULONG*)gTmpClutPtr2, gPlayerSprite[i].clutFlipped);
//		Set0Transparent(gPlayerSprite[i].clutFlipped);
	}
	FreeMem(gTmpClutPtr2);
	FreeMem(gTmpClutPtr);
	SetJerseyNumberValues();
	if (mode == 0)
		SetJerseyNumberCluts();
}

/*-------------------------------------------------------------
  GetArcadeSectors ~ Get Sector numbers for Arcade files
-------------------------------------------------------------*/
void GetArcadeSectors(void)
{
	gAnimationSector[0] = GetFileSector("/anim/anims.mfl");
	gAnimSetSector[0] = GetFileSector("/anim/anims2.mfl");
	gAnimSetSector[1] = GetFileSector("/anim/anims22.mfl");
	gAnimSetSector[2] = GetFileSector("/anim/anims23.mfl");
	gAnimationSector[1] = GetFileSector("/anim/anims2.mfl");
	gReplayAnimSetSector[0][0] = GetFileSector("/anim/anims2r.mfl");
	gReplayAnimSetSector[1][0] = GetFileSector("/anim/anims2r2.mfl");
	gReplayAnimSetSector[2][0] = GetFileSector("/anim/anims2r3.mfl");
	
	gReplayAnimSetSector[0][1] = GetFileSector("/anim/animsrk.mfl");
	gReplayAnimSetSector[1][1] = GetFileSector("/anim/animsrk2.mfl");
	gReplayAnimSetSector[2][1] = GetFileSector("/anim/animsrk3.mfl");
	
	gReplayAnimSetSector[0][2] = GetFileSector("/anim/animsrp.mfl");
	gReplayAnimSetSector[1][2] = GetFileSector("/anim/animsrp2.mfl");
	gReplayAnimSetSector[2][2] = GetFileSector("/anim/animsrp3.mfl");
	gCoinAnimSector = GetFileSector("/anim/cointoss.mfl");

	gIsometricAnimSector[0] = GetFileSector("/anim/iso21.mfl");
	gIsometricAnimSector[1] = GetFileSector("/anim/iso22.mfl");
	gIsometricAnimSector[2] = GetFileSector("/anim/iso23.mfl");
	
	gSideViewAnimSector[0] = GetFileSector("/anim/side21.mfl");
	gSideViewAnimSector[1] = GetFileSector("/anim/side22.mfl");
	gSideViewAnimSector[2] = GetFileSector("/anim/side23.mfl");

	gRefereeNormalSector = GetFileSector("/anim/refanim.mfl");
	gRefereeIsometricSector = GetFileSector("/anim/refiso.mfl");
	gRefereeSideViewSector = GetFileSector("/anim/refside.mfl");
	
//	gPlayerMarkerSector = GetFileSector("/anim/plmark.mfl");
}

/*-------------------------------------------------------------
  SelectRandomAnimationSet ~
-------------------------------------------------------------*/
void SelectRandomAnimationSet(void)
{
	gAnimSetNum = GetRandom(3);
	#ifdef FORCE_ANIM_SET
		gAnimSetNum = FORCE_ANIM_SET;
		DebugPrintf(0, "FORCE ");
	#endif
//	 gAnimSetNum = 2;   // debug
	TDAnimLoadedNum = gAnimSetNum;
#if PC_DEBUG
//	DebugPrintf(0, "AnimSet: %d\n", gAnimSetNum);
#endif PC_DEBUG
}

/*-------------------------------------------------------------
  LoadArcadePerm ~
-------------------------------------------------------------*/
void LoadArcadePerm(void)
{
//	int i;

	if (gArcadePermLoaded)
		return;

	 Cmarker();
	 DumpAllocLoc();
	if (!gAnimationSector[0])
		GetArcadeSectors();
	AllocAIMemory();
	if (!gAnimBuff)
		gAnimBuff = (char*)AllocMem(64 * 128);  // big enough for play call 

	LoadRefereeAnim();
	if (gAnimationsFile[0] == NULL)
		gAnimationsFile[0] = LoadFileSectors(gAnimationSector[0]);
	gJnumPicPtr = GetMultiFilePtr(gAnimationsFile[0],"jnums.mfl");
	LoadLogos();
	gArcadePermLoaded = True;
	 Cmarker();
	 DumpAllocLoc();
}

/*-------------------------------------------------------------
  FreeArcadePerm ~
-------------------------------------------------------------*/
void FreeArcadePerm(void)
{
//	int i;

	 Cmarker();
	FreeLogos(True);
	ClearAllPolySprites();
	FreeFileData(gAnimationsFile[0]);
	gAnimationsFile[0] = NULL;
	FreeModel();
	gBaseClutPtr[0] = NULL;		// Team Cluts aren't loaded
	FreeRefereeAnim();
	FreeMem(gAnimBuff);
	gAnimBuff = NULL;
	FreeAIMemory();
	 DumpAllocLoc();
	gArcadePermLoaded = False;
	 Cmarker();
}

/*-------------------------------------------------------------
  LoadArcadeTmp ~
-------------------------------------------------------------*/
void LoadArcadeTemp(int wait)
{
	int i, type, loaded = False;

	if (gArcadeTempLoaded)
	{
		Cerror();
		return;
	}

//	 Cmarker();
//	 DebugPrintf(0, "Anim Sect %x\n", gAnimationSector[1]);
//	InitLogos();
	ClearLoadList();
//	 gShadowClut = MakeShadowClut();  // for now
	for (i = 0; i < 3; i++)  // Set Tpages for players  // for now
	{
		gAnimPage[i] = alloc_VRam(VRAM_TEXTURE, 0);
		SetVramTPage(gAnimPage[i], 1, 0);  /* 8 bit clut */
	}

//	if (!gAnimationSector[1])
//		gAnimationSector[1] = GetFileSector("/anim/anims2.mfl");
//	gAnimationsFile[1] = LoadFileSectors(gAnimationSector[1]);
//	gAnimationsFile[1] = AllocAddLoadList(gAnimationSector[1]);
	if (gInstantReplayMode)
	{
		if (gReplayAnimFlags & USED_KICKING_ANIM)
			type = 1;
		else if (gReplayAnimFlags & USED_PUNTING_ANIM)
			type = 2;
		else
			type = 0;
		
		 DebugPrintf(0, "RP anim %X %d %d\n", gReplayAnimFlags, gAnimSetNum, type);
		
		gAnimationsFile[1] = (MultiFile_t*)
				AllocAddLoadList(gReplayAnimSetSector[gAnimSetNum][type]);
	}
	else if (CoinActiveFlag)  // coin Toss
	{
		gAnimationsFile[1] = AllocAddLoadList(gCoinAnimSector);
		gCoinTossFile = gAnimationsFile[1];
	}
	else if (gGameCameraStyle != NORMAL_CAMERA_STYLE)
	{
		switch (gGameCameraStyle)
		{
			case ISOMETRIC_CAMERA_STYLE:
			case ISOMETRIC_LOW_CAMERA_STYLE:
				gAnimationsFile[1] 
					= AllocAddLoadList(gIsometricAnimSector[gAnimSetNum]);
				break;
			case SIDELINE_CAMERA_STYLE:
			case SIDELINE_LOW_CAMERA_STYLE:
				gAnimationsFile[1] 
					= AllocAddLoadList(gSideViewAnimSector[gAnimSetNum]);
				break;
		}
		
	}
	else
		gAnimationsFile[1] = AllocAddLoadList(gAnimSetSector[gAnimSetNum]);
	
	AllocCrowd();

//	if (!gPlayerMarkerSector)
//		gPlayerMarkerSector = GetFileSector("/anim/plmark.mfl");
//	gPlayerMarkerData = LoadFileSectors(gPlayerMarkerSector);
//	gPlayerMarkerData = AllocAddLoadList(gPlayerMarkerSector);

	StartLoadList();
	if (wait)
	{
		while(!loaded)
			loaded = CheckLoadList();
	}
	 Cmarker();
	gArcadeTempLoaded = True;
}

/*-------------------------------------------------------------
  FreeArcadeTemp ~
-------------------------------------------------------------*/
void FreeArcadeTemp(void)
{
	int i;

	if (!gArcadeTempLoaded)
		return;

	FreeCrowd();

	FreeMem(gAnimationsFile[1]);
	gAnimationsFile[1] = NULL;

	for (i = 0; i < MAX_PLAYER_ANIMS; i++)
	{
		if (gAnimNames[i].fileNum == 1)
		{
			gAnimInfo[gAnimNames[i].pgNum].base = NULL;
		}
	}
	 FreeStadiumVram();
	 FreeSidePeople();
	 Cmarker();
//	 DumpAllocLoc();
//	 DumpVramAlloc();
	Show5SecClock(False);
	gArcadeTempLoaded = False;
	AnimationsNotLoaded();    // Tell AI animations are not loaded
}

/*-------------------------------------------------------------
  InitReplayAnimInfo ~
-------------------------------------------------------------*/
void InitReplayAnimInfo(int num, char *basePtr, int file, void *jnumPosDir)
{
	register int i, frames;
	// Multiply for starting frame in each direction
	BYTE dirMult[16] =  { 0, 1,1,1, 2, 3,3,3, 4, 3,3,3, 2, 1,1,1 };
	BYTE aceMult[16] =  { 0, 0,0,1, 1, 1,1,2, 2, 2,2,1, 1, 1,1,0 };
	BYTE allMult[16] =  { 0, 7,7,7, 6, 5,5,5, 4, 3,3,3, 2, 1,1,1 };
	
//	BYTE dirMult[16] =  { 0, 0,1,1, 2, 2,3,3, 4, 4,3,3, 2, 2,1,1 };
//	BYTE aceMult[16] =  { 0, 0,0,1, 1, 1,1,2, 2, 2,2,1, 1, 1,1,0 };
//	BYTE allMult[16] =  { 0, 0,7,7, 6, 6,5,5, 4, 4,3,3, 2, 2,1,1 };

	register BYTE *ptr;

	if (file == 1)
		gAnimInfo[num].compression = 1;
	else
		gAnimInfo[num].compression = 2;
	gAnimInfo[num].base = basePtr;
	gAnimInfo[num].jnumPosDir = (void*)jnumPosDir;
	if (basePtr == NULL)
		return;

	ptr = dirMult;
	frames = NumFramesPerPlayerGraphic[num];
	switch (num)
	{
		case PG_FGHOLDER_NUM:
		case PG_PASSING_NUM:
			ptr = allMult;
			break;

		case PG_SPIKEBALL_NUM:
		case PG_QBPLAYCALL_NUM:
			for (i = 0; i < 16; i++)
			{
				gAnimInfo[num].frames[i] = 0;
				gAnimInfo[num].dirMult[i] = 0;
				
			}
			ptr = NULL;
			break;

		default:
			ptr = dirMult;
			break;
	}

	if (ptr)
	{
		for (i = 0; i < 16; i++)
		{
			gAnimInfo[num].frames[i] = *(ptr + i) * frames;
			gAnimInfo[num].dirMult[i] = *(ptr + i);
		}
	}
}

/*-------------------------------------------------------------
  InitAnimInfo ~ Setup Animation info
-------------------------------------------------------------*/
void InitAnimInfo(int num, char *basePtr, int file, void *jnumPosDir)
{
	register int i, frames;
	// Multiply for starting frame in each direction
	BYTE dirMult[16] =  { 0, 1,1,1, 2, 3,3,3, 4, 3,3,3, 2, 1,1,1 };
	BYTE passMult[16] = { 1, 0,0,0, 0, 0,0,0, 0, 2,2,2, 2,2,2,2 };
	BYTE abdMult[16] =  { 0, 1,1,1,2,2,2,2, 2,2,2,2,2,1,1,1 };
	BYTE abcMult[16] =  { 0, 1,1,1,2,2,2,2, 2,2,2,2,2,1,1,1 };
	BYTE cdeMult[16] =  { 0, 0,0,0, 0, 1,1,1, 2, 1,1,1, 0, 0,0,0 };
	BYTE abMult[16] =   { 0, 1,1,1, 1,1,1,1, 1,1,1,1,1, 1,1,1 };

	register BYTE *ptr;

	gAnimInfo[num].compression = 2;
	gAnimInfo[num].jnumPosDir = (void*)jnumPosDir;
	if (gInstantReplayMode && !gGameLoopFlags2.DoingAutoReplay)
	{
		InitReplayAnimInfo(num, basePtr, file, jnumPosDir);
		return;
	}

	gAnimInfo[num].base = basePtr;
	if (basePtr == NULL)
		return;

	ptr = dirMult;
	frames = NumFramesPerPlayerGraphic[num];
	switch (num)
	{
//		case PG_KICKING_NUM: // Only 2 directions used
//		case PG_BACKPEDAL_NUM:
//			for (i = 0; i < 16; i++)
//			{
//				gAnimInfo[num].frames[i] = (i / 8) * frames;
//			}
//			ptr = NULL;
//			break;

		case PG_PUNTING_NUM:  // Only 1 direction
		case PG_KICKING_NUM: 
		case PG_FGHOLDER_NUM:
		case PG_BACKPEDAL_NUM:
		case PG_BACKPEDAL_WBALL_NUM:
		case PG_QBPLAYCALL_NUM:
		case PG_DEFENSE_PRESNAP1_NUM:
		case PG_DEFENSE_PRESNAP2_NUM:
		case PG_DEFENSE_POSTTACKLE_NUM:
//		case PG_HANDOFF_NUM:
		case PG_SPIKEBALL_NUM:
		case PG_RECEIVERSTANCE_NUM:
		case PG_SIDESTEP_NUM:
			for (i = 0; i < 16; i++)
			{
				gAnimInfo[num].frames[i] = 0;
				gAnimInfo[num].dirMult[i] = 0;
			}
			ptr = NULL;
			break;

		case PG_PASSING_NUM:
			ptr = passMult;
			break;

		case PG_FALLONBELLY2_NUM:
		case PG_FALLONBACK_NUM:
		case PG_ROLL_NUM:
			ptr = abdMult;
			break;

		case PG_JUMPCATCH_NUM:
		case PG_RUNNINGCATCH_NUM:
			ptr = abcMult;
			break;

		case PG_STANDINGCATCH_NUM:
			ptr = cdeMult;
			break;

		case PG_RUNNINGHEADTURN_NUM:
			ptr = abMult;
			break;

		default:
			ptr = dirMult;
			break;
	}

	if (ptr)
	{
		for (i = 0; i < 16; i++)
		{
			gAnimInfo[num].frames[i] = *(ptr + i) * frames;
			gAnimInfo[num].dirMult[i] = *(ptr + i);
		}
	}
}


/*-------------------------------------------------------------
  InitPlayerAnim ~ Init Player Animations
-------------------------------------------------------------*/
void InitPlayerAnim(int perm)
{
	int i, c1 = 0, c2;
	USHORT u,v;
	short texNum, page, team;
	PolySpritePtr sprite, shadow;
//	GsIMAGE tim;
	char *ptr;
	vram_t *clut;
//	RECT rec;   /* Debug */
	static struct
	{
		short u, v;
	} TextureLocData[8] =
		{
			0,0,  0,64,  0,128,  0,192,
			64,0, 64,64, 64,128, 64,192
		};


	#if PC_DEBUG
	 if (PlayerLeftHanded(0))
		DebugPrintf(0, "Left Handed QB\n");
	#endif
		
//	gLeftHandedQB = PlayerLeftHanded(0);
//	 gLeftHandedQB = True;  //debug
	LoadArcadePerm();
	if (!gArcadeTempLoaded)
	{
		LoadStadiumTims(1);
		InitStadiumTextures();
		LoadScoreboardScreenPic(1);
		SetScoreboardScreenState(SCOREBOARD_SCREEN_PICTURE);
		FreeScoreboardScreenMfl();
		LoadArcadeTemp(1);
		ClearLoadList();
	}
//	 free_VRam(gShadowClut);  //??
	 gShadowClut = MakeShadowClut();  // debug

	ClearAllPolySprites();
//	CreatePolySprite(10,10);
	for (i = 0; i < NUM_PLAYERS_ON_FIELD; i++)	/* Create sprites */
	{
		gPlayerSprite[i].sprite =
				CreatePolySprite(DefaultPlayerWidth, DefaultPlayerHeight);
		SetPolySpriteFlags(PSPRITE_HIGH_PRIORITY, True, gPlayerSprite[i].sprite);
		SetPolySpriteFlags(PSPRITE_PLAYER, True, gPlayerSprite[i].sprite);

		gPlayerSprite[i].shadow =
				CreatePolySprite(DefaultPlayerWidth, DefaultPlayerHeight);
		gPlayerSprite[i].prevPixels = NULL;
	}

	InitPlayerMarkers();
	InitBallGraphic();
	InitArcGraphic();

	 DebugPrintf(0, "Anim %x %x\n", gAnimationsFile[0], gAnimationsFile[1]);

    // -- Set Animations --
	for (i = 0; i < MAX_PLAYER_ANIMS; i++)
	{
		gAnimInfo[i].base = NULL;
		gAnimInfo[i].jnumPosDir = NULL;
		gAnimInfo[i].compression = 2;
	}
	i = 0;
	while (gAnimNames[i].pgNum >= 0)
	{
		if (!CoinActiveFlag || (gAnimNames[i].fileNum == 0))
		{
			ptr = GetMultiFilePtr((MultiFile_t *)gAnimationsFile[ gAnimNames[i].fileNum ],
								 gAnimNames[i].name);
			InitAnimInfo(gAnimNames[i].pgNum, ptr, 
						gAnimNames[i].fileNum, gAnimNames[i].jnumPosDir);
//			 DebugPrintf(0, "%d %s, %x\n",
//			 		gAnimNames[i].pgNum, gAnimNames[i].name, ptr);
		}
		i++;
	}
         
	InitPlayerColors(1);
	DrawSync(0);
//	 MarkTime();
	/* Set up Texture pointers */
	for (i = 0; i < NUM_PLAYERS_ON_FIELD; i++)
	{
		texNum = i % 8;
		page = i / 8;
		team = i / 11;
		u = TextureLocData[texNum].u;
		v = TextureLocData[texNum].v;

		gPlayerSprite[i].texU = u;
		gPlayerSprite[i].texV = v;
		gPlayerSprite[i].texX = (u / 2) + gAnimPage[page]->x;
		gPlayerSprite[i].texY = v + gAnimPage[page]->y;

//		gPlayerSprite[i].sprite =
//				CreatePolySprite(DefaultPlayerWidth, DefaultPlayerHeight);
		sprite = gPlayerSprite[i].sprite;

		TexturePolySprite(gAnimPage[page]->tpage, u,v,
					FRAME_WIDTH -1, FRAME_HEIGHT -1,
					gPlayerSprite[i].clut->CLUT_ID, sprite);

#if MAKE_SHADOWS	/*-- Create shadow --*/
//		gPlayerSprite[i].shadow =
//				CreatePolySprite(DefaultPlayerWidth, DefaultPlayerHeight);
		shadow = gPlayerSprite[i].shadow;

		TexturePolySprite(gAnimPage[page]->tpage, u,v,
				FRAME_WIDTH -1, FRAME_HEIGHT -1,
				gShadowClut->CLUT_ID, shadow);
//		MakePolySpriteShadow(40,50, shadow);
//		MakePolySpriteShadow(80,180, shadow);
		MakePolySpriteShadow(80,110, shadow);
#endif
//		SetPlayerFrame(i, 0, 0, 0);
	}
	InitJerseyNumberSprites();
	
	InitAllPlayerSizes();
	InitRefereeAnim();
	InitScoreboard();
	InitLogoSprites();
	Show5SecClock(True);
//	ShowPlayClock(True);
	InitPlayClockSprite();
	AnimationsLoaded();   // Tell AI Animations are loaded
}

/*-------------------------------------------------------------
  UpdatePlayerTextures ~
-------------------------------------------------------------*/
void UpdatePlayerTextures(void)
{
	int i;
//	static int start = 0;
	register PlayerAnim_t *plyr;
	RECT rec;

	rec.w = FRAME_WIDTH / 2;		/* */
	rec.h = FRAME_HEIGHT;		/* */
	for (i = 0; i < NUM_PLAYERS_ON_FIELD; i++)    /*##*/
	{
		plyr = &gPlayerSprite[i];
		if (plyr->pixels != plyr->prevPixels)
		{
			if (plyr->pixels != NULL && (((ULONG)plyr->pixels & 3) == 0) )
			{
//				DecompressFrame(gAnimBuff, plyr->pixels);
				if (plyr->compression == 1)
					DecompressFrameBytes(gAnimBuff, plyr->pixels);
				else
					DecompressFrameShorts(gAnimBuff, plyr->pixels);
				rec.x = plyr->texX;		/* */
				rec.y = plyr->texY;		/* */
				LoadImage(&rec, (unsigned long *)gAnimBuff);    /*##*/
			}
			else
			{
#if PC_DEBUG
				DebugPrintf(0, "Addr error plyr %d %X anim%d frm%d dir%d\n ",
					i, plyr->pixels, PlayerGraphicState[i], 
					PlayerGraphicSequence[i], PlayerFacing[i]);
#endif PC_DEBUG
			}
			plyr->prevPixels = plyr->pixels;
		}
	}
	UpdateRefereeTextures();
}

#if 0
/*-------------------------------------------------------------
  UpdatePlayerTextures ~
-------------------------------------------------------------*/
void UpdatePlayerTextures2(void)
{
	int i, j, pnum;
/*	static int start = 0;    /*##*/
	PlayerAnim_t *plyr;
	char *buf;
	RECT rec;
	int tx, ty, t1,t2;

/*	start = 0;    /*##*/
/*	end = NUM_PLAYERS_ON_FIELD;    /*##*/

	rec.w = 64 / 2;		/* */
	rec.h = 255;		/* */
/*	DrawSync(0);    /*##*/
	pnum = 0;
/*	for (j = 0; j < 6; j++)    /*##*/
	for (j = 0; j < 1; j++)    /*##*/
	{
		rec.x = gAnimPage[j/2]->x + ((j % 2) * 32);		/* */
		rec.y = gAnimPage[j/2]->y;
		rec.w = 64 / 2;		/* */
		rec.h = 254;		/* */
		for (i = 0; i < 4, pnum < NUM_PLAYERS_ON_FIELD; i++)
		{
			buf = gAnimBuff + (i * FRAME_BYTES);
			plyr = &gPlayerSprite[pnum];
			DecompressFrame(buf, plyr->pixels);    /*##*/
	/*		DrawSync(0);    /*##*/
			pnum++;
		}
		tx = rec.x;
		ty = rec.y;
		DebugPrintf(0, "j=%d Pg(%d,%d) ", j,
					   gAnimPage[j/2]->x, gAnimPage[j/2]->y);
		DebugPrintf(0, "A:%x ", gAnimBuff);
		DebugPrintf(0, "Rec: %d,%d %d,%d\n", rec.x, rec.y, rec.w, rec.h);
		LoadImage(&rec, (unsigned long *)gAnimBuff);    /* 1/2 tpage*/    /*##*/
/*		DrawSync(0);    /*##*/
	}
	DebugPrintf(1, "loaded\n");
	DrawSync(0);    /*##*/
}
#endif //0

/**************** Referee Code ***************************/

/*-------------------------------------------------------------
  SetRefereeFrame ~
-------------------------------------------------------------*/
void SetRefereeFrame(int refNum, int anim, int dir, int frameNum)
{
	char *ptr;
	int frameGet, refAnim, flip, flipShadow, dir2;

//	 return;
	if (refNum < 3)
		refNum += REF_SLOT_NUM;
	if ((refNum > NUM_PLAYER_SLOTS) || (refNum < REF_SLOT_NUM) )
		return;

//	 anim = REF_RUNNING_NUM;  // Debug
	if (anim == 0)
		anim = REF_RUNNING_NUM;
	if (anim == REF_GOHOME_NUM)
    {
		anim = REF_TURNING_NUM;
//		return;
	}
		
	refAnim = anim - FIRST_REF_ANIM; // adjust for referee
//	 frameNum = 0;
	if (frameNum < 0)
		frameNum = 0;
	frameNum %= NumFramesPerPlayerGraphic[anim];
	if (frameNum > (int)NumFramesPerPlayerGraphic[anim]
	    	|| anim > MAX_PG_GRAPHICS || anim < FIRST_REF_ANIM)
   	{
   		DebugPrintf(0, "Ref Anim Error: #%d frm %d\n", anim, frameNum);
   		return;
   	}

	dir2 = (12 + dir + gCamera.dir) & 15;			
	frameGet = (int)gRefGraphic.animStart[refAnim][dir2] + frameNum;
	if (frameGet == -1)
		frameGet = (int)gRefGraphic.animStart[REF_RUNNING_NUM][0] + frameNum;

	gPlayerSprite[refNum].pixels =
			GetMultiFileByNum((MultiFile_t *)gRefGraphic.filePtr, frameGet);

//	 if (JpadData[0] & JPADselect)
//	 	DebugPrintf(0, "refdir %d %d %d\n", dir, gCamera.dir, dir2);
	if (dir <= 8)
		flipShadow = TRUE;
	else
		flipShadow = FALSE;

	if (dir2 < 8)
		flip = TRUE;
	else
		flip = FALSE;
//	flipShadow = flip;
//	if (gCamera.dir > 8)
//	{
//		flip = flip ^ TRUE;
//	}

 	
//	 if (JpadData[0] & JPADselect)
//	 	DebugPrintf(0, "refdir %d %d %d  %d\n", dir, gCamera.dir, dir2, flip);
					
//	if (gCamera.flags & CAMERA_FLIPZ_FLAG)
//		flip = flip ^ TRUE;
//	if (gCurrentPlayType == KICKOFF_PLAY)
//		flip = flip ^ TRUE;

//	if (refAnim == (FIRST_REF_ANIM + REF_GOHOME_NUM) ) // Gohome is backwards
//		 FlipPolySpriteX(TRUE, gPlayerSprite[refNum].sprite); // For demo
//	else
//		 FlipPolySpriteX(FALSE, gPlayerSprite[refNum].sprite); // For demo

	 FlipPolySpriteX(flip, gPlayerSprite[refNum].sprite);
	 FlipPolySpriteX(flipShadow, gPlayerSprite[refNum].shadow);
}


/*-------------------------------------------------------------
  LoadRefereeAnim ~ 
-------------------------------------------------------------*/
void LoadRefereeAnim(void)
{
	ULONG sector;
	
	 Cmarker();
	if (gRefereeFilePtr == NULL)
	{
		gRefereeFilePtr = (char*)AllocMem(REFEREE_ANIM_BYTES);
	}
	switch(gGameCameraStyle)
	{
	    case ISOMETRIC_CAMERA_STYLE:
	    case ISOMETRIC_LOW_CAMERA_STYLE:
	    	sector = gRefereeIsometricSector;
	    	break;
	    case SIDELINE_CAMERA_STYLE:
	    case SIDELINE_LOW_CAMERA_STYLE:
	    	sector = gRefereeSideViewSector;
	    	break;
	    default:  // NORMAL_CAMERA_STYLE
	    	sector = gRefereeNormalSector;
	    	break;
	}
	LoadFileSectorsTo(sector, gRefereeFilePtr);
//	 DebugPrintf(0, "Referee Anims:%X\n", gRefereeFilePtr);
}

/*-------------------------------------------------------------
  FreeRefereeAnim ~ 
-------------------------------------------------------------*/
void FreeRefereeAnim(void)
{								   
	 Cmarker();
	if (gRefereeFilePtr != NULL)
		FreeMem(gRefereeFilePtr);
	gRefereeFilePtr = NULL;
}

/*-------------------------------------------------------------
  InitRefereeAnim ~
-------------------------------------------------------------*/
void InitRefereeAnim(void)
{
	char *filePtr, *ptr;
	int i, j;
	USHORT u,v;
	int pageX, pageY;
	PolySpritePtr sprite, shadow;
	static BYTE dirMult[16] =  { 0, 1,1,1, 2, 3,3,3, 4, 3,3,3, 2, 1,1,1 };

	static struct
	{
		int num;
		char *name;
	} refAnimNames[] =
		{
			{ REF_RUNNING_NUM, "rfrunX00.cmp", },
			{REF_1STDOWN_NUM, "rffstx00.cmp" },
			{REF_PERSONAL_FOUL_NUM, "rfprf00.cmp" },
			{REF_STANDING_NUM, "rfstda00.cmp" },
//			{REF_CLIPPING_NUM, "rfclp00.cmp" },
			{REF_ROUGHING_KICKER_NUM, "rfrgk00.cmp" },
			{REF_ROUGHING_PASSER_NUM, "rfrgp00.cmp" },
			{REF_TOUCHDOWN_NUM, "rftch00.cmp" },
			{REF_SAFETY_NUM, "rfsaf00.cmp" },
			{REF_PENALTY_REFUSED_NUM, "rfpnl00.cmp" },
			{REF_OFFSIDE_NUM, "rfoff00.cmp" },
			{REF_INTERFERENCE_NUM, "rfint00.cmp" },
//			{REF_INELLIGIBLE_RCVR_NUM, "rfinl00.cmp" },
//			{REF_ILLEGAL_HANDS_NUM, "rfilh00.cmp" },
			{REF_FALSE_START_NUM, "rffls00.cmp" },
			{REF_DELAY_NUM, "rfdly00.cmp" },
			{REF_TURNING_NUM, "rftrna00.cmp" },
//			{REF_GOHOME_NUM, "rfhom00.cmp" },
			{ -1, NULL }
		};


//	 return;   // debug
//	filePtr = GetMultiFilePtr((MultiFile_t *)gAnimationsFile[0], "refanim.mfl");
	filePtr = gRefereeFilePtr;
	gRefGraphic.filePtr = filePtr;
    
    // -- Set Animations --
	for (i = 0; i < MAX_REF_ANIMS; i++)
		gRefGraphic.animStart[i][0] = -1;
	i = 0;
	while (refAnimNames[i].name != NULL)
	{
		int baseNum, d, num;
		
		baseNum = GetNumberInMultiFile((MultiFile_t *)filePtr, refAnimNames[i].name);
		num = refAnimNames[i].num - FIRST_REF_ANIM;
		
		switch(refAnimNames[i].num)
		{
			case REF_TURNING_NUM:
			case REF_STANDING_NUM:
				for (j = 0; j < 16; j++)
				{
					gRefGraphic.animStart[num][j] = baseNum + dirMult[j];
				}
				break;
			default:	// 1 Direction only
				for (j = 0; j < 16; j++)
				{
					gRefGraphic.animStart[num][j] = baseNum;
				}
				break;
		}
		i++;
	}
	
	pageX = gAnimPage[2]->x;
	pageY = gAnimPage[2]->y;
	gRefGraphic.tpage = GetTPage(0, 0, pageX, pageY);
	ptr = GetMultiFilePtr((MultiFile_t *)filePtr, "refclut.clt");
	gRefGraphic.clut = register_CLUT((ULONG*)ptr, VRAM_CLUT256);
//	Set0Transparent(gRefGraphic.clut);

//	gPlayerSprite[REF_SLOT_NUM].texU = 128;
//	gPlayerSprite[REF_SLOT_NUM].texV = 128;

//	gPlayerSprite[REF_SLOT_NUM + 1].texU = 128;
//	gPlayerSprite[REF_SLOT_NUM + 1].texV = 128 + 64;

	i = REF_SLOT_NUM;
//	for (i = REF_SLOT_NUM; i < NUM_PLAYER_SLOTS -1; i++)	// Create sprites */
//	{
//		u = gPlayerSprite[i].texU;
//		v = gPlayerSprite[i].texV;
		
		u = v = 128;
		gPlayerSprite[i].texX = (u / 4) + pageX;
		gPlayerSprite[i].texY = v + pageY;

		gPlayerSprite[i].sprite =
				CreatePolySprite(DefaultPlayerWidth, DefaultPlayerHeight);
		sprite = gPlayerSprite[i].sprite;
		SetPolySpriteFlags(PSPRITE_HIGH_PRIORITY, True, sprite);
		SetPolySpriteFlags(PSPRITE_PLAYER, True, sprite);
		
		TexturePolySprite(gRefGraphic.tpage, u,v,
					FRAME_WIDTH -1, FRAME_HEIGHT -1,
					gRefGraphic.clut->CLUT_ID, sprite);

#if MAKE_SHADOWS	/*-- Create shadow --*/
		gPlayerSprite[i].shadow =
				CreatePolySprite(DefaultPlayerWidth, DefaultPlayerHeight);
		shadow = gPlayerSprite[i].shadow;

		TexturePolySprite(gRefGraphic.tpage, u,v,
				FRAME_WIDTH -1, FRAME_HEIGHT -1,
				gShadowClut->CLUT_ID, shadow);
//		MakePolySpriteShadow(40,50, shadow);
		MakePolySpriteShadow(80,180, shadow);
#endif
		SetRefereeFrame(i,0,0,0);
//	}
}

/*-------------------------------------------------------------
  UpdateRefereeTextures ~
-------------------------------------------------------------*/
void UpdateRefereeTextures(void)
{
	int i;
//	static int start = 0;
	register PlayerAnim_t *plyr;
	RECT rec;

//	 return;  // DEbug
	rec.w = FRAME_WIDTH / 4;	// 16 colors
	rec.h = FRAME_HEIGHT;
	for (i = REF_SLOT_NUM; i < NUM_PLAYER_SLOTS; i++)
	{
		plyr = &gPlayerSprite[i];
		if (plyr && plyr->pixels != NULL && (((ULONG)plyr->pixels & 3) == 0))
		{
//			DecompressFrame(gAnimBuff, plyr->pixels);
//			DecompressFrameShorts(gAnimBuff, plyr->pixels);
			DecompressFrameBytes(gAnimBuff, plyr->pixels);
			rec.x = plyr->texX;		/* */
			rec.y = plyr->texY;		/* */
			LoadImage(&rec, (unsigned long *)gAnimBuff);    /*##*/
		}
		else
		{
#if PC_DEBUG
//			DebugPrintf(0, "Anim Addr error ref %d  %X\n", i, plyr->pixels);
#endif
		}
	}

}