/***************************************************************
	Player Animation code

	Copyright 1994 GameTek
	All rights reserved.

	By Christopher G. Deel & Mike Heilemann
****************************************************************/

#ifndef _PLYRANIM_H
#define _PLYRANIM_H 1

#define COWBOYS_NUM 25
#define DOLPHINS_NUM 12
#define REDSKINS_NUM 27
#define PANTHERS_NUM 18

// --- Default sprite width & height ---
#define DefaultPlayerWidth	192
#define DefaultPlayerHeight	128

//#define DefaultPlayerHeight	192
//#define DefaultPlayerWidth	250

//--- Player CLUTS
#define CLUT_SEG_SIZE	6

#define CLUT_HELMET		1
//#define CLUT_HEAD		33
#define CLUT_ARMS		73
#define CLUT_HANDS		89
#define CLUT_SEGMENTS	137
#define CLUT_SEG_ON		CLUT_SEGMENTS
#define CLUT_SEG_OFF	CLUT_SEG_ON + CLUT_SEG_SIZE
// --- New Player Cluts ---
#define CLUT_HEAD		32
#define CLUT_JNUM		140

extern vram_t *gAnimPage[3];
 
typedef struct
{
	char *pixels;    	/* pointer to compressed pixel data */
	char *prevPixels;
	short texX, texY;	/* Texture position in Vram */
	short texU, texV;	/* Texture position inside Tpage */
	PolySpritePtr_t sprite;	
	PolySpritePtr_t shadow;	
	PolySpritePtr_t jnumSprite;	
	int jnumX, jnumY;
	vram_t *clut;
	vram_t *clutFlipped;
	int compression;
} PlayerAnim_t;

extern PlayerAnim_t	gPlayerSprite[];

typedef struct
{
	char *base;		/* pointer to Anim data */
//	int numFrames;	/* frames per anim */
	int compression;
	short frames[16];	/* Frame #s */
	UBYTE dirMult[16];
	void *jnumPosDir;	// This is really JnumPos_t *jnumPosDir (jnumber.h)
} AnimInfo_t;

extern AnimInfo_t gAnimInfo[];

extern MultiFile_t *gAnimationsFile[2];

extern char *gAnimBuff;
extern int gAnimSetNum;
extern char *gJnumPicPtr;
extern USHORT gTeamClutData1[256], gTeamClutData2[256], gSkinColorsBuff[256];

// ---- Player Marker Info ----
typedef struct
{
	PolySpritePtr sprite;
	short u,v, texWidth, texHeight, width, height, clutNum;
//	short screenX, screenY;
} PlayerMarker_t;

extern PlayerMarker_t gPlayerMarker[];

enum
{
	PLAYER_MARKER_BALL_CARRIER = 2,
	PLAYER_MARKER_EMPTY3,
	PLAYER_MARKER_1,
	PLAYER_MARKER_2,
	PLAYER_MARKER_3,
	PLAYER_MARKER_4,
	PLAYER_MARKER_5,
	PLAYER_MARKER_6,
	PLAYER_MARKER_7,
	PLAYER_MARKER_8,
	PLAYER_MARKER_SQUARE,
	PLAYER_MARKER_TRIANGLE,
	PLAYER_MARKER_CIRCLE,
	PLAYER_MARKER_X,
	PLAYER_MARKER_REPLAY,
	PLAYER_MARKER_L,
	PLAYER_MARKER_R,
	MAX_PLAYER_MARKERS
};


/* PLYRANIM.C */
int GetScreenPosFrom3d(SVECTOR *vec1, SVECTOR *vec2);

void LoadRefereeAnim(void);
void FreeRefereeAnim(void);

vram_t *MakeShadowClut ( void );
USHORT *GetPlayerClutPtr ( int team , int player );
void SetClutSection ( USHORT *new , USHORT *clut , int start , int count );
void SetClutToNumber ( USHORT *clut , int team , int number , int flip );
void MovePlayerSpriteTo ( int x , int y , int z , int plyrNum );
void GetPlayerSpriteLocation ( int pnum , SVECTOR *vec );
char * GetFramePtr(int anim, int dir, int frameNum);
void SetPlayerFrame ( int plyrNum , int anim , int dir , int frameNum );
void ResizePlayer ( int plyrNum , int wide , int tall );
void InitAllPlayerSizes ( void );
void MovePlayerMarkerTo(int x, int y, int z, int on, int size, int num);
void ClearPlayerMarkers ( void );
void InitPlayerMarkers ( void );
void SetBallSpriteOnOff ( int on );
void MoveBallSpriteTo ( int x , int y , int z );
void SetBallFrame ( int advance );
void InitBallGraphic ( void );
void InitPlayerColors ( int mode );
void GetArcadeSectors ( void );
void LoadArcadePerm ( void );
void LoadArcadeTemp ( int wait );
void FreeArcadeTemp ( void );
void FreeArcadePerm ( void );
void InitPlayerAnim ( int perm );
void UpdatePlayerTextures ( void );
void UpdatePlayerTextures2 ( void );
void InitRefereeAnim ( void );
void SetRefereeFrame ( int refNum , int anim , int dir , int frameNum );
void UpdateRefereeTextures ( void );

void InitAnimInfo(int num, char *basePtr, int file, void *jnumPosDir);

#endif
