/*--------+--------+--------+--------+--------+--------+--------+-------+---*/
/*                                                                          */
/*            (c) Copyright Sony Computer Entertainment 1994                */
/*                        All Rights Reserved                               */
/*                                                                          */
/*                                                                          */
/*  Filename:       profile.c                                               */
/*  Author:         Andy Fitter                                             */
/*  Created:        8/8/1994                                                */
/*                                                                          */
/*  Description:                                                            */
/*  Profiling code - uses hsync root counter to time code                   */
/*                                                                          */
/*  Change History:                                                         */
/*                                                                          */
/*  Author    Date     Change                                               */
/*  ------    ----     ------                                               */
/*  AJF       18/8/94  Made it work, start of timing work                   */
/*  AJF       19/8/94  Continue profiling code, nearly done now...          */
/*  AJF       23/9/94  Added Video mode detection code (ProfileDispMode())  */
/*	                    n.b.  Only works with lib2.0 or newer                */
/*  MDG		  5/12/94  Added extra count tied to DrawSyncCallback for 	    */
/*                     timing drawing only.  Must ProfileStartCount soon    */
/*                     after DrawOTag for this count to be acurate.         */
/*                     New routines : ProfileCallBack (local), ProfileRead	 */
/*                     and ProfileAddDrawOT.  Old routines will still work. */
/*                     Use ProfileReadCount instead of ProfileStopCount.    */
/*                     Use ProfileAddDrawOT to add drawing timing bar.      */
/*                                                                          */
/*--------+--------+--------+--------+--------+--------+--------+-------+---*/

/**************************************************************************
	Chris Deel 4-2-96	Modified to enable and disable 
			Added room for more bars and made more general
			Removed DrawSync Callback (not needed);
***************************************************************************/

#include <sys\types.h>
#include <libgte.h>
#include <libgpu.h>
#include <stdio.h>
#include <asm.h>
#include <kernel.h>
#include <libsn.h>
#include <libetc.h>

#include "profile.h"

#if PROFILE
/*-------- #defines and macros ------+--------+--------+-------+---*/

#define NUM_BARS 20

#define	TOP		30
#define	LEFT		25
//#define 	HEIGHT	10
#define 	HEIGHT	5
#define	GRIDSIZE	65
//#define	GRIDSIZE	30

#define	DRAW_TOP	35

//#define	MODE_NTSC 	1
//#define	MODE_PAL		2
#define	HBL_MID_VAL	((525+625)/4)

typedef struct
{
	short xgrid;
	char scalefactor;
	char nummarks;
} SCALE;

/*-------- Globals and Statics ------+--------+--------+--------+-------+---*/
static unsigned long EHbl;
static short divisor;
static short nummarks;
//static short finalCount;
static short  GscaleFactor;
static POLY_F4	bargraph[NUM_BARS][2];
static POLY_F3	markers[9][2];
//static POLY_F3	markers2[9];
static int barvalue[NUM_BARS];
int gProfilerEnabled = 1;

//static short drawCount;
//static short pad1;

static SCALE scales[] =
{
	{ 260, 0, 2 },
	{ 130, 1, 3 },
	{ 65,  2, 5	}
};

/*-------- Function definitions -----+--------+--------+--------+-------+---*/

/*-------------------------------------------------------------
  EnableProfiler ~ 
-------------------------------------------------------------*/
void EnableProfiler(int on)
{
	gProfilerEnabled = on;
}

/*-------------------------------------------------------------
-------------------------------------------------------------*/
void ProfileInit(int scale)
{
 	char count, i, db;
 	int top;

	int grid = scales[scale].xgrid;
	nummarks = scales[scale].nummarks;
	divisor = scales[scale].scalefactor;

	EnterCriticalSection();
	EHbl = OpenEvent(RCntCNT1, EvSpINT, EvMdNOINTR, NULL);
	EnableEvent(EHbl);
	SetRCnt(RCntCNT1, 64000, RCntMdINTR);
	ExitCriticalSection();

	for (count = 0; count < nummarks; count++ )
	{
		for (db = 0; db < 1; db++)
		{
		  	SetPolyF3(&markers[count][db]);
			setRGB0(&markers[count][db],0x80,0x80,0xFF);
			setXY3(&markers[count][db],
				LEFT+count*grid-4,TOP-10,
				LEFT+count*grid+4,TOP-10,
				LEFT+count*grid,TOP);
		}
	}

	for (i = 0; i < NUM_BARS; i++)
	{
		for (db = 0; db < 1; db++)
		{
			SetPolyF4(&bargraph[i][db]);
			setRGB0(&bargraph[i][db],0xFF,0xFF,0xFF);
//			setRGB1(&bargraph[i][db],0xFF,0,   0xFF);
//			setRGB2(&bargraph[i][db],0xFF,0xFF,0xFF);
//			setRGB3(&bargraph[i][db],0xFF,0,   0);
			if ((i % 3) == 0)
				setRGB0(&bargraph[i][db],0xFF,0x80,0x80);
			top = TOP + i * (HEIGHT + 2);
			setXY4(&bargraph[i][db], LEFT, top, 100, top, 
					LEFT, top + HEIGHT, 100, top + HEIGHT);
		}
	}
}


/*--------+--------+--------+--------
------+--------+--------+-------+---*/
void ProfileStartCount()
{
	ResetRCnt(RCntCNT1);
	StartRCnt(RCntCNT1);
}

/*--------+--------+--------+--------+--------+--------+--------+-------+---*/
/*--------+--------+--------+--------+--------+--------+--------+-------+---*/
short ProfileStopCount()
{
	StopRCnt(RCntCNT1);
//	finalCount = GetRCnt(RCntCNT1)>>divisor;
//	return finalCount;
}

/*-------------------------------------------------------------
  ReadProfileValue ~ CGD
-------------------------------------------------------------*/
int ReadProfileValue(int barnum)
{
	register int val;
	
	val = GetRCnt(RCntCNT1)>>divisor;
	barvalue[barnum] = val;
	return val;
}

/*-------------------------------------------------------------
  DrawProfileBars() ~ 
-------------------------------------------------------------*/
void DrawProfileBars(unsigned long * ot, int db)
{
	char count;
	
	if (gProfilerEnabled)
	{
		for (count = 0; count < nummarks; count++ )
		{
			AddPrim(ot,&markers[count][db]);
		}
	
		for (count = 0; count < NUM_BARS; count++)
		{
			bargraph[count][db].x1 = barvalue[count] + LEFT;
			bargraph[count][db].x3 = barvalue[count] + LEFT;
			AddPrim(ot, &bargraph[count][db]);
			barvalue[count] = 0;
		}
	}
}

/*--------+--------+--------+--------+--------+--------+--------+-------+---*/
/*--------+--------+--------+--------+--------+--------+--------+-------+---*/
int ProfileDispMode(void)
{
	int count;

	VSync(0);
	ProfileInit(0);
	ProfileStartCount();
	VSync(0);
	count = ProfileStopCount();

//	if (count < HBL_MID_VAL)
//		return MODE_NTSC;
//	else
//		return MODE_PAL;
}

#else // No Profile
void ProfileInit ( int scale )
{
}
void ProfileStartCount ( void )
{
}
short ProfileStopCount ( void )
{
}
int ReadProfileValue ( int barnum )
{
}
void DrawProfileBars ( unsigned long *ot , int db)
{
}
int ProfileDispMode ( void )
{
}


#endif //PROFILE