/**************************************************
	Polygon based Sprite handler

	Copyright 1994 GameTek
	By Christopher Deel
*************************************************/

#include <gtemac.h>
#include <inline.h>
#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include "prjhdrs.h"	/* Project header files */
#include "camera.h"
//#include "arcade.h"

#include "fbequ.h"
#include "fbram.h"

#define TESTING 	False	/* Testing this module */
#define TEST_NO3D 	False	/* For Testing Sprite Animation */

//#define MaxPolySprites 880
#define MaxPolySprites 842
//#define MaxPolySprites 910

#define ALLOC_SPRITE_BUFFER 1

#if ALLOC_SPRITE_BUFFER 
//	PolySpritePtr *gPSpritePtrs;
//	PolySpritePtr gPSpritePtrs[MaxPolySprites];
	
	PolySpriteType *gPolySpriteData = NULL;
	#define SpriteNumPointerMacro(num) \
		((PolySpritePtr)((ULONG)gPolySpriteData + (num * sizeof(PolySpriteType))))
#else
//	PolySpritePtr gPSpritePtrs[MaxPolySprites];
	PolySpriteType gPolySpriteData[MaxPolySprites];
#endif  //ALLOC_SPRITE_BUFFER

UBYTE gSpriteDB = 0, gSpritesInitalised = False;
int MaxSpritesEverUsed = 0;

/*-------------------------------------------------------------
  AllocSpriteMemory ~ 
-------------------------------------------------------------*/
void AllocSpriteMemory(void)
{
#if ALLOC_SPRITE_BUFFER	
	if (gPolySpriteData == NULL)
	{
		gPolySpriteData = 
			(PolySpritePtr)AllocMem(MaxPolySprites * sizeof(PolySpriteType));
//		gPSpritePtrs = (PolySpritePtr*)AllocMem(MaxPolySprites * 4);
		 DebugPrintf(0, "Sprites at:%X, %dk\n", gPolySpriteData, 
		 		(MaxPolySprites * sizeof(PolySpriteType))/1024);
	}
#endif	//ALLOC_SPRITE_BUFFER	
}  /* End AllocSpriteMemory() */

/*-------------------------------------------------------------
  FreeSpriteMemory ~ 
-------------------------------------------------------------*/
void FreeSpriteMemory(void)
{
#if ALLOC_SPRITE_BUFFER	
	if (gPolySpriteData != NULL)
	{
		FreeMem(gPolySpriteData);
//		FreeMem(gPSpritePtrs);
		gPolySpriteData = NULL;
	}
#endif  //ALLOC_SPRITE_BUFFER	
}  /* End FreeSpriteMemory() */

/*~~-----------------------------
    Initialize Polygon Sprites
-------------------------------*/
void InitPolySprites(void)
{
	register int n;

//	AllocSpriteMemory();
	if (gSpritesInitalised)
		ClearAllPolySprites();
//	for (n=0; n < MaxPolySprites; n++)
//	{
//		gPSpritePtrs[n] = NULL;
//	}
	gSpritesInitalised = True;
}

/*-------------------------------------------------------------
  SetPolySpritePos ~ Set Initial position of poly sprite
-------------------------------------------------------------*/
void SetPolySpritePos(int x, int y, int z, PolySpritePtr sprite)
{
	register int x1,y1,z1, x2,y2,z2;

	if (!sprite)  return;  // Error
		
	x1 = x - (sprite->width / 2);
	y2 = y;
	z1 = z;
	x2 = x1 + sprite->width;
	y1 = y2 - sprite->height;
	z2 = z1;
	setVector(&(sprite->ipos[0]), x1,y1,z1);
	setVector(&(sprite->ipos[1]), x2,y1,z1);
	setVector(&(sprite->ipos[2]), x1,y2,z2);
	setVector(&(sprite->ipos[3]), x2,y2,z2);
}

/*~~-------------------------
    Create a poly sprite
---------------------------*/
PolySpritePtr CreatePolySprite(int width, int height)
{
	register int i, db, spnum;
	BOOL ok = False;
	register PolySpritePtr sprite = NULL;
	register POLY_FT4 *spoly;

	if (gPolySpriteData == NULL)
	{
		Cerror();
		return NULL;
	}
	
	for (i = 0; i < MaxPolySprites && !ok; i++)
	{
//		if (gPSpritePtrs[i] == NULL)
		sprite = SpriteNumPointerMacro(i);
		if (!(sprite->flags & PSPRITE_USED))
		{
			ok = True;
			spnum = i;
		}
	}
	if (ok)
	{
		MaxSpritesEverUsed = max(MaxSpritesEverUsed, spnum); // for test
//		sprite = (PolySpritePtr)AllocMem(sizeof(PolySpriteType));

	#if ALLOC_SPRITE_BUFFER		
		sprite = SpriteNumPointerMacro(spnum);
	#else	
		sprite = &gPolySpriteData[spnum];
	#endif //ALLOC_SPRITE_BUFFER

		sprite->width = width;
		sprite->height = height;
		SetPolySpritePos(0, SPRITE_GROUND_Y, 0, sprite);
		for (db=0; db < 2; db++)  /* For both display buffers */
		{
			spoly = &(sprite->poly[db]);
			SetPolyFT4(spoly);
			setRGB0(spoly, 0x80, 0x80, 0x80);
			SetSemiTrans(spoly, 0);
			SetShadeTex(spoly, 1);
//		 	setXYWH(spoly, 10, 10, 40, 40);    /* not needed */

			spoly->tpage = 0;
		}
		sprite->flags = PSPRITE_DISPLAY;
		SetBit(sprite->flags, PSPRITE_USED);
//		gPSpritePtrs[spnum] = sprite;
	}
	else
	{
		sprite = NULL;
		Cerror();	// out of sprites
	}
	return sprite;
}

/*~~-------------------------
---------------------------*/
void DeletePolySprite(PolySpritePtr sprite)
{
	register int n,done;

	if (!sprite)  return;   // NULL pointer error
	
	done = False;
//	ResetBit(sprite->flags, PSPRITE_USED);
	sprite->flags = 0;
	return;

#if 0		
	for (n=0; n < MaxPolySprites && !done; n++)
	{
		if (gPSpritePtrs[n] == sprite)
		{
//			FreeMem(sprite);
			gPSpritePtrs[n]->flags = 0;
			gPSpritePtrs[n] = NULL;
			done = True;
		}
	}
	if (!done)
		Cerror();

#endif // 0
}


/*-------------------------------------------------------------
  IsSpriteUsed ~ Check if sprite is in use
-------------------------------------------------------------*/
BOOL IsSpriteUsed(PolySprite_t *sprite)
{
	if (sprite->flags & PSPRITE_USED)
		return TRUE;
	else
		return FALSE;
}  /* End IsSpriteUsed() */

/*-------------------------------------------------------------
  DeletePolySpritesWithFlags ~ 
-------------------------------------------------------------*/
void DeletePolySpritesWithFlags(ULONG flags)
{
	register int n;
	register PolySpritePtr sprite;

	for (n=0; n < MaxPolySprites; n++)
	{
//		sprite = gPSpritePtrs[n];
		sprite = SpriteNumPointerMacro(n);
		if ((sprite->flags & flags))
		{
			DeletePolySprite(sprite);
//			sprite->flags = 0;
//			gPSpritePtrs[n] = NULL;
		}
	}
}

/*~~-------------------------
---------------------------*/
void ClearAllPolySprites(void)
{
	register int n, used;
	PolySpritePtr sprite;

	used = 0;
	for (n=0; n < MaxPolySprites; n++)
	{
		sprite = SpriteNumPointerMacro(n);
		DeletePolySprite(sprite);
		
//		if (gPSpritePtrs[n])
//		{
//			used++;
//			FreeMem(gPSpritePtrs[n]);
//			gPSpritePtrs[n]->flags = 0;
//			gPSpritePtrs[n] = NULL;
//		}
	}
#if PC_DEBUG
//	DebugPrintf(0," %d of %d Sprites used\n", used, MaxPolySprites);
	DebugPrintf(0," %d of %d Sprites used\n", MaxSpritesEverUsed, MaxPolySprites);
#endif
}

/*-------------------------------------------------------------
  ResizePolySprite ~ Set size of sprite polygon
-------------------------------------------------------------*/
void ResizePolySprite(int width, int height, PolySpritePtr sprite)
{
	if (!sprite)  return;   // NULL pointer error
	sprite->width = width;
	sprite->height = height;
//	SetPolySpritePos(0, SPRITE_GROUND_Y, 0, sprite);
}

/*~~-------------------------
    Put Texture on sprite
---------------------------*/
void TexturePolySprite(
		Ushort tpage, Ushort u, Ushort v,
		Ushort width, Ushort height, /* Bitmap */
		Ushort clut,   /* Colors */
		PolySpritePtr sprite
	)
{
	register POLY_FT4 *spoly;
	int db;

	if (!sprite)
	{
		Cerror();
		return;   // NULL pointer
    }
	for (db=0; db<2; db++)
	{
		spoly = &(sprite->poly[db]);

		spoly->tpage = tpage;
		spoly->clut = clut;
		setUVWH(spoly, u, v, width, height);
	}
}

/*-------------------------------------------------------------
  FlipPolySpriteTexture ~ Flip texture of poly sprite
-------------------------------------------------------------*/
void FlipPolySpriteX(int flip, PolySpritePtr sprite)
{
	register Ushort tmp;
	register POLY_FT4 *spoly;
	ULONG flag;
	int db;

	if (!sprite)  return;   // NULL pointer
	flag = sprite->flags & PSPRITE_XFLIP;
	if ( (flip && !flag) || (!flip && flag) )
	{
		sprite->flags ^= PSPRITE_XFLIP;
		for (db = 0; db < 2; db++)
		{
			spoly = &sprite->poly[db];
			tmp = spoly->u0;        /* u0 <-> u1 */
			spoly->u0 = spoly->u1;
			spoly->u1 = tmp;
			tmp = spoly->v0;		/* v0 <-> v1 */
			spoly->v0 = spoly->v1;
			spoly->v1 = tmp;
			tmp = spoly->u2;		/* u2 <-> u3 */
			spoly->u2 = spoly->u3;
			spoly->u3 = tmp;
			tmp = spoly->v2;		/* v2 <-> v3 */
			spoly->v2 = spoly->v3;
			spoly->v3 = tmp;
		}
	}
}

/*-------------------------------------------------------------
  FlipPolySpriteTexture ~ Flip texture of poly sprite
-------------------------------------------------------------*/
void FlipPolySpriteY(int flip, PolySpritePtr sprite)
{
	register Ushort tmp;
	register POLY_FT4 *spoly;
	ULONG flag;
	int db;

	if (!sprite)  return;   // NULL pointer
	flag = sprite->flags & PSPRITE_YFLIP;
	if ( (flip && !flag) || (!flip && flag) )
	{
		sprite->flags ^= PSPRITE_YFLIP;
		for (db = 0; db < 2; db++)
		{
			spoly = &sprite->poly[db];
			tmp = spoly->u0;        /* u0 <-> u2 */
			spoly->u0 = spoly->u2;
			spoly->u2 = tmp;
			tmp = spoly->v0;		/* v0 <-> v2 */
			spoly->v0 = spoly->v2;
			spoly->v2 = tmp;
			tmp = spoly->u1;		/* u1 <-> u3 */
			spoly->u1 = spoly->u3;
			spoly->u3 = tmp;
			tmp = spoly->v1;		/* v1 <-> v3 */
			spoly->v1 = spoly->v3;
			spoly->v3 = tmp;
		}
	}
}

#if 0
/*-------------------------------------------------------------
  RotatePolySpriteTexture90 ~ 
-------------------------------------------------------------*/
void RotatePolySpriteTexture90(int yes, PolySpritePtr sprite)
{
	USHORT u0,v0;
	register POLY_FT4 *spoly;
	ULONG flag;
	int db;
	
	if (!sprite)  return;   // NULL pointer
	flag = sprite->flags & PSPRITE_ROT90;
	if (yes)
	{
		if (!flag)
		{
			SetBit(sprite->flags, PSPRITE_ROT90);
			spoly = &sprite->poly[db];
			u0 = spoly->u0; v0 = spoly->v0;
			spoly->u0 = spoly->u2; spoly->v0 = spoly->v2;
			spoly->u2 = spoly->u3; spoly->v2 = spoly->v3;
			spoly->u3 = spoly->u1; spoly->v3 = spoly->v1;
			spoly->u1 = u0; spoly->v1 = v0;
		}
	}
	else	// No
	{
		if (flag)
		{
			ResetBit(sprite->flags, PSPRITE_ROT90);
			spoly = &sprite->poly[db];
			u0 = spoly->u0; v0 = spoly->v0;
			spoly->u0 = spoly->u1; spoly->v0 = spoly->v1;
			spoly->u1 = spoly->u3; spoly->v1 = spoly->v3;
			spoly->u3 = spoly->u2; spoly->v3 = spoly->v2;
			spoly->u2 = u0; spoly->v2 = v0;
		}
	}
}  /* End RotatePolySpriteTexture90() */
#endif 0

/*-------------------------------------------------------------
  SetPolySpriteClut ~
-------------------------------------------------------------*/
void SetPolySpriteClut(USHORT clutid, PolySpritePtr sprite)
{
	int db;

	if (!sprite)  return;   // NULL pointer
	for (db=0; db<2; db++)
	{
		sprite->poly[db].clut = clutid;
	}
}

/*-------------------------------------------------------------
  SetPolySpriteShade ~
-------------------------------------------------------------*/
void SetPolySpriteShade(int r, int g, int b,
						int transparent, int shade,
						PolySpritePtr sprite)
{
	int db;
	POLY_FT4 *spoly;

	if (!sprite)  return;   // NULL pointer
	for (db=0; db < 2; db++)  /* For both display buffers */
	{
		spoly = &(sprite->poly[db]);
		setRGB0(spoly, r, g, b);
		SetSemiTrans(spoly, transparent);
		SetShadeTex(spoly, shade);
	}
}

/*-------------------------------------------------------------
  SetPolySpriteFlags ~
-------------------------------------------------------------*/
ULONG SetPolySpriteFlags(ULONG flags, int on, PolySpritePtr sprite)
{
	if (!sprite)  return;   // NULL pointer
	if (on)
		sprite->flags |= flags;
	else
		sprite->flags &= ~flags;
	return sprite->flags;
}

/*-------------------------------------------------------------
  SetPolySpriteOnOff ~
-------------------------------------------------------------*/
void SetPolySpriteOnOff(int on, PolySpritePtr sprite)
{
	if (sprite)
	{
		if (on)
			sprite->flags |= PSPRITE_DISPLAY;
		else
			sprite->flags = sprite->flags & (~PSPRITE_DISPLAY);
	}		
}

/*-------------------------------------------------------------
  MakePolySpriteShadow ~
-------------------------------------------------------------*/
void MakePolySpriteShadow(int skew, int tallmult, PolySpritePtr sprite)
{
	register int x1,y1,z1, x2,y2,z2;
	int height;

	height = (sprite->height * tallmult) / 100;
	x2 = sprite->ipos[3].vx - skew/6    ;
//	y1 = sprite->ipos[3].vy;
	y1 = SPRITE_GROUND_Y - 16;
	z2 = sprite->ipos[3].vz - 10;

	x1 = x2 - sprite->width;
	y2 = y1;
	z1 = z2 + height;

	// change from 1,3,0,2
	setVector(&(sprite->ipos[0]), x1 + skew, y1, z1);
	setVector(&(sprite->ipos[1]), x2 + skew, y2, z1);
	setVector(&(sprite->ipos[2]), x1, y2, z2);
	setVector(&(sprite->ipos[3]), x2, y1, z2);
	SetPolySpriteShade(0x80,0x80,0x80, 1, 1, sprite);
	sprite->flags |= PSPRITE_SHADOW;
}

/*-------------------------------------------------------------
  MakePolySpriteFloor ~
-------------------------------------------------------------*/
void MakePolySpriteFloor(PolySpritePtr sprite)
{
	register int x1,y1,z1, x2,y2,z2;

	// Assume IPOS 0,0,0
	x1 = 0 - (sprite->width / 2);
	y1 = SPRITE_GROUND_Y - 16;
	z2 = 0 - (sprite->width / 2);

	x2 = x1 + sprite->width;
	y2 = y1;
	z1 = z2 + sprite->height;

	// change from 1,3,0,2
	setVector(&(sprite->ipos[0]), x1, y1, z1);
	setVector(&(sprite->ipos[1]), x2, y2, z1);
	setVector(&(sprite->ipos[2]), x1, y2, z2);
	setVector(&(sprite->ipos[3]), x2, y1, z2);
//	SetPolySpriteShade(0x80,0x80,0x80, 1, 1, sprite);
	sprite->flags |= PSPRITE_FLOOR;
}


/*-------------------------------------------------------------
  MakePolySprite2d ~
-------------------------------------------------------------*/
void MakePolySprite2d(PolySpritePtr sprite)
{
	if (!sprite)  return;   // NULL pointer
	sprite->flags |= PSPRITE_2D;
}

/*~~------------------------------
	Set PolySprite trans values
---------------------------------*/
void MovePolySpriteTo(int x, int y, int z, PolySpritePtr sprite)
{
	if (!sprite)  return;   // NULL pointer
/*	setVector(&(sprite->trans), x, y, z);    /*##*/
	sprite->trans.vx = x;
	sprite->trans.vy = y;
	sprite->trans.vz = z;
}

/*~~------------------------------
	Move PolySprite Relative
---------------------------------*/
void MovePolySprite(SVECTOR *vect, PolySpritePtr sprite)
{
	if (!sprite)  return;   // NULL pointer
	sprite->trans.vx += vect->vx;
	sprite->trans.vy += vect->vy;
	sprite->trans.vz += vect->vz;
}

/*-------------------------------------------------------------
  CopySpriteData ~
-------------------------------------------------------------*/
void CopySpriteData(PolySpritePtr src, PolySpritePtr dest)
{
	CopyBytes((UBYTE*)src, (UBYTE*)dest, sizeof(PolySpriteType));
}


/*~~---------------------------------------------
    ~Add Field lines to Drawing Table
-----------------------------------------------*/
void DrawFieldLines(void)
{
	register PolySpritePtr sprite;
	register int n, otz, drawthis;
	register ULONG flags;
	register POLY_FT4 *spoly;
	long d0, flg;
	int otz1;
	int lowcam, weather, colorMinus;
	int color;

//	MATRIX mtx1;
//	SVECTOR svect;
	static MATRIX mmat = {	/* Matrix */
			ONE,  0,	0,	/* for vertical polygon */
			0,	ONE,	0,
			0,	  0,  ONE,
		};
	static MATRIX shadowMat;

//	reversed = ((gCamera.dir > 8)? True: False);
	lowcam = ((gCamera.offset.vy > -700)  ? True: False);
	weather = GameOptions[GAME_OPTNS_WEATHER];
	colorMinus = (gCamera.offset.vy / 100);

	PushMatrix();    /*##*/
	for (n=0; n < MaxPolySprites; n++)
	{
		drawthis = True;
//		sprite = gPSpritePtrs[n];
		sprite = SpriteNumPointerMacro(n);
		flags = sprite->flags;
		if (sprite && (flags & PSPRITE_DISPLAY)
		    		&& (flags & PSPRITE_USED)
		    		&& (flags & PSPRITE_FIELD_LINE) )
		{
			spoly = &(sprite->poly[gSpriteDB]);
//			PushMatrix();    /*##*/
			gte_RotTrans(&sprite->trans,  (VECTOR *)&shadowMat.t, &flg);// MRH set to macro
			gte_SetTransMatrix(&shadowMat);	// MRH set to macro

//			otz = RotTransPers4(
			otz = RotAverage4(
						&sprite->ipos[0], &sprite->ipos[1],
						&sprite->ipos[2], &sprite->ipos[3],
						(long*)&spoly->x0, (long*)&spoly->x1,
						(long*)&spoly->x2, (long*)&spoly->x3, &d0, &flg
				);

			if (otz > -10 && otz < 11000)
			{
//				otz /= 8;
				otz = otz >> OTZSHIFT;
				otz -= 9;

//				setSemiTrans(spoly, 1);	 // no need for tranparent shadow MRH set to macro
				//setRGB0(spoly, 0x20, 0xf0, 0x20);
				if (flags & PSPRITE_LOW_PRIORITY)
					otz += 1;
				else if (flags & PSPRITE_HIGH_PRIORITY)
					otz -= 1;
//				otz += 2;
				if (spoly->y0 == spoly->y2)
					spoly->y2++;
				if (spoly->y1 == spoly->y3)
					spoly->y3++;
				if (!(flags & PSPRITE_PLAY_LINE))
				{
//					color = 270 - ((otz * 8) / 6) + otz / 15;
					color = 270 - ((otz << 3) / 6) + otz / 15;
//					color -= (gCamera.offset.vy / 100);	// y is neg
					color -= colorMinus;
					if (color < -10)
						drawthis = False;
					if (weather == SNOWY_WEATHER_CONDITION)
						limitRange(color, 90, 255);
					else
						limitRange(color, 60, 255);
					if (color < 80)
					   	setRGB0(spoly, color, color + 7, color);
					else
					   	setRGB0(spoly, color, color, color);
				}	   	
//			   if (otz > 300)
//				   	drawthis = False;
				if (otz > 100 && lowcam)
				{
//					otz -= (otz/60);
					otz -= (otz >> 6);
				}
				otz -= otz / 8;  // new field
				limitRange(otz, 4, OTSIZE);    /*##*/
				if (drawthis)
//					AddPrimToOT(otz,(ULONG*)spoly); /* Poly is first entry */
					addPrim(gPtrGpuOt + otz, (ULONG*)spoly);	// MRH
				
			} // otz range
//			PopMatrix();    /*##*/
		}
	}
	PopMatrix();    /*##*/
}

/*-------------------------------------------------------------
  Draw2dSprite ~
-------------------------------------------------------------*/
void Draw2dSprite(PolySpritePtr sprite)
{
	register POLY_FT4 *spoly;
	register int otz;

	spoly = &(sprite->poly[gSpriteDB]);
	setXYWH(spoly, sprite->trans.vx, sprite->trans.vy,
							sprite->width, sprite->height);
	otz = sprite->trans.vz;
	GsSortPoly(spoly, gPtrGsOt, otz);
}


/*~~---------------------------------------------
    ~Add Poly sprites to Drawing Table
-----------------------------------------------*/
void DrawPolySpritesSub(void)
{
	register PolySpritePtr sprite;
	register POLY_FT4 *spoly;
	register ULONG flags;
	register int n, otz, drawthis;
	long d0, flg;
	int otz1, dir;
//	int lowcam, weather;

//	MATRIX mtx1;
//	SVECTOR svect;
	static int vAngle = 0;
	MATRIX mmat = {	/* Matrix */
			ONE,  0,	0,	/* for vertical polygon */
			0,	ONE,	0,
			0,	  0,  ONE,
		};
	static long angles[16] = 
			{ 0, 
			    (4096/16),
			    2 * (4096/16),
			    3 * (4096/16),	
			    4 * (4096/16),	
			    5 * (4096/16),	
			    6 * (4096/16),	
			    7 * (4096/16),	
			    8 * (4096/16),	
			    9 * (4096/16),	
			    10 * (4096/16),	
			    11 * (4096/16),	
			    12 * (4096/16),	
			    13 * (4096/16),	
			    14 * (4096/16),	
			    15 * (4096/16),	
			};
	static MATRIX shadowMat;


//	RotMatrixX(GetCameraAngleYZ(), &mmat);
//	MulRotMatrix(&mmat);
//	dir = (gCamera.dir + 12) & 15;
//	RotMatrixY(angles[dir], &mmat);
//	RotMatrixZ(-GetCameraAngleXZ(), &mmat);
	
//	RotMatrixX(-GetVerticalCameraAngle(), &mmat);
	for (n=0; n < MaxPolySprites; n++)
	{
		drawthis = True;
//		sprite = gPSpritePtrs[n];
		sprite = SpriteNumPointerMacro(n);
		flags = sprite->flags;
		if (sprite && (flags & PSPRITE_DISPLAY) 
		    			&& (flags & PSPRITE_USED)
		    			&& !(flags & PSPRITE_FIELD_LINE) )
		{
			spoly = &(sprite->poly[gSpriteDB]);
			PushMatrix();    /*##*/

			if (sprite && (flags & PSPRITE_2D))
			{
		    	Draw2dSprite(sprite);
			}
			else
			{
			    if ( !(flags & (PSPRITE_SHADOW | PSPRITE_FLOOR)) ) //normal
				{
					gte_RotTrans(&sprite->trans, (VECTOR *)&mmat.t, &flg); 	  // MRH set to macro
					gte_SetRotMatrix(&mmat);	// sprite never rotates! // MRH set to macro
					gte_SetTransMatrix(&mmat);	// sprite only moves // MRH set to macro
				}
				else /* shadow */
				{
					gte_RotTrans(&sprite->trans,  (VECTOR *)&shadowMat.t, &flg);// MRH set to macro
					gte_SetTransMatrix(&shadowMat);	// MRH set to macro
				}

//				otz = RotTransPers4(
				otz = RotAverage4(
						&sprite->ipos[0], &sprite->ipos[1],
						&sprite->ipos[2], &sprite->ipos[3],
						(long*)&spoly->x0, (long*)&spoly->x1,
						(long*)&spoly->x2, (long*)&spoly->x3, &d0, &flg
					);


//				otz /= 8;
				otz = otz >> OTZSHIFT;
				if (otz < 0)
					drawthis = False;
					
				if ((flags & PSPRITE_NO_PRIORITY)) //Don't Adjust Priority
				{
					otz -= 3;
				}
				else    // Adjust priority for sprites in front
				{
					otz -= (11);

					if ( (flags & PSPRITE_SHADOW) )
					{
						otz += 1;
						setSemiTrans(spoly, 1);		// MRH set to macro
					}
					if (flags & PSPRITE_LOW_PRIORITY)
						otz += 1;
					else if (flags & PSPRITE_HIGH_PRIORITY)
						otz -= 1;
					if (flags & (PSPRITE_PLAYER) )
						otz -= 2;
					if (flags & (PSPRITE_JNUM) )
						otz -= 5;
					
					if ((flags & PSPRITE_ENDZONE_PERSON))
					{
						otz += otz/500;
					}
				    else //	not for endzone people
				    {
						otz -= otz / 8;  // new field
					}
				}
				limitRange(otz, 4, OTSIZE); 
				if (drawthis)
//					AddPrimToOT(otz,(ULONG*)spoly); /* Poly is first entry */
					addPrim(gPtrGpuOt + otz, (ULONG*)spoly);	// MRH
			}
			PopMatrix();
		}
	}
//	PopMatrix();    /*##*/
	gSpriteDB ^= 1;	/* Switch DB num  (XOR) */
}

/*-------------------------------------------------------------
  DrawPolySprites ~ 
-------------------------------------------------------------*/
void DrawPolySprites(void)
{
	EnterCriticalSection();
	SetSpadStack(0x1f8003fc);
	ExitCriticalSection();
	
	DrawFieldLines();
	DrawPolySpritesSub();
	
	EnterCriticalSection();
	ResetSpadStack();
	ExitCriticalSection();
}

/*-------------------------------------------------------------
  CreateSpriteGrid ~ Create Grid of meshed sprites
-------------------------------------------------------------*/
void CreateXZSpriteGrid( SubTexture_t *tex,
						SVECTOR *vec1, SVECTOR *vec2,
						int xdiv, int zdiv,
						int flipx, int flipz,
						PolySpritePtr *spriteList )
{
	register int i, j, x1, x2, y, z1, z2;
	register PolySpritePtr sprite;
	int startx, startz, endx, endz, xstep, zstep, spriteNum;
	int u1,v1, ustep, vstep;
	SVECTOR *tmpVec;

//	 Cmarker();
	startx = vec1->vx;
	startz = vec1->vz;
	endx = vec2->vx;
	endz = vec2->vz;
	y = vec1->vy;
	if (flipx)
	{
		i = startx;
		startx = endx;
		endx = i;
	}
	if (flipz)
	{
		i = startz;
		startz = endz;
		endz = i;
	}
	xstep = (endx - startx) / xdiv;
	zstep = (endz - startz) / zdiv;
	ustep = (tex->width) / xdiv;
	vstep = (tex->height) / zdiv;
//	 DebugPrintf(0, "%d,%d %d,%d step:%d,%d\n",
//	 		startx, startz, endx, endz, xstep, zstep);
	spriteNum = 0;
	for (j = 0; j < zdiv; j++)
	{
		for (i = 0; i < xdiv; i++)
		{
			sprite = spriteList[spriteNum];
			if (sprite)
			{
				x1 = startx + (i * xstep);
				x2 = x1 + xstep;
				z1 = startz + (j * zstep);
				z2 = z1 + zstep;
				u1 = tex->u + (i * ustep);
				v1 = tex->v + (j * vstep);
				sprite->flags |= PSPRITE_FLOOR;
				ResizePolySprite(xstep, zstep, sprite);
				TexturePolySprite(tex->texID, u1, v1,
						ustep -1, vstep -1,
						tex->clut->CLUT_ID, sprite);
				setVector(&sprite->ipos[0], x1, y, z1);
				setVector(&sprite->ipos[1], x2, y, z1);
				setVector(&sprite->ipos[2], x1, y, z2);
				setVector(&sprite->ipos[3], x2, y, z2);
//			 DebugPrintf(0, "%d,%d  %d,%d %d,%d  %d,%d \n",i,j, x1,z1, x2,z2, u1,v1);
				spriteNum++;
			}
		}
	}
}

/**************** SubTexture Routines ************/

/*-------------------------------------------------------------
  InitSubTexture ~
  		colors:0 = 4 bit,  1 = 8bit, 2 = 16bit  color
  		abr: Semi Transparency rate (see GetTPage)
-------------------------------------------------------------*/
void InitSubTexture(SubTexture_t *subtex, vram_t *vrtex, vram_t *clut ,
					int u, int v, int width, int height, int colors, int abr )
{
	subtex->u = u;
	subtex->v = v;
	subtex->width = width;
	subtex->height = height;
	subtex->texture = vrtex;
	subtex->clut = clut;
	subtex->colors = colors;
	subtex->texID = GetTPage(colors, abr, vrtex->x, vrtex->y);
	if (clut)
		subtex->clut->CLUT_ID = GetClut(clut->x, clut->y);
}

/*-------------------------------------------------------------
  FreeSubTexture ~ Free VRAM from subtexture
-------------------------------------------------------------*/
void FreeSubTexture(SubTexture_t *subtex)
{
	free_VRam(subtex->clut);
	free_VRam(subtex->texture);
	subtex->clut = NULL;
	subtex->texture = NULL;
}

/*-------------------------------------------------------------
  LoadSubTexturePixels ~
-------------------------------------------------------------*/
void LoadSubTexturePixels(ULONG *pixels, SubTexture_t *subtex)
{
	RECT rec;
	static char div[4] = {4, 2, 1, 1};

	rec.x = (subtex->u / div[subtex->colors]) + subtex->texture->x;
	rec.y = subtex->v + subtex->texture->y;
	rec.w = subtex->width / div[subtex->colors];
	rec.h = subtex->height;
	LoadImage(&rec, (ULONG *)pixels);
}

/*-------------------------------------------------------------
  SetPolySpriteSubTexture ~
-------------------------------------------------------------*/
void SetPolySpriteSubTexture(SubTexture_t *subtex, PolySpritePtr sprite)
{
	TexturePolySprite(subtex->texID, subtex->u, subtex->v,
					subtex->width -1, subtex->height -1,
					subtex->clut->CLUT_ID, sprite);
}


/**************************************************************
					Test Routines */

#if TESTING


#endif