#ifndef _PSPRITE_H
#define _PSPRITE_H 1
/**************************************************
	Polygon based Sprite handler  (.H)

	Copyright 1994,95,96 GameTek
	By Christopher Deel
*************************************************/

#define SPRITE_GROUND_Y (16)
//#define SPRITE_GROUND_Y (26)

/*-------- Poly Sprite Flags --------*/
#define PSPRITE_DISPLAY (1)
#define PSPRITE_XFLIP	(2)
#define PSPRITE_YFLIP	(4)
#define PSPRITE_SHADOW	(8)
#define PSPRITE_FLOOR	(1 << 4)
#define PSPRITE_2D		(1 << 5)
#define PSPRITE_COLORED	(1 << 6)
#define PSPRITE_LOW_PRIORITY	(1 << 7)
#define PSPRITE_HIGH_PRIORITY	(1 << 8)
#define PSPRITE_FIELD_LINE		(1 << 9)
#define PSPRITE_PLAY_LINE		(1 << 10)
#define PSPRITE_PLAYER			(1 << 11)
#define PSPRITE_ROT90			(1 << 12)
#define PSPRITE_BALL			(1 << 13)
#define PSPRITE_SIDE_PERSON		(1 << 14)
#define PSPRITE_ENDZONE_PERSON	(1 << 15)
#define PSPRITE_USED			(1 << 16)
#define PSPRITE_JNUM			(1 << 17)
#define PSPRITE_NO_PRIORITY		(1 << 18)

/*--- PolySpriteType ----*/
typedef struct
{
	ULONG flags;		/* Flags */
	short width,height;
	SVECTOR trans;		/* Translation in X,Y,Z */
	POLY_FT4 poly[2];	/* One for each display buffer */
	SVECTOR ipos[4];	/* Initial Position in X,Y,Z   4 corners */
} PolySpriteType;

typedef PolySpriteType PolySprite_t;
typedef PolySpriteType *PolySpritePtr;
typedef PolySpriteType *PolySpritePtr_t;

//------- 3D Line Type --------
typedef struct
{
	GsLINE gsLine[2];
	SVECTOR pos[2];
	ULONG flags;
} Line3DType;

typedef short LineHandle_t;

#define LINE_ACTIVE_FLAG 	1
#define LINE_DISPLAY_FLAG	(1 << 1)

/*--------- Sub Textures ---------- */
typedef struct
{
	vram_t *texture;
	vram_t *clut;
	USHORT texID;
	UBYTE u, v, width, height;
	UBYTE colors;
} SubTexture_t;

    
/* psprite.c */
void FreeSpriteMemory ( void );
void InitPolySprites ( void );
void SetPolySpritePos ( int x , int y , int z , PolySpritePtr sprite );
PolySpritePtr CreatePolySprite ( int width , int height );
void DeletePolySprite ( PolySpritePtr sprite );
void DeletePolySpritesWithFlags ( ULONG flags );
void ClearAllPolySprites ( void );
void ResizePolySprite ( int width , int height , PolySpritePtr sprite );
void TexturePolySprite ( Ushort tpage , Ushort u , Ushort v , Ushort width , Ushort height , Ushort clut , PolySpritePtr sprite );
void FlipPolySpriteX ( int flip , PolySpritePtr sprite );
void FlipPolySpriteY ( int flip , PolySpritePtr sprite );
void RotatePolySpriteTexture90 ( int yes , PolySpritePtr sprite );
void SetPolySpriteClut ( USHORT clutid , PolySpritePtr sprite );
void SetPolySpriteShade ( int r , int g , int b , int transparent , int shade , PolySpritePtr sprite );
ULONG SetPolySpriteFlags ( ULONG flags , int on , PolySpritePtr sprite );
void SetPolySpriteOnOff ( int on , PolySpritePtr sprite );
void MakePolySpriteShadow ( int skew , int tallmult , PolySpritePtr sprite );
void MakePolySpriteFloor ( PolySpritePtr sprite );
void MakePolySprite2d ( PolySpritePtr sprite );
void MovePolySpriteTo ( int x , int y , int z , PolySpritePtr sprite );
void MovePolySprite ( SVECTOR *vect , PolySpritePtr sprite );
void CopySpriteData ( PolySpritePtr src , PolySpritePtr dest );
void DrawFieldLines ( void );
void Draw2dSprite ( PolySpritePtr sprite );
void DrawPolySpritesSub ( void );
void DrawPolySprites ( void );
void InitSubTexture ( SubTexture_t *subtex , vram_t *vrtex , vram_t *clut , int u , int v , int width , int height , int colors , int abr );
void FreeSubTexture ( SubTexture_t *subtex );
void LoadSubTexturePixels ( ULONG *pixels , SubTexture_t *subtex );
void SetPolySpriteSubTexture ( SubTexture_t *subtex , PolySpritePtr sprite );

void CreateXZSpriteGrid( SubTexture_t *tex, 
						SVECTOR *vec1, SVECTOR *vec2, 
						int xdiv, int zdiv, 
						int flipx, int flipz,
						PolySpritePtr *spriteList );
												
#endif