/***********************************************
	PSXPAD.C  PSX Joypad routines

 	By Christopher Deel

	Copyright 1995  GameTek
	All rights reserved

	Updated for Lib 3.5  Feb-27-97
***********************************************/

#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include "prjhdrs.h"	/* Project header files */

#define DEBUG_THIS 1

#define PAD_HISTORY_MAX 20

u_long  JpadDataBoth;

UWORD AnyPadData, AnyPadTrig; 	// Data from all pads ORed together
UWORD LastPad;		// When using AnyPadData the last pad pressed is stored in here
UWORD JpadData[8];	// Data from Joy pads
UWORD JpadPrev[8];  // Previos Data from Joy pads
UWORD JpadTrig[8];  // One shot data from joy pads
UBYTE JpadID[8];	// IDs of joypads  0=No pad
WORD JpadRepeat[8];  // Repeat rate counters -1 = NO REPEAT
int JpadNeeded[8];
int MultiTapID[2];

ULONG gPadHistory[PAD_HISTORY_MAX];
static int gPadHistoryIdx = 0;

int gTotalPads = 0;

UBYTE gPadBuffer[2][PADBUFFLEN];

UWORD PadCount, FramesElapsed;
WORD JpadRepeatFreq = 32;

/*-------------------------------------------------------------
  InitPads ~
-------------------------------------------------------------*/
void InitPads(int n)
{
	int i;

//	gTotalPads = PadInit(0);   /* reset PAD */
//	InitPAD(&gPadBuffer[0][0], PADBUFFLEN, &gPadBuffer[1][0], PADBUFFLEN);
	InitTAP(&gPadBuffer[0][0], PADBUFFLEN, &gPadBuffer[1][0], PADBUFFLEN);
//	StartPAD();
	StartTAP();
	ChangeClearPAD(0);
	for (i = 0; i < 8; i++)
	{
		JpadData[i] = 0;
		JpadPrev[i] = 0;
		JpadID[i] = 0;
		JpadNeeded[i] = 0;
	}
	for (i = 0; i < PAD_HISTORY_MAX; i++)
		gPadHistory[i] = 0;
#if PC_DEBUG
	DebugPrintf(0, "%d Pads Connected\n", gTotalPads);
#endif
}

/*-------------------------------------------------------------
-------------------------------------------------------------*/
void StopPads(void)
{
	PadStop();
}

/*-------------------------------------------------------------
  PadBuffToData ~
-------------------------------------------------------------*/
u_short PadBuffToData(char *ptr)
{
	register unsigned short data;

	data = (u_short)( ((*ptr & 0xff)<<8) | (*(ptr+1)&0xff) );
	data = ~data & 0xffff;
	return data;
}

/*-------------------------------------------------------------
  ReadMultiTap ~ Read MultiTap data and convert
-------------------------------------------------------------*/
void ReadMultiTap(char *buff, int padnum)
{
	int i;
	UBYTE id;
	register char *ptr;

	for (i = 0; i < 4; i++)
	{
		ptr = buff + (i * 8);
		if (*ptr != 0)
			id = 0;
		else
			id = *(ptr + 1);
		switch(id)
		{
			case PADID_MULTI :
				break;
			case PADID_CON :
				JpadData[padnum] = PadBuffToData(ptr + 2);
				JpadID[padnum] = id;
				break;
			case PADID_MOUSE :
				break;
			case PADID_ANA :
				break;

			case 0:
			case 0xFF:
			default :
				JpadData[padnum] = 0;
				JpadID[padnum] = 0;
				break;
		}
		padnum++;
	}
}

/*-----------------------------------------------
 * Read control-pad
---------------------------------------------- */
void ReadPads(void)
{
	register int i;
	register UWORD data, prev, any, anyTrig;
	int padnum;
	UBYTE id;

	FramesElapsed = VSync(-1) - PadCount;	// Calc no. of vblanks since last read
  	PadCount=VSync(-1);						// Update our internal vblank counter


	for (i = 0; i < 8; i++)
	{
	    JpadPrev[i] = JpadData[i];
	    JpadID[i] = 0;
	}
//	AnyPadPrev = AnyPadData;

	padnum = 0;
	for (i = 0; i < 2; i++)
	{
		if (gPadBuffer[i][0] != 0)
		{
			id = 0;

			if(gPadBuffer[i][1] == PADID_MULTI)
			    padnum += 3;
		}
		else
			id = gPadBuffer[i][1];

		MultiTapID[i] = 0;
		switch(id)
		{
			case PADID_MULTI :
				MultiTapID[i] = id;
				ReadMultiTap(&gPadBuffer[i][2], padnum);
				padnum += 3;
				break;

			case PADID_CON:
			default:
				JpadData[padnum] = PadBuffToData(&gPadBuffer[i][2]);
				JpadID[padnum] = id;
				break;

			case PADID_MOUSE:
				break;
			case PADID_ANA:
				break;

			case 0:
			case 0xFF:
				JpadData[padnum] = 0;
				JpadID[padnum] = 0;
				break;
		}
		padnum++;
	}


	if (JpadData[0] && JpadData[0] != gPadHistory[gPadHistoryIdx])
	{
		i = gPadHistoryIdx;
		i++;
		i %= PAD_HISTORY_MAX;
		gPadHistory[i] = JpadData[0];
		gPadHistoryIdx = i;
	}

	any = anyTrig = 0;

	for (i = 0; i < 8; i++)
	{
	    prev = JpadPrev[i];
	    data = JpadData[i];
	    any |= data;

		if(JpadRepeatFreq != -1 && data && data == prev)
		{
			JpadRepeat[i]+=FramesElapsed;

			if(JpadRepeat[i] > JpadRepeatFreq)
			{
				prev = 0;
				JpadRepeat[i] %= JpadRepeatFreq;
			}
		}
		else
		{
			JpadRepeat[i] = 0;
		}

	    JpadTrig[i] = (data ^ prev) & data;

		if(JpadTrig[i])
		{
		    anyTrig |= JpadTrig[i];
			LastPad = i;
		}
	}

	AnyPadTrig = anyTrig;
	AnyPadData = any;
}

/*-------------------------------------------------------------
  ReadPadsOr ~
-------------------------------------------------------------*/
void ReadPadsOr(void)
{
	int i;
	ushort save[8];

	for (i = 0; i < 8; i++)
		save[i] = JpadData[i];
	ReadPads();
	for (i = 0; i < 8; i++)
		JpadData[i] |= save[i];
}


/*-------------------------------------------------------------
  CheckPadButtons ~ Check for multiple buttons pressed
-------------------------------------------------------------*/
int CheckPadButtons(int pad, USHORT mask)
{
	if ( (JpadData[pad] & mask) == mask)
		return True;
	else
		return False;
}

/*-------------------------------------------------------------
  GetPadxyz ~ Get Pad XYZ input and add to SVECTOR
-------------------------------------------------------------*/
void GetPadxyz(int pad, int right, int mult, SVECTOR *vect)
{
	register USHORT data;

	data = JpadData[pad];
	if (right)  /* Use right side of pad */
	{
		if (data & JPADcircle)		vect->vx += mult;
		if (data & JPADsquare)		vect->vx -= mult;
		if (data & JPADtriangle)	vect->vy += mult;
		if (data & JPADx)			vect->vy -= mult;
		if (data & JPADl)			vect->vz += mult;
		if (data & JPADm)			vect->vz -= mult;
	}
	else
	{
		if (data & JPADLright)		vect->vx += mult;
		if (data & JPADLleft)		vect->vx -= mult;
		if (data & JPADLup)			vect->vy += mult;
		if (data & JPADLdown)		vect->vy -= mult;
		if (data & JPADn)			vect->vz += mult;
		if (data & JPADo)			vect->vz -= mult;
	}
}

/*-------------------------------------------------------------
  IsNewButton ~ Check for New Button Press
-------------------------------------------------------------*/
BOOL IsNewButton(void)
{
	static BOOL clear = False;
	BOOL ret;

	ReadPads();
	ret = False;
	if (IsAnyPadButton(0xFFFF))
	{
		if (clear)
			ret = True;
		else
			ret = False;
		clear = False;
	}
	else
	{
		clear = True;
		ret = False;
	}
	return ret;
}

/*~~-------------------------
    Is any button pressed?
---------------------------*/
int IsAnyButton(void)
{
	ReadPads();
	PollPC();

	if (IsAnyPadButton(0xFFFF))
		return 1;
	else
	    return 0;
}

/*-------------------------------------------------------------
  IsAnyPadButton ~ Check for a button pressed on any Pad
    return (pad +1) that matches;
-------------------------------------------------------------*/
int IsAnyPadButton(USHORT mask)
{
	int i, pad = 0;

	for (i = 0; i < 8; i++)
	{
		if (JpadData[i] & mask)
		{
			pad = i + 1;
//			pad |=(JpadData[i] & mask);
		}
	}
	return pad;
}

/*~~---------------------------------
    Wait for All buttons released
    then a button press.
-----------------------------------*/
void WaitForButton(void)
{
	while (IsAnyPadButton(0xFFFF))
	{
		ReadPads();
		PollPC();
	}
	while (!IsAnyPadButton(0xFFFF))
	{
		ReadPads();
		PollPC();
	}
}

/*-------------------------------------------------------------
  ClearPadHistory ~
-------------------------------------------------------------*/
//void ClearPadHistory(void)
//{
//	int i;

//	for (i = 0; i < PAD_HISTORY_MAX; i++)
//		gPadHistory[i] = 0;
//}  /* End ClearPadHistory() */

/*-------------------------------------------------------------
  CheckPadHistory ~
-------------------------------------------------------------*/
int CheckPadHistory(int pad, ULONG *buttons)
{
	int match, idx1, idx2;

	idx1 = gPadHistoryIdx;
	idx2 = 0;
	while (buttons[idx2])
		idx2++;
	idx2--;
	match = True;
	if (idx2 < 2)
		match = FALSE;
	while(match && idx2 >= 0)
	{
		if (buttons[idx2] != gPadHistory[idx1])
			match = False;
		idx2--;
		idx1--;
		if (idx1 < 0)
			idx1 = PAD_HISTORY_MAX;
	}
//	if (buttons[idx2] != gPadHistory[idx1])
//		match = False;
	return match;
}

/*-------------------------------------------------------------
  GetPadButton ~
-------------------------------------------------------------*/
int GetPadButton(USHORT mask)
{
	int i, pad = 0;

	ReadPads();
	PollPC();

	for (i = 0; i < 8; i++)
	{
		if (JpadData[i] & mask)
		{
			pad |=(JpadData[i] & mask);
		}
	}
	return pad;
}

/*-------------------------------------------------------------
  SetPadNeeded ~
   padnum = Pad #    or  -1 Set all plugged in pads
-------------------------------------------------------------*/
void SetPadNeeded(int padnum, int needed)
{
	register int i;

	if (padnum < 0)
	{
		for (i = 0; i < 8; i++)
		{
			if (JpadID[i])
				JpadNeeded[i] = TRUE;
			else
				JpadNeeded[i] = FALSE;
		}
	}
	else
		JpadNeeded[padnum] = needed;
}  /* End SetPadNeeded() */

/*-------------------------------------------------------------
  CheckNeededPads ~
-------------------------------------------------------------*/
int CheckNeededPads(void)
{
	int i, error;

	error = 0;
	for (i = 0; i < 8 && !error; i++)
	{
		if (JpadNeeded[i] && !JpadID[i])
			error = i + 1;
	}
	return error;
}  /* End CheckNeededPads() */


/*-------------------------------------------------------------
	FlushPads
-------------------------------------------------------------*/

void FlushPads(void)
{
	INT i;

	for (i=0; i<8; i++)
	{
		/*JpadPrev[i] = JpadData[i] = */ JpadTrig[i] = 0;
	}

	return;
}


#if DEBUG_THIS


/*-------------------------------------------------------------
  TestPadHistory ~
-------------------------------------------------------------*/
void TestPadHistory(void)
{
	int i;
	ULONG vb;
	static ULONG test[] = { JPADtriangle, JPADx, JPADsquare, JPADcircle, 0};

	vb = GetVBlankCount();
	while (1)
	{
		ReadPads();
		if (GetVBlankCount() > vb)
		{
			for (i = 0; i < PAD_HISTORY_MAX; i++)
			{
				DebugPrintf(0, "%x  ", gPadHistory[i]);
			}
			DebugPrintf(0, "\n");
			vb = GetVBlankCount() + 60;
			if (CheckPadHistory(0, test) )
				DebugPrintf(0, "Match!\n");
		}
	}
}

#endif DEBUG_THIS
