#include <stdio.h>
#include <sys/types.h>

#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include "prjhdrs.h"

#include "fbgequ.h"
#include "fbequ.h"
#include "fbgequ.h"
#include "fbram.h"
#include "shell.h"
#include "fbstats.h"


//**************************************************************************
// Data structures & defines for menu system

#include "menu.h"
#include "fbmenu.h"

//**************************************************************************

#define		MAX_GAMES	240
#define		SCHEDTM_Y	55

#define		MATCHES_DISPLAYED	9

UBYTE SchedFlag;

typedef struct {
	BYTE	month,		// Month
			day,		// Day of the month
			week,		// Week number of game
			away,		// Away Team number
			home;		// Home Team number
} schedule_t;

extern  WORD SeasonMode;
extern	BYTE WeekNum;
extern	schedule_t schedule[];
extern	UBYTE MyGameType;
extern  WORD GameOptions[];
extern  char Weeks[4][17];

int InitSchedule(void);
int TermSchedule(void);
int MainSchedule(void);
void DrawSchedule(void);
void GetCityName(int teamno, char *tmstr);

MenuItem ScheduleItemList[] = { END_MENU };	// A null menu list

MenuObj	ScheduleMenu = {
	(MenuItem *)ScheduleItemList,		// Menu object
	InitSchedule,						// Menu Init
	TermSchedule,						// Menu termination
	MainSchedule,						// Menu proc (called each frame)
	0, 0, 512, 240,						// Full screen
};

Sint16	SchedOffset;
UBYTE	*SchedTable;
Sint32	MaxGames;

UBYTE *Months[12] = {
					"JAN",
					"FEB",
					"MAR",
					"APR",
					"MAY",
					"JUN",
					"JUL",
					"AUG",
					"SEP",
					"OCT",
					"NOV",
					"DEC",
};

/***************************************************************************/
// Machine Specific functions


int DoSchedule(void)
{
	Display_Menu(SCHEDULE_MENU);
}


/***************************************************************************/


int InitSchedule(void)
{
	char *ptr;
	char fname[14];
	int i,j;

  /*Set graphics mode*/

	initVRamHandling();

  /*initialize the background for the shell*/

	LoadBackgroundCD(0,"/shells/bkg/nflsched.tim");	// Load & free from memory

  /*Init the multifile pointers*/

	Shell_MFile = (unsigned long *)LoadCDFile("/shells/stats/schedule.mfl");
	DebugPrintf(0,"schedule.mfl loaded into %x\n", Shell_MFile);

	InitFonts();
	Register_Font_Cluts();
	LoadFont("fontdpc2.tim", 3);

  /*load the shell graphics template files*/

	InitTemplates();

	multifile = Shell_MFile;
	LoadTemplate("schedule.llx");

	SchedOffset = MenuItemNum = MenuCursorY = 0;

	JpadRepeatFreq = 10;

	SchedTable = (char *)AllocMem(240);

	i = j = MaxGames = 0;

	while(Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][i] && i<17)
	{
		if(Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][i]==schedule[j].week)
		{
			while(Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][i]==schedule[j].week)
			{
				SchedTable[MaxGames++] = j++;
			}

			i++;
		}
		else j++;
	}

	return(0);
}


/***************************************************************************/


int MainSchedule(void)
{
	int retcode,i;

	retcode = InputHandler(0, 0);


	if(retcode == SQUARE)
	{
		if(SchedOffset > 0)
		{
			SchedOffset -= MATCHES_DISPLAYED;
		}
		if (SchedOffset < 0)
			SchedOffset = 0;
		retcode = 0;
	}
	else if(retcode == CIRCLE)
	{
		if(SchedOffset < MaxGames-MATCHES_DISPLAYED)
		{
			SchedOffset += MATCHES_DISPLAYED;
		}
		if (SchedOffset > MaxGames-MATCHES_DISPLAYED)
			SchedOffset = MaxGames-MATCHES_DISPLAYED;
		retcode = 0;
	}

	if(MenuDirY < 0)
	{
		if(SchedOffset > 0)
		{
			SchedOffset--;
		}
	}
	else if(MenuDirY > 0)
	{
		if(SchedOffset < MaxGames-MATCHES_DISPLAYED)
		{
			SchedOffset++;
		}
	}

	if(SchedOffset > 0)
	{
		if(MenuDirY == -1) PlayEffect(CURSOR_SFX);

		PrintStringXYAbs(UP_ARROW,284,43,PEN_RED);
	}

	if(SchedOffset < MaxGames-MATCHES_DISPLAYED)
	{
		if(MenuDirY == 1) PlayEffect(CURSOR_SFX);

		PrintStringXYAbs(DOWN_ARROW,284,190,PEN_RED);
	}

	PrintStringXYAbs("Page Up",65,200,PEN_WHITE);
	PrintStringXYAbs("Page Down",315,200,PEN_WHITE);

	DrawSchedule();

	return(retcode);
}


/***************************************************************************/


int TermSchedule(void)
{
	FreeMem(SchedTable);
	Std_Menu_CleanUp();					// Menu termination
}


/***************************************************************************/


void DrawSchedule(void)
{
	int i,acol,hcol,game;
	int week = 0;
	BYTE last_month = -1;
	BYTE last_day = -1;


	for(i=0; i<MATCHES_DISPLAYED; i++)
	{

		game = SchedTable[i+SchedOffset];

		if(schedule[game].month != last_month ||
			schedule[game].day != last_day)
		{
			last_month = schedule[game].month;
			last_day = schedule[game].day;
			sprintf(TempStr,"%s %d",Months[schedule[game].month],schedule[game].day);
			PrintStringXY(TempStr,20,SCHEDTM_Y+(i*15),PEN_YELLOW);
		}

		PrintStringXY("AT",278,SCHEDTM_Y+(i*15),PEN_YELLOW);

		if(HumanTeamNum == schedule[game].away) acol = PEN_BLUE;
		else acol = PEN_WHITE;
		if(HumanTeamNum == schedule[game].home) hcol = PEN_BLUE;
		else hcol = PEN_WHITE;

		if(HumanTeamNum == schedule[game].away) acol = PEN_BLUE;
		else acol = PEN_WHITE;
		if(HumanTeamNum == schedule[game].home) hcol = PEN_BLUE;
		else hcol = PEN_WHITE;

		if(schedule[game].week < Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][WeekNum-1]
			&& MyGameType != EXHIBITION_GAME)
		{
			if((MyGameType == SEASON_GAME && !SeasonMode)
				|| GameOptions[GAME_OPTNS_SEASON_LEN]==3)	// if password or playoff season only we aint got scores to show
			{
				if(acol != PEN_BLUE) acol = PEN_RED;
				if(hcol != PEN_BLUE) hcol = PEN_RED;
			}
			else											//Show past game scores
			{
				sprintf(TempStr,"%d",MemoryCardMimicPtr->GameScores[game][0]);
//				PrintStringXY(TempStr,465,SCHEDTM_Y+(i*15), PEN_BLUE);
				PrintStringXYCenter(TempStr,470,SCHEDTM_Y+(i*15), PEN_BLUE);
				sprintf(TempStr,"%d",MemoryCardMimicPtr->GameScores[game][1]);
//				PrintStringXY(TempStr,85,SCHEDTM_Y+(i*15), PEN_BLUE);
				PrintStringXYCenter(TempStr,106,SCHEDTM_Y+(i*15), PEN_BLUE);
			}
		}

		GetCityName(schedule[game].home, TempStr);
		PrintStringXYCenter(TempStr,380,SCHEDTM_Y+(i*15), hcol);

		GetCityName(schedule[game].away, TempStr);
		PrintStringXYCenter(TempStr,200,SCHEDTM_Y+(i*15), acol);

	}
}


/***************************************************************************/


void GetCityName(int teamno, char *tmstr)
{
	char *extra = "";

	if(teamno == 14) extra = " J";		// Check for NY teams - add J or G
	else if(teamno == 28) extra = " G";

	sprintf(tmstr,"%s%s",SpecificTeamData[teamno].TeamCity,extra);

	return;
}


/***************************************************************************/


int DisplaySchedule(void)
{
	return(Display_Menu(SCHEDULE_MENU));
}


/***************************************************************************/
