/*************************************************************************
 * SEASON.C - Contains all of the routines needed for season simulations,
 *		Both password and ram card.
 *
 *	Started on: 2-23-96
 *	Written by: Mike Heilemann
 *************************************************************************/
#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include <r3000.h>
#include <asm.h>
#include <kernel.h>
#include <sys/file.h>

#include "prjhdrs.h"	/* Project header files */
#include "fbequ.h"
#include "fbram.h"
#include "fbstats.h"
#include "movie.h"
#include "season.h"
#include "shell.h"

extern struct teamRAMstats *SeasonStatsPtr;
extern struct memorycardmimic *MemoryCardMimicPtr;	// 24K ????	22k+
extern Sint16 CardSlot;

Sint32 ByGame;
Sint16 SeasonMode;			// 0 = Password
Sint32 PreSimGame = FALSE;	// Didn't do CPU vs CPU, use stats
Sint32 JustPlayed = FALSE;

/*
enum MonthNames{
	JAN,
	FEB,
	MAR,
	APR,
	MAY,
	JUN,
	JUL,
	AUG,
	SEP,
	OCT,
	NOV,
	DEC
};


extern signed char WeekNum,
				TeamNum,
				SeasonNum,
				playoff[11],	// Who wins the playoffs (team 0 or 1)
				playoff_game_team[22],
				season[240];

typedef struct {
	char	month,		// Month
			day,		// Day of the month
			week,		// Week number of game
			away,		// Away Team number
			home;		// Home Team number
} schedule_t;

*/

//typedef struct {
//	char week,		// Week number of game
//		  away,		// Away Team number
//		  home;		// Home Team number
//		  month,
//		  day;
//} schedule_t;



// WEEKS IN A SEASON: 0 = Quarter, 1 = Half, 2 = Full, 3 = Playoffs
char Weeks[4][17] = {{1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17},
  							{1, 2,11,12,13,14,15,16, 0, 0, 0, 0, 0, 0, 0, 0, 0},
							{1, 2,11,12, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0},
  							{1, 2, 3, 4, 5, 6, 7, 8, 9,10,11,12,13,14,15,16,17}};


//char playoff_table[11][NUM_SEASONS] =
//{{1,0,1,0,1,0,0,0,0,1,1,0,1,0,1,0,1,1,0,0,1,0,1,1,0,0,1,0,0,0,1,0},// WC1  vs WC2 AFC
//	{1,0,1,0,0,1,1,1,1,1,0,1,1,1,0,1,0,1,1,1,0,0,1,0,1,0,1,1,1,0,1,1},// DIV3 vs WC3	AFC
//	{0,0,0,1,1,1,0,1,0,1,1,1,0,1,0,1,1,0,0,1,1,0,0,0,0,0,0,1,0,1,0,1},// WC1  vs WC2	NFC
//	{1,1,1,1,1,1,1,0,1,1,0,0,1,0,0,1,0,1,0,0,0,1,1,1,1,1,1,0,0,1,1,1},// DIV3 vs WC3	NFC
//	{1,1,1,1,1,0,1,1,1,0,1,0,0,0,1,1,1,1,1,0,1,1,0,0,0,1,0,1,1,1,1,1},// DIV1 vs W0 AFC
//	{0,1,0,1,1,1,1,1,0,1,1,1,1,1,0,0,1,1,0,0,1,1,0,1,0,1,0,1,0,1,0,0},// DIV2 vs W1 AFC
//	{1,1,1,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,1,0},// DIV1 vs W2 NFC
//	{1,1,0,1,1,1,1,1,0,1,1,1,1,1,1,1,1,1,1,0,1,0,1,1,1,0,0,1,1,1,0,1},// DIV2 vs W3 NFC
//	{1,0,1,1,0,0,1,0,0,0,0,0,0,0,0,0,0,1,1,1,0,1,0,0,0,1,1,0,0,1,0,0},// W4 vs W5	AFC
//	{0,0,1,0,1,1,0,0,1,0,0,1,0,1,0,0,1,0,1,1,0,1,0,0,1,0,0,0,1,0,0,1},// W6 vs W7	NFC
//	{0,1,1,1,1,0,1,1,0,1,0,0,0,1,1,0,0,1,0,1,1,1,0,1,0,0,0,1,1,1,1,1}};// AFC W8 vs NFC W9 (SUPERBOWL)

char playoff_table[11][NUM_SEASONS] =
   {{0,1,0,1,0,1,1,1,1,0,0,1,0,1,0,1,0,0,1,1,0,1,0,0,1,1,0,1,1,1,0,1},// WC1  vs WC2 AFC
	{0,1,0,1,1,0,0,0,0,0,1,0,0,0,1,0,1,0,0,0,1,1,0,1,0,1,0,0,0,1,0,0},// DIV3 vs WC3	AFC
	{1,1,1,0,0,0,1,0,1,0,0,0,1,0,1,0,0,1,1,0,0,1,1,1,1,1,1,0,1,0,1,0},// WC1  vs WC2	NFC
	{0,0,0,0,0,0,0,1,0,0,1,1,0,1,1,0,1,0,1,1,1,0,0,0,0,0,0,1,1,0,0,0},// DIV3 vs WC3	NFC
	{0,0,0,0,0,1,0,0,0,1,0,1,1,1,0,0,0,0,0,1,0,0,1,1,1,0,1,0,0,0,0,0},// DIV1 vs W0 AFC
	{1,0,1,0,0,0,0,0,1,0,0,0,0,0,1,1,0,0,1,1,0,0,1,0,1,0,1,0,1,0,1,1},// DIV2 vs W1 AFC
	{0,0,0,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1},// DIV1 vs W2 NFC
	{0,0,1,0,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,1,0,1,0,0,0,1,1,0,0,0,1,0},// DIV2 vs W3 NFC
	{0,1,0,0,1,1,0,1,1,1,1,1,1,1,1,1,1,0,0,0,1,0,1,1,1,0,0,1,1,0,1,1},// W4 vs W5	AFC
	{1,1,0,1,0,0,1,1,0,1,1,0,1,0,1,1,0,1,0,0,1,0,1,1,0,1,1,1,0,1,1,0},// W6 vs W7	NFC
	{1,0,0,0,0,1,0,0,1,0,1,1,1,0,0,1,1,0,1,0,0,0,1,0,1,1,1,0,0,0,0,0}};// AFC W8 vs NFC W9 (SUPERBOWL)




#if 1

#include "sched97.dat"

#else

				//   1996 NFL schedule

//BOBBY - The new additions will as follows eg. Jan 3
//											 MON DAY Week A   H
//											{ JAN, 3, 1,  4,  8},	//   0 OAK  @ BALT
//

schedule_t schedule[NUM_GAMES_IN_SEASON] = {
											 { SEP, 1, 1,  4,  8}, //   0 OAK  @ BALT
											 { SEP, 1, 1, 16, 18}, //   1 ATL  @ CARL
											 { SEP, 1, 1,  0,  7}, //   2 KC   @ HOUS
											 { SEP, 1, 1,  5,  9}, //   3 PITT @ JACK
											 { SEP, 1, 1, 29, 11}, //   4 ARIZ @ INDY
											 { SEP, 1, 1, 21, 23}, //   5 DET  @ MIN
											 { SEP, 1, 1,  6, 17}, //   6 CINC @ ST.L
											 { SEP, 1, 1, 26, 27}, //   7 PHIL @ WASH
											 { SEP, 1, 1, 20, 24}, //   8 GBAY @ TBAY
											 { SEP, 1, 1, 13, 12}, //   9 NENG @ MIAM
											 { SEP, 1, 1, 19, 15}, //	10 NEWO @ SF
											 { SEP, 1, 1, 14,  3}, //  11 NYJ  @ DENV
											 { SEP, 1, 1,  2,  1}, //  12 SEAT @ SD
											 { SEP, 1, 1, 10, 28}, //  13 BUFF @ NYG
											 { SEP, 2, 1, 25, 22}, //  14 DALL @ CHIC

											 { SEP, 8, 2, 23, 16}, //  15 MINN @ ATL
											 { SEP, 8, 2, 13, 10}, //  16 NENG @ BUFF
											 { SEP, 8, 2, 24, 21}, //  17 TBAY @ DET
											 { SEP, 8, 2,  7,  9}, //  18 HOUS @ JACK
											 { SEP, 8, 2,  4,  0}, //  19 OAK  @ KC
											 { SEP, 8, 2, 18, 19}, //  20 CARL @ NEWO
											 { SEP, 8, 2, 11, 14}, //  21 INDY @ NYJ
											 { SEP, 8, 2,  8,  5}, //  22 BALT @ PITT
											 { SEP, 8, 2, 22, 27}, //  23 CHIC @ WASH
											 { SEP, 8, 2, 28, 25}, //  24 NYG  @ DALL
											 { SEP, 8, 2,  6,  1}, //  25 CINC @ SD
											 { SEP, 8, 2, 17, 15}, //  26 ST.L @ SF
											 { SEP, 8, 2,  3,  2}, //  27 DENV @ SEAT
											 { SEP, 8, 2, 12, 29}, //  28 MAIM @ ARIZ
											 { SEP, 9, 2, 26, 20}, //  29 PHIL @ GBAY

											 { SEP, 15, 3, 23, 22}, //  30 MINN @ CHIC
											 { SEP, 15, 3, 19,  6}, //  31 NEWO @ CINC											 { 3,  1, 20},
											 { SEP, 15, 3,  1, 20}, //  32 SD   @ GBAY
											 { SEP, 15, 3,  8,  7}, //  33 BALT @ HOUS
											 { SEP, 15, 3, 14, 12},	//  34 NYJ  @ MIAM
											 { SEP, 15, 3, 29, 13},	//  35 ARIZ @ NENG
											 { SEP, 15, 3, 21, 26},	//  36 DET  @ PHIL
											 { SEP, 15, 3, 11, 25}, //  37 INDY @ DALL
											 { SEP, 15, 3,  9,  4}, //  38 JACK @ OAK
											 { SEP, 15, 3,  0,  2}, //  39 KC   @ SEAT
											 { SEP, 15, 3, 27, 28},	//  40 WASH @ NYG
											 { SEP, 15, 3, 24,  3}, //  41 TBAY @ DENV
											 { SEP, 16, 3, 10,  5}, //  42 BUFF @ PITT

											 { SEP, 22, 4, 15, 18}, //  43 SF   @ CARL
											 { SEP, 22, 4,  3,  0}, //  44 DENV @ KC
											 { SEP, 22, 4, 20, 23},	//  45 GBAY @ MINN
											 { SEP, 22, 4, 29, 19}, //  46 ARIZ @ NEWO
											 { SEP, 22, 4, 28, 14}, //  47 NYG  @ NYJ
											 { SEP, 22, 4, 27, 17},	//  48 WASH @ ST.L
											 { SEP, 22, 4,  2, 24},	//  49 SEAT @ TBAY
											 { SEP, 22, 4, 25, 10}, //  50 DALL @ BUFF
											 { SEP, 22, 4, 22, 21}, //  51 CHIC @ DET
											 { SEP, 22, 4,  9, 13}, //  52 JACK @ NENG
											 { SEP, 22, 4,  1,  4},	//  53 SD   @ OAK
											 { SEP, 22, 4, 26, 16}, //  54 PHIL @ ATL
											 { SEP, 23, 4, 12, 11},	//  55 MAIM @ INDY

											 { SEP, 29, 5, 19,  8},	//  56 NEWO @ BALT
											 { SEP, 29, 5,  4, 22}, //  57 OAK  @ CHIC
											 { SEP, 29, 5,  3,  6}, //  58 DENV @ CINC
											 { SEP, 29, 5, 18,  9},	//  59 CARL @ JACK
											 { SEP, 29, 5, 23, 28}, //  60 MINN @ NYG
											 { SEP, 29, 5,  7,  5}, //  61 HOUS @ PITT
											 { SEP, 29, 5, 21, 24}, //  62 DET  @ TBAY
											 { SEP, 29, 5, 17, 29}, //  63 ST.L @ ARIZ
											 { SEP, 29, 5,  0,  1}, //  64 KC   @ SD
											 { SEP, 29, 5, 16, 15}, //  65 ATL  @ SF
											 { SEP, 29, 5, 20,  2}, //  66 GBAY @ SEAT
											 { SEP, 29, 5, 14, 27}, //  67 NYJ  @ WASH
											 { SEP, 30, 5, 25, 26},	//  68 DALL @ PHIL

											 { OCT, 6, 6, 13,  8},	//  69 NENG @ BALT
											 { OCT, 6, 6, 20, 22},	//  70 GBAY @ CHIC
											 { OCT, 6, 6, 16, 21},	//  71 ATL  @ DET
											 { OCT, 6, 6,  2, 12},	//  72 SEAT @ MAIM
											 { OCT, 6, 6, 18, 23},	//  73 CARL @ MINN
											 { OCT, 6, 6,  4, 14},	//  74 OAK  @ NYJ
											 { OCT, 6, 6, 11, 10},	//  75 INDY @ BUFF
											 { OCT, 6, 6,  1,  3},	//  76 SD   @ DENV
											 { OCT, 6, 6,  9, 19},	//  77 JACK @ NEWO
											 { OCT, 6, 6, 15, 17},	//  78 SF   @ ST.L
											 { OCT, 6, 6,  7,  6},	//  79 HOUS @ CINC
											 { OCT, 7, 6,  5,  0},	//  80 PITT @ KC

											 { OCT, 13, 7,  7, 16},	//  81 HOUS @ ATL
											 { OCT, 13, 7, 12, 10},	//  82 MAIM @ BUFF
											 { OCT, 13, 7, 17, 18},	//  83 ST.L @ CARL
											 { OCT, 13, 7, 29, 25},	//  84 ARIZ @ DALL
											 { OCT, 13, 7, 14,  9},	//  85 NYJ  @ JACK
											 { OCT, 13, 7, 27, 13},	//  86 WASH @ NENG
											 { OCT, 13, 7, 22, 19},	//  87 CHIC @ NEWO
											 { OCT, 13, 7,  6,  5},	//  88 CINC @ PITT
											 { OCT, 13, 7, 23, 24},	//  89 MINN @ TBAY
											 { OCT, 13, 7, 26, 28},	//  90 PHIL @ NYG
											 { OCT, 13, 7, 21,  4},	//  91 DET  @ OAK
											 { OCT, 13, 7,  8, 11},	//  92 BALT @ INDY
											 { OCT, 14, 7, 15, 20},	//  93 SF   @ GBAY

											 { OCT, 17, 8,  2,  0},	//  94 SEAT @ KC
											 { OCT, 20, 8, 19, 18},	//  95 NEWO @ CARL
											 { OCT, 20, 8, 16, 25},	//  96 ATL  @ DALL
											 { OCT, 20, 8, 13, 11},	//  97 NENG @ INDY
											 { OCT, 20, 8, 12, 26},	//  98 MAIM @ PHIL
											 { OCT, 20, 8, 28, 27},	//  99 NYG  @ WASH
											 { OCT, 20, 8, 24, 29},	// 100 TBAY @ ARIZ
											 { OCT, 20, 8,  8,  3},	// 101 BALT @ DENV
											 { OCT, 20, 8,  5,  7},	// 102 PITT @ HOUS
											 { OCT, 20, 8, 10, 14},	// 103 BUFF @ NYJ
											 { OCT, 20, 8,  6, 15},	// 104 CINC @ SF
											 { OCT, 20, 8,  9, 17},	// 105 JACK @ ST.L
											 { OCT, 21, 8,  4,  1},	// 106 OAK  @ SD

											 { OCT, 27, 9,  5, 16},	// 107 PITT @ ATL
											 { OCT, 27, 9, 17,  8},	// 108 ST.L @ BALT
											 { OCT, 27, 9,  9,  6},	// 109 JACK @ CINC
											 { OCT, 27, 9, 28, 21},	// 110 NYG  @ DET
											 { OCT, 27, 9, 24, 20},	// 111 TBAY @ GBAY
											 { OCT, 27, 9, 15,  7},	// 112 SF   @ HOUS
											 { OCT, 27, 9, 18, 26},	// 113 CARL @ PHIL
											 { OCT, 27, 9, 11, 27},	// 114 INDY @ WASH
											 { OCT, 27, 9, 14, 29},	// 115 NYJ  @ ARIZ
											 { OCT, 27, 9,  0,  3},	// 116 KC   @ DENV
											 { OCT, 27, 9, 25, 12},	// 117 DALL @ MAIM
											 { OCT, 27, 9,  1,  2},	// 118 SD   @ SEAT
											 { OCT, 27, 9, 10, 13},	// 119 BUFF @ NENG
											 { OCT, 28, 9, 22, 23},	// 120 CHIC @ MINN

											 { NOV, 3, 10, 18, 16},	// 121 CARL @ ATL
											 { NOV, 3, 10,  6,  8},	// 122 CINC @ BALT
											 { NOV, 3, 10, 24, 22},	// 123 TBAY @ CHIC
											 { NOV, 3, 10, 26, 25},	// 124 PHIL @ DALL
											 { NOV, 3, 10, 21, 20},	// 125 DET  @ GBAY
											 { NOV, 3, 10,  1, 11},	// 126 SD   @ INDY
											 { NOV, 3, 10,  0, 23},	// 127 KC   @ MINN
											 { NOV, 3, 10, 29, 28},	// 128 ARIZ @ NYG
											 { NOV, 3, 10, 17,  5},	// 129 ST.L @ PITT
											 { NOV, 3, 10, 27, 10},	// 130 WASH @ BUFF
											 { NOV, 3, 10, 12, 13},	// 131 MAIM @ NENG
											 { NOV, 3, 10,  7,  2},	// 132 HOUS @ SEAT
											 { NOV, 3, 10, 15, 19},	// 133 SF   @ NEWO
											 { NOV, 4, 10,  3,  4},	// 134 DENV @ OAK

											 { NOV, 10, 11,  5,  6},	// 135 PITT @ CINC
											 { NOV, 10, 11, 20,  0},	// 136 GBAY @ KC
											 { NOV, 10, 11, 11, 12},	// 137 INDY @ MIAM
											 { NOV, 10, 11,  7, 19},	// 138 HOUS @ NEWO
											 { NOV, 10, 11, 13, 14},	// 139 NENG @ NYJ
											 { NOV, 10, 11, 10, 26},	// 140 BUFF @ PHIL
											 { NOV, 10, 11, 16, 17},	// 141 ATL  @ ST.L
											 { NOV, 10, 11,  4, 24},	// 142 OAK  @ TBAY
											 { NOV, 10, 11, 29, 27},	// 143 ARIZ @ WASH
											 { NOV, 10, 11, 22,  3},	// 144 CHIC @ DENV
											 { NOV, 10, 11,  8,  9},	// 145 BALT @ JACK
											 { NOV, 10, 11, 25, 15},	// 146 DALL @ SF
											 { NOV, 10, 11, 23,  2},	// 147 MINN @ SEAT
											 { NOV, 10, 11, 28, 18},	// 148 NYG  @ CARL
											 { NOV, 11, 11, 21,  1},	// 149 DET  @ SD

											 { NOV, 17, 12, 19, 16},	// 150 NEWO @ ATL
											 { NOV, 17, 12,  6, 10},	// 151 CINC @ BUFF
											 { NOV, 17, 12,  2, 21},	// 152 SEAT @ DET
											 { NOV, 17, 12, 14, 11},	// 153 NYJ  @ INDY
											 { NOV, 17, 12, 22,  0},	// 154 CHIC @ KC
											 { NOV, 17, 12,  3, 13},	// 155 DENV @ NENG
											 { NOV, 17, 12, 27, 26},	// 156 WASH @ PHIL
											 { NOV, 17, 12,  9,  5},	// 157 JACK @ PITT
											 { NOV, 17, 12, 18, 17},	// 158 CARL @ ST.L
											 { NOV, 17, 12, 28, 29},	// 159 NYG  @ ARIZ
											 { NOV, 17, 12, 12,  7},	// 160 MAIM @ HOUS
											 { NOV, 17, 12, 24,  1},	// 161 TBAY @ SD
											 { NOV, 17, 12,  8, 15},	// 162 BALT @ SF
											 { NOV, 17, 12, 23,  4},	// 163 MINN @ OAK
											 { NOV, 18, 12, 20, 25},	// 164 GBAY @ DALL

											 { NOV, 24, 13,  9,  8},	// 165 JACK @ BALT
											 { NOV, 24, 13, 14, 10},	// 166 NYJ  @ BUFF
											 { NOV, 24, 13, 21, 22},	// 167 DET  @ CHIC
											 { NOV, 24, 13, 16,  6},	// 168 ATL  @ CINC
											 { NOV, 24, 13, 18,  7},	// 169 CARL @ HOUS
											 { NOV, 24, 13,  1,  0},	// 170 SD   @ KC
											 { NOV, 24, 13,  3, 23},	// 171 DENV @ MINN
											 { NOV, 24, 13, 11, 13},	// 172 INDY @ NENG
											 { NOV, 24, 13, 19, 24},	// 173 NEWO @ TBAY
											 { NOV, 24, 13, 15, 27},	// 174 SF   @ WASH
											 { NOV, 24, 13, 26, 29},	// 175 PHIL @ ARIZ
											 { NOV, 24, 13, 25, 28},	// 176 DALL @ NYG
											 { NOV, 24, 13,  4,  2},	// 177 OAK  @ SEAT
											 { NOV, 24, 13, 20, 17},	// 178 GBAY @ ST.L
											 { NOV, 25, 13,  5, 12},	// 179 PITT @ MAIM

											 { NOV, 28, 14,  0, 21},	// 180 KC   @ DET
											 { NOV, 28, 14, 27, 25},	// 181 WASH @ DALL
											 { DEC,  1, 14,  5,  8},	// 182 PITT @ BALT
											 { DEC,  1, 14, 24, 18},	// 183 TBAY @ CARL
											 { DEC,  1, 14, 22, 20},	// 184 CHIC @ GBAY
											 { DEC,  1, 14, 10, 11},	// 185 BUFF @ INDY
											 { DEC,  1, 14,  6,  9},	// 186 CINC @ JACK
											 { DEC,  1, 14, 29, 23},	// 187 ARIZ @ MINN
											 { DEC,  1, 14, 28, 26},	// 188 NYG  @ PHIL
											 { DEC,  1, 14,  2,  3},	// 189 SEAT @ DENV
											 { DEC,  1, 14, 17, 19},	// 190 ST.L @ NEWO
											 { DEC,  1, 14,  7, 14},	// 191 HOUS @ NYJ
											 { DEC,  1, 14, 12,  4},	// 192 MAIM @ OAK
											 { DEC,  1, 14, 13,  1},	// 193 NENG @ SD
											 { DEC,  2, 14, 15, 16},	// 194 SF   @ ATL

											 { DEC,  5, 15, 26, 11},	// 195 PHIL @ INDY
											 { DEC,  8, 15, 17, 22},	// 196 ST.L @ CHIC
											 { DEC,  8, 15,  8,  6},	// 197 BALT @ CINC
											 { DEC,  8, 15,  3, 20},	// 198 DENV @ GBAY
											 { DEC,  8, 15,  9,  7},	// 199 JACK @ HOUS
											 { DEC,  8, 15, 28, 12},	// 200 NYG  @ MAIM
											 { DEC,  8, 15, 16, 19},	// 201 ATL  @ NEWO
											 { DEC,  8, 15,  1,  5},	// 202 SD   @ PITT
											 { DEC,  8, 15, 27, 24},	// 203 WASH @ TBAY
											 { DEC,  8, 15, 25, 29},	// 204 DALL @ ARIZ
											 { DEC,  8, 15, 14, 13},	// 205 NYJ  @ NENG
											 { DEC,  8, 15, 18, 15},	// 206 CARL @ SF
											 { DEC,  8, 15, 10,  2},	// 207 BUFF @ SEAT
											 { DEC,  8, 15, 23, 21},	// 208 MINN @ DET
											 { DEC,  9, 15,  0,  4},	// 209 KC   @ OAK

											 { DEC, 14, 16, 26, 14},	// 210 PHIL @ NYJ
											 { DEC, 14, 16,  1, 22},	// 211 SD   @ CHIC
											 { DEC, 15, 16, 17, 16},	// 212 ST.L @ ATL
											 { DEC, 15, 16,  8, 18},	// 213 BALT @ CARL
											 { DEC, 15, 16, 13, 25},	// 214 NENG @ DALL
											 { DEC, 15, 16, 20, 21},	// 215 GBAY @ DET
											 { DEC, 15, 16,  6,  7},	// 216 CINC @ HOUS
											 { DEC, 15, 16, 11,  0},	// 217 INDY @ KC
											 { DEC, 15, 16, 24, 23},	// 218 TBAY @ MINN
											 { DEC, 15, 16, 19, 28},	// 219 NEWO @ NYG
											 { DEC, 15, 16, 15,  5},	// 220 SF   @ PITT
											 { DEC, 15, 16, 27, 29},	// 221 WASH @ ARIZ
											 { DEC, 15, 16,  4,  3},	// 222 OAK  @ DENV
											 { DEC, 15, 16,  2,  9},	// 223 SEAT @ JACK
											 { DEC, 16, 16, 10, 12},	// 224 BUFF @ MAIM

											 { DEC, 21, 17, 13, 28},	// 225 NENG @ NYG
											 { DEC, 21, 17, 19, 17},	// 226 NEWO @ ST.L
											 { DEC, 22, 17,  7,  8},	// 227 HOUS @ BALT
											 { DEC, 22, 17,  0, 10},	// 228 KC   @ BUFF
											 { DEC, 22, 17,  5, 18},	// 229 PITT @ CARL
											 { DEC, 22, 17, 11,  6},	// 230 INDY @ CINC
											 { DEC, 22, 17, 23, 20},	// 231 MINN @ GBAY
											 { DEC, 22, 17, 16,  9},	// 232 ATL  @ JACK
											 { DEC, 22, 17, 12, 14},	// 233 MAIM @ NYJ
											 { DEC, 22, 17, 29, 26},	// 234 ARIZ @ PHIL
											 { DEC, 22, 17, 22, 24},	// 235 CHIC @ TBAY
											 { DEC, 22, 17,  2,  4},	// 236 SEAT @ OAK
											 { DEC, 22, 17, 25, 27},	// 237 DALL @ WASH
											 { DEC, 22, 17,  3,  1},	// 238 DENV @ SD
											 { DEC, 23, 17, 21, 15},	// 239 DET  @ SF
// Playoffs
											 { DEC, 23, 18, 22, 20},	// 239 DET  @ SF
											 { DEC, 23, 18, 26, 23},	// 239 DET  @ SF
											 { DEC, 23, 18,  3,  0},	// 239 T  @ SF
											 { DEC, 23, 18,  4, 10},	// 239 DET  @ SF

											 { DEC, 23, 19, 20, 15},	// 239 DET  @ SF
											 { DEC, 23, 19, 26, 25},	// 239 DET  @ SF
											 { DEC, 23, 19,  0,  5},	// 239 DET  @ SF
											 { DEC, 23, 19, 10, 12},	// 239 DET  @ SF

											 { DEC, 23, 20, 25, 15},	// 239 DET  @ SF
											 { DEC, 23, 20,  5, 12},	// 239 DET  @ SF

											 { DEC, 23, 21, 15, 12}};	// 239 DET  @ SF

#endif



signed char SeasonNum,
	  HumanTeamNum,
	  WeekNum,
	  playoff_game_team[22] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}, // Table for who plays who in playoffs (6vs7, 8vs2, etc)
	  playoff_slot[12] = {-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1,-1},	// Who makes the playoffs
	  playoff[11] = {0,0,0,0,0,0,0,0,0,0,0},	// Who wins the playoffs (team 0 or 1)
	  wins[17] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	  ties[17] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	  playoff_loss = 0,
	  tie[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0}, // Record
	  win[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	  loss[30] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	  rank[30],		// Rank @ end of season used to determine playoffs by record.
	  num_games[3] = {2, 8, 5}, // Tie, Home win, Away win
	  season[240];
int TotalScores[30];

/*************************************************************************
 * BUILD SEASON - Creates a season from the decoded password data.
 *************************************************************************/
void BuildSeason(void)
{
//	register int i,
	int i,
					 j,
					 num;
	char *Season_MFile,
		  file_name[35],
		  *SeasonResults;	// 240 total games

	Season_MFile = LoadCDFile("/SEASON/SEASON.MFL");// Got season
	sprintf(file_name, "SEASON%d.DAT", SeasonNum);
	SeasonResults = (char *)GetMultiFilePtr(Season_MFile, file_name);
	memcpy(season, SeasonResults, 240);
	FreeFileData(Season_MFile);							// Season File Loaded so dump memory

// Go through list and update your team in ram
	num = -1;
	if (GameOptions[GAME_OPTNS_SEASON_LEN] != 3)		// If not playoffs only season
		for (i = 0; i < 240; i++)	// Do entire season
			for (j = 0; j < 17; j++)
				if (Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][j] == schedule[i].week)
				{
					if (HumanTeamNum == schedule[i].home)		// 1 = Home Team Won
					{
						num++;
//						printf("I:%d HUM:%d WIN:%d TIE:%d NUM:%d\n", i, HumanTeamNum, wins[num], ties[num], num);
						if (ties[num])
							season[i] = 0;
						else
						if (wins[num]) 			// We won
							season[i] = 1;
						else
							season[i] = 2;
					}
					else
					if (HumanTeamNum == schedule[i].away)		// 2 = Away Team Won
					{
						num++;
//						printf("I:%d HUM:%d WIN:%d TIE:%d NUM:%d\n", i, HumanTeamNum, wins[num], ties[num], num);
						if (ties[num])
							season[i] = 0;
						else
						if (wins[num]) 			// We won
							season[i] = 2;
						else
							season[i] = 1;
					}
				}

	RankTeams();
}


/*************************************************************************
 * RANK TEAMS - Ranks teams 0-14 and 15-30 only the first 6 matter as only
 *		6 teams from each division make the playoffs...
 *************************************************************************/
void RankTeams(void)
{
	register short i,
						j,
						tmp;
//						init = 0;
	char tmprank[30];


	for (i = 0; i < 12; i++)
		playoff_slot[i] = -1;
	for (i = 0; i < 30; i++)
	{
		win[i] = 0;
		loss[i] = 0;
		tie[i] = 0;
	}

//	if (GameOptions[GAME_OPTNS_SEASON_LEN] != 3)
//	{
//		if (!SeasonStatsPtr)
//		{
//	 		init = 1;
//			SeasonStatsPtr	= (TEAMRAMStats *)AllocMem(NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats));
//		}
//
//		i = AllTeamMemoryCardMimicToSeasonStats();	// returns 0 if okay
//	}

	for (i = 0; i < NUM_GAMES_IN_SEASON; i++)	// Do entire season
//	for (i = 0; schedule[i].week <= Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][WeekNum-1]; i++)
//	for (i = 0; schedule[i].week <= 17; i++)
	{
		for (j = 0; j < 17; j++)
			if (Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][j] == schedule[i].week)
				switch (season[i])
				{
					case 0: tie[schedule[i].home]++;
							  tie[schedule[i].away]++;
							  break;

					case 1: win[schedule[i].home]++;			// Home Team Won
							  loss[schedule[i].away]++;
							  break;

					case 2: loss[schedule[i].home]++;		// Away Team Won
							  win[schedule[i].away]++;
							  break;
				}
	}

	for (i = 0; i < 30; i++)	// Set up array
	{
		tmprank[29-i] = (((win[i] * 2) + tie[i]) * 100) / ((win[i] + loss[i] + tie[i]) * 2);
		rank[i] = 29-i; // Reverse order to solve sort problem (when sorting it puts them in reverse order...)
	}

	for (i = 0; i < 29; i++)
		for (j = i+1; j < 30; j++)
		{
			if (tmprank[i] <= tmprank[j])
			{
				tmp = rank[i];
				rank[i] = rank[j];
				rank[j] = tmp;
				tmp = tmprank[i];
				tmprank[i] = tmprank[j];
				tmprank[j] = tmp;
			}
			if (tmprank[i] == tmprank[j])
			{
//				if ((SeasonStatsPtr+rank[i])->allplyrstats[GS_TOTAL_SCORE] <
//					(SeasonStatsPtr+rank[j])->allplyrstats[GS_TOTAL_SCORE] &&
//					GameOptions[GAME_OPTNS_SEASON_LEN] != 3)
				if (TotalScores[i] < TotalScores[j])
					{
						tmp = rank[i];
						rank[i] = rank[j];
						rank[j] = tmp;
						tmp = tmprank[i];
						tmprank[i] = tmprank[j];
						tmprank[j] = tmp;
					}
			}
		}

//	printf("\n");
//	for (i = 0; i < 30; i++)
//		printf("%2d> %2d %-10s - %d %d %d - %d SCORE:%d\n", i, rank[i], SpecificTeamData[rank[i]].TeamNickName, win[rank[i]], loss[rank[i]], tie[rank[i]], tmprank[i], TotalScores[i]);	// After...
//	printf("\n");

//	if (init)
//	{
//		FreeMem(SeasonStatsPtr);
//		SeasonStatsPtr = 0;
//	}
}


/*************************************************************************
 * DO PLAYOFFS - Figures out who is in the playoffs up to the week we are
 *		in...
 *
 * NOTE: The way playoffs work is they are prefigured what slots win, then
 * 	you fill in	the teams who should make it into the slots, and go from
 *		there...
 *************************************************************************/
void DoPlayoffs(void)
{
	register char	i,
						j,
						week,
  						num = 0,
						num1 = 0;
	char 				tmp[30];

// Figure out division leaders...
	for (i = 0; i < 26; i++)	// One division might really suck...
	{
		num = rank[i] / 5;
		if (playoff_slot[num] == -1)
			playoff_slot[num] = rank[i];
	}

// Sort Division Leaders to determine seating
	for (i = 0; i < 3; i++)
	{
		j = 0;
		while(playoff_slot[i] != rank[j])
			j++;
		tmp[i] = j;
	}

	for (i = 0; i < 2; i++)
		for (j = 1; j < 3; j++)
		{
			if (tmp[i] > tmp[j])
			{
				num = tmp[i];
				tmp[i] = tmp[j];
				tmp[j] = num;
				num = playoff_slot[i];
				playoff_slot[i] = playoff_slot[j];
				playoff_slot[j] = num;
			}
		}

	for (i = 3; i < 6; i++)
	{
		j = 0;
		while(playoff_slot[i] != rank[j])
			j++;
		tmp[i] = j;
	}
	for (i = 3; i < 5; i++)
		for (j = 4; j < 6; j++)
		{
			if (tmp[i] > tmp[j])
			{
//				printf("%d> %d  %d> %d\n", i, playoff_slot[i], j, playoff_slot[j]);
				num = tmp[i];
				tmp[i] = tmp[j];
				tmp[j] = num;
				num = playoff_slot[i];
				playoff_slot[i] = playoff_slot[j];
				playoff_slot[j] = num;
			}
		}

	num = 6;
	num1 = 9;
	for (i = 1; i < 27; i++)	// Worst case. (0 is always a div winner so don't check it)
	{
		week = 1;
		for (j = 0; j < 6; j++)
			if (rank[i] == playoff_slot[j])
				week = 0;

		if (week && rank[i] < 15 && num < 9)			// AFC Wildcards
//		if (week && rank[i] < 15 && num > 5)			// AFC Wildcards
			playoff_slot[num++] = rank[i];

		if (week && rank[i] > 14 && num1 < 12)			// NFC Wildcards
//		if (week && rank[i] > 14 && num1 > 8)			// NFC Wildcards
			playoff_slot[num1++] = rank[i];
	}

//	for (i = 0; i < 12; i++)
//		printf("%d> %d %s\n", i, playoff_slot[i], SpecificTeamData[playoff_slot[i]].TeamNickName);
//	printf("\n");
//	getch();

	DoPlayoffTree();
}


/*************************************************************************
 *
 *************************************************************************/
void DoPlayoffTree(void)
{
	register char	i,
						j,
						num = 0;
	signed char		playoff_game[22] = {6,7,2,8,9,10,5,11,0,0,1,1,2,3,3,4,4,5,6,7,8,9}; // Table for who plays who in playoffs (6vs7, 8vs2, etc)


// Figure out what teams are supposed to win.
// WEEK 1
// Game  0: slot 6 & 7		// AFC WC1  vs WC2
// Game  1: slot 2 & 8		// AFC DIV3 vs WC3
// Game  2: slot 9 & 10		// NFC WC1  vs WC2
// Game  3: slot 5 & 11		// NFC DIV3 vs WC3
	for (i = 0; i < 4; i++)
	{
		playoff_game_team[i*2] = j = playoff_slot[playoff_game[i*2]];			// First Team
		playoff_game_team[(i*2)+1] = num = playoff_slot[playoff_game[(i*2)+1]];	// Second Team
//		printf("%d:%10s VS %d:%s \n", j, SpecificTeamData[j].TeamNickName, num, SpecificTeamData[num].TeamNickName);
		if (!playoff_table[i][SeasonNum])
			playoff[i] = j;
		else
			playoff[i] = num;

		if (j == HumanTeamNum)			// Check Human Team
			if (playoff_loss == 1) 		// Lost in Week 1
				playoff[i] = num;
			else
				playoff[i] = j;

		if (num == HumanTeamNum)  		// Check Human Team
			if (playoff_loss == 1) 		// Lost in Week 1
				playoff[i] = j;
			else
				playoff[i] = num;
	}


// WEEK 2
// Game  4: slot W0 & 0		// AFC DIV1 vs W0
// Game  5: slot W1 & 1		// AFC DIV2 vs W1
// Game  6: slot W2 & 3		// NFC DIV1 vs W2
// Game  7: slot W3 & 4		// NFC DIV2 vs W3
	for (; i < 8; i++)
	{
		playoff_game_team[i*2] = j = playoff_slot[playoff_game[i*2+1]];  	// First Team
		playoff_game_team[(i*2)+1] = num = playoff[playoff_game[i*2]];			// Second Team
//		printf("%d:%s VS %d:%s \n", j, SpecificTeamData[j].TeamNickName, num, SpecificTeamData[num].TeamNickName);
		if (!playoff_table[i][SeasonNum])
			playoff[i] = j;
		else
			playoff[i] = num;

		if (j == HumanTeamNum)			// Check Human Team
			if (playoff_loss == 2) 		// Lost in Week 2
				playoff[i] = num;
			else
				playoff[i] = j;

		if (num == HumanTeamNum)  		// Check Human Team
			if (playoff_loss == 2) 		// Lost in Week 2
				playoff[i] = j;
			else
				playoff[i] = num;
	}

// WEEK 3 & 4
// Game  8: slot W4 & W5	// AFC W4 vs W5
// Game  9: slot W6 & W7 	// NFC W6 vs W7
// Game 10: slot W8 & W9	// AFC W8 vs NFC W9 (SUPERBOWL)
	for (; i < 11; i++)
	{
		playoff_game_team[i*2] = j = playoff[playoff_game[i*2]];			// First Team
		playoff_game_team[(i*2)+1] = num = playoff[playoff_game[(i*2)+1]];	// Second Team
//		printf("%d:%s VS %d:%s \n", j, SpecificTeamData[j].TeamNickName, num, SpecificTeamData[num].TeamNickName);
		if (!playoff_table[i][SeasonNum])
			playoff[i] = j;
		else
			playoff[i] = num;

		if (j == HumanTeamNum)			// Check Human Team
			if (playoff_loss == 3) 		// Lost in Week 3
				playoff[i] = num;
			else
				playoff[i] = j;

		if (num == HumanTeamNum)  		// Check Human Team
			if (playoff_loss == 3) 		// Lost in Week 3
				playoff[i] = j;
			else
				playoff[i] = num;
	}

//	for (i = 0; i < 22; i++)
//		printf("GAME:%D %d> %d %s\n", i/2, i, playoff_game_team[i], SpecificTeamData[playoff_game_team[i]].TeamNickName);
//	printf("\n");

//	printf("\nSEASON: %d\n", SeasonNum);
//	for (i = 0; i < 11; i++)
//		printf("%s\n",	SpecificTeamData[playoff[i]].TeamNickName);
//	getch();
}


/*************************************************************************
 *	CREATE PASSWORD - Generates password for season.
 *
 *	Team #			-  5 Bits	- 30 teams
 *	Week #			-  5 Bits	- 17 weeks, + 4 for playoffs... 21 total
 *	Season Table # -  6 Bits	- 32 seasons
 *	Win/Loss			- 16 Bits	- 0 Home Team won, 1 Away Team won
 *	Playoff Loss	-  4 Bits	- When playoffs were lost (if they were)
 *	Options			-  2 Bits
 *	Season Length	-  2 Bits
 * Ties				- 16 Bits
 *	Checksum			-  4 Bits
 *************************************************************************/
void CreatePassword(char password[15])
{
	int i,
		 check = 0;

	for (i = 0; i < 11; i++)
		password[i] = 0;

	// TEAM NUMBER
	password[0] = ((HumanTeamNum & 0x10) >> 1);
	password[1] = (HumanTeamNum & 0x0F);

	// WEEK NUMBER
	password[0] += ((WeekNum & 0x10) >> 2);
	password[2] = (WeekNum & 0x0F);

	// SEASON NUMBER
	password[0] += ((SeasonNum & 0x30) >> 3);
	password[3] = (SeasonNum & 0x0F);

	// WIN/LOSS
	for (i = 0; i < 4; i++)
	{
		password[i+4] = (wins[i*4] << 3) + (wins[i*4+1] << 2) + (wins[i*4+2] << 1) + wins[i*4+3];
		password[i+10] = (ties[i*4] << 3) + (ties[i*4+1] << 2) + (ties[i*4+2] << 1) + ties[i*4+3];
	}

	// PLAYOFF LOSS WEEK
	password[9] = playoff_loss;

	password[8] = (GameOptions[GAME_OPTNS_DIFFICULTY] & 0x03);
	password[8] += ((GameOptions[GAME_OPTNS_SEASON_LEN] & 0x03) << 2);

	for (i = 0; i < 14; i++)
	{
		check += password[i];
	}
	password[14] = check & 0x0F;

	for (i = 0; i < 14; i++)
	{
		password[i] ^= password[14];
		password[i] += 'A';
	}
	password[i] += 'A';
}


/*************************************************************************
 *	PARSE PASSWORD - Breaks the password back down and checks the checksum.
 *
 *	Team #			-  5 Bits	- 30 teams
 *	Week #			-  5 Bits	- 17 weeks, + 4 for playoffs... 21 total
 *	Season Table # -  6 Bits	-
 *	Win/Loss			- 16 Bits	- NOTE: We have to search for bys when setting this up
 *	Options			-  2 Bits
 *	Season Length	-  2 Bits
 * Ties				- 16 Bits
 *	Checksum			-  5 Bits
 *************************************************************************/
short ParsePassword(char *password)
{
	UBYTE Shadow[15];
	int i,
		 check = 0;

	memcpy(Shadow,MemoryCardMimicPtr->password,15);

	password[14] -= 'A';

	for (i = 0; i < 14; i++)		// We don't need pass[10] its the main checksum...
	{
		password[i] -= 'A';
		password[i] ^= password[14];
		check += password[i];
	}
	check = check & 0x0F;

	if (check != password[14])
	{
		DebugPrintf(0, "CHECKSUM FAILED... OLD CHECK: %d  NEW CHECK: %d\n", password[10], check);
		return(1);
	}

	// TEAM NUMBER
	HumanTeamNum = ((password[0] & 0x08) << 1) + password[1];
	if (HumanTeamNum < 0 || HumanTeamNum > 29)
	{
		HumanTeamNum = 0;
		return(1);
	}

	// WEEK NUMBER
	WeekNum = ((password[0] & 0x04) << 2) + password[2];
	if (WeekNum < 0 || WeekNum > 21)
	{
		WeekNum = 0;
		return(1);
	}

	// SEASON NUMBER
	SeasonNum = ((password[0] & 0x03) << 3) + password[3];

	// DIFFICULTY
	GameOptions[GAME_OPTNS_DIFFICULTY] = (password[8] & 0x03);

	// SEASON LENGTH
	GameOptions[GAME_OPTNS_SEASON_LEN] = ((password[8] >> 2) & 0x03);

	// WIN/LOSS
	for (i = 0; i < 4; i++)
	{
		wins[i*4]   = (password[i+4] & 0x08) >> 3;
		wins[i*4+1] = (password[i+4] & 0x04) >> 2;
		wins[i*4+2] = (password[i+4] & 0x02) >> 1;
		wins[i*4+3] = (password[i+4] & 0x01);

	// TIES
		ties[i*4]   = (password[i+10] & 0x08) >> 3;
		ties[i*4+1] = (password[i+10] & 0x04) >> 2;
		ties[i*4+2] = (password[i+10] & 0x02) >> 1;
		ties[i*4+3] = (password[i+10] & 0x01);
	}

	// PLAYOFF LOSS WEEK
	playoff_loss = password[9]; // Start week 18

//	printf("TEAM: %2d  ", HumanTeamNum);
//	printf("WEEK: %2d  ", WeekNum);
//	printf("SEASON: %2d  ", SeasonNum);
//	printf("Wins: ");
//	for (i = 0; i < 16; i++)
//		printf("%d ", wins[i]);
//	printf("\n");

	memcpy(MemoryCardMimicPtr->password,Shadow,15);

	return(0);
}


/*************************************************************************
 * UPDATE WINS - Updates the win array.  NOTE: Do not call when team loses
 *		and do NOT call when the team has a by that week as well.
 *
 * NOTE: ONLY IF A WIN, UPDATE WIN STRUCT.
 *************************************************************************/
void UpdateWins(void)
{
	register signed int	num = -1;
	register int i,
					 j;

	if (WeekNum > 17)
		return;

	for (i = 0; schedule[i].week <= Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][WeekNum-1] && i < 240; i++)	// Put your results in
	{
		for (j = 0; j < 17; j++)
		{
			if (schedule[i].week == Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][j])
			{
  				if (schedule[i].home == HumanTeamNum || schedule[i].away == HumanTeamNum)
  					num++;
			}
		}
	}

	wins[num] = 1;
}


/*************************************************************************
 * UPDATE TIES - Updates the tie array.  NOTE: Do not call when team loses
 *		and do NOT call when the team has a by that week as well.
 *
 * NOTE: ONLY IF A TIE, UPDATE TIE STRUCT.
 *************************************************************************/
void UpdateTies(void)
{
	register signed int	num = -1;
	register int i,
					 j;

	if (WeekNum > 17)
		return;

	for (i = 0; schedule[i].week <= Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][WeekNum-1] && i < 240; i++)	// Put your results in
	{
		for (j = 0; j < 17; j++)
		{
			if (schedule[i].week == Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][j])
			{
  				if (schedule[i].home == HumanTeamNum || schedule[i].away == HumanTeamNum)
  					num++;
			}
		}
	}

	ties[num] = 1;
}


/*************************************************************************
 * GET OPPONENT - Pass your team and week number
 *		Returns: team you play against.
 *				OR	-1 = team has a by this week
 *				OR -2 = team is out of or didn't make playoffs
 *************************************************************************/
signed short GetOpponent(short week, short team)
{
	int i,
		 j,
		 tmp;
	char num[5] = {0, 8, 16, 20, 22};	// Teams playing per week of playoffs

	// If short season, check to see if they are all done
	if (Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][week-1] == 0 && week < 18)
	{
		WeekNum = 18;
		week = 18;
	}

	if (week < 18)
	{
		for (i = 0; schedule[i].week <= Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][week-1] && i < 240; i++)	// Put your results in
			if (Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][week-1] == schedule[i].week)
  			{
  				if (schedule[i].home == team)
  					return(schedule[i].away+100);	// Away team is 100-129...

  				if (schedule[i].away == team)
  					return(schedule[i].home);	// Home team is 0-29
  			}
		return(-1);		// team not playing this week..
	}
	else	// Playoffs
	{
		if (playoff_loss || WeekNum > 21)
			return(-2);	// Playoffs, but HUMAN team didn't make it

		tmp = WeekNum;
		WeekNum = week;
		BuildSeason();
		DoPlayoffs();
		WeekNum = tmp;

 		tmp = week - 18;
		for (i = num[tmp]; i < num[tmp+1]; i += 2)
		{
			if	(playoff_game_team[i] == team)
				return(playoff_game_team[i+1]+100);		// Away team is 100-129...
			if	(playoff_game_team[i+1] == team)
				return(playoff_game_team[i]);				// Home team is 0-29
		}

		// CHECK IF HUMAN MADE PLAYOFFS WITH 1st OR 2nd BEST RECORD IN DIV
		if (week == 18)
			for (; i < num[tmp+2]; i += 2)
				if	((playoff_game_team[i] == team) || (playoff_game_team[i+1] == team))
					return(-1);	// By

		// FORCE PLAYOFF TEAM FOR PLAYOFF SEASON
		if (GameOptions[GAME_OPTNS_SEASON_LEN] == PLAYOFF_SEASON)	// ONLY GETS HERE IF IN PLAYOFF SEASON MODE && that team wouldn't normally make playoffs
		{
			if (HumanTeamNum < 15)
				tmp = 6;
			else
				tmp = 9;

			playoff_slot[tmp] = team;
			DoPlayoffTree();	// update w/ human team in game

	  		tmp = week - 18;
			for (i = num[tmp]; i < num[tmp+1]; i += 2)
			{
				if	(playoff_game_team[i] == team)
					return(playoff_game_team[i+1]+100);
				if	(playoff_game_team[i+1] == team)
					return(playoff_game_team[i]);
			}
		}
	}
	playoff_loss = -1;
	return(-2);	// Never Made Playoffs
}


/*************************************************************************
 * INIT NEW SEASON -
 *
 *	RETURNS: -1 - If this week is a by (only in playoff season mode)
 *************************************************************************/
short InitNewSeason(void)
{
	int i;
	signed int team;
	char s[40];


	for (i = 0; i < 17; i++)	// Init wins & ties
	{
		wins[i] = 0;
		ties[i] = 0;
	}

	for (i = 0; i < 30; i++)	// Init Total Scores
		TotalScores[i] = 0;

	SeasonNum = GetRandom(NUM_SEASONS);

	playoff_loss = 0;
	WeekNum = 1;
	if (GameOptions[GAME_OPTNS_SEASON_LEN] == PLAYOFF_SEASON)
	{
		WeekNum = 18;	// Go to playoffs
		GameType = PLAYOFF_GAME;
	}
	else
		GameType = SEASON_GAME;

	memset(MemoryCardMimicPtr, 0, sizeof(struct memorycardmimic));
	CreatePassword(MemoryCardMimicPtr->password);
	team = GetOpponent(WeekNum, HumanTeamNum);

	if (team == -1)	// Team they picked had a by first week of playoffs
		return(BY_GAME);

	if (team >= 100)	// They are the Away team
		GameTeamData[1].TeamNumber = team-100;
	else
	{
		GameTeamData[1].TeamNumber = HumanTeamNum;
		GameTeamData[0].TeamNumber = team;
	}

	return(0);
}


/*************************************************************************
 * CONTINUE RAM SEASON -
 *
 *	NOTE: If NO ram card, DONT call this function, get password and
 *			ParsePassword(PassWord);
 *
 *	RETURNS:  -1 - If this week is a by
 *				  0 - NO error
 *	  FILE_ERROR - File error
 *				 -2 - Playoff loss
 *************************************************************************/
short ContinueRamSeason(signed char port, signed char bank, char seas_num)
{
	register int team;
	int i;
	char  file_name[40],
		   *SeasonResults;

	short retcode = 0;

// READ DATA FROM CARD...
 	if (bank > -1)
	{
		sprintf(file_name, "%s%d", SEASONFILE, seas_num);
		retcode = ReadFile((unsigned short *)MemoryCardMimicPtr, port, bank, file_name);
		if (retcode == FILE_ERROR)
			return(retcode);
	}

	if(ParsePassword(MemoryCardMimicPtr->password)) return(BAD_PASSWORD);

	BuildSeason();			// Season up to this week, so standings are correct.
	if (WeekNum > 17)
		DoPlayoffs();

	WeekNum++;

	if (WeekNum < 18)
		GameType = SEASON_GAME;
	else
	if (WeekNum < 21)
		GameType = PLAYOFF_GAME;
	else
		GameType = CHAMPIONSHIP_GAME;

	team = GetOpponent(WeekNum, HumanTeamNum);


	if (team < 0)
	{
		return(team);		// This week was a by or Lost in playoffs
	}

	if (team >= 100)		// They are the Away team
	{
		GameTeamData[0].TeamNumber = HumanTeamNum;
		GameTeamData[1].TeamNumber = team-100;
	}
	else
	{
		GameTeamData[0].TeamNumber = team;
		GameTeamData[1].TeamNumber = HumanTeamNum;
	}

//	printf("\nSEASON: %d\n", SeasonNum);
//	for (i = 0; i < 22; i++)
//		printf("%d:%s\n", i,	SpecificTeamData[playoff_game_team[i]].TeamNickName);

	return(retcode);
}


/*************************************************************************
 * DO SEASON WEEK - If they have a RAMCARD write new game data to it...
 *
 *	NOTE: if NO ram card, DONT call this function, just
 *			CreatePassword(PassWord);
 *************************************************************************/


short DoSeasonWeek(signed char port, signed char bank, char seas_num)
{
	register int i = 0,
					 j,
					 error = 0,
					 numgames;
	Sint32 tries;

	char ch,
		  *file_ptr,
		  *Season_MFile,
		  *SeasonResults,
		  s[40],
		  sjisdesc[32],
		  file_name[35],
		  desc[30],
		  num[5] = {0, 8, 16, 20, 22};	// Teams playing per week of playoffs

	unsigned long *ptr;

	TEAMRAMStats BuffPtr[2];

	SeasonStatsPtr	= (TEAMRAMStats *)AllocMem(NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats));
	AllTeamMemoryCardMimicToSeasonStats();	// returns 0 if okay
	for (i = 0; i < 30; i++)
		TotalScores[i] = (SeasonStatsPtr+i)->allplyrstats[GS_TOTAL_SCORE];

// IF SIM GAME
	if (PreSimGame)
	{
		if (!GameOptions[GAME_OPTNS_QTR_LENGTH])	// Dont allow 2 minute season sim
			GameOptions[GAME_OPTNS_QTR_LENGTH] = 1;// 5 minute quarters...

		Season_MFile = LoadCDFile("/SEASON/SEASON.MFL");// Got season
		sprintf(file_name, "SEASON%d.DAT", SeasonNum);
		SeasonResults = (char *)GetMultiFilePtr(Season_MFile, file_name);
		memcpy(season, SeasonResults, 240);
		FreeFileData(Season_MFile);							// Season File Loaded so dump memory

		if (WeekNum < 18)
		{
//			for (i = (WeekNum-1) * 12; schedule[i].week <= WeekNum; i++)
			for (i = 0; schedule[i].week <= Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][WeekNum-1] && i < 240; i++)	// Put your results in
  				if (schedule[i].week == Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][WeekNum-1])
				{
//					printf("I:%d W:%d - %d vs %d - %d\n", i, schedule[i].week, schedule[i].away, schedule[i].home, season[i]);
					if (schedule[i].home == HumanTeamNum)
						switch(season[i])
						{
							case 0: UpdateTies();
									  break;
							case 1: UpdateWins();		// Home Team Won
									  break;
							default: break;
						}
					else
					if (schedule[i].away == HumanTeamNum)
						switch(season[i])
						{
							case 0: UpdateTies();
									  break;
							case 2: UpdateWins();		// Away Team Won
									  break;
							default: break;
						}
				}
		}
		else
		{
			for (i = num[WeekNum - 18]; i < num[WeekNum-17]; i += 2)
				if (playoff_game_team[i] == HumanTeamNum && playoff_table[i/2][SeasonNum])
					playoff_loss = WeekNum - 17;	// Week team lost the playoffs
				else
				if (playoff_game_team[i+1] == HumanTeamNum && !playoff_table[i/2][SeasonNum])
					playoff_loss = WeekNum - 17;	// Week team lost the playoffs

		}
	}
	else
// IF PLAYER WON
	if ((GameTeamData[0].TeamNumber == HumanTeamNum && GameTeamData[0].TotalScore > GameTeamData[1].TotalScore) ||
		 (GameTeamData[1].TeamNumber == HumanTeamNum && GameTeamData[1].TotalScore > GameTeamData[0].TotalScore) &&
		 !ByGame)
	{
		UpdateWins();	// Put a win into the password
	}
	else
// IF PLAYER TIED
	if ((GameTeamData[0].TeamNumber == HumanTeamNum && GameTeamData[0].TotalScore == GameTeamData[1].TotalScore) ||
		 (GameTeamData[1].TeamNumber == HumanTeamNum && GameTeamData[1].TotalScore == GameTeamData[0].TotalScore) &&
		 !ByGame)
	{
		UpdateTies();	// Put a win into the password
	}
	else
// IF PLAYER LOST IN PLAYOFFS && not a by
	if ((WeekNum > 17 && (GameTeamData[0].TeamNumber == HumanTeamNum || GameTeamData[1].TeamNumber == HumanTeamNum)) &&
		 !ByGame)
		playoff_loss = WeekNum - 17;	// Week team lost the playoffs

	BuildSeason();		// Season up to this week

	// CREATE NEW PASSWORD
//	CreatePassword(MemoryCardMimicPtr->password);

	// DO NORMAL SEASON
//	for (i = (WeekNum-1) * 12; schedule[i].week <= WeekNum; i++)
//		if (schedule[i].week == WeekNum)
	if (WeekNum < 18)
	{
		for (i = 0; schedule[i].week <= Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][WeekNum-1] && i < 240; i++)	// Put your results in
			if (Weeks[GameOptions[GAME_OPTNS_SEASON_LEN]][WeekNum-1] == schedule[i].week)
			{
				// One of the teams just played a game
//				if ((schedule[i].home == GameTeamData[0].TeamNumber || schedule[i].away == GameTeamData[0].TeamNumber)
				if ((schedule[i].home == HumanTeamNum || schedule[i].away == HumanTeamNum)
					&& PreSimGame == FALSE) // Just played
				{
					error = GameStatsToSeasonStats(GameTeamData[0].TeamNumber, &StatsData[0]);
					error = GameStatsToSeasonStats(GameTeamData[1].TeamNumber, &StatsData[1]);
					UpdateSeasonGameScores(i, &StatsData[0]);
				}
				else	// Non human controlled teams
				{
					switch (season[i])
					{
						case 0: ch = 'T';
								  break;

						case 1: ch = 'H';
								  break;

						case 2: ch = 'A';
								  break;
					}

//					sprintf(s, "/GAMES/GAMES%d/GAME%d.MFL", i / 30, i);
					sprintf(s, "/GAMES/GAMES%d/GAME%d.MFL", ((GameOptions[GAME_OPTNS_QTR_LENGTH] - 1) * 10) + i / 30, i);
					ptr = (unsigned long *)LoadCDFile(s);

					#if PC_DEBUG
					 if(!ptr) asm("break 0");
					#endif

					sprintf(s, "G%02d%02d%c%02d.GM%d", schedule[i].home, schedule[i].away, ch, GetRandom(num_games[season[i]]), GameOptions[GAME_OPTNS_QTR_LENGTH]);
//					sprintf(s, "G%02d%02d%c%02d.GM%d", schedule[i].home, schedule[i].away, ch, gamenum, GameOptions[GAME_OPTNS_QTR_LENGTH]);

					file_ptr = (char *)GetMultiFilePtr(ptr, s);

					if (!file_ptr)
					{
						sprintf(s, "G%02d%02d%c%02d.GM%d", schedule[i].home, schedule[i].away, ch, GetRandom(2), GameOptions[GAME_OPTNS_QTR_LENGTH]);
						DebugPrintf(0,"Sim Game Not Found/n");
						file_ptr = (char *)GetMultiFilePtr(ptr, s);
					}

					memcpy(BuffPtr, file_ptr, sizeof(TEAMRAMStats) * 2);
					if (ch == 'T')
					{
						BuffPtr[0].allplyrstats[GS_TIE] =
						BuffPtr[1].allplyrstats[GS_TIE] =
						BuffPtr[0].alloppplyrstats[GS_TIE] =
						BuffPtr[1].alloppplyrstats[GS_TIE] = 1;
					}
					error = GameStatsToSeasonStats(schedule[i].home, &BuffPtr[0]);
					error = GameStatsToSeasonStats(schedule[i].away, &BuffPtr[1]);
					UpdateSeasonGameScores(i, &BuffPtr[0]);
					FreeMem(ptr);
				}
			}
	}
	else 	// Playoffs
	{
		for (i = num[WeekNum - 18]; i < num[WeekNum-17]; i += 2)
		{
			// One of the teams just played a game
			if ((playoff_game_team[i] == HumanTeamNum || playoff_game_team[i+1] == HumanTeamNum)
				&& PreSimGame == FALSE) // Just played
			{
				MemoryCardMimicPtr->GameScores[i/2+240][0] = GameTeamData[0].TotalScore;
				MemoryCardMimicPtr->GameScores[i/2+240][1] = GameTeamData[1].TotalScore;
			}
			else	// Non human controlled teams
			{

				numgames = GetRandom(num_games[playoff_table[i/2][SeasonNum]+1]);

				// Sim Superbowl force a loss if simming...
				if (WeekNum == 21 && PreSimGame && (playoff_game_team[i] == HumanTeamNum || playoff_game_team[i+1] == HumanTeamNum))
				{
					if (playoff_game_team[i] == HumanTeamNum)
						ch = 'A';
					if (playoff_game_team[i+1] == HumanTeamNum)
						ch = 'H';
					playoff_loss = WeekNum - 17;	// Week team lost the playoffs

					numgames = GetRandom(4);
				}
				else
				{
					switch (playoff_table[i/2][SeasonNum])
					{
						case 0: ch = 'H';
								  break;

						case 1: ch = 'A';
								  break;
					}
				}

//				sprintf(s, "/GAMES/GAMES%d/GAME%d.MFL", (i/2+240) / 30, i/2+240);
				sprintf(s, "/GAMES/GAMES%d/GAME%d.MFL", ((GameOptions[GAME_OPTNS_QTR_LENGTH] - 1) * 10) + (i/2+240) / 30, i/2+240);
				ptr = (unsigned long *)LoadCDFile(s);

// Note 0 at end has to be changed to quarter length (0 = 2, 1 = 5, 2 = 10, 3 = 15)
				sprintf(s, "G%02d%02d%c%02d.GM%d", schedule[i/2+240].home, schedule[i/2+240].away, ch, numgames, GameOptions[GAME_OPTNS_QTR_LENGTH]);
				file_ptr = (char *)GetMultiFilePtr(ptr, s);

				if (!file_ptr)
				{
					sprintf(s, "G%02d%02d%c%02d.GM%d", schedule[i/2+240].home, schedule[i/2+240].away, ch, GetRandom(2), GameOptions[GAME_OPTNS_QTR_LENGTH]);
					DebugPrintf(0,"Sim Playoff Game Not Found/n");
					file_ptr = (char *)GetMultiFilePtr(ptr, s);
				}


				memcpy(BuffPtr, file_ptr, sizeof(TEAMRAMStats) * 2);
				UpdateSeasonGameScores(i/2+240, &BuffPtr[0]);
				FreeMem(ptr);
			}
		}
	}

	CreatePassword(MemoryCardMimicPtr->password);

	for (i = 0; i < 30; i++)
		TotalScores[i] = (SeasonStatsPtr+i)->allplyrstats[GS_TOTAL_SCORE];
	AllTeamSeasonStatsToMemoryCardMimic();	// returns 0 if okay

	if (bank > -1)
	{
		sprintf(s, "%s%d", SEASONFILE, seas_num);
		sprintf(desc, "%s%d %s", SEASONSTR ,WeekNum, SpecificTeamData[HumanTeamNum].TeamNickName);
		String2SJIS((unsigned char *)desc, (unsigned short *)sjisdesc);
		i = 0;

		tries = 3;

		while (OpenFile((char *)MemoryCardMimicPtr, sizeof(struct memorycardmimic), port, bank, s, sjisdesc)
				&& tries)
		{
			tries--;
			if(!tries) error = WRITE_ERROR;
		}
	}

	FreeMem(SeasonStatsPtr);
	SeasonStatsPtr = 0;

	JustPlayed = TRUE;

	return(error);
}


/*************************************************************************/

void UpdateSeason(void)
{
	if(SeasonMode)
		GetCardBank();
	else
		CardSlot = -1;

	if (ByGame == BY_GAME)
		LoadTeamPic(HumanTeamNum);

	DoSeasonWeek(0,CardSlot,0);

}

/*************************************************************************/

int ContinueSeason(void)
{

	if(SeasonMode)
	{
		if(GetCardBank())
		{
			return(ContinueRamSeason(0,CardSlot,0));
		}
		else return(FILE_ERROR);
	}
	else
	{
		return(ContinueRamSeason(0,-1,0));
	}
}
