/*************************************************************************
 *	shell.c for the PSX Football.
 *
 *	Copyright 1997 GAMETEK
 *	All rights reserved.
 *
 * Written by Tom McWilliams
 * Started: 3-17-97
 *
 *	Notes: Includes Crazy Menu Interpreter
 *
 *************************************************************************/

#include <stdio.h>
#include <sys/types.h>

#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include "prjtypes.h"
#include "debug.h"

#include "overlays.h"
#include "mmvram.h"
#include "multifil.h"
#include "fonts.h"
#include "psxpad.h"

#include "menus.h"

#include "card.h"		// for card stuff-will be moved out

// for shell stuff
#include "fbgequ.h"
#include "fbequ.h"
#include "fbram.h"
#include "shell.h"

#include "shellsys.h"

// external prototypes

extern int GameMode;

// menu stuff

#define MF_ACTIVE  0x0001
#define MF_DISPLAY 0x0002

struct MenuCtrl
{
	int Flags;					// menu flags
	int Pad;						// pad for control (-1 = all)
	int Level;					// this menu's level (for menu trees)
	int Mode;						// this menu's mode
	menu_t MenuPtr;			// top of menu data
	var_t *VarPtr;			// ptr to this menu's variable
	BYTE *Font;					// font type
	BYTE Spacing;				// font spacing
	int Color;					// current display color
	int RegColor;				// regular text color
	int HiColor;				// hilight color
	int LoColor;				// lolight color
	int TextTop;				// for clipping
	int TextBottom;
	int ScrollOn;				// flag to use TextX,TextY global offsets
};

void InitMenu (void *menu);
int DisplayMenu (struct MenuCtrl *menuctrl);
int DoMenu (menu_t menu);
int UpdateMenu (void);

// global variables

MultiFile_t *Shell_MFile;		// points to the general purpose info mfile for this shell
MultiFile_t *Home_MFile;		// for helmet graphics
MultiFile_t *Away_MFile;

MultiFile_t *ShellData;			// data always resident for shell

ShellObj **ShellObject;			// ptr to shell graphics object ptrs

int HomeHelmetR;		// for spinning helmets
int AwayHelmetR;

// external global variables

extern WORD SeasonMode;

// hostile variables

LONG	TeamNo;				// these are used by Nigel in menu stuff.  They should be replaced with global game vars
LONG	PadNo;
LONG	WhichTeam;
LONG	OppStats;


UBYTE MyGameType;		// ?? (this should be an int)
int GameMode;				// ??

ULONG *This_MFile;		// used by team stats etc..

WORD SyncFlag = TRUE;	// Flag to synchronise display buffer flip required for shell/movies

// font stuff

int MenuScroll;					// general flag
int CursorX,CursorY;		// current cursor position
int TextX,TextY;				// offset for text menus (clipping added right now)

// menu stuff

struct MenuCtrl MenuCtrl;
int MenuLevel;			// current active menu level
int MenuLevelMsg;		// signal for new menu level
void (*MenuExec)();	// code to execute called from menu
int MenuPad[2];			// current pad to operate menus (home,away)

#define HOMEPAD 0
#define AWAYPAD 1

#define CENTER_X 256
#define CENTER_Y 120

#define PEN_HILITE PEN_YELLOW
#define PEN_LOLITE PEN_BLUE

#define ONLEFT  (CENTER_X-128)
#define ONRIGHT (CENTER_X+128)

//
// prototypes for functions called from menus
//

// external
void Display_Substitute (int pad);

void DisplayTmStats ();
void DisplayTmStandings ();
void DoCredits ();

// internal
void PlayerStats ();
void LoadEditedPlays ();
void CreateTeam ();
void SeasonCreateTeam ();
void ViewTeams ();
void DoSubstitutions ();
void LeagueLeaders ();
void PlayerStats ();
void EditControls ();

// menu stuff
void PDScroller ();
void PDScrollUp ();

void MenuScreenSetup ();
void MenuScreenCleanup ();
void MenuScreenCleanupH ();

// machine specific calls for menus
void PrintMenuText (char *str,struct MenuCtrl *mc);
void PrintMenuTextCentered (char *str,struct MenuCtrl *mc);
int GetCaptainPadNum (int team);

void NewRAMCard ();
void LoadRAMCard ();
void NewPassword ();
void LoadPassword ();


///////////////////////////////////////////////////////////////////////////
//
// generic submenus
//

var_t TestVar0;			// TEMP!!
var_t TestVar1;			// TEMP!!
var_t TestVar2;			// TEMP!!
var_t TestVar3;			// TEMP!!

submenu_t MenuButtons[]=
{
	M_hmenu,NULL,
	M_jpad+JPADtriangle,M_exit,-1,M_eol,	// backer upper
	M_xy,60,220,
	M_call,&DrawTriangle,M_text,"    Back     ",
	M_call,&DrawCross,M_text,"    Select",
	M_end
};

var_t HomeAwaySel;
submenu_t HomeAwayMenu[]=
{
	M_hmenu,&HomeAwaySel,
	M_ACTIVE,
	M_color,PEN_GREY,M_hicolor,PEN_HILITE,
	M_x,CENTER_X-128,M_textc,"home",M_eol,
	M_x,CENTER_X+128,M_textc,"away",M_eol,
	M_end
};

submenu_t OnOffMenu[]=
{
	M_hmenu,NULL,
	M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
	M_text,"on ",M_eol,
	M_text,"off",M_eol,
	M_end
};

submenu_t VolMenu[]=
{
	M_hmenu,NULL,
	M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
	M_text,"0  ",M_eol,
	M_text,"1  ",M_eol,
	M_text,"2  ",M_eol,
	M_text,"3  ",M_eol,
	M_text,"4  ",M_eol,
	M_text,"5  ",M_eol,
	M_end
};

///////////////////////////////////////////////////////////////////////////
//
// Pull Down Menus
//

var_t PDSel;		// shared by pull down menus

submenu_t StadiumOptionsMenu[]=
{
	M_pdmenu,&PDSel,
	M_ACTIVE,
	M_x,40,M_dy,12,M_text,"Field",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_STADIUM],
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"grass ",M_eol,
		M_text,"astroturf ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Weather",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_WEATHER],
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"normal ",M_eol,
		M_text,"rain ",M_eol,
		M_text,"snow ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Wind",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_WIND],
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"none ",M_eol,
		M_text,"low ",M_eol,
		M_text,"high ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Wind Direction",M_x,240,
		M_hmenu,&TestVar0,
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"east ",M_eol,
		M_text,"west ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Scrimmage Marker",M_x,240,
		M_linkvar,OnOffMenu,&GameOptions[GAME_OPTNS_SCRIMMAGE_MARKER],M_eol,
	M_x,40,M_dy,12,M_text,"1st Down Marker",M_x,240,
		M_linkvar,OnOffMenu,&GameOptions[GAME_OPTNS_1STDOWN_MARKER],M_eol,
	M_end
};

submenu_t GameOptionsMenu[]=
{
	M_pdmenu,&PDSel,
	M_ACTIVE,
	M_x,40,M_dy,12,M_text,"Quarter Length",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_QTR_LENGTH],
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"2 ",M_eol,
		M_text,"5 ",M_eol,
		M_text,"10 ",M_eol,
		M_text,"15 ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Difficulty",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_DIFFICULTY],
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"easy ",M_eol,
		M_text,"normal ",M_eol,
		M_text,"hard ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Speed",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_GAME_SPEED],
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"slow ",M_eol,
		M_text,"normal ",M_eol,
		M_text,"fast ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Fumbles",M_x,240,
		M_linkvar,OnOffMenu,&GameOptions[GAME_OPTNS_FUMBLES],M_eol,
	M_x,40,M_dy,12,M_text,"Human Effect",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_HUMAN_EFFECT],
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"normal ",M_eol,
		M_text,"high ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Show Names",M_x,240,
		M_linkvar,OnOffMenu,&GameOptions[GAME_OPTNS_SHOW_NAMES],M_eol,
	M_x,40,M_dy,12,M_text,"Score Indicator",M_x,240,
		M_linkvar,OnOffMenu,&TestVar0,M_eol,
	M_x,40,M_dy,12,M_text,"Man In Motion",M_x,240,
		M_linkvar,OnOffMenu,&TestVar1,M_eol,
	M_end
};

submenu_t TeamOptionsMenu[]=
{
	M_pdmenu,&PDSel,
	M_ACTIVE,
	M_x,40,M_dy,12,M_text,"Controls",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_CONTROLS],
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"default ",M_eol,
		M_text,"change ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Passing Interface",M_x,240,
		M_hmenu,&TestVar0,
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"this ",M_eol,
		M_text,"that ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Play Call Method",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_PLAY_CALL_METHOD],
		M_HILITE,M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"formation ",M_eol,
		M_text,"run/pass ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Coverage Icons",M_x,240,
		M_setpad,HOMEPAD,M_text,"home ",M_linkvar,OnOffMenu,&GameOptions[GAME_OPTNS_REC_ICONS],
		M_dx,96,
		M_setpad,AWAYPAD,M_text,"away ",M_linkvar,OnOffMenu,&GameOptions[GAME_OPTNS_REC_ICONS_AWAY],
		M_eol,
	M_x,40,M_dy,12,M_text,"Pass Arc",M_x,240,
		M_setpad,HOMEPAD,M_text,"home ",M_linkvar,OnOffMenu,&GameOptions[GAME_OPTNS_PASS_ARC],
		M_dx,96,
		M_setpad,AWAYPAD,M_text,"away ",M_linkvar,OnOffMenu,&GameOptions[GAME_OPTNS_PASS_ARC_AWAY],
		M_eol,
	M_x,40,M_dy,12,M_text,"Auto Pitch/Handoff",M_x,240,
		M_setpad,HOMEPAD,M_text,"home ",M_linkvar,OnOffMenu,&TestVar0,
		M_dx,96,
		M_setpad,AWAYPAD,M_text,"away ",M_linkvar,OnOffMenu,&TestVar0,
		M_eol,
	M_x,40,M_dy,12,M_text,"Auto QB",M_x,240,
		M_setpad,HOMEPAD,M_text,"home ",M_linkvar,OnOffMenu,&TestVar1,
		M_dx,96,
		M_setpad,AWAYPAD,M_text,"away ",M_linkvar,OnOffMenu,&TestVar1,
		M_eol,
	M_x,40,M_dy,12,M_text,"Bluff Mode",M_x,240,
		M_setpad,HOMEPAD,M_text,"home ",M_linkvar,OnOffMenu,&TestVar2,
		M_dx,96,
		M_setpad,AWAYPAD,M_text,"away ",M_linkvar,OnOffMenu,&TestVar2,
		M_eol,
	M_setpad,-1,M_end
};

submenu_t PenaltyOptionsMenu[]=
{
	M_pdmenu,&PDSel,
	M_ACTIVE,
	M_x,40,M_dy,12,M_text,"Delay of Game",M_x,280,
		M_linkvar,OnOffMenu,&TestVar0,M_eol,
	M_x,40,M_dy,12,M_text,"Intentional Grounding",M_x,280,
		M_linkvar,OnOffMenu,&TestVar1,M_eol,
	M_x,40,M_dy,12,M_text,"Offsides",M_x,280,
		M_linkvar,OnOffMenu,&TestVar2,M_eol,
	M_x,40,M_dy,12,M_text,"Interference",M_x,280,
		M_linkvar,OnOffMenu,&TestVar3,M_eol,
	M_x,40,M_dy,12,M_text,"False Start",M_x,280,
		M_linkvar,OnOffMenu,&TestVar0,M_eol,
	M_x,40,M_dy,12,M_text,"Personal Fouls",M_x,280,
		M_linkvar,OnOffMenu,&TestVar1,M_eol,
	M_end
};

submenu_t VolumeOptionsMenu[]=
{
	M_pdmenu,&PDSel,
	M_ACTIVE,
	M_x,40,M_dy,12,M_text,"Announcer Volume",M_x,280,
		M_linkvar,VolMenu,&GameOptions[GAME_OPTNS_ANNOUNCER_VOL],M_eol,
	M_x,40,M_dy,12,M_text,"Referee Volume",M_x,280,
		M_linkvar,VolMenu,&GameOptions[GAME_OPTNS_REFEREE_VOL],M_eol,
	M_x,40,M_dy,12,M_text,"Crowd Volume",M_x,280,
		M_linkvar,VolMenu,&GameOptions[GAME_OPTNS_CROWD_VOL],M_eol,
	M_x,40,M_dy,12,M_text,"Sound Effects Volume",M_x,280,
		M_linkvar,VolMenu,&GameOptions[GAME_OPTNS_SFX_VOL],M_eol,
	M_x,40,M_dy,12,M_text,"Music Volume",M_x,280,
		M_linkvar,VolMenu,&GameOptions[GAME_OPTNS_MUSIC_VOL],M_eol,
	M_end
};

////////////////////////////////////////////////////////////////////
var_t PDMenuSel;
submenu_t GamePDMenu[]=
{
	M_hmenu,&PDMenuSel,
	M_MUTEX,
	M_x,CENTER_X,
	M_textc,"Stadium Options)",M_link,StadiumOptionsMenu,M_eol,
	M_textc,"(Game Options)",M_link,GameOptionsMenu,M_eol,
	M_textc,"(Team Options)",M_link,TeamOptionsMenu,M_eol,
	M_textc,"(Penalty Options)",M_link,PenaltyOptionsMenu,M_eol,
	M_textc,"(Volume Options",M_link,VolumeOptionsMenu,M_eol,
	M_end
};

////////////////////////////////////////////////////////////////////
//
// Core Menus
//
var_t MenuSel;

menu_t PreGameMenu[]=
{
	&MenuScreenSetup,&MenuScreenCleanupH,
	M_vmenu,NULL,
	M_call,&SpinHomeHelmet,		// show name & spinning helmets
	M_call,&DrawHomeName,
	M_call,&SpinAwayHelmet,
	M_call,&DrawAwayName,
	M_vmenu,&MenuSel,
	M_HILITE,M_color,PEN_WHITE,M_hicolor,PEN_HILITE,
	M_scrollon,M_top,108,M_bot,208,
	M_xy,CENTER_X,108-12,
	M_dy,12,M_textc,"Start Game",M_jpad+JPADx,M_exit,1,M_eol,
	M_dy,12,M_textc,"Load Edited Plays",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&LoadEditedPlays,M_eol,
	M_dy,12,M_textc,"Ask Jimmy",M_link,HomeAwayMenu,M_jpad+JPADx,    M_eol,
	M_dy,12,M_textc,"Player Stats",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&PlayerStats,M_eol,
	M_dy,12,M_textc,"Substitutions",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&DoSubstitutions,M_eol,
	M_dy,12,M_textc,"Team Comparison",M_eol,
	M_dy,12,M_link,GamePDMenu,M_ifactive,M_call,&PDScrollUp,M_eol,
	M_end,
	M_link,MenuButtons,
	M_call,&PDScroller,		// scroll menu into clipped window
	M_end
};

menu_t NewSeasonMenu[]=
{
	&MenuScreenSetup,&MenuScreenCleanup,
	M_vmenu,NULL,
		M_vmenu,&MenuSel,
			M_HILITE,M_color,PEN_WHITE,M_hicolor,PEN_HILITE,
			M_xy,CENTER_X,20,
			M_dy,12,M_textc,"Start RAM Card Season",M_jpad+JPADx,M_exec,&NewRAMCard,M_exit,1,M_eol,
			M_dy,12,M_textc,"Start Password Season",M_jpad+JPADx,M_exec,&NewPassword,M_exit,1,M_eol,
		M_end,
	M_link,MenuButtons,
	M_end
};

menu_t ContinueSeasonMenu[]=
{
	&MenuScreenSetup,&MenuScreenCleanup,
	M_vmenu,NULL,
		M_vmenu,&MenuSel,
			M_HILITE,M_color,PEN_WHITE,M_hicolor,PEN_HILITE,
			M_xy,CENTER_X,20,
			M_dy,12,M_textc,"Continue RAM Card Season",M_jpad+JPADx,M_exec,&LoadRAMCard,M_exit,1,M_eol,
			M_dy,12,M_textc,"Continue Password Season",M_jpad+JPADx,M_exec,&LoadPassword,M_exit,1,M_eol,
		M_end,
	M_link,MenuButtons,
	M_end
};

menu_t SetupSeasonMenu[]=
{
	&MenuScreenSetup,&MenuScreenCleanup,
	M_vmenu,NULL,
	M_xy,CENTER_X,60,M_call,&SpinHomeHelmetAt,	// show helmet
	M_xy,CENTER_X,100,M_call,&DrawHomeNameAt,		// show name
	M_vmenu,&MenuSel,
	M_HILITE,M_color,PEN_WHITE,M_hicolor,PEN_HILITE,
	M_xy,CENTER_X,140,
	M_x,40,M_dy,12,M_text,"Length",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_SEASON_LEN],
		M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"Full ",M_eol,
		M_text,"8 ",M_eol,
		M_text,"4 ",M_eol,
		M_text,"Playoffs ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Difficulty",M_x,240,
		M_hmenu,&GameOptions[GAME_OPTNS_DIFFICULTY],
		M_color,PEN_GREY,M_hicolor,PEN_HILITE,M_locolor,PEN_LOLITE,
		M_text,"Easy ",M_eol,
		M_text,"Normal ",M_eol,
		M_text,"Hard ",M_eol,
		M_end,M_eol,
	M_x,40,M_dy,12,M_text,"Create Team",M_x,240,
		M_jpad+JPADx,M_exec,&SeasonCreateTeam,M_eol,
	M_x,40,M_dy,12,M_text,"Continue",M_x,240,
		M_jpad+JPADx,M_exit,1,M_eol,
	M_end,
	M_link,MenuButtons,
	M_end
};


menu_t SeasonMenu[]=
{
	&MenuScreenSetup,&MenuScreenCleanup,
	M_vmenu,NULL,
	M_vmenu,&MenuSel,
	M_HILITE,M_color,PEN_WHITE,M_hicolor,PEN_HILITE,
	M_xy,CENTER_X,20,
	M_dy,12,M_textc,"Play Game",M_jpad+JPADx,M_exit,1,M_eol,
	M_dy,12,M_textc,"NFL Schedule",M_jpad+JPADx,M_exec,&LoadEditedPlays,M_eol,
	M_dy,12,M_textc,"Standings",M_jpad+JPADx,M_exec,&DisplayTmStandings,M_eol,
	M_dy,12,M_textc,"View Team",M_jpad+JPADx,M_exec,&ViewTeams,M_eol,
	M_dy,12,M_textc,"League Leaders",M_jpad+JPADx,M_exec,&LeagueLeaders,M_eol,
	M_end,
	M_link,MenuButtons,
	M_end
};


menu_t SeasonPreGameMenu[]=
{
	&MenuScreenSetup,&MenuScreenCleanup,
	M_vmenu,&MenuSel,
	M_HILITE,M_color,PEN_WHITE,M_hicolor,PEN_HILITE,
	M_xy,CENTER_X,20,
	M_dy,12,M_textc,"Start Game",M_jpad+JPADx,M_exit,1,M_eol,
	M_dy,12,M_textc,"Load Edited Plays",M_jpad+JPADx,M_exec,&LoadEditedPlays,M_eol,
	M_dy,12,M_textc,"Ask Jimmy",M_link,HomeAwayMenu,M_jpad+JPADx,    M_eol,
	M_dy,12,M_textc,"Player Stats",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&PlayerStats,M_eol,
	M_dy,12,M_textc,"Substitutions",M_jpad+JPADx,M_exec,&DoSubstitutions,M_eol,
	M_dy,12,M_textc,"Team Comparison",M_eol,
	M_dy,12,M_textc,"Options",M_eol,
	M_dy,12,M_link,GamePDMenu,M_eol,
	M_end,
	M_link,MenuButtons,
	M_end
};


menu_t PauseMenu[]=
{
	&MenuScreenSetup,&MenuScreenCleanup,
	M_vmenu,&MenuSel,
	M_HILITE,M_color,PEN_WHITE,M_hicolor,PEN_HILITE,
	M_xy,CENTER_X,20,
	M_dy,12,M_textc,"Resume Game",M_jpad+JPADx,M_exit,1,M_eol,
	M_dy,12,M_textc,"Quit Game",M_jpad+JPADx,M_exit,2,M_eol,
	M_dy,12,M_textc,"Camera",M_eol,
	M_dy,12,M_textc,"Time Outs",M_eol,
	M_dy,12,M_textc,"Team Stats", /*M_link,HomeAwayMenu,*/ M_jpad+JPADx,M_exec,&DisplayTmStats,M_eol,
	M_dy,12,M_textc,"Player Stats",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&PlayerStats,M_eol,
	M_dy,12,M_textc,"Substitutions",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&DoSubstitutions,M_eol,
	M_dy,12,M_textc,"Set Audibles",M_link,HomeAwayMenu,M_jpad+JPADx,    M_eol,
	M_dy,12,M_textc,"No Huddle",M_link,HomeAwayMenu,M_jpad+JPADx,    M_eol,
	M_dy,12,M_textc,"Defense Agressiveness",M_link,HomeAwayMenu,M_jpad+JPADx,    M_eol,
	M_dy,12,M_textc,"Block Assignments",M_link,HomeAwayMenu,M_jpad+JPADx,    M_eol,
	M_dy,12,M_textc,"Options",M_eol,
	M_dy,12,M_link,GamePDMenu,M_eol,
	M_end
};


menu_t HalftimeMenu[]=
{
	&MenuScreenSetup,&MenuScreenCleanup,
	M_vmenu,&MenuSel,
	M_HILITE,M_color,PEN_WHITE,M_hicolor,PEN_HILITE,
	M_xy,CENTER_X,20,
	M_dy,12,M_textc,"Resume Game",M_jpad+JPADx,M_exit,1,M_eol,
	M_dy,12,M_textc,"Load Edited Plays",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&LoadEditedPlays,M_eol,
	M_dy,12,M_textc,"Jimmy's Halftime Speech",M_link,HomeAwayMenu,M_jpad+JPADx,    M_eol,
	M_dy,12,M_textc,"Player Stats",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&PlayerStats,M_eol,
	M_dy,12,M_textc,"Substitutions",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&DoSubstitutions,M_eol,
	M_dy,12,M_textc,"Options",M_eol,
	M_dy,12,M_link,GamePDMenu,M_eol,
	M_end
};


menu_t PostGameMenu[]=
{
	&MenuScreenSetup,&MenuScreenCleanup,
	M_vmenu,&MenuSel,
	M_HILITE,
	M_color,PEN_WHITE,M_hicolor,PEN_HILITE,
	M_xy,CENTER_X,20,
	M_dy,12,M_textc,"Exit",M_jpad+JPADx,M_exit,1,M_eol,
	M_dy,12,M_textc, "Jimmy's MVP",		M_eol,
	M_dy,12,M_textc, "Jimmy's Postgame Analysis",M_link,HomeAwayMenu,M_jpad+JPADx,    M_eol,
	M_dy,12,M_textc, "Player Stats",M_link,HomeAwayMenu,M_jpad+JPADx,M_exec,&PlayerStats,M_eol,
	M_dy,12,M_textc, "Standings",M_jpad+JPADx,M_exec,&DisplayTmStandings,M_eol,
	M_dy,12,M_textc, "Team Comparison",		M_eol,
	M_dy,12,M_textc, "View Credits",M_jpad+JPADx,&DoCredits,M_eol,
	M_end
};


////////////////////////////////////////////////////////////////////////
//
// Calls From Menus
//

void PDScrollUp ()		// this scrolls up pulldown menu when active
{
	MenuScroll=1;
}

void PDScroller ()		// this scrolls down menu items when transitioning back from pulldown menus
{
	if (MenuScroll)
	{
		if (TextY < 12*6) TextY+=12;
	}
	else
	{
		if (TextY) TextY-=12;
	}
	MenuScroll=0;
}

void LoadEditedPlays ()
{
	if (GetCardBank ())
	{
		LoadingMessage ();
		LoadOverlay (PLEDIT_OVERLAY);
		LoadPlayFile (HomeAwaySel);
	}
	else /*    TimedMessage(5, CARD_MISSING_ERROR)   */;
}

void SeasonCreateTeam ()
{
	DisplayTrading (0);
}

void CreateTeam ()
{
	DisplayTrading (HomeAwaySel);
}


void EditControls ()
{
	LoadingMessage ();
//	DoControllers ();
}

void PlayerStats ()
{
	ViewPlayer (HomeAwaySel);
}


void ViewTeams ()
{
	extern BYTE HumanTeamNum;


	InitControl (-1);
	ViewTeam (HumanTeamNum == GameTeamData[0].TeamNumber ? 0:1);
}

void LeagueLeaders ()
{
	int ret;

	do
	{
		ret=DisplayLeaders ();
		switch (ret)
		{
			case -1: break;
			case  7: DisplayLgLeaders (); break;
			default: DisplayPlyrLeaders (); break;
		}
	}
	while (ret != -1);
}

void DoSubstitutions ()
{
	Display_Substitute (HomeAwaySel);
}

////////////////////////////////////////////////////////////////////////// stuff just for the psx!!!

int CheckSeasonFile (void)
{
	int i , j, error = 0;
	char s[40];
	extern WORD CardSlot;


//huh?	NumFiles = 0;

	if(!GetCardBank()) return(-1);

	sprintf(s, "%s%d", SEASONFILE, 0);
	if (!PromptForOverwrite(0, CardSlot, 256, 62, s))
		error = 1;

	return(error);
}


// New RAM card season: Prompt for overwrite & card insertion

void NewRAMCard ()
{
	int n,size;
	char str[35];
	extern WORD CardSlot;
	extern memfile_t MemFile[16];

	SeasonMode = -1;

	if (n = CheckSeasonFile())
	{
		if (n == -1) TimedMessageXY(256, 62, 4, "Insert Memory Card into Slot 1");
	}
	else
	{
		GetFiles (0, CardSlot);
		size=0;
		for (n = 0; n < 15; n++)
		{
			if (MemFile[n].blocks)
				size += MemFile[n].blocks;
			else
				break;
		}

		sprintf (str, "%s%d", SEASONFILE, 0);
		if (CheckFile (0, CardSlot, str))			// if season exists
		size -= 3;

		if (size > 12)
			TimedMessage (5, "Not enough room on Memory Card to save file.");
		else
			SeasonMode = 1;		// RAM card season
	}
}

// Load RAM card season

void LoadRAMCard ()
{
	int n;
	char str[35];
	extern WORD CardSlot;
	extern memfile_t MemFile[16];


	SeasonMode = -1;

	if (!GetCardBank())			// If no card display message
	{
		TimedMessageXY (256, 62, 5, "Insert Memory Card into Slot 1");
	}
	else
	{
		GetFiles (0, CardSlot);
		sprintf (str,"%s%d",SEASONFILE,0);
		for (n = 0; n < 15; n++)
		{
			if (!strcmp (str,MemFile[n].name))
			{
				SeasonMode = 1;		// RAM card season
				return;
			}
		}
		TimedMessageXY (256, 62, 5, "Season File not found");
	}
}

void NewPassword ()
{
	SeasonMode = 0;		// Set to password
}

void LoadPassword ()
{
	int done;
	
	extern  int Std_Menu_Frame(void);


	SeasonMode = -1;

	ResetStrings();
	FreeAllDialogBoxes();
	DoMenuStrip();
	InitPasswordBox();
	ClearPassword();

	done = 0;
	while (!done)
	{
//		AnyPads = GetAllPads (8);
		GetAllPads (8);
		done = DoName (15);
		DoFrameLoop (1, Std_Menu_Frame);
		PrintPassword ();
	}

	if (done > 0 && !ParsePassword (MemoryCardMimicPtr->password))
	{
		SeasonMode = 0;		// Password season
		ResetStrings();
	}

	FreeDialogBox(0);
	FreeDialogBox(1);
}

/// end of stuff just for the psx!!!!

////////////////////////////////////////////////////////////////////////
// Menu Screens
//

void MenuScreenSetup ()
{
	InitShellGraphics ();

//tom	if (!GameMode) PlaySong(3);			// load options song

	LoadShellBackground ();

	InitFonts ();
	Register_Font_Cluts ();
	LoadFont ("fontdpc2.tim",TIM_FONT0);

//	memcpy (GameOptionsVarTab,GameOptionsVars,17);				// set default values
//	memcpy (PlayerOptionsVarTab,PlayerOptionsVars,16);

	MenuSel=0;			// menu selections
	PDMenuSel=0;
	PDSel=0;

	FlushPads ();
}

void MenuScreenCleanup ()
{
	FreeShellGraphics ();
}

void MenuScreenCleanupH ()		// aligns helmets
{
	while (!AlignHelmets ()) DoFrameLoop (1,UpdateMenu);
	DoFrameLoop (2,UpdateMenu);
	FreeShellGraphics ();
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////
//
// Main Shell Loop
//

int DisplayMainGameShell ()
{
	int ret;


	ShellData=NULL;
	Home_MFile=NULL;
	Away_MFile=NULL;

	GameMode = PREGAME;

	MenuPad[0]=0;		// tom!! mirror game options controllers later
	MenuPad[1]=1;

	while (1)
	{
		DoTitleScreen ();		// does demo if no input

		LoadOverlay (MOVIE_OVERLAY);

		ret = Display_Main_Menu_Shell ();

		LoadShellFiles ();		// Each of following options will free Shell data before returning

		switch (ret)
		{
			case 0:
				Dprintf ("Exhibition Game\n");
				ExhibitionGame ();
				break;
			case 1:
				Dprintf ("Practice\n");
				Practice ();
				break;
			case 2:
				Dprintf ("Play Editor\n");
				PlayEditor ();
				break;
			case 3:
				Dprintf ("Continue Season\n");
				ContSeason ();
				break;
			case 4:
				Dprintf ("New Season\n");
				NewSeason ();
				break;
			case 5:
				Dprintf ("Team Manager\n");
				TeamManager ();
				break;
			case -1:	// back up
				break;
		}
		Dprintf ("return: %d\n",ret);
	}
}

LoadTitle ()
{
	FreeGraphics ();

	PlaySong (5);

	DrawSync (0);
	LoadBackgroundCD (0,"/shells/mmenu/title.tim");
	DrawSync (0);
	SetupFont ("fontdpc2.tim");
}

DoTitleScreen ()
{
	int flash;
	extern int Std_Menu_Frame(void);


	LoadTitle ();
	FlushPads ();
	CheckAttractMode (1);

	while (!GetAllPads (1))
	{
		ResetStrings();

		if(flash++ & 0x10)
		{
			PrintStringXYCenter("PRESS START",96,185,PEN_WHITE);
		}

		DoFrameLoop (1,Std_Menu_Frame);

		if (CheckAttractMode (0))
		{
			ResetStrings ();
			FreeBackgrounds ();
			FreeAllTemplates ();

			LoadOverlay (FBAI_OVERLAY);

			DebugPrintf (0,"Starting arcade..");
			DumpHeap ();

			DoAttractMode ();

			DebugPrintf (0,"Returned from arcade..");
			DumpHeap ();

			LoadTitle ();
		}
	}
}

int RunGame (void)
{
	int i;

	initVRamHandling ();
	ShowLoadingScreen ();	// CGD
	LoadOverlay (FBAI_OVERLAY);
	i = RunFootballGame ();
	LoadOverlay (MOVIE_OVERLAY);

	return (i);
}

ExhibitionGame ()
{
	int seq,ret;


	GameType = EXHIBITION_GAME;

	PlaySong (5);

	seq=0;
	while (seq >= 0)
	{
		FlushPads ();
		switch (seq)
		{
			case 0:
				if (ret==-1) FreeTeamGraphics ();
				ret = DoTeamSelectScreen ();
				if (ret > 0)
				{
					LoadPlayerDataForGame ();
					LoadTeamGraphics ();
				}
				break;
			case 1:
				ret = DoPadSelectScreen ();
				break;
			case 2:
				SetHomeStadiumOptions ();
				ret = DoMenu (PreGameMenu);
				break;
			case 3:
				seq=-99;
				break;
		}
		seq+=ret;
	}

	FreeShellFiles ();

	if (ret > 0)
	{
		ret = RunGame ();
		if (!ret)
		{
			PlayPostGameMovie ();
			EndGameShell (0);
		}
	}
}
					
NewSeason ()
{
	int seq,ret;

	extern LONG UseTradedTeam;
	extern BYTE HumanTeamNum;


	GameType = SEASON_GAME;

	LoadOverlay (MOVIE_OVERLAY);
	PlaySong (8);

	seq=0;
	while (seq >= 0)
	{
		FlushPads ();
		switch (seq)
		{
			case 0:
				ret = DoMenu (NewSeasonMenu);
				if ((ret > 0) && (SeasonMode < 0)) ret=0;
				break;
			case 1:
				if (ret==-1) FreeTeamGraphics ();
				ret = DoTeamSelectScreen ();
				if (ret > 0)
				{
					LoadTeamGraphics ();
					LoadPlayerDataForGame ();
				}
				break;
			case 2:
			{
				UseTradedTeam = FALSE;

				ret = DoMenu (SetupSeasonMenu);

				if (UseTradedTeam)
				{
					LoadCreatedTeam (HumanTeamNum==GameTeamData[1].TeamNumber, 0);
					MemoryCardMimicPtr->UseTradedTeam = TRUE;
				}
				break;
			}
			case 3:
				ret = PlayTeamBioMovie (HumanTeamNum);
				break;
			case 4:
				ret = DoPadSelectScreen ();
				break;
			case 5:
				SetHomeStadiumOptions ();
				ret = DoMenu (SeasonPreGameMenu);
				break;
			case 6:
				if (HumanTeamNum == GameTeamData[0].TeamNumber)
					ret = PlayTeamBioMovie(GameTeamData[1].TeamNumber);
				else
					ret = PlayTeamBioMovie(GameTeamData[0].TeamNumber);
				break;
			case 7:
				seq=-99;
				break;
		}
		seq+=ret;
	}

	FreeShellFiles ();

	if (ret > 0)
	{
		ret = RunGame ();

		if (!ret)
		{
			UpdateSeason ();
			DisplayTmStandings ();
			PlayPostGameMovie ();
			EndGameShell (1);
//		if (!playoff_loss) SeasonFlag = TRUE;
		}
	}
}


ContSeason ()		// not quite operational yet!!
{
	int seq,ret;

 
	LoadOverlay (MOVIE_OVERLAY);
	PlaySong(3);

	GameType = SEASON_GAME;

#if 0
	if(SeasonFlag)
	{
		GoToFxn2 = 1;

		memcpy (ShadowPassword,MemoryCardMimicPtr->password,15);
TestHeap();
		ByGame = ContinueSeason();
TestHeap();
		memcpy (MemoryCardMimicPtr->password,ShadowPassword,15);
	}

	/* Don't permit season to continue if an error occurred */

	if(ByGame > 0)
	{
		DebugPrintf(0,"Season Memcard Error");
		break;	// Test for error condition and exit season if found
	}
#endif

	seq=0;
	while (seq >= 0)
	{
		FlushPads ();
		switch (seq)
		{
			case 0:
				ret = DoMenu (ContinueSeasonMenu);
				if ((ret > 0) && (SeasonMode < 0)) ret=0;
				break;
			case 1:
				if (ret==-1) FreeTeamGraphics ();
				LoadTeamGraphics ();
				LoadPlayerDataForGame ();
				ret = DoMenu (SeasonMenu);
				break;
			case 2:
				ret = DoPadSelectScreen ();
				break;
			case 3:
				ret = DoMenu (SeasonPreGameMenu);
				break;
			case 4:
				seq=-99;
				break;
		}
		seq+=ret;
	}

	FreeShellFiles ();

	if (ret > 0)
	{
		ret = RunGame ();
		if (!ret)
		{
			UpdateSeason();
			DisplayTmStandings();
			PlayPostGameMovie();
			EndGameShell(1);
//		if (!playoff_loss) SeasonFlag = TRUE;
		}
	}
}


Practice ()
{
	int ret,seq;


	PracticeMode = TRUE;
	GameType = PRACTICE_GAME;

	PlaySong(1);

	seq=0;
	while (seq >= 0)
	{
		FlushPads ();
		switch (seq)
		{
			case 0:
				ret = DoTeamSelectScreen ();
				LoadPlayerDataForGame ();
				break;
			case 1:
				SetHomeStadiumOptions ();
				ret = DoMenu (PreGameMenu);
				break;
			case 2:
				seq=-99;
				break;
		}
		seq+=ret;
	}
	FreeShellFiles ();

	if (ret > 0)
	{
		RunGame ();
	}
}


PlayEditor ()
{
	PlaySong (4);

	LoadOverlay (PLEDIT_OVERLAY);

	Display_Play_Editor_Shell (0);

	FreeShellFiles ();
}


TeamManager ()
{
}

//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Routines Shared by Shell Files
//

////////////////////////////////////////////////////////////////////////
// Shell Graphics Routines
//

// allocate vram for names + spinning helmets incase they are used

InitShellGraphics ()
{
	char str[35];
	int n;


	InitObjects ();

	ShellObject=LoadObjects (ShellData,GetMultiFilePtr (ShellData,"shell.mdf") );

	if (Home_MFile)
	{
		sprintf (str,"name%02d.mdf",GameTeamData[HOME_TEAM_SLOT].TeamNumber);		// load nicknames into vram
		LoadObject (ShellObject[HOMENAME],GetMultiFilePtr (Home_MFile,str));
	}

	if (Away_MFile)
	{
		sprintf (str,"name%02d.mdf",GameTeamData[AWAY_TEAM_SLOT].TeamNumber);
		LoadObject (ShellObject[AWAYNAME],GetMultiFilePtr (Away_MFile,str));
	}

	for (n=0; n<SHELLOBJS; n++) ShellObject[n]->Flags &= ~OBJ_DISPLAY;		// turn off until used

	HomeHelmetR=(1<<2)+1;		// reset helmet positions so they are drawn on next spin
	AwayHelmetR=(7<<2)-1;
}

FreeShellGraphics ()
{
	FreeObjects ();
	ShellObject=NULL;
}

LoadShellFiles ()
{
	DumpHeap ();
	Dprintf ("Loading League Player Data...\n");
	DumpHeap ();
	LoadLeaguePlayerData ();
	Dprintf ("Loading Shell Files...\n");
	ShellData = (MultiFile_t*)LoadCDFile ("/shells/shell.mfl");
	DumpHeap ();
}

FreeShellFiles ()
{
	FreeTeamGraphics ();
	if (ShellData) FreeFileData (ShellData);
	ShellData=NULL;
	FreeLeaguePlayerData ();
}

LoadShellBackground ()
{
	initTexture (0,(ULONG*)GetMultiFilePtr (ShellData,"backgrnd.tim") );
}

LoadTeamGraphics ()
{
	char str[35];

	if (Home_MFile==NULL)
	{
		Dprintf ("Loading Home Team Graphics...\n");
		sprintf (str,"/shells/helmets0/helmet%02d.mfl",GameTeamData[HOME_TEAM_SLOT].TeamNumber);
		Home_MFile = (MultiFile_t*)LoadCDFile (str);
	}
	if (Away_MFile==NULL)
	{
		Dprintf ("Loading Away Team Graphics...\n");
		sprintf (str,"/shells/helmets0/helmet%02d.mfl",GameTeamData[AWAY_TEAM_SLOT].TeamNumber);
		Away_MFile = (MultiFile_t*)LoadCDFile (str);
	}
}

FreeTeamGraphics ()
{
	if (Home_MFile) FreeFileData (Home_MFile);
	if (Away_MFile) FreeFileData (Away_MFile);
	Home_MFile=NULL;
	Away_MFile=NULL;
}

// this displays objects at cursor location

void DrawTriangle ()
{
	SetObjectPos (ShellObject[MENUTRIANGLE],CursorX,CursorY);
	ShellObject[MENUTRIANGLE]->Flags |= OBJ_DISPLAY;
}
void DrawSquare ()
{
	SetObjectPos (ShellObject[MENUSQUARE],CursorX,CursorY);
	ShellObject[MENUSQUARE]->Flags |= OBJ_DISPLAY;
}
void DrawCross ()
{
	SetObjectPos (ShellObject[MENUCROSS],CursorX,CursorY);
	ShellObject[MENUCROSS]->Flags |= OBJ_DISPLAY;
}
void DrawCircle ()
{
	SetObjectPos (ShellObject[MENUCIRCLE],CursorX,CursorY);
	ShellObject[MENUCIRCLE]->Flags |= OBJ_DISPLAY;
}

void DrawHomeName ()
{
	ShellObject[HOMENAME]->Flags |= OBJ_DISPLAY;
}

void DrawHomeNameAt ()
{
	SetObjectCenterPos (ShellObject[HOMENAME],CursorX,CursorY);
	ShellObject[HOMENAME]->Flags |= OBJ_DISPLAY;
}

void DrawAwayName ()
{
	ShellObject[AWAYNAME]->Flags |= OBJ_DISPLAY;
}

void DrawAwayNameAt ()
{
	SetObjectCenterPos (ShellObject[AWAYNAME],CursorX,CursorY);
	ShellObject[AWAYNAME]->Flags |= OBJ_DISPLAY;
}

// this displays helmets and spins them when called

void SpinHomeHelmet ()
{
	char str[35];

	HomeHelmetR--;
	HomeHelmetR &= (8<<2)-1;
	if (ShellObject && !(HomeHelmetR & 3))
	{
		sprintf (str,"H%02d%d.mdf",GameTeamData[HOME_TEAM_SLOT].TeamNumber,HomeHelmetR >> 2);
		LoadObject (ShellObject[HOMEHELM],GetMultiFilePtr (Home_MFile,str));

		ShellObject[HOMEHELM]->Flags |= OBJ_DISPLAY;
	}
}

void SpinHomeHelmetAt ()		// uses Cursor X,Y
{
	SetObjectCenterPos (ShellObject[HOMEHELM],CursorX,CursorY);
	SpinHomeHelmet ();
}

void SpinAwayHelmet ()
{
	char str[35];

	AwayHelmetR++;
	AwayHelmetR &= (8<<2)-1;
	if (ShellObject && !(AwayHelmetR & 3))
	{
		sprintf (str,"H%02d%d.mdf",GameTeamData[AWAY_TEAM_SLOT].TeamNumber,AwayHelmetR >> 2);
		LoadObject (ShellObject[AWAYHELM],GetMultiFilePtr (Away_MFile,str));

		ShellObject[AWAYHELM]->Flags |= OBJ_DISPLAY;
	}
}

void SpinAwayHelmetAt ()		// uses Cursor X,Y
{
	SetObjectCenterPos (ShellObject[AWAYHELM],CursorX,CursorY);
	SpinAwayHelmet ();
}

int AlignHelmets ()				// waits for helmets to stop then returns 1
{
	int happy;

	happy=((HomeHelmetR == 1<<2) && (AwayHelmetR == 7<<2));
	if (HomeHelmetR != 1<<2) SpinHomeHelmet ();
	if (AwayHelmetR != 7<<2) SpinAwayHelmet ();

	return happy;
}


//////////////////////////////////////////////////////////////////////////////////////////////////////////////
//
// Crazy Menu System
//

/////////////////////////////////////// psx specific menu calls

int GetCaptainPadNum (int team)
{
	return GameTeamData[team].ControllerNum[0];
}

void PrintMenuText (char *str,struct MenuCtrl *mc)
{
	int handle;


	ClipString.x=0;
	ClipString.w=512;
	ClipString.y=mc->TextTop;
	ClipString.h=mc->TextBottom - mc->TextTop;

	handle = CreateStringClip (str,mc->Font,CursorX,CursorY - (mc->ScrollOn?TextY:0),FONT0 | mc->Spacing);
	if (handle != -1) ChangeStringCLUT (strings.string[handle],FontClutIDs[mc->Color]->CLUT_ID);
	CursorX += GetStrLen (str,strings.spacing[handle]);
}

void PrintMenuTextCentered (char *str,struct MenuCtrl *mc)
{
	int handle;


	ClipString.x=0;
	ClipString.w=512;
	ClipString.y=mc->TextTop;
	ClipString.h=mc->TextBottom - mc->TextTop;

	handle = CreateStringClip (str,mc->Font,CursorX,CursorY - (mc->ScrollOn?TextY:0),FONT0 | mc->Spacing);
	if (handle != -1)
	{
		CenterString (handle,str,CursorX);
		ChangeString (handle,str,FONT0 | mc->Spacing);
		ChangeStringCLUT (strings.string[handle],FontClutIDs[mc->Color]->CLUT_ID);
	}
}

/////////////////////////////////////// end of psx specific menu calls

///////////////////////
// Display Menus
///////////////////////

// Callback for display loop

int UpdateMenu (void)
{
	PollPC();

///	if (JpadTrig[0] & JPADRup)
	{
		UpdateBackground();

		DisplayAllStrings();
		DisplayObjects ();
		TPageViewer ();				// debug
	}
	return (1);
}

// The main event

DoMenu (menu_t menu)
{
	struct MenuCtrl *root;
	void (*setup)();
	void (*cleanup)();
	int ret;


	setup  =(void(*)()) ((LONG*)menu)[0];			// copy ptrs from menu
	cleanup=(void(*)()) ((LONG*)menu)[1];
	(LONG*)menu+=2;

	FreeGraphics ();

	(*setup)();				// draw background, load fonts, etc.

	FlushPads ();

	MenuCtrl.Flags=MF_ACTIVE | MF_DISPLAY;			// set root defaults
	MenuCtrl.Pad=-1;														// default is all on
	MenuCtrl.Level=0;
	MenuCtrl.Mode=M_HILITE;
	MenuCtrl.MenuPtr=menu;
	MenuCtrl.VarPtr=NULL;
	MenuCtrl.Font=fontdpc2;
	MenuCtrl.Spacing=PROPORTIONAL;
	MenuCtrl.Color=PEN_WHITE;
	MenuCtrl.RegColor=PEN_WHITE;
	MenuCtrl.HiColor=PEN_HILITE;
	MenuCtrl.LoColor=PEN_LOLITE;
	MenuCtrl.TextTop=0;
	MenuCtrl.TextBottom=240;
	MenuCtrl.ScrollOn=0;

	MenuLevel=0;
 	TextX=TextY=0;
 	MenuScroll=0;
 	CursorX=CursorY=0;

	do
	{
		JpadRepeatFreq=-1;

		ResetStrings ();
		root=MenuCtrl.MenuPtr;			// save menu ptr 'cause its gonna get updated
		MenuExec=NULL;
		MenuLevelMsg=MenuLevel;
		ret=DisplayMenu (&MenuCtrl);
		MenuLevel=MenuLevelMsg;
		MenuCtrl.MenuPtr=root;

		if (MenuExec)
		{
			(*cleanup)();
			(*MenuExec)();
			FreeGraphics ();
			(*setup)();
		}

		DoFrameLoop (1,UpdateMenu);
	} while (!ret);

	(*cleanup)();
	ResetStrings ();

	return ret;
}

// The recursive thing

int DisplayMenu (struct MenuCtrl *mc)
{
	struct MenuCtrl mcx;	// for passing to children
	int tmp,ret;
	int menutype;
	int menuline;
	int xsav,ysav;
	LONG *p;
	ULONG toke;
	int jpad;


	memcpy (&mcx,mc,sizeof(mcx));		// copy to working info

	p=mcx.MenuPtr;

	xsav=CursorX;
	ysav=CursorY;

	menuline=0;
	ret=0;

	menutype=(int)*p++;		// menu header

	switch (menutype)
	{
		case M_pdmenu:
			mcx.Level++;			// now at a new tree level
			if (*p) mcx.VarPtr=(var_t*)*p;	// set menu's variable if there
			p++;
			break;
		case M_hmenu:
		case M_vmenu:
			if (*p) mcx.VarPtr=(var_t*)*p;	// set menu's variable if there
			p++;
			break;
		default:
			Dprintf ("bad menu header $%04x\n",menutype);
			while (1);
	}

	do
	{
		mcx.Flags = mc->Flags;

		if (mcx.Pad >= 0) jpad=JpadTrig[mcx.Pad];		// set current pad to read
		else jpad=AnyPadTrig;

		if (mcx.VarPtr)
		{
			if ((*mcx.VarPtr != menuline) || (mcx.Level > MenuLevel)) mcx.Flags &= ~MF_ACTIVE;		// set active flag

			mcx.Color=mcx.RegColor;
			if (*mcx.VarPtr==menuline) mcx.Color=mcx.LoColor;		// lolite selection
			if ((mcx.Flags & MF_ACTIVE) && (mcx.Level==MenuLevel)) mcx.Color=mcx.HiColor;		// hilite selection if in active menu

			switch (mcx.Mode)
			{
				case M_MUTEX:
					if (*mcx.VarPtr != menuline) mcx.Flags &= ~MF_DISPLAY;	// blank mutually exclusive selections
					break;
				case M_ACTIVE:
					if (!(mc->Flags & MF_ACTIVE)) mcx.Flags &= ~MF_DISPLAY;	// blank selections if parent not active
					break;
			}
		}

		toke=(ULONG)*p++;

		switch (toke)
		{
// menu modes
			case M_MUTEX:
			case M_HILITE:
			case M_ACTIVE:
				mcx.Mode=(int)*(p-1);
				break;
// sub menus
			case M_link:				// link to a sub menu by address
				mcx.MenuPtr=(menu_t*)*p++;
				tmp=DisplayMenu (&mcx);
				if (!ret) ret=tmp;
				break;
			case M_linkvar:			// link to a sub menu by address & specify var
			{
				var_t *temp;

				temp=mcx.VarPtr;
				mcx.MenuPtr=(menu_t*)*p++;
				mcx.VarPtr=(var_t*)*p++;
				tmp=DisplayMenu (&mcx);
				if (!ret) tmp=ret;
				mcx.VarPtr=temp;
				break;
			}
			case M_hmenu:				// sub menu
			case M_vmenu:
			case M_pdmenu:
				mcx.MenuPtr=p-1;
				tmp=DisplayMenu (&mcx);
				if (!ret) ret=tmp;
				p=mcx.MenuPtr;
				break;
// printing stuff
			case M_text:			// draw text
				if (mcx.Flags & MF_DISPLAY) PrintMenuText ((char*)*p,&mcx);
				p++;
				break;
			case M_textc:			// centered text
				if (mcx.Flags & MF_DISPLAY) PrintMenuTextCentered ((char*)*p,&mcx);
				p++;
				break;
// color stuff
			case M_color:	mcx.RegColor=mcx.HiColor=mcx.LoColor=(int)*p++;	break;
			case M_hicolor:	mcx.HiColor=(int)*p++;	break;
			case M_locolor:	mcx.LoColor=(int)*p++;	break;
// cursor stuff
			case M_xy:	CursorX=(int)*p++;	CursorY=(int)*p++;	break;
			case M_x:		CursorX=(int)*p++;	break;
			case M_y:		CursorY=(int)*p++;	break;
			case M_dx:	CursorX+=(int)*p++;	break;
			case M_dy:	CursorY+=(int)*p++;	break;
			case M_dxy:	CursorX+=(int)*p++;	CursorY+=(int)*p++;	break;
			case M_top:	mcx.TextTop=(int)*p++;	break;
			case M_bot:	mcx.TextBottom=(int)*p++;	break;
			case M_scrolloff:	mcx.ScrollOn=0;	break;
			case M_scrollon:	mcx.ScrollOn=1;	break;
// control
			case M_setpad:		// set joypad to read
				if (*p==-1) mcx.Pad=-1;
				else mcx.Pad=GetCaptainPadNum (*p);		// home/away captain pad
				p++;
				break;
			case M_ifactive:	// if not active skip to end of line
				if (!(mcx.Flags & MF_ACTIVE)) while (*p != M_eol) p+=(*p & 0xFF)+1;
				break;
			case M_call:			// call function
				((void(*)())*p++)();
				break;
			case M_exec:			// execute address and redraw
				MenuExec=(void(*)())*p++;
				break;
			case M_eol:				// end of line
				menuline++;
				break;
			case M_exit:			// exit and return code
				ret=*p++;
				break;
			case M_end:				// end of menu
				break;
// joypad triggers
			default:
			{
				if (toke & M_jpad)		// check triggers (skip to end of line if no trigger)
				{
					if ((mcx.Flags & MF_ACTIVE) && (mcx.Level >= MenuLevel))
					{
						if ((toke & 0xFFFF) & jpad) break;
					}
					while ((*p != M_eol) && (*p != M_end)) p+=(*p & 0xFF)+1;		// skip to end of line
				}
				else
				{
					Dprintf ("unknown menu token! $%x\n",toke);
					while (1);
				}
			}
		}
	}	while (toke != M_end);

	mc->MenuPtr=p;		// adjust parent ptr

	if (!ret && (mcx.VarPtr != NULL) && (mc->Flags & MF_ACTIVE) && (mcx.Level >= MenuLevel))		// if parent active do input
	{
		if (*mcx.VarPtr >= menuline) *mcx.VarPtr=menuline-1;		// limit check
		if (*mcx.VarPtr < 0) *mcx.VarPtr=0;

		switch (menutype)		// process input for current menu
		{
			case M_hmenu:
				if ((jpad & JPADLleft ) && (*mcx.VarPtr > 0))					 (*mcx.VarPtr)--;
				if ((jpad & JPADLright) && (*mcx.VarPtr < menuline-1)) (*mcx.VarPtr)++;
				break;
			case M_vmenu:
				if ((jpad & JPADLup   ) && (*mcx.VarPtr > 0))					 (*mcx.VarPtr)--;
				if ((jpad & JPADLdown ) && (*mcx.VarPtr < menuline-1)) (*mcx.VarPtr)++;
				break;
			case M_pdmenu:
				if (mcx.Level==MenuLevel)
				{
					if (jpad & JPADLup  )
					{
						if (*mcx.VarPtr > 0) (*mcx.VarPtr)--;
						else MenuLevelMsg--;								// back out of menu
					}
					if ((jpad & JPADLdown) && (*mcx.VarPtr < menuline-1)) (*mcx.VarPtr)++;
				}
				else if (mc->Level==MenuLevel)		// jump into pulldown menu if parent's level is on
				{
					if (jpad & JPADLdown ) MenuLevelMsg=mcx.Level;		// force input to current level
				}
				break;
		}
	}

	CursorX=xsav;
	CursorY=ysav;

	return ret;
}

int Display_Game_Options_Shell (int pad)
{
}

int Display_Player_Options_Shell (int pad)
{
}

// called from gameloop

extern  int Std_Menu_Frame(void);

void ShowScreen(BYTE *screen, INT button_out)
{
	UWORD i;


	  /*Set graphics mode*/

	initVRamHandling();

  /*initialize the background for the shell*/

	LoadBackgroundCD(0,screen);

	fade_in(10,1);

	for(i=0; i<200; i++) DoFrameLoop(1, Std_Menu_Frame);

	FadeDown(10);

	FreeBackgrounds();
}

// end of shell.c //

