//
//	shellsys.c for the PSX.
//
//	Copyright 1997 GAMETEK
//	All rights reserved.
//
// Written by Tom McWIlliams
// Started: 3-01-97
//
//	Notes:
//  recursive multidata files not supported
//	multidata blocks must start on 4 byte boundries
// 	SetObjectPos () don't work
//

#define SHOWLOADS 0

#include <stdio.h>
#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include <strings.h>
#include <memory.h>

#include "prjhdrs.h"

#include "multifil.h"

#include "shellsys.h"

// for handling multidata files
#define ChunkType(p) (*(ULONG*)p)
#define NextChunk(p) ((void*)((ULONG)p+((ULONG*)p)[1]+(((((ULONG*)p)[1]^3)+1)&3)))

#define wordat(p) (((UBYTE*)p)[0]|(((UBYTE*)p)[1]<<8))
#define longat(p) (((UBYTE*)p)[0]|(((UBYTE*)p)[1]<<8)|((ULONG)((UBYTE*)p)[2]<<16)|((ULONG)((UBYTE*)p)[3]<<24))

// global variables / arrays

ULONG *MiscBuffer;			// buffer for decompression, etc...

// sobj's are allocated in order from an array.  Ya Can't delete them once they are there.

int SobjAlloc;									// current ptr in list to allocate
ShellObj *SobjPtrs[NUM_SOBJS];	// ptr lists returned by LoadObjects (pre-stuffed!)
ShellObj Sobj[NUM_SOBJS];				// shell object array
GfxObj 	 Gobj[NUM_GOBJS];				// graphics object structures

// currently this allocates blocks within pre-allocated tpages which must be contiguous and not wrap!

int gVramX;		// current WORD available in tpage area
int gVramY;		// current line available in each tpage area
int gVramH[NUM_TPAGES];		// lowest line used within each tpage
vram_t Tex[NUM_GOBJS];		// vram structure for textured objects


// internal prototypes

void LoadObjPixels		(ShellObj *sobj,void *data,int cmp);
void LoadObjClut 			(ShellObj *sobj,void *data);
vram_t *AllocTexture (int mode,int w,int h);
void FreeTexture (vram_t *tex);
GfxObj *AddGfxObj (ShellObj *sobj);
GfxObj *AllocGfxObj ();


/////////////////////////////////////////////////////////////////////////
//
// Initialize shell objects
//
/////////////////////////////////////////////////////////////////////////

void InitObjects ()
{
	int n;
	extern vram_t texture[];		// <-- signal to vram handler that I've claimed my own tpages


	MiscBuffer = (ULONG*)AllocMem (0x4000);
	if (MiscBuffer==NULL) Dprintf ("Could not allocate buffer for shellsys\n");

	SobjAlloc=0;		// reset pointer list index for LoadObjects()

	for (n=0; n<NUM_SOBJS; n++) Sobj[n].Type=0;
	for (n=0; n<NUM_GOBJS; n++) Gobj[n].Attr=0;

// stuff for manual allocation of textures: this claims fixed tpages

	AllocBottomTpages (8);
#if 0
	for (n=0; n<NUM_TPAGES; n++)
	{
		if (texture[n+TPAGES].used)
		{
			Dprintf ("Shellsys stomping on tpage #%d\n",n+TPAGES);
			while (1);
		}
		texture[n+TPAGES].used=1;		// claim tpages and tell vram handler I got 'em
	}
#endif

	for (n=0; n<NUM_GOBJS; n++) Tex[n].used=0;

	gVramX = 0;
	gVramY = 0;
	for (n=0; n<NUM_TPAGES; n++) gVramH[n]=0;
}


/////////////////////////////////////////////////////////////////////////
//
// Allocate New Object and create any associated structures
//
/////////////////////////////////////////////////////////////////////////

ShellObj *NewObject (UWORD type,void *file)
{
	ShellObj *sobj;
	int d,n;

	sobj=NULL;

	for (n=0; n<NUM_SOBJS; n++)
	{
		if (!Sobj[n].Type)		// check if available
		{
			switch (type)				// allocate data structure based on type
			{
				case OBJ_POLYTEX:
				case OBJ_MENU:
					Sobj[n].X=0;
					Sobj[n].Y=0;
					Sobj[n].W=0;
					Sobj[n].H=0;
					Sobj[n].SW=0;
					Sobj[n].SH=0;
					Sobj[n].Depth=0;
					Sobj[n].Type=type;
					Sobj[n].Data=NULL;
					sobj=&Sobj[n];
					break;
				default:
					Dprintf ("ERROR: Unknown ShellObj Type\n");
					break;
			}
			break;
		}
	}

	if (sobj)
	{
		sobj->Flags=OBJ_DISPLAY;		// default flags
//(later)	if (file) LoadObject (sobj,file);		// fill structure if multidata file ptr supplied
	}
	else Dprintf ("ERROR: Could not allocate ShellObj\n");

	return sobj;
}


/////////////////////////////////////////////////////////////////////////
//
// Load Multiple Objects and data from multidata file
// returns ptr to ptr list to objects
//
/////////////////////////////////////////////////////////////////////////

ShellObj **LoadObjects (MultiFile_t *mfl,void *file)
{
	ShellObj **sobjptr;
	ShellObj *sobj;
	UBYTE *data,*eof;


	sobjptr=NULL;		// ptr to top of ptr list

	data = (UBYTE*)file + 8;		// 1st chunk
	eof  = NextChunk (file);		// get end of file

	while (data < eof)
	{
#if SHOWLOADS
		Dprintf ("data=$%x eof=$%x  block='%c%c%c%c' size=$%x\n",data,eof,data[0],data[1],data[2],data[3],longat(data+4) );
#endif
		switch (ChunkType(data))
		{
			case _GOBJ_:
			{
				int x,y,d;

				x=wordat(data+ 8);	// x,y,depth
				y=wordat(data+10);
				d=wordat(data+12);
#if SHOWLOADS
				Dprintf ("loading %s at xy=%d,%d depth=%d\n",data+14,x,y,d);
#endif
				sobj=NewObject (OBJ_POLYTEX,NULL);
				sobj->X=x;
				sobj->Y=y;
				sobj->Depth=d;

				if (sobj)
				{
					if (sobjptr==NULL) sobjptr=&SobjPtrs[SobjAlloc];		// save ptr to top of list
					SobjPtrs[SobjAlloc++]=sobj;													// add sobj to ptr list
					LoadObject (sobj,GetMultiFilePtr (mfl,data+14));		// name of graphics file
				}
				break;
			}
			default:
				Dprintf ("Unkown mdf chunk:'%c%c%c%c'\n",data[0],data[1],data[2],data[3]);
				while (1);
				break;
		}
		data = NextChunk (data);		// next chunk
	}
#if SHOWLOADS
	Dprintf ("End of mdf\n");
#endif

	return sobjptr;
}


/////////////////////////////////////////////////////////////////////////
//
// Load Object data from multidata file
//
/////////////////////////////////////////////////////////////////////////

void LoadObject (ShellObj *sobj,void *file)
{
	UBYTE *data,*eof;

	data = (UBYTE*)file + 8;		// 1st chunk
	eof  = NextChunk (file);		// get end of file

	while (data < eof)
	{
#if SHOWLOADS
		Dprintf ("data=$%x eof=$%x  block='%c%c%c%c' size=$%x\n",data,eof,data[0],data[1],data[2],data[3],longat(data+4) );
#endif

		switch (longat(data))		// block type
		{
			case _PIXL_:		// this will allocate any GfxObj's
				LoadObjPixels (sobj,data+8,0);
				break;
			case _PIXC_:		// this will allocate any GfxObj's
				LoadObjPixels (sobj,data+8,1);
				break;
			case _CLUT_:		// this will copy clut to all existing GfxObj's in ShellObj
				LoadObjClut (sobj,data+8);
				break;
			default:
				Dprintf ("Unkown mdf chunk:'%c%c%c%c'\n",data[0],data[1],data[2],data[3]);
				while (1);
				break;
		}
		data = NextChunk (data);
	}
#if SHOWLOADS
	Dprintf ("End of mdf\n");
#endif
}


/////////////////////////////////////////////////////////////////////////
//
// Load Object Pixel data from a multidata file
//
/////////////////////////////////////////////////////////////////////////

void LoadObjPixels (ShellObj *sobj,void *data,int cmp)		// cmp 1=compressed
{
	register int mode,u,v,w,h;
	RECT rect;
	GfxObj *gobj;
	POLY_FT4 *poly;
	vram_t *tex;
	ULONG *buff;


	mode = wordat(data+0);		// mode

	if (sobj->Data==NULL)			// create GfxObj's for shell object if none there
	{
		sobj->W = sobj->SW = wordat(data+4);		// width in pixels
		sobj->H = sobj->SH = wordat(data+6);		// height in lines

		tex = AllocTexture (mode,sobj->W,sobj->H); 	// allocate texture in vram

		rect.x = tex->x;
		rect.y = tex->y;
		rect.w = tex->w;
		rect.h = tex->h;

		while (rect.w)
		{
			w=rect.w;
			h=rect.h;

			if ((rect.x % 64) + w > 64) w = 64 - (rect.x % 64);		// split image if too wide for tpage

			gobj=AddGfxObj (sobj);

			gobj->Texture = tex;
			gobj->Attr &= ~GOBJ_MODE;		// set bitmap mode
			gobj->Attr |= mode;
			gobj->X = (rect.x - tex->x) << (2-mode);
			gobj->Y = rect.y - tex->y;
			gobj->W = w << (2-mode);
			gobj->H = h;

			poly=&gobj->Poly;

			setPolyFT4 (poly);
			poly->tpage = GetTPage (mode,0,rect.x,rect.y);

			u =(rect.x %  64) << (2-mode);		// get loc in tpage
			v = rect.y % 256;

			rect.x += w;				// next piece
			rect.w -= w;

			w=w << (2-mode);		// get byte width
	
			setXYWH (poly,sobj->X + gobj->X - CENTER_X - sobj->W/2,sobj->Y + gobj->Y - CENTER_Y - sobj->H/2,w,h);		// default centered
			setUVWH (poly,u,v,w-(u+w == 256>>mode),h-(v+h == 256));

			setRGB0 (poly,0x80,0x80,0x80);		// SET TRANSPARENCY MODE
			setSemiTrans (poly,0);
//		setShadeTex (poly,1);
		}
	}

	if (cmp)			// access pixel data
	{
//	Dprintf ("Decompressing Image at $%x\n",data+12);
		DecompressFrameBytes (MiscBuffer,data+12);
		buff=MiscBuffer;
	}
	else buff=(ULONG*)((UBYTE*)data+12);

	gobj = (GfxObj*)sobj->Data;
	tex = gobj->Texture;

	rect.x = tex->x;
	rect.y = tex->y;
	rect.w = tex->w;
	rect.h = tex->h;

//	Dprintf ("Loading Image xywh=%d,%d,%d,%d\n",rect.x,rect.y,rect.w,rect.h);
	LoadImage (&rect,buff);
	DrawSync (0);
}


/////////////////////////////////////////////////////////////////////////
//
// Load Object Clut from a multidata file
//
/////////////////////////////////////////////////////////////////////////

void LoadObjClut (ShellObj *sobj,void *data)
{
	register int mode;
	GfxObj *gobj;
	RECT rect;
	ULONG clut[256];


	mode = wordat(data+0);		// get mode

	gobj=(GfxObj*)sobj->Data;
	if (gobj)
	{
		if (gobj->Clut==NULL) gobj->Clut = alloc_VRam (mode+1, 0);		// allocate clut space if non-existent

		if (mode==0)		// if it is 16 colors (4 bit)...
		{
			rect.x = gobj->Clut->x + 16;
			rect.y = gobj->Clut->y;
			rect.w = 240;
			rect.h = 1;
			StoreImage (&rect,clut);		// Load it, cos LOAD CLUT ONLY LOADS 256 even in 4 bit mode
		}

		gobj->Clut->CLUT_ID = LoadClut ((ULONG*)((UBYTE*)data+8),gobj->Clut->x,gobj->Clut->y);
		gobj->Poly.clut = gobj->Clut->CLUT_ID;

		if (mode==0) LoadImage (&rect,clut);		// if it is 16 colors (4 bit) Store it back
		DrawSync (0);

		while (gobj->Next)		// copy clut to all polys
		{
			gobj->Next->Clut=gobj->Clut;
			gobj->Next->Poly.clut=gobj->Clut->CLUT_ID;
			gobj=gobj->Next;
		}
	}
}


/////////////////////////////////////////////////////////////////////////
//
// Allocate VRAM for texture
// This routine should eventually be done automatically by an alloc_VRam () call
/////////////////////////////////////////////////////////////////////////

vram_t *AllocTexture (mode,w,h)
int mode;		// sony mode
int w;			// width in pixels
int h;			// height in lines
{
	vram_t *tex;
	int n;

#if SHOWLOADS
	Dprintf ("Attempting to alloc tex with mode=%d w,h=%d,%d\n",mode,w,h);
#endif
	tex=NULL;
	for (n=0; n<NUM_GOBJS; n++)
	{
		if (!Tex[n].used)
		{
			tex=&Tex[n];
			break;
		}
	}

	w >>= 2-mode;		// get width in bytes

	if (tex)
	{
		while (1)		// search for a spot in tpages
		{
			if (w > 64)		// if graphics is wider than tpage allocate it and it will be split later
			{
				if (((gVramX + w)/64 < NUM_TPAGES) && (gVramY + h < 256))	// check if room in following tpages for width & height
				{
					for (n=gVramX/64; n<= (gVramX + w)/64; n++)		// check if y room across tpages
					{
						if (gVramY < gVramH[n]) goto NextTP;
					}
					break;
				}
			}
			else
			{
				if (((gVramX%64) + w < 64) && (gVramY + h < 256)) break;					// check if room in tpage for width & height
			}
NextTP:
			gVramX = gVramX - (gVramX % 64) + 64;		// next tpage
			if (gVramX/64 >= NUM_TPAGES) gVramX=0;		// wrap around (forever?!)
			gVramY = gVramH[gVramX/64];
		}

		tex->used=1;
//	tex->num=1;			// N/A
//	tex->CLUT_ID=;	// N/A
//	tex->tpage=;		// N/A

		tex->x = TPAGEX + gVramX; 		// set rect for image in VRAM
		tex->y = TPAGEY + gVramY;
		tex->w = w;
		tex->h = h;

		for (n=gVramX/64; n<= (gVramX + w)/64; n++)		// set available y in tpages this tex uses
		{
			if (gVramH[n] < (gVramY + h)) gVramH[n]=gVramY + h;
		}

		gVramX += w;					// add width within TPage
#if SHOWLOADS
		Dprintf ("alloced vram x,y,w,h=%d,%d,%d,%d\n",tex->x,tex->y,tex->w,tex->h);
#endif
	}
	else Dprintf ("Could not allocate Gobj\n");

	return tex;
}

/////////////////////////////////////////////////////////////////////////
//
// Free VRAM for texture (in your dreams)
//
/////////////////////////////////////////////////////////////////////////

void FreeTexture (vram_t *tex)
{
	tex->used=0;
}


/////////////////////////////////////////////////////////////////////////
//
// Display Objects
//
/////////////////////////////////////////////////////////////////////////

void DisplayObjects (void)
{
	register int n;
	register GfxObj *gobj;

	for (n=0; n<NUM_SOBJS; n++)
	{
		if (Sobj[n].Type)
		{
			if (Sobj[n].Flags & OBJ_DISPLAY) switch (Sobj[n].Type)
			{
				case OBJ_MENU:
					break;
				case OBJ_POLYTEX:
					gobj=(GfxObj *)Sobj[n].Data;
					while (gobj)
					{
						GsSortPoly (&gobj->Poly,gPtrGsOt,Sobj[n].Depth);
						gobj=gobj->Next;
					}
					break;
			}
		}
	}
}




/////////////////////////////////////////////////////////////////////////
//
// Free Shell Object & associated structures
//
/////////////////////////////////////////////////////////////////////////

void FreeObject (ShellObj *sobj)
{
	if (sobj==NULL) return;

	switch (sobj->Type)
	{
		case OBJ_MENU:
			break;
		case OBJ_POLYTEX:
			if (sobj->Data)
			{
				GfxObj *gobj;

				if (gobj=(GfxObj*)sobj->Data)
				{
					if (gobj->Texture)	FreeTexture (gobj->Texture);
					if (gobj->Clut)			free_VRam (gobj->Clut);
				}
				gobj->Attr=0;
			}
			sobj->Data=NULL;
			break;
		default:
			break;
	}
	sobj->Type=0;
}


/////////////////////////////////////////////////////////////////////////
//
// Free All Shell Objects & associated structures
//
/////////////////////////////////////////////////////////////////////////

void FreeObjects ()
{
	int n;


	for (n=0; n<NUM_SOBJS; n++) FreeObject (&Sobj[n]);

	for (n=0; n<NUM_TPAGES; n++) texture[n+TPAGES].used=0;		// for vram handler

	FreeMem (MiscBuffer);
}


/////////////////////////////////////////////////////////////////////////
//
// Set Object Luminosity
//
/////////////////////////////////////////////////////////////////////////

void SetObjectRGB (ShellObj *sobj,UBYTE r,UBYTE g,UBYTE b)
{
	GfxObj *gobj;

	gobj=(GfxObj*)sobj->Data;

	setRGB0 (&gobj->Poly,r,g,b);
}


/////////////////////////////////////////////////////////////////////////
//
// Add Graphics Object to Shell Object
//
/////////////////////////////////////////////////////////////////////////

GfxObj *AddGfxObj (ShellObj *sobj)
{
	GfxObj *gobj,*gobjptr;


	gobj=AllocGfxObj ();		// tack on gfxobjects
	if (gobj)
	{
		gobjptr=(GfxObj*)sobj->Data;
		if (gobjptr)
		{
			while (gobjptr->Next) gobjptr=gobjptr->Next;
			gobjptr->Next=gobj;
		}
		else (GfxObj*)sobj->Data=gobj;
	}

	return gobj;
}


/////////////////////////////////////////////////////////////////////////
//
// Alloc a Graphics Object
// returns NULL if can't allocate
//
/////////////////////////////////////////////////////////////////////////

GfxObj *AllocGfxObj ()
{
	GfxObj *gobj;
	int d;


	for (d=0; d<NUM_GOBJS; d++)
	{
		if (!Gobj[d].Attr)
		{
			Gobj[d].Next=NULL;
			Gobj[d].Attr=GOBJ_INUSE;		// alloc this GfxObj
			setPolyFT4 (&Gobj[d].Poly);
			Gobj[d].Texture=NULL;
			Gobj[d].Clut=NULL;

			gobj=&Gobj[d];
			break;
		}
	}

	if (gobj==NULL) Dprintf ("ERROR: Could not allocate GfxObj\n");

	return gobj;
}


/////////////////////////////////////////////////////////////////////////
//
// Set Object Position (upper left is 0,0)
//
/////////////////////////////////////////////////////////////////////////

void SetObjectPos (ShellObj *sobj,int x,int y)
{
	int w,h;
	GfxObj *gobj;


	sobj->X=x;
	sobj->Y=y;

	switch (sobj->Type)
	{
		case OBJ_POLYTEX:
			gobj = (GfxObj*)sobj->Data;

			while (gobj)
			{
				w = gobj->Poly.x1 - gobj->Poly.x0;
				h = gobj->Poly.y2 - gobj->Poly.y0;
				setXYWH (&gobj->Poly,x + gobj->X-CENTER_X,y + gobj->Y-CENTER_Y,w,h);
				gobj = gobj->Next;
			}
			break;
	}
}


/////////////////////////////////////////////////////////////////////////
//
// Set Object Center Position (based on size so size object first)
//
/////////////////////////////////////////////////////////////////////////

void SetObjectCenterPos (ShellObj *sobj,int x,int y)
{
	int w,h;
	GfxObj *gobj;


	sobj->X = x;
	sobj->Y = y;

	x -= CENTER_X + sobj->SW/2;		// offset all polys to center handle
	y -= CENTER_Y + sobj->SH/2;

	switch (sobj->Type)
	{
		case OBJ_POLYTEX:
			gobj = (GfxObj*)sobj->Data;

			while (gobj)
			{
				w = gobj->Poly.x1 - gobj->Poly.x0;
				h = gobj->Poly.y2 - gobj->Poly.y0;
				setXYWH (&gobj->Poly,x + gobj->X,y + gobj->Y,w,h);
				gobj = gobj->Next;
			}
			break;
	}
}


/////////////////////////////////////////////////////////////////////////
//
// Set Object Size (full size is 4096)
// please don't try to scale multi poly objects. They won't be aligned
//
/////////////////////////////////////////////////////////////////////////

void SetObjectSize (ShellObj *sobj,int size)
{
	POLY_FT4 *poly;
	int x,y,w,h;

	if (size < 0) size=0;

	sobj->SW = (sobj->W * size) >> 12;		// scale object pixel width/height
	sobj->SH = (sobj->H * size) >> 12;

	switch (sobj->Type)
	{
		case OBJ_POLYTEX:
		{
			GfxObj *gobj;

			gobj = (GfxObj*)sobj->Data;
			while (gobj)
			{
				poly = &gobj->Poly;

				x = poly->x0;
				y = poly->y0;
				w = (gobj->W * size) >> 12;
				h = (gobj->H * size) >> 12;

				setXYWH (poly,x,y,w,h);

				gobj = gobj->Next;
			}
			break;
		}
	}
}

/////////////////////////////////////////////////////////////////////////
//
// Frees any graphics associated with display system
//
/////////////////////////////////////////////////////////////////////////

void FreeGraphics ()
{
	FreeAllDialogBoxes ();
	initVRamHandling ();
}


/////////////////////////////////////////////////////////////////////////
//
// Window on the World
//
/////////////////////////////////////////////////////////////////////////

void TPageViewer ()
{
#if PC_DEBUG
	POLY_FT4 poly[32];
	int x,y,w,h;
	int n;

	if (JpadData[0] == (JPADRfront1 | JPADRfront2 | JPADLfront1 | JPADLfront2))
	{
		w = 512;		// size of window
		h = 160;

		x = 0;
		y = (240-h) / 2;

		for (n=24; n<32; n++)
		{
			setPolyFT4 (&poly[n]);
			poly[n].tpage = GetTPage (2,0,64*(n%16),256*(n/16));		// mode=2
			setXYWH (&poly[n],x-CENTER_X,y-CENTER_Y,w/8,h);
			setUVWH (&poly[n],0,0,(UBYTE)64,(UBYTE)255);
			setRGB0 (&poly[n],0x80,0x80,0x80);
			setSemiTrans (&poly[n],0);

			GsSortPoly (&poly[n],gPtrGsOt,0);

			x += 512/8;
		}
	}
#endif
}

// end of shellsys.c

