
//
//	shellsys.h -  Data Structures
//

#ifndef _SHELLSYS_H
#define _SHELLSYS_H 1

#include "multifil.h"
// double include problem right now...  #include "mmvram.h"

#define NUM_SOBJS 32
#define NUM_GOBJS 32

// location of fixed tapges used

#define NUM_TPAGES 8

#define TPAGES 24
#define TPAGEX ( 64 * (TPAGES%16))
#define TPAGEY (256 * (TPAGES/16))


#define CENTER_X 256
#define CENTER_Y 120

// Multidata chunk types

#define CHUNK(a,b,c,d) (a|(b<<8)|((ULONG)c<<16)|((ULONG)d<<24))

#define _MDFL_ CHUNK('M','D','F','L')		// file header

#define _GOBJ_ CHUNK('G','O','B','J')		// graphics object

#define _PIXL_ CHUNK('P','I','X','L')		// pixel data
#define _PIXC_ CHUNK('P','I','X','C')		// pixel data compressed
#define _CLUT_ CHUNK('C','L','U','T')		// clut

// ShellObj Types

#define OBJ_MENU		0x0001
#define OBJ_POLYTEX	0x0801

// ShellObj Flags

#define OBJ_DISPLAY 0x00000001

typedef struct ShellObj ShellObj;
struct ShellObj
{
	UWORD		Type;
	ULONG		Flags;
	WORD		X;			// current x,y handle
	WORD		Y;
	WORD		W;			// pixel size of entire image
	WORD		H;
	WORD		SW;			// pixel size scaled
	WORD		SH;
	WORD		Depth;
	void		*Data;
	int			(*Func)(void);
};

// GfxObj attributes

#define GOBJ_MODE			0x00000003	// 0 = 4 bit,1 = 8 bit,2 = 16 bit
#define	GOBJ_INUSE		0x80000000	// structure allocated to a ShellObj

typedef struct GfxObj GfxObj;
struct GfxObj
{
	GfxObj		*Next;			// link to more objects (too big for 1 tpage)
	WORD			X;					// offset from ShellObj X,Y
	WORD			Y;
	WORD			W;					// size (saved for scaling)
	WORD			H;
	ULONG 		Attr;				// Object attribute flags
  POLY_FT4	Poly;				// Polygon for object
  vram_t		*Texture;		// TPAGE data
  vram_t		*Clut;			// Clut vram
};

// object

// TPAGE FLAGS

#define USE_TIM_CLUT 0x01
#define KEEP_DATA		0x02

// Prototypes

// shellsys machine independent library calls

ShellObj *NewObject		(UWORD type,void *data);
ShellObj **LoadObjects (MultiFile_t *mfl,void *file);
void LoadObject (ShellObj *sobj,void *file);
void InitObjects ();
void FreeObjects ();
void FreeObject (ShellObj *sobj);
void DisplayObjects ();
void SetObjectPos (ShellObj *sobj,int x,int y);
void SetObjectCenterPos (ShellObj *sobj,int x,int y);
void SetObjectSize (ShellObj *sobj,int size);
void SetObjectRGB (ShellObj *sobj,UBYTE r,UBYTE g,UBYTE b);
void FreeGraphics ();

void TPageViewer ();

#endif

// end of shellsys.h

