/*************************************************************************
	Simulate a play	
	
	Author:	Mike Knauer - original
	 		Chris Deel - Current
	

		Copyright (C) 1997 Gametek Inc.
		All rights Reserved

*************************************************************************/

#include <stdio.h>
#include <stdlib.h>
#include <ctype.h>      /* for toupper  */

#include "prjtypes.h"

//#include "season.h"	// for season sim

#include "fbequ.h"
#include "fbram.h"
#include "fbbutton.h"
//#include "fbirp.h"

//#include "fakepsx.h"
//#include "biosvid.h"    /* old file - no need to carry it around  */

#include  "fbplays.h"
#include  "fbforms.h"
#include  "misc.h"

#include "SimTable.h"

#ifdef __MSDOS__
//like PSX
#define Dprintf printf  
#define PC_DEBUG 1
#endif //__MSDOS__

extern struct play ThePlays[];   // This should be put into an H file



/*-------------------------------------------------------------
  DumpSimTableEntry ~ 
-------------------------------------------------------------*/
void DumpSimTableEntry(void *entry, int pass)
{
	int i, max;
	WORD *ptr;
	
	ptr = entry;
	if (pass)
	{
		max = 5;
		Dprintf("\nPass Tbl: ");
	}
	else
	{
		max = 7;
		Dprintf("\nRun  Tbl: ");
	}
	for (i = 0; i < max; i++)
	{
		Dprintf("%5d", *(ptr + i));
	}
	Dprintf("");
}  /* End DumpSimTableEntry() */

/*-------------------------------------------------------------
  CalcDefPlayTypeNum ~ 
-------------------------------------------------------------*/
int CalcDefPlayTypeNum(int flags)
{
	int playType, i;
	int table[] = 
	    {ALL_PURPOSE, STOP_RUN, STOP_PASS_SHORT, STOP_PASS_DEEP, 
	        BLITZ, SPECIAL_TEAMS_DEF };
	
	playType = ALL_PURPOSE;
	for (i = 0; i < 5; i++)		        
	{
		if (flags & table[i])
			playType = i;
	}
	return playType;
}  /* End CalcDefPlayTypeNum() */


/*-------------------------------------------------------------
  SimFumble ~ Check if fumble happened and Sim if so
  		Return TRUE if fumble happened
-------------------------------------------------------------*/
BOOL SimFumble(int pct)
{
	if (random(1000) < pct)
	{
		Dprintf("\nFUMBLE! (not Simulated)");
		return TRUE;
	}
	else
		return FALSE;
}  /* End SimFumble() */

/*-------------------------------------------------------------
  SimPuntPlay ~ 
-------------------------------------------------------------*/
void SimPuntPlay(void)
{
	KickReturner = 	BallCarrier = 20;
	KickerSlot = 0;

	CatchBallX =  StartBallX + ((39 + (rand()&7)) >>YARD_SHIFT);	// punt dist.
	ForwardProgress = OPXPos(BallCarrier) =
		CatchBallX - (rand()&7)*YARDS;
}  /* End SimPuntPlay() */


/*-------------------------------------------------------------
  SimKickoffPlay ~ 
-------------------------------------------------------------*/
void SimKickoffPlay(void)
{
	KickReturner = 	BallCarrier = 0;
	KickerSlot = 11;
	CatchBallX = DEF_GOAL_LINE;
	ForwardProgress = OPXPos(BallCarrier) = DEF_GOAL_LINE + 21*YARDS;
}  /* End SimKickoffPlay() */


/*-------------------------------------------------------------
  SimRunningPlay ~ 
-------------------------------------------------------------*/
void SimRunningPlay(int offPlayType, int defPlayFlags)
{
	int defPlayType;
	RunSimTable_t  simPct, *tptr;
	int rnd1, gain, base, range;
	
	defPlayType = CalcDefPlayTypeNum(defPlayFlags);
	
	tptr = &RunSimTable[offPlayType][defPlayType];
//	memcpy(simPct, tptr, sizeof(simPct));
	CopyBytes((UBYTE*)tptr, (UBYTE*)&simPct, sizeof(RunSimTable_t));
	 DumpSimTableEntry(&simPct, 0);	

	if (SimFumble(simPct.Fumble))
	{
//		return;		// EXIT here if fumble!
	}
	 
	rnd1 = random(1000); 	
	// Adjust table for percent range
	simPct.NoGain += simPct.Loss;
	simPct.Gain5 += simPct.NoGain;
	simPct.Gain10 += simPct.Gain5;
	simPct.Gain20 += simPct.Gain10;
	simPct.GainMax += simPct.Gain20;
	
	if (rnd1 < simPct.Loss)
	{
		base = 1 * YARD;
		range = 6 * YARD;
	}
	else if (rnd1 < simPct.NoGain)
	{
		base = -5 * YARD;
		range = 6 * YARD;
	}
	else if (rnd1 < simPct.Gain5)
	{
		base = -1 * YARD;
		range = 3 * YARD;
	}
	else if (rnd1 < simPct.Gain10)
	{
		base = 0;
		range = 7 * YARD;
	}
	else if (rnd1 < simPct.Gain20)
	{
		base = 3 * YARD;
		range = 9 * YARD;
	}
	else //if (rnd1 < simPct.GainMax)
	{
		base = 20 * YARD;
		range = 80 * YARD;
	}
	
	gain = base + random(range);
	 	
	BallCarrier = 5;  //debug HalfBack for now

	ForwardProgress = OPXPos(BallCarrier) = StartBallX + gain;
				
	 Dprintf("\nRnd: %d base:%d  Gain:%d .%d", 
	 			rnd1, base/YARD, gain/YARD, gain&0xFF); //debug
}  /* End SimRunningPlay() */

/*-------------------------------------------------------------
  SimPassingPlay ~ 
-------------------------------------------------------------*/
void SimPassingPlay(int offPlayType, int defPlayFlags)
{
	int defPlayType;
	PassSimTable_t  simPct, *tptr;
	int rnd1, gain, base, range;
	
	defPlayType = CalcDefPlayTypeNum(defPlayFlags);
	
	tptr = &PassSimTable[offPlayType - PASS_SHORT][defPlayType];
	CopyBytes((UBYTE*)tptr, (UBYTE*)&simPct, sizeof(PassSimTable_t));
	 DumpSimTableEntry(&simPct, 1);	
//	 DumpSimTableEntry(tptr, 1);	

	if (SimFumble(simPct.Fumble))
	{
//		return;		// EXIT here if fumble!
	}

	rnd1 = random(1000); 	
	base = range = gain = 0;
		
	if (rnd1 < simPct.Sack)
	{
		gain = -5 * YARDS;
	}
	else  // no Sack
	{
		rnd1 = random(1000); 	
		// Adjust table for percent range
		simPct.Interception += simPct.Incomplete;
		simPct.Complete += simPct.Interception;

		if (rnd1 < simPct.Incomplete)
		{
			Dprintf(" Incomplete ");
			gain = 0;
		}
		else if (rnd1 < simPct.Interception)
		{
			Dprintf(" Interception (not simulated) ");
			gain = 0;
		}
		else if (rnd1 < simPct.Complete)
		{
			switch(offPlayType)
			{
				case PASS_SHORT:
					base = -2 * YARD;
					range = 10 * YARD;
					break;
				case PASS_MEDIUM:
					base = 6 * YARD;
					range = 10 * YARD;
					break;
				case PASS_DEEP:
					base = 14 * YARD;
					range = 30 * YARD;
					break;
			}
		}
		
		gain = base + random(range);
		Passer  = 0;
		BallCarrier = PassCatcher = (rand()&3) + 1;
		ForwardProgress = OPXPos(BallCarrier) =	StartBallX + gain;
	}	 
	 Dprintf("\nRnd: %d base:%d  Gain:%d .%d", 
	 			rnd1, base/YARD, gain/YARD, gain&0xFF); //debug

}  /* End SimPassingPlay() */

/*-------------------------------------------------------------
  SimThePlay ~ 
-------------------------------------------------------------*/
void SimThePlay(void)
{
	int offPlayNum, offPlayType;
	int defPlayNum, defPlayFlags;
	char *offPlayName, *defPlayName;
	
	offPlayNum = GameTeamData[OffensiveSlotNum].PlayNum;
	offPlayType = ThePlays[offPlayNum].PlayTypeDesignator;
	defPlayNum = GameTeamData[DefensiveSlotNum].PlayNum;
	defPlayFlags = ThePlays[defPlayNum].PlayTypeDesignator;
	offPlayName = PlayNames[offPlayNum];
	defPlayName = PlayNames[defPlayNum];

	 Dprintf("\n");
	 Dprintf("\nPlays: %s vs. %s", offPlayName, defPlayName);
	 Dprintf("\nPlay Types:%d %d %Xh",PlayType, offPlayType, defPlayFlags);
	 
	 
	if(PlayFlags & POS_PUNT_PLAY_BFLAG)
	{
		SimPuntPlay();
	}
	else if(PlayType == KICKOFF_PLAY)
	{
		SimKickoffPlay();
	}
	else
	{
		switch(offPlayType)
		{
			case RUN_INSIDE:
			case RUN_OUTSIDE:
				SimRunningPlay(offPlayType, defPlayFlags);
				break;
			
			case PASS_SHORT:
			case PASS_MEDIUM:
			case PASS_DEEP:
				SimPassingPlay(offPlayType, defPlayFlags);
				break;
			
			default:
				Dprintf(" Error! %s %d\n", __FILE__, __LINE__);
				BallCarrier = 0;  //debug HalfBack for now
				ForwardProgress = OPXPos(BallCarrier) = StartBallX;
				break;
		}
	}

	SetBit1(EventFlags,TACKLE_ENDED_BFLAG);
	Tackler = (rand()&7) +11;
	Dprintf("\nEnd SimThePlay   %d %d", ForwardProgress/YARDS, Tackler);
}  /* End SimThePlay() */

