/*************************************************************************
 * SOUND.C - Contains all routines for sound effects and music.
 *
 *	Started: 	10-13-95 (Friday the 13th)
 *	Written by: Mike Heilemann
 *************************************************************************/

#define SOUND_C 1

#include <sys/types.h>
#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsnd.h>
#include <libsn.h>
#include <libspu.h>

#include "version.h"
#include "prjtypes.h"
#include "debug.h"
#include "psxpad.h"
#include "mmvram.h"
#include "fonts.h"

#include "fbequ.h"
#include "fbram.h"
#include "gameloop.h"
#include "movie.h"
#include "sound.h"
#include "multifile.h"
#include "vab.h"

#define SOUND_TEST_COMMAND 1

#define LOOP  16
#define LOW_L 17
#define LOW_R 18
#define MID_L 19
#define MID_R 20
#define HI_L  21
#define HI_R  22
#define PHRASE_VOICE 23

#define VOL_RANGE 10

#define	COMPLETE 5
#define MAX_PHRASE 30
#define	NOT_READY -2
#define	READY -1
#define SayPhrase() (phrase_part = READY)
#define	SIZE_OF_VH	3104
#define	NUM_FLYIN_VABS 6
#define	NICKNAMES 4

extern int gLoadListState;

void TransferVb(char *body, short num);


INT LoadASectorMember(WORD filenum, WORD membernum, WORD vbnum, char *vhbuff);
SectorFile_t *SFL[18];
SectorFile_t *InitSectorFile(char *fname);
int LoadSectorFile(char *fname);


/*********************************/

short PhraseLoadList[MAX_PHRASE][2];
short PhraseLoadCount, PhraseLoadIndex, PhraseLoadState;
ULONG PhraseSectors[120][2];
INT	  PhraseFlag = 0;			// Triggers phrase
INT	  SoundInit = 0;

/*********************************/

short voice[24],	// for which voice is used
		seq[2] = {-1,-1},
		effect_vab[NUM_VAB],
		mus_vab,
//		loop = 0,
		phrase[MAX_PHRASE],
		phrase_part,
		phrase_pitch;

char *header[NUM_VAB],		// VAB Header
	  *mus_header,	// VAB Body
	  *mus_body,	// VAB Body
	  MusicFade = 0;

BOOL VabHeaderAlloced[NUM_VAB];  //CGD

char sfxvol[6] = {0, 30, 40, 50, 75, 100};
char musvol[6] = {0, 20, 35, 50, 75, 95};
short pause_voices[48];

char seq_table[SS_SEQ_TABSIZ * NUM_MUSIC_VOICES];	// #2 = # music voices, #3 = # seq in sep

char CurSong;

char *ShellSFX = NULL;

signed int ticker;

unsigned long *sequence,	// So sequence data space can be freed
				  *head_ptr,
				  *body_ptr;


/*
 * defines for sound
 */
void OpenEffectVhMul(char *file, short num);
void OpenEffectVbMul(char *file, short num);



char *SFL_Fnames[]={
				"GENERAL",		// Arcade announcer phrases
				"TEAMCAPT",		// Team captains
				"CITY1",		// City names for start of phrase
				"CITY1",		// City names start of phrase
				"NICK1",		// Nicknames
				"1STDOWN",		// First downs
				"2NDDOWN",		// Second downs
				"3RDDOWN",		// Third downs
				"4THDOWN",		// Fourth downs
				"GAIN",			// Gain run/pass
				"SACKS",		// Loss sacks 1-13
				"INTENDED",		// Intended for ...
				"LOSS",			// Loss (generic) 1-13
				"NUMBERS1",		// First part of phrase
				"NUMBERS1",		// Second part of phrase
				"BYNUM",		// By number ...
				"WELCOME",		// Welcome to the home of
				NULL,
				"PLAYERS",		// Player names
				"ONYARDLN",		// On the ?? yard line
};

WORD SFL_BaseDefs[]={
	ANNOUNCER_PHRASES,
	TEAM_CAPT_SFX,
	CITY_NAMES1_SFX,
	CITY_NAMES2_SFX,
	NICKNAME_SFX,
	FIRST_DOWN_SFX,
	SECOND_DOWN_SFX,
	THIRD_DOWN_SFX,
	FOURTH_DOWN_SFX,
	GAIN_SFX,
	SACKED_FOR_SFX,
	INTENDED_FOR_SFX,
	LOSS_ON_SFX,
	NUMBERS1_SFX,
	NUMBERS2_SFX,
	BY_NUMBER_SFX,
	WELCOME_TO_SFX,
	PLAYERS_NAMES_SFX,
};

/*************************************************************************
 * INIT SOUND - Initializes PSX sound system, Loads VAB's into Sound RAM
 *		Frees VAB Body, and sets up voices and effects.
 *************************************************************************/

void InitSound(void)
{
	short i;
	char fname[40];

	if(!ShellSFX)	// Only ever need to this once on boot up
	{
		ShellSFX = (char *)LoadCDFile("/SOUND/SHELL/SHELL.VB");

		i=0;

		if(!SoundInit)
		{
			while(SFL_Fnames[i])
			{
				SFL[i]=InitSectorFile(SFL_Fnames[i]);
				i++;
			}

			SoundInit=TRUE;
		}
	}

	SsInit ();		/* reset sound */

	SsSetTableSize(seq_table, NUM_MUSIC_VOICES, 1);

	/* set tick mode */
//	SsSetTickMode (SS_TICK60);
	SsSetTickMode (SS_TICK120);

	for (i = 0; i < NUM_VAB; i++)	// Zero sequences
	{
		effect_vab[i] = -1;
		header[i] = NULL;
	}

	for (i = 0; i < 24; i++)				// Zero voices
		voice[i] = -1;

	InitEffectsHead();		// Setup Effects headers

// Set Master Volume to MAX
	SsSetMVol(127, 127);

	SsStart();
}


/*-------------------------------------------------------------
  FreeEffectsHeadMfl ~
-------------------------------------------------------------*/
void FreeEffectsHeadMfl(void)
{
	if (head_ptr)
		FreeMem(head_ptr);
	head_ptr = NULL;
}  /* End FreeEffectsHeadMfl() */

/*************************************************************************
 * INIT EFFECTS - Opens up Arcade VAB's for game.
 *************************************************************************/
void InitEffectsHead(void)
{
// Open & transfer EFFECT VAB data
	if (head_ptr == NULL)
		head_ptr = (unsigned long *)LoadCDFile("/sound/sound.mfl");

	OpenEffectVhMul("SHELL",	0);
	OpenEffectVhMul("VAB0",		1);	// which slot to put it
	OpenEffectVhMul("VAB2",		2);
	OpenEffectVhMul("VAB30",	3);	// do a random one for colors & numbers.
	OpenEffectVhMul("VAB4",		4);
	OpenEffectVhMul("VAB5",		5);
	OpenEffectVhMul("PREGAME",	9);
}


/*************************************************************************
 * INIT EFFECTS BODY -
 *************************************************************************/
void InitEffectsBody(void)
{
// Open & transfer EFFECT VAB data
	body_ptr = (unsigned long *)LoadCDFile("/sound/soundbod.mfl");

	OpenEffectVbMul("SHELL",	0);
	OpenEffectVbMul("VAB0",		1);	// Load up Crowd VAB
	InitCrowd();				// Start Crowd Engine
	OpenEffectVbMul("VAB2",		2);
	OpenEffectVbMul("VAB4",		4);
//	OpenEffectVbMul("VAB5",		5);

	FreeMem(body_ptr);
}

#ifdef	TRUE_SOUND

/*************************************************************************
 * INIT PHRASE - Sets up a phrase to be triggered by Play Phrase
 *
 * NOTE: max phrase parts are 15.
 *************************************************************************/
void InitPhrase(short *PhrasePtr, short num)
{
	short i, j=0, k;

//	if (phrase[0] || num == 0)			// Is one playing or none to play
	if (PhraseFlag || num == 0)			// Is one playing or none to play
		return;

//// Phrase flyin counter
	PhraseLoadCount = PhraseLoadIndex = PhraseLoadState = 0;
	phrase_part = NOT_READY;

	// Free up any phrases in SPU

	for(i=11; i< NUM_FLYIN_VABS; i++) CloseEffectVab(i);
////


	if (GameOptions[GAME_OPTNS_ANNOUNCER_VOL] == 0 && PhrasePtr[0] >= ANNOUNCER_PHRASES)
		return;
	else
	if (GameOptions[GAME_OPTNS_REFEREE_VOL] == 0 && PhrasePtr[0] < ANNOUNCER_PHRASES)
		return;

	phrase[1]=0;

	for (i = 0; i < num; i++)
	{
		if (PhrasePtr[i] < 1) 	// Its a neg number, so split it...
		{
#if 0
			if (PhrasePtr[i] > -20)										// 1 - 10
			{
				phrase[j++] = (PhrasePtr[i] * -1) + ZERO_SFX;
			}
			else
			{
				phrase[j++] = (PhrasePtr[i] / -10) + EIGHTTEEN_SFX;	// Make them positive
				if (PhrasePtr[i] % 10)
				{
					phrase[j++] = ((PhrasePtr[i] % 10) * -1) + ZERO_SFX;
				}
			}
			phrase[j] = ANNOUNCER_PHRASES;	// Temp test phrase (all numbers)
#endif
		}
		else
		{
			phrase[j] = PhrasePtr[i];
		}

//// Temp setup to test flying in of sounds

		if(phrase[j] >= ANNOUNCER_PHRASES)
		{
			if(phrase[j] < TEAM_CAPT_SFX) k=0;
			else if(phrase[j] < CITY_NAMES1_SFX) k=1;
			 else if(phrase[j] < CITY_NAMES2_SFX) k=2;
			  else if(phrase[j] < NICKNAME_SFX) k=3;
			   else if(phrase[j] < FIRST_DOWN_SFX) k=4;
			    else if(phrase[j] < SECOND_DOWN_SFX) k=5;
			     else if(phrase[j] < THIRD_DOWN_SFX) k=6;
			      else if(phrase[j] < FOURTH_DOWN_SFX) k=7;
			       else if(phrase[j] < GAIN_SFX) k=8;
			        else if(phrase[j] < SACKED_FOR_SFX) k=9;
			         else if(phrase[j] < INTENDED_FOR_SFX) k=10;
			          else if(phrase[j] < LOSS_ON_SFX) k=11;
			           else if(phrase[j] < NUMBERS1_SFX) k=12;
			            else if(phrase[j] < NUMBERS2_SFX) k=13;
			             else if(phrase[j] < BY_NUMBER_SFX) k=14;
			              else if(phrase[j] < WELCOME_TO_SFX) k=15;
			               else if(phrase[j] < PLAYERS_NAMES_SFX) k=16;
						    else k=-1;

			if(k != -1)
			{
				Dprintf("FLYIN PHRASE: No. %d from %s\n",phrase[j]-SFL_BaseDefs[k], SFL_Fnames[k]);

				PhraseLoadList[PhraseLoadCount][0]= (phrase[j]-SFL_BaseDefs[k])<<1;
//				PhraseLoadList[PhraseLoadCount][0]= PhraseLoadCount+ANNOUNCER_PHRASES;
				PhraseLoadList[PhraseLoadCount][1]= k;

				phrase[j]= PhraseLoadCount+ANNOUNCER_PHRASES;	// offset to use unassigned effects[] elements
				effect[phrase[j]].vab = VAB11+PhraseLoadCount;
				effect[phrase[j]].tone = 0;
				effect[phrase[j]].prog = 0;
				PhraseLoadCount++;
			}
		}

////
		j++;
	}

	phrase_pitch = 20 + GetRandom(108);
	phrase[j] = 0;		// Zero last one
//	phrase_part = 100;
//	phrase_part = -1;
//	phrase_part = NOT_READY;
	ticker = 0;
	if(!PhraseLoadCount) phrase_part = READY;
	PhraseFlag = TRUE;	// Temp trigger until Mike gets his stuff setting it.
}
#endif	//TRUE_SOUND


/*************************************************************************
 * PLAY PHRASE - Called from within gameloop.
 *************************************************************************/
void PlayPhrase(void)
{
	register int vol,
					 part;

//	if (phrase[0] == 0 || phrase_part == NOT_READY)		// No phrase playing or waiting on background load
	if (!PhraseFlag || phrase_part == NOT_READY)		// No phrase playing or waiting on background load
		return;

//	if (phrase_part == 100)		// First time in (if we start in init, it goes straight to second sample)
//	{
//		phrase_part = -1;
//		ticker = 0;
//	}

	part = SpuGetKeyStatus(SPU_VOICECH(PHRASE_VOICE));	// Wait before playing next sample.
	if (part == SpuOff)	// if not playing
		ticker--;

	if (ticker < 0 && part == SpuOff)	// if not playing
	{
		phrase_part++;
		part = phrase[phrase_part];
		if (part == 0 || phrase_part == MAX_PHRASE)	// No more or at end.
		{
//			phrase[0] = 0;	// Stop playing
			PhraseFlag = 0;	// Stop playing
			return;
		}
		if (part >= ANNOUNCER_PHRASES)	// Announcer
			vol = effect[part].voll * sfxvol[GameOptions[GAME_OPTNS_ANNOUNCER_VOL]] / 100; // Adjust for Effect level
		else
			vol = effect[part].voll * sfxvol[GameOptions[GAME_OPTNS_REFEREE_VOL]] / 100; // Adjust for Effect level
		SsUtKeyOnV(PHRASE_VOICE, effect[part].vab, effect[part].prog, effect[part].tone, effect[part].note, phrase_pitch, vol, vol);
		ticker = 8;
	}
}


/*************************************************************************
 * STOP PHRASE - Stops current phrase.
 *************************************************************************/


void StopPhrase(void)
{
// 	phrase[0] = 0;	// Stop playing
 	PhraseFlag = 0;	// Stop playing
	SsUtKeyOffV(PHRASE_VOICE);
	VSync(0);
}


/*************************************************************************
 * PRE LOAD PHRASE VABS - Preloads current phrase.
 *************************************************************************/

char PhraseVBBuffer[1024*32];
char PhraseVHBuffer[NUM_FLYIN_VABS][SIZE_OF_VH];
char TmNamesVHBuffer[2][SIZE_OF_VH];

WORD TestSound;

void CheckPhraseLoad(void)
{
	INT result;
	BYTE s[40];

	if(!IsBackgroundLoading() && PhraseLoadCount) gLoadListState = 5;  // Set to complete to start loading cycle

	if(CheckLoadList() && PhraseLoadCount)	// Are there any phrases in the queue
	{
		switch(PhraseLoadState)
		{
			case 0:
					ClearLoadList();

					AddLoadList(GetSectorFileByNum(SFL[PhraseLoadList[PhraseLoadIndex][1]],
						PhraseLoadList[PhraseLoadIndex][0]+1), (char *) &PhraseVHBuffer[PhraseLoadIndex][0]);
//					AddLoadList(GetSectorFileSector(SFL[PhraseLoadList[PhraseLoadIndex][1]],
//						"WELC00.VH"), (char *) &PhraseVHBuffer[PhraseLoadIndex][0]);


					StartLoadList();
					header[11+PhraseLoadIndex] = (char *)&PhraseVHBuffer[PhraseLoadIndex][0];
					VabHeaderAlloced[11+PhraseLoadIndex] = NULL;
					PhraseLoadState++;
					break;

			case 1:
					ClearLoadList();

					AddLoadList(GetSectorFileByNum(SFL[PhraseLoadList[PhraseLoadIndex][1]],
						PhraseLoadList[PhraseLoadIndex][0]), PhraseVBBuffer);
//					AddLoadList(GetSectorFileSector(SFL[PhraseLoadList[PhraseLoadIndex][1]],
//						"WELC00.VB"), PhraseVBBuffer);

					StartLoadList();
					PhraseLoadState++;
					break;

			case 2:
					TransferVb(PhraseVBBuffer, 11+PhraseLoadIndex);
					PhraseLoadCount--;
					PhraseLoadState=0;
					PhraseLoadIndex++;
					if(!PhraseLoadCount)
					{
						phrase_part = READY;	// Allow phrase to start playing
						ClearLoadList();		// Clear list to enable background load system
					}
					break;
		}
	}
}



#ifdef	TRUE_SOUND
/*************************************************************************
 * PLAY EFFECT - Plays effect NUM on an empty voice if possible
 *************************************************************************/
short PlayEffect(short num)
{
	register short i,
						vol;

	if(num >=	FIRST_DOWN_SFX)
		vol = effect[num].voll * sfxvol[GameOptions[GAME_OPTNS_REFEREE_VOL]] / 100; // Adjust for Effect level
	else
		vol = effect[num].voll * sfxvol[GameOptions[GAME_OPTNS_SFX_VOL]] / 100; // Adjust for Effect level

	if (vol < 0)
		vol = 0;

	if (num > CURSOR_SFX)
		i = SsUtKeyOn(effect[num].vab, effect[num].prog, effect[num].tone, effect[num].note, 20 + GetRandom(108), vol, vol);
	else
		i = SsUtKeyOn(effect[num].vab, effect[num].prog, effect[num].tone, effect[num].note, FINE, vol, vol);

	voice[i] = num;
	return(i);
}


/*************************************************************************
 * PLAY EFFECT CHECK - Plays effect NUM on an empty voice if possible.
 *		If the current effect is playing, it returns the voice it is playing
 *		on, otherwise it plays the effect.
 *************************************************************************/
short PlayEffectCheck(short num)
{
	short i;

	for (i = 0; i < 24; i++)
		if (voice[i] == num)	// if it is playing exit
			if (CheckEffect(i))
				return(i);

	return(PlayEffect(num));
}


/*************************************************************************
 * PLAY EFFECT PITCH CHECK - Plays effect NUM on an empty voice if possible.
 *		If the current effect is playing, it returns the voice it is playing
 *		on, otherwise it plays the effect.
 *
 *	This version randomly changes the pitch from 62 - 66
 *************************************************************************/
short PlayEffectPitchCheck(short num, short pitch)
{
	register short i,
						vol;

	vol = effect[num].voll * sfxvol[GameOptions[GAME_OPTNS_SFX_VOL]] / 100; // Adjust for Effect level
	if (vol < 0)
		vol = 0;

	for (i = 0; i < 24; i++)
		if (voice[i] == num)	// if it is playing exit
			if (CheckEffect(i))
				return(i);

	i = SsUtKeyOn(effect[num].vab, effect[num].prog, effect[num].tone, pitch, 20 + GetRandom(108), vol, vol);

	voice[i] = num;
	return(i);
}


/*************************************************************************
 * CHECK EFFECT - Checks to see if effect is currently still playing.
 *		Returns: 1 if ON  - SpuOn | SpuOnEnvOff | SpuOffEnvOn
 *		Returns: 0 if OFF - SpuOff
 *************************************************************************/
short CheckEffect(short voice)
{
	long i;

	i = SpuGetKeyStatus(SPU_VOICECH(voice));

	if (i == SpuOff)
		return(0);
	return(1);
}


#endif	//TRUE_SOUND

/*************************************************************************
 * STOP EFFECT - Checks all voices and stops effect NUM on all voices.
 *************************************************************************/
void StopEffect(short num)
{
	short i;

	for (i = 0; i < 24; i++)
		if (voice[i] == num)
			SsUtKeyOff(i, effect[num].vab, effect[num].prog, effect[num].tone, effect[num].note);
}

#ifdef	TRUE_SOUND

/*************************************************************************
 * SET VOICE VOLUME - Set voice volume by values passed in (for both Left
 *		and Right channels)
 *************************************************************************/
void SetVoiceVolume(short voice, short voll, short volr)
{
	register short	vol_l,
						vol_r;

	vol_l = voll * sfxvol[GameOptions[GAME_OPTNS_SFX_VOL]] / 100; // Adjust for Effect level
	vol_r = volr * sfxvol[GameOptions[GAME_OPTNS_SFX_VOL]] / 100; // Adjust for Effect level
	if (vol_l < 0)
		vol_l = 0;
	if (vol_r < 0)
		vol_r = 0;

	SsUtSetVVol(voice, vol_l, vol_r);
}

#endif	//TRUE_SOUND

/*************************************************************************
 * CHANGE VOICE VOLUME - Increment or decrememt voice volume by value
 *		passed in (for both Left and Right channels)
 *************************************************************************/
void ChangeVoiceVolume(short voice, short incl, short incr)
{
short voll,
		volr;

	SsUtGetVVol(voice, &voll, &volr);

	voll += incl;
	volr += incr;

	if (voll > 127)		// Be sure we don't make vol > 127
		voll = 127;
	if (volr > 127)
		volr = 127;

	SsUtSetVVol(voice, voll, volr);
}


/*************************************************************************
 * STOP SOUND - Stop Sound System & clean up memory.
 *************************************************************************/
void StopSound(void)
{
	char i;

	CurSong = 0;

//	SsVabClose (mus_vab);					/* Close vab data */
//	FreeFileData(mus_header);				// Free MUSIC VAB Header

//	SsEnd();		/* sound system end */	  //CGD

	StopPhrase();	//CGD
	CloseMusVab();	//CGD
	for (i = 0; i < NUM_VAB; i++)
	{
		if(effect_vab[i] != -1)
			CloseEffectVab(effect_vab[i]);	/* Close vab data */
	}

	FreeMem(head_ptr);
	head_ptr = NULL;
	SsEnd();								/* sound system end */
	SsQuit();
	VSync(0);
}


/*************************************************************************
 * CLOSE VAB BODY - Close the VAB Body for use, keeping the header intact.
 *************************************************************************/
void CloseVabBody(char num)
{
	register short i;

	if (effect_vab[num] == -1)
		return;

	for (i = 0; i < NUM_EFFECT_VOICES; i++) 	// Put VAB into effect
		if (effect[i].vab == effect_vab[num]) 			// If not used they are 100-1XX
			effect[i].vab = 100+num;		// Make sure they have the same VAB

	SsVabClose(effect_vab[num]);			// Close VAB data
	effect_vab[num] = -1;
}


/*************************************************************************
 * CLOSE EFFECT VAB - Closes and frees the Effect VAB so another may be
 *		loaded.
 *************************************************************************/
void CloseEffectVab(char num)
{
	if (header[num])
	{
		CloseVabBody(num);
		if (VabHeaderAlloced[num])	 //CGD
			FreeFileData(header[num]);	// Free EFFECT VAB Header
		header[num] = NULL;
	}
}


/*************************************************************************
 * CLOSE MUS VAB - Closes and frees the Music VAB so another may be loaded
 *************************************************************************/
void CloseMusVab(void)
{
	if (mus_header)
	{
		SsVabClose(mus_vab);					// Close VAB data
		FreeFileData(mus_header);			// Free MUSIC VAB Header
		mus_header = NULL;
	}
}

/*************************************************************************
 * OPEN EFFECT VH - Opens a sound effect Headder for use.
 *
 *	NOTE: File name must not have an	extension as we add the correct ones.
 *************************************************************************/

void OpenEffectVh(char *file, short num)
{
	char s[40];

	/* Load VAB Headder */
	strcpy(s, "/SOUND/");
	strcat(s, file);
	strcat(s, ".VH");
	header[num] = (char *)LoadCDFile(s);
	VabHeaderAlloced[num] = TRUE;
}

/*************************************************************************
 * OPEN EFFECT VH MUL - Opens a sound effect Headder from a multifile.
 *
 *	NOTE: File name must not have an	extension as we add the correct ones.
 *************************************************************************/
void OpenEffectVhMul(char *file, short num)
{
	char s[40];

	/* Load VAB Headder */
	strcpy(s, file);
	strcat(s, ".VH");
	header[num] = (char *)GetMultiFilePtr(head_ptr, s);
	VabHeaderAlloced[num] = FALSE;
}

/*************************************************************************
 * TRANSFER VB - Transfers a VAB Body to sound ram.
 *************************************************************************/
void TransferVb(char *body, short num)
{
	register int i;

	/* open vab head */
	CloseVabBody(num);	// Make sure it is not there already...
	effect_vab[num] = SsVabOpenHead((unsigned char*)header[num], -1);

	if (effect_vab[num] == -1)
	{
		DebugPrintf(0,"OPEN OF VAB HEADER FAILED\n");
		return; // FAILED
	}

	if (SsVabTransBody((unsigned char*)body, effect_vab[num]) != effect_vab[num])
	{
		DebugPrintf(0,"TRANSFER FAILED\n");
		return; // FAILED
	}

	for (i = 0; i < NUM_EFFECT_VOICES; i++) 	// Put VAB into effect
	{
		if (effect[i].vab == num+100) 			// If not used they are 100-1XX
			effect[i].vab = effect_vab[num];	// Make sure they have the same VAB
	}

	SsVabTransCompleted (SS_WAIT_COMPLETED);	// Be sure it is loaded
}


/*************************************************************************
 * OPEN EFFECT VB - Opens a sound effect Body for use.
 *
 *	NOTE: File name must not have an	extension as we add the correct ones.
 *************************************************************************/
void OpenEffectVb(char *file, short num)
{
	char 	   s[40],
				*body;			// VAB Body

// Open & transfer EFFECT VAB data
	/* Load VAB Body */
	strcpy(s, "/SOUND/");
	strcat(s, file);
	strcat(s, ".VB");
	body = (char *)LoadCDFile(s);

	TransferVb(body, num);
	FreeFileData(body);						// Free EFFECT VAB Body
}


/*************************************************************************
 * OPEN EFFECT VB MUL - Opens a sound effect Body for use.
 *
 *	NOTE: File name must not have an	extension as we add the correct ones.
 *************************************************************************/
void OpenEffectVbMul(char *file, short num)
{
	char 	   s[40],
				*body;			// VAB Body

// Open & transfer EFFECT VAB data
	/* Load VAB Body */
	strcpy(s, file);
	strcat(s, ".VB");
	body = (char *)GetMultiFilePtr(body_ptr, s);

	TransferVb(body, num);
}


/*************************************************************************
 * OPEN EFFECT VAB - Opens a sound effect VAB for use.
 *
 *	NOTE: File name must not have an	extension as we add the correct ones.
 *************************************************************************/
void OpenEffectVab(char *file, short num)
{
	OpenEffectVh(file, num);
	OpenEffectVb(file, num);
}


/*************************************************************************
 * OPEN MUSIC VAB - Opens a music VAB for use.
 *
 *	NOTE: File name must not have an	extension as we add the correct ones.
 *************************************************************************/
void OpenMusicVab(char *file)
{
	char s[80];

// Open & transfer MUSIC VAB data
	/* Load VAB Headder */
	strcpy(s, file);
	strcat(s, ".VH");

	mus_header = (char *)LoadCDFile(s);

	/* Load VAB Body */
	strcpy(s, file);
	strcat(s, ".VB");
	mus_body = (char *)LoadCDFile(s);


	mus_vab = SsVabOpenHead((unsigned char*)mus_header, -1);
	if (mus_vab == -1) {
		DebugPrintf(0,"OPEN FAILED\n");
		return; // FAILED
	}

	if (SsVabTransBody((unsigned char*)mus_body, mus_vab) != mus_vab) {
		DebugPrintf("ERROR IN VAB TRANSFER OF %d\n", file);
		return; // FAILED
	}
	SsVabTransCompleted (SS_WAIT_COMPLETED);	// Be sure it is loaded
	FreeFileData(mus_body);					// Free MUSIC VAB Body

}


/*************************************************************************
 * OPEN SEQUENCE - Loads sequence data from CD and sets VOL to max.
 *************************************************************************/
void OpenSequence(char *file)
{
	char s[30];

	strcpy(s, file);
	strcat(s, ".SEQ");

	sequence = (unsigned long *)LoadCDFile(s);
	seq[0] = SsSeqOpen(sequence, mus_vab);
}


/*************************************************************************
 * PLAY SEQUENCE - Starts a sequence loaded with OpenSequence.
 *
 *		times = -1 Play infinitly (must be stopped with StopSequence)
 *		times =  # Play # times
 *************************************************************************/
void PlaySequence(short times)
{
	if (times == -1)
		SsSeqPlay(seq[0], SSPLAY_PLAY, SSPLAY_INFINITY);
	else
		SsSeqPlay(seq[0], SSPLAY_PLAY, times);

	SsSeqSetVol(seq[0], 0, 0);
	FadeMusic(0, 120);
}


/*************************************************************************
 * PLAY SONG - Loads and plays a song.
 *************************************************************************/
void PlaySong(char num)
{
	char s[30];

	if (num == CurSong)
		return;

	num &=0x7F;

	sprintf(s, "/SOUND/MUSIC/SONG%d", num);

	CloseVabBody(0);						// Close SHELL so no SRAM fragmentation.
	StopSequence();						// Kills music
	CloseMusVab();							// Dumps VAB

	OpenMusicVab(s);						// Open and load music VAB
	OpenSequence(s);						// Open new song
	PlaySequence(-1);						// Start it...

//*********************
//	OpenEffectVb("SHELL/SHELL", 0);	// Reload Shell sounds (music must be 1st)

//	if(!ShellSFX)
//		ShellSFX = (char *)LoadCDFile("/SOUND/SHELL/SHELL.VB");

	TransferVb(ShellSFX, 0);
//*********************

	CurSong = num;
}


/*************************************************************************
 * FADE MUSIC - Fades sound based upon MusicFade var 0 = up, 1 = down,
 *		0 = none.  MAX VOL = 70
 *************************************************************************/
void FadeMusic(char MusicFade, short ticks)
{
	if (seq[0] == -1) return;

	if (MusicFade == 0)	// Up
	{
		SsSeqSetCrescendo(seq[0], musvol[GameOptions[GAME_OPTNS_MUSIC_VOL]], ticks);
	}
	else
	if (MusicFade == 1)	// Down
	{
//		SsSeqSetCrescendo(seq[0], -musvol[GameOptions[GAME_OPTNS_MUSIC_VOL]], ticks);
		SsSeqSetDecrescendo(seq[0], musvol[GameOptions[GAME_OPTNS_MUSIC_VOL]], ticks);
	}
}


/*************************************************************************
 * STOP SEQUENCE - Stops currently playing sequence.
 *************************************************************************/
void StopSequence(void)
{
	if (seq[0] != -1)
	{
		CurSong = 0;
		SsSeqStop(seq[0]);
		SsSeqClose(seq[0]);					// Dumps Sequence
		FreeFileData(sequence);	 			// Free Sequence data
		seq[0] = -1;
	}
}


/*************************************************************************
 * SET MUSIC VOLUME - Sets the volume of a sequence (voice of music).
 *		Valid levels are 0-127
 *
 *	NOTE: Currently sets current both channels (Left & Right) to same Vol.
 *************************************************************************/
void SetMusicVolume(void)
{
	SsSeqSetVol(seq[0], musvol[GameOptions[GAME_OPTNS_MUSIC_VOL]], musvol[GameOptions[GAME_OPTNS_MUSIC_VOL]]);
}


/*************************************************************************
 * SET NEXT SEQUENCE - Sets up next sequence to play (SEQ2) when the
 *		current one is finished (SEQ1).
 *************************************************************************/
void SetNextSequence(short seq1, short seq2)
{
	SsSeqSetNext(seq1, seq2);
}


/*************************************************************************
 * INIT CROWD - Initiates crowd and sets volume to 0
 *************************************************************************/
void InitCrowd(void)
{
//	SsSetReservedVoice(17);	// Reserve channels 17 - 23 for crowd and announcer.
	SsSetReservedVoice(16);	// Reserve channels 16 - 23 for crowd and announcer.

	SsUtKeyOnV(LOW_L, effect[LOWCROWDL].vab, effect[LOWCROWDL].prog, effect[LOWCROWDL].tone, effect[LOWCROWDL].note, FINE, 127, 127);
	SsUtKeyOnV(LOW_R, effect[LOWCROWDR].vab, effect[LOWCROWDR].prog, effect[LOWCROWDR].tone, effect[LOWCROWDR].note, FINE, 127, 127);
	SsUtKeyOnV(MID_L, effect[MIDCROWDL].vab, effect[MIDCROWDL].prog, effect[MIDCROWDL].tone, effect[MIDCROWDL].note, FINE, 127, 127);
	SsUtKeyOnV(MID_R, effect[MIDCROWDR].vab, effect[MIDCROWDR].prog, effect[MIDCROWDR].tone, effect[MIDCROWDR].note, FINE, 127, 127);
	SsUtKeyOnV(HI_L , effect[HICROWDL].vab, effect[HICROWDL].prog, effect[HICROWDL].tone, effect[HICROWDL].note, FINE, 127, 127);
	SsUtKeyOnV(HI_R , effect[HICROWDR].vab, effect[HICROWDR].prog, effect[HICROWDR].tone, effect[HICROWDR].note, FINE, 127, 127);

	voice[LOW_L] = LOWCROWDL;
	voice[LOW_R] = LOWCROWDR;
	voice[MID_L] = MIDCROWDL;
	voice[MID_R] = MIDCROWDR;
	voice[HI_L ] = HICROWDL;
	voice[HI_R ] = HICROWDR;

	SetVoiceVolume(LOW_L, 0, 0);
	SetVoiceVolume(LOW_R, 0, 0);
	SetVoiceVolume(MID_L, 0, 0);
	SetVoiceVolume(MID_R, 0, 0);
	SetVoiceVolume(HI_L, 0, 0);
	SetVoiceVolume(HI_R, 0, 0);
}

#ifdef	TRUE_SOUND

/*************************************************************************
 * INIT LOOP - Sets loop to play in addition to regular crowd.
 *************************************************************************/
void InitSFXLoop(short sound)
{
	SsUtKeyOffV(LOOP);
	SsUtKeyOnV(LOOP, effect[sound].vab, effect[sound].prog, effect[sound].tone, effect[sound].note, FINE, effect[sound].voll, effect[sound].volr);
}


/*************************************************************************
 * SET CROWD LEVEL - Sets crowd level and plays the right crowd mix
 *************************************************************************/
void SetCrowdLevel(short level1, short level2, short level3, short level4, short percent)
{
	register	char left,
					  right;
	register short old_effect_level;

	int	i;
	short level[4];

	level[0] = level1;
	level[1] = level2;
	level[2] = level3;
	level[3] = level4;

	percent = percent << 1;					// Get % from 0 - 200 = % for each channel

	old_effect_level = GameOptions[GAME_OPTNS_SFX_VOL];		// Save old Effect level
	GameOptions[GAME_OPTNS_SFX_VOL] = GameOptions[GAME_OPTNS_CROWD_VOL];		// Set it to default, crowd uses set volume...

// CROWD SOUND
	for (i = 0; i < 4; i++)
	{
		left  = (level[i] * percent) / 100;	// Pan left/right
		right = (level[i] * abs(percent - 200)) / 100;

		if (left > 127)							// Stay in bounds (max vol = 127)
			left = 127;
		if (right > 127)
			right = 127;

		if (left < 20)								// Stay in bounds (max vol = 127)
			left = 20;
		if (right < 20)
			right = 20;

		if (i < 3)
		{
			SetVoiceVolume(LOW_L + (i << 1), left, 0);		// Left Channel
			SetVoiceVolume(LOW_R + (i << 1), 0, right);		// Right Channel
		}
		else
			SetVoiceVolume(LOOP, left, right);	// Loop Channel
	}

	GameOptions[GAME_OPTNS_SFX_VOL] = old_effect_level;		// Set Effect level back to previous
}

#endif	//TRUE_SOUND

/*************************************************************************
 * STOP CROWD - Terminates crowd effects.
 *************************************************************************/
void StopCrowd(void)
{
	StopEffect(LOWCROWDL);
	StopEffect(LOWCROWDR);
	StopEffect(MIDCROWDL);
	StopEffect(MIDCROWDR);
	StopEffect(HICROWDL);
	StopEffect(HICROWDR);
	SsUtKeyOffV(LOOP);
	SsSetReservedVoice(23);	// Release channels 17 - 23 for use by music.
}

#ifdef	TRUE_SOUND

/*************************************************************************
 * INIT AUDIO - Called from FBAIMAIN.C - Stops sound, Dump VAB,
 *		Load Effects, and Start crowd Engine
 *************************************************************************/
void InitAudio(void)
{
	char s[40];
	int i;

	FadeMusic(1,60);

	CloseVabBody(0);		// Close SHELL so no SRAM fragmentation.
	StopSequence();		// Kills music
	CloseMusVab();

	InitEffectsBody();

	for(i=0; i<2; i++)
		LoadASectorMember(NICKNAMES, GameTeamData[i].TeamNumber, 5+i, &TmNamesVHBuffer[i][0]);

// If we don't do a fly by of the stadium, load up VAB3.
//	if (gSkipCoinTossFlag || gAttractMode)
		OpenEffectVb("VAB3/VAB30",	3);	// do a random one for colors & numbers.
//	else
//		OpenEffectVb("VAB3/PREGAME", 9);
}

#endif	//	TRUE_SOUND

/*************************************************************************
 * FREE AUDIO - Called at end of game - Stops sound, Dump VAB,
 *		Loads Music, and Starts Sequence
 *************************************************************************/
void FreeAudio(void)
{
	StopPhrase();			// Kill playing phrase
	StopCrowd();	 		// Kill Crowd

	CloseEffectVab(7);	// Dump Home Team VAB
	CloseEffectVab(8);	// Dump Away Team VAB

	CloseVabBody(0);
	CloseVabBody(1);		// Dump VAB Bodys from Sound Ram
	CloseVabBody(2);
	CloseVabBody(3);
	CloseVabBody(4);
	CloseVabBody(5);

//	CloseVabBody(9);		//CGD
//	CloseVabBody(10);		//CGD
}

/*************************************************************************
 * PLAY PREGAME - Plays "Ladys and Gentlemen, Welcome to the home of your
 *		City, Team.
 *************************************************************************/
void PlayPregame(void)
{
	short s[15];

	s[0] = WELCOME_TO_SFX;
//	s[1] = TO_THE_HOME_SFX;
//	s[2] = SILENCE_SFX;
//	s[3] = SILENCE_SFX;
//	s[4] = HOME_CITY;
//	if (GameTeamData[0].TeamNumber < 30)
//	{
//		s[5] = SILENCE_SFX;
//		s[6] = SILENCE_SFX;
//		s[7] = HOME_TEAM;
//		InitPhrase(s, 8);
//	}
//	else
//	{
//		InitPhrase(s, 5);
//	}

	InitPhrase(s, 1);
}


/*************************************************************************
 * FREE PREGAME - Closes Pregame VAB and loads up QB VAB
 *************************************************************************/
void FreePregame(void)
{
	while (phrase_part == 100 || phrase[phrase_part])
		;	// Wait till they are finished talking before dumping

	CloseVabBody(9);	// Get rid of #3 PREGAME
	OpenEffectVab("VAB3/COINTOSS", 10);	// do a random one for colors & numbers.
}


/*************************************************************************
 * PLAY COIN TOSS -
 *************************************************************************/
void PlayCoinToss(char num, char team)
{
	short s[15];

	switch(num)
	{
		case 0: s[0] = AWAYTM_SFX;
				  s[1] = SILENCE_SFX;
				  s[2] = CALL_IT_SFX;
				  s[3] = SILENCE_SFX;
				  s[4] = HEADS_OR_SFX;
				  InitPhrase(s, 5);
				  break;
		case 1: s[0] = HOMETM_SFX + team;
				  s[1] = SILENCE_SFX;
				  s[2] = SILENCE_SFX;
				  s[3] = CHOOSE_SFX;
				  InitPhrase(s, 4);
				  break;
		case 2: s[0] = HOMETM_SFX + team;
				  s[1] = SILENCE_SFX;
				  s[2] = SILENCE_SFX;
				  s[3] = CHOOSE_SIDE_SFX;
				  InitPhrase(s, 4);
				  break;
	}

}


/*************************************************************************
 * FREE COIN TOSS -
 *************************************************************************/
void FreeCoinToss(void)
{
	while (phrase_part == 100 || phrase[phrase_part])
		;	// Wait till they are finished talking before dumping
	CloseEffectVab(10);	// Get rid of COINTOSS
	OpenEffectVb("VAB3/VAB30",	3);	// do a random one for colors & numbers.
}


int Shell_Frame_DPC(void);
extern Sint32 PreSimGame;	// Didn't do CPU vs CPU, use stats

/******************* Sound Test routines *****************/


/*************************************************************************
 * DO SOUND TEST COMMAND - Sound Testing
 *************************************************************************/

void DoSoundTestCommand(short command)
{

	#if SOUND_TEST_COMMAND
//	#if 0

	short snd[2];
	register short i;
	char s[50],
		  x,
		  y;

	switch(command -1)
	{
		case 0: i = 0;
				  break;
		case 1: i = BLOCK_IT_SFX;			// SIDELINE
				  break;
		case 2: i = GRUNT0_SFX;				// SFX
				  break;
		case 3: i = READY_SFX;				// QB Sounds
				  break;
		case 4: i = FIRST_DOWN_SFX;			// REF
				  break;
		case 5: i = ANNOUNCER_PHRASES;		// ANNOUNCER
				  break;
		case 6:
				i =	FIRST_DOWN_SFX;
				  break;
		case 7:
				i =	GAIN_SFX;
				  break;
		case 8:
					i = TEAM_BIO_21_MOV;
					StopCrowd();
					ResetStrings();
				  	ClearScreen();
					DoFrameLoop(1, Shell_Frame_DPC);
//					sprintf(s, "MOVIE:%d", i);
//					y = CreateString(s, fontdpc9, 20, 70, FONT0 | PROPORTIONAL); /* */

					PlayMovie(i);
					while(1)
					{
						ReadPads();

						if (IsNewButton())
						{
							if (JpadData[0] & JPADtriangle)
							{
								i -= 10;
								if (i < 0)
									i = 0;
								PlayMovie(i);
							}
							if (JpadData[0] & JPADLup)
							{
								i--;
								if (i < 0)
									i = 0;
								PlayMovie(i);
							}

							if (JpadData[0] & JPADx)
							{
								i += 10;
								if (i > KONAMI_MOV)
									i = KONAMI_MOV;
								PlayMovie(i);
							}
							if (JpadData[0] & JPADLdown)
							{
								i++;
								if (i > KONAMI_MOV)
									i = KONAMI_MOV;
								PlayMovie(i);
							}

							if (JpadData[0] & JPADcircle)
							{
								PlayMovie(i);
							}

							if (JpadData[0] & JPADsquare)
							{
								InitCrowd();
								return;
							}
						}
					}
		case 9: PreSimGame ^= TRUE;
				  return;
		default: i = 0;
	}

	StopCrowd();
	ResetStrings();
  	ClearScreen();
	sprintf(s, "EFFECT:%3d V:%2d P:%2d T:%2d ", i, effect[i].vab, effect[i].prog, effect[i].tone);
	y = CreateString(s, fontdpc9, 20, 70, FONT0 | PROPORTIONAL); /* */
	x = PlayEffectCheck(i);
	while(1)
	{
		ReadPads();

		CheckPhraseLoad();

		DoFrameLoop(1, Shell_Frame_DPC);
		if (IsNewButton())
		{
			if (JpadData[0] & JPADtriangle)
			{
				StopEffect(i);
//				x = PlayEffectCheck(--i);

				snd[0]=--i;
				PhraseFlag=FALSE;
				InitPhrase(snd,1);


				sprintf(s, "EFFECT:%3d V:%2d P:%2d T:%2d", i, effect[i].vab, effect[i].prog, effect[i].tone);
				ChangeString(y, s, FONT0 | PROPORTIONAL); /* */
				DoFrameLoop(1, Shell_Frame_DPC);
			}

			if (JpadData[0] & JPADx)
			{
				StopEffect(i);
//				x = PlayEffectCheck(++i);

				snd[0]=++i;
				PhraseFlag=FALSE;
				InitPhrase(snd,1);

				sprintf(s, "EFFECT:%3d V:%2d P:%2d T:%2d", i, effect[i].vab, effect[i].prog, effect[i].tone);
				ChangeString(y, s, FONT0 | PROPORTIONAL); /* */
				DoFrameLoop(1, Shell_Frame_DPC);
			}

			if (JpadData[0] & JPADcircle)
			{
//				x = PlayEffectCheck(i);
				PhraseFlag=FALSE;
				InitPhrase(snd,1);
			}

			if (JpadData[0] & JPADsquare)
			{
				InitCrowd();
				return;
			}
		}
	}
	#endif //SOUND_TEST_COMMAND
}


/*************************************************************************
 * FREE PREGAME - Closes QB VAB & openes cointoss VAB
 *************************************************************************/
void InitOvertimeAudio(void)
{
	while (phrase_part == 100 || phrase[phrase_part])
		;	// Wait till they are finished talking before dumping
	CloseVabBody(3);	// Get rid of #3 QB GAME Samples
	OpenEffectVab("VAB3/COINTOSS",	10);	// do a random one for colors & numbers.
}


/*************************************************************************
 * PAUSE SOUND - Turns volumes on all channels to 0
 *
 *		Pause = 0, Pause OFF
 *		Pause = 1, Pause ON
 *************************************************************************/
void PauseSound(int pause)
{
	int i;

	if (pause)
		for (i = 0; i < 48; i += 2)
		{
			SsUtGetVVol(i/2, &pause_voices[i], &pause_voices[i+1]);
			SsUtSetVVol(i/2, 0, 0);
		}
	else
		for (i = 0; i < 48; i += 2)
		{
			SsUtSetVVol(i/2, pause_voices[i], pause_voices[i+1]);
		}
}

/*-------------------------------------------------------------
  IsSoundPlaying ~   CGD
-------------------------------------------------------------*/
BOOL IsSoundPlaying(void)
{
	int i;
	BOOL playing = FALSE;

	for (i = 0; i < 24; i++)
	{
		if (CheckEffect(i))
	    	playing = TRUE;
    }
//	if (phrase[0] != 0)		// phrase playing
	if (PhraseFlag)			// phrase playing
		playing = TRUE;

    return playing;
}  /* End IsSoundPlaying() */


#if PC_DEBUG
/*-------------------------------------------------------------
  PrintSoundInfo ~
-------------------------------------------------------------*/
void PrintSoundInfo(void)
{
	int i;

	for (i = 0; i < NUM_VAB; i++)
	{
		DebugPrintf(0, "Vab %d: hdr=%X  effect_vab=%d Alloc %d\n",
				i, header[i], effect_vab[i], VabHeaderAlloced[i]);
	}
}  /* End PrintSoundInfo() */
#endif

/*-------------------------------------------------------------
  CloseAllVabs ~  CGD
-------------------------------------------------------------*/
void CloseAllVabs(void)
{
	int i;

	CloseMusVab();
	VSync(0);
	for (i = 0; i < NUM_VAB; i++)
	{
		CloseEffectVab(i);
	}
	VSync(0);
}  /* End CloseAllVabs() */

/*-------------------------------------------------------------
  EndGameSound ~  CGD
-------------------------------------------------------------*/
void EndGameSound(void)
{
	ULONG vb;

	#if PC_DEBUG
		PrintSoundInfo();
		Cmarker();
	#endif

	VSync(0);
	StopCrowd();
	vb = GetVBlankCount() + (2 * 60);
	while ((GetVBlankCount() < vb) && IsSoundPlaying()) ;
	VSync(0);
	CloseMusVab();
	 Cmarker();
	FreeAudio();
	 Cmarker();
	CloseAllVabs();
	 Cmarker();
	StopSound();
	 Cmarker();
	FreeEffectsHeadMfl();
	 Cmarker();
	 DumpHeap();
	SpuWrite0(512 * 1024);
	VSync(0);
//	InitEffectsHead();
	InitSound();
	 Cmarker();

}  /* End EndGameSound() */


/*************************************************************************/

SectorFile_t *InitSectorFile(char *fname)
{
	ULONG base;
	char sfname[80];

	sprintf(sfname,"/SOUND/%s.SFL",fname);
	base = GetFileSector(sfname);
	return(LoadSectorFileDirectory(NULL,base,NULL));
}

/*************************************************************************/

INT LoadASectorMember(WORD filenum, WORD membernum, WORD vbnum, char *vhbuff)
{
	ClearLoadList();
	AddLoadList(GetSectorFileByNum(SFL[filenum], (membernum*2)+1), vhbuff);
	StartLoadList();
	header[vbnum] = vhbuff;
	VabHeaderAlloced[vbnum] = NULL;
	while(!CheckLoadList());
	ClearLoadList();
	AddLoadList(GetSectorFileByNum(SFL[filenum], membernum*2), PhraseVBBuffer);
	StartLoadList();
	while(!CheckLoadList());
	TransferVb(PhraseVBBuffer, vbnum);

	return(0);
}

/************************** End of File: SOUND.C  ************************/
