/*************************************************************************

					File: SOUND.H

	 Contains all structures and externs for sound effects
	 and music for the PlayStation Football Game.


	Started: 	10-13-95 (Friday the 13th)
	Written by: Mike Heilemann



 *************************************************************************/


#ifdef PSXMODE
	#define	TRUE_SOUND	// normal PSX audio
//	#undef	TRUE_SOUND	// for PSX sim test only
#else
	#undef	TRUE_SOUND	// no audio on PC
#endif

#define	NUM_TEAMS	30

// Max voices is 24
//#define NUM_VAB 11
#define NUM_VAB 15
#define NUM_EFFECT_VOICES 200		//512
//#define NUM_MUSIC_VOICES ((24 * 16) - NUM_EFFECT_VOICES) / 16
#define NUM_MUSIC_VOICES 8
#define INFINITE_MUSIC -1
#define SOUND_LEVEL_MAX 5

#define FINE 70	// All Samples have a fine controll adjustment of 70


#define MAX_VOLUME			127
#define MAX_CROWD_VOLUME	127
#define	HALF_VOLUME			(MAX_VOLUME/2)
#define	QTR_VOLUME			(MAX_VOLUME/4)
#define	EIGHTH_VOLUME		(MAX_VOLUME/8)

// Defines for coin toss
#define HEADS_TAILS 	0
#define CHOOSE 		1
#define CHOOSE_SIDES	2

typedef struct {	// Contains stuff for vab, volume and pitchbending.
	short prog,		// Which VAB program to play from.
			tone,		// Tone to play (within program)
			note,		// Note to play, usually 60.
			vab,		// VAB ID returned by registered VAB
			voll,		// For Volume and Panning both
			volr;		// For Volume and Panning both
} effect_t;

extern short voice[24];	// for which voice is used
extern effect_t effect[NUM_EFFECT_VOICES];
extern short CurLoopSFXNum;
//extern char MusicFade;

void InitSound(void);
//void InitEffects(void);
void InitEffectsHead(void);
void InitEffectsBody(void);
void InitPhrase(short *PhrasePtr, short num);
void PlayPhrase(void);

short PlayEffect(short num);
short PlayEffectCheck(short num);
short PlayEffectPitchCheck(short num, short pitch);
void StopEffect(short num);
short CheckEffect(short voice);

void SetVoiceVolume(short voice, short voll, short volr);
void ChangeVoiceVolume(short voice, short incl, short incr);
void SetVoicePitch(short num, short bend);
void StopSound(void);
void CloseEffectVab(char num);
void CloseMusVab(void);

void OpenEffectVab(char *file, short num);
void OpenEffectVh(char *file, short num);
void OpenEffectVb(char *file, short num);

void OpenMusicVab(char *file);
void OpenSequence(char *file);
void PlaySequence(short times);
void PlaySong(char num);
void FadeMusic(char MusicFade, short ticks);
void StopSequence(void);
void SetMusicVolume(void);
void SetNextSequence(short seq1, short seq2);

void InitSFXLoop(short sound);

void InitCrowd(void);
void SetCrowdLevel(short level1, short level2, short level3, short level4, short percent);
void StopCrowd(void);

void InitAudio(void);
void FreeAudio(void);

void PlayPregame(void);
void FreePregame(void);
void PlayCoinToss(char num, char team);
void FreeCoinToss(void);

/**   AI   phrase prototypes	*/
void LoadAnnouncerPhrase(void);	// triggers InitPhrase to start BG load
void SayAnnouncerPhrase(void);	// sets PhraseFlag True


enum vabs {
	VAB0 = 100,
	VAB1,
	VAB2,
	VAB3,
	VAB4,
	VAB5,
	VAB6,
	VAB7,
	VAB8,
	VAB9,
	VAB10,
	VAB11,
	VAB12,
	VAB13,
	VAB14,
	VAB15
};


/*****************
 * VAB2 - GRUNTS *
 *****************/
enum vab2 {
	GRUNTS = 0,
	TACKLES,
	CATCHES,
	BLOCKS,
	DIVES,
	WHISTLES,
	KICKS
};

enum sfx {
// SHELL VAB
	SELECT_SFX,
	ERROR_SFX,
	CURSOR_SFX,

// VAB 0 - Crowd & Sidelines
// 13 sounds
  	LOWCROWDL,
  	LOWCROWDR,
 	MIDCROWDL,
 	MIDCROWDR,
 	HICROWDL,
 	HICROWDR,
 	CHANT,					// Defense Loop
 	YEAH,						// Crowd Swell (AHHHH excited)
 	SWELL0,					//
 	SWELL1,					// Crowd Swell (AHHHH too bad)
	BOO_LOOP_SFX,			// Boo Loop
	AHHH_SFX,				// AHHH (excited)
	OHHH_SFX,				// OHHH (too bad)

// VAB 1 - Sidelines
	BLOCK_IT_SFX,			// Block it
	BOO_SFX,					// Boo
	COME_ON_SFX,			// Come On
	DEFENSE_LOOP_SFX,		// Defense Loop (Best)
	GET_HIM1_SFX,			// Get Him
	CHAT1_SFX,				// Random crowd chatter (undecipherable)
	CHAT2_SFX,				// Random crowd chatter (undecipherable)
	CHAT3_SFX,				// Random crowd chatter (undecipherable)
	CHAT4_SFX,				// Random crowd chatter (Playit)
	CHAT5_SFX,				// Random crowd chatter (undecipherable)
	CHAT6_SFX,				// Random crowd chatter (All right)
	CHAT7_SFX,				// Random crowd chatter (undecipherable)
	CHAT8_SFX,				// Random crowd chatter (undecipherable)
	YOU_SUCK_SFX,			// You suck

// VAB 2
// 8
	GRUNT0_SFX,
	GRUNT1_SFX,
	GRUNT2_SFX,
	GRUNT3_SFX,
	GRUNT4_SFX,
	GRUNT5_SFX,
	GRUNT6_SFX,
	GRUNT7_SFX,

// 5
	TACKLE_SFX,
	TACKLE1,
	TACKLE2,
	TACKLE3,
	TACKLE4,

// 2
	CATCH,
	CATCH1,

// 1
	BLOCK,

// 3
	DIVE0,
	DIVE1,
	DIVE2,

// 1
	WHISTLE_SFX,
	WHISTLE1_SFX,
	WHISTLE2_SFX,
	WHISTLE3_SFX,

// 2
	KICK_SFX,
	PUNT_SFX,


// VAB3 - QB - 2 VAB's 0 & 1
	READY_SFX,				// Ready
	SET_SFX,					// Set
	HUT_SFX,					// Hut
	BREAK_SFX,				// -CLAP-

	NUM0_SFX,				// 0: 16		- 1:
	NUM1_SFX,				// 0: 42		- 1:
	NUM2_SFX,				// 0: 28 	- 1:
	COLOR0_SFX,				// 0: Blue	- 1:
	COLOR1_SFX, 			// 0: Red	- 1:

// VAB3 - DEFENSE
	FUMBLE_SFX,				// Fumble
	GET_HIM_SFX,			// Get Em
	LEFT_SIDE_SFX,			// They goin to the left side
	LOOK_LEFT_SFX,			// Look Left
	OVER_THERE_SFX,		// Over there
	RUN_SFX,					// Its a run
	REVERSE_SFX,			// Reverse
	THE_MIDDLE_SFX,		// Up the Middle
	THYPSSN_SFX,			// They Passin
	WATCH_EM_SFX,			// Watch em
	INTERCEPT_SFX,			// Interception

// VAB 4 - REF STUFF
	REF_FIRST_DOWN_SFX,		// First down
	ENCROACH_SFX,			// Encroachment
	FALSE_START_SFX,		// False start
	INTERFERENCE_SFX,		// Interference
	GOOD_SFX,				// Its good
	NOGOOD_SFX,				// No good
	OFFSIDES_SFX,			// Offsides
	SAFETY_SFX,				// Safety
	TOUCHDOWN_SFX,			// Touchdown

	ON_THE_SFX,				// On the
	DELAY_OF_GAME_SFX,		// Delay of game
	PERSONAL_SFX,			// Personal Foul
	DEFENSE_SFX,			// Defense
	OFFENSE_SFX,			// Offense
	TWO_MINUTE_WARNING_SFX,
	GROUNDING_SFX,
	SILENCE_SFX,


// VAB 5 - NUMBERS

	BALL_INTERCEPTED_SFX,	// Ball Intercepted

	ANNOUNCER_PHRASES = BALL_INTERCEPTED_SFX,

	BALL_FUMBLED_SFX,		// Ball Fumbled
	BALL_OUT_OF_BOUNDS_SFX,	// Ball out of bounds
	BALL_DROPPED_SFX,		// Ball dropped
	BROUGHT_DOWN_SFX,		// Brought Down by
	CAUGHT_SFX,				//
	DOWNED_SFX,				//
	DROPPED_SFX,			//
	FUMBLE_RECOVERED_SFX,	//
	HALFTIME_SCORE_SFX,		//
	INCOMPLETE_PASS_SFX,	//
	INCOMPLETE_SFX,			//
	INCOMPLETE1_SFX = INCOMPLETE_SFX,		// Incomplete (backward compatibility)
	INTERCEPTION_SFX,		// Interception
	PLAY_BLOCKED_SFX,		//
	PASS_CAUGHT_SFX,		//
	PASS_INTERCEPTED_SFX,	//
	PUNT_FORMATION,			// In punt formation
	TACKLED_BY_SFX,			// Tackled by ...

//	TIMEOUT_SFX,			// Time out
//	BACK_TO_RECEIVE,		// Back to receive
//	FIELD_GOAL,				// To attempt a field goal

	LADIES_AND_GENTS_SFX,	// Ladys and Gentlemen,
 	THAT_ENDS_1ST_SFX,		// That ends the 1st quarter
 	THAT_ENDS_2ND_SFX,		// That ends the 2nd quarter
 	THAT_ENDS_3RD_SFX,		// That ends the 3rd quarter
 	THAT_ENDS_4TH_SFX,		// That ends the 4th quarter
 	THAT_ENDS_GAME_SFX = THAT_ENDS_4TH_SFX,		// That ends the game (not available)
	FINAL_SCORE,			// Final Score

// COINTOSS
	CALL_IT_SFX,			//	Call it
	CHOOSE_SFX,				// Choose
	CHOOSE_SIDE_SFX,		//	Choose Sides
	HEADS_OR_SFX,			// Heads or Tails

// Blocks of SFX

	TEAM_CAPT_SFX,										// Team Captain
	CITY_NAMES1_SFX		= TEAM_CAPT_SFX+NUM_TEAMS, 		// City 1st
	CITY_NAMES2_SFX		= CITY_NAMES1_SFX,		   		// City 2nd
	NICKNAME_SFX		= CITY_NAMES1_SFX+NUM_TEAMS,	// Home nickname
	FIRST_DOWN_SFX		= NICKNAME_SFX+NUM_TEAMS,		//
	SECOND_DOWN_SFX		= FIRST_DOWN_SFX+9,				//
	THIRD_DOWN_SFX		= SECOND_DOWN_SFX+18,			//
	FOURTH_DOWN_SFX		= THIRD_DOWN_SFX+18,			//
	GAIN_SFX			= FOURTH_DOWN_SFX+18,	// 22 groups of 3 ie. RUN,PASS,PLAY
	SACKED_FOR_SFX		= GAIN_SFX+66,
	INTENDED_FOR_SFX	= SACKED_FOR_SFX+13,	// Is this right Nigel???
	LOSS_ON_SFX			= INTENDED_FOR_SFX+99,
	NUMBERS1_SFX		= LOSS_ON_SFX+13,		// Numbers for start of phrase (0-102)
	NUMBERS2_SFX		= NUMBERS1_SFX,			// Numbers for end of phrase (0-102)
	BY_NUMBER_SFX		= NUMBERS2_SFX+102,	    // Player numbers ie. NUMER 48 (1-99)
	WELCOME_TO_SFX		= BY_NUMBER_SFX+99,		// Welcome to the Home of your...

};


#define ARROW_MOVE_SOUND CURSOR_SFX	// BONK
#define PRESS_SOUND SELECT_SFX		// DRUM
#define WRONG_SOUND ERROR_SFX


/************************* End of File: SOUND.H  *********************/
