
#include <stdio.h>
#include <sys/types.h>

#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include "prjhdrs.h"

#include "fbgequ.h"
#include "fbequ.h"
#include "fbram.h"
#include "fbstats.h"
#include "shell.h"
#include "season.h"


//**************************************************************************
// Data structures & defines for menu system

#include "menu.h"
#include "fbmenu.h"

//**************************************************************************

#define	TMSTAND_SCROLL_SPEED 16

#define	TMSTAT_X	180
#define	TMSTAT_Y	80


#define LLNUM_ITEMS  10

#define OPP			32

extern BYTE playoff_game_team[];
extern LONG OppStats;
extern LONG JustPlayed;
extern WORD NumStrings;
extern WORD Scrolling, ScrollCount;
extern BYTE  HumanTeamNum;
extern BYTE WeekNum;
extern BYTE TempStr[];
extern BYTE LdrIndex;

static WORD ItemOffset = 0;
static WORD ColumnOffset = 0;
static BYTE Centering = 1;

static WORD NetYards;
static WORD TotalPlays;
static WORD GamesPlayed;
static LONG pct_table[30][2];

LONG BadgeNum =0;
LONG BadgeAnim[3], BadgeSpeed[3], BadgeDeccel[3];
UBYTE *GameDataBuf0, *GameDataBuf1, *GameDataBuf2;	// General purpose pointers (used by other modules also)
UBYTE TsNick[2];

struct teamRAMstats *dataptr[2];
void GetDataPtrs(void);

LONG round(LONG result);
void Print_Decimal(BYTE *text, LONG number);

INT InitLgLeaders(void);
INT TermLgLeaders(void);
INT MainLgLeaders(void);

INT InitTmStandings(void);
INT MainTmStandings(void);
INT TermTmStandings(void);
void PrintDivisionStandings(INT division);

INT InitPlayoffs(void);
INT TermPlayoffs(void);
INT MainPlayoffs(void);
void DisplayPlayoffsInfo(void);


INT InitLeaders(void);
INT TermLeaders(void);
INT MainLeaders(void);



WORD GetGamesPlayed(struct teamRAMstats *datptr);
WORD GetWinsLossTies(struct teamRAMstats *datptr);
WORD GetTotalPoints(struct teamRAMstats *datptr);
WORD GetScoreByQuarter(struct teamRAMstats *datptr);
WORD Get1stDowns(struct teamRAMstats *datptr);
WORD Get3rdDowns(struct teamRAMstats *datptr);
WORD Get4thDowns(struct teamRAMstats *datptr);
WORD GetNetYards(struct teamRAMstats *datptr);
WORD GetAvgYardsPerGame(struct teamRAMstats *datptr);
WORD GetTotalPlays(struct teamRAMstats *datptr);
WORD GetAvgYardsPerPlay(struct teamRAMstats *datptr);
WORD GetNetYardsPassing(struct teamRAMstats *datptr);
WORD GetAvgYardsPassing(struct teamRAMstats *datptr);
WORD GetAttComp(struct teamRAMstats *dataptr);
WORD GetAvgPerPass(struct teamRAMstats *dataptr);
WORD GetCompPct(struct teamRAMstats *dataptr);
WORD GetNetYardsRushing(struct teamRAMstats *dataptr);
WORD GetAvgYardsRushing(struct teamRAMstats *dataptr);
WORD GetTotalRushes(struct teamRAMstats *dataptr);
WORD GetAvgPerRushes(struct teamRAMstats *dataptr);
WORD GetPuntAvg(struct teamRAMstats *dataptr);
WORD GetPenalties(struct teamRAMstats *dataptr);
WORD GetDefensiveSacks(struct teamRAMstats *dataptr);
WORD GetTurnovers(struct teamRAMstats *dataptr);
WORD GetGiveaways(struct teamRAMstats *dataptr);
WORD GetAvgTimePoss(struct teamRAMstats *dataptr);
void SwapStat(INT i);
void SortByStat(INT column);
void DrawLgLeaders(void);
void PrintString(BYTE *TempStr,INT x,INT y,INT color);


void DisplayWinsLossTies(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayScoreByQuarter(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayTotalPoints(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayTotal1stDowns(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayTotal3rdDowns(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayTotal4thDowns(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayTotalNetYards(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayAvgYards(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayTotalPlays(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayAvgYardsPerPlay(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayNetYardsPassing(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayAvgYardsPassing(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayAttComp(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayAvgPerPass(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayCompPct(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayNetYardsRushing(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayAvgYardsRushing(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayTotalRushes(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayAvgPerRush(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayPuntAvg(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayPenaltiesYardsLost(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayDefensiveSacks(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayTurnovers(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayGiveaways(struct teamRAMstats *dataptr, INT x, INT y, INT color);
void DisplayAvgTimePoss(struct teamRAMstats *dataptr, INT x, INT y, INT color);


// Array of pointers to functions that pass a teamRAMstats ptr & return a WORD

WORD (*(LLStatsFunctions[]))(struct teamRAMstats *dataptr) = {
		GetWinsLossTies,
		GetTotalPoints,
		GetScoreByQuarter,
		Get1stDowns,
		Get3rdDowns,
		Get4thDowns,
		GetNetYards,
		GetAvgYardsPerGame,
		GetTotalPlays,
		GetAvgYardsPerPlay,
		GetNetYardsPassing,
		GetAvgYardsPassing,
		GetAttComp,
		GetCompPct,
		GetAvgPerPass,
		GetNetYardsRushing,
		GetAvgYardsRushing,
		GetTotalRushes,
		GetAvgPerRushes,
		GetPuntAvg,
		GetPenalties,
		GetDefensiveSacks,
		GetTurnovers,
		GetGiveaways,
		GetAvgTimePoss,
		NULL,
};


void (*(LLDisplayFunctions[]))(struct teamRAMstats *dataptr, INT x, INT y, INT color) ={
		DisplayWinsLossTies,
		DisplayTotalPoints,
		DisplayScoreByQuarter,
		DisplayTotal1stDowns,
		DisplayTotal3rdDowns,
		DisplayTotal4thDowns,
		DisplayTotalNetYards,
		DisplayAvgYards,
		DisplayTotalPlays,
		DisplayAvgYardsPerPlay,
		DisplayNetYardsPassing,
		DisplayAvgYardsPassing,
		DisplayAttComp,
		DisplayCompPct,
		DisplayAvgPerPass,
		DisplayNetYardsRushing,
		DisplayAvgYardsRushing,
		DisplayTotalRushes,
		DisplayAvgPerRush,
		DisplayPuntAvg,
		DisplayPenaltiesYardsLost,
		DisplayDefensiveSacks,
		DisplayTurnovers,
		DisplayGiveaways,
		DisplayAvgTimePoss,
		NULL,
};




BYTE StatType[] = {0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,1,0,0,0,0};

BYTE *LLStatsStr1[]={
					" ",
					"Points",
                    "Score",
                    "1st",
                    "3rd Downs",
                    "4th Downs",
                    "Total",
                    "Avg Per",
                    "Total",
                    "Avg Per",
                    "Net Yards",
                    "Avg Per",
                    "Att/",
                    "Completion",
                    "Avg Per",
                    "Net Yards",
                    "Avg Per",
                    "Total",
                    "Avg Per",
                    " ",
                    "Penalties",
                    "Defensive",
                    " ",
           			"Takeaways-",
                    "Avg Time",
};

BYTE *LLStatsStr2[]={
    				"Win/Loss/Tie",
         			"Total",
         			"By Qtr",
      				"Downs",
           			"Made/Att",
           			"Made/Att",
       				"Net Yards",
         			"Game",
       				"Plays",
         			"Play",
           			"Passing",
         			"Game",
      				"Completions",
            		"Pct",
         			"Pass",
           			"Rushing",
         			"Game",
       				"Rushes",
         			"Rush",
   					"Punts/Avg",
           			"Yards Lost",
           			"Sacks",
    				"Turnovers",
                    "Giveaways",
          			"Possession",
};

WORD LLXpos[3] ={150,270,380};

WORD LLDivision;
WORD NumTeams;
WORD DivBase;

INT InitTmStats(void);
INT MainTmStats(void);
INT TermTmStats(void);

LONG PrintGamesPlayed(MenuItem *thisitem, WORD itemnum);
LONG PrintWinsLossTies(MenuItem *thisitem, WORD itemnum);
LONG PrintScoreByQuarter(MenuItem *thisitem, WORD itemnum);
LONG PrintScore(MenuItem *thisitem, WORD itemnum);
LONG PrintTotalPoints(MenuItem *thisitem, WORD itemnum);
LONG PrintTotal1stDowns(MenuItem *thisitem, WORD itemnum);
LONG PrintTotal2ndDowns(MenuItem *thisitem, WORD itemnum);
LONG PrintTotal3rdDowns(MenuItem *thisitem, WORD itemnum);
LONG PrintTotal4thDowns(MenuItem *thisitem, WORD itemnum);
LONG PrintTotalNetYards(MenuItem *thisitem, WORD itemnum);
LONG PrintAvgYards(MenuItem *thisitem, WORD itemnum);
LONG PrintTotalPlays(MenuItem *thisitem, WORD itemnum);
LONG PrintAvgYardsPerPlay(MenuItem *thisitem, WORD itemnum);
LONG PrintNetYardsPassing(MenuItem *thisitem, WORD itemnum);
LONG PrintAvgNetYardsPassing(MenuItem *thisitem, WORD itemnum);
LONG PrintAttComp(MenuItem *thisitem, WORD itemnum);
LONG PrintAvgPerPass(MenuItem *thisitem, WORD itemnum);
LONG PrintCompPct(MenuItem *thisitem, WORD itemnum);
LONG PrintNetYardsRushing(MenuItem *thisitem, WORD itemnum);
LONG PrintAvgYardsRushing(MenuItem *thisitem, WORD itemnum);
LONG PrintTotalRushes(MenuItem *thisitem, WORD itemnum);
LONG PrintAvgPerRush(MenuItem *thisitem, WORD itemnum);
LONG PrintPuntAvg(MenuItem *thisitem, WORD itemnum);
LONG PrintPenaltiesYardsLost(MenuItem *thisitem, WORD itemnum);
LONG PrintDefensiveSacks(MenuItem *thisitem, WORD itemnum);
LONG PrintTurnovers(MenuItem *thisitem, WORD itemnum);
LONG PrintGiveawaysTakeaways(MenuItem *thisitem, WORD itemnum);
LONG PrintAvgTimePoss(MenuItem *thisitem, WORD itemnum);


LONG WAFCResults(MenuItem *thisitem, WORD itemnum);
LONG CAFCResults(MenuItem *thisitem, WORD itemnum);
LONG EAFCResults(MenuItem *thisitem, WORD itemnum);
LONG WNFCResults(MenuItem *thisitem, WORD itemnum);
LONG CNFCResults(MenuItem *thisitem, WORD itemnum);
LONG ENFCResults(MenuItem *thisitem, WORD itemnum);


MenuItem TmStatsItemList[] = {
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+0 , "Games Played",PrintGamesPlayed,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+15, "Wins/Loss/Tie",PrintWinsLossTies,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+30, "Total Points",PrintTotalPoints,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+45, "Score By Qtr",PrintScoreByQuarter,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+60, "1st Downs",PrintTotal1stDowns,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+75, "3rd Downs Made/Att",PrintTotal3rdDowns,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+90, "4th Downs Made/Att",PrintTotal4thDowns,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+105, "Total Net Yards",PrintTotalNetYards,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+120, "Avg Per Game",PrintAvgYards,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+135, "Total Plays",PrintTotalPlays,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+150, "Avg Per Play",PrintAvgYardsPerPlay,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+165, "Net Yards Passing",PrintNetYardsPassing,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+180, "Avg Per Game",PrintAvgNetYardsPassing,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+195, "Att/Completions",PrintAttComp,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+210, "Completions Pct",PrintCompPct,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+225, "Avg Per Pass",PrintAvgPerPass,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+240, "Net Yards Rushing",PrintNetYardsRushing,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+255, "Avg Per Game",PrintAvgYardsRushing,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+270, "Total Rushes",PrintTotalRushes,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+285, "Avg Per Rush",PrintAvgPerRush,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+300, "Punts/Avg",PrintPuntAvg,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+315, "Penalties/Yards Lost",PrintPenaltiesYardsLost,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+330, "Defensive Sacks",PrintDefensiveSacks,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+345, "Turnovers",PrintTurnovers,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+360, "Takeaways-Giveaways",PrintGiveawaysTakeaways,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+375, "Avg Time Of Poss",PrintAvgTimePoss,
	END_MENU,
};

MenuItem TmStatsGmItemList[] = {
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+0, "Score",PrintTotalPoints,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+15, "Score By Qtr",PrintScoreByQuarter,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+30, "1st Downs",PrintTotal1stDowns,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+45, "3rd Downs Made/Att",PrintTotal3rdDowns,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+60, "4th Downs Made/Att",PrintTotal4thDowns,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+75, "Total Net Yards",PrintTotalNetYards,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+90, "Total Plays",PrintTotalPlays,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+105, "Avg Per Play",PrintAvgYardsPerPlay,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+120, "Net Yards Passing",PrintNetYardsPassing,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+135, "Att/Completions",PrintAttComp,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+150, "Completions Pct",PrintCompPct,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+165, "Avg Per Pass",PrintAvgPerPass,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+180, "Net Yards Rushing",PrintNetYardsRushing,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+195, "Total Rushes",PrintTotalRushes,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X+20, TMSTAT_Y+210, "Avg Per Rush",PrintAvgPerRush,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+225, "Punts/Avg",PrintPuntAvg,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+240, "Penalties/Yards Lost",PrintPenaltiesYardsLost,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+255, "Defensive Sacks",PrintDefensiveSacks,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+270, "Turnovers",PrintTurnovers,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+285, "Takeaways-Giveaways",PrintGiveawaysTakeaways,
	ACTIVE | HI_YELLOW | PEN_WHITE, TMSTAT_X, TMSTAT_Y+300, "Time Of Poss",PrintAvgTimePoss,
	END_MENU,
};



MenuObj	TmStatsMenu = {
	(MenuItem *)TmStatsItemList,
	InitTmStats,
	TermTmStats,
	MainTmStats,
	0, TMSTAT_Y-1, 512, 125,
};

WORD xpos[2] ={100,412};

BYTE *StatsMsg[2];

//**************************************************************************

LONG PrintScore(MenuItem *thisitem, WORD itemnum)
{
	return(0);
}

//**************************************************************************

LONG PrintGamesPlayed(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{

		GamesPlayed = GetGamesPlayed(dataptr[i]);

//		GamesPlayed = dataptr[i]->allplyrstats[GS_WIN]+
//					  	 dataptr[i]->allplyrstats[GS_LOSS]+
//					  	 dataptr[i]->allplyrstats[GS_TIE];


		sprintf(TempStr,"%d", GamesPlayed);

		PrintStringXYCenter(TempStr,xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

//**************************************************************************

LONG PrintWinsLossTies(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayWinsLossTies(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}
	return(0);
}


void DisplayWinsLossTies(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		sprintf(TempStr,"%d/%d/%d",dataptr->allplyrstats[GS_WIN],
							 dataptr->allplyrstats[GS_LOSS],
							 dataptr->allplyrstats[GS_TIE]);

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintScoreByQuarter(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayScoreByQuarter(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayScoreByQuarter(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	if(TheQuarter < 5)
	{
		sprintf(TempStr,"%d/%d/%d/%d", dataptr->allplyrstats[GS_SCOREQ1],
										  dataptr->allplyrstats[GS_SCOREQ2],
										  dataptr->allplyrstats[GS_SCOREQ3],
										  dataptr->allplyrstats[GS_SCOREQ4]);
	}
	else
	{
		sprintf(TempStr,"%d/%d/%d/%d/%d", dataptr->allplyrstats[GS_SCOREQ1],
										  dataptr->allplyrstats[GS_SCOREQ2],
										  dataptr->allplyrstats[GS_SCOREQ3],
										  dataptr->allplyrstats[GS_SCOREQ4],
										  dataptr->allplyrstats[GS_SCOREQ5]);
	}

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintTotalPoints(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayTotalPoints(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}
	return(0);
}

void DisplayTotalPoints(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		sprintf(TempStr,"%d",dataptr->allplyrstats[GS_TOTAL_SCORE]);

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintTotal1stDowns(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayTotal1stDowns(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}
	return(0);
}


void DisplayTotal1stDowns(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		sprintf(TempStr,"%d",dataptr->allplyrstats[GS_1STDOWNS]);

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintTotal3rdDowns(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayTotal3rdDowns(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}
	return(0);
}


void DisplayTotal3rdDowns(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		sprintf(TempStr,"%d/%d",dataptr->allplyrstats[GS_3RDDOWNS_MADE],
							 dataptr->allplyrstats[GS_3RDDOWNS_ATT]);

		PrintString(TempStr,x,y,color);
}


//**************************************************************************

LONG PrintTotal4thDowns(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayTotal4thDowns(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayTotal4thDowns(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		sprintf(TempStr,"%d/%d",dataptr->allplyrstats[GS_4THDOWNS_MADE],
							 dataptr->allplyrstats[GS_4THDOWNS_ATT]);

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintTotalNetYards(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayTotalNetYards(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayTotalNetYards(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		NetYards =	GetNetYards(dataptr);

//					dataptr[i]->allplyrstats[GS_PASS_YARDS] +
//					dataptr[i]->allplyrstats[GS_PASS_YARDS] +
//					dataptr[i]->allplyrstats[GS_SACK_YARDS];

		sprintf(TempStr,"%d", NetYards);

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintAvgYards(MenuItem *thisitem, WORD itemnum)
{
	INT i;


	for (i=0;i<2;i++)
	{
		DisplayAvgYards(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}


void DisplayAvgYards(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		NetYards = GetNetYards(dataptr);

//					dataptr[i]->allplyrstats[GS_PASS_YARDS] +
//					dataptr[i]->allplyrstats[GS_PASS_YARDS] +
//					dataptr[i]->allplyrstats[GS_SACK_YARDS];

		GamesPlayed = GetGamesPlayed(dataptr);

		if(GamesPlayed)
//			sprintf(TempStr,"%d", (NetYards*100)/GamesPlayed);
			PrintDecimal(TempStr, (NetYards*10)/GamesPlayed);
		else
			sprintf(TempStr,"%d", 0);

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintTotalPlays(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayTotalPlays(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayTotalPlays(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		TotalPlays = dataptr->allplyrstats[GS_NUM_RUN_PLAYS]
							+ dataptr->allplyrstats[GS_NUM_PASS_PLAYS];

		sprintf(TempStr,"%d", TotalPlays);

		PrintString(TempStr,x,y,color);
}


//**************************************************************************

LONG PrintAvgYardsPerPlay(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayAvgYardsPerPlay(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayAvgYardsPerPlay(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		INT avyd;

		NetYards = GetNetYards(dataptr);
		TotalPlays = GetTotalPlays(dataptr);

		if(TotalPlays)
		{
			PrintDecimal(TempStr,(NetYards*10)/TotalPlays);
//			avyd = round(avyd);
		}
		else
			sprintf(TempStr,"%d",0);

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintNetYardsPassing(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayNetYardsPassing(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayNetYardsPassing(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		sprintf(TempStr,"%d", dataptr->allplyrstats[GS_PASS_YARDS]
							+ dataptr->allplyrstats[GS_SACK_YARDS]);
		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintAvgNetYardsPassing(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayAvgYardsPassing(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayAvgYardsPassing(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{

		LONG stat;

		if(GamesPlayed = GetGamesPlayed(dataptr))
		{
			stat = ((dataptr->allplyrstats[GS_PASS_YARDS]
								+ dataptr->allplyrstats[GS_SACK_YARDS])*10)
								/ GamesPlayed;

//			sprintf(TempStr,"%d",round(stat));
			PrintDecimal(TempStr,stat);
		}
		else sprintf(TempStr,"%d",0);

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintAttComp(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayAttComp(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayAttComp(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		sprintf(TempStr,"%d / %d",	dataptr->allplyrstats[GS_PASS_ATT],
									dataptr->allplyrstats[GS_PASS_COMP]);
		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintAvgPerPass(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayAvgPerPass(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayAvgPerPass(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
		LONG stat;

		if(dataptr->allplyrstats[GS_PASS_COMP])
		{
			stat = ((dataptr->allplyrstats[GS_PASS_YARDS] +
								dataptr->allplyrstats[GS_SACK_YARDS])*10)/
								dataptr->allplyrstats[GS_PASS_COMP];



			PrintDecimal(TempStr, stat);
//			sprintf(TempStr,"%d", round(stat));
		}
		else sprintf(TempStr,"%d",0);

		PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintCompPct(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayCompPct(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayCompPct(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	INT j;

	if(dataptr->allplyrstats[GS_PASS_COMP])
		j = (dataptr->allplyrstats[GS_PASS_ATT]<<8)/dataptr->allplyrstats[GS_PASS_COMP];
	else
		j = 0;

	if(j) sprintf(TempStr,"%d%%", (100<<8)/j);
	else sprintf(TempStr,"%d%%", 0);

	PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintNetYardsRushing(MenuItem *thisitem, WORD itemnum)
{
	INT i;


	for (i=0;i<2;i++)
	{
		DisplayNetYardsRushing(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayNetYardsRushing(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	sprintf(TempStr,"%d", dataptr->allplyrstats[GS_RUSH_YARDS]);

	PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintAvgYardsRushing(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayAvgYardsRushing(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayAvgYardsRushing(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	GamesPlayed = GetGamesPlayed(dataptr);

	if(GamesPlayed)
		PrintDecimal(TempStr, dataptr->allplyrstats[GS_RUSH_YARDS]*10/(GamesPlayed));
	else
		sprintf(TempStr, "%d", 0);
//	sprintf(TempStr,"%d", dataptr->allplyrstats[GS_RUSH_YARDS]*10/(GamesPlayed));

	PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintTotalRushes(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayTotalRushes(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayTotalRushes(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	sprintf(TempStr,"%d", dataptr->allplyrstats[GS_RUSH_ATT]);

	PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintAvgPerRush(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayAvgPerRush(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}


void DisplayAvgPerRush(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
  	if(dataptr->allplyrstats[GS_RUSH_ATT])
	{
		PrintDecimal(TempStr, dataptr->allplyrstats[GS_RUSH_YARDS]*10/
							  dataptr->allplyrstats[GS_RUSH_ATT]);
	}
	else
	{
		sprintf(TempStr,"%d",0);
	}

	PrintString(TempStr,x,y,color);
}


//**************************************************************************

LONG PrintPuntAvg(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayPuntAvg(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}


void DisplayPuntAvg(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	INT avg;

	if(dataptr->allplyrstats[GS_PUNT])
		avg = (dataptr->allplyrstats[GS_PUNT_YARDS]<<8)/dataptr->allplyrstats[GS_PUNT];
	else
		avg =0;

	sprintf(TempStr,"%d/%d", dataptr->allplyrstats[GS_PUNT],round(avg));

	PrintString(TempStr,x,y,color);
}

//**************************************************************************


LONG PrintPenaltiesYardsLost(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayPenaltiesYardsLost(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}



void DisplayPenaltiesYardsLost(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	sprintf(TempStr,"%d / %d", dataptr->allplyrstats[GS_PENALTY_NUM],
							  dataptr->allplyrstats[GS_PENALTY_YARDS]);

	PrintString(TempStr,x,y,color);
}

//**************************************************************************

LONG PrintDefensiveSacks(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayDefensiveSacks(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}


void DisplayDefensiveSacks(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	sprintf(TempStr,"%d", dataptr->allplyrstats[GS_SACK_NUM]);

	PrintString(TempStr,x,y,color);
}


//**************************************************************************

LONG PrintTurnovers(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayTurnovers(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}


void DisplayTurnovers(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	sprintf(TempStr,"%d", dataptr->allplyrstats[GS_PASS_INT]+dataptr->allplyrstats[GS_FUM_LOST]);

	PrintString(TempStr,x,y,color);
}


//**************************************************************************

LONG PrintGiveawaysTakeaways(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayGiveaways(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}

void DisplayGiveaways(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	sprintf(TempStr,"%d", (dataptr->alloppplyrstats[GS_PASS_INT] + dataptr->alloppplyrstats[GS_FUM_LOST])
						  - (dataptr->allplyrstats[GS_PASS_INT]+dataptr->allplyrstats[GS_FUM_LOST]));

	PrintString(TempStr,x,y,color);
}


//**************************************************************************

LONG PrintAvgTimePoss(MenuItem *thisitem, WORD itemnum)
{
	INT i;

	for (i=0;i<2;i++)
	{
		DisplayAvgTimePoss(dataptr[i],xpos[i],thisitem->item_y+MenuTxt_Yoffset,PEN_YELLOW);
	}

	return(0);
}


void DisplayAvgTimePoss(struct teamRAMstats *dataptr, INT x, INT y, INT color)
{
	UWORD secs,mins;

	secs = (UWORD) dataptr->allplyrstats[GS_TOP];

	mins = secs/60;
	secs %= 60;

	sprintf(TempStr,"%d:%02d", mins,secs);

	PrintString(TempStr,x,y,color);
}

//**************************************************************************

void PrintString(BYTE *TempStr,INT x,INT y,INT color)
{
	if(Centering)
		PrintStringXYCenter(TempStr,x,y,color);
	else
		PrintStringXY(TempStr,x,y,color);
}

//**************************************************************************

void GetDataPtrs(void)
{

	switch(MyGameType)
	{
		case EXHIBITION_GAME:

				if(!GameMode)
				{
					StatsMsg[0] = StatsMsg[1]="1996";

					if(OppStats)
					{
						dataptr[0] = (struct teamRAMstats *)&GameTeamData[WhichTeam].allplyrstats;
						TsNick[0]=GameTeamData[WhichTeam].TeamNumber;
						dataptr[1] = (struct teamRAMstats *)&GameTeamData[WhichTeam^1].allplyrstats;
						TsNick[1]=OPP;
					}
					else
					{
						dataptr[0] = (struct teamRAMstats *)&GameTeamData[0].allplyrstats;
						TsNick[0]=GameTeamData[0].TeamNumber;
						dataptr[1] = (struct teamRAMstats *)&GameTeamData[1].allplyrstats;
						TsNick[1]=GameTeamData[1].TeamNumber;
					}
				}
				else
				{
					StatsMsg[0] = StatsMsg[1]="game";

					dataptr[0] = &StatsData[0];
					TsNick[0]=GameTeamData[0].TeamNumber;
					dataptr[1] = &StatsData[1];
					TsNick[1]=GameTeamData[1].TeamNumber;
				}
				break;

		case SEASON_GAME:
				if(!GameMode)
				{
					if(WeekNum==1 || !SeasonMode || GameOptions[GAME_OPTNS_SEASON_LEN]==3) // No season stats yet so show ROM stats (includes PLAYOFF season)
					{
						StatsMsg[0] = StatsMsg[1] = "1996";

						if(OppStats)
						{
							dataptr[0] = (struct teamRAMstats *)&GameTeamData[WhichTeam].allplyrstats;
							TsNick[0]=GameTeamData[WhichTeam].TeamNumber;
							dataptr[1] = (struct teamRAMstats *)&GameTeamData[WhichTeam^1].allplyrstats;
							TsNick[1]=OPP;
						}
						else
						{
							dataptr[0] = (struct teamRAMstats *)&GameTeamData[0].allplyrstats;
							TsNick[0]=GameTeamData[0].TeamNumber;
							dataptr[1] = (struct teamRAMstats *)&GameTeamData[1].allplyrstats;
							TsNick[1]=GameTeamData[1].TeamNumber;
						}
					}
					else
					{
						StatsMsg[0] = StatsMsg[1]="Season";

						if(OppStats)
						{
							dataptr[0] = SeasonStatsPtr+GameTeamData[WhichTeam].TeamNumber;
							TsNick[0]=GameTeamData[WhichTeam].TeamNumber;
							dataptr[1] = SeasonStatsPtr+GameTeamData[WhichTeam^1].TeamNumber;
							TsNick[1]=OPP;
						}
						else
						{
							dataptr[0] = SeasonStatsPtr+GameTeamData[0].TeamNumber;
							TsNick[0]=GameTeamData[0].TeamNumber;
							dataptr[1] = SeasonStatsPtr+GameTeamData[1].TeamNumber;
							TsNick[1]=GameTeamData[1].TeamNumber;
						}
					}
				}
				else
				{
					StatsMsg[0] = StatsMsg[1]="Game";

					dataptr[0] = &StatsData[0];
					TsNick[0]=GameTeamData[0].TeamNumber;
					dataptr[1] = &StatsData[1];
					TsNick[1]=GameTeamData[1].TeamNumber;
				}
				break;

		case PRACTICE_GAME:
				if(!GameMode)
				{
					StatsMsg[0] = StatsMsg[1] = "1996";

					dataptr[0] = (struct teamRAMstats *)&GameTeamData[0].allplyrstats;
					TsNick[0]=GameTeamData[0].TeamNumber;
					dataptr[1] = (struct teamRAMstats *)&GameTeamData[1].allplyrstats;
					TsNick[1]=OPP;
				}
				else
				{
					StatsMsg[0] = "game";
					StatsMsg[1] = "1996";

					dataptr[0] = &StatsData[0];
					dataptr[1] = (struct teamRAMstats *)&GameTeamData[0].allplyrstats;
					TsNick[0]=TsNick[1]=GameTeamData[0].TeamNumber;
				}
				break;
	}
}

//**************************************************************************

INT InitTmStats(void)
{
	BYTE *ptr;
	BYTE fname[64];
	WORD i;


  /*Set graphics mode*/

	initVRamHandling();

  /*initialize the background for the shell*/

	LoadTeamPic(GameTeamData[WhichTeam].TeamNumber);

  /*Init the multifile pointers*/

	multifile = Shell_MFile = (unsigned long *)LoadCDFile("/shells/stats/tmstats.mfl");
 	DebugPrintf(0,"tmstats.mfl loaded into %x\n", Shell_MFile);

	SetupFont("fontdpc7.tim");
	CurFont = fontdpc7;

  /*load the shell graphics template files*/

	InitTemplates();

	LoadTemplate("tmstats.llx");
	LoadTemplate("tmstats2.llx");

//	LoadNickname(GameTeamData[WhichTeam].TeamNumber, 0, 0);


	/* Init general purpose counters */

	MenuCursorX = TMSTAT_X;
	MenuCursorY	= TMSTAT_Y;

	if(!GameMode)				// if pregame show full menu list
	{
		TmStatsMenu.menu_items = TmStatsItemList;
		ScrollYLimit = -(17*15);
	}
	else
	{
		TmStatsMenu.menu_items = TmStatsGmItemList;
		ScrollYLimit = -(12*15);
	}

	JpadRepeatFreq = 10;				// Set joy repeat rate

	if(MyGameType == SEASON_GAME)
	{
		SeasonStatsPtr	= (TEAMRAMStats *)AllocMem(NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats));
		AllTeamMemoryCardMimicToSeasonStats();	// returns 0 if okay
	}

	GetDataPtrs();

	for(i=0; i<2; i++)
		LoadNickname(TsNick[i], 0, i*2);


	if(OppStats)
	{
/*
		if(GameDataBuf0	= (BYTE *)AllocMem(sizeof(struct gameteamdata)))
		{
			memcpy(GameDataBuf0,&GameTeamData[WhichTeam^1],sizeof(struct gameteamdata));
			memcpy(&GameTeamData[WhichTeam^1],&GameTeamData[WhichTeam],sizeof(struct gameteamdata));
		}

		if(GameDataBuf1	= (BYTE *)AllocMem(MAX_NUM_STATS_1TEAM*sizeof(WORD)))
		{
			memcpy(GameDataBuf1,&dataptr[WhichTeam^1]->allplyrstats,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		}

		if(GameDataBuf2	= (BYTE *)AllocMem(MAX_NUM_STATS_1TEAM*sizeof(WORD)))
		{
			memcpy(GameDataBuf2,&dataptr[WhichTeam^1]->alloppplyrstats,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		}

		memcpy(&dataptr[WhichTeam^1]->allplyrstats,&dataptr[WhichTeam]->alloppplyrstats,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		memcpy(&dataptr[WhichTeam^1]->alloppplyrstats,&dataptr[WhichTeam]->allplyrstats,MAX_NUM_STATS_1TEAM*sizeof(WORD));
*/
//		if(GameDataBuf0	= (BYTE *)AllocMem(sizeof(struct gameteamdata)))
//		{
//			memcpy(GameDataBuf0,&GameTeamData[WhichTeam^1],sizeof(struct gameteamdata));
//			memcpy(&GameTeamData[WhichTeam^1],&GameTeamData[WhichTeam],sizeof(struct gameteamdata));
//		}

		if(GameDataBuf1	= (BYTE *)AllocMem(MAX_NUM_STATS_1TEAM*sizeof(WORD)))
		{
			memcpy(GameDataBuf1,&dataptr[1]->allplyrstats,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		}

		if(GameDataBuf2	= (BYTE *)AllocMem(MAX_NUM_STATS_1TEAM*sizeof(WORD)))
		{
			memcpy(GameDataBuf2,&dataptr[1]->alloppplyrstats,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		}

		memcpy(&dataptr[1]->allplyrstats,&dataptr[0]->alloppplyrstats,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		memcpy(&dataptr[1]->alloppplyrstats,&dataptr[0]->allplyrstats,MAX_NUM_STATS_1TEAM*sizeof(WORD));

	}

	return(0);
}

//**************************************************************************

INT MainTmStats(void)
{
	INT retcode, oldy, i;

	retcode = InputHandler(TMSTAT_Y, TMSTAT_Y+(0*15));

	oldy = ClipString.y;
	ClipString.y = 0;

	for(i=0; i<2; i++)
	{
		PrintStringXYCenter(StatsMsg[i],xpos[i],68,PEN_BLUE);
	}

	ClipString.y = oldy;

	if(MenuTxt_Yoffset)
	{
		PrintStringXYAbs(UP_ARROW,250,70,PEN_RED);
	}

	if(!GameMode)
	{
		if(MenuTxt_Yoffset > -(17*15)) PrintStringXYAbs(DOWN_ARROW,252,212,PEN_RED);
	}
	else
	{
		if(MenuTxt_Yoffset > -(12*15)) PrintStringXYAbs(DOWN_ARROW,252,212,PEN_RED);
	}


	if(retcode == TRIANGLE) retcode = -1;

	return(retcode);
}

//**************************************************************************

INT TermTmStats(void)
{
/*
	if(GameDataBuf0)
	{
		memcpy(&GameTeamData[WhichTeam^1],GameDataBuf0,sizeof(struct gameteamdata));
		FreeMem(GameDataBuf0);
		GameDataBuf0=NULL;
	}

	if(GameDataBuf1)
	{
		memcpy(&dataptr[WhichTeam^1]->allplyrstats,GameDataBuf1,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		FreeMem(GameDataBuf1);
		GameDataBuf1=NULL;
	}

	if(GameDataBuf2)
	{
		memcpy(&dataptr[WhichTeam^1]->alloppplyrstats,GameDataBuf2,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		FreeMem(GameDataBuf2);
		GameDataBuf2=NULL;
	}
*/
//	if(GameDataBuf0)
//	{
//		memcpy(&GameTeamData[WhichTeam^1],GameDataBuf0,sizeof(struct gameteamdata));
//		FreeMem(GameDataBuf0);
//		GameDataBuf0=NULL;
//	}

	if(GameDataBuf1)
	{
		memcpy(&dataptr[1]->allplyrstats,GameDataBuf1,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		FreeMem(GameDataBuf1);
		GameDataBuf1=NULL;
	}

	if(GameDataBuf2)
	{
		memcpy(&dataptr[1]->alloppplyrstats,GameDataBuf2,MAX_NUM_STATS_1TEAM*sizeof(WORD));
		FreeMem(GameDataBuf2);
		GameDataBuf2=NULL;
	}

	if(MyGameType == SEASON_GAME)
	{
		FreeMem(SeasonStatsPtr);
		SeasonStatsPtr = NULL;
	}
//	FreeTemplate(1);
	Std_Menu_CleanUp();
	return(0);
}

//**************************************************************************

INT DisplayTmStats(void)
{
	return(Display_Menu(TMSTATS_MENU));
}

//**************************************************************************

#define TMSTND_X	20
#define TMSTND_Y	60

MenuItem TmStandingsItemList[] = {
	ACTIVE | PEN_YELLOW, TMSTND_X, TMSTND_Y+0, "AFC WEST", WAFCResults,
	ACTIVE | PEN_YELLOW, TMSTND_X, TMSTND_Y+(6*15), "AFC CENTRAL", CAFCResults,
	ACTIVE | PEN_YELLOW, TMSTND_X, TMSTND_Y+0+(12*15), "AFC EAST", EAFCResults,
	ACTIVE | PEN_YELLOW, TMSTND_X+512, TMSTND_Y+0, "NFC WEST", WNFCResults,
	ACTIVE | PEN_YELLOW, TMSTND_X+512, TMSTND_Y+(6*15), "NFC CENTRAL", CNFCResults,
	ACTIVE | PEN_YELLOW, TMSTND_X+512, TMSTND_Y+(12*15), "NFC EAST", ENFCResults,
	END_MENU,
};


MenuObj	TmStandingsMenu = {
	(MenuItem *)TmStandingsItemList,
	InitTmStandings,
	TermTmStandings,
	MainTmStandings,
	-512, TMSTND_Y-1, 3*512, 150,
};


//**************************************************************************

LONG WAFCResults(MenuItem *thisitem, WORD itemnum)
{
	PrintDivisionStandings(0);
	return(0);
}

//**************************************************************************

LONG CAFCResults(MenuItem *thisitem, WORD itemnum)
{
	PrintDivisionStandings(5);
	return(0);
}

//**************************************************************************

LONG EAFCResults(MenuItem *thisitem, WORD itemnum)
{
	PrintDivisionStandings(10);
	return(0);
}

//**************************************************************************

LONG WNFCResults(MenuItem *thisitem, WORD itemnum)
{
	PrintDivisionStandings(15);
	return(0);
}

//**************************************************************************

LONG CNFCResults(MenuItem *thisitem, WORD itemnum)
{
	PrintDivisionStandings(20);
	return(0);
}

//**************************************************************************

LONG ENFCResults(MenuItem *thisitem, WORD itemnum)
{
	PrintDivisionStandings(25);
	return(0);
}

//**************************************************************************

extern BYTE fontdpcx[96];

INT DivStandPosY[6] = { TMSTND_Y+(1*15),TMSTND_Y+(7*15), TMSTND_Y+(13*15), TMSTND_Y+(1*15), TMSTND_Y+(7*15), TMSTND_Y+(13*15) };
INT DivStandPosX[6] = { TMSTND_X+20, TMSTND_X+20, TMSTND_X+20, TMSTND_X+20+512, TMSTND_X+20+512, TMSTND_X+20+512 };
void PrintDivisionStandings(INT division)
{
	INT tmptm, tmppct, tmp, i;
	INT total_games, wins, losses;
	BYTE flag,
		  *x;


	for(i=0; i<5; i++)
	{
		total_games =	((SeasonStatsPtr+i+division)->allplyrstats[GS_WIN]+
						(SeasonStatsPtr+i+division)->allplyrstats[GS_LOSS]+
						(SeasonStatsPtr+i+division)->allplyrstats[GS_TIE]) << 1;

		wins =	((SeasonStatsPtr+i+division)->allplyrstats[GS_WIN] << 1) +
				((SeasonStatsPtr+i+division)->allplyrstats[GS_TIE]);

	//****************************************************
	// TEMP FIX FOR SIM GAME BUG
	//
		if(!total_games) total_games++;
	//
	//****************************************************
		pct_table[i][0] = i+division;
		pct_table[i][1] = (wins*1000)/total_games;

	}

	flag = TRUE;

	while(flag)
	{
		flag = 0;

		for(i=4; i>0; i--)
		{
			if(pct_table[i][1] > pct_table[i-1][1])
			{
				flag++;

				tmptm = pct_table[i][0];
				tmppct = pct_table[i][1];

				pct_table[i][0] = pct_table[i-1][0];
				pct_table[i][1] = pct_table[i-1][1];

				pct_table[i-1][0] = tmptm;
				pct_table[i-1][1] = tmppct;
			}
			else
			if(pct_table[i][1] == pct_table[i-1][1])
				if ((SeasonStatsPtr+pct_table[i][0])->allplyrstats[GS_TOTAL_SCORE] >
					(SeasonStatsPtr+pct_table[i-1][0])->allplyrstats[GS_TOTAL_SCORE])
				{
					flag++;

					tmptm = pct_table[i][0];
					tmppct = pct_table[i][1];

					pct_table[i][0] = pct_table[i-1][0];
					pct_table[i][1] = pct_table[i-1][1];

					pct_table[i-1][0] = tmptm;
					pct_table[i-1][1] = tmppct;
				}
		}
	}


	for(i=0; i<5; i++)
	{
		sprintf(TempStr,"%s", SpecificTeamData[pct_table[i][0]].TeamNickName);
		PrintStringXY(TempStr,DivStandPosX[division/5],DivStandPosY[division/5]+(i*15),PEN_WHITE);

		SpacingFlag = 0;

		sprintf(TempStr,"%2d",(SeasonStatsPtr+pct_table[i][0])->allplyrstats[GS_WIN]);
		PrintStringXY(TempStr,DivStandPosX[division/5]+120,DivStandPosY[division/5]+(i*15),PEN_WHITE);

		sprintf(TempStr,"%2d",(SeasonStatsPtr+pct_table[i][0])->allplyrstats[GS_LOSS]);
		PrintStringXY(TempStr,DivStandPosX[division/5]+120+(2*16),DivStandPosY[division/5]+(i*15),PEN_WHITE);

		sprintf(TempStr,"%2d",(SeasonStatsPtr+pct_table[i][0])->allplyrstats[GS_TIE]);
		PrintStringXY(TempStr,DivStandPosX[division/5]+120+(4*16),DivStandPosY[division/5]+(i*15),PEN_WHITE);


//			sprintf(TempStr,".%03d%",pct_table[i][1]);
//			if (!pct_table[i][1])
//				sprintf(TempStr," .000");
//			else
		if (pct_table[i][1] == 1000)
			sprintf(TempStr,"1.000");
		else
			sprintf(TempStr," .%03d%",pct_table[i][1]);

		SpacingFlag = PROPORTIONAL;

		x = CurFont;
		CurFont = fontdpcx;
		PrintStringXY(TempStr,DivStandPosX[division/5]+120+(7*16),DivStandPosY[division/5]+(i*15),PEN_WHITE);
		CurFont = x;
		SpacingFlag = 0;

		if(SeasonMode)
		{
			sprintf(TempStr,"%3d",(SeasonStatsPtr+pct_table[i][0])->allplyrstats[GS_TOTAL_SCORE]);
			PrintStringXY(TempStr,DivStandPosX[division/5]+120+(12*16),DivStandPosY[division/5]+(i*15),PEN_WHITE);

			sprintf(TempStr,"%3d",(SeasonStatsPtr+pct_table[i][0])->alloppplyrstats[GS_TOTAL_SCORE]);
			PrintStringXY(TempStr,DivStandPosX[division/5]+120+(16*16),DivStandPosY[division/5]+(i*15),PEN_WHITE);
		}

		SpacingFlag = PROPORTIONAL;
	}

}

//**************************************************************************


INT InitTmStandings(void)
{
	BYTE *ptr;
	BYTE fname[64];
	WORD i;

  //Set graphics mode

	initVRamHandling();

	LoadBackgroundCD(0,"/shells/bkg/standing.tim");	// Load & free from memory

  //Init the multifile pointers

	Shell_MFile = (unsigned long *)LoadCDFile("/shells/stats/tmstand.mfl");
	DebugPrintf(0,"tmview.mfl loaded into %x\n", Shell_MFile);

  //initialize the background for the shell

	SetupFont("fontdpc2.tim");

  //load the shell graphics template files

	InitTemplates();

	multifile = Shell_MFile;
	LoadTemplate("tmstand.llx");

	// Init general purpose counters

	MenuCursorX = TMSTND_X;
	MenuCursorY	= TMSTND_Y;

	ScrollYLimit = -(8*15);

	JpadRepeatFreq = 5;				// Set joy repeat rate

	SeasonStatsPtr	= (TEAMRAMStats *)AllocMem(NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats));
	AllTeamMemoryCardMimicToSeasonStats();	// returns 0 if okay

	SyncFlag = TRUE;	// for VBI sync

	return(0);

}

//**************************************************************************

INT StrCount;

INT MainTmStandings(void)
{
	INT retcode,i;
	INT stringid;

	retcode = InputHandler(TMSTND_Y, TMSTND_Y+(0*15));

	if(!Scrolling)
	{
		SpacingFlag = 0;

		StrCount = NumStrings;

		PrintStringXYAbs("W L T  PCT  PF  PA",TMSTND_X+156, TMSTND_Y-15, PEN_YELLOW);
		SpacingFlag = PROPORTIONAL;

		if((MenuDirX == -1 && MenuTxt_Xoffset)||(MenuDirX == 1 && !MenuTxt_Xoffset))
		{
			Scrolling = -(MenuDirX*TMSTAND_SCROLL_SPEED);
			ScrollCount = (512 / TMSTAND_SCROLL_SPEED)-1;
		}

		if(MenuItemNum) PrintStringXYAbs(UP_ARROW,20,50,PEN_RED);
		if(MenuItemNum < 8) PrintStringXYAbs(DOWN_ARROW,20,206,PEN_RED);

		if(MenuTxt_Xoffset) PrintStringXYAbs(LT_ARROW,60,50,PEN_RED);
		else PrintStringXYAbs(RT_ARROW,60,50,PEN_RED);
	}
	else
	{
		MenuTxt_Xoffset+= Scrolling;

//		for(i=0; i< NumStrings-1; i++)
		for(i=0; i< StrCount; i++)
			ChangeStringXY(strings.string[i], Scrolling, 0);

		if(--ScrollCount == 0)
		{
			Scrolling/=2;

			if(Scrolling)
				ScrollCount++;
		}
	}

	return(retcode);
}

//**************************************************************************

INT TermTmStandings(void)
{
	Scrolling = 0;
	FreeMem(SeasonStatsPtr);
	{
		SeasonStatsPtr = NULL;
	}
	Std_Menu_CleanUp();
	return(0);
}

//**************************************************************************

INT DisplayTmStandings(void)
{
	if(WeekNum < TOTAL_WEEKS) return(Display_Menu(TMSTAND_MENU));
	else
		return(Display_Menu(PLAYOFFS_MENU));
}

//**************************************************************************

#define LGLDRS_X	20
#define LGLDRS_Y	60


MenuItem LgLeadersItemList[] = {
//	ACTIVE | PEN_YELLOW, LGLDRS_X, LGLDRS_Y+0, "", WAFCResults,
//	ACTIVE | PEN_YELLOW, TMSTND_X, TMSTND_Y+(6*15), "AFC CENTRAL", CAFCResults,
//	ACTIVE | PEN_YELLOW, TMSTND_X, TMSTND_Y+0+(12*15), "AFC EAST", EAFCResults,
//	ACTIVE | PEN_YELLOW, TMSTND_X+512, TMSTND_Y+0, "NFC WEST", WNFCResults,
//	ACTIVE | PEN_YELLOW, TMSTND_X+512, TMSTND_Y+(6*15), "NFC CENTRAL", CNFCResults,
//	ACTIVE | PEN_YELLOW, TMSTND_X+512, TMSTND_Y+(12*15), "NFC EAST", ENFCResults,
	END_MENU,
};


MenuObj	LgLeadersMenu = {
	(MenuItem *)LgLeadersItemList,
	InitLgLeaders,
	TermLgLeaders,
	MainLgLeaders,
	0, LGLDRS_Y-1, 512, 165,
};


//**************************************************************************


INT InitLgLeaders(void)
{
	BYTE *ptr;
	BYTE fname[64];
	WORD i;

  //Set graphics mode

	initVRamHandling();

  //initialize the background for the shell

	LoadBackgroundCD(0,"/shells/bkg/lteamsta.tim");	// Load & free from memory

	multifile = Shell_MFile = (unsigned long *)LoadCDFile("/shells/stats/tmstats.mfl");
 	DebugPrintf(0,"tmstats.mfl loaded into %x\n", Shell_MFile);

	This_MFile = (unsigned long *)LoadCDFile("/shells/stats/logos.mfl");
	DebugPrintf(0,"Logos loaded\n");

	SetupFont("fontdpc4.tim");

	CurFont = fontdpc4;

  //load the shell graphics template files

	InitTemplates();

	LoadTemplate("tmstats2.llx");

	multifile = This_MFile;
	LoadTemplate("logos.llx");

	for(i=0; i<3; i++)
	{
		if(i!=LLDivision) template[1].action[i] &=DISPLAY^0xFFFF;
	}

	FreeMem(This_MFile);

	multifile = Shell_MFile;


	MenuCursorX = LGLDRS_X;
	MenuCursorY	= LGLDRS_Y;

//	InitPlayStats();

	JpadRepeatFreq = 5;				// Set joy repeat rate

	SeasonStatsPtr	= (TEAMRAMStats *)AllocMem(NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats));
	AllTeamMemoryCardMimicToSeasonStats();	// returns 0 if okay

	switch(LLDivision)
	{
		case 0: NumTeams = 15; DivBase = 0; break;
		case 1: NumTeams = 15; DivBase = 15; break;
		case 2: NumTeams = 30; DivBase = 0; break;
	}

	ScrollYLimit = -((NumTeams-LLNUM_ITEMS)*15);

	PopupDialogBox(LGLDRS_X+36, LGLDRS_Y+3, 440, 155, 1, 0x02, 0x20, 0x20);

	return(0);

}

INT TermLgLeaders(void)
{
//	ClearScreen();					// reuired for PSX

	ResetStrings();
	FreeMem(SeasonStatsPtr);
	SeasonStatsPtr = NULL;
//	FreeTemplate(0);
	FreeAllTemplates();
	FreeFileData((BYTE *)Shell_MFile);
	FreeBackgrounds();
	return(0);
}

//**************************************************************************

LONG ColumnColor[]={PEN_BLUE,PEN_WHITE,PEN_WHITE};

INT MainLgLeaders(void)
{
	INT retcode,i,oldcolm;

	oldcolm = ColumnOffset;

	retcode = InputHandler(LGLDRS_Y, LGLDRS_Y+(0*15));

	if(MenuDirX)
	{
		ColumnOffset+=MenuDirX;
		if(ColumnOffset < 0) ColumnOffset = 0;
		else
			if(ColumnOffset > 24) ColumnOffset = 24;
	}


	if(ColumnOffset)
		PrintStringXYAbs(LT_ARROW,70,50,PEN_RED);

	if(ColumnOffset < 24)
		PrintStringXYAbs(RT_ARROW,90,50,PEN_RED);

	if(MenuItemNum)
		PrintStringXYAbs(UP_ARROW,40,52,PEN_RED);

	if(MenuItemNum < 5)
		PrintStringXYAbs(DOWN_ARROW,40,212,PEN_RED);


	DrawLgLeaders();

	if(oldcolm != ColumnOffset) PlayEffect(CURSOR_SFX);

	return(retcode);
}


//**************************************************************************

void DrawLgLeaders(void)
{
	INT i,j,oldy;

	SortByStat(ColumnOffset);

	for(i=0; i<3 && ColumnOffset+i < 25; i++)
	{
		oldy = ClipString.y;
		ClipString.y = 0;
//		PrintStringXYAbs(LLStatsStr[ColumnOffset+i],LLXpos[i],LGLDRS_Y-20, PEN_YELLOW);
		PrintStringXYCenter(LLStatsStr1[ColumnOffset+i],LGLDRS_X+LLXpos[i]+40,LGLDRS_Y-20, PEN_YELLOW);
		PrintStringXYCenter(LLStatsStr2[ColumnOffset+i],LGLDRS_X+LLXpos[i]+40,LGLDRS_Y-10, PEN_YELLOW);
		ClipString.y = oldy;
	}

	for(i= 0; i < LLNUM_ITEMS; i++)
	{
		sprintf(TempStr,"%2d",i+1+MenuItemNum);
		PrintStringXY(TempStr,LGLDRS_X,LGLDRS_Y+5+((i+MenuItemNum)*15),PEN_YELLOW);
		PrintStringXY(SpecificTeamData[pct_table[i+MenuItemNum][0]].TeamNickName,LGLDRS_X+40,LGLDRS_Y+5+((i+MenuItemNum)*15),PEN_WHITE);

		for(j=0; j<3 && ColumnOffset+j < 25; j++)
			(*(LLDisplayFunctions[ColumnOffset+j]))(SeasonStatsPtr+pct_table[i+MenuItemNum][0],LGLDRS_X+40+(LLXpos[j]),LGLDRS_Y+5+((i+ItemOffset)*15),ColumnColor[j]);
	}
}

//**************************************************************************

void SortByStat(INT column)
{
	INT i;
	BYTE flag = TRUE;

	for(i=0; i<NumTeams; i++)
	{
		pct_table[i][0] = i+DivBase;
		pct_table[i][1] = (*(LLStatsFunctions[column]))(SeasonStatsPtr+i+DivBase);
	}


	while(flag)
	{
		flag = FALSE;

		for(i=NumTeams-1; i>0; i--)
		{
			if(StatType[column])
			{
				if(pct_table[i][1] < pct_table[i-1][1])
				{
					SwapStat(i);
					flag++;
				}
			}
			else
			{
				if(pct_table[i][1] > pct_table[i-1][1])
				{
					SwapStat(i);
					flag++;
				}
			}
		}

	}

}

//**************************************************************************

void SwapStat(INT i)
{
	INT tmpstat, tmptm;

	tmptm = pct_table[i][0];
	tmpstat = pct_table[i][1];

	pct_table[i][0] = pct_table[i-1][0];
	pct_table[i][1] = pct_table[i-1][1];

	pct_table[i-1][0] = tmptm;
	pct_table[i-1][1] = tmpstat;
}

//**************************************************************************

WORD GetGamesPlayed(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_WIN]+
				dataptr->allplyrstats[GS_LOSS]+
				dataptr->allplyrstats[GS_TIE]);
}

//**************************************************************************

WORD GetWinsLossTies(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_WIN] + (dataptr->allplyrstats[GS_TIE]>>1));
}

//**************************************************************************

WORD GetTotalPoints(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_TOTAL_SCORE]);
}

//**************************************************************************

WORD GetScoreByQuarter(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_SCOREQ1]+
				  dataptr->allplyrstats[GS_SCOREQ2]+
				  dataptr->allplyrstats[GS_SCOREQ3]+
				  dataptr->allplyrstats[GS_SCOREQ4]+
				  dataptr->allplyrstats[GS_SCOREQ5]);

}

//**************************************************************************

WORD Get1stDowns(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_1STDOWNS]);
}

//**************************************************************************

WORD Get3rdDowns(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_3RDDOWNS_MADE]);
}

//**************************************************************************

WORD Get4thDowns(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_4THDOWNS_MADE]);
}

//**************************************************************************

WORD GetNetYards(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_PASS_YARDS] +
					dataptr->allplyrstats[GS_RUSH_YARDS] +
					dataptr->allplyrstats[GS_SACK_YARDS]);
}

//**************************************************************************

WORD GetAvgYardsPerGame(struct teamRAMstats *dataptr)
{
		GamesPlayed = GetGamesPlayed(dataptr);

		return(GetNetYards(dataptr)/(!GamesPlayed ? 1:GamesPlayed));
}

//**************************************************************************

WORD GetTotalPlays(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_NUM_RUN_PLAYS]
				+ dataptr->allplyrstats[GS_NUM_PASS_PLAYS]);
}

//**************************************************************************


WORD GetAvgYardsPerPlay(struct teamRAMstats *dataptr)
{
		TotalPlays = GetTotalPlays(dataptr);

		return(GetNetYards(dataptr)/(!TotalPlays ? 1:TotalPlays));
}

//**************************************************************************

WORD GetNetYardsPassing(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_PASS_YARDS]
				+ dataptr->allplyrstats[GS_SACK_YARDS]);
}

//**************************************************************************

WORD GetAvgYardsPassing(struct teamRAMstats *dataptr)
{
		GamesPlayed = GetGamesPlayed(dataptr);

		return ((dataptr->allplyrstats[GS_PASS_YARDS]
			   		+ dataptr->allplyrstats[GS_SACK_YARDS])/(!GamesPlayed ? 1:GamesPlayed));
}

//**************************************************************************

WORD GetAttComp(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_PASS_COMP]);
}

//**************************************************************************

WORD GetAvgPerPass(struct teamRAMstats *dataptr)
{
		return((dataptr->allplyrstats[GS_PASS_YARDS]
			  	+ dataptr->allplyrstats[GS_SACK_YARDS])/
			  	(!dataptr->allplyrstats[GS_PASS_ATT]?1:dataptr->allplyrstats[GS_PASS_ATT]));
}

//**************************************************************************

WORD GetCompPct(struct teamRAMstats *dataptr)
{
	INT j;

	j = (dataptr->allplyrstats[GS_PASS_ATT]<<8)/
									(!dataptr->allplyrstats[GS_PASS_COMP] ? 1:dataptr->allplyrstats[GS_PASS_COMP]);

	return(j ? (100<<8)/j : 0);
}

//**************************************************************************

WORD GetNetYardsRushing(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_RUSH_YARDS]);
}

//**************************************************************************

WORD GetAvgYardsRushing(struct teamRAMstats *dataptr)
{
		GamesPlayed = GetGamesPlayed(dataptr);

		return(dataptr->allplyrstats[GS_RUSH_YARDS]/(!GamesPlayed ? 1:GamesPlayed));
}

//**************************************************************************

WORD GetTotalRushes(struct teamRAMstats *dataptr)
{
		return (dataptr->allplyrstats[GS_RUSH_ATT]);
}

//**************************************************************************


WORD GetAvgPerRushes(struct teamRAMstats *dataptr)
{
	if(dataptr->allplyrstats[GS_RUSH_ATT])
		return(dataptr->allplyrstats[GS_RUSH_YARDS]/dataptr->allplyrstats[GS_RUSH_ATT]);
	else
		return(0);
}

//**************************************************************************

WORD GetPuntAvg(struct teamRAMstats *dataptr)
{
	 if(dataptr->allplyrstats[GS_PUNT])
		return(dataptr->allplyrstats[GS_PUNT],
				  dataptr->allplyrstats[GS_PUNT_YARDS]/dataptr->allplyrstats[GS_PUNT]);
	else
		return(0);
}

//**************************************************************************

WORD GetPenalties(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_PENALTY_NUM]);
}

//**************************************************************************

WORD GetDefensiveSacks(struct teamRAMstats *dataptr)
{
		return (dataptr->allplyrstats[GS_SACK_NUM]);
}

//**************************************************************************

WORD GetTurnovers(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_PASS_INT] + dataptr->allplyrstats[GS_FUM_LOST]);
}

//**************************************************************************

WORD GetGiveaways(struct teamRAMstats *dataptr)
{
		return((dataptr->alloppplyrstats[GS_PASS_INT]+dataptr->alloppplyrstats[GS_FUM_LOST])
			  - (dataptr->allplyrstats[GS_PASS_INT]+dataptr->allplyrstats[GS_FUM_LOST]));
}

//**************************************************************************

WORD GetAvgTimePoss(struct teamRAMstats *dataptr)
{
		return(dataptr->allplyrstats[GS_TOP]);
}

//**************************************************************************

INT DisplayLgLeaders(void)
{
	return(Display_Menu(LGLEADERS_MENU));
}




//**************************************************************************
//**************************************************************************
//**************************************************************************
// League Leaders Main Menu
//**************************************************************************
//**************************************************************************
//**************************************************************************

#define LEADERS_X	40
#define LEADERS_Y	80


//BYTE LeaderStates[]={0,2,1,5,4,3};

MenuItem LeadersItemList[] = {
	ACTIVE | HI_YELLOW | PEN_WHITE, LEADERS_X, LEADERS_Y+0, "Team Stats", NULL,
	ACTIVE | HI_YELLOW | PEN_WHITE, LEADERS_X, LEADERS_Y+15, "Passing", NULL,
	ACTIVE | HI_YELLOW | PEN_WHITE, LEADERS_X, LEADERS_Y+30, "Rushing", NULL,
	ACTIVE | HI_YELLOW | PEN_WHITE, LEADERS_X, LEADERS_Y+45, "Receiving", NULL,
	ACTIVE | HI_YELLOW | PEN_WHITE, LEADERS_X, LEADERS_Y+60, "Kicking", NULL,
	ACTIVE | HI_YELLOW | PEN_WHITE, LEADERS_X, LEADERS_Y+75, "Punts", NULL,
	ACTIVE | HI_YELLOW | PEN_WHITE, LEADERS_X, LEADERS_Y+90, "Kick Returns", NULL,
	ACTIVE | HI_YELLOW | PEN_WHITE, LEADERS_X, LEADERS_Y+105, "Exit", NULL,
	END_MENU,
};

MenuObj	LeadersMenu = {
	(MenuItem *)LeadersItemList,
	InitLeaders,
	TermLeaders,
	MainLeaders,
	0, LEADERS_Y-1, 512, 165,
};


//**************************************************************************


INT InitLeaders(void)
{
	INT i;

  //Set graphics mode

	initVRamHandling();

  //initialize the background for the shell

	LoadBackgroundCD(0,"/shells/bkg/leaguele.tim");	// Load & free from memory

  //Init the multifile pointers

	Shell_MFile = (unsigned long *)LoadCDFile("/shells/stats/leaguele.mfl");
	DebugPrintf(0,"leaguele.mfl loaded into %x\n", Shell_MFile);

	SetupFont("fontdpc2.tim");

  //load the shell graphics template files

	InitTemplates();

	multifile = Shell_MFile;
	LoadTemplate("leaguele.llx");

	MenuCursorX = LEADERS_X-40;
	MenuCursorY	= LEADERS_Y-4;

	JpadRepeatFreq = 20;				// Set joy repeat rate

	ScrollYLimit = 0;

	for(i=0; i<3; i++)
	{
		BadgeAnim[i] = BadgeSpeed[i] = BadgeDeccel[i] = 0;
		setShadeTex(&template[0].poly[i], 0);
	}

	PopupDialogBox(30, MenuCursorY+6, 140, 12, 2, 0x00, 0x40, 0x50);

	return(0);
}

INT TermLeaders(void)
{
	ResetStrings();
//	FreeTemplate(0);
	FreeAllTemplates();
	FreeFileData((BYTE *)Shell_MFile);
	FreeBackgrounds();
	return(0);
}

//**************************************************************************


BYTE *BadgeNames[3]={"A0","N0","NFL0"};


INT MainLeaders(void)
{
	INT retcode,i;

	SetHighlight();

	for(i=0; i<3; i++)
	{
		if(i==BadgeNum || BadgeDeccel[i])
		{
			if(--BadgeSpeed[i] < 0)
			{
				BadgeSpeed[i] = 4;
				BadgeAnim[i]++;
				BadgeAnim[i]%=8;

				if(BadgeDeccel[i]) BadgeDeccel[i]--;

				sprintf(TempStr,"%s%d",BadgeNames[i],BadgeAnim[i]);
				ChangePolyCmpTexture(0, i, TempStr);
			}
		}

		if(i==BadgeNum) setRGB0(&template[0].poly[i], 0x80, 0x80, 0x80);
		else setRGB0(&template[0].poly[i], 0x40, 0x40, 0x40);
	}

	if(MenuDirX)
	{
		PlayEffect(CURSOR_SFX);
		BadgeDeccel[BadgeNum]= (8-BadgeAnim[BadgeNum])+8;
		BadgeNum+=MenuDirX;

		if(BadgeNum<0) BadgeNum=2;
		BadgeNum%=3;
	}

	switch(retcode = InputHandler(LEADERS_Y, LEADERS_Y+(6*15)))
	{
		case CROSS:
						switch(MenuItemNum)
						{
							case 0: retcode = 7; break;
							case 1: retcode = 1; break;
							case 2: retcode = 2; break;
							case 3: retcode = 3; break;
							case 4: retcode = 4; break;
							case 5: retcode = 5; break;
							case 6: retcode = 6; break;
							case 7: retcode = -1; break;
						}

						LdrIndex = retcode-1;
						break;

		case TRIANGLE:	retcode= -1;
						break;

		default:		retcode = 0;
	}

	MoveDialog(0, 30, MenuCursorY+6);				// Move to X, Y

	LLDivision = BadgeNum;

	return(retcode);
}

//**************************************************************************

INT DisplayLeaders(void)
{
	return(Display_Menu(LEADERS_MENU));
}

//**************************************************************************
//**************************************************************************
// Playoffs Display
//**************************************************************************
//**************************************************************************


#define PLAYOFFS_X	20
#define PLAYOFFS_Y	50

#define LOGOS_TEMPLATE 5

MenuItem PlayoffsItemList[] = {
	ACTIVE | PEN_YELLOW, PLAYOFFS_X, PLAYOFFS_Y+0, "Div 1st", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X, PLAYOFFS_Y+32, "Wild 1st", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X, PLAYOFFS_Y+64, "Wild 2nd", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X, PLAYOFFS_Y+96, "Div 2nd", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X, PLAYOFFS_Y+128, "Div 3rd", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X, PLAYOFFS_Y+160, "Wild 3rd", NULL,

	ACTIVE | PEN_YELLOW, PLAYOFFS_X+1040, PLAYOFFS_Y+0, "Div 1st", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X+1040, PLAYOFFS_Y+32, "Wild 1st", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X+1040, PLAYOFFS_Y+64, "Wild 2nd", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X+1040, PLAYOFFS_Y+96, "Div 2nd", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X+1040, PLAYOFFS_Y+128, "Div 3rd", NULL,
	ACTIVE | PEN_YELLOW, PLAYOFFS_X+1040, PLAYOFFS_Y+160, "Wild 3rd", NULL,

	END_MENU,
};


MenuObj	PlayoffsMenu = {
	(MenuItem *)PlayoffsItemList,
	InitPlayoffs,
	TermPlayoffs,
	MainPlayoffs,
//	-20, TMSTND_Y-1, 512+20, 165,
	-512, 0, 3*512, 220,
};


LONG PlayoffPos[][2]={
						PLAYOFFS_X+100+32,PLAYOFFS_Y+26,	//0
						PLAYOFFS_X+100+32,PLAYOFFS_Y+58,
						PLAYOFFS_X+100+30,PLAYOFFS_Y+122,
						PLAYOFFS_X+100+30,PLAYOFFS_Y+154,

						PLAYOFFS_X+950+32,PLAYOFFS_Y+26,	//2
						PLAYOFFS_X+950+32,PLAYOFFS_Y+58,
						PLAYOFFS_X+950+30,PLAYOFFS_Y+122,
						PLAYOFFS_X+950+30,PLAYOFFS_Y+154,

						PLAYOFFS_X+220+32,PLAYOFFS_Y-6,	//4
						PLAYOFFS_X+220+32,PLAYOFFS_Y+42,
						PLAYOFFS_X+220+30,PLAYOFFS_Y+90,
						PLAYOFFS_X+220+30,PLAYOFFS_Y+138,

						PLAYOFFS_X+830+32,PLAYOFFS_Y-6,	//6
						PLAYOFFS_X+830+32,PLAYOFFS_Y+42,
						PLAYOFFS_X+830+30,PLAYOFFS_Y+90,
						PLAYOFFS_X+830+30,PLAYOFFS_Y+138,

						PLAYOFFS_X+340+32,PLAYOFFS_Y+18,	//8
						PLAYOFFS_X+340+32,PLAYOFFS_Y+114,
						PLAYOFFS_X+710+30,PLAYOFFS_Y+18,
						PLAYOFFS_X+710+30,PLAYOFFS_Y+114,

						PLAYOFFS_X+460+32,PLAYOFFS_Y+66,	//10
						PLAYOFFS_X+590+30,PLAYOFFS_Y+66,
};

//**************************************************************************


INT InitPlayoffs(void)
{
	BYTE *ptr;
	BYTE fname[64];
	WORD i;

  //Set graphics mode

	initVRamHandling();

	LoadBackgroundCD(0,"/shells/bkg/standing.tim");	// Load & free from memory

  //Init the multifile pointers

	Shell_MFile = (unsigned long *)LoadCDFile("/shells/stats/playoffs.mfl");
	DebugPrintf(0,"playoffs.mfl loaded into %x\n", Shell_MFile);

	SetupFont("fontdpc2.tim");

  //load the shell graphics template files

	InitTemplates();

	multifile = Shell_MFile;
	LoadTemplate("tmstand.llx");
	LoadTemplate("ltree1.llx");
	LoadTemplate("ltree2.llx");
	LoadTemplate("rtree1.llx");
	LoadTemplate("rtree2.llx");
	LoadTemplate("logos.llx");

	MovePolyToXY(&template[LOGOS_TEMPLATE].poly[0],-256+20,-(120+10));
	MovePolyToXY(&template[LOGOS_TEMPLATE].poly[1],-256+430,-(120+10));

	// Init general purpose counters

	MenuCursorX = TMSTAT_X;
	MenuCursorY	= TMSTAT_Y;

//	InitPlayStats();

	ScrollYLimit = 1000;		// No Scroll

	JpadRepeatFreq = 2;				// Set joy repeat rate

	SeasonStatsPtr	= (TEAMRAMStats *)AllocMem(NUM_TEAMS_IN_LEAGUE * sizeof(TEAMRAMStats));
	AllTeamMemoryCardMimicToSeasonStats();	// returns 0 if okay

	return(0);

}

//**************************************************************************

static WORD ShadowNumStrings;

INT MainPlayoffs(void)
{
	INT retcode,i;

//	SetHighlight();

	retcode = InputHandler(0, 0);		// Cursor pos doesn't matter


	if(!Scrolling)
	{
		DisplayPlayoffsInfo();

		SpacingFlag = 0;
		SpacingFlag = PROPORTIONAL;

		if((MenuDirX == -1 && MenuTxt_Xoffset)||(MenuDirX == 1 && MenuTxt_Xoffset > -638))
		{
			Scrolling = -(MenuDirX*TMSTAND_SCROLL_SPEED);
			ScrollCount = (320 / TMSTAND_SCROLL_SPEED)-1;
		}

		ShadowNumStrings = NumStrings;	// don't want to scroll any new strings created past this point !

		switch(MenuTxt_Xoffset)
		{
			case 0:		template[LOGOS_TEMPLATE].action[1] &= DISPLAY^0xFFFF;
						template[LOGOS_TEMPLATE].action[0] |= DISPLAY;
						PrintStringXYAbs(RT_ARROW,332,25,PEN_RED);
						break;

			case -638:	template[LOGOS_TEMPLATE].action[0] &= DISPLAY^0xFFFF;
						template[LOGOS_TEMPLATE].action[1] |= DISPLAY;
						PrintStringXYAbs(LT_ARROW,186,25,PEN_RED);
						break;

			case -319:	template[LOGOS_TEMPLATE].action[0] |= DISPLAY;
						template[LOGOS_TEMPLATE].action[1] |= DISPLAY;
						PrintStringXYAbs(RT_ARROW,332,25,PEN_RED);
						PrintStringXYAbs(LT_ARROW,186,25,PEN_RED);
						break;
			default:
						PrintStringXYAbs(RT_ARROW,332,25,PEN_RED);
						PrintStringXYAbs(LT_ARROW,186,25,PEN_RED);
						break;
		}

	}
	else
	{
		MenuTxt_Xoffset+= Scrolling;

		for(i=0; i< ShadowNumStrings; i++)
			ChangeStringXY(strings.string[i], Scrolling, 0);


		for(i=0; i<2; i++)
		{
			MovePolyXY(&template[1].poly[i], Scrolling , 0);
			MovePolyXY(&template[1].poly[i+2], Scrolling , 0);
			MovePolyXY(&template[2].poly[i], Scrolling , 0);

			MovePolyXY(&template[3].poly[i], Scrolling , 0);
			MovePolyXY(&template[3].poly[i+2], Scrolling , 0);
			MovePolyXY(&template[4].poly[i], Scrolling , 0);
		}

		if(--ScrollCount == 0)
		{
			Scrolling/=2;

			if(Scrolling)
				ScrollCount++;
		}

	}

	return(retcode);
}

//**************************************************************************

INT TermPlayoffs(void)
{
	ULONG i;

	JustPlayed = FALSE;

	Scrolling = 0;

	FreeMem(SeasonStatsPtr);
	SeasonStatsPtr = NULL;
	Std_Menu_CleanUp();
	return(0);
}


void DisplayPlayoffsInfo(void)
{
	LONG i,j;
	BYTE color;

	switch(WeekNum-TOTAL_WEEKS)
	{
		case 0:	i = 7;
				j = -1;
				break;
		case 1: i = 15;
				j = 7;
				break;
		case 2: i = 19;
				j = 15;
				break;
		case 3: i = 21;
				j = 19;
				break;
		default: i = 21;
				 j = 21;
				break;
	}

	for(;i>=0; i--)
	{
		if(playoff_game_team[i] == HumanTeamNum)
		{
			color = PEN_BLUE;
		}
		else
		{
			if(i>j)
				color = PEN_WHITE;
			else
				color = PEN_RED;
		}

		sprintf(TempStr,"%s", SpecificTeamData[playoff_game_team[i]].TeamNickName);
		PrintStringXYCenter(TempStr,PlayoffPos[i][0]+MenuTxt_Xoffset,PlayoffPos[i][1],color);

		if(SeasonMode || (JustPlayed == TRUE && i>j))
		{
			sprintf(TempStr,"%d",MemoryCardMimicPtr->GameScores[(i>>1)+TOTAL_GAMES][i%2]);
			PrintStringXYCenter(TempStr,PlayoffPos[i][0]+MenuTxt_Xoffset,PlayoffPos[i][1]+13,PEN_WHITE);
		}
	}

}

LONG round(LONG result)
{
	if(result & 0x00000080)
		result+=0x100;
	return(result>>8);
}

