#include <stdio.h>
#include <sys/types.h>

#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>
#include "prjhdrs.h"

#include "fbgequ.h"
#include "fbequ.h"
#include "fbram.h"
#include "shell.h"


//**************************************************************************
// Data structures & defines for menu system

#include "menu.h"
#include "fbmenu.h"

//**************************************************************************

#define	ITEMS_X	60
#define	ITEMS_Y	100
#define	TMRATE_X 256
#define	TMRATE_Y 70


#define NAME_TEMPLATE	1
#define TMVIEW_TEMPLATE 0
#define	RATINGS_BAR		0
#define	RATINGS_BAR2	8		// Unused in Team stats

#define CURSORY_MIN 20
#define CURSORY_MAX 200

extern void ChangePolyTimTexture(short num, short index, unsigned long *texture);
extern InitControl(int pad);

int InitViewTeam(void);
int MainViewTeam(void);
int TermViewTeam(void);
void ViewTeam(Sint16 team);


MenuItem GmViewTeamItemList[] = {

	ACTIVE	| MENU_ITEM | PEN_WHITE, ITEMS_X, ITEMS_Y+0 , "Team Stats",NULL,
	ACTIVE	| MENU_ITEM | PEN_WHITE, ITEMS_X, ITEMS_Y+15, "Player Stats",NULL,
	ACTIVE	| MENU_ITEM | PEN_WHITE, ITEMS_X, ITEMS_Y+30, "NFL Schedule",NULL,
//	ACTIVE	| MENU_ITEM | PEN_WHITE, ITEMS_X, ITEMS_Y+45, "Play Book",NULL,
	ACTIVE	| MENU_ITEM | PEN_WHITE, ITEMS_X, ITEMS_Y+45, "Substitutions",NULL,
	ACTIVE	| MENU_ITEM | PEN_WHITE, ITEMS_X, ITEMS_Y+60, "Exit",NULL,

	ACTIVE | PEN_YELLOW, TMRATE_X, TMRATE_Y+0, "Passing",NULL,
	ACTIVE | PEN_YELLOW, TMRATE_X, TMRATE_Y+15, "Running",NULL,
	ACTIVE | PEN_YELLOW, TMRATE_X, TMRATE_Y+30, "Receiving",NULL,
	ACTIVE | PEN_YELLOW, TMRATE_X, TMRATE_Y+45, "Off. Line",NULL,
	ACTIVE | PEN_YELLOW, TMRATE_X, TMRATE_Y+60, "Def. Line",NULL,
	ACTIVE | PEN_YELLOW, TMRATE_X, TMRATE_Y+75, "Linebackers",NULL,
	ACTIVE | PEN_YELLOW, TMRATE_X, TMRATE_Y+90, "Def. Backs",NULL,
	ACTIVE | PEN_YELLOW, TMRATE_X, TMRATE_Y+105, "Spec. Teams",NULL,

	END_MENU,
};

MenuObj	TeamViewMenu = {
	(MenuItem *)GmViewTeamItemList,		// Menu object
	InitViewTeam,						// Menu Init
	TermViewTeam,						// Menu termination
	MainViewTeam,						// Menu proc (called each frame)
	0, 0, 512, 240,						// Full screen
};



/***************************************************************************/

// Machine Specific functions


int InitViewTeam(void)
{
	Uint32 i;
	char *ptr;
	char fname[64];

  /*Set graphics mode*/

	initVRamHandling();

  /*initialize the background for the shell*/

	LoadTeamPic(GameTeamData[WhichTeam].TeamNumber);

  /*Init the multifile pointers*/

	Shell_MFile = (unsigned long *)LoadCDFile("/shells/stats/tmview.mfl");
	DebugPrintf(0,"tmview.mfl loaded into %x\n", Shell_MFile);

	This_MFile = (unsigned long *)LoadCDFile("/shells/names/nickname.mfl");
	DebugPrintf(0,"nickname.mfl loaded into %x\n", This_MFile);

  /*initialize the background for the shell*/

	SetupFont("fontdpc2.tim");

  /*load the shell graphics template files*/

	InitTemplates();

	multifile = Shell_MFile;
	LoadTemplate("tmview.llx");

	multifile = This_MFile;
	LoadTemplate("nickname.llx");

	LoadNickname(TeamNo, NAME_TEMPLATE, 0);

	FreeFileData((char *)This_MFile);

	multifile = Shell_MFile;

	MovePolyToXY(&template[NAME_TEMPLATE].poly[0],160-256, 20-120);
	MovePolyToXY(&template[NAME_TEMPLATE].poly[1],288-256, 20-120);

//	MovePolyToXY(&template[TMVIEW_TEMPLATE].poly[FBALL], (-248), -80);

	/* Init general purpose counters */

	MenuCursorX = ITEMS_X-16;
	MenuCursorY	= ITEMS_Y;

	ScrollYLimit = 1000;		// Just deactivate Y scrolling by putting way out of range

	JpadRepeatFreq = 20;

	for(i = 0; i < 8; i++)
		MovePolyToXY(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR2+i],512,0);	// Put unused bars offscreen

	return(0);

}

int MainViewTeam(void)
{
	int retcode,i;

	FreeAllDialogBoxes();

	PopupDialogBox(MenuCursorX, MenuCursorY, 155, 12, 2, 0x30, 0x00, 0x30);

//	PrintStringXYCenter(SpecificTeamData[TeamNo].TeamCity,256, 20, PEN_YELLOW);

	for(i=0; i<8; i++)
	{
		ChangeABSPolyWH(&template[TMVIEW_TEMPLATE].poly[RATINGS_BAR+i],
							SpecificTeamData[TeamNo].Stats[i], 5);
	}

//	AnimFootBall();

	switch(retcode = InputHandler(ITEMS_Y, ITEMS_Y+(15*4)))
	{
		case TRIANGLE:	retcode = -1;
						break;
		case CROSS:		retcode = MenuItemNum+1;
						break;
		default:		retcode = 0;
	}


	return(retcode);
}


void ViewTeam(Sint16 team)
{
	int retcode= 0;

	WhichTeam = team;						// Store whether home or away
	TeamNo = GameTeamData[team].TeamNumber;	// Convert Home-Away value to actual Team No.

	while(retcode !=-1)
	{
		retcode = Display_Menu(TEAM_VIEW_MENU);

		switch(retcode-1)
		{
			case 0:	Display_Menu(TMSTATS_MENU); break;
			case 1: Display_Menu(PLAYER_VIEW_MENU); break;
			case 2:	Display_Menu(SCHEDULE_MENU); break;
//			case 3:	DoPlayBook(); break;
			case 3: Display_Menu(SUBSTITUTION_MENU);break;
			case 4: retcode =-1; break;
		}
	}

}


int TermViewTeam(void)
{
//	FreeTemplate(1);
	Std_Menu_CleanUp();					// Menu termination
}

//***************************************************************************

// For Arcade game to call directly (accepts pad parameter)

int Display_Player_Stats(int pad)
{
	InitControl(pad);
	Display_Menu(PLAYER_VIEW_MENU);
}

//***************************************************************************

int Display_Substitute(int pad)
{
	InitControl(pad);
	Display_Menu(SUBSTITUTION_MENU);
}

//***************************************************************************

int Display_Game_Stats(int pad)
{
	InitControl(pad);
	Display_Menu(TMSTATS_MENU);
}

//***************************************************************************

