/*************************************************************************
 *	tselshl.c for the PSX.
 *
 *	Copyright 1997 GAMETEK
 *	All rights reserved.
 *
 * Written by Tom McWilliams
 * Started: 2-24-97
 *
 *	Notes: Team Select Screen/Pad Select Screen
 *
 *************************************************************************/

#include <stdio.h>
#include <sys/types.h>

#include <libetc.h>
#include <libgte.h>
#include <libgpu.h>
#include <libsn.h>
#include <libgs.h>

#include "prjhdrs.h"

#include "fbequ.h"
#include "fbram.h"

#include "multifil.h"
#include "shellsys.h"

#include "shell.h"


#define CONFNUM 6				// number of conferences

#define OBJ_HOME		0		// team sel object numbers
#define OBJ_AWAY		6
#define OBJ_CONF		0
#define OBJ_LOGOS 	1

#define LOGORADIUS 80		// team select logos
#define LOGOTILT 16
#define LOGOSPEED 12

#define OBJ_HOMEBOX 8		// pad sel object numbers
#define OBJ_AWAYBOX (OBJ_HOMEBOX + MAX_USERS_PER_TEAM)

// external prototypes

extern void LoadBackgroundCD (int num, char *fname);

// internal prototypes

// global variables

char *confname[]={ "afcwest","afccentr","afceast","nfcwest","nfccentr","nfceast" };

// teamselect info for each team

struct teaminfo
{
	int IsAway;						// 0=home 1=away
	int Team;							// current team selected
	int LastConf;					// last conference loaded
	LONG LogoAngle;				// the logo's rotation
	int LogoX,LogoY;			// X,Y center for rotation
	ShellObj **Objects;		// ptrs to this team's logo objects (first of 5)
};

// Callback for display loop

int UpdateGraphics (void)
{
	PollPC();

///	if (JpadTrig[0] & JPADRup)
	{
		UpdateBackground();

		DisplayAllStrings();
		DisplayObjects ();
 		TPageViewer ();		//debug
	}
	return (1);
}

// update graphics for current team selections

DrawTeamStuff (MultiFile_t *mfl,struct teaminfo *ti,int hl)
{
	ShellObj *tsobj;
	char filename[35];
	int n,x,y;
	long z;
	long angle;


	// helmet
	sprintf (filename,"H%02d%c.mdf",ti->Team,ti->IsAway ? 'L' : 'R');
	LoadObject (ShellObject[HOMEHELM+ti->IsAway],GetMultiFilePtr (mfl,filename));

	// name
	sprintf (filename,"NAME%02d.mdf",ti->Team);
	LoadObject (ShellObject[HOMENAME+ti->IsAway],GetMultiFilePtr (mfl,filename));

	// conference
	sprintf (filename,"%s.mdf",confname[ti->Team/5]);
	LoadObject (ti->Objects[OBJ_CONF],GetMultiFilePtr (mfl,filename));

	// logos

	for (n=0; n<5; n++)
	{
		tsobj=ti->Objects[OBJ_LOGOS+n];

		angle = ((359 - ti->LogoAngle + (n * 360 / 5)) % 360) * 4096 / 360;

		z = ccos(angle);
		y = z * LOGOTILT / 4096;
		x = csin(angle) * LOGORADIUS / 4096;
	/*	if (ti->IsAway) */ y=-y;			// tilt logos opposite

		z = (z + 4096)/2;		// make z positive for scaling
		tsobj->Depth = 80 - 80 * z / 4096;

		SetObjectSize (tsobj,z*3 - 8192 + (9900 - z*11/4) * (z < 3600) );

		SetObjectCenterPos (tsobj,ti->LogoX + x,ti->LogoY + y);

		z = 0x60 * z / 4096 + 0x20;		// get luma val 0x40-0x80

		SetObjectRGB (tsobj,z,z,z);

//	if ((n==(ti->Team % 5)) && hl)

		if (ti->LastConf != ti->Team/5)
		{
			sprintf (filename,"TmLogo%02d.mdf",ti->Team - (ti->Team % 5) + n);
			LoadObject (tsobj,GetMultiFilePtr (mfl,filename));
		}
	}
	ti->LastConf=ti->Team/5;
}


DoTeamPick (MultiFile_t *mfl,struct teaminfo *ti,int pad)
{
	int m,n;
	int upd;


	upd=0;

	if (JpadTrig[pad] & JPADLup)
	{
		if (ti->Team >= 5) ti->Team-=5;
		else ti->Team=ti->Team % 5 + CONFNUM*5-5;
		upd=1;
	}
	if (JpadTrig[pad] & JPADLdown)
	{
		if (ti->Team < (CONFNUM*5-5)) ti->Team+=5;
		else ti->Team=ti->Team % 5;
		upd=1;
	}
	if (JpadTrig[pad] & JPADLleft)
	{
		if (ti->Team % 5) ti->Team--;
		else ti->Team += 5-1;
	}
	if (JpadTrig[pad] & JPADLright)
	{
		if ((ti->Team % 5) < 5-1) ti->Team++;
		else ti->Team -= 5-1;
	}

	if ((ti->Team % 5)*(360/5) != ti->LogoAngle)		// rotate logos
	{
		n = LOGOSPEED;
		m = ((ti->Team % 5)*(360/5) - ti->LogoAngle);
		if (m < 0)
		{
			m = -m;
			n = -n;
		}
		if (m > 360/2) n = -n;

		ti->LogoAngle += n;
		if (ti->LogoAngle < 0) ti->LogoAngle += 360;
		ti->LogoAngle %= 360;
		upd=1;
	}

	if (upd) DrawTeamStuff (mfl,ti,1);
}

////////////////////////////////////////////////////////////////////////////////
//
// Team Select Screen
//

int DoTeamSelectScreen ()
{
	int ret;
	MultiFile_t *mfl;		// multifile ptr
	ShellObj **sobj;
	struct teaminfo HomeTeam,AwayTeam;
	int awaypad,homepad,pad0,pad1;
	int n;

	FreeGraphics ();

	Dprintf ("Loading Team Select Background...\n");
	LoadBackgroundCD (0,"/shells/tsel/slctpsx.tim");
	DrawSync(0);

	Dprintf ("Loading Team Select Graphics...\n");
	mfl = (MultiFile_t*)LoadCDFile ("/shells/tsel/tsel.mfl");

	InitShellGraphics ();

	sobj=LoadObjects (mfl,GetMultiFilePtr (mfl,"teamsel.mdf") );

// Load Fonts

//	InitFonts ();
//	Register_Font_Cluts ();
//	LoadFont ("fontdpc2.tim", 3);

	HomeTeam.IsAway=0;
	AwayTeam.IsAway=1;

	HomeTeam.Objects=sobj+OBJ_HOME;
	AwayTeam.Objects=sobj+OBJ_AWAY;

	HomeTeam.Team=GameTeamData[HOME_TEAM_SLOT].TeamNumber;		// set starting teams
	AwayTeam.Team=GameTeamData[AWAY_TEAM_SLOT].TeamNumber;

	HomeTeam.LogoAngle=(HomeTeam.Team % 5) * (360 / 5);
	AwayTeam.LogoAngle=(AwayTeam.Team % 5) * (360 / 5);

	HomeTeam.LogoX = HomeTeam.Objects[OBJ_LOGOS]->X;			// save logo center position
	HomeTeam.LogoY = HomeTeam.Objects[OBJ_LOGOS]->Y;
	AwayTeam.LogoX = AwayTeam.Objects[OBJ_LOGOS]->X;
	AwayTeam.LogoY = AwayTeam.Objects[OBJ_LOGOS]->Y;

	HomeTeam.LastConf=-1;		// flag to update conference logos
	AwayTeam.LastConf=-1;

	if (GameType==SEASON_GAME)							// set up screen for 1 team only in season mode
	{
		for (n=0; n<5; n++) AwayTeam.Objects[OBJ_LOGOS + n]->Flags &= ~OBJ_DISPLAY;
		AwayTeam.Objects[OBJ_CONF]->Flags &= ~OBJ_DISPLAY;
	}
	else
	{
		ShellObject[AWAYNAME]->Flags |= OBJ_DISPLAY;
		ShellObject[AWAYHELM]->Flags |= OBJ_DISPLAY;
	}		
	ShellObject[HOMENAME]->Flags |= OBJ_DISPLAY;		// turn on graphics
	ShellObject[HOMEHELM]->Flags |= OBJ_DISPLAY;

	DrawTeamStuff (mfl,&HomeTeam,0);
	DrawTeamStuff (mfl,&AwayTeam,0);

	JpadRepeatFreq=-1;

	awaypad=homepad=pad0=pad1=-1;

	for (n=0; n<8; n++)		// assign first 1 or 2 pads to teams
	{
		if (JpadID[n])
		{
			if (homepad < 0) homepad=awaypad=pad0=n;
			else if (pad1 < 0) pad1=n;
		}
	}

	if (GameType==SEASON_GAME)
	{
		pad1=-1;		// no 2nd joy on season
		awaypad=-1;
	}

	ret=1;
	while ((homepad >= 0) || (awaypad >= 0))
	{
 		if (AnyPadData & JPADtriangle)
 		{
 			ret=-1;			// back out
 			break;
 		}

		if (pad1 >= 0) if (JpadTrig[pad1])
		{
			awaypad=pad1;					// 2nd guy comes in/changes mind
 			for (n=0; n<5; n++) AwayTeam.Objects[OBJ_LOGOS+n]->Flags |= OBJ_DISPLAY;
		}

		if ((awaypad != pad0) && JpadTrig[pad0])
		{
			homepad=pad0;		// 1st guy changes mind
 			for (n=0; n<5; n++) HomeTeam.Objects[OBJ_LOGOS+n]->Flags |= OBJ_DISPLAY;
		}

 		if ((awaypad >= 0) && (homepad != awaypad))
 		{
 			DoTeamPick (mfl,&AwayTeam,awaypad);
 			if (JpadTrig[awaypad] & JPADx)				// leave picked logo
 			{
 				for (n=0; n<5; n++) if (AwayTeam.Team % 5 != n) AwayTeam.Objects[OBJ_LOGOS+n]->Flags &= ~OBJ_DISPLAY;
 				awaypad=-1;
 			}
 		}

 		if (homepad >= 0)
 		{
 			DoTeamPick (mfl,&HomeTeam,homepad);
 			if (JpadTrig[homepad] & JPADx)
 			{
 				for (n=0; n<5; n++) if (HomeTeam.Team % 5 != n) HomeTeam.Objects[OBJ_LOGOS+n]->Flags &= ~OBJ_DISPLAY;
 				homepad=-1;
 				DrawTeamStuff (mfl,&AwayTeam,1);		// show hilite on awayteam now if 1 player picking
 			}
 		}
 
		DoFrameLoop (1,UpdateGraphics);		// Pads also read by frame process
	}

 	GameTeamData[HOME_TEAM_SLOT].TeamNumber = HomeTeam.Team;
 	GameTeamData[AWAY_TEAM_SLOT].TeamNumber = AwayTeam.Team;

	DoFrameLoop (1,UpdateGraphics);

	FreeShellGraphics ();

	FreeFonts ();
	FreeBackgrounds ();

	FreeFileData ((char*)mfl);

	return ret;
}

////////////////////////////////////////////////////////////////////////////////
//
// Pad Select Screen
// this assumes shell & teamgraphics multifiles are present
//

int DoPadSelectScreen ()
{
	int ret;
	ShellObj **pads;
	int padx[8];						// save pad unselected position
	int pady[8];
	int padteam[8];					// -1=left 0=not selected 1=right
	int padaway,padhome;		// pads per team
	int padpick;						// bit for each pad 1=on a team
	int padstart;						// bit for each pad 0=start hit
	int away,home;					// for temp goo
	int n;


	FreeGraphics ();
	InitShellGraphics ();

	pads = LoadObjects (ShellData,GetMultiFilePtr (ShellData,"padsel.mdf") );

	LoadBackgroundCD (0,"/shells/tsel/slctpsx.tim");
	DrawSync(0);

// Load Fonts

//	InitFonts ();
//	Register_Font_Cluts ();
//	LoadFont ("fontdpc2.tim", 3);

	DrawHomeName ();		// turn on names
	DrawAwayName ();

	FlushPads ();
	JpadRepeatFreq=-1;

	for (n=0; n<8; n++)
	{
		padteam[n]=0;
		padx[n]=pads[n]->X;		// save original x,y pos of all pads
		pady[n]=pads[n]->Y;
	}
	padaway=padhome=0;
	padstart=padpick=0xFF;

	ret=1;

	while (padstart & padpick)
	{
 		if (AnyPadData & JPADtriangle)
 		{
 			ret=-1;		// back out
 			break;
 		}

		padpick=0;
		for (n=0; n<8; n++)		// turn on the valid pads
		{
			if (JpadID[n])
			{
				SetObjectRGB (pads[n],0x80,0x80,0x80);
	
				padpick |= (1<<n);
				switch (padteam[n])
				{
					case 0:		// center, no selection
					  if ((JpadTrig[n] & JPADLleft ) && (padhome < MAX_USERS_PER_TEAM)) { padteam[n]=-1; padhome++; }
						if ((JpadTrig[n] & JPADLright) && (padaway < MAX_USERS_PER_TEAM)) { padteam[n]= 1; padaway++; }
						break;
					case -1:	//  left, home team
						if (JpadTrig[n] & JPADLright) { padteam[n]=0; padhome--; }
						break;
					case  1:	// right, away team
						if (JpadTrig[n] & JPADLleft ) { padteam[n]=0; padaway--; }
						break;
				}
				if (JpadTrig[n]) padstart=0xFF;
				if (JpadTrig[n] & JPADx) padstart=0;		//padstart &= ~(1<<n);
				if (!padteam[n]) padpick &= ~(1<<n);
			}
			else
			{
				SetObjectRGB (pads[n],0x40,0x40,0x40);

				if (padteam[n]== 1) padaway--;
				if (padteam[n]==-1) padhome--;
				padteam[n]=0;
				padstart |= (1<<n);
			}
		}

		if (!padpick && (padstart==0xFF)) padpick=0xFF;		// if no pads picked any pad can start cpu vs cpu

		away=home=0;
		for (n=0; n<8; n++)		// position the pads on screen and place in boxes if selected
		{
			switch (padteam[n])
			{
				case 0:		// unselected, draw at original place
					SetObjectCenterPos (pads[n],padx[n],pady[n]);
					break;
				case -1:	// left/home selected, draw in box
					SetObjectCenterPos (pads[n],pads[OBJ_HOMEBOX+home]->X,pads[OBJ_HOMEBOX+home]->Y);
					home++;
					break;
				case  1:	// right/away selected, draw in box
					SetObjectCenterPos (pads[n],pads[OBJ_AWAYBOX+away]->X,pads[OBJ_AWAYBOX+away]->Y);
					away++;
					break;
			}
		}

		SpinHomeHelmet ();
		SpinAwayHelmet ();

		DoFrameLoop (1,UpdateGraphics);		// Pads also read by frame process
	}

	if (ret==1)
	{
		for (n=0; n<8; n++) if (!padteam[n]) pads[n]->Flags &=~OBJ_DISPLAY;	// turn off unused pads
		DoFrameLoop (1,UpdateGraphics);

		away=home=0;		// stuff pads for game
		for (n=0; n<MAX_USERS_PER_TEAM; n++)
		{
			GameTeamData[HOME_TEAM_SLOT].ControllerNum[n]=-1;
			GameTeamData[AWAY_TEAM_SLOT].ControllerNum[n]=-1;
		}
		for (n=0; n<8; n++) switch (padteam[n])
		{
			case -1:
				GameTeamData[HOME_TEAM_SLOT].ControllerNum[away++]=n;
				break;
			case  1:
				GameTeamData[AWAY_TEAM_SLOT].ControllerNum[home++]=n;
				break;
		}
	}

	while (!AlignHelmets ()) DoFrameLoop (1,UpdateGraphics);
	DoFrameLoop (2,UpdateGraphics);

	FreeShellGraphics ();

	FreeFonts ();
	FreeBackgrounds ();

	return ret;
}

// end of tselshl.c //

