/**************************************************************


					FBMOVE.C

	INITIAL AUTHOR: M. Knauer       Start Date: 2/9/95

	Modify Date:


	Copyright 1995  GameTek, Inc.



	This file contains low-level movement and collision detection
	routines for the football players.


	NOTE: All AI routines in the main AI loop refer to the
	"CURRENT PLAYER" (usually the loop counter of the AI loop 0-21 -
	often refered to by variable "gPNum")

	Any references to the players Data Stuctures are "hidden" in
	a macro starting wtih the initials CP. This allows us to
	more freely change the data structure (if needed) from the initial
	non-linked array based DS's (easier to debug) to a structure based
	data structure

	E.g.  CPXPos =  PlayerXPosition[gPNum]     (current as of 5/19/95)
		could be changed to
		 CPXPos =  Player[gPNum].XPosition
		or something like
		 initialize  	CP = Player[gPNum]
		 in macro		CPXPos =  CP->XPosition

**************************************************************/

	#include  "fbequ.h"
	#include  "fbRAM.h"			/* external VARs needed by this file	*/
	#include  "FBPlays.h"		/* Script enums					*/
	#include  "FBforms.h"		/* Script enums					*/
	#include  "sound.h"		/* SFX enums					*/
//	#include  "sound1.h"		/* SFX enums					*/
	#include  "fbbutton.h"		/* SFX enums					*/
	#include  "FBpenal.h"		/* Penalty enums & protos				*/

#ifdef __psx__
	#include "debug.h"		// CGD for Cerror()
#endif	

/**  following used as conditional compile	*****/

#define	ERROR_CORRECTION	TRUE

/*------------------ MACROS  ------------------------*/

#define   SetAI(xyz)  CPAIState = (xyz)

#define PERCENT50   ((random0_99) < 50)
#define PERCENT25   ((random0_99) < 25)


#define	NORMAL_2ND_STATE_TICKS	4


#define PLAYER_ACTION(functionname) 	void functionname(void)
//#define PLAYERACTION(functionname) 	void functionname(void)

#define FRAME_RATE_20H

#ifdef  FRAME_RATE_20H
#define HELLO 	1
#endif
/*---------------  function prototypes  -----------*/

WORD  TryManyMoves(UWORD PossibleDirToMove);
WORD  TryMove(UWORD PossibleDirToMove,WORD speedtablevalue);
WORD  TryMoveDelta(void);	// for 360 degree movement - dives
//WORD  TryMove360(void);	// for 360 degree movement - dives
void PushOtherGuy(WORD guyhit, WORD PushSpeedFactor);
BOOL	RunToTargetSub(void);
void TryManyMovesOrInit2nd(void); // in fbdefai.c

void  MakeTargetInBoundsNS(void);
void  GeneratePlayerSphereOfInfluence(UWORD PNum);

WORD  IncrementPlayerSpeed(void);
WORD DecrementPlayerSpeed(WORD DecelFactor,WORD minspeed);
WORD  FigureMaxSpeed(void);

void  FaceBallandShuffle(void);
void  Run(void);
void  Walk(void);
void  Shuffle(WORD shuffletype);	/* a STATE subroutine	*/
void  InitGraphics(WORD graphicsnum);
BOOL  InitGraphicsQ(WORD AIgraphicsnum);	// a Question with an answer
void  InitGraphicsRandom(WORD AIgraphicsnum);

BOOL  ReachedTarget(void);   	// keep player in bounds
BOOL  ReachedTargetOOB(void);	// okay if player goes OOB

WORD  CheckForTackleAndFumble(WORD GuyHit);
void  SetNewBallCarrier(WORD NewBallCarrier);

//void  GoToNextState(WORD  ParamIncValue);

void Init2ndState(WORD tickstostayinstate);
void Do2ndState(void);

BOOL DefShouldGoTo4PointStance(void);

void DoState(void);
void SetPlayState(WORD PStateNum);	//could be used anywhere

void SetCPRatings(WORD extraratings);	//sets gPNum's constant ratings

//BOOL IsItaTD(void);
BOOL ShouldFumbleHappenOnTackle(void);
BOOL FumbleShouldHappenOnHike(WORD possiblefumbler);
BOOL HeShouldFumbleGeneric(WORD possiblefumbler);
void InitFumble(WORD fumbletype,WORD DirToGo,WORD NeedSFX);
void InitBallFumble(WORD fumbletype,WORD DirToGo);

BOOL TackleShouldHappenNow(void);
void SetReactTime(WORD timeuntilreaction);

void ReleasePass(BOOL TruePass);	/*  called by QB ACTION	   */
void InitPlayerPass(voidFCNPTR action);
void NormalPassingAction(void);
void CheckForReceiverContact(WORD guyhit);

void InitDefensePreSnapShift(void);
void DoDefPreSnapAnim(void);	//if necessary - pointing or hand waving

void InitLongHike(void);
void StartDive(BOOL startthisframe);
void SetDeltaParameters(UWORD tx, UWORD ty, WORD speed0_20);

BOOL SwapControllers(WORD newplayer, WORD oldplayer);

void CheckForPersonalPlayerFoul(void);	// based on team penalty rating or HUMAN

void RunInPlace(void);
void SetLookToPassGraphics(WORD dirtoface);
void FaceDownFieldRandom(void);

void  	StartTackle(void);
BOOL HumanEffectValid(WORD guy);

void TestDir(void);
UWORD FindDirection(WORD DeltaX, WORD DeltaY);
WORD Distance(WORD x, WORD y);
WORD Distance2Players(WORD p1, WORD p2);

BOOL	TurnPlayer(WORD FinalDirection, WORD WayToTurn);
BOOL	TurnPlayerOptimal(WORD FinalDirection);

WORD	AIPlayEffectCheck(WORD sfxnum);

void	NoAction(void);		/*  a NULL action	*/
void	NoAI(void);			/*  a NULL action	*/

void TurnOffKickInterface(void);		//make sure off if EOQ or Delay
void QuickFigurePass(WORD pnum);
void InitFreezeQBInHandoff(void);
void DefenseAudioForRunPlay(void);

/**********  A.I.  STATES	*************************/

void	FutureBCLookForBall(void);
void	FutureBCLookForBallPitch(void);
void	StopAfterHandOff(void);		/* an AI State used by QB	*/
void   FakeHandoff(void);	// used by QB in PlayAction passes



BOOL	ICanGetBall(void);
BOOL	HeCanGetBall(WORD pnum);	  /*  Can "He" (pnum) get ball ??? 	*/


void SetCPAction(voidFCNPTR action);


/**********  A.I.  ACTIONS	*************************/

void	HumanPassingAction(void);
void	GoTo3PtStanceInitAction(void);
//void	GoTo3PtStanceAction(void);
void	GoTo4PtStanceInitAction(void);
//void	GoTo4PtStanceAction(void);
void	DiveAction(void);
void 	BeTacklerAction(void);
void	GetUpAction(void);
void 	FreezeActionOnGround(void);
void 	LayOnGroundGroundAction(void);
void	FreezeAWhileAction(void);	// an ACTION
void	PuntAction(void);
PLAYER_ACTION(QBCallPlays);		// head turn at LOS



/******* Ball Actions	*********************/

void BallPreSnap(void);
void PlayerHasBall(void);
void BallAfterTackle(void);     		/*  a BALL ACTION		   */
void BallBeingPassed(void);
void InitBallBeingPassed(void);
void InitBouncingBall();
void BallLongHikeAction(void);

/*--- ball action subroutines	----*/
BOOL FigurePassHeight(void);
void MoveBallInAir(void);
void FigureBallSphereOfInfluence(void);
void UpdatePlayerWithBall(void);


/*************************   EXTERNAL   TABLES, VARIABLES  ********/

WORD	NumDefPresnapAnims;

//extern	UWORD	NumFramesPerPlayerGraphic[];
//extern  WORD  *All_Scripts[];

extern  const struct State  AllStates[];

extern  struct formation *AllFormations[];




#ifdef __MSDOS__
//	#include  <stdio.h>
//	#include  <stdlib.h>		/* for rand	*/
//	#include  <conio.h>
extern WORD DisplayLevel;
#endif

#ifdef PC_GRAPHIC_MODE
void DrawCross(WORD x,WORD y, WORD color);
#endif


#define	TURNTORIGHT		1
#define	TURNTOLEFT	   -1



#ifdef	DEBUGMODE

UWORD tx[16] = {100,150,200,200,200,200,200,150,100,050,000,000,000,000,000,99};
UWORD ty[16] = {000,000,000,050,100,150,200,200,200,200,200,150,100,050,000,000};
//UWORD ty[16] = {001,001,001,051,101,151,201,201,201,201,201,151,101,051,001,001};


void TestDir(void)
{
//WORD i,dx,dy,dir;
EFFVAR i,dx,dy,dir;

	for(i=0; i<16; i++)
	{
		dx = tx[i] - 100;
		dy = ty[i] - 100;
//		dir = FindDirection(dx, dy);
		dir = FindDirection2Points(tx[i],ty[i],100,100);
		DEBUG(i);
		TRACE(tx[i]);
		TRACE(ty[i]);
		TRACE(dir);
		dy = dir;		/* i hate complier warnings	*/
	}
}

#endif

#define MAX_CELEBRATIONS	3
WORD Celebrations[MAX_CELEBRATIONS] = {
	AI_TDCELEBRATE0_NUM,
	AI_TDCELEBRATE1_NUM,
	AI_TDCELEBRATE2_NUM,
};


/************************************************************************

					 E R R O R      S T A T E

	Since PSX pads to 4 bytes, a SCRIPT of 3 STATES (with no  parameters
	will pad the last 2 bytes with ZEROS, therefore an extra GoToNextState
	will use the zeros - to trap this, STATE #0 IS now the ERROR state.

************************************************************************/

STATEDEF(ErrorState)	// this is STATE #0
{
#ifdef PC_ALWAYS_MODE
	printf("Player %d in ERROR STATE",gPNum);
#endif

#ifdef PSX_MODE
	Cerror();
#endif

}


BOOL ImOnHomeTeam(void)
{	// Home team is slot 0 in GameTeamData[x]
	return( (gPNum<11?OffensiveSlotNum:DefensiveSlotNum) == 0);
}


/**********************************************************************

			 R U N     T O      R E L A T I V E


  References global main AI loop variable CURRENTPLAYER(CP) via a MACRO

	Note: allows
-----------------------------------------------------------------------*/

#define RTR_DIR_PARAM 1
#define RTR_X_PARAM 2
#define RTR_Y_PARAM 3

void RunToRelSub(void);


STATEDEF(RunToRel)

//void  RunToRel(void)
{
WORD  *scriptptr;   // temp
//WORD	yoff,dir;
EFFVAR	yoff,dir;


	if(!CPStateInit)
	{   /* init target to run to  */

		scriptptr =  CPScriptPtr;
		CPTargetX = CPXPos + *(scriptptr+RTR_X_PARAM);
		yoff = *(scriptptr+RTR_Y_PARAM);
		dir = *(scriptptr+RTR_DIR_PARAM);
		if(GameTeamData[CurTeamSlot].PlayFlipped)
		{
			yoff = -yoff;
			dir = FlipDir[dir];
		}
		CPTargetY = CPYPos + yoff;
//		CPFacing = CPDirToGo = dir;
		CPDirToGo = dir;  // don't set facing - turn optimal should handle

		CPStateInit = TRUE;
		MakeTargetInBoundsNS();
	}

	 RunToRelSub();
}



STATEDEF(RunToRelRandomX)	// used for outside guys on punt or Kickoff

//void  RunToRel(void)
{
WORD  *scriptptr;   // temp
//WORD	yoff,dir;
EFFVAR	yoff,dir;


	if(!CPStateInit)
	{   /* init target to run to  */
		scriptptr =  CPScriptPtr;
		CPTargetX = CPXPos + *(scriptptr+RTR_X_PARAM) + (rand()&(8*YARD-1));
		yoff = *(scriptptr+RTR_Y_PARAM);
		dir = *(scriptptr+RTR_DIR_PARAM);
		if(GameTeamData[CurTeamSlot].PlayFlipped)
		{
			yoff = -yoff;
			dir = FlipDir[dir];
		}
		CPTargetY = CPYPos + yoff;

//		CPFacing = CPDirToGo = dir;
		CPDirToGo = dir;	// don't set facing - turn optimal should handle

		CPStateInit = TRUE;
		MakeTargetInBoundsNS();
	#if 0
		DEBUG(RuntoRelInit);
		TRACE(gPNum);
		TRACE(CPTargetY);
	#endif
	}

	 RunToRelSub();
}

/*************************************************************************

				 B A C K    P E D A L

	Setting the BackPedal Bit Flag in PlayerFlags does two things:
		1) Player can only move at half his max speed
		2) the "Run()" subroutine will reposition the direction he's facing
			for correct graphics display.


	WARNING: BackPedal Bit Flag in PlayerFlags is RESET everytime
		a player's STATE changes (in "GoToNextState(x)").


*************************************************************************/


STATEDEF(RunBackwardsState)
{
/*	11/2/96 - actually RUN backwards, not back pedal
	(no BP for offensive guys except QB)	***/

	if(gPNum == QB && gPNum == BallCarrier)
	{
		Do_Back_Pedal_Macro;
	}
	else
	{
		SetBit(CPFlags,I_RUN_BACKWARD_BFLAG);
	}
// call the RunToRel STATE which will use the passed parameters to this STATE
	RunToRel();

}



/*--------------------------------------------------------------

		Run to Relative to Starting Ball Location

  References global main AI loop variable CURRENTPLAYER(CP) via a MACRO

--------------------------------------------------------------*/
STATEDEF(RunToRelSB)
{
WORD  *scriptptr;   // temp
//WORD	yoff,dir;
EFFVAR	yoff,dir;

	if(!CPStateInit)
	{   /* init target to run to  */
		CPStateInit = TRUE;

		scriptptr =  CPScriptPtr;
		CPTargetX = StartBallX + *(scriptptr+RTR_X_PARAM);
		yoff = *(scriptptr+RTR_Y_PARAM);
		dir = *(scriptptr+RTR_DIR_PARAM);
		if(GameTeamData[CurTeamSlot].PlayFlipped)
		{
			yoff = -yoff;
			dir = FlipDir[dir];
		}
		CPTargetY = StartBallY + yoff;
//		CPFacing = CPDirToGo = dir;
		CPDirToGo = dir;	// don't set facing - turn optimal should handle

		MakeTargetInBoundsNS();

		if(PlayStateNum < PS_PlayHappeningNUM && ReachedTarget())
		{ // if continual pick audibles pre snap, would move 1 step every time
			GoToNextState(4);	// bug # 3157
			return;
		}
	}


	RunToRelSub();
}

/*--------------------------------------------------------------
   called after initing target in regular or Starting Ball states

	References CURRENTPLAYER

---------------------------------------------------------------*/

void RunToRelSub(void)
{
//WORD i;
EFFVAR	i;

  #if	0
	DEBUG(RuntoRel);
	TRACE(gPNum);
  #endif

	if((i = TryManyMoves(CPDirToGo)) == NO_COLLISION)
	{
		Run();
		if(ReachedTarget())
			GoToNextState(4);
	}
	else if (i == CANT_MOVE)  /* couldn't move in one of 4 primary directions   */
	{
		Init2ndState(NORMAL_2ND_STATE_TICKS);

	}
}


/*********************************************************

	Following used by guys leaving huddle and running to L.O.S.

	Target (actual formation start) set up in InitTeamAI().

		NO COLLISION DETECTION !!!!
		USES  DELTA  Movement method

********************************************************************/
STATEDEF(HuddleRunToLOSState)	// or to sidelines
{
/*  NOTE: target set up in InitAI() based on formation	*/
//WORD speed;
EFFVAR	speed;

	if(CPFlags1 & I_HAVE_BEEN_HCONTROLLED_BFLAG)	// defense only
	{
		GoToNextState(1);	// bug #1433
		return;
	}


 /* delta method - less twiching when going to line	*/
	if(PlayType == KICKOFF_PLAY && gPNum < 11)	// bug #1799
	{ // offense on K.O.
		InitGraphicsRandom(AI_RUNNING_NUM);	// bug # 2554
		speed = TO_LOS_KO_SPEED;
	}
	else if(	   (CPTargetY > StartBallY + 10*YARDS)
		|| (CPTargetY < StartBallY - 10*YARDS)
		|| PlayType == KICKOFF_PLAY
	)
	{
		InitGraphicsRandom(AI_RUNNING_NUM);	// bug # 2554
		speed = TO_LOS_FAST_SPEED;
	}
	else
	{
		speed = TO_LOS_SLOW_SPEED;
		SetBit(CPFlags,FORCE_WALK_BFLAG);
		InitGraphicsRandom(AI_WALKING_NUM);	// bug # 2554
	}

	if(!CPStateInit)
	{
		CPStateInit = TRUE;



		CPDirToGo = CPFacing = FindCPDirectionToTargetMACRO;
		SetDeltaParameters(CPTargetX,CPTargetY,speed);	// figure dx,dy for 360 degree diving
//		InitGraphicsRandom(AI_RUNNING_NUM);	// bug # 2554
//		InitGraphicsRandom(AI_WALKING_NUM);	// bug # 2554

	}


	CPXPos += CPDeltaX;
	CPYPos += CPDeltaY;

	CPSpeed = 12<<SPEED_SHIFT;	// makes the RUN a run, not a walk(2ticks/frame update)
	Run();

	GeneratePlayerSphereOfInfluence(gPNum);

	if(ReachedTargetOOB())		// okay to go Out Of Bounds
	{
		CPXPos = CPTargetX;	  // make the current EXACTLY what the target is
		CPYPos = CPTargetY;
		GeneratePlayerSphereOfInfluence(gPNum);
		GoToNextState(1);
	}
}





STATEDEF(DefAtLOS)
{
	No_Back_Pedal_Macro;
	InitGraphics(AI_2PTSTANCE_NUM);
	CPDirToGo = CPFacing = WEST_DIR;	//dirotgo added 2/11/96 bug #1245
	GoToNextState(1);
}

STATEDEF(OffAtLOS)
{
	InitGraphics(AI_STAND_NUM);
	CPFacing = EAST_DIR;
	GoToNextState(1);
}


#if 1
BYTE SLRunTable[2][2] = {

	TRUE,	//Home,offdir=0
	FALSE,	//Home,offdir=1
	FALSE,	//Away,offdir=0
	TRUE,	//Away,offdir=1

};


BOOL OffRunToNorthSideLines(void)
{   //
EFFVAR HorA;


	HorA = (OffensiveSlotNum == HOME_TEAM_SLOT?0:1);
	return(SLRunTable[HorA][OffenseDir]);
}



#else
BYTE QuarterConvertTable[6] = {0,0,1,1,0,0};

BYTE SLRunTable[2][2][2] = {

	FALSE,	//Q=0,1,4,5,Home,offdir=0
	TRUE,	//Q=0,1,4,5,Home,offdir=1
	TRUE,	//Q=0,1,4,5,Away,offdir=0
	FALSE,	//Q=0,1,4,5,Away,offdir=1

	TRUE,	//Q=2,3,Home,offdir=0
	FALSE,	//Q=2,3,Home,offdir=1
	FALSE,	//Q=2,3,Away,offdir=0
	TRUE,	//Q=2,3,Away,offdir=1

};
BOOL OffRunToNorthSideLines(void)
{   //
EFFVAR HorA,index;


	HorA = (OffensiveSlotNum == HOME_TEAM_SLOT?0:1);
	index = QuarterConvertTable[TheQuarter];
	return(SLRunTable[index][HorA][OffenseDir]);
}

#endif



STATEDEF(RunToSideLinesState)
{
//UWORD y;
UEFFVAR	y;

	if(!CPStateInit)
	{

		y = NORTH_SIDELINE-3*YARDS;	//assume

		if(gPNum < START_DEF_SLOT_NUM)	// player on offense???
		{ // offensive runs to north if Offdir = 0
//			if(OffenseDir)	// bug #2368
//			if(OffensiveSlotNum == AWAY_TEAM_SLOT)
			if(!OffRunToNorthSideLines())
			{
				y = SOUTH_SIDELINE+1*YARDS;
			}
			CPTargetX = gPNum*YARDS+DEF_GOAL_LINE+45*YARDS;
		}
		else
		{ // defensive runs to south if offdir = 0
//			if(OffenseDir == ZERO ) // bug #2368
//			if(DefensiveSlotNum == AWAY_TEAM_SLOT)
			if(OffRunToNorthSideLines())
			{  // if off. run north, def. must go south
				y = SOUTH_SIDELINE+1*YARDS;
			}
			CPTargetX = gPNum*YARDS+DEF_GOAL_LINE+34*YARDS;
		}

		CPTargetY = y + (rand()&511);	// 0 -2 yards so they're not "perfect"

		CPDirToGo = CPFacing = FindCPDirectionToTargetMACRO;
//		CPStateInit = TRUE; let next sub set StateInit = TRUE
	}

/* okay to call STATE as a STATE sub as long as same number of parameters */

	HuddleRunToLOSState();	// will GoToNextState when reached target

}



STATEDEF(ReachedSideLinesState)
{
// face toward center of field

	if(TurnPlayerOptimal(gPNum < START_DEF_SLOT_NUM ? SOUTH_DIR : NORTH_DIR))
	{
		 Shuffle(NORMAL_SHUFFLE);
	}
	else
		InitGraphics(AI_STAND_NUM);

//	stay here forever

}



/********************************************************************

					C O I N    T O S S   STUFF

********************************************************************/
#if 0
STATEDEF(InitAtSideLinesState)
{
//WORD randnum;
EFFVAR	randnum;

	randnum = (rand()&511);		// 0 -2 yards

	if(gPNum < START_DEF_SLOT_NUM)
	{ // offensive on north
		CPTargetY = NORTH_SIDELINE-3*YARDS+randnum;
		CPTargetX = gPNum*YARDS+DEF_GOAL_LINE+45*YARDS;
		CPDirToGo = CPFacing = SOUTH_DIR;
	}
	else
	{ // defensive on south
		CPTargetY = SOUTH_SIDELINE+1*YARDS+randnum;
		CPTargetX = gPNum*YARDS+DEF_GOAL_LINE+34*YARDS;
		CPDirToGo = CPFacing = NORTH_DIR;
	}
	GoToNextState(1);

}


STATEDEF(CoinTossToHuddle)
{
//	if(CoinTossDone)
	{



	}

}

#endif


/********************************************************************

			S E C O N D A R Y      S T A T E

	If a computer controlled player cannot move in one of his primary
	directions then he goes into a secondary state that will move in
	in a different direction for a number of ticks (AI frames);

**********************************************************************/


void	Init2ndState(WORD tickstostayinstate)
{

 /* give one of two random directions to go based on where he wants to go*/
 	CPDirToGo += ((PERCENT50)? 6:-6);
	CPDirToGo &= (MAX_DIR-1);

	CPStateInit = ZERO;	/* re-init normal state	when done with this*/

	CPState2Count = tickstostayinstate;

	Do2ndState();
}

/*------------------------------------------------------------------*/

void	Do2ndState(void)
{
	DEBUG(In2nd state);
	TRACE(gPNum);
	if(TryManyMoves(CPDirToGo) == NO_COLLISION)
	{
		Run();
	}
	else
		Shuffle(NORMAL_SHUFFLE);

	CPState2Count--;

/*  do nothing if can't move in secondary state	*/
}

/*-----------------------------------------------------------------
			MAKE  TARGET   IN  BOUNDS

	Will make sure target stays in the field of play.

	USES:  gPNum

	WARNING: Field Postions are all UNSIGNED  WORDS - NO NEGATIVES !!!

-----------------------------------------------------------------*/

void  MakeTargetInBoundsNS(void)
{


	if(CPTargetY < NORTH_SIDELINE)
		CPTargetY = NORTH_SIDELINE+CLOSE_TOSL;

	else if(CPTargetY > SOUTH_SIDELINE)
		CPTargetY = SOUTH_SIDELINE-CLOSE_TOSL;



}


/******************************************************************

					TRY   MANY   MOVES



	Will try to move a player in a direction first by 1) seeing
	if okay to move directly in that direction. If not, then 2)try
	the two diagonals. If still can't move, 3) try a perpendicular.
	Doesn't use minor diagonals (e.g. NNE, SSW)

	These directions are in a table with highest priority direction
	at end of table.
	In a choice between diagonals or perpend. - first check the
	"upfield" direction.

	Will do Collision detection on all other players for every
	possible move (everytime "TryMove" is called).

	If he can't move in any of the 4 directions, go to a secondary
	state which moves the player in an opposite direction for a certain
	number of frames.

	INPUTS:   WORD -  direction you want to move

	RETURNS:	WARNING - be careful how you handle these return values!!!

	NO_COLLISION or CANT_MOVE or TACKLE_STARTED
	  following values used as return values for collision detect routines
	if number is positive then that is the number of the guy collided
	with, else there will be one of the following negative numbers.

	NO_COLLISION means that the players X,Y have been changed and it's
	up to the calling routine to do the grahics.
	CANT_MOVE means that the player couldn't move in one of 4 directions
	 (if "TryManyMoves()", so the player most likely should be put
	 into a "secondary STATE".
	Anything else (e.g. "TACKLE_STARTED" means the graphics and
	movement have been taken care of so do nothing.


	****WARNING**
	this is mainly used in offensive player script,as it tries to
	avoid collisions.  Def players should use "TryManyMovesPush"

-------------------------------------------------------------------*/

WORD TryManyMoves(UWORD PossibleDirToMove)
{
//WORD  i, posdir,sp;
EFFVAR	i, posdir,sp;
//WORD  guyhit,who;
EFFVAR	guyhit,who;


	sp = IncrementPlayerSpeed();

	for(i=3; i>=0; i--)
	{
		posdir = TryManyMovesTable[PossibleDirToMove][i];
		if((guyhit = TryMove(posdir,sp))
				 == NO_COLLISION) /* then the player can move here  */
		{
			if(!(CPFlags & IM_SPINNING_BFLAG) )
				TurnPlayerOptimal(PossibleDirToMove);	//use direction passed in
//			TurnPlayerOptimal(posdir);		// not the way he ended up going
			return(NO_COLLISION);
		}
		else if (guyhit == OUT_OF_BOUNDS)
			continue;
		else if((gPNum < 6 && guyhit <6)
//		{   /* if "key" guys on offense collide, slow the "lowest" one	*/
//			who =  min(gPNum,guyhit);
//			OPSpeed(who) /= 2;
//			OPAcceleration(who) = BASIC_ACCELERATION;
//		}
//		else if(gPNum >10 && guyhit >10)
		   || (gPNum >10 && guyhit >10))
		{     /* if "key" guys on defense collide, slow the slowest one	*/

			who =  min(gPNum,guyhit); //assume
			if(CPSpeed == OPSpeed(guyhit) )	// if equal use lowest
			{
//				OPSpeed(min(gPNum,guyhit)) /= 2;
			}
			else if(CPSpeed < OPSpeed(guyhit))
			{
				who = gPNum;		//CPSpeed /=2;
			}
			else
			{
				who = guyhit;
//				 OPSpeed(guyhit) /= 2;
			}
			OPSpeed(who) >>= 2;	// divide by 4
			OPAcceleration(who) = BASIC_ACCELERATION;

		}
		else  /* I hit somebody - should I tackle him?  */
			if  (CheckForTackleAndFumble(guyhit))
			{
	#if PC_ALWAYS_MODE  	// does this ever happen?? - once in 20 simed games 9/9/96
				//RunToBallTargetState & LBread
//				printf("TMM tackle - pnum = %d",gPNum);
//				getch();
	#endif
				return(TACKLE_STARTED);

			}
	}

	return(CANT_MOVE);


}





BOOL NoDefOffsideIfOptionOff(UWORD PossibleXPosition)
{		// def can't go offsides if that GameOptionis OFF
	// Saturn bug#193
return(   (PlayStateNum < PS_PlayHappeningNUM)
	   && (GameOptions[GAME_OPTNS_OFFSIDES] == OPTION_OFF)
	   && (gPNum > LAST_OFF_SLOT_NUM)	// on defense?
	   && (PossibleXPosition < StartBallX + NEUTRAL_ZONE_LENGTH)
	  );
}

/******************************************************************

					TRY   A   MOVE

	Will try to move "gPNum" in "PossibleDirToMove" based on players speed.
	(PCurSpeed tells how big of a "step" to take)

	SPEED should have already been inc'ed or dec'ed
	before calling this routine.

	NOTE: Speed passed to routine is a number 0 -15 for table lookup.

	Will check to see if "gPNum"s possible move will collide
	with any of the other players (in their "Sphere of Influence").
	If not, then update "gPNum"s X and Y.


	!!!!!!!!   DOES  NOT  UPDATE  GRAPHICS  !!!!!!!!!!

	USES:   gPNum

	RETURNS:  NO_COLLISON if didn't hit anybody (and updates PNums
				X, Y0
				else
				# of player collided with

----------------------------------------------------------------------*/
#define Y_CHECK_DIST (3*HALFYARD)

WORD TryMove(UWORD PossibleDirToMove,WORD speedtablevalue)
{

//register WORD  i;
register EFFVAR	 i;
//register UWORD PossibleXPosition, PossibleYPosition;
register UEFFVAR	PossibleXPosition, PossibleYPosition;
//WORD sorti,p;
EFFVAR	sorti,p;
//UWORD keysorty;
EFFVAR	keysorty;

//WORD stloop = 0;

#if 0		//  debug only !!!
	if(PossibleDirToMove > 15 || speedtablevalue > 15 || speedtablevalue < 0)
	{
		PossibleDirToMove  = PossibleDirToMove;	// set B.P. here
	}
#endif

	PossibleXPosition =	CPXPos + SpeedTable[speedtablevalue][PossibleDirToMove].X;
	PossibleYPosition =	CPYPos + SpeedTable[speedtablevalue][PossibleDirToMove].Y;


	if(CollisionDetectActive)
	{
	 /* do collision detect with all other players on field  */
	//	if(GenValue > 8 && gPNum < 11)	// TEST only !!!
	//		stloop = 11;
		if( !(PlayOverFlags & OKAY_TO_GO_OOB_BFLAG))
		{	// everybody can go OOB if Ball is OOB
			if( gPNum != BallCarrier &&
				(PossibleYPosition < NORTH_SIDE_LINE+1
					|| PossibleYPosition > SOUTH_SIDE_LINE-1))
				return(OUT_OF_BOUNDS);

			if(PossibleXPosition > OFF_END_LINE-1)
				return(OUT_OF_BOUNDS_EAST);


		}
#ifdef PC_DEBUG_MODE
		else
		{
			if(CPYPos < NORTH_SIDELINE)
				i = 1;	// test only set BP here
		}
#endif
/*		// def can't go offsides if that GameOptionis OFF
		if(PlayStateNum < PS_PlayHappeningNUM)
		{
			if( (GameOptions[GAME_OPTNS_OFFSIDES] == OPTION_OFF)
			   && (gPNum > LAST_OFF_SLOT_NUM)
			   && (PossibleXPosition < StartBallX + NEUTRAL_ZONE_LENGTH)
			  )
			{
				i= 1;
				return(OUT_OF_BOUNDS);
			}
		}	*/

		// def can't go offsides if that GameOptionis OFF
		if(NoDefOffsideIfOptionOff(PossibleXPosition))
		{
			i= 1;
			return(OUT_OF_BOUNDS);
		}

/*
	idea here is not to check all 21 other players for collision,
	but to check based on a sorted list such that once I "miss" in
	one direction (say less than), I don't have to check any more
	less thans, so I then do the same for the greater thans,

	be sure to open up a big enough variance such that even if closest player
	in Ysort moved before me and away form me, and 2nd closest in Ysort
	moved before me but toward me, ( and 2nd will Y swap with 1st
	next time thru) that i get all
*/
		sorti = OPYSortPos(gPNum);	// my position in sorted table
		keysorty = PossibleYPosition - Y_CHECK_DIST;
		for (i = sorti-1; i > -1; i--)
		{
			p = PlayersYSortList[i];
			if(keysorty > OPYPos(p) )
				break;
			else
			{	// do collision on p
				if(  PossibleXPosition > PlayerSphereInfluence[p].Low.X
				 &&  PossibleXPosition < PlayerSphereInfluence[p].High.X
				 &&  PossibleYPosition > PlayerSphereInfluence[p].Low.Y
				 &&  PossibleYPosition < PlayerSphereInfluence[p].High.Y)
				return(p);    /*   this is the guy we just hit    */
			}
		}

		keysorty += 2*Y_CHECK_DIST;

		for (i = sorti+1; i < 22; i++)
		{
			p = PlayersYSortList[i];
			if(keysorty < OPYPos(p) )
				break;
			else
			{	// do collision on p
				if(  PossibleXPosition > PlayerSphereInfluence[p].Low.X
				 &&  PossibleXPosition < PlayerSphereInfluence[p].High.X
				 &&  PossibleYPosition > PlayerSphereInfluence[p].Low.Y
				 &&  PossibleYPosition < PlayerSphereInfluence[p].High.Y)
				return(p);    /*   this is the guy we just hit    */
			}
		}


#if 0

		for(i=stloop; i<NUM_PLAYERS_ON_FIELD; i++)
		{
			if(i != gPNum   /* don't check current player */
				 &&  PossibleXPosition > PlayerSphereInfluence[i].Low.X
				 &&  PossibleXPosition < PlayerSphereInfluence[i].High.X
				 &&  PossibleYPosition > PlayerSphereInfluence[i].Low.Y
				 &&  PossibleYPosition < PlayerSphereInfluence[i].High.Y)
			return(i);    /*   this is the guy we just hit    */
		}
#endif
	/*  if we made it thru loop then he didn't make contact with anyone
		so update his X, Y      */
	}

	CPXPos = PossibleXPosition;
	CPYPos = PossibleYPosition;
	GeneratePlayerSphereOfInfluence(gPNum);
	return(NO_COLLISION);

}





WORD TryMoveDelta(void)
{

//register WORD  i;
register EFFVAR	 i;
//register UWORD PossibleXPosition, PossibleYPosition;
register UEFFVAR	PossibleXPosition, PossibleYPosition;
//WORD sorti,p;
EFFVAR	sorti,p;
//UWORD keysorty;
EFFVAR	keysorty;

	PossibleXPosition =	CPXPos + CPDeltaX;
	PossibleYPosition =	CPYPos + CPDeltaY;


	if(CollisionDetectActive)
	{
	 /* do collision detect with all other players on field  */
		if( !(PlayOverFlags & OKAY_TO_GO_OOB_BFLAG))
		{	// everybody can go OOB if Ball is OOB
			if( gPNum != BallCarrier &&
				(PossibleYPosition < NORTH_SIDE_LINE+1
					|| PossibleYPosition > SOUTH_SIDE_LINE-1))
				return(OUT_OF_BOUNDS);

			if(PossibleXPosition > OFF_END_LINE-1)
				return(OUT_OF_BOUNDS);


		}

		// def can't go offsides if that GameOptionis OFF
		if(NoDefOffsideIfOptionOff(PossibleXPosition))
		{
			i= 1;
			return(OUT_OF_BOUNDS);
		}

/*
	idea here is not to check all 21 other players for collision,
	but to check based on a sorted list such that once I "miss" in
	one direction (say less than), I don't have to check any more
	less thans, so I then do the same for the greater thans,

	be sure to open up a big enough variance such that even if closest player
	in Ysort moved before me and away form me, and 2nd closest in Ysort
	moved before me but toward me, ( and 2nd will Y swap with 1st
	next time thru) that i get all
*/
		sorti = OPYSortPos(gPNum);	// my position in sorted table
		keysorty = PossibleYPosition - Y_CHECK_DIST;
		for (i = sorti-1; i > -1; i--)
		{
			p = PlayersYSortList[i];
			if(keysorty > OPYPos(p) )
				break;
			else
			{	// do collision on p
				if(  PossibleXPosition > PlayerSphereInfluence[p].Low.X
				 &&  PossibleXPosition < PlayerSphereInfluence[p].High.X
				 &&  PossibleYPosition > PlayerSphereInfluence[p].Low.Y
				 &&  PossibleYPosition < PlayerSphereInfluence[p].High.Y)
				return(p);    /*   this is the guy we just hit    */
			}
		}

		keysorty += 2*Y_CHECK_DIST;

		for (i = sorti+1; i < 22; i++)
		{
			p = PlayersYSortList[i];
			if(keysorty < OPYPos(p) )
				break;
			else
			{	// do collision on p
				if(  PossibleXPosition > PlayerSphereInfluence[p].Low.X
				 &&  PossibleXPosition < PlayerSphereInfluence[p].High.X
				 &&  PossibleYPosition > PlayerSphereInfluence[p].Low.Y
				 &&  PossibleYPosition < PlayerSphereInfluence[p].High.Y)
				return(p);    /*   this is the guy we just hit    */
			}
		}

	/*  if we made it thru loop then he didn't make contact with anyone
		so update his X, Y      */
	}

	CPXPos = PossibleXPosition;
	CPYPos = PossibleYPosition;
	GeneratePlayerSphereOfInfluence(gPNum);
	return(NO_COLLISION);

}




#if 0
WORD TryMoveOld(UWORD PossibleDirToMove,WORD speedtablevalue)
{

//register WORD  i;
register EFFVAR  i;
//register UWORD PossibleXPosition, PossibleYPosition;
register UEFFVAR PossibleXPosition, PossibleYPosition;
//WORD stloop = 0;
UEFFVAR	stloop = 0;

#if 0		//  debug only !!!
	if(PossibleDirToMove > 15 || speedtablevalue > 15 || speedtablevalue < 0)
	{
		PossibleDirToMove  = PossibleDirToMove;	// set B.P. here
	}
#endif

	PossibleXPosition =	CPXPos + SpeedTable[speedtablevalue][PossibleDirToMove].X;
	PossibleYPosition =	CPYPos + SpeedTable[speedtablevalue][PossibleDirToMove].Y;


	if(CollisionDetectActive)
	{
	 /* do collision detect with all other players on field  */
	//	if(GenValue > 8 && gPNum < 11)	// TEST only !!!
	//		stloop = 11;
		if( !(PlayOverFlags & BALL_OUT_OF_BOUNDS_BFLAG))
		{	// everybody can go OOB if Ball is OOB
			if( gPNum != BallCarrier &&
				(PossibleYPosition < NORTH_SIDE_LINE+1
					|| PossibleYPosition > SOUTH_SIDE_LINE-1))
				return(OUT_OF_BOUNDS);

			if(PossibleXPosition > OFF_END_LINE-1)
				return(OUT_OF_BOUNDS);


		}

		for(i=stloop; i<NUM_PLAYERS_ON_FIELD; i++)
		{
			if(i != gPNum   /* don't check current player */
				 &&  PossibleXPosition > PlayerSphereInfluence[i].Low.X
				 &&  PossibleXPosition < PlayerSphereInfluence[i].High.X
				 &&  PossibleYPosition > PlayerSphereInfluence[i].Low.Y
				 &&  PossibleYPosition < PlayerSphereInfluence[i].High.Y)
			return(i);    /*   this is the guy we just hit    */
		}
	/*  if we made it thru loop then he didn't make contact with anyone
		so update his X, Y      */
	}

	CPXPos = PossibleXPosition;
	CPYPos = PossibleYPosition;
	GeneratePlayerSphereOfInfluence(gPNum);
	return(NO_COLLISION);

}
#endif


#if 0
void TestCollision(void)
{
//WORD i,sorti,p;
EFFVAR	i,sorti,p;
//UWORD keysorty;
UEFFVAR	keysorty;

	sorti = OPYSortPos(gPNum);
	keysorty = CPYPos-YARD;

	for (i = sorti-1; i > -1; i--)
	{
		if(keysorty < OPYPos(p = PlayersYSortList[i]) )
			break;
		else
		{	// do collision on p
		}
	}

	keysorty += 2*YARD;

	for (i = sorti+1; i < 22; i++)
	{
		if(keysorty > OPYPos(p = PlayersYSortList[i]) )
			break;
		else
		{	// do collision on p
			p = p;
		}
	}

}

#endif


#if 0
WORD TryMove360(void)
{

//register WORD  i;
register EFFVAR  i;
//register UWORD PossibleXPosition, PossibleYPosition;
register UEFFVAR PossibleXPosition, PossibleYPosition;
//WORD stloop = 0;
EFFVAR	stloop = 0;

	PossibleXPosition =	CPXPos + CPDeltaX;
	PossibleYPosition =	CPYPos + CPDeltaY;


	if(CollisionDetectActive)
	{
	 /* do collision detect with all other players on field  */
		if( gPNum != BallCarrier &&
			(PossibleYPosition < NORTH_SIDE_LINE+1
				|| PossibleYPosition > SOUTH_SIDE_LINE-1))
			return(OUT_OF_BOUNDS);

		if(PossibleXPosition > OFF_END_LINE-1)
			return(OUT_OF_BOUNDS);

		for(i=stloop; i<NUM_PLAYERS_ON_FIELD; i++)
		{
			if(i != gPNum   /* don't check current player */
				 &&  PossibleXPosition > PlayerSphereInfluence[i].Low.X
				 &&  PossibleXPosition < PlayerSphereInfluence[i].High.X
				 &&  PossibleYPosition > PlayerSphereInfluence[i].Low.Y
				 &&  PossibleYPosition < PlayerSphereInfluence[i].High.Y)
			return(i);    /*   this is the guy we just hit    */
		}
	/*  if we made it thru loop then he didn't make contact with anyone
		so update his X, Y      */
	}

	CPXPos = PossibleXPosition;
	CPYPos = PossibleYPosition;
	GeneratePlayerSphereOfInfluence(gPNum);
	return(NO_COLLISION);

}

#endif



/******************************************************************

				 I N C    P L A Y E R     S P E E D

	Increments the "Current Players" speed.


	NOTE: Like YARDS, Speeds are shifted left to allow for
	different accelerations.

	A bit

	Returns a valid speed number 0-15 AND Updates Current players speed.

	NOTE:	Speed = 0 shouldn't do anything (but could map to speed 0)
			Speed 1-255 map to Speed 0
			Speed 256-511 --> speed 1


******************************************************************/
#define	MAX_ACCELERATION 0x1000


WORD IncrementPlayerSpeed(void)

{

//WORD posspeed,maxspeed;	//,flag;
EFFVAR	posspeed,maxspeed;	//,flag;

//	if(CPBurstSpeed != ZERO)
	if(CPFlags1 & BURST_ACTIVE_BFLAG)
		posspeed = CPSpeed + (CPAcceleration*4);	//faster accel. if bursting
	else
		posspeed = CPSpeed + CPAcceleration;

	maxspeed = FigureMaxSpeed();
	if(posspeed > maxspeed)
	{	// once reached max, reset accel to max (accel not max if collision)
		posspeed = maxspeed;
		CPAcceleration = MAX_ACCELERATION;
	}

	CPSpeed = posspeed;	// | flag;

	if(CPFlags & IS_HCONTROLLED_BFLAG)
		CPSpeed = posspeed = maxspeed;	// instant acceleration for human

	return(posspeed>>SPEED_SHIFT);	/* adjust to true speed integer	*/

}


/*-----------------------------------------------------------------

	RETURNS: -1 if player has Stopped else an index into Speedtable

	NOTE: Decelfactor should be between 1-5
			minspeed should be between 2-5
		Minspeed used so a blocker/blockee does not go to zero

-----------------------------------------------------------------*/

WORD DecrementPlayerSpeed(WORD DecelFactor,WORD minspeed)
{

//WORD rv,posspeed;
EFFVAR	rv,posspeed;

	if(DecelFactor != HUMAN_ACCEL_DEC)
	{		// will reset when reach max speed
		CPAcceleration = BASIC_ACCELERATION;
	}

	minspeed <<= SPEED_SHIFT;	// adjust to true WORD value

	if(CPSpeed < minspeed)	// if already below min speed
		minspeed = ZERO;		// then ignore min

	posspeed = CPSpeed - (DecelFactor * CPAcceleration);

	if(posspeed < minspeed)
	   posspeed = minspeed;

	if(posspeed <= ZERO)
	{
		posspeed = ZERO;
		rv = -1;
	}
	else
	{
		rv = posspeed>>SPEED_SHIFT;	/* adjust to true speed integer	*/
	}

	CPSpeed = posspeed;

	return(rv);
}


/**********************************************************************/
#if 1		// new table 6/1/96 - human can do partial bursts
WORD BurstSpeedTable[] = {    // revised 10/8/96
	0,				// index 0 not used
					//now  10 ticks  = 1/2 second
	3<<SPEED_SHIFT,
	5<<SPEED_SHIFT,
	5<<SPEED_SHIFT,
	5<<SPEED_SHIFT,
	5<<SPEED_SHIFT,

	5<<SPEED_SHIFT,
	5<<SPEED_SHIFT,
	5<<SPEED_SHIFT,
	5<<SPEED_SHIFT,
	5<<SPEED_SHIFT,

//	4<<SPEED_SHIFT,
//	3<<SPEED_SHIFT,
   -1,					// last entry must be negative !!!
};
/*               // new table 6/1/96 - human can do partial bursts
WORD BurstSpeedTable[] = {
	0,				// index 0 not used
					//  10 ticks  = 1/2 second
	2<<SPEED_SHIFT,
	2<<SPEED_SHIFT,
	4<<SPEED_SHIFT,
	4<<SPEED_SHIFT,
	4<<SPEED_SHIFT,

	4<<SPEED_SHIFT,
	4<<SPEED_SHIFT,
	3<<SPEED_SHIFT,
	3<<SPEED_SHIFT,
	3<<SPEED_SHIFT,

   -1,					// last entry must be negative !!!
};	*/

#else

#define MAX_BURST_TICKS 16
WORD BurstSpeedTable[MAX_BURST_TICKS] = {
	0,				// index 0 means not happening
	1<<SPEED_SHIFT,
	2<<SPEED_SHIFT,
	3<<SPEED_SHIFT,
	4<<SPEED_SHIFT,
	4<<SPEED_SHIFT,
	4<<SPEED_SHIFT,
	4<<SPEED_SHIFT,

	3<<SPEED_SHIFT,
	2<<SPEED_SHIFT,
	1<<SPEED_SHIFT,
	0<<SPEED_SHIFT,
   -(1<<SPEED_SHIFT),
   -(2<<SPEED_SHIFT),
   -(2<<SPEED_SHIFT),
   -(1<<SPEED_SHIFT),
};

#endif
/*------------------------------------------------------------*/

WORD FigureMaxSpeed(void)
/* speeds are 0x000 (an actual move - NOT no move) to 0x0fff which scale
	to lookup values of 0-15   (MAX_NUM_SPEEDS = 16	as of 2/6/96	*/
{
//WORD maxsp, spdinc;
EFFVAR	maxsp, spdinc;


	maxsp = CPMaxSpeed;	//assumption - could be modified below

	if(CPFlags & BACK_PEDAL_BFLAG)
	{
		if(gPNum < 11)		//offense (QB) is faster
		{
//			maxsp /= 2;
			maxsp =  (maxsp*3)/4;	// QB drop back was too slow
			maxsp = max(maxsp,6<<SPEED_SHIFT);

		}
		else
		{
//			maxsp /= 3;	// was 2 but Bernie cried about it being too fast
			maxsp /= 2;	// sorry bernie
//			maxsp /= 4;	// sorry bernie
//			maxsp *= 3;	// sorry bernie	- 75% (10/3/96)
		}
//		maxsp = min(maxsp,3<<SPEED_SHIFT);
	}
	else if(CPFlags & FORCE_WALK_BFLAG)
	{
		maxsp /=4;
	}
	else if (CPFlags1 & BURST_ACTIVE_BFLAG)
	{
#if 1	//NEW_BURST

		spdinc = BurstSpeedTable[++CPBurstCounter];
		if(spdinc < ZERO)	// burst has ended
		{ // bursting over - max  speed is now always lower
			CPNumSpeedBursts++;		// finished this many bursts
			CPBurstCounter = ZERO;
			CPMaxSpeed -= (1<<SPEED_SHIFT);
			if(gPNum == BallCarrier)
			{	// make fatigue level lower
				CPMinMaxSpeed -= (1<<SPEED_SHIFT);
				if(CPMinMaxSpeed < (3<<SPEED_SHIFT))
					CPMinMaxSpeed = (3<<SPEED_SHIFT);
			}
			if(CPMaxSpeed < CPMinMaxSpeed)
				CPMaxSpeed = CPMinMaxSpeed;
		}
		else
			maxsp += spdinc;
#else
		maxsp += BurstSpeedTable[CPBurstCounter];

		if(++CPBurstCounter >= MAX_BURST_TICKS)
		{ // bursting over - max  speed is now always lower
			CPBurstCounter = ZERO;
			CPMaxSpeed -= (1<<SPEED_SHIFT);
			if(CPMaxSpeed < CPMinMaxSpeed)
				CPMaxSpeed = CPMinMaxSpeed;
		}
#endif

		if(maxsp > (MAX_NUM_SPEEDS<<SPEED_SHIFT)-1)		// 0xfff is okay
			maxsp = (MAX_NUM_SPEEDS<<SPEED_SHIFT)-1;

	}

	if(CPFlags & IM_SPINNING_BFLAG)
		maxsp -= (2<<SPEED_SHIFT);

	if (maxsp < (3<<SPEED_SHIFT))	// handle negative increments on SpeedBurst
		maxsp = (3<<SPEED_SHIFT);	// only go this slow

	return(maxsp);

}


void InitSpeedBurst(void)
{
#if 0
	if(CPBurstSpeed == ZERO)	// don't init if already bursting
	{
		CPBurstSpeed = 1;		// later used as index
		CPNumSpeedBursts++;
	}
#else
	/* new method - CPBurstCounter does NOT reset if re-inited
		this allows a human (or CPU) to do "partial" bursts
		once CPBurstCounter reachs max, then player loses
		1 from max speed (and CPNumSpeedBursts++)

****    WARNING  - the BURST_ACTIVE_BFLAG is RESET every Frame (in DOAction)
	*/
	SetBit(CPFlags1,BURST_ACTIVE_BFLAG);


#endif
}


/*********************************************************************
	Due to new Speedburst logic which clears the burst flag every frame,

	CPCPUBurstTimer
		0		no speed burst but can be triggered
	   1-10		Burst is NOT happening - count down to 1
				(i.e. delay so we don't burst all the time
	   11+		Burst IS  happening - count down to 1
*********************************************************************/
#define CPU_BURST_DELAY_FRAMES	10

void InitCPUBurst(WORD frames)	// frames to burst (if not currently bursting)
{	// a subroutine of any CPU STATE

	if( (CPNumSpeedBursts < 6)	// only this many per play
		&&(CPCPUBurstTimer == ZERO)	// I can start bursting
	  )
	{
		InitSpeedBurst();	// do it now
		CPCPUBurstTimer = CPU_BURST_DELAY_FRAMES + frames;	// and next X frames
	}
}


void HandleCPCPUBursting(void)	// called every frame in DoAction
{
	if(CPCPUBurstTimer > ZERO)
	{	// bursting??
		if(CPCPUBurstTimer > CPU_BURST_DELAY_FRAMES)	// Yes -still bursting
		{
			InitSpeedBurst();	// keep doing it
		}
		CPCPUBurstTimer--;		// no - delay until next burst
	}
}




STATEDEF(SetMaxSpeedState)	// overrides players max speed
{
	/*  used as 1st STATE in SCRIPT of Punt Returner	*/

	CPMaxSpeed = *(CPScriptPtr+1);	//passed as a parameter in SCRIPT
	if(CPMaxSpeed >= MAX_NUM_SPEEDS)
	{

#ifdef PC_ALWAYS_MODE
		printf("\nIllegal Speed in SetMaxSpeedState");
#endif
		CPMaxSpeed = MAX_NUM_SPEEDS-1;
	}
//	CPMaxSpeed = 22;	//passed as a parameter in SCRIPT
	CPMaxSpeed <<= SPEED_SHIFT;	// shift to high byte
	CPSpeed = CPMaxSpeed;	//give  instant acceleration
	SetBit(CPFlags,IM_PUNT_RETURNER_BFLAG);
	No_Back_Pedal_Macro;

	GoToNextState(2);

}




/******************************************************************

			 G O    T O    N E X T     S T A T E

	When a State if finished, it calls this routine which does a bit
	of clean up and then inc's the Player Script Pointer to the next
	state depending on the number of parameters of the current state.


	USES: gPNum

	INPUTS: number of parameters in current state(used to inc ScriptPtr)

--------------------------------------------------------------------*/

void GoToNextState(UEFFVAR ParamIncValue)
{

	CPScriptPtr += (int)ParamIncValue;

	CPStateInit = FALSE;   /* the next state has not been inited */

	/* no back pedaling	or walking or running backward */
	ResetBit(CPFlags,(BACK_PEDAL_BFLAG | FORCE_WALK_BFLAG | I_RUN_BACKWARD_BFLAG));
//	CPFlags &= ~(BACK_PEDAL_BFLAG | FORCE_WALK_BFLAG);	/* no back pedaling	or walking */

	CPCPUBurstTimer = ZERO;		// stop all bursting
//	SetAI(NoAI);

}






/************************************************************************

					REACHED TARGET

	Will check to see if reached target based on the direction  they
	were supposed to be going.

	NOTE: Minor directions (NNE, SSW) use Major (N,S,E,W) checks

	Reaching target is based on the DIRECTION a player is going,
	therefore if he's going N,S,E,W we only check one condition
	however, some problems can occur if on a diagonal and target
	was OutOf Bounds(north or south) - the Y is adjusted in bounds,
	but the X is not, however on a diagonal two conditions must be met
	( X > targetX AND Y > target Y). If direction is figured every frame
	then it's no problem, but if not then conditions are not met until
	player is at original [OOB] target. To compensate, we also check
	to keep players in the field of play.

	If you don't care abouyt this, use routine ReachedTargetOOB()

**************************************************************************/
//#define CLOSE_TOSL	YARD
BOOL  ReachedTarget(void)
{

UEFFVAR x,y;


	x = CPXPos;
	y = CPYPos;

	switch (CPDirToGo)
	{
		case NNE_DIR:
		case NORTH_DIR:
		case NNW_DIR:
						if (y <= NORTH_SIDELINE+CLOSE_TOSL || y <= CPTargetY)
								return(1);
							return(0);
					//		break;

		case NE_DIR:
						if ((y <= NORTH_SIDELINE+CLOSE_TOSL || x >= OFF_END_LINE-CLOSE_TOEL)
							||  (y <= CPTargetY && x >= CPTargetX))
								return(1);
						return(0);
					//	break;


		case ENE_DIR:
		case EAST_DIR:
		case ESE_DIR:	if (x >= OFF_END_LINE-CLOSE_TOEL || x >= CPTargetX)
							return(1);
						return(0);
					 //	break;

		case SE_DIR:
				  #if 1     // mk
						if ((y >= SOUTH_SIDELINE-CLOSE_TOSL || x >= OFF_END_LINE-CLOSE_TOEL)
							||  (y >= CPTargetY && x >= CPTargetX))
								return(1);
						return(0);
					   //	break;
				  #else		// mh
						if (y >= SOUTH_SIDELINE || y >= CPTargetY)
								if (x >= OFF_GOAL_LINE || x >= CPTargetX)
									return(1);
							return(0);
						 //	break;
				  #endif

		case SSE_DIR:
		case SOUTH_DIR:
		case SSW_DIR:	if (y >= SOUTH_SIDELINE-CLOSE_TOSL || y >= CPTargetY)
								return(1);
							return(0);
						//	break;

		case SW_DIR:
						if (   (y >= SOUTH_SIDELINE-CLOSE_TOSL)
							|| (x <= DEF_END_LINE+CLOSE_TOEL)
							|| (y >= CPTargetY	 &&  x <= CPTargetX) )
									return(1);
							return(0);
						  //	break;

		case WSW_DIR:
		case WEST_DIR:
		case WNW_DIR:
			//			if (x <= DEF_END_LINE+CLOSE_TOEL || x <= CPTargetX)
						if (x <= CPTargetX)		//Bug#2270 - don't check endline if in huddle
								return(1);
							return(0);
						  //	break;

		case NW_DIR:
							if (   (y <= NORTH_SIDELINE+CLOSE_TOSL)
								|| (x <= DEF_END_LINE+CLOSE_TOEL)
								|| (y <= CPTargetY && x <= CPTargetX) )
									return(1);
							return(0);
						  //	break;
	}
	return(0);
}


/*-------------------------------------------------------*/

BOOL  ReachedTargetOOB(void)	// don't care if out of bounds
{							// used by guys running to sidelines
//UWORD x,y;
UEFFVAR	x,y;
BOOL rv= FALSE;

	x = CPXPos;
	y = CPYPos;

	switch (CPDirToGo)
	{
		case NNE_DIR:
		case NORTH_DIR:
		case NNW_DIR:
						if (y<= CPTargetY)
							rv= TRUE;
						break;

		case NE_DIR:
						if ( (y <= CPTargetY && x >= CPTargetX))
							rv= TRUE;
						break;

		case ENE_DIR:
		case EAST_DIR:
		case ESE_DIR:	if (x >= CPTargetX)
							rv= TRUE;
						break;

		case SE_DIR:
						if(y >= CPTargetY && x >= CPTargetX)
							rv= TRUE;
						break;

		case SSE_DIR:
		case SOUTH_DIR:
		case SSW_DIR:	if (y >= CPTargetY)
							rv= TRUE;
						break;

		case SW_DIR:
						if ( (y >= CPTargetY &&  x <= CPTargetX) )
							rv= TRUE;
						break;

		case WSW_DIR:
		case WEST_DIR:
		case WNW_DIR:	if (x <= CPTargetX)
							rv= TRUE;
						break;

		case NW_DIR:
						if ( (y <= CPTargetY && x <= CPTargetX) )
							rv= TRUE;
						break;


	}
	return(rv);
}



/********************************************************************

		Generate  Player  Sphere  Of  Influence

	Sets up a player "Sphere of Influence" into which no other player
	can move. Used for collision detection.  This is a player's "logical"
	size in the X,Y directions.

	WARNING - SI Range is always tweaked
	Started @ 3/4 YARD - this led to guy "walking thru" other guys
	Changed 2/12/96 to 5/4 YARD, but this seemed too big, so we
	made it variable depending on player. BC is bigger than other guys.


---------------------------------------------------------------------*/
void  GeneratePlayerSphereOfInfluence(UWORD pnum)

/*  DON'T use "1.5*YARD" - some compilers don't like floating constants */

//#define SI_RANGE  (3*QTRYARD)	//  (3*HALFYARD)
#define SI_RANGE  	(4*QTRYARD)	// #bug 1119, 1466 change 2/12/96
#define BC_SI_RANGE	(5*QTRYARD)	// change 2/16/96 -BC is "bigger"

{
//register WORD sirange;
register EFFVAR sirange;

	if(pnum == BallCarrier)
		sirange = BC_SI_RANGE/2;
	else
		sirange = SI_RANGE/2;

	PlayerSphereInfluence[pnum].Low.X= PlayerXPosition[pnum]-sirange;
	PlayerSphereInfluence[pnum].High.X= PlayerXPosition[pnum]+sirange;
	PlayerSphereInfluence[pnum].Low.Y= PlayerYPosition[pnum]-sirange;
	PlayerSphereInfluence[pnum].High.Y= PlayerYPosition[pnum]+sirange;
}



/**  does all players   */
void  GenerateAllPlayersSphereOfInfluence(void)
{
//	WORD i;
UEFFVAR	i;

	for(i=0; i<NUM_PLAYERS_ON_FIELD; i++)
		GeneratePlayerSphereOfInfluence(i);
}


/***************************************************************/

/**  first STATE of DefFormationShiftScript  -
	target was set up in InitDefensePreSnapShift		*/

void CheckForDefBackPedalState(void)
{

//WORD	dirtotarget;
EFFVAR	dirtotarget;

	if(CPFlags1 & I_HAVE_BEEN_HCONTROLLED_BFLAG)
	{
		GoToNextState(1);	// bug #1433
		return;
	}

	if(!CPStateInit)
	{	/* check once to init back pedal - StateInit TRUE  in next subroutine */
		CPDirToGo = CPFacing = dirtotarget= FindCPDirectionToTargetMACRO;
		if(dirtotarget >= EAST_DIR-2 && dirtotarget <= EAST_DIR+2)
			Do_Back_Pedal_Macro;
		CPStateInit = TRUE;
	}

	CPFacing = FindCPDirectionToTargetMACRO;
	if(AIFrameCounter%4 == ZERO)
		CPDirToGo = CPFacing;

	CPXPos += SpeedTable[5][CPDirToGo].X;	// NO collision detect
	CPYPos += SpeedTable[5][CPDirToGo].Y;
	GeneratePlayerSphereOfInfluence(gPNum);
	Run();

	if(ReachedTarget())
	{
		CPXPos = CPTargetX;	  // make the current EXACTLY what the target is
		CPYPos = CPTargetY;
		GeneratePlayerSphereOfInfluence(gPNum);
		GoToNextState(1);
	}

}



BOOL IncFramesCheckAtEnd(void)
{
//WORD maxframesinanim;
EFFVAR	maxframesinanim;
BOOL rv;

/* NOTE:  if there are 12 frames in an anim (0-11) then MAX FRAMES is 11 ***/

	maxframesinanim = MAX_FRAMES_MACRO(CPAIGraphicState);
	if(++CPAIGraphicSequence >= maxframesinanim)
	{
		CPAIGraphicSequence = maxframesinanim;	// just to be safe
		rv = TRUE;
	}
	else
		rv = FALSE;
	return(rv);
}


#if 0	// test only - force it
void FindDefStrutter(void)
{
/* when BC is tackled he "looks" for an appropriate def. guy to celebrate */
	// we know BC is not NOBODY

EFFVAR	temp;

	temp = gPNum;

	if( (BallCarrier < 11)
//	&&  (BallXPosition < StartBallX)
//	&&  (PlayType != KICKOFF_PLAY)
	  )
	{
		for(gPNum = 11; gPNum < 22; gPNum++)
		{	// see if doing nothing inside box in front of BC
			if( (CPAction == NoAction)
			  && CPYPos > BallYPosition-8*HALFYARD
			  && CPYPos < BallYPosition+8*HALFYARD
			  && CPXPos > BallXPosition+1*HALFYARD
			  && CPXPos < BallXPosition+10*HALFYARD
			)
			{
				CameraGuy = DefStrutter = gPNum;	// and put camera on him
				break;			// only 1
			}

		}
	}

	gPNum = temp;
}
#else
void FindDefStrutter(void)
{
/* when BC is tackled he "looks" for an appropriate def. guy to celebrate */
	// we know BC is not NOBODY

//WORD temp;
EFFVAR	temp;

	temp = gPNum;

	if( (BallCarrier < 11)	// only if off. BC tackled
	&&  (BallXPosition < StartBallX)	// must be a loss
	&&  (PlayType != KICKOFF_PLAY)		// and NOT a kickoff play
	  )
	{
		for(gPNum = 11; gPNum < 22; gPNum++)
		{	// see if doing nothing inside box in front of BC
			if( (CPAction == NoAction)
			  && CPYPos > BallYPosition-3*HALFYARD
			  && CPYPos < BallYPosition+3*HALFYARD
			  && CPXPos > BallXPosition+1*HALFYARD
			  && CPXPos < BallXPosition+4*HALFYARD
			)
			{
				CameraGuy = DefStrutter = gPNum;	// and put camera on him
				break;			// only 1
			}

		}
	}

	gPNum = temp;
}
#endif

//void DefPostTackleStrut(void)
//{
//	InitGraphics(AI_DEFENSE_POSTTACKLE);
//	SetCPAction(CycleAnimActionAndEnd);
//
//}

/**********************************************************************

				 T H E     A C T I O N S

	"ACTIONS" are the "lowest level of AI, always done first.
	Usually a player has "NoAction", in which case he does either

		a) his STATE (of a SCRIPT) and then his AI(SMARTS) if CPU controlled
	OR	b) if human controlledm what the human wants him to do.


	However, if there is an ACTION pending, the player does that ACTION
	and, in nearly all cases, does NOT do his SCRIPT or human controlling.

***********************************************************************/


void SetCPAction(voidFCNPTR action)
{

	ResetBit(CPFlags, (IM_DIVING_BFLAG | IM_SPINNING_BFLAG
				| IM_LYING_ON_GROUND_BFLAG));

	CPAction = action;

}

/*  following is an ACTION which go through a players current animation and
	ends the ACTION when the anim is over (used for 3,4 point stance	*/
PLAYER_ACTION(CycleAnimActionAndEnd)
{
	if(IncFramesCheckAtEnd())
	{
		SetCPAction(NoAction);
//		ResetBit(CPFlags1,IM_POINTING_BFLAG);
	}
}


PLAYER_ACTION(CycleAnimActionForever)	// anim must loop !!
{
	CYCLE_GRAPHICS_MACRO(CPAIGraphicState);
}



PLAYER_ACTION(CycleAnimActionAndEndCamera)
{
	if(IncFramesCheckAtEnd())
	{
		SetCPAction(NoAction);
		CameraGuy = NOBODY;
//		ResetBit(CPFlags1,IM_POINTING_BFLAG);
	}
}



/***************    OFFENSIVE  LINE    ACTIONS	********************/

WORD QBCallGraphicsTable[4] = {
	AI_QBPLAYCALL_L1_NUM,    // look left short time
	AI_QBPLAYCALL_L2_NUM,    // look left longer time
	AI_QBPLAYCALL_R1_NUM,
	AI_QBPLAYCALL_R2_NUM
};

void InitQBCallPlays(void)	// turns QB head side to side
{
//WORD rnum;
EFFVAR	rnum;

//	if(CPAction != QBCallPlays)	// could be call by audible - if so , don't restart
//	{
		rnum = rand();	// only call rand once
/*****
		if(rnum & 0x0100)
			InitGraphics(AI_QBPLAYCALL1_NUM);	// looks to right (or left)
		else
			InitGraphics(AI_QBPLAYCALL2_NUM);	// looks to right (or left)
****/
		InitGraphics(QBCallGraphicsTable[rnum&0x0003]);

		SetCPAction(QBCallPlays);	// head turn at LOS
		CPTimer = QUARTER_SECOND+(rnum&31);
//		CPFacing = (rnum & 0x0040) ? SOUTH_DIR : NORTH_DIR;
		CPFacing = EAST_DIR;
//	}
}


PLAYER_ACTION(QBCallPlays)		// head turn at LOS
{

	if(GameTeamData[OffensiveSlotNum].TeamFlags & NO_MORE_CPU_SAY_HUT_BFLAG)
	{	// no head turn if human saying "hut" - bug #1825
		CPAIGraphicSequence = 0;
		CPFacing = EAST_DIR;
	}
	else if(CPTimer < ZERO)	// anim is running ??
	{

		if(IncFramesCheckAtEnd())
		{
//			if(GameTeamData[OffensiveSlotNum].TeamFlags & NO_MORE_CPU_SAY_HUT_BFLAG)
//			{ 	// don't do again if human takes over
//				CPAction = NoAction;	// bug #1792
//			}
//			else
//			{
				InitQBCallPlays();	// start again
//			}
		}
	}
	else	// timer counting down - when < 0 then re-init head turn
	{
//		if(GameTeamData[OffensiveSlotNum].TeamFlags & NO_MORE_CPU_SAY_HUT_BFLAG)
//		{ 	// don't do again if human takes over
//			CPAction = NoAction;	// bug #1792
//		}
		/*else*/ if( (CPTimer -= TICKS_PER_AI_FRAME) <= 0)	/* this was setup before setting up this action	*/
		{	// time for new one
			CPAIGraphicSequence = 0;
		}
	}
}




PLAYER_ACTION(GoTo3PtStanceInitAction)
{


	InitGraphics(AI_3PTSTANCE_NUM);

//	CPAIGraphicSequence = CPAIGraphicCounter = ZERO;
//	SetCPAction(GoTo3PtStanceAction);
	SetCPAction(CycleAnimActionAndEnd);

}

PLAYER_ACTION(GoTo4PtStanceInitAction)
{ /* once timer counts down then start gys to go to 4 pt stance	*/
	if( (CPTimer -= TICKS_PER_AI_FRAME) <= 0)	/* this was setup before setting up this action	*/
	{
		InitGraphics(AI_4PTSTANCE_NUM);

//		CPAIGraphicSequence = CPAIGraphicCounter = ZERO;
//		SetCPAction(GoTo4PtStanceAction);
		SetCPAction(CycleAnimActionAndEnd);
		CPFacing = WEST_DIR;
	}
}

#if 0
PLAYER_ACTION(GoTo3PtStanceAction)
{
	CPAIGraphicState = AI_3PTSTANCE_NUM;

	/*  don't run animation at full frame rate	*/
//	if(++CPAIGraphicCounter >= 2)		/* "greater than" test to be sure	*/
//	{
//		CPAIGraphicCounter = 0;

		if(++CPAIGraphicSequence >= MAX_FRAMES_MACRO(AI_3PTSTANCE_NUM))
		{	/*  if beginning of last frame number, no more action	*/
			SetCPAction(NoAction);
		}
//	}
}



//#if 0
PLAYER_ACTION(GoTo4PtStanceAction)
{
	CPAIGraphicState = AI_4PTSTANCE_NUM;

	/*  don't run animation at full frame rate	*/
//	if(++CPAIGraphicCounter >= 2)		/* "greater than" test to be sure	*/
//	{
//		CPAIGraphicCounter = 0;

		if(++CPAIGraphicSequence >= MAX_FRAMES_MACRO(AI_4PTSTANCE_NUM))
		{	/*  if beginning of last frame number, no more action	*/
			SetCPAction(NoAction);
		}
//	}
}

#endif



/*------------------------------------------------------------------

				 J U M P I N G     A C T I O N S

------------------------------------------------------------------*/
#define JUMP_Z_INC (14*INCHES)		// 15 HTZ AI rate
//#define JUMP_Z_INC (10*INCHES)	// 20HTZ rate
//#define JUMP_Z_INC	(FOOT/2)

#if 1	// new 10/22/96 - Jump up and down combined and smoother

WORD JumpHeightTable1[] = {	// WARNING - MUST match # frames in anim
	// table is TOTAL (not cumulative) height off ground
0*INCHES,	//	PG_HANDSUP_NUM,0,
0*INCHES,	//	PG_HANDSUP_NUM,0,
0*INCHES,	//	PG_HANDSUP_NUM,1,	// human starts here

7*INCHES,	//	PG_HANDSUP_NUM,2,
11*INCHES,	//	PG_HANDSUP_NUM,2,
15*INCHES,	//	PG_HANDSUP_NUM,2,
18*INCHES,	//	PG_HANDSUP_NUM,2,

20*INCHES,	//	PG_HANDSUP_NUM,2,	// highest (was 27  - 11/2/96)

18*INCHES,	//	PG_HANDSUP_NUM,2,
15*INCHES,	//	PG_HANDSUP_NUM,2,
11*INCHES,	//	PG_HANDSUP_NUM,2,
7*INCHES,	//	PG_HANDSUP_NUM,2,

0*INCHES,	//	PG_HANDSUP_NUM,1,	// code forces land on this frame
};

WORD JumpHeightTable2[] = {	// WARNING - MUST match # frames in anim
	// table is TOTAL (not cumulative) height off ground
0*INCHES,	//	PG_HANDSUP_NUM,0,
0*INCHES,	//	PG_HANDSUP_NUM,0,
0*INCHES,	//	PG_HANDSUP_NUM,1,	// human starts here

16*INCHES,	//	PG_HANDSUP_NUM,2,
24*INCHES,	//	PG_HANDSUP_NUM,2,
30*INCHES,	//	PG_HANDSUP_NUM,2,
34*INCHES,	//	PG_HANDSUP_NUM,2,

37*INCHES,	//	PG_HANDSUP_NUM,2,	// highest

34*INCHES,	//	PG_HANDSUP_NUM,2,
30*INCHES,	//	PG_HANDSUP_NUM,2,
24*INCHES,	//	PG_HANDSUP_NUM,2,
16*INCHES,	//	PG_HANDSUP_NUM,2,

0*INCHES,	//	PG_HANDSUP_NUM,1,	// code forces land on this frame
};


WORD JumpHeightTable3[] = {	// WARNING - MUST match # frames in anim
	// table is TOTAL (not cumulative) height off ground
0*INCHES,	//	PG_HANDSUP_NUM,0,
0*INCHES,	//	PG_HANDSUP_NUM,0,
0*INCHES,	//	PG_HANDSUP_NUM,1,	// human starts here

18*INCHES,	//	PG_HANDSUP_NUM,2,
27*INCHES,	//	PG_HANDSUP_NUM,2,
34*INCHES,	//	PG_HANDSUP_NUM,2,
40*INCHES,	//	PG_HANDSUP_NUM,2,

43*INCHES,	//	PG_HANDSUP_NUM,2,	// highest

40*INCHES,	//	PG_HANDSUP_NUM,2,
34*INCHES,	//	PG_HANDSUP_NUM,2,
27*INCHES,	//	PG_HANDSUP_NUM,2,
18*INCHES,	//	PG_HANDSUP_NUM,2,

0*INCHES,	//	PG_HANDSUP_NUM,1,	// code forces land on this frame
};



PLAYER_ACTION(JumpUpAction)	// and Down	(new 10/22/96)
{
EFFVAR	maxframesinanim,guyhit,weight;

	if(InitGraphicsQ(AI_JUMPUP_NUM))	// if necessary
	{
		if(CPFlags & IS_HCONTROLLED_BFLAG)
			CPAIGraphicSequence = 2;	// no crouch to jump for human
	}
	else	// if not 1st time, inc it
		CPAIGraphicSequence++;

	maxframesinanim = MAX_FRAMES_MACRO(CPAIGraphicState);
	if(CPAIGraphicSequence >= maxframesinanim)
	{
		CPAIGraphicSequence = maxframesinanim;	// just to be safe
		SetCPAction(NoAction);
		CPZPos = ZERO;	// just to be sure
	}
	else
	{ //  jumping height based on weight!?!
		weight = GetPlayerWeight(gPNum);
		if(weight > 250)
			CPZPos = JumpHeightTable1[CPAIGraphicSequence];
		else if(weight > 220)
			CPZPos = JumpHeightTable2[CPAIGraphicSequence];
		else
			CPZPos = JumpHeightTable3[CPAIGraphicSequence];
	}
/***
	if(gPNum&1)	// test for now
		CPZPos = JumpHeightTable1[CPAIGraphicSequence];
	else
		CPZPos = JumpHeightTable2[CPAIGraphicSequence];
***/
	if(CPSpeed > ZERO)	// bug #1688
	{
		guyhit =  TryMove(CPDirToGo,CPSpeed>>SPEED_SHIFT);
		CheckForTackleAndFumble(guyhit);
	}

}
#else
	obsolete 10/22/96	*******************************************/
PLAYER_ACTION(JumpDownAction)	// actually coming down from jump
{
//WORD guyhit;
EFFVAR	guyhit;

	CPZPos -= JUMP_Z_INC;

	if( CPZPos >0xf000		// ZPOS is UWORD - need High check
		|| CPZPos == ZERO)
	{
		CPZPos = ZERO;
//		if(--CPAIGraphicSequence <= 2)	// was ZERO, bug #1457
//		{
			CPAIGraphicSequence = 2;	// be safe
			SetCPAction(NoAction);
//		}
	}

	if(CPSpeed > ZERO)	// bug #1688
	{
		guyhit =  TryMove(CPDirToGo,CPSpeed>>SPEED_SHIFT);
	//	if(guyhit >= ZERO && (!(EventFlags & TACKLE_STARTED_BFLAG)) )
		CheckForTackleAndFumble(guyhit);
	}

}

PLAYER_ACTION(JumpUpAction)
{
//WORD maxframesinanim,guyhit;
EFFVAR	maxframesinanim,guyhit;

	if(InitGraphicsQ(AI_HANDSUP_NUM))	// if necessary
	{
		if(CPFlags & IS_HCONTROLLED_BFLAG)
			CPAIGraphicSequence = 2;	// no crouch to jump for human
	}
	else
		CPAIGraphicSequence++;

	maxframesinanim = MAX_FRAMES_MACRO(CPAIGraphicState);
//	if(CPAIGraphicSequence > maxframesinanim)
	if(CPAIGraphicSequence >= maxframesinanim)
	{
		/*  if greater than last frame number, jump higher	*/
		CPAIGraphicSequence = maxframesinanim;	// just to be safe
		CPZPos += JUMP_Z_INC;
		if(CPZPos > 3*FEET)
			SetCPAction(JumpDownAction);
	}

	if(CPSpeed > ZERO)	// bug #1688
	{
		guyhit =  TryMove(CPDirToGo,CPSpeed>>SPEED_SHIFT);
	//	if(guyhit >= ZERO && (!(EventFlags & TACKLE_STARTED_BFLAG)) )
		CheckForTackleAndFumble(guyhit);
	}

}
#endif	/*----------------------------------------------------*/




WORD HurdleHeightTable[] = {	// WARNING - MUST match # frames in anim
 0*INCHES,	//	PG_BURST_NUM,0,
 0*INCHES,	//	PG_BURST_NUM,1,
 5*INCHES,	//	PG_RUNNINGWITHBALL_NUM,4,
16*INCHES,	//PG_BURST_NUM,2,
24*INCHES,	//	PG_BURST_NUM,2,
30*INCHES,	//	PG_BURST_NUM,3,
16*INCHES,	//	PG_BURST_NUM,3,
 0*INCHES,	//	PG_BURST_NUM,1,
 0*INCHES,	//	PG_BURST_NUM,4,
 0*INCHES,	//	PG_RUNNINGWITHBALL_NUM,3,
};

#if 1
PLAYER_ACTION(BCJumpUpAction)	// BallCarrier "hurdles"
{						// and jump down

	if(!InitGraphicsQ(AI_HURDLEWITHBALL_NUM))	// if necessary
		CPAIGraphicSequence++;


	if(CPAIGraphicSequence > MAX_FRAMES_MACRO(CPAIGraphicState))
	{	// restart at seq. 4 of run with ball
		CPAIGraphicSequence = 4;
		CPAIGraphicState = AI_RUNNINGWITHBALL_NUM;
		SetCPAction(NoAction);
		CPZPos = ZERO;	// just to be sure
	}
	else
	{

		CPZPos = HurdleHeightTable[CPAIGraphicSequence];
	}

	if(CPSpeed > ZERO)
	{
		TryMove(CPDirToGo,CPSpeed>>SPEED_SHIFT);
	}
}



#else
PLAYER_ACTION(BCJumpDownAction)
{
/* BallCarrier does [graphically] whatever he was doing	- most likely running
*/

//	CYCLE_GRAPHICS_MACRO(CPAIGraphicState);

	if(TryMove(CPDirToGo,CPSpeed>>SPEED_SHIFT) == NOBODY)
	{
//		if( (AIFrameCounter&3) == ZERO)	// bug #1449
		if( (AIFrameCounter&1) == ZERO)	// bug #1662
			Run();	// bug # 1342 - only animate every other frame
		CPZPos -= JUMP_Z_INC;
		if(CPZPos >0xf000)	// ZPos is unsigned - need high check
		{
			CPZPos = ZERO;
			SetCPAction(NoAction);
		}
	}
	else
	{
		CPZPos = ZERO;
		SetCPAction(NoAction);
	}
}





PLAYER_ACTION(BCJumpUpAction)	// BallCarrier "hurdles"
{

//	CYCLE_GRAPHICS_MACRO(CPAIGraphicState);

	if(TryMove(CPDirToGo,CPSpeed>>SPEED_SHIFT) == NOBODY)
	{
		if( (AIFrameCounter&1) == ZERO)	// bug #1449,
			Run();	// bug # 1342
		CPZPos += JUMP_Z_INC;
		if(CPZPos > (2*FEET+6*INCHES) )
			SetCPAction(BCJumpDownAction);
	}
	else
	{	// hit somebody - go down
//		CPZPos = ZERO;
//		SetCPAction(NoAction);
		SetCPAction(BCJumpDownAction);

	}
}
#endif



PLAYER_ACTION(JumpForCatchAction)
{
EFFVAR	guyhit;

	if( !InitGraphicsQ(AI_JUMPCATCH_NUM))
	{
		if(IncFramesCheckAtEnd())
		{	// randomly fall if he caught ball (based on Agility)
			if( ImBallCarrierMACRO)
			{	// good players don't fall as often - bug #2094
				if(CPAgility < randper)
					StartDive(FALSE);	// FALSE = don't start dive this time
				else
					SetCPAction(NoAction);
			}
			else	// don't fall if not BC - bug #2093
				SetCPAction(NoAction);
		}
	}

	guyhit =  TryMove(CPDirToGo,CPSpeed>>SPEED_SHIFT);
	CheckForTackleAndFumble(guyhit);

}




void StartJump(void)	/* a STATE (or Human controlled) subroutine	*/
{
	// only do if no other actions happening
//	if(CPAction == NoAction && gPNum > LAST_OFF_SLOT_NUM)	// player on Def
	if(CPAction == NoAction || (CPFlags & IM_SPINNING_BFLAG))
	{
		CPFacing = CPDirToGo;		//bug # 1291
		if(ImBallCarrierMACRO)
		{
			SetCPAction(BCJumpUpAction);		// set ACTION for next time
			BCJumpUpAction();		// ... and do it this time
		}
		else if( CPAIGraphicState == AI_RUNNINGCATCH_NUM)
		{	// rec or close def. - takes into account only ABC dirs
			SetCPAction(JumpForCatchAction);		// set ACTION for next time
			JumpForCatchAction();		// ... and do it now
		}
		else if(   (gPNum == TheReceiver)
				&& (CPFacing < SSW_DIR)		// don't have anims in all directions
//				&& (CPFacing < NNW_DIR )
			   )
		{	// - takes into account only ABC dirs
			SetCPAction(JumpForCatchAction);		// set ACTION for next time
			JumpForCatchAction();		// ... and do it now
		}
		else
		{
			SetCPAction(JumpUpAction);		// set ACTION for next time
			JumpUpAction();		// ... and do it now
		}
	}
}


/*------------------------------------------------------------------

				 D I V I N G /  T A C K L I N G     A C T I O N S

	Diving modified 2/16/96 to use new AI anim and offset tables
	(old DiveAction in "fbgrave.txt"

	Also 4/?/96 - Diving uses DELTAs to player can dive in 360 degrees,
	not just 16 dirs.

------------------------------------------------------------------*/

//GAME_OPTNS_WEATHER 	            0-normal 1-rain 2-snow
WORD ExtraDiveFramesWeather[MAX_WEATHER_OPTIONS] = {-7,-3,0};

PLAYER_ACTION(DiveAction)
{
//WORD maxframesinanim;
EFFVAR	maxframesinanim;
//WORD guyhit;
EFFVAR	guyhit;

	if(!InitGraphicsQ(AI_DIVE_NUM))	// if necessary
		CPAIGraphicSequence++;

	//  extra error checking
	GameOptions[GAME_OPTNS_WEATHER] = SetMinMax(GameOptions[GAME_OPTNS_WEATHER],0,2);

	maxframesinanim = MAX_FRAMES_MACRO(AI_DIVE_NUM);
	maxframesinanim += ExtraDiveFramesWeather[GameOptions[GAME_OPTNS_WEATHER]];

	if(CPAIGraphicSequence >= 11)	// WARNING - hardwired num
	{ 	//	no BC fumbling from now on
		SetBit(CPFlags,IM_LYING_ON_GROUND_BFLAG);
	}

	if(CPAIGraphicSequence >= maxframesinanim)
	{
			/*  if beginning of last frame number, no more action	*/
		CPAIGraphicSequence = maxframesinanim;	// just to be safe

		CPZPos = ZERO;		// make sure he's back on ground
		ResetBit(CPFlags,IM_DIVING_BFLAG);
		SetBit(CPFlags,IM_LYING_ON_GROUND_BFLAG);

		// timer set up in StartDive
		// followingline got guys stuck @EOP, however we need all guys to get
		// up at same time, so it's init in PlayState "PS_GetUpAfterPlay"
		if(!(EventFlags.TACKLE_STARTED_BFLAG))   //
		{
			if( (CPTimer -= TICKS_PER_AI_FRAME) <= 0)	//wait a bit before getting up
			{
				SetCPAction(GetUpAction);
			}
		}
	}
	else
	{
		SetBit(CPFlags,IM_DIVING_BFLAG);

//		guyhit =  TryMove(CPDirToGo,DIVE_SPEED);	// mod. 1/13/96 bug #1257
		guyhit =  TryMoveDelta();	// mod. 1/13/96 bug #1257

		if(guyhit >= ZERO)	// we hit somebody
		{
			AIPlayEffectCheck(GRUNT1_SFX);

			if(PlayStateNum < PS_PlayHappeningNUM)
			{
//				AIPlayEffectCheck(GRUNT1_SFX);
				if(gPNum > 10 && guyhit < 11)
				{	// def hit off before start of play
					InitDefPenaltyParameters(gPNum,ENCROACHMENT_PENALTY,StartBallX);
//					SetDefOffsides(DEF_ENCROACHMENT_TYPE);	// called if collision before snap OR past LOS at snap
				}
			}
			else if(PlayStateNum > PS_PlayHappeningNUM)
			{	// play over - check for personal fouls
				if(   guyhit == PenaltyBallCarrier
				   && CPFlags & IS_HCONTROLLED_BFLAG)	// human not getting enough fouls
				{
					CheckForPersonalPlayerFoul();
				}
			}
			else
			{
				CheckForReceiverContact(guyhit);
				CheckForTackleAndFumble(guyhit);	// Should we tackle him?
//				SetBit(EventFlags,FIRST_HIT_BFLAG);
			}

			PushOtherGuy(guyhit,CPPushFactor);
		}
	}
}



PLAYER_ACTION(GetTackledAction)
{                 /*  set graphics to max frames and then freeze	*/
	if(IncFramesCheckAtEnd())
	{
		if(gPNum == BallCarrier)	// still BC (fumble)?
		{
			SetCPAction(FreezeActionOnGround);
			SetBit1(EventFlags,TACKLE_ENDED_BFLAG);	// the play is now over
			ExtraTackleX  =  TempExtraTackleX;	// used if BC falls forward

			FindDefStrutter();	// any body going to celebrate my tackle?
	//		SetBallAction(BallAfterTackle);    // in Check play over
		}
		else
			SetCPAction(GetUpAction);

	}
}

#if 0
PLAYER_ACTION(GetTackledInitAction)
{
	InitGraphics(AI_TACKLED_NUM);
	SetCPAction(GetTackledAction);
	CPSpeed = ZERO;
//	CPZPos = ZERO;		// abort any jumping
}
#endif


void CheckForPersonalPlayerFoul(void)	// based on team penalty rating or HUMAN
{		//teams rated 100-40 based on 1995 penalties (Raiders,Skins highest)
EFFVAR teampenaltyfactor = GameTeamData[DefensiveSlotNum].STD.PenaltyFactor/3;
//WORD firstplaystate = PS_Wait4cNUM;

//	if(CPFlags & IS_HCONTROLLED_BFLAG)	// human has bigger time period
//		firstplaystate = PS_Wait4bNUM;	no - human okay @ wait4c

	if(  PlayStateNum >= PS_Wait4cNUM	// all checks moved to actual checker
//	if(  PlayStateNum >= firstplaystate	// all checks moved to actual checker
		&& PlayStateNum < PS_Wait5NUM
//	   && gPNum > 10		// defense only
//	   && PersonalFouler == NOBODY	// test only NO make last guy on pile
//	   && GameOptions[GAME_OPTNS_PFOULS] == OPTION_ON
//	   && TheDown < 4
//	   && (AIFrameCounter & 1)	//50% chance
//		   && (randper < teampenaltyfactor)	// chance based on team
	  )
	{
		if(CPFlags & IS_HCONTROLLED_BFLAG)	// human is constant
		{	// "... human is a mercenary on his team..." MJK 11/7/96
//			teampenaltyfactor = 70;
			teampenaltyfactor = 170;
		}

		if(randper < teampenaltyfactor)	// chance based on team
		{
			PersonalFouler = gPNum;
		#if PC_ALWAYS_MODE
			printf("\nPossible Personal Fouler = %d, team = %s\n",
				gPNum,GameTeamData[DefensiveSlotNum].STD.TeamNickName);
		#endif
		}
	}
}

PLAYER_ACTION(BeTacklerAction)
{                 /*  set graphics to max frames and then freeze	*/

	InitGraphics(AI_DIVE_NUM);	// if necessary
//	if(CPFlags & IS_HCONTROLLED_BFLAG)	// human not getting enough fouls
//	{
//		CheckForPersonalPlayerFoul();	// test only - way too many fouls if left in
//	}
	if(IncFramesCheckAtEnd())
	{ 	/*  if beginning of last frame number, then freeze*/
		CheckForPersonalPlayerFoul();
		SetCPAction(FreezeActionOnGround);

	}
}


#if 1
PLAYER_ACTION(GetUpAction)	// new 8/5/96 - get up based on how fell down
{
//WORD gstate,newanim;	// local vars save code
EFFVAR	gstate,newanim;	// local vars save code

	CPSpeed = ZERO;
	ResetBit(CPFlags,IM_LYING_ON_GROUND_BFLAG);

	gstate = CPAIGraphicState;	// figure array index once and save in local == AI_TACKLED_FFF_NUM)

	if( gstate == AI_TACKLED_FFF_NUM
	 || gstate == AI_TACKLED_FBB_NUM
	 || gstate == AI_QBSLIDE_NUM
	  )
	{

		CPFacing += 8;		// reverse facing
		CPFacing &= 15;
		CPDirToGo = CPFacing;
		newanim = AI_GETUPFROMBACK_NUM;		// with roll
	}
	else if (gstate == AI_TACKLED_FFR_NUM)
	{
		newanim = AI_GETUPFROMBACK_NUM;		// with roll

	}
	else if  (gstate == AI_TACKLED_FBR_NUM)
	{
		CPFacing += 8;		// reverse facing
		CPFacing &= 15;
		CPDirToGo = CPFacing;
		newanim = AI_GETUPFROMBELLY_NUM;   	// default

	}
	else	// tackle FFS, FBF, or dive
		newanim = AI_GETUPFROMBELLY_NUM;   	// default

	InitGraphics(newanim);   	// inits frame #0
	SetCPAction(CycleAnimActionAndEnd);	// next time cycle all frames to end
}

#else 	// old - retired 8/5/96
PLAYER_ACTION(GetUpAction)
{

	CPSpeed = ZERO;
	ResetBit(CPFlags,IM_LYING_ON_GROUND_BFLAG);

	if(CPAIGraphicState == AI_TACKLED_FBB_NUM)
		InitGraphics(AI_GETUPFROMBACK_NUM);   	// inits frame #0
	else
		InitGraphics(AI_GETUPFROMBELLY_NUM);   	// inits frame #0
	SetCPAction(CycleAnimActionAndEnd);	// next time cycle all frames to end

//	if(--CPAIGraphicSequence <= ZERO)
//	{
//			/*  if beginning of last frame number, no more action	*/
//		CPAIGraphicSequence = ZERO;	// just to be safe
//		SetCPAction(NoAction);
//	}
//	CPSpeed = ZERO;

}

#endif

PLAYER_ACTION(DelayedGetUpAction)	// used by BC to get up after tackle
{
// WARNING - CPTimer must be inited before invoking this ACTION

	if( (CPTimer -= TICKS_PER_AI_FRAME) <= 0)	//wait a bit before getting up
	{
		SetCPAction(GetUpAction);
	}
	else
	{
		SetBit(CPFlags,IM_LYING_ON_GROUND_BFLAG);
	}
}

PLAYER_ACTION(QBSlideAction)
{
BOOL atend = FALSE;

	if( !InitGraphicsQ(AI_QBSLIDE_NUM))	// if needed
		atend = IncFramesCheckAtEnd();

	if(atend)
	{
//		SetCPAction(FreezeActionOnGround);
		CPTimer = SECOND;
		SetCPAction(DelayedGetUpAction);
		if(ImBallCarrierMACRO)
		{
//			SetBit1(EventFlags,(TACKLE_STARTED_BFLAG | TACKLE_ENDED_BFLAG));	//kludgey
			SetBit1(EventFlags,TACKLE_STARTED_BFLAG);	//kludgey
			SetBit1(EventFlags,TACKLE_ENDED_BFLAG);	//kludgey
		}
	}
	else
	{
		TryMove(CPDirToGo,11); 	// was 5, bug #1746
		SetBit(CPFlags,IM_LYING_ON_GROUND_BFLAG);
	}
}



/*--------------------------------------------------------------------*/
void ConvertCurrentPlayerAIToActualGraphics(void);	// proto
WORD TackleSounds[8] = {
	TACKLE_SFX,TACKLE1,TACKLE2,TACKLE3,
	TACKLE4,TACKLE1,TACKLE2,TACKLE3
//	TACKLE8,TACKLE8,TACKLE10,TACKLE11,
//	TACKLE12,TACKLE13,TACKLE14,TACKLE_SFX
};

//WORD DirVsDirTable[16] = {0,1,1,1,2,2,2,2,3,-2,-2,-2,-2,-1,-1,-1};
BYTE HitDirTable[16] =   {0,0,0,1,1,1,2,2,2, 2, 2, 1, 1, 1, 0, 0};

#ifdef SATURNMODE
WORD HitFromBack[3][2] = {
	AI_TACKLED_FFS_NUM,AI_TACKLED_FFR_NUM,	//anim set 0
	AI_TACKLED_FFS_NUM,AI_TACKLED_FFR_NUM,	//anim set 0
	AI_TACKLED_FFS_NUM,AI_TACKLED_FFR_NUM,	//anim set 0
//	AI_TACKLED_FFS_NUM,AI_TACKLED_FFS_NUM,	//anim set 1
//	AI_TACKLED_FFS_NUM,AI_TACKLED_FFR_NUM	//anim set 2
};

WORD HitFromFront[3][2] = {
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
//	AI_TACKLED_FBR_NUM,AI_TACKLED_FFF_NUM,	//anim set 1
//	AI_TACKLED_FBF_NUM,AI_TACKLED_FBF_NUM	//anim set 2
};
#else      //  -------- PSX -----------------


/**  TEST ONLY for 4/25/97 ALPHA		*/
WORD HitFromBack[3][2] = {
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
};

WORD HitFromFront[3][2] = {
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
};



/***
WORD HitFromBack[3][2] = {
	AI_TACKLED_FFS_NUM,AI_TACKLED_FFR_NUM,	//anim set 0
	AI_TACKLED_FFS_NUM,AI_TACKLED_FFS_NUM,	//anim set 1
	AI_TACKLED_FFS_NUM,AI_TACKLED_FFR_NUM	//anim set 2
};

WORD HitFromFront[3][2] = {
	AI_TACKLED_FBB_NUM,AI_TACKLED_FBB_NUM,	//anim set 0
	AI_TACKLED_FBR_NUM,AI_TACKLED_FFF_NUM,	//anim set 1
	AI_TACKLED_FBF_NUM,AI_TACKLED_FBF_NUM	//anim set 2
};
******************/
#endif

#define ETDIAGX	HALF_YARD
/*  Give extra yardage to BC when he falls forward  --
	note graphics map minor diags to "B"dir anim, so only EAST dir
	("A"dir anim) gets full extension when being tackled	*/
WORD ExtraTackleXTable[MAX_DIR] = {
	0,ETDIAGX,ETDIAGX,ETDIAGX,ETDIAGX*2,ETDIAGX,ETDIAGX,ETDIAGX,
	0,0,0,0, 0,0,0,0};


void  StartTackle(void)   	/* a STATE subroutine	*/
{
//WORD temp,dir,diff;
EFFVAR	temp,dir,diff;
//WORD hittype, tanim;
EFFVAR	hittype, tanim;

	if(CPAction != DiveAction)
	{
		CPZPos = ZERO;	// start on ground - bug #2070
		InitGraphics(AI_DIVE_NUM);
		SetCPAction(BeTacklerAction);
	}
	else	// allower pre-tackle diver to get Personal foul
	{     // set BP below to see how much it happens
		SetCPAction(BeTacklerAction);	// start w/o initing DIVE graphics

	}

	if( !(EventFlags.TACKLE_STARTED_BFLAG))	// if tacklee isn't already down
	{
#if PC_ALWAYS_MODE
//		printf("TS");
#endif
		dir = FindCPDirectionToPlayer(BallCarrier);

		temp = gPNum;			// save "current"
		gPNum = BallCarrier;	// set "current" player to BC


#if 1
		if(OPFlags(BallCarrier) & IM_LYING_ON_GROUND_BFLAG)
		{
			// leave same graphics if lying on ground (thru dive or slide) - bug # 2081
			tanim = CPAIGraphicState;  // see if BC gets extra distance
		}
		else
		{
			diff = dir - CPFacing;		// range is -15 to 15
			if(diff < 0)		// -15 maps to 1, -14 to 2, etc
				diff +=	16;
			hittype = HitDirTable[diff];

			if(hittype == 0)	// hit from behind
			{
				tanim = HitFromBack[TDAnimLoadedNum][(rand()&1)];
			}
			else if(hittype == 1)	// hit from side
			{
				CPDirToGo = CPFacing = dir;		// face the tackler
				tanim = HitFromFront[TDAnimLoadedNum][(rand()&1)];
			}
			else 				// hit from front
			{
				tanim = HitFromFront[TDAnimLoadedNum][(rand()&1)];

			}
			InitGraphics(tanim);
		}

		if(gPNum < 11)	// BC on offense???
		{
			if(  tanim == AI_TACKLED_FFS_NUM
			  || tanim == AI_TACKLED_FFR_NUM
			  || tanim == AI_DIVE_NUM
			  )
			{
				TempExtraTackleX = ExtraTackleXTable[CPFacing];
			}
		}
#else
		if(OPFlags(BallCarrier) & IM_LYING_ON_GROUND_BFLAG)
		{
			// leave same graphics if lying on ground (thru dive or slide) - bug # 2081
		}
		else if( dir > SW_DIR	// head on collision with BC=EAST & DEF=WEST?
		  &&dir < NW_DIR
		  && CPFacing > NE_DIR
		  && CPFacing < SE_DIR
		  )
			InitGraphics(AI_TACKLED_FBB_NUM);
		else if (AIFrameCounter&3)
			InitGraphics(AI_TACKLED_FFS_NUM);
		else
			InitGraphics(AI_TACKLED_FFR_NUM);


#endif
		SetCPAction(GetTackledAction);

		CPSpeed = ZERO;
#ifdef FRAME_RATE_30HERTZ	// convert this frame if double movement rate
		ConvertCurrentPlayerAIToActualGraphics();	// must do this frame
#endif
		ResetBit(CPFlags, (IM_DIVING_BFLAG | IM_SPINNING_BFLAG));

		gPNum = temp;	// restore Tackler
	}

	SetBit(OPFlags(BallCarrier),IM_BEING_TACKLED_BFLAG);
	SetBit1(EventFlags,TACKLE_STARTED_BFLAG);

	if(Tackler == NOBODY)
		Tackler = gPNum;	// save for stats - 1st guy to hit

	AIPlayEffectCheck(TackleSounds[rand()&7]);

}




BOOL StartQBSlide(void)
{
BOOL rv = FALSE;	// assume not slide


	if(CPMyFormPosition == QB
//		&& CPFacing > ENE_DIR
//		&& CPFacing < ESE_DIR
		&& CPFacing > NNE_DIR	// bug # 1747
		&& CPFacing < SSE_DIR
		&& CPAction != QBSlideAction
		&& CPXPos > StartBallX		// bug # 2369
	  )
	{
		CPDirToGo = CPFacing;
		SetCPAction(QBSlideAction);		// set ACTION for next time

		QBSlideAction();	// and do it now
		rv = TRUE;

	}

	return(rv);
}





PLAYER_ACTION(KneelToEndPlay)	// down a Kickoff
{
	CPFacing = CPDirToGo = EAST_DIR;

	InitGraphics(AI_GOTOHOLDER_NUM);
	if(IncFramesCheckAtEnd())
	{
		SetCPAction(NoAction);
		//kludgey EOP
//		SetBit(EventFlags,(TACKLE_STARTED_BFLAG | TACKLE_ENDED_BFLAG));
		SetBit1(EventFlags,TACKLE_STARTED_BFLAG);
		SetBit1(EventFlags,TACKLE_ENDED_BFLAG);
	}
}



BOOL CPUDownKickOff(WORD kr)	// CPU Kick returner
{	// called from BallACTION or player script
BOOL rv = FALSE;
EFFVAR tempguy;

	tempguy = gPNum;	// save current player

	gPNum = kr;

	if( (!(EventFlags.BC_OUT_OF_EZ_BFLAG)) // hasn't gone out of EZ yet
	  && CPControllerNum == NOBODY)	// "human" operated (but maybe not controlled)
	{
		if(BallXPosition < DEF_GOAL_LINE-6*YARDS)
		{	// if deep in EZ, down it
			rv = TRUE;
		}
		else if(BallXPosition < DEF_GOAL_LINE-3*YARDS && (rand()&1) )
		{	// 50% chance in in "mid-range"
			rv = TRUE;
		}
	}

	if(rv)
	{
		CPAction = KneelToEndPlay;
		SetNewScript(WatchEOPScriptNUM);
	}

	gPNum = tempguy;	// restore Key player num

	return(rv);

}


BOOL DownKickOffValid(void)	// human pressing dive button
{
BOOL rv = FALSE;

	if( (PlayType == KICKOFF_PLAY)
	  &&(gPNum < START_DEF_SLOT_NUM)
	  &&(ImBallCarrierMACRO)
	  &&(CPXPos < DEF_GOAL_LINE+1)
	  &&(!(EventFlags.BC_OUT_OF_EZ_BFLAG)) // hasn't gone out of EZ yet
	  )
	{	// downing Kick in EZ is okay
		if(CPAction != KneelToEndPlay)
			CPAction = KneelToEndPlay;
		rv =  TRUE;
	}
	return(rv);

}



BOOL OnMyTeam(WORD guy);
#define FindCPDirectionToPoint(x,y) FindDirection( ((x)-(CPXPos)),((y)-(CPYPos)))

/*---------------------------------------------------------------------

			D I V I N G

	Diving uses DELTAs to player can dive in 360 degrees,
	not just 16 dirs.

---------------------------------------------------------------------*/
//#define DIVE_TWEAK	1
#define DIVE_TWEAK	2	//11/16/962

// new method using delta (360 degree)

void StartDive(BOOL startthisframe)       	/* a STATE subroutine	*/
{
EFFVAR	keyp,speed;
UEFFVAR	tx,ty;
EFFVAR	dirtotarg,revface,disttotarget;
BOOL straightdive = FALSE;

	if(CPAction != DiveAction)	// this makes sure CPU guys won't double-dive
	{
		keyp = gPNum;	// assume I'm the key (if no BC, go my current dir)

	// players on opposite team of BC will automatically dive toward him
		if(BallCarrier >= ZERO)
		{ // there is a Ball Carrier
			if(ImBallCarrierMACRO)
			{        // then continue (dive) in current direction
				if(StartQBSlide())
					return;
			}
			else if(!OnMyTeam(BallCarrier))
			{  // BC on other team
				CPDirToGo =  FindCPDirectionToPlayer(BallCarrier);
				keyp = BallCarrier;
				if( (OPSpeed(BallCarrier) < (4<<SPEED_SHIFT)+1)	// BC is slow
//				   || (CPKeyDistance < 3*YARDS)	// or Im close
// following test moved lower in routine 11/16/96
///				   || (CPKeyDistance < 5*HALF_YARD)	// or Im closer (11/6/96)
				  )
				{
					straightdive = TRUE;
				}
			}
		}
		else if (BallFlags & PASS_DEFLECTED_BFLAG)
		{	// jump to ball if deflected - bug #
			CPDirToGo =  FindCPDirectionToBallMACRO;
		}
		else if (TheReceiver != NOBODY && gPNum != TheReceiver)
		{
			CPDirToGo =  FindCPDirectionToPlayer(TheReceiver);
			keyp = TheReceiver;

		}

		CPFacing = CPDirToGo;   // make sure we face to dir we want to go(graphics only)

	// target is "in front" of key player (dir. key is running)
		speed = (OPSpeed(keyp)>>SPEED_SHIFT);
		if(keyp == gPNum)		// if Im key, 0 caused no movement on dive
			speed++;			// so be sure to move

	// dive where we assume keyp will be
		//NOTE: in ST[x][dir], x is a tweakable value
		tx = OPXPos(keyp) + (SpeedTable[DIVE_TWEAK][OPDirToGo(keyp)].X) * speed;
		ty = OPYPos(keyp) + (SpeedTable[DIVE_TWEAK][OPDirToGo(keyp)].Y) * speed;


		disttotarget = Distance(tx,ty);

		if(keyp == BallCarrier)	// okay, i'll admit - it's a kludge
		{
			if(CPKeyDistance < 5*HALF_YARD	// or Im closer (11/6/96)
			  && disttotarget < 5*HALF_YARD)
			{
				straightdive = TRUE;
			}
			else
			{
				straightdive = straightdive;	// bp here
			}

		}

#if PC_GRAPHIC_MODE
	DrawCross(tx,ty,15);
#endif
	/*  check to make sure not diving "backward" - bug #2050	*/
		dirtotarg = FindDirection2Points(CPXPos,CPYPos,tx,ty);

		revface = (CPFacing+8)&15;
/***
		diff = dirtotarg - revface;		// range is -15 to 15
		if(diff < 0)		// -15 maps to 1, -14 to 2, etc
			diff +=	16;
		diff = HitDirTable[diff];
		if(diff  == ZERO
		   || straightdive
		  )					****/

		if(revface == dirtotarg
		   || revface == ((dirtotarg-1)&15)	// slow code, fast programming time
		   || revface == ((dirtotarg+1)&15)
		   || revface == ((dirtotarg-2)&15)
		   || revface == ((dirtotarg+2)&15)
		   || straightdive
		  )
		{     //then dive straight [at BC]
			tx = OPXPos(keyp) + (SpeedTable[2][OPDirToGo(keyp)].X);
			ty = OPYPos(keyp) + (SpeedTable[2][OPDirToGo(keyp)].Y);
//			tx = OPXPos(keyp);
//			ty = OPYPos(keyp);
		}
		else
		{
			CPFacing = CPDirToGo = FindCPDirectionToPoint(tx,ty);
		}

		SetDeltaParameters(tx,ty,DIVE_SPEED);	// figure dx,dy for 360 degree diving

		SetCPAction(DiveAction);		// set ACTION for next time

/* set time to wait while on ground -
		variable depending on random, reaction  - BC stays down longer */
		CPTimer = 1*QUARTER_SECOND + (rand()&31) + ((100-CPReaction)>>1); //
		if(gPNum == BallCarrier)	// keep [human] BC down
			CPTimer +=  (2*QUARTER_SECOND + (rand()&31));

		if(startthisframe)
			DiveAction();	// and do it now
	}
}

/*-------------------------------------------------------------------

 if using deltas for dive, CPDeltaX = SPeed10value * deltaX%
	deltaX% = (deltaX[myX to target X] *100)/ distance to target;
	so CPDeltaX = (SPeed10value * deltaX *100)/D

	Also used for Huddle2LOS and GoToSidelines(running)

--------------------------------------------------------------------*/

void SetDeltaParameters(UWORD targetx, UWORD targety, WORD speed0_20)
{
LONG dx,dy,divedist;
//WORD dist;
EFFVAR	dist;

	dx = (LONG)targetx - (LONG)CPXPos;
	dy = (LONG)targety - (LONG)CPYPos;
	dist = Distance(targetx,targety);


	divedist = SpeedTable[speed0_20][EAST_DIR].X;	// max distance (Y=0)

	if(dist == ZERO)	// no divide by zero
	{
		CPDeltaX = 	CPDeltaY = ZERO;
	}
	else
	{
		CPDeltaX = (divedist*dx)/dist;
		CPDeltaY = (divedist*dy)/dist;

	}
}


#if 0	//PC_DEBUG_MODE

UWORD dtx[16] ={0x5700,0x5800,0x5900,0x5700,0x5800,0x5900,0x5700,0x5800,0x5900};
UWORD dty[16] ={0x5700,0x5700,0x5700,0x5800,0x5800,0x5800,0x5900,0x5900,0x5900};

void TestDive(void)
{
//WORD i,lc;
EFFVAR	i,lc;

	gPNum = 0;
	CPXPos = CPYPos = 0x5800;
	printf("\ndivedist = %d",SpeedTable[DIVE_SPEED][EAST_DIR].X);	// max distance (Y=0)

//	for(i=0; i<9; i++)
	for(i=0x3800,lc=0; i<0x7801; i+=0x0100,lc++)
	{
//		SetDiveParameters(dtx[i],dty[i]);
		SetDeltaParameters(i,0x5000,DIVE_SPEED);
		printf("\n%2d dx = %4d, dy = %4d",lc,CPDeltaX,CPDeltaY);
		if(lc%25 == 24)
			getch();
	}

}
#endif

/*------------------------------------------------------------------*/
/***********************************************************************

				 S P I N N I N G

	SPINNING is one of the few ACTIONs which also does the STATE.

***********************************************************************/
void SpinMoveSubroutine(void)
/* needed because ACTION invokes STATE and we don't
 want to do that first time through			*/

{
	SetBit(CPFlags,IM_SPINNING_BFLAG);

// inc twice but be sure to catch correct finish of spin
	CPFacing++;
	CPFacing &=15;
	if(CPFacing == CPDirToGo)
		SetCPAction(NoAction);      // resets IM_SPINNING
	else
	{
		CPFacing++;
		CPFacing &=15;
		if(CPFacing == CPDirToGo)
			SetCPAction(NoAction);      // resets IM_SPINNING
	}
}


PLAYER_ACTION(SpinMoveAction)
{
	SpinMoveSubroutine();

	DoState();		// still do whatever they were supposed to do
}



void InitSpinMove(void)		// called from a STATE
{
	CPNumSpinMoves++;
	SetCPAction(SpinMoveAction);  // will do next frame
	SpinMoveSubroutine();		// will set IM_SPINNING_BFLAG
}


/***********************************************************************

 following initiated if a human toggles off a defensive player before
	the snap - idea is to put guy back to a normal looking position

********************************************************************/

void DefBeforeSnapAction(void)
{

	if(CPFacing != WEST_DIR)
	{
		Shuffle(NORMAL_SHUFFLE);
//		InitGraphics(AI_STAND_NUM);
		TurnPlayerOptimal(WEST_DIR);
		CPZPos /= 2;	//bug #1583 - toggled off of while jumping

	}
	else
	{
		CPZPos = ZERO;	//bug #1583 - toggled off of while jumping
		CPDirToGo = WEST_DIR;		// in case they dive
		SetCPAction(NoAction);	// assume - could change below

		if(DefShouldGoTo4PointStance())
		{      // goto  4pt stance if not already in it

			if(CPAIGraphicState != AI_4PTSTANCE_NUM)
			{
				InitGraphics(AI_4PTSTANCE_NUM);
				SetCPAction(CycleAnimActionAndEnd);
			}
		}
		else	// normal guy facing West
		{
			InitGraphics(AI_2PTSTANCE_NUM);
		}
	}
}



/**********************************************************************

				 T H E    S T A T E S



***********************************************************************

		SNAP (HIKE)   STATES

	Note:  Humans CAN NOT take control of offensive players until
	QB (or punter) has ball.

	invoked by "SetBit(EventFlags,OK_FOR_HUMAN_OFF_CONTROL_BFLAG)"

***********************************************************************/
//void SetCPRatings(void);	//sets gPNum's constant ratings

STATEDEF(Wait4SnapShuffle)	/* used by guys who went in motion	*/
{
//	CPSpeed = ZERO;		// in case guy was in motion
	if(BallSnapped)
	{
		SetCPRatings(0);	//sets gPNum's constant ratings in case changed
		GoToNextState(1);
	}
	else
		Shuffle(NORMAL_SHUFFLE);		// Shuffle() sets speed to a low value
}


STATEDEF(Wait4Snap)		/*  used by both OFFENSE and DEFENSE	*/
{
EFFVAR percent;

	CPSpeed = ZERO;		// in case guy was in motion

	if(BallSnapped)
	{ // make all linemen down in crouch get up
	  //	- assume they're in "last" sequence when started
/* test only
		SetCPRatings();	//sets gPNum's constant ratings in case changed
*/
		if(BallCarrier == QB
		   && OPXPos(BallCarrier) > StartBallX-2*YARDS
		   && DistanceToPlayer(BallCarrier) < 9*HALF_YARD
		   && gPNum > 10
		  )
		{  	// QB Sneak
			if(OPFlags(BallCarrier) & IS_HCONTROLLED_BFLAG)
				percent = 50; //80	// punish Mr. Locke (and all humans)
			else
				percent = 15; //20	// give CPU (normal sneak a break

			if(randper < percent)
			{
				StartDive(TRUE);	// this frame
				GoToNextState(1);
				return;				// always leave any STATE once GoToNextState
			}
		}

		if(CPStateInit != 99)	// use 99 so def. pointing guy will leave this STATE
		{
			CPReactTime = 4 + (rand()&15);	// 1 -5 frames
			CPStateInit = 99;
		}
	// offense react right a way, defense takes a while
		if(gPNum < 11 || TimeIntoPlay > CPReactTime)
		{
			if(	CPAIGraphicState == AI_3PTSTANCE_NUM
				|| CPAIGraphicState == AI_4PTSTANCE_NUM)
			{   // need to get up quickly since going down to stance table  doubles frames

				if(CPFlags1 & IM_BLOCKING_HUMAN_BFLAG)	//get guy out of 3pt stance NOW!
				{
					GoToNextState(1);
				}
				else
				{
					--CPAIGraphicSequence;		//
					if(CPXPos < StartBallX-4*YARDS)	// fb
						--CPAIGraphicSequence;		//
					if(--CPAIGraphicSequence <= ZERO)
					{
						CPAIGraphicSequence = ZERO; // just to be safe
						GoToNextState(1);
					}
				}
			}
			else
				GoToNextState(1);
		}
	}
	else
	{
		DoDefPreSnapAnim();	//if necessary - pointing or hand waving
	}
}





void DoDefPreSnapAnim(void)	//if necessary - pointing or hand waving
{
//WORD rnum;
EFFVAR	rnum;
//WORD t = -1;
EFFVAR	t = -1;

	if(	gPNum == 11 && PlayType != KICKOFF_PLAY
		&&  (InterfaceFlags & AMINS_LOADED_BFLAG)
	  )
	{ /*  do a def. animation - pointing or raising hands to crowd */
		rnum = rand();	// only do one call to rand()

		switch (CPStateInit)	// use StateInit as internal State(switch) counter
		{

			case 0:		//1st time in - init counter
				NumDefPresnapAnims  = ZERO;
				// NO break - continue with below

			case 1:	// set timer for next anim
//				CPTimer = SECOND + (rnum&0xff); // 1.0 to 5.1 seconds
				CPTimer = HALF_SECOND + (rnum&0x7f); // 0.5 to 2.5 seconds
				CPStateInit = 2;
				CPFacing = WEST_DIR;
				InitGraphics(AI_2PTSTANCE_NUM);
				break;

			case 2:	// if timer over, start anim
				if( (CPTimer -= TICKS_PER_AI_FRAME) <= 0)
				{
					if( ((rnum&0x0003) == ZERO) )   //bug #1860-happened too much
					{
						if(TheDown == 3 && ImOnHomeTeam())
							t = AI_DEFENSE_PRESNAP2_NUM;	// arms up
						else
						{
							t = AI_DEFENSE_PRESNAP1_NUM;	// pointing
//							AIPlayEffectCheck(GET_HIM_SFX);
							if(rnum&0x0010 )
								AIPlayEffectCheck(OVER_THERE_SFX);
							else
								AIPlayEffectCheck(WATCH_EM_SFX);
						}
					}

					if(t >= ZERO)
					{
						CPFacing = (rnum & 0x0040) ? SOUTH_DIR : NORTH_DIR;
						InitGraphics(t);
						CPStateInit = 3;
				// pointing needs to be a STATE, not an ACTION
				// so human can interrupt
				//	SetCPAction(CycleAnimActionAndEnd);
					//	SetBit(CPFlags1,IM_POINTING_BFLAG);
					}
					else
						CPStateInit = 1;	// next time in STATE re-init timer
				}
				break;

			case 3:		// executing animation
				if(IncFramesCheckAtEnd())	//done???
				{
					if(++NumDefPresnapAnims > 1)
						CPStateInit = 4;	// only do it twice
					else
						CPStateInit = 1;	// next time in STATE re-init timer
				}
				break;

			default:
//			case 4:
				CPFacing = WEST_DIR;
				InitGraphics(AI_2PTSTANCE_NUM);
				break;
		}	// end switch
	}
}


#if 0
void AllowHumanOffensiveControl(void)
{
//WORD i,cntl;
EFFVAR	i,cntl;

	SetBit1(EventFlags,OK_FOR_HUMAN_OFF_CONTROL_BFLAG);
	for(i=1; i < 4; i++)
	{	// allow non QB to control players even if pressing down before
		if( (cntl = GameTeamData[OffensiveSlotNum].ControllerNum[i]) != NO_CONTROLLER)
				PreviousJoyReading[cntl] = ZERO;
	}

}
#endif
/*--------------------------------------------------------------------

				H I K I N G     S T A T E S

--------------------------------------------------------------------*/

void QBGetsBall(BOOL punt)		/*  a STATE subroutine	*/
{
//WORD i;
EFFVAR	i;

	if(FumbleShouldHappenOnHike(gPNum))
	{
		InitFumble(BALL_HIT_GUY_FUMBLE,rand()&15,NEED_FUMBLE_SFX);
	}
	else
	{
		// next line for bug #1363 (player 0 doesn;t get ball on hike)
//		SwapControllers(gPNum,ControlWhichPlayer[GameTeamData[OffensiveSlotNum].ControllerNum[0]]);
		SwapControllers(gPNum,0);	//normally QB

		SetNewBallCarrier(gPNum);		// sets BS(PlayerHasBall)
		if( !(GameTeamData[OffensiveSlotNum].PlayNum == SpecialTeamsFieldGoalNUM
		   ||	GameTeamData[OffensiveSlotNum].PlayNum == SpecialTeamsQBDownBallNUM))
		{
			SetBit1(EventFlags,OK_FOR_HUMAN_OFF_CONTROL_BFLAG);
//			AllowHumanOffensiveControl();
		}
		if ( (i = GameTeamData[OffensiveSlotNum].ControllerNum[0]) != -1)
// clear any current stuff(which will become previous for next frame)
// this allows a human to controll the QB EVEN if he was holding JPad now
				ResetBit(AIJoyPadData[i],JPADL_MASK);
//				ResetBit(PreviousJoyReading[i],JPADL_MASK);


		if(punt)
		{
			SetCPAction(PuntAction);	// next time do it
			PuntAction();		// and do it now!
		}

		GoToNextState(1);
	}
}


STATEDEF(QBSnap)	/* normal hike on pass plays - get ball directly from center */
{
	if(BallSnapped)
	{
		QBGetsBall(FALSE);		// will GoToNextState, FALSE is QB, TRUE is punter
	}
}

	/* below is normal hike on run plays - get ball directly from center	*/
STATEDEF(QBSnapThenHandOff)
{
	if(BallSnapped)
	{
		QBGetsBall(FALSE);		// will GoToNextState, FALSE is QB, TRUE is punter

		SetAI(StopAfterHandOff);
	}
}


STATEDEF(QBSGSnap)	/*  inits ball to be hike backward in AIR to QB or punter */
{
	if(BallSnapped)
	{
		InitLongHike();
		GoToNextState(1);
	}
}


STATEDEF(QBSGSnapThenHandOff)
{
	if(BallSnapped)
	{
		InitLongHike();
		SetAI(StopAfterHandOff);
		GoToNextState(1);
	}
}

STATEDEF(QBGetSGSnap)
{
	if(HikeXPosition < (CPXPos + 5* QTR_YARD))	// was YARD - mod. 2/11/96 bug #1302
	{
		QBGetsBall(FALSE);		// will GoToNextState, FALSE is QB, TRUE is punter
		if(CPFlags1 & IM_HOLDER_BFLAG)
		{
			InitGraphics(AI_FGHOLDER_NUM);
			CPAIGraphicSequence = 1;	//have ball in hands (AI_FGHOLDER_NUM);
		}
		else // only use punting anim for 1 frame - shows players getting ball
		{
//		punting anim caused extra memory problems on PSX - use pitch instead
//			InitGraphics(AI_PUNTING_NUM);	// if necessary
			InitGraphics(AI_QBPITCH_NUM);		// if necessary

		}
	}
}


STATEDEF(PunterGetSGSnap)	// punter must immediately go to punt ACTION
{							// ... so human can't taake over
	if(HikeXPosition < (CPXPos + 5* QTR_YARD))	// was YARD - mod. 2/11/96 bug #1302
	{
		QBGetsBall(TRUE);		// will GoToNextState - TRUE means punt
	}

}

#ifdef NEW_LONG_HIKE
/*   new Long hike uses different ball parameters so as not
		 to interfere with Kicking arc stuff
	*/


void InitLongHike(void)		/*  shotgun or Punt	*/
{ 						/* a subroutine of a Player STATE	*/
//WORD catchheight;
EFFVAR	catchheight;

	if(PlayFlags & POS_FG_PLAY_BFLAG)
//	if(PlayType == FIELD_GOAL_PLAY || PlayType == EX_PT1_PLAY)
// PlayType Not set to FGPlay at InitAI - only when ball kicked
		catchheight = 2*FEET;
	else
		catchheight = 4*FEET;

	HikeXPosition = BallXPosition;
	HikeYPosition = BallYPosition;	// only for HB snap
	HikeZPosition = ZERO;	//ball starts on ground and will rise to QB's hands
	HikeTargetX = CPXPos;		// ball will begin dropping after reaching this target


	if(HikeXPosition-HikeTargetX == ZERO)	// screen illegal divide
		HikeBallDeltaZ  = FOOT/4;
	else
		HikeBallDeltaZ = catchheight/((HikeXPosition-HikeTargetX)/HikeSpeed);

	HikeXPosition -= HikeSpeed;
	HikeDeltaY = (CPYPos-HikeYPosition)/8;	// only for HB snap


	SetBallAction(BallLongHikeAction);

}

#else

#define HikeSpeed	200
void InitLongHike(void)		/*  shotgun or Punt	*/
{ /* a subroutine of a Player STATE	*/
//WORD catchheight;
EFFVAR	catchheight;

	if(PlayFlags & POS_FG_PLAY_BFLAG)
//	if(PlayType == FIELD_GOAL_PLAY || PlayType == EX_PT1_PLAY)
// PlayType Not set to FGPlay at InitAI - only when ball kicked
		catchheight = 2*FEET;
	else
		catchheight = 4*FEET;

	BallDeltaX = -HikeSpeed;		/* 256 = 1 yard per frame  */
	BallDeltaY = 0;				// straight back
	BallZPosition = ZERO;	//ball starts on ground and will rise to QB's hands
	BallTargetX = CPXPos;		// ball will begin dropping after reaching this target


	if(BallXPosition-BallTargetX == ZERO)	// screen illegal divide
		CurrentBallDeltaZ  = FOOT/4;
	else
		CurrentBallDeltaZ = catchheight/((BallXPosition-BallTargetX)/HikeSpeed);

	SetBallAction(BallLongHikeAction);

}
#endif

/*--------------------------------------------------------------------

				Q U A R T E R B A C K     S T A T E S

--------------------------------------------------------------------*/

STATEDEF(QBInitFleaFlicker)		// 1st STATE of Fleaflick SCRIPT
{
	ResetBit(CPFlags,CAN_PASS_BFLAG);	//override normal setup from PlayInit
	QBEndOfPlayScriptNum = QBFleaFlickerPart2ScriptNUM;	// called after HO is completed
	QBHandoffFreezeTime = QUARTER_SECOND-3;	//12;
	GoToNextState(1);
}

STATEDEF(QBFleaFlickerAfterHandOff)
{
	SetBit(CPFlags,CAN_PASS_BFLAG);
//	CPFacing = WEST_DIR;	 still in pitch anim - lloks bad
	FutureBC = gPNum;

	QBEndOfPlayScriptNum = WatchEOPScriptNUM;
	GoToNextState(1);

}

STATEDEF(QBFleaFlickerWaitForPitch)
{	// make sure QB has ball BEFORE Look for Receiver
	if(ImBallCarrierMACRO)
	{
		GoToNextState(1);
	}
	else
	{
		FaceBallandShuffle();
	}

}

STATEDEF(SetQBAsFBCState)	// for flea flicker - part of Rb script
{
	FutureBC = ZERO;	// slot 0 = QB
	GoToNextState(1);
}

STATEDEF(QBLookToPassState)	// used on shotgun draw
{
	if(!CPStateInit)
	{
		CPStateInit = TRUE;
		CPTimer = *(CPScriptPtr+1);		/* get # of ticks to wait	*/
	}
	SetLookToPassGraphics(-1);		// means random
	if( (CPTimer -= TICKS_PER_AI_FRAME) <= 0)
		GoToNextState(2);

}



STATEDEF(InitQBDrawPlayState)	// used by QB all draws before snap
{

	SetBit(CPFlags1,QB_CAN_BPEDAL_BFLAG);
	GoToNextState(1);

}
/******************************************************************/
#if 1
STATEDEF(DoNothing)
{
	gPNum = gPNum;		// need a line to set a break point on
//	RunInPlace();
//	DEBUG(State = DoNothing  );
//	TRACE(CPAIGraphicSequence);
}
#endif


/*******************************************************************/




STATEDEF(SetFutureBCHandoffState)
{
//	SetAI(FutureBCLookForBall);
	FutureBC = gPNum;
	SetBit(OPFlags(0),CAN_HANDOFF_BFLAG);	// tell QB it's okay to give it to me

	GoToNextState(1);
}

STATEDEF(SetFutureBCReverseState)
{
//	SetAI(FutureBCLookForBall);
	FutureBC = gPNum;
	SetBit(OPFlags(0),CAN_HANDOFF_BFLAG);	// tell QB it's okay to give it to me
	SetBit(CPFlags1,IM_REVERSE_GUY_BFLAG);	// for def. audio

	GoToNextState(1);

}

STATEDEF(SetFutureBCPitchState)
{
//	SetAI(FutureBCLookForBall);
	FutureBC = gPNum;
	SetBit(OPFlags(0),CAN_HANDOFF_BFLAG); // tell [human] QB it's okay to give it to me

	GoToNextState(1);
}




STATEDEF(ShuffleWaitTilIGetBall)	// used by RBs on Draw plays
{
	if(gPNum == BallCarrier)	// given to me by ball (or QB AI)logic
	{
		GoToNextState(1);
	}
	else
	{
		CPFacing = EAST_DIR;
		Shuffle(BLOCKING_SHUFFLE);
	}

}

STATEDEF(PShuffleWait)
{
	if(!CPStateInit)
	{
		CPStateInit = TRUE;
		CPTimer = *(CPScriptPtr+1);		/* get # of ticks to wait	*/
	}
	Shuffle(NORMAL_SHUFFLE);
	if( (CPTimer -= TICKS_PER_AI_FRAME) <= 0)
		GoToNextState(2);

}



STATEDEF(Wait4CatchShuffle)	/* used by screen blockers	*/
{
	if( (PlayFlags & OK_TO_SCREEN_BLOCK_BFLAG)
//	if( (PassCatcher != NOBODY && PassCatcher == BallCarrier )
		|| BallXPosition > StartBallX)
	{
		GoToNextState(1);		// should be LeadBlocker
	}
	else
	{
		FaceDownFieldRandom();
		Shuffle(NORMAL_SHUFFLE);		// Shuffle() sets speed to a low value
	}
}



void FaceDownFieldRandom(void)	// used by QB looking to pass or lead blockers
{

/* if facing directly down field then randomly chose down or diags,
	else if on diag then face down field		*/

	if(CPFacing == DOWN_FIELD_DIR-2 || CPFacing == DOWN_FIELD_DIR+2)
		CPFacing = DOWN_FIELD_DIR;
	else if(PERCENT50)					/*  50% look directly down field	*/
		CPFacing = DOWN_FIELD_DIR;
	else if(PERCENT50)
		CPFacing = DOWN_FIELD_DIR+2;	/* 25% look to right	*/
	else
		CPFacing = DOWN_FIELD_DIR-2;	/* 25% look to left	*/

}



STATEDEF(FreezePlayerWait)	/* keep in same graphic state and frame #	*/
{
	if(!CPStateInit)
	{
		CPStateInit = TRUE;
		CPTimer = *(CPScriptPtr+1);		/* get # of ticks to wait	*/
	}
	if( (CPTimer -= TICKS_PER_AI_FRAME) <= 0)
		GoToNextState(2);

}

//	CPTimer  MUST be set up before invoking this ACTION
// ( used to hold handoffframe a bit)
void FreezeAWhileAction(void)	// an ACTION
{
	if( (CPTimer -= TICKS_PER_AI_FRAME) < ZERO)
	{
		CPTimer = ZERO;
		SetCPAction(NoAction);
	}
}

/********************************************************************

					 S A V E     Q B

	After a receiver runs his route, he starts to "save the QB" by
	run to a random target in a box bounded on Y by +- 8 yards from
	QB (current ball Y) and on the X by StartBallX and
	StartBallX+31 YARDS

	However, if in this STATE and QB is "peeking in" here to see
	where I will be, I must tell QB to abort due to the random numbers
	used to figure my target.

********************************************************************/

void SaveQB(void)
{
UEFFVAR  r1,r2;

 // tell QB that this guy (possible receiver) is blocking

	if(!CPStateInit)
	{	// pick a point within a "box" around QB

		if(gPNum == DUMMY_SLOT_NUM)
		{   // qb is peeking
			r1 = AIFrameCounter*317;	//keep constant when QB peeking
			r2 = AIFrameCounter*173;
		}
		else
		{
			r1 = rand();
			r2 = rand();
		}
		CPTargetX = StartBallX + (r1&(32*YARDS-1));
		if(CPTargetX > OFF_END_LINE-1)
			CPTargetX =  OFF_END_LINE-CLOSE_TOEL;

		CPTargetY = BallYPosition + (r2&(16*YARDS-1))-8*YARDS;
		MakeTargetInBoundsNS();
		CPDirToGo = FindCPDirectionToTargetMACRO;	// only do target once
		CPStateInit = TRUE;

#ifdef PC_GRAPHIC_MODE
//		DrawCross(CPTargetX,CPTargetY,11);
#endif
	}

	if (TryManyMoves(CPDirToGo) != CANT_MOVE)
	{
		Run();
	}
	else
	{
		CPStateInit = FALSE;	// do it again
	}

	if (ReachedTarget())
	{
		CPStateInit = FALSE;	// do it again
		// second time thru
//		AbortWhereWillPlayerBe = TRUE;	// used in WhereWillPlayerBe (QB passing logic in FBBall.c)
	}

}


STATEDEF(SaveQBShort)
{
//	RunHome();
	SaveQB();
}

STATEDEF(SaveQBMedium)
{
//	RunAway();
	SaveQB();
}

STATEDEF(SaveQBDeep)
{
//    RunToMama();
	SaveQB();
}


STATEDEF(QBWatchEOP)
{
	FaceBallandShuffle();
}





/*******************************************************************

					MAN  IN   MOTION   STATES

--------------------------------------------------------------------*/
STATEDEF(WaitForReady)
{	/* used by guy doing shift (right before motion)	*/
	if(PlayStateNum >= PS_InitPlayShiftNUM)
			GoToNextState(1);	// skip over QB min time to hike ball
}

STATEDEF(WaitToGoInMotion)
// 1 Parameter on "stack" - tells CPU minimum time to hike ball
{

//	if(PlayStateNum >= 	PS_INIT_PLAY_SHIFT_NUM)
	if(PlayStateNum >= PS_SaySetNUM)
			GoToNextState(2);	// skip over QB min time to hike ball
	else
		Shuffle(NORMAL_SHUFFLE);
}


/******************** 	WARNING 	***********
- this STATE assumes a very specific SCRIPT - if ball is snapped
 the STATE is incremented 9 times to the STATE after WAIT4SNAP		*/

STATEDEF(GoInMotionBeforeMotion)
{
	SetBit(CPFlags,FORCE_WALK_BFLAG);
	if(BallSnapped)
//		GoToNextState(9);	// skip over parameter AND next state which
		GoToNextState(11);	// skip over parameter AND next state which
	else						//  ... should be waitfor snap
		RunToRelSB();

}

/*********************** 	WARNING 	***********
- this STATE assumes a very specific SCRIPT - if ball is snapped
 the STATE is incremented 5 times to the STATE after  WAIT4SNAP		*/
STATEDEF(GoInMotion)
{
//WORD i;
EFFVAR	i;

	if(!CPStateInit)	// will be set TRUE in RunToRelSB
	{
		for(i=11; i<22; i++)
		{
			if(OPGuyToCover(i) == gPNum)	// is this def. covering me?
			{
				SetOPNewScript(CoverManInMotionScriptNUM,i);
				No_Back_Pedal_Macro;
			}
		}
	}

	if(BallSnapped)
		GoToNextState(5);	// skip over parameter AND next state which
	else						//  ... should be waitfor snap
		RunToRelSB();

}





STATEDEF(InitRecParameters)	/* should be 1st STATE of any receiver SCRIPT */
{

	CPUPassType[gPNum]= *(CPScriptPtr+1);	// type of pass CPU QB should throw to each rec.
	PlayerOptimalPassTime[gPNum] = *(CPScriptPtr+2); // only need MAX RECEIVER_SLOTS

	GoToNextState(3);
}



STATEDEF(TurnRightSW)	/* TURNS without any X,Y movement	*/
{
	if(GameTeamData[CurTeamSlot].PlayFlipped)
	{
		if(TurnPlayer(NW_DIR,TURNTOLEFT))
			GoToNextState(1);
	}
	else		/* normal	*/
	{
		if(TurnPlayer(SW_DIR,TURNTORIGHT))
			GoToNextState(1);
	}
}

STATEDEF(TurnLeftNW)
{
	if(GameTeamData[CurTeamSlot].PlayFlipped)
	{
		if(TurnPlayer(SW_DIR,TURNTORIGHT))
			GoToNextState(1);
	}
	else		/* normal	*/
	{
		if(TurnPlayer(NW_DIR,TURNTOLEFT))
			GoToNextState(1);
	}
}

STATEDEF(TurnPlayerState)	// usd by punt returner
{             // MUST pass a parameter in SCRIPT

	if(TurnPlayerOptimal(*(CPScriptPtr+1)))
		GoToNextState(2);
}


/************************************************************************

					 PITCHING

	Pitching modified 6/26/96  - old way had turning built into
	the anim.  This new way the AI turns until facing the FBC.

	During the pitch, do the following (in the same frame)
		1) see if QB facing FBC, if so, pitch it
		2) turn QB 1/16 based on TurnDir
		3) repeat step 1
		4) turn QB 1/16 based on TurnDir
		5) repeat step 1
	This way we check 3 times each frame (the 1st check takes into
	account any movement of the FBC from the previous frame).


	NOTE: QB should turn at least 225 degrees, i.e if the FBC is to
	Qb left at hike, QB startd to turn to right and continues past 180
	degrees until he ends up facing the FBC.

*************************************************************************/
//WORD HumanPitchDir;
EFFVAR	HumanPitchDir;
void InitPitch(void);	// in FBBall.c
#define MAX_DIR_TURNS_PER_PITCH_FRAME	2



BOOL PitchBallSub(WORD turnvalue)	// a subroutine called by Pitching STATEs
{	// returns true if pitch was inited
//WORD i,dir2FBC;
EFFVAR	i,dir2FBC;
BOOL rv = FALSE;


	InitGraphics(AI_QBPITCH_NUM);	// only 1 frame in this anim

	if(FutureBC == NOBODY)
	{
#ifdef PC_ALWAYS_MODE
		printf("\nPitch Error to illegal FBC - %d",FutureBC);
#endif
		FutureBC = 5;	// error correction
	}


	dir2FBC = FindCPDirectionToPlayer(FutureBC);

	for(i=0; i<MAX_DIR_TURNS_PER_PITCH_FRAME+1; i++) //check current facing and X "turns"
	{

		if(CPFacing == dir2FBC)
		{	// release ball
			SwapControllers(FutureBC,gPNum);
			SetBit(OPFlags1(FutureBC), MUST_LIFT_OFF_CONTROLLER_BFLAG);
			InitPitch();
			InitGraphics(AI_HANDOFF_NUM);	// no ball in hands
			InitFreezeQBInHandoff();	// for a bit (inits ACTION)
//			GoToNextState(1);
			rv = TRUE;
			break;
		}
		else if (i < MAX_DIR_TURNS_PER_PITCH_FRAME)
		{
			CPFacing += turnvalue;
			CPFacing &= (MAX_DIR-1);
			CPDirToGo = CPFacing;
		}
	}

	return(rv);
}



STATEDEF(PitchBallRightState)
{
//WORD turnvalue;
EFFVAR	turnvalue;

	if(ImBallCarrierMACRO)
	{
		if(GameTeamData[CurTeamSlot].PlayFlipped)
			turnvalue = 1;		//to left
		else		//normal (if 225 degree movement before ball release)
			turnvalue = -1; 	//normal

		if(PitchBallSub(turnvalue))
			GoToNextState(1);	// pitch completed
	}
	else
		GoToNextState(1);

}

STATEDEF(PitchBallLeftState)
{
//WORD turnvalue;
EFFVAR	turnvalue;

	if(ImBallCarrierMACRO)
	{
		if(GameTeamData[CurTeamSlot].PlayFlipped)
			turnvalue = -1; // to right
		else
			turnvalue = 1; 	//normal (if 225 degree movement before ball release)

		if(PitchBallSub(turnvalue))
			GoToNextState(1);	// pitch completed
	}
	else
		GoToNextState(1);
}


PLAYER_ACTION(HumanPitch)
{
	PitchBallSub(HumanPitchDir);
	if(gPNum != BallCarrier)
	{
//		CPAction = NoAction; PitchBallSub freezes QB a while
		SetNewScript(QBEndOfPlayScriptNum);	 // a VARIABLE!!!
	}

}

void InitHumanPitch(void)	// if human pitch valid, init ACTION here
{
//WORD i;
EFFVAR	i;

	// figure if quicker to turn L or R
	i = FindCPDirectionToPlayer(FutureBC);
	i = i - CPFacing;		// range is -15 to 15
	if(i < 0)		// -15 maps to 1, -14 to 2, etc
		i +=	16;
	HumanPitchDir = i<9? 1: -1;

	CPAction = HumanPitch;	// next frame do this
	HumanPitch();			//and do it now

}

/************************************************************************/

STATEDEF(StandStraight)
{
	RunInPlace();
//	GoToNextState(1);
}


STATEDEF(InitRunBlock)
{
//	CPFlags |= RUN_BLOCKER_BFLAG;
	CPDirToPush = *(CPScriptPtr+1);	// WARNING - has flags in upper bits
	GoToNextState(2);
}

STATEDEF(DoNothingState)
{
	Shuffle(NORMAL_SHUFFLE);
}


STATEDEF(DoNothingUpdateGraphics)
{
#if 0	// TEST OF TD celebration
//WORD maxframesinanim;
EFFVAR	maxframesinanim;

	if(  (TDAnimLoadedNum >= 0)
	  && (TDAnimLoadedNum < MAX_CELEBRATIONS)
	  &&  (InterfaceFlags & AMINS_LOADED_BFLAG)
	  )
	{
		if(!CPStateInit)
		{
            CPXPos = CPTargetX;	// readjust guys who moved
			CPYPos = CPTargetY;	// readjust guys who moved

			CPAIGraphicCounter = CPAIGraphicSequence = 0;
			CPAIGraphicState = Celebrations[TDAnimLoadedNum];
//			InitGraphics(Celebrations[TDAnimLoadedNum]);
			CPStateInit = TRUE;
		}
		else
		{
			maxframesinanim = MAX_FRAMES_MACRO(CPAIGraphicState);
			if(++CPAIGraphicSequence > maxframesinanim)
			{
			/*  if greater than last frame number, jump higher	*/
				CPAIGraphicSequence = maxframesinanim;	// just to be safe
				CPStateInit = FALSE;
			}
		}

	}
	else
#endif
		Shuffle(NORMAL_SHUFFLE);
}



STATEDEF(Wait4Kicker)
{
	GoToNextState(1);
}


STATEDEF(Wait4BallInAir)
{
	GoToNextState(1);
}


STATEDEF(CheckForOnsideState)
{

	if(KickTargetX < StartBallX -20*YARDS)
	{
		GoToNextState(1);
		return;
	}
	else if(!CPStateInit)
	{
		CPStateInit = TRUE;
		CPTargetX = KickTargetX;
		CPTargetY = KickTargetY;
		CPDirToGo = FindCPDirectionToTargetMACRO;
	}


	if( (AIFrameCounter%6) == ZERO)
		CPDirToGo = FindCPDirectionToTargetMACRO;


	if (TryManyMoves(CPDirToGo) != CANT_MOVE)
	{
		Run();
		if (ReachedTarget())
		{
			GoToNextState(1);
		}
	}
	else
	{
		Init2ndState(4);	// parameter is number of ticks to reverse dir.

	}



}

/***
#if 1
STATEDEF(RunToTargetSub)
{
	GoToNextState(1);
}
#endif
STATEDEF(LeadBlocker)
{
	RunInPlace();
}
STATEDEF(InitOffKO)
{
	GoToNextState(1);
}
STATEDEF(Initpopup2State)
{
	GoToNextState(1);
}


STATEDEF(Removepopup2State)
{
	GoToNextState(1);
}

STATEDEF(SetToBlockStance)
{
	GoToNextState(1);
}

STATEDEF(InitKickReturner)
{
	GoToNextState(1);
}

***/

/***********************************************************************

				 S P E C I A L   Q B    S T A T E S / ACTIONS

	Bobby's "tricky" plays - Qb down ball, fake down ball,

***********************************************************************/
void TriggerDefJumpBlock(WORD dir);	// all def. players - if nesssary



PLAYER_ACTION(QBPumpFakeAction)	// used by humans
{

	CPAIGraphicState = AI_PUMPFAKE_NUM;

//	if(CPAIGraphicSequence > 2) // force def. to jump
		TriggerDefJumpBlock(CPFacing);	// all def. players - if nesssary


	if(IncFramesCheckAtEnd())
	{
		CPAction = NoAction;	/* next time back to STATE	*/
		TurnOffKickInterface();	 //
	}

}

void DefenseSayTheyPassing(void)
{
	if(randper < 50)
		AIPlayEffectCheck(THYPSSN_SFX);

}

void InitPumpFake(void)		// a sub. of human controlled player
{
//WORD fakerec;
EFFVAR	fakerec;

	InitGraphics(AI_PUMPFAKE_NUM);
	CPGraphicSequence = ZERO;		// force init

	fakerec = Receivers[rand()&1];	// randomly pick a receiver

	// allow arc to be seen during fake (or else we know it's a fake)
	CPDirToGo = CPFacing = FindCPDirectionToPlayer(fakerec); /* QB faces receiver	*/
	QuickFigurePass(fakerec);

	SetCPAction(QBPumpFakeAction);	// next frame continue with the ACTION
	DefenseSayTheyPassing();	// only somtimes

};




PLAYER_ACTION(QBFakeDownBallAction)	// a pump fake used for "wacky" play-fake down ball
{

	CPAIGraphicState = AI_PASSING_NUM;

	BallDirection = CPFacing = SW_DIR;
	CPAIGraphicSequence++;
	if (CPAIGraphicSequence >= PASS_END_FRAME_NUM)	// > to be sure
	{
		CPAction = NoAction;	/* next time back to STATE	*/
	}

}


PLAYER_ACTION(QBDownBallAction)
{

	CPAIGraphicState = AI_PASSING_NUM;

	BallDirection = CPFacing = SW_DIR;
	CPAIGraphicSequence++;
	if(CPAIGraphicSequence == PASS_RELEASE_FRAME_NUM)	// 9 (?)
	{

		ReleasePass(FALSE);	// FALSE means not a real pass to anyone

	}
	else if (CPAIGraphicSequence >= MAX_FRAMES_MACRO(CPAIGraphicState))
	{
		CPAction = NoAction;	/* next time back to STATE	*/
//		SwapControllers(TheReceiver,gPNum);		//swap at end of passing
	}

}


/*----------------------------------------------------*/

STATEDEF(FakeThrowDown)
{
	InitGraphics(AI_PASSING_NUM);

	SetCPAction(QBFakeDownBallAction);
	GoToNextState(1);
}


STATEDEF(KneelDown)	/*  last STATE of QB Kneel SCRIPT - why we have this only Bobby knows */
{

	if(CPStateInit > 5)	// if inited (and ACTION over)
	{
//		SetBit(EventFlags,(TACKLE_STARTED_BFLAG | TACKLE_ENDED_BFLAG));	//kludgey EOP
		SetBit1(EventFlags,TACKLE_STARTED_BFLAG);	//kludgey EOP
		SetBit1(EventFlags,TACKLE_ENDED_BFLAG);	//kludgey EOP
//		GoToNextState(1);	// no - stay in kneel bug +
	}
	else if (CPStateInit == ZERO)
	{

		InitGraphics(AI_GOTOHOLDER_NUM);
		CPAction = CycleAnimActionAndEnd;	// go to holder pos. via 3 pt stance

		CPFacing = CPDirToGo = EAST_DIR;

//		SetCPAction(GoTo3PtStanceInitAction);	// start the action
//		CPStateInit = TRUE;
	}

	CPStateInit++;
}


STATEDEF(ThrowDown)
{
	TheReceiver = 2;	//needed for BallBeing Passed
	SetCPAction(QBDownBallAction);
	IntendedReceiver = NOBODY;
	GoToNextState(1);

}






/***********************************************************************

				 S P E C I A L   T E A M S     S T A T E S


		Kicking , FG Holder

***********************************************************************/
void InitPuntedBall(void);	// in FBBall.c
void InitKickedoffBall(void);	// in FBBall.c
void InitFieldGoal(void);

STATEDEF(KickHold)
{
//WORD maxframesinanim;
EFFVAR	maxframesinanim;

	if(!CPStateInit)
	{
		InitGraphics(AI_FGHOLDER_NUM);	// should have been inited in QBGETSGhike
		CPStateInit = TRUE;
		CPFacing = CPDirToGo = EAST_DIR;
	}

	maxframesinanim = MAX_FRAMES_MACRO(CPAIGraphicState);

	if(gPNum == BallCarrier)	// if I still have ball
	{		// leave in NEXT to last frame(ball in hands)
		if(++CPAIGraphicSequence >= maxframesinanim-1)
		{
			CPAIGraphicSequence = maxframesinanim-1;	// just to be safe
		}
	}
	else	// last frame of anim is after ball leaves hands
	{
		CPAIGraphicSequence = maxframesinanim;
	}

// NOTE: stay here forever - never [directly] leave STATE


}




STATEDEF(SetHolderGraphicsState)
{
	SetBit(CPFlags1,IM_HOLDER_BFLAG);

	if(InterfaceFlags & AMINS_LOADED_BFLAG)
	{
	//	InitGraphics(AI_FGHOLDER_NUM);
		InitGraphics(AI_GOTOHOLDER_NUM);
		CPAction = CycleAnimActionAndEnd;	// go to holder pos. via 3 pt stance

		CPFacing = CPDirToGo = EAST_DIR;

		GoToNextState(1);
	}
	else
	{
		Shuffle(NORMAL_SHUFFLE);
	}
}




#define	PUNT_RELEASE_FRAME	4	//new 4/24/97 (9 physical frames in  punt)
//#define	PUNT_RELEASE_FRAME	7	//6		// note: 8 frames in punt, I think
//#define	PUNT_RELEASE_FRAME	10		// note: 13 frames in punt, I think

#define	KICK_RELEASE_FRAME	3		// note: 6 frames in kick, I think
//#define	KICK_RELEASE_FRAME	10		// note: 13 frames in kick, I think
void SetPuntReturnGuy(void);

PLAYER_ACTION(PuntAction)	// MUST be ACTION, not STATE, so human can't interrupt
{

	if(!InitGraphicsQ(AI_PUNTING_NUM))	// if necessary
		CPAIGraphicSequence++;
//	InitGraphics(AI_PUNTING_NUM);	// will init if not already punting
	CPStateInit = TRUE;
	CPFacing = CPDirToGo = EAST_DIR;

	TryMove(CPDirToGo,1);

	if(CPAIGraphicSequence == PUNT_RELEASE_FRAME)
	{	/*  if beginning of last frame number, release ball	*/
		InitPuntedBall();
		SetPuntReturnGuy();
	}
	else if(CPAIGraphicSequence == MAX_FRAMES_MACRO(CPAIGraphicState))
	{	/*  if at end of last frame number, exit this STATE	*/
		SetCPAction(NoAction);	// all done
	}
}


STATEDEF(DefKickerKickOff)
{
//WORD maxframes;
EFFVAR	maxframes;

	if(!CPStateInit)
	{
		InitGraphics(AI_KICKING_NUM);
		CPStateInit = TRUE;
		CPFacing = CPDirToGo = WEST_DIR;
	}
	else
	{
		TryMove(CPDirToGo,1);
		if(++CPAIGraphicSequence == KICK_RELEASE_FRAME)
		{	/*  if toe hits ball of last frame number, release ball	*/
			InitKickedoffBall();
		}


		maxframes = BallTargetX > StartBallX-19*YARDS ?
						KICK_RELEASE_FRAME : MAX_FRAMES_MACRO(CPAIGraphicState);

		if(CPAIGraphicSequence >= maxframes)
		{	/*  if at end of last frame number, exit this STATE	*/
			/*  now human can control players	*/
			SetBit1(EventFlags,OK_FOR_HUMAN_OFF_CONTROL_BFLAG);
			SetBit1(EventFlags,OK_FOR_HUMAN_DEF_CONTROL_BFLAG);
			/*  and both teams can toggle to closest to ball target  */
			SetBit(GameTeamData[OffensiveSlotNum].TeamFlags,TOGGLE_TO_CLOSEST_VALID_BFLAG);
			SetBit(GameTeamData[DefensiveSlotNum].TeamFlags,TOGGLE_TO_CLOSEST_VALID_BFLAG);

			GoToNextState(1);
		}
	}
}


STATEDEF(OffKickerKickBall)		// field goal /extra point
{

//WORD temp;
EFFVAR	temp;

	if(!CPStateInit)
	{
		InitGraphics(AI_KICKING_NUM);
		CPStateInit = TRUE;
		CPFacing = CPDirToGo = EAST_DIR;

	/* put following here as if fumble or block, they don't get 2nd chance */
		if(PlayType == NORMAL_PLAY)
			PlayType =  FIELD_GOAL_PLAY;	//not an Ex Pt
	}
	else
	{
		TryMove(CPDirToGo,1);
		if(++CPAIGraphicSequence == KICK_RELEASE_FRAME)
		{	/*  if beginning of last frame number, release ball	*/
			InitFieldGoal();
//			AllowHumanOffensiveControl();
			SetBit1(EventFlags,OK_FOR_HUMAN_OFF_CONTROL_BFLAG);

		// change holder graphic NOW! so not two balls (bug #1963)
			temp = gPNum;
			gPNum = HOLDER_SLOT_NUM;
			KickHold();		// do the STATE-sets graphic state of holder
			gPNum = temp;


		}
		else if(CPAIGraphicSequence == MAX_FRAMES_MACRO(CPAIGraphicState))
		{	/*  if at end of last frame number, exit this STATE	*/
			GoToNextState(1);
		}
	}
}




STATEDEF(PRWait4Punt)
{

	if(!CPStateInit)
	{
		InitGraphics(AI_SHUFFLE_NUM);
		CPFacing = CPDirToGo = WEST_DIR;
		CPStateInit = TRUE;
		SetBit(CPFlags,IM_PUNT_RETURNER_BFLAG);
		SetCPRatings(0);	// reset his speed to normal
	}


	if(BallCarrier != NOBODY && BallXPosition > StartBallX)
	{	// it's a fake
		SetNewScript(ChaseBallScriptNUM);
	}
	else if(BallTargetX < StartBallX
			|| (BallTargetX > OFF_GOAL_LINE - 9*YARDS) // don't get punt here
			|| (BallTargetY < YARD)) // this handles NO target
	{
		FaceBallandShuffle();
	}
	else
		GoToNextState(1);


}

/********************************************************************

					 T U R N    P L A Y E R

	This is a re-entrant routine which will turn a player 1 direction
	(1/16 of 360 degrees) every frame.

	RETURNS: TRUE if player is facing direction he wants to turn to
	(Therefore, this subroutine can be called by both an ACTION or a STATE)


	"WayToTurn" is either 1(right) or -1(left)

********************************************************************/

BOOL	TurnPlayer(WORD FinalDirection, WORD WayToTurn)
{
	CPDirToGo = FinalDirection;

	if(CPFacing == FinalDirection)
	{	/* already there?	*/
		return(TRUE);
	}
	else
	{
		CPFacing += WayToTurn;
		CPFacing &= (MAX_DIR-1);	/* keep it 0-15	*/

		if(CPFacing == FinalDirection)
		{	/* finally there?	*/
			return(TRUE);
		}
	}
	TRACE(CPFacing);
	TRACE(CPDirToGo);
	return(FALSE);
}


/*--------------------------------------------------------------------

						TURN   PLAYER OPTIMAL

	following table is number of dirs a player should turn where the
	index is the difference between FinalDirection and CPFacing - the idea
	is to quickly turn guys facing opposite direction(i.e. 8 is completely
	opposite,so a player will turn 3 dirs; diffs of 7 and 9 are nearly
	opposite, so turn a player 2 turns left or right
	4 and 12 are at right angles - turn a player one turn per "tick".

	Negative numbers mean turn turn left, positive mean turn right.
	NOTE: differences of -15 to -1 are mapped to 1 to 15

	Just changes DirFacing - NOT the graphic!!!!

------------------------------------------------------------------------*/
WORD TurnSteps[16] = {0,1,1,1,2,2,2,2,3,-2,-2,-2,-2,-1,-1,-1};

/*  following figures the best way to turn a player to the "Final Direction"
	RETURNS:   TRUE if player facing the Final direction
				else FALSE

	NOTE: Updates CPFacing but NOT CPDirToGo
*/
BOOL TurnPlayerOptimal(WORD FinalDirection)
{
//WORD turndir,facing,diff;
EFFVAR	turndir,facing,diff;

	if(CPFlags & BACK_PEDAL_BFLAG)	//Back pedaling is special case
	{
		if(gPNum < 11)
			CPFacing = DOWN_FIELD_DIR;	// offense already up field
		else			 	// no turning if back pedaling
			CPFacing = 	TOWARD_OFFENSE_DIR;	// defense always this way
		return(TRUE);
	}

	facing = CPFacing;	   // use local var as opposed to index into player array
	if(FinalDirection == facing)
	{
		return(TRUE);
	}
	else
	{
	#if 0
		if (FinalDirection >= 8)	// for upper boundry problem, look at lower portion
			if (facing < FinalDirection && facing > FinalDirection-8)
				turndir = 1;		// 1 = turn right
			else
				turndir = -1;		// -1 = turn left
		else					// for lower boundry problem look at higher portion
			if (facing > FinalDirection && facing < FinalDirection+8)
				turndir = -1;
			else
				turndir = 1;

		if(abs(FinalDirection-facing) == 8)
			turndir *= 3;
	#else
		diff = FinalDirection - facing;		// range is -15 to 15
		if(diff < 0)		// -15 maps to 1, -14 to 2, etc
			diff +=	16;
		turndir = TurnSteps[diff];

	#endif
		CPFacing += turndir;
		CPFacing &= (MAX_DIR-1);	/* keep it 0-15	*/

		if(CPFacing == FinalDirection)
		{	/* finally there?	*/
			return(TRUE);
		}
	}
//	TRACE(CPFacing);
//	TRACE(CPDirToGo);
	return(FALSE);
}

/*
WORD BitMask[16] = {
	0x0001,
	0x0002,
	0x0004,
	0x0008,

	0x0010,
	0x0020,
	0x0040,
	0x0080,

	0x0100,
	0x0200,
	0x0400,
	0x0800,

	0x1000,
	0x2000,
	0x4000,
	0x8000
};
*/
/********************************************************************


	 G R A P H I C      O R I E N T E D    S U B R O U T I N E S

	Running, walking, shuffling, dancing, etc.

-----------------------------------------------------------------*/

void InitGraphics(WORD AIgraphicsnum)
{
	if(CPAIGraphicState != AIgraphicsnum)
	{
		CPAIGraphicCounter = CPAIGraphicSequence = 0;
		CPAIGraphicState = AIgraphicsnum;
	}

}

BOOL InitGraphicsQ(WORD AIgraphicsnum)	// a Question with an answer
{
	if(CPAIGraphicState != AIgraphicsnum)
	{
		CPAIGraphicCounter = CPAIGraphicSequence = 0;
		CPAIGraphicState = AIgraphicsnum;
		return(TRUE);
	}
	return(FALSE);
}


void InitGraphicsNoReset(WORD AIgraphicsnum)
{ /* used to try to get a smooth transition between different running frames */
	if(CPAIGraphicState != AIgraphicsnum)
	{
		CPAIGraphicState = AIgraphicsnum;
		if(CPAIGraphicSequence > MAX_FRAMES_MACRO(AIgraphicsnum))
			CPAIGraphicCounter = CPAIGraphicSequence = 0;
	}

}

void InitGraphicsRandom(WORD AIgraphicsnum)
{ // used when breaking huddle so guys don't walk in unison - bug #2554 et. al.
EFFVAR maxframes;

	if(CPAIGraphicState != AIgraphicsnum)
	{
		maxframes = MAX_FRAMES_MACRO(AIgraphicsnum);	// if 10 frames(0-9) max = 9
		CPAIGraphicState = AIgraphicsnum;
		CPAIGraphicCounter = rand()&1;	// for huddletoLOS

		CPAIGraphicSequence = (rand()%(maxframes+1));

		if(CPAIGraphicSequence > maxframes)
			CPAIGraphicSequence = 0;
	}

}


BOOL ShowBackPedal(void)
{
	if(CPFlags & BACK_PEDAL_BFLAG)
	{
		if(gPNum == BallCarrier)
			InitGraphics(AI_BACKPEDAL_WBALL_NUM);
		else if ( (CPDirToGo&7) == ZERO)	// N or S??
			InitGraphics(AI_SIDESTEP_NUM);
		else
			InitGraphics(AI_BACKPEDAL_NUM);
		CYCLE_GRAPHICS_MACRO(CPAIGraphicState);
		return TRUE;
	}
	return FALSE;
}

#if 0
void  Walk(void)	this way caused skating!!!!!!
{
	if(!ShowBackPedal())
	{
		InitGraphics(AI_RUNNING_NUM);
	/* need to slow down graphics	*/
		if(++CPAIGraphicCounter > 1)		/* "greater than" test to be sure	*/
		{
			CPAIGraphicCounter = 0;
			CYCLE_GRAPHICS_MACRO(AI_RUNNING_NUM);
		}
	}
}
#else // NO Walking graphic !!!
void  Walk(void)
{
	if(!ShowBackPedal())
	{
		InitGraphics(AI_WALKING_NUM);
	/* need to slow down graphics	*/
//		if(++CPAIGraphicCounter >= 1)		/* "greater than" test to be sure	*/
//		{
//			CPAIGraphicCounter = 0;
			CYCLE_GRAPHICS_MACRO(AI_WALKING_NUM);
//		}
	}
}
#endif

BOOL OkayForRunningCatchAnim(void)
	// a special routine cause the conditional was sooo long!
{
BOOL rv=FALSE;

//	if(CPFlags & IS_HCONTROLLED_BFLAG)
	if(  gPNum == TheReceiver
       && CPControllerNum >= ZERO)	// "human" operated (but maybe not controlled
	{
		if(
			(CPFlags1 & OKAY_FOR_RCATCH_ANIM_BFLAG)
//		 && (CPFacing >= NE_DIR)	//don't have all anims in all dirs
//		 && (CPFacing <= SE_DIR)
		 && (CPFacing <= SOUTH_DIR)	// have ABC dirs
		  )
			rv = TRUE;

	}
	else	// computer controlled
	{
		if (
//				(gPNum == TheReceiver)
				(BallFlags & PASS_COMING_DOWN_BFLAG)
//			 && (BallXPosition > CPXPos - 4*YARDS)	//override passcomingdown - bug # 1475
			 && (BallXPosition > CPXPos - 3*YARDS)	//#1581 -again
			 && (BallXPosition < CPXPos + 2*YARDS)	//don't do if ball past me
			 && (BallYPosition > CPYPos - 3*YARDS)	//bug # 1581 - smaller square
			 && (BallYPosition < CPYPos + 3*YARDS)
//			 && (CPFacing >= NE_DIR)	// bug #1366
//			 && (CPFacing <= SE_DIR)
			 && (CPFacing <= SOUTH_DIR)	// have ABC dirs
			 && (!(BallFlags & NOBODY_CATCH_PASS_BFLAG))	//bug # 1899
			 && (PlayStateNum == PS_PlayHappeningNUM)		//bug # 2047
		   )	//whoa - is this the conditional statement of the year?!
			rv = TRUE;


	}
	return(rv);

}


void AdjustRunFramesBasedOnSpeed(void)
{ //	WARNING - 7/25/96  Assumes all RUN anims have 10 frames!!!
#if 0
WORD curframenum;


	curframenum = CPAIGraphicSequence;
	if(CPSpeed < (LOWEST_NORMAL_SPEED<<SPEED_SHIFT))
	{	// 6 frame anim if slow
		if(curframenum == 4 || curframenum == 9)
			CYCLE_GRAPHICS_MACRO(AI_RUNNING_NUM);

		if(curframenum == 5 || curframenum == 0)
			CYCLE_GRAPHICS_MACRO(AI_RUNNING_NUM);
	}
	else if (CPSpeed < ((HIGHEST_NORMAL_SPEED<<SPEED_SHIFT)/2))
	{	// skip 2 frames if not too fast
		if(curframenum == 5 || curframenum == 0)
			CYCLE_GRAPHICS_MACRO(AI_RUNNING_NUM);
	}
#endif
}

/*-----------------------------
	WARNING - 7/25/96  Assumes all RUN anims have 10 frames!!!
							--------------------------------------*/
void  Run(void)
{
//	if( (CPFlags & FORCE_WALK_BFLAG) || (CPSpeed < (3<<SPEED_SHIFT)) )
	if( (CPFlags & FORCE_WALK_BFLAG)) // huddle to LOS sets this (among others)
	{
		Walk();
	}
	else
	{
		if(ShowBackPedal())
		{
		}
		else if(CPFlags1 & BURST_ACTIVE_BFLAG)
		{
			InitGraphicsNoReset(AI_BURST_NUM);
			CYCLE_GRAPHICS_MACRO(AI_BURST_NUM);
		}
		else if(gPNum == BallCarrier)
		{
			InitGraphicsNoReset(AI_RUNNINGWITHBALL_NUM);
			CYCLE_GRAPHICS_MACRO(AI_RUNNINGWITHBALL_NUM);

			if( (CPXPos > StartBallX + 10*YARDS)      // off. way past LOS
			  || (BallCarrier >= START_DEF_SLOT_NUM)	// or any Def.
			  || (OPMyFormPosition(BallCarrier) == QB && CPXPos > StartBallX) // QB past LOS
			  )
			{			// carrying the ball makes one tired
				CPMaxSpeed -= 10;	// will lose 1 speed factor after 25.6 frames
				if(CPMaxSpeed < CPMinMaxSpeed)
					CPMaxSpeed = CPMinMaxSpeed;
			}
			AdjustRunFramesBasedOnSpeed();


		}
		else if(OkayForRunningCatchAnim())
		{
			InitGraphicsNoReset(AI_RUNNINGCATCH_NUM);
			CYCLE_GRAPHICS_MACRO(AI_RUNNINGCATCH_NUM);
			AdjustRunFramesBasedOnSpeed();
		}
		else if( (gPNum == TheReceiver)
			 && (!(BallFlags & PASS_COMING_DOWN_BFLAG))
			 && (CPFacing >= NE_DIR)	// bug #1366
			 && (CPFacing <= SE_DIR)
			 && (BallYPosition < CPYPos)	// bug #1706
			 )
		{
			InitGraphicsNoReset(AI_RUNNING_HEADTURN_NUM);
			CYCLE_GRAPHICS_MACRO(AI_RUNNING_HEADTURN_NUM);
			AdjustRunFramesBasedOnSpeed();
		}
		else if(CPFlags & I_RUN_BACKWARD_BFLAG)
		{
			InitGraphics(AI_RUNNING_NUM);

			CPFacing = (CPDirToGo+8)&15;	// face reverse of dir. going
			if(--CPAIGraphicSequence < ZERO)
				CPAIGraphicSequence =  MaxAIFrames[AI_RUNNING_NUM];
		}
		else
		{
			InitGraphics(AI_RUNNING_NUM);
#if 1
			if(CPSpeed < (3<<SPEED_SHIFT))
			{		//delay running anim if player at low speed
				if(++CPAIGraphicCounter >= 2)		/* "greater than" test to be sure	*/
				{
					CPAIGraphicCounter = 0;
					CYCLE_GRAPHICS_MACRO(AI_RUNNING_NUM);
//					CYCLE_GRAPHICS_MACRO(AI_WALKING_NUM);
				}
			}
			else
				CYCLE_GRAPHICS_MACRO(AI_RUNNING_NUM);
#else
		CYCLE_GRAPHICS_MACRO(AI_RUNNING_NUM);
		AdjustRunFramesBasedOnSpeed();
#endif

		}

	}
}



void RunInPlace(void)
{
	CPSpeed = ZERO;
	CPFacing = gPNum&15;	// test only
	Run();

}

/*-------------------------------------------------------------------

						 S H U F F L I N G

 have a player stand around and wiggle a bit to show he's not dead

	NOTE: On PSX, shuffle graphic is different for each direction,
	i.e N/S = hands on waist, E/W bowing(tired), NW/NE adjust helmet.
	AI doesn't care as all are sequences are the same. i.e 0,0,0,1,1,2, etc


-------------------------------------------------------------------*/

void Shuffle(WORD shuffletype)	/* a STATE subroutine	*/
{

//	if(PlayStateNum > PS_PlayHappeningNUM)
//    	CPFacing = (gPNum%8);		// bug #1998 et. al. now done in FaceBallAndShuffle

	if(CPSpeed < SHUFFLE_SPEED)
		IncrementPlayerSpeed();
	else
		CPSpeed = SHUFFLE_SPEED;

	if(gPNum == BallCarrier)
	{
		InitGraphics(AI_SHUFFLEWITHBALL_NUM);
	}
	else
	{
		if(shuffletype == BLOCKING_SHUFFLE)
		{
			InitGraphics(AI_BLOCKING_WAIT_NUM);
		}
		else if(CPAIGraphicState != AI_SHUFFLE_NUM)
		{   //init at random frame - bug # 1403
			CPAIGraphicState = AI_SHUFFLE_NUM;
//			CPAIGraphicSequence = rand()%MAX_FRAMES_MACRO(AI_SHUFFLE_NUM);
  // possible problem 10/10/96 - can't return to huddle
			CPAIGraphicSequence = rand()% (MAX_FRAMES_MACRO(AI_SHUFFLE_NUM)/2);
		}
		//		InitGraphics(AI_SHUFFLE_NUM);
	}


	CYCLE_GRAPHICS_MACRO(CPAIGraphicState);
}


/*************************************************************************
 * FACE BALL - Faces Current Player at Ball and does RANDOM SHUFFLE.
 *************************************************************************/
void FaceBallandShuffle(void)
{
#if 1	// modifued 6/20/96
	CPDirToGo = FindCPDirectionToBallMACRO;

	if(!(CPFlags & IM_SPINNING_BFLAG) )
		TurnPlayerOptimal(CPDirToGo);

	Shuffle(NORMAL_SHUFFLE);

#else
	if(PlayStateNum <= PS_PlayHappeningNUM)
	{
		CPDirToGo = FindCPDirectionToBallMACRO;
	}
	else	// at end of play
	{
//		CPDirToGo = (gPNum&15);
		CPDirToGo = FindCPDirectionToBallMACRO;

		if(!(CPFlags & IM_SPINNING_BFLAG) )
			TurnPlayerOptimal(CPDirToGo);
	}
	Shuffle(NORMAL_SHUFFLE);
#endif
}



/**********************************************************************
			QB Run To Future Ball Carrier

	Since the following STATE has no exit, the QB AI MUST have
	been set up earlier to break outta here when RB takes ball.

**********************************************************************/

BOOL QBStopBeforeHandOff(WORD fbc)
{
WORD ydiff;
BOOL rv = FALSE;

	ydiff = CPYPos - OPYPos(fbc);

	if (ydiff < (BC_SI_RANGE/2 + 70)
		&& ydiff > (-BC_SI_RANGE/2 - 70)
		&& (OPDirToGo(fbc) > NE_DIR
		&& OPDirToGo(fbc) < SE_DIR)	// if he is near my Y & going east
		&& CPXPos > OPXPos(fbc)-YARD	// not if fbc past me
		)
	{
		if(ydiff > ZERO)	// put close enough to handoff, but not bump into
		{
			CPYPos = OPYPos(fbc) + BC_SI_RANGE/2 + 35;
			CPFacing = NORTH_DIR;
		}
		else
		{
			CPYPos = OPYPos(fbc) - BC_SI_RANGE/2 - 35;
			CPFacing = SOUTH_DIR;
		}
		GeneratePlayerSphereOfInfluence(gPNum);
//		CPFacing = FindCPDirectionToPlayer(fbc);	// find direction of Future Ball Carrier
		InitGraphics(AI_QBPITCH_NUM);	// with ball in hands
		rv = TRUE;
	}

	return(rv);

}
#if 1
STATEDEF(QBRunToFutureBC)
{
//WORD fbc,dist;
EFFVAR	fbc,dist;
//WORD ydiff;

	if((fbc = FutureBC) <0)	/* just to be safe...	*/
			fbc = 4;		/* ... assume normal RB slot	*/

//	ydiff = CPYPos - OPYPos(fbc);

//	if (ydiff < 100 && ydiff > -100
//	 && (OPDirToGo(fbc) > ENE_DIR && OPDirToGo(fbc) < ESE_DIR))	// if he is near my Y & going east
//	{
//		Run();
//		return;	// figure out something better to do here...
//	}

	dist = Distance(OPXPos(fbc), OPYPos(fbc));
	if (dist > 512) // > 2 yards
	{	// use defensive logic to track FBC down
		CPDirToGo = DirVsDirTable[FindCPDirectionToPlayer(fbc)][OPDirToGo(fbc)];  // first is dir to opponent 2nd is dir of opponent

	}
	else if (QBStopBeforeHandOff(fbc))	//
	{
		return; // don't move this time
	}
	else
	{ 			// If Behind me & I'm Faster, keep going at his speed...
//		if (CPXPos > OPXPos(fbc) && CPSpeed > OPSpeed(fbc)) // If you are faster stay with him...
//			CPSpeed = OPSpeed(fbc);

		CPDirToGo = FindCPDirectionToPlayer(fbc);	// find direction of Future Ball Carrier
	}

	if (TryManyMoves(CPDirToGo) != CANT_MOVE)
		Run();
	else
		Init2ndState(2);		// only do for a short time

	if(CPXPos > StartBallX)    // how'd I get here??
	{	// obviously didn't make handoff in time
		SetNewScript(BallCarrierAvoidScriptNUM);
		SetAI(NoAI);
	}
	else
		QBStopBeforeHandOff(fbc);

}



#else

STATEDEF(QBRunToFutureBC)
{
//WORD fbc,dist;
EFFVAR	fbc,dist;
WORD ydiff;

	if((fbc = FutureBC) <0)	/* just to be safe...	*/
			fbc = 4;		/* ... assume normal RB slot	*/

	ydiff = CPYPos - OPYPos(fbc);

//	if (ydiff < 100 && ydiff > -100
//	 && (OPDirToGo(fbc) > ENE_DIR && OPDirToGo(fbc) < ESE_DIR))	// if he is near my Y & going east
//	{
//		Run();
//		return;	// figure out something better to do here...
//	}

	dist = Distance(OPXPos(fbc), OPYPos(fbc));
	if (dist > 512) // > 2 yards
	{	// use defensive logic to track FBC down
		CPDirToGo = DirVsDirTable[FindCPDirectionToPlayer(fbc)][OPDirToGo(fbc)];  // first is dir to opponent 2nd is dir of opponent

	}
	else if (ydiff < (BC_SI_RANGE/2 + 70)
			&& ydiff > (-BC_SI_RANGE/2 - 70)
			&& (OPDirToGo(fbc) > ENE_DIR
			&& OPDirToGo(fbc) < ESE_DIR)	// if he is near my Y & going east
			&& CPXPos > OPXPos(fbc)-YARD	// not if fbc past me
			)
	{
		if(ydiff > ZERO)	// put close enough to handoff, but not bump into
		{
			CPYPos = OPYPos(fbc) + BC_SI_RANGE/2 + 35;
			CPFacing = NORTH_DIR;
		}
		else
		{
			CPYPos = OPYPos(fbc) - BC_SI_RANGE/2 - 35;
			CPFacing = SOUTH_DIR;
		}
		GeneratePlayerSphereOfInfluence(gPNum);
//		CPFacing = FindCPDirectionToPlayer(fbc);	// find direction of Future Ball Carrier
		InitGraphics(AI_QBPITCH_NUM);	// with ball in hands
		return;
	}
	else
	{ 			// If Behind me & I'm Faster, keep going at his speed...
//		if (CPXPos > OPXPos(fbc) && CPSpeed > OPSpeed(fbc)) // If you are faster stay with him...
//			CPSpeed = OPSpeed(fbc);

		CPDirToGo = FindCPDirectionToPlayer(fbc);	// find direction of Future Ball Carrier
	}

	if (TryManyMoves(CPDirToGo) != CANT_MOVE)
		Run();
	else
		Init2ndState(2);		// only do for a short time

	if(CPXPos > StartBallX)    // how'd I get here??
	{	// obviously didn't make handoff in time
		SetNewScript(BallCarrierAvoidScriptNUM);
		SetAI(NoAI);
	}
}

#endif



void	ChaseBallIfClose(void)          /*  an A.I.  state	*/
{
	if(BallCarrier >= 0)
	{
//		if(OPXPos(BallCarrier) + 20*YARDS > CPXPos)
		if(DistanceToPlayer(BallCarrier) < 30*YARDS)
		{
			SetNewScript(ChaseBallScriptNUM);
			SetAI(NoAI);
		}
	}
}

STATEDEF(TestState1)
{
//	QBRunToFutureBC();
/*  used on KO by def	*/
	SetAI(ChaseBallIfClose);
	GoToNextState(1);


}

STATEDEF(TestState2)
{
	QBRunToFutureBC();
}


void InsertFaceArray(void);
STATEDEF(TwitchState)
{
	if(!CPStateInit)
	{
		CPFacing = CPDirToGo = SE_DIR;
		CPStateInit = TRUE;
		if(gPNum == ZERO)
		{
			SetNewBallCarrier(gPNum);
		}
	}

	if( (AIFrameCounter & 3) == ZERO)
	{
		if(CPDirToGo == SE_DIR)
			CPDirToGo = NE_DIR;
		else
			CPDirToGo = SE_DIR;
	}

	if(CPXPos < OFF_GOAL_LINE)
	{
		if (TryManyMoves(CPDirToGo) != CANT_MOVE)
			Run();
#ifdef PC_DEBUG_MODE
		InsertFaceArray();
#endif
	}

}

STATEDEF(Twitch2State)
{
	if(!CPStateInit)
	{
		CPFacing = CPDirToGo = SE_DIR;
		CPStateInit = TRUE;
		if(gPNum == ZERO)
		{
			SetNewBallCarrier(gPNum);
		}
	}

	if(CPDirToGo == SE_DIR)
		CPDirToGo = NE_DIR;
	else
		CPDirToGo = SE_DIR;

	if(CPXPos < OFF_GOAL_LINE)
	{
		if (TryManyMoves(CPDirToGo) != CANT_MOVE)
			Run();
#ifdef PC_DEBUG_MODE
		InsertFaceArray();
#endif
	}

}


STATEDEF(FalseStartState)
{
	SetBit(MoreEventFlags, FALSE_START_PENDING_BFLAG);
	if(CPAIGraphicState == AI_3PTSTANCE_NUM || CPAIGraphicState == AI_4PTSTANCE_NUM)
	{   //
//		--CPAIGraphicSequence;
		if(--CPAIGraphicSequence <= ZERO)
		{
			CPAIGraphicSequence = ZERO; // just to be safe
			PenaltyGuy = gPNum;	// penalty on me
			GoToNextState(1);
		}
	}
	else
		GoToNextState(1);

}

STATEDEF(FalseStartStatePart2)
{
	InitGraphics(AI_STAND_NUM);
	SetBit(MoreEventFlags, FALSE_START_HAPPENED_BFLAG);
	PenaltyGuy = gPNum;	// penalty on me
	GoToNextState(1);

}

/*****************************************************************

				P  A S S I N G    L O G I C


******************************************************************/

void InitCPUPass(WORD rec)
{
	IntendedReceiver = TheReceiver = rec;
	PassType = CPUPassType[TheReceiver];


	if(PassType < NO_PASS || PassType >= MAX_PASS_TYPES)
		PassType = LOB_PASS;	// error correction

	InitPlayerPass(NormalPassingAction);

	SetNewScript(QBEndOfPlayScriptNum);	 // a VARIABLE!!!
}





UWORD DistanceToClosestDef(WORD offguy)
{
//WORD i;
EFFVAR	i;
//UWORD closestdist = 0xffff;	// set to max
UEFFVAR	closestdist = 0xffff;	// set to max
//UWORD d;
UEFFVAR	d;

	for (i = START_DEF_SLOT_NUM; i < LAST_DEF_SLOT_NUM+1; i++)
	{
		d = Distance2Players(offguy,i);
		if ( d < closestdist)
		{
			closestdist = d;
		}
	}
	return (closestdist);
}

UWORD DirectionToClosestDef(WORD offguy)
{
//WORD i;
EFFVAR	i;
//UWORD closestdist = 0xffff;	// set to max
UEFFVAR	closestdist = 0xffff;	// set to max
//UWORD d,guy;
UEFFVAR	d,guy;

	for (i = START_DEF_SLOT_NUM; i < LAST_DEF_SLOT_NUM+1; i++)
	{
		d = Distance2Players(offguy,i);
		if ( d < closestdist)
		{
			closestdist = d;
			guy = i;
		}
	}

	return(FindDirection((OPXPos(guy)-OPXPos(offguy)),(OPYPos(guy)-OPYPos(offguy))));
}



BOOL ThrowToThisGuy(WORD receiver)
{
//UWORD dist, percent;
EFFVAR	dist, percent;

	dist = DistanceToClosestDef(receiver);

	if(dist < 2*YARDS)	// covered pretty close
	{
		percent = 10;
	}
	else if(dist < 4*YARDS)	// covered  close
	{
		percent = 50;
	}
	else if(dist < 6*YARDS)	// covered  close
	{
		percent = 75;
	}
	else
	{
		percent = 95;
	}

	if(randper < percent)
		return(TRUE);
	else
		return(FALSE);

}



#if 0	 //- not used - to LOS guys still got stuck
BOOL RunToTargetSub(void)	// used by guys running to LOS
{
//WORD rv = FALSE;
EFFVAR	rv = FALSE;

	if (TryManyMoves(CPDirToGo) != CANT_MOVE)
	{
		Run();
		if (ReachedTarget())
		{
			rv = TRUE;
		}
	}
	else
	{
		Init2ndState(5);	// parameter is number of ticks to reverse dir.
		CPStateInit = TRUE;

	}

	return(rv);
}
#endif

enum CPUQBSubstates  {
	CPUQB_NOT_INITED,
	CPUQB_THROW_TO_PRIMARY_SUBSTATE,
	CPUQB_THROW_TO_SECONDARY_SUBSTATE,
	CPUQB_THROW_TO_3RDGUY_SUBSTATE,
	CPUQB_SCRAMBLING_SUBSTATE,
	CPUQB_NEVER_PASS_SUBSTATE		// if OPTION AutoQB off
};


extern WORD TryManyMovesPush(WORD dir);	// prototype



/***************************************************************

					CPU   QB   LOGIC

	Who to pass to or when to scramble


	CPU QB throws to CPURecs[3] where 0 = primary, 1 = 2nd guy,
	etc. - set up in SetReceivers (in FBAIINIT.c)

	CPStateInit is actually used as a sub-state index.

***************************************************************/

STATEDEF(LookForReceiver)
{
//WORD rec,i,perpdir;
EFFVAR	rec,i,perpdir;
BOOL QBActive = FALSE;
//WORD secclosestunblocked,closestunblocked;
EFFVAR	secclosestunblocked,closestunblocked;
//WORD highyguy,lowyguy;
EFFVAR	highyguy,lowyguy;
//WORD keydist;
EFFVAR	keydist;
//WORD mindist;

	rec = NOBODY;

 /* if QB doesn't have ball, human must have taken control and passed it
	( control then returns to last CPU QB STATE (which is this)		*/
	if(gPNum != BallCarrier)
	{
		FaceBallandShuffle();
		SetNewScript(QBEndOfPlayScriptNum);	 // a VARIABLE!!!
	}
	else	// DON'T  use CASE statement - need to drop through some times
	{

		if(CPStateInit == CPUQB_SCRAMBLING_SUBSTATE)
		{	// QB is scrambling
//			QBActive = TRUE;	// bug - could be false if timeout

			CPTimer -= TICKS_PER_AI_FRAME;
			if(CPTimer < 0)
			{
				CPStateInit = CPUQB_NOT_INITED;	// if yes, restart logic
			}
			else
			{
				TryManyMovesPush(CPDirToGo);
				QBActive = TRUE;
			}

		}

		if(CPStateInit == CPUQB_NOT_INITED)
		{		// first time in state OR done with scrambling

		//bug #2485 - if toggled off primary, set primary to toggled guy
			if( !(PrimaryReceiver == Receivers[0]
				|| PrimaryReceiver == Receivers[1]
				|| PrimaryReceiver == Receivers[2]))
						PrimaryReceiver	= Receivers[2];


			if( (!(Q_CPUTeamMacro(OffensiveSlotNum)))
			   && GameOptions[GAME_OPTNS_AUTO_QB] == OPTION_OFF)
			{
				CPStateInit = CPUQB_NEVER_PASS_SUBSTATE;
			}
			else if(randper < 70)
			{
				rec = PrimaryReceiver;	//CPURecs[0] is same thing as primary

//				if(PlayerOptimalPassTime[rec] < AIFrameCounter)
				if(PlayerOptimalPassTime[rec] < 1)
				{	// we missed this guy, look for somebody else
					CPStateInit = CPUQB_THROW_TO_SECONDARY_SUBSTATE;	// throw to secondary
				}
				else
				{
					CPStateInit = CPUQB_THROW_TO_PRIMARY_SUBSTATE;	// throw to primary
				}
			}
			else
				CPStateInit = CPUQB_THROW_TO_SECONDARY_SUBSTATE;	// throw to secondary
		}


#if	1  // ON for real passing , OFF for Bobby test passing (or test scrambling)

		if(CPStateInit == CPUQB_THROW_TO_PRIMARY_SUBSTATE)	// passing to primary
		{
			//Receivers[CPURecIndex[CPURecs[0]] is same thing as primary
			rec = PrimaryReceiver;
//			if(PlayerOptimalPassTime[rec] < AIFrameCounter)
			if(AIFrameCounter > PlayerOptimalPassTime[rec])
			{
				InitCPUPass(rec);
				QBActive = TRUE;
			}
		}


		if(CPStateInit == CPUQB_THROW_TO_SECONDARY_SUBSTATE)	// passing to secondary
		{
			rec = Receivers[CPURecIndex[1]];

			if(PlayerOptimalPassTime[rec] < AIFrameCounter)
			{	// too late for this guy
				InitCPUPass(rec);	// then pass to last guy
				QBActive = TRUE;

			}
			else
			{
				CPStateInit = CPUQB_THROW_TO_3RDGUY_SUBSTATE;
			}
		}

		if(CPStateInit == CPUQB_THROW_TO_3RDGUY_SUBSTATE)
		{
			rec = Receivers[CPURecIndex[2]];

			if(PlayerOptimalPassTime[rec] < AIFrameCounter)
			{	// too late for this guy
				InitCPUPass(rec);
				QBActive = TRUE;
			}
		}

		if(CPStateInit == CPUQB_NEVER_PASS_SUBSTATE)// if OPTION AutoQB off
		{
			QBActive = FALSE;	// only scramble, never pass
		}
/*
		else	// something new  - 9/14/96
		{
			for (i=0; i<3; i++)
			{
				rec = Receivers[CPURecIndex[i]];

//				if( (PlayerOptimalPassTime[rec] > AIFrameCounter - 5)
//				   && (PlayerOptimalPassTime[rec] < AIFrameCounter + 5)
//				  )
				if( (PlayerOptimalPassTime[rec] < AIFrameCounter) )
				{
					mindist = (4 * YARDS)-7*AIFrameCounter;
					if(RecDistToClosestDef[CPURecIndex[i]] > mindist)
					{
						InitCPUPass(rec);
						QBActive = TRUE;
						break;
					 }
				}
			}
		}
*/
#else		// use this to test receiver timing
		if(GenValue > 5)
		{	// never throw

		}
		else
		{
			if(GenValue == ZERO)
				rec = PrimaryReceiver;
			else rec = GenValue;

			if(PlayerOptimalPassTime[rec] < AIFrameCounter)
			{
				InitCPUPass(rec);
				QBActive = TRUE;
			}
		}
#endif

		if(!QBActive)
		{	// check for scramble

			for(i = 0; i <11; i++)	// find 1st unblocked based on distance
			{
				if( !(OP_BLOCKED_MACRO(DefenseKeyDistance[i].slotnum)))
					break;
			}

			if(i<10) 	// use 10 not 11
			{
				keydist = (CPXPos > StartBallX-3*YARDS)? 3*YARDS : 12*HALF_YARD;

				if(DefenseKeyDistance[i].distance < keydist)
				{	// somebody is close
					closestunblocked = DefenseKeyDistance[i].slotnum;
					i++;
					for(; i <11; i++)	// find 2st unblocked based on distance
					{
						if( !(OP_BLOCKED_MACRO(DefenseKeyDistance[i].slotnum)))
						break;
					}
					if((i < 11) && (DefenseKeyDistance[i].distance < keydist))
					{	// two guys are close
						secclosestunblocked = DefenseKeyDistance[i].slotnum;

						// QB should go in perpendicular dir to dir from def1 to def2

						if( OPYPos(secclosestunblocked) < OPYPos(closestunblocked) )
						{
							lowyguy = OPYPos(secclosestunblocked);
							highyguy = OPYPos(closestunblocked);
						}
						else
						{
							lowyguy = OPYPos(closestunblocked);
							highyguy = OPYPos(secclosestunblocked);
						}


						perpdir = FindDirection2Points(OPXPos(lowyguy),OPYPos(lowyguy),OPXPos(highyguy),OPYPos(highyguy));

				// find midX of 2 def.  - must to two divides then add to avoid UWROD wrap
						if(CPXPos > (OPXPos(secclosestunblocked)/2 + OPYPos(closestunblocked)/2))
						{  // if past midpoint, run forward
							CPDirToGo = (perpdir - 4)&15;
						}
						else
						{   // if behind midpoint go backward
							CPDirToGo = (perpdir + 4)&15;
						}
						CPTimer = QUARTER_SECOND;

					}
					else
					{  // only one guy is close - go in opposite dir
						CPDirToGo = ((FindCPDirectionToPlayer(closestunblocked))+8)&15;
						CPTimer = QUARTER_SECOND;
					}

					if( (randper <75)
					   ||( (!(Q_CPUTeamMacro(OffensiveSlotNum)))
						   && GameOptions[GAME_OPTNS_AUTO_QB] == OPTION_OFF)
					  )

					{	// 75% of time start scramble
						CPStateInit = CPUQB_SCRAMBLING_SUBSTATE;
					}
					else
					{
						InitCPUPass(PrimaryReceiver);
						QBActive = TRUE;
					}
				}
				else
				{
					if(rec < 0)
					{
						i = -1;		// -1 means random
					}
					else
					{
						i = FindDirection2Points(CPXPos,CPYPos,OPXPos(rec),OPYPos(rec));
					}
					// does pump fake if rec behind QB
					SetLookToPassGraphics(i);
				}
			}
			else	// here if everybody is blocked (highly unlikely)
				SetLookToPassGraphics(EAST_DIR);

		}
	}
}





#if 0 	//*********************************** retired 9/14/96 ************
STATEDEF(LookForReceiverOld)
{
//WORD rec,i,perpdir;
EFFVAR	rec,i,perpdir;
BOOL QBActive = FALSE;
//WORD secclosestunblocked,closestunblocked;
EFFVAR	secclosestunblocked,closestunblocked;
//WORD highyguy,lowyguy;
EFFVAR	highyguy,lowyguy;
//WORD keydist;
EFFVAR	keydist;

	rec = NOBODY;

 /* if QB doesn't have ball, human must have taken control and passed it
	( control then returns to last CPU QB STATE (which is this)		*/
	if(gPNum != BallCarrier)
	{
		FaceBallandShuffle();
		SetNewScript(QBEndOfPlayScriptNum);	 // a VARIABLE!!!
	}
	else	// DON'T  use CASE statement - need to drop through some times
	{

		if(CPStateInit == CPUQB_SCRAMBLING_SUBSTATE)
		{	// QB is scrambling
//			QBActive = TRUE;	// bug - could be false if timeout
//			if(RunToTargetSub())	// reached target?
			CPTimer -= TICKS_PER_AI_FRAME;
			if(CPTimer < 0)
			{
				CPStateInit = CPUQB_NOT_INITED;	// if yes, restart logic
			}
			else
			{
				TryManyMovesPush(CPDirToGo);
				QBActive = TRUE;
			}

		}


		if(CPStateInit == CPUQB_NOT_INITED)		// first time in state
		{

#if ERROR_CORRECTION
			{
				if(PrimaryReceiver<0 || PrimaryReceiver >5)
				{
				#ifdef PC_ERROR_MODE
					printf("\nError on Primary Receiver- %d",PrimaryReceiver);
				#endif
				// error correct this one time
//					PrimaryReceiver = 1;   no - could be tackle eligible

				}
	//			for(i=0; i<3; i++)
	//				if(Receivers[i]<0 || Receivers[i] >5)	// error correction
	//						Receivers[i] = 1;
			}
#endif
			if( (!(Q_CPUTeamMacro(OffensiveSlotNum)))
			   && GameOptions[GAME_OPTNS_AUTO_QB] == OPTION_OFF)
			{
				CPStateInit = CPUQB_NEVER_PASS_SUBSTATE;

			}
			else if(randper < 70)
			{
				rec = PrimaryReceiver;

				if(PlayerOptimalPassTime[rec] < AIFrameCounter)
				{	// we missed this guy, look for somebody else
					CPStateInit = CPUQB_THROW_TO_SECONDARY_SUBSTATE;	// throw to secondary
				}
				else
				{
					CPStateInit = CPUQB_THROW_TO_PRIMARY_SUBSTATE;	// throw to primary
				}
			}
			else
				CPStateInit = CPUQB_THROW_TO_SECONDARY_SUBSTATE;	// throw to secondary
		}


#if	1  // ON for real passing , OFF for Bobby test passing (or test scrambling)

		if(CPStateInit == CPUQB_THROW_TO_PRIMARY_SUBSTATE)	// passing to primary
		{
			rec = PrimaryReceiver;
			if(PlayerOptimalPassTime[rec] < AIFrameCounter)
			{
				InitCPUPass(rec);
				QBActive = TRUE;
			}
		}


		if(CPStateInit == CPUQB_THROW_TO_SECONDARY_SUBSTATE)	// passing to secondary
		{
			if(Receivers[0] == PrimaryReceiver)
				rec = Receivers[1];
			else
				rec = Receivers[0];

			if(PlayerOptimalPassTime[rec] < AIFrameCounter)
			{	// too late for this guy
				InitCPUPass(Receivers[2]);	// then pass to last guy
				QBActive = TRUE;

			}
			else
			{
				CPStateInit = CPUQB_THROW_TO_3RDGUY_SUBSTATE;
			}
		}

		if(CPStateInit == CPUQB_THROW_TO_3RDGUY_SUBSTATE)
		{
			if(Receivers[0] == PrimaryReceiver)
				rec = Receivers[2];
			else
				rec = Receivers[0];


			if(PlayerOptimalPassTime[rec] < AIFrameCounter)
			{	// too late for this guy
				InitCPUPass(rec);
				QBActive = TRUE;
			}
		}

		if(CPStateInit == CPUQB_NEVER_PASS_SUBSTATE)// if OPTION AutoQB off
		{
			QBActive = FALSE;	// only scramble, never pass
		}


#else		// use this to test receiver timing
		if(GenValue > 5)
		{	// never throw

		}
		else
		{
			if(GenValue == ZERO)
				rec = PrimaryReceiver;
			else rec = GenValue;

			if(PlayerOptimalPassTime[rec] < AIFrameCounter)
			{
				InitCPUPass(rec);
				QBActive = TRUE;
			}
		}
#endif

		if(!QBActive)
		{	// check for scramble

			for(i = 0; i <11; i++)	// find 1st unblocked based on distance
			{
				if( !(OP_BLOCKED_MACRO(DefenseKeyDistance[i].slotnum)))
					break;
			}

			if(i<10) 	// use 10 not 11
			{
				keydist = (CPXPos > StartBallX-3*YARDS)? 3*YARDS : 12*HALF_YARD;

				if(DefenseKeyDistance[i].distance < keydist)
				{	// somebody is close
					closestunblocked = DefenseKeyDistance[i].slotnum;
					i++;
					for(; i <11; i++)	// find 2st unblocked based on distance
					{
						if( !(OP_BLOCKED_MACRO(DefenseKeyDistance[i].slotnum)))
						break;
					}
					if((i < 11) && (DefenseKeyDistance[i].distance < keydist))
					{	// two guys are close
						secclosestunblocked = DefenseKeyDistance[i].slotnum;

						// QB should go in perpendicular dir to dir from def1 to def2

						if( OPYPos(secclosestunblocked) < OPYPos(closestunblocked) )
						{
							lowyguy = OPYPos(secclosestunblocked);
							highyguy = OPYPos(closestunblocked);
						}
						else
						{
							lowyguy = OPYPos(closestunblocked);
							highyguy = OPYPos(secclosestunblocked);
						}


						perpdir = FindDirection2Points(OPXPos(lowyguy),OPYPos(lowyguy),OPXPos(highyguy),OPYPos(highyguy));

				// find midX of 2 def.  - must to two divides then add to avoid UWROD wrap
						if(CPXPos > (OPXPos(secclosestunblocked)/2 + OPYPos(closestunblocked)/2))
						{  // if past midpoint, run forward
							CPDirToGo = (perpdir - 4)&15;
						}
						else
						{   // if behind midpoint go backward
							CPDirToGo = (perpdir + 4)&15;
						}
						CPTimer = QUARTER_SECOND;

					}
					else
					{  // only one guy is close - go in opposite dir
						CPDirToGo = ((FindCPDirectionToPlayer(closestunblocked))+8)&15;
						CPTimer = QUARTER_SECOND;
					}

					CPStateInit = CPUQB_SCRAMBLING_SUBSTATE;

				}
				else
				{
					if(rec < 0)
					{
						i = -1;		// -1 means random
					}
					else
					{
						i = FindDirection2Points(CPXPos,CPYPos,OPXPos(rec),OPYPos(rec));
					}
					// does pump fake if rec behind QB
					SetLookToPassGraphics(i);
				}
			}
			else	// here if everybody is blocked (highly unlikely)
				SetLookToPassGraphics(EAST_DIR);

		}
	}
}

#endif




/*********************************************************************

					 F I N D     D I R E C T I O N
																N
															 -,-|+,- (x,y)
	RETURNS:  direction (0-15) to go			  quadrants =  3|0
															 W-----E
	INPUTS:   Delta X ( target - where at)        		       2|1
				Delta Y ( target - where at)          	     -,+|+,+
																S

***********************************************************************/
WORD DirStartValue[4] = { EAST_DIR, EAST_DIR, WEST_DIR, WEST_DIR};
WORD DirIncValue[4] =   { 1,-1,-1,1};


UWORD FindDirection(WORD DeltaX, WORD DeltaY)
{

//WORD quad = 0;
EFFVAR	quad = 0;
//WORD posdir, incval;
EFFVAR	posdir, incval;
register LONG   dx,dy;

	dx = (LONG)DeltaX;
	dy = (LONG)DeltaY;

	if(dx < 0)
	{
		dx = - dx;
		quad = 2;     /* quad 2 or 3  */
	}

	if(dy < 0)
	{
		dy = - dy;
		quad++;     /* quad now 1 or 3  */
	}

/* all delta values now absolute value  */

	posdir = DirStartValue[quad];
	incval = DirIncValue[quad];


	if (dy < (dx>>2))   /* divided by 4 */
	{
//    posdir = posdir + 0*incval;
	}
	else
	if (dy < 3*dx/4)
	{
		posdir += incval;
	}
	else
	if (dy < 4*dx/3)
	{
		posdir += incval+incval;  /* times 2  */
	}
	else
	if (dy < (dx<<2))   /* times 4  */
	{
		posdir += incval+incval+incval; /*times 3  */
	}
	else
		posdir += incval<<2;  /* times 4  */


	return(posdir & (MAX_DIR-1));


}


/*********************************************************************

					P L A Y E R     A. I. (SMARTS)


*********************************************************************/


void	NoAction(void)		/*  a NULL action	*/
{
		DoState();
}

void	NoAI(void)		/*  a NULL action	*/
{
}

void FreezeAction(void)
{
	/* a NULL action - no way to exit- does NOT do STATE	*/
}

void FreezeActionOnGround(void)
{
	SetBit(CPFlags,IM_LYING_ON_GROUND_BFLAG);
	CPSpeed = CPZPos = ZERO;		// abort any jumping

	if(gPNum == CameraGuy)	// will this fix lock-up in sim???
		CameraGuy = NOBODY;
	/* a NULL action - no way to exit- does NOT do STATE	*/
}


PLAYER_ACTION(FreezeQBInHandoff)
{
	if( (CPTimer -= TICKS_PER_AI_FRAME) < ZERO)
	{
		CPTimer = ZERO;
		SetCPAction(NoAction);
		DefenseAudioForRunPlay();
	}
}

void InitFreezeQBInHandoff(void)
{
//	CPTimer = QUARTER_SECOND+12; //was just QUARTER_SECOND; bug 1315, 1316
//	CPTimer = QUARTER_SECOND+24; //even longer, bug#2456 7/19/96
//	CPTimer = QUARTER_SECOND+12; //too long -screwed up fleaflick 7/31/96
	CPTimer = QBHandoffFreezeTime; // variable 10/7/96 - shorter for flea
	SetCPAction(FreezeQBInHandoff);
}

/*-------------------------------------------------------------------*/

void DoHandoff(void)
{ 	// a sub because both HUMAN and CPU have different handoff routines

	/* Handoff is a 1 frame anim with QB hands out with no ball	*/

	InitGraphics(AI_HANDOFF_NUM);	// no ball in hands
//	InitGraphics(AI_QBPITCH_NUM);	// do a frame of pitch
//	CPAIGraphicSequence = 2;		// force frame 2

	CPFacing = FindCPDirectionToPlayer(FutureBC); // bug #1448

	SetNewBallCarrier(FutureBC);		// sets BS(PlayerHasBall)
	SetNewScript(QBEndOfPlayScriptNum);	// this is a VARIABLE !!!
	SwapControllers(BallCarrier,gPNum);	// .. if necesary
	SetBit(OPFlags1(BallCarrier), MUST_LIFT_OFF_CONTROLLER_BFLAG);	//bug #2457
	SetAI(NoAI);

	InitFreezeQBInHandoff();

}


#if	1
/*  Before the Future Ball Carrier (his AI) TOOK the ball.
	Now the QB [AI] GIVES him the ball
*/
void	StopAfterHandOff(void)		/* an AI State used by QB	*/
{

	/* see if futureBC nearby....	*/
	if(FutureBC > ZERO)
	{
		if(HeCanGetBall(FutureBC))
		{  /*... then force QB to new Script which will just stop him	*/

			if(HeShouldFumbleGeneric(FutureBC) )
			{
				InitFumble(BALL_HIT_GUY_FUMBLE,BallDirection,NEED_FUMBLE_SFX);
			}
			else
			{
				DoHandoff();

			}
		}
	}
}

#else

void	StopAfterHandOff(void)		/* an AI State used by QB	*/
{

	/* if somebody's AI took the ball away from me....	*/

	if(gPNum != BallCarrier)
	{  /*... then force QB to new Script which will just stop him	*/
		SetNewScript(WatchEOPScriptNUM);

		SetAI(NoAI);
	}
}
#endif

STATEDEF(FakeHandoff)	// used on Play Action passes
{
WORD dir;

	if(CPFlags & I_HAVE_BEEN_HCONTROLLED_BFLAG)
	{	// if was human and now not- don't do handoff
		GoToNextState(2);

	}
	else if(!CPStateInit)
	{
		CPStateInit = TRUE;

		InitGraphics(AI_QBPITCH_NUM);	// only 1 frame in this anim

		dir =  *(CPScriptPtr+1);	//parameter in SCRIPT
		if(GameTeamData[CurTeamSlot].PlayFlipped)
		{
			dir = FlipDir[dir];
		}
		CPFacing = dir;

		CPTimer = QUARTER_SECOND;	//+4; 	bug # 2866
		SetBit(PlayFlags,POS_HANDOFF_BFLAG);	// tell LBs it might be a Handoff
//		DefenseAudioForRunPlay();
	}
	else
	{
		ResetBit(PlayFlags,POS_HANDOFF_BFLAG);	// only 1 chance for LBs

		if( (CPTimer -= TICKS_PER_AI_FRAME) < ZERO)
		{
			CPTimer = ZERO;
			GoToNextState(2);
			DefenseAudioForRunPlay();	// moved here from above - 11/19/96
//			SetCPAction(NoAction);
		}
	}

/**  can't use ACTION as human needs to take control
	InitGraphics(AI_QBPITCH_NUM);	// only 1 frame in this anim

	CPFacing =  *(CPScriptPtr+1);	//parameter in SCRIPT

	CPTimer = QUARTER_SECOND+4; 	//
	SetCPAction(FreezeAWhileAction);

	GoToNextState(2);
**/
}


WORD DefRunningLeftSFXs[4] = {
	LEFT_SIDE_SFX,		// whose left-off. or def??
	LOOK_LEFT_SFX,
	RUN_SFX,   		// "it's a run"
	WATCH_EM_SFX,
//	RUN_SFX,
};

WORD DefRunningUpMiddleSFXs[4] = {
	THE_MIDDLE_SFX,
	THE_MIDDLE_SFX,
	RUN_SFX,   		// "it's a run"
	WATCH_EM_SFX,
//	RUN_SFX,   		// "it's a run"
};

WORD DefRunningAnyWhereSFXs[4] = {
	WATCH_EM_SFX,
//	LEFT_SIDE_SFX,		// whose left-off. or def??
	LOOK_LEFT_SFX,
	RUN_SFX,   		// "it's a run"
	THE_MIDDLE_SFX,

};


void DefenseAudioForRunPlay(void)
{	// used after QB hands off /pitches or if play action
WORD randnum,sfx,guy;

	randnum = randper;
	if(randnum < 40)	// only say something 1/2 the time
	{
//		sfx = RUN_SFX; 		// "it's a run" - assumption

		randnum &=3;	// 0 - 3 for indexing below

		sfx = DefRunningAnyWhereSFXs[randnum];	// assume Play Action

				guy = FutureBC >= ZERO ? FutureBC : BallCarrier;
		if(OPFlags1(guy) & IM_REVERSE_GUY_BFLAG)
		{
			sfx = REVERSE_SFX;
		}
		else if(guy > ZERO)	// make sure legal
		{
			if(OPDirToGo(guy) > NE_DIR
			 && OPDirToGo(guy) < SE_DIR)
			{
				sfx = DefRunningUpMiddleSFXs[randnum];
			}
			else if(OPDirToGo(guy) <= NE_DIR
					|| OPDirToGo(guy) > NW_DIR)

			{
				sfx = DefRunningLeftSFXs[randnum];
			}
		}

		AIPlayEffectCheck(sfx);

	}
}


/********************************************************************

			 G E T     B A L L    C H E C K S

	Checks if the players X,Y is within the BALLs Sphere of Influence
	In most cases will return FALSE, so put most likely true checks
	(i.e. height) at end
-----------------------------------------------------------------*/

BOOL	ICanGetBall(void)      /*  Can "I" (gPNum) get ball ??? 	*/
{

	return(
		(CPXPos > BallSphereInfluence.Low.X)  &&
		(CPXPos < BallSphereInfluence.High.X) &&
		(CPYPos > BallSphereInfluence.Low.Y)  &&
		(CPYPos < BallSphereInfluence.High.Y) &&
		( !(CPFlags & IM_LYING_ON_GROUND_BFLAG)) &&
		(BallZPosition < GET_BALL_NORMAL_HEIGHT+CPZPos) );
}

BOOL	HeCanGetBall(WORD pnum)	  /*  Can "He" (pnum) get ball ??? 	*/

{
	return(
		(OPXPos(pnum) > BallSphereInfluence.Low.X)  &&
		(OPXPos(pnum) < BallSphereInfluence.High.X) &&
		(OPYPos(pnum) > BallSphereInfluence.Low.Y)  &&
		(OPYPos(pnum) < BallSphereInfluence.High.Y) &&
		( !(OPFlags(pnum) & IM_LYING_ON_GROUND_BFLAG)) &&
		(BallZPosition <  GET_BALL_NORMAL_HEIGHT+OPZPos(pnum) )  );
}


/*---------------------------------------------------------------------

	If the defense just got the ball, set the offense to go after them,
	and the rest of the defense to block

---------------------------------------------------------------------*/
void ClearPlayerAIStuff(WORD pnum)
{
	OPStateInit(pnum) = FALSE;
	OPAIState(pnum) = NoAI;
	ResetBit(OPFlags(pnum),BACK_PEDAL_BFLAG  | I_RUN_BACKWARD_BFLAG);
}

void SetPlayersForDefJustGotBall(void)
{
//WORD i;
EFFVAR	i;

	for(i=0; i<11; i++)
	{
		OPScriptPtr(i) = All_Scripts[ChaseBallScriptNUM];
		ClearPlayerAIStuff(i);
	}
	for(i=11; i<22; i++)
	{
		OPScriptPtr(i) = All_Scripts[DefBeBlockerScriptNUM];
		ClearPlayerAIStuff(i);
	}
	OPScriptPtr(BallCarrier) = All_Scripts[DefBallCarrierAvoidScriptNUM];

	/*  Offensive can now toggle to closest guy  */
	SetBit(GameTeamData[OffensiveSlotNum].TeamFlags,TOGGLE_TO_CLOSEST_VALID_BFLAG);
	/* def can now dive   */
	ResetBit(GameTeamData[DefensiveSlotNum].TeamFlags,TOGGLE_TO_CLOSEST_VALID_BFLAG);


}


void SetPlayersForOffJustGotBall(void)	// only used if offense got fumble
{
//WORD i;
EFFVAR	i;

	for(i=0; i<11; i++)
	{
		OPScriptPtr(i) = All_Scripts[BeBlockerScriptNUM];
		ClearPlayerAIStuff(i);
	}
	for(i=11; i<22; i++)
	{
		OPScriptPtr(i) = All_Scripts[ChaseBallScriptNUM];
		ClearPlayerAIStuff(i);
	}
	OPScriptPtr(BallCarrier) = All_Scripts[BallCarrierAvoidScriptNUM];

	/*  Defensive can now toggle to closest guy  */
	SetBit(GameTeamData[DefensiveSlotNum].TeamFlags,TOGGLE_TO_CLOSEST_VALID_BFLAG);
	/* off can now dive   */
//	ResetBit(GameTeamData[OffensiveSlotNum].TeamFlags,TOGGLE_TO_CLOSEST_VALID_BFLAG);
// no - keep them toggling


}




#if 1	// now just sets AI STATE
void AIStateGetFumble(void)
{
	// if within X yards or 2 closest guys from each team
	if(CPKeyDistance < 12 *YARDS
	   || gPNum == OffenseKeyDistance[0].slotnum
	   || gPNum == OffenseKeyDistance[1].slotnum
	   || gPNum == DefenseKeyDistance[0].slotnum
	   || gPNum == DefenseKeyDistance[1].slotnum
	  )
	{
		SetNewScript(ChaseBallScriptNUM);
		CPAIState = NoAI;
		ResetBit(CPFlags,(IM_BLOCKED_BFLAG));	// bug #1648?
/*  would never be in AI statee if player doing ACTYIN so following is unreachable
		if(CPAction == FreezeActionOnGround)
		{	// get tackled guy up - but let ALL other ACTIONs continue
			CPAction = GetUpAction;
		}  */
	}
}

void SetAllPlayersGetFumble(void)
{
//WORD i;
EFFVAR	i;

	for(i=0; i<22; i++)
	{
		OPAIState(i) = AIStateGetFumble;	// use "smarts" to get ball
		if(CPAction == FreezeActionOnGround)
		{	// get tackled guy up - but let ALL other ACTIONs continue
			CPAction = GetUpAction;
		}
	}
}

#else

void SetAllPlayersGetFumble(void)
{
//WORD i;
EFFVAR	i;

	for(i=0; i<22; i++)
	{					// leave action alone?
		OPScriptPtr(i) = All_Scripts[ChaseBallScriptNUM];
		OPStateInit(i) = FALSE;
		OPAIState(i) = NoAI;
		ResetBit(OPFlags(i),(IM_BLOCKED_BFLAG));	// bug #1648?

		if(OPAction(i) == FreezeActionOnGround)
		{	// get tackled guy up - but let ALL other ACTIONs continue
			OPAction(i) = GetUpAction;
		}
	}
}
#endif


/*-------------------------------------------------------------------

					 N E W    B A L L  C A R R I E R

	Used when somebody catches pass or gets a free ball (fumble).

	NOTE: "LastBallCarrier" should never be NOBODY, ths is usefull
	for defensive AI and  during fumble,

-------------------------------------------------------------------*/

void  SetNewBallCarrier(WORD NewBallCarrier)
{
	LastBallCarrier = BallCarrier = NewBallCarrier;
	BallFlags = ZERO;	// don't show	4/1/96
	BallTargetX = BallTargetY = ZERO;
	SetBallAction(PlayerHasBall);


}




void SetStandingCatchGraphics(void)
{

	if( CPFacing > SSW_DIR		// don't have anims in all directions
	 && CPFacing < NNW_DIR )
	{
		InitGraphics(AI_STANDINGCATCH_NUM);
	}
	else
		InitGraphics(AI_STAND_NUM);
}

/**********************************************************************

		 R E C E I V E R    R U N    T O     G E T    B A L L

	This STATE is forced to be "TheReceiver"s STATE once the pass
	has been thrown.  (Now when Pass starts to come down so he will
	stay in his script (and do cuts) as long as possible.

	He will run to the pass target unless the pass has dropped below
	a certain height, at which time he will run to the ball.

	It is the BALL logic which actually determines if the receiver
	get the ball.

	Also used by Punt Returner

**********************************************************************/

STATEDEF(RunToBallTargetState)	/* used by receiver to chase ball being passed	*/
{
//WORD i, catchheight;
EFFVAR	i, catchheight;
BOOL refiguredir = FALSE;

	catchheight = (PlayType == PUNT_PLAY)? 2*FEET : GET_BALL_NORMAL_HEIGHT-10;

	switch (CPStateInit)
	{
		case  ZERO:		/* first time in - tell em to go to target	*/
//			CPTargetX = BallTargetX;
//			CPTargetY = BallTargetY;

			refiguredir = TRUE;
			CPStateInit = 1;
//			CPDirToGo = FindCPDirectionToTargetMACRO;

			//break;		NO break - fall through to case 1


		case	1:		/* running to target	*/

			if( (AIFrameCounter & 3) == ZERO)
				refiguredir = TRUE;

			if(refiguredir)
			{
				if( (BallZPosition < GET_BALL_NORMAL_HEIGHT-40)	//-1)
//				   && (BallZPosition != ZERO  && )
				  )
				{	/* if ball "low", go for it	*/
					CPTargetX = BallXPosition;
					CPTargetY = BallYPosition;
				}
				else
				{
					CPTargetX = BallTargetX;
					CPTargetY = BallTargetY;
				}

				CPDirToGo = FindCPDirectionToTargetMACRO;
			}

			if((i = TryManyMoves(CPDirToGo)) == NO_COLLISION)
			{
				Run();
				if(ReachedTarget())
					CPStateInit = 2;
			}
			else if (i == CANT_MOVE)  /* couldn't move in one of 4 primary directions   */
			{
				Init2ndState(NORMAL_2ND_STATE_TICKS);
			}
			break;


		default:	/* should be at target	*/
#if 0

	 /*  if ball is "low" AND def. is near, OR pass was deflected
						run to ball, not target	*/
			if( ((BallZPosition < GET_BALL_NORMAL_HEIGHT+1) && (DefenseKeyDistance[0].distance < 2*YARDS))
				|| (BallFlags & PASS_DEFLECTED_BFLAG)
			  )
			{
				CPDirToGo = FindCPDirectionToBallMACRO;
				TryManyMovesOrInit2nd();
			}
			else
			{
				if(CPDistanceToBallMACRO > 3 *YARDS)
					FaceBallandShuffle();
				else
					Run();		// should make receiver do running catch
#else

			CPDirToGo = FindCPDirectionToBallMACRO;

			if( ((BallZPosition < catchheight) && (DefenseKeyDistance[0].distance < 2*YARDS))
				|| (BallFlags & PASS_DEFLECTED_BFLAG)
			  )
			{
				TryManyMovesOrInit2nd();
			}
			else
			{
				TurnPlayerOptimal(CPDirToGo);	// adjust facing
				SetStandingCatchGraphics();
			}

#endif
	}	/* end switch	*/

}


//PLAYER_ACTION(WaitAfterCatchBall)	// triggered in "DeterminePassOutcome()"
//{	// we know he's doing standing catch
//}



/*---------------------------------------------------------------------

	Kick Returner not invoked until a few seconds after ball kicked
	MOD. 2/19/96 - now handles 2 KR on same play with a passed parameter
	from SCRIPT
---------------------------------------------------------------------*/

STATEDEF(KickReturnerGetKickState)
{
//WORD krtype;
UEFFVAR	krtype;

	krtype = *(CPScriptPtr+1);	/* parameter tells left , right or ALL	*/

	if(BallTargetX | BallTargetY)
	{	// there is a ball target
		if( (krtype == KR_RIGHT && BallTargetY < MID_FIELDY)
		   || (krtype == KR_LEFT && BallTargetY > MID_FIELDY-1)
		  )
		{	// ball on other side of field, be lead blocker
			CPTargetX = BallTargetX + 5*YARDS;
			CPTargetY = BallTargetY;
		}
		else
		{
			if(BallZPosition < GET_BALL_NORMAL_HEIGHT)
			{
				CPTargetX = BallXPosition;
				CPTargetY = BallYPosition;
			}
			else
			{
				CPTargetX = BallTargetX;
				CPTargetY = BallTargetY;
			}
		}
	}
	else
	{
		CPTargetX = BallXPosition;
		CPTargetY = BallYPosition;
	}


	MakeTargetInBoundsNS();	   // don't go OOB to get ball bug #1407

	if(CPTargetX == CPXPos && CPTargetY == CPYPos)
		FaceBallandShuffle();
	else
	{
		CPDirToGo = FindCPDirectionToTargetMACRO;
		if(TryManyMoves(CPDirToGo) == NO_COLLISION)
		{
			Run();
			if(ReachedTarget())
			{		//make it the same
				CPXPos = CPTargetX;
				CPYPos = CPTargetY;
			}

		}
		else
			FaceBallandShuffle();

	}

	if(BallCarrier != NOBODY)	// somebody got ball?
	{
		if(gPNum == BallCarrier)	// given to me by ball AI logic
		{
			if(!CPUDownKickOff(gPNum))	// CPU Kick returner - if necessary
				GoToNextState(2);	// should be BCAVoid
		}
		else
		{
			SetNewScript(LeadBlockerScriptNUM);
		}
	}
}


/*********************************************************************

					BALL CARRIER AVOID

				Tables and Subroutines

*********************************************************************/

#define YOU_DECIDE -1

WORD BCRunTableEqMen[16] = {		//  dir to run based on closest guy
	ESE_DIR,		// NORTH_DIR
	ESE_DIR,		// NNE_DIR
	SE_DIR,		// NE_DIR
	SE_DIR,		// ENE_DIR

	YOU_DECIDE,		// EAST_DIR		was SE
	NE_DIR,		// ESE_DIR
	NE_DIR,		// SE_DIR
	ENE_DIR,		// SSE_DIR

	ENE_DIR,		// SOUTH_DIR
	ENE_DIR,		// SSW_DIR
	ENE_DIR,		// SW_DIR
	EAST_DIR,	// WSW_DIR

	EAST_DIR,	// WEST_DIR
	EAST_DIR,	// WNW_DIR
	ESE_DIR,		// NW_DIR
	ESE_DIR};	// NNW_DIR


WORD BCRunTableMMOnL[16] = {	 // dir to run based on closest guy, but most opponents on left
	ESE_DIR,		// NORTH_DIR
	ESE_DIR,		// NNE_DIR
	SE_DIR,		// NE_DIR
	SE_DIR,		// ENE_DIR

	SE_DIR,		// EAST_DIR
	SSE_DIR,		// ESE_DIR - try to ru around
	SOUTH_DIR,	// SE_DIR
	EAST_DIR,	// SSE_DIR

	EAST_DIR,	// SOUTH_DIR
	EAST_DIR,	// SSW_DIR
	EAST_DIR,	// SW_DIR
	EAST_DIR,	// WSW_DIR

	EAST_DIR,	// WEST_DIR
	EAST_DIR,	// WNW_DIR
	ESE_DIR,		// NW_DIR
	ESE_DIR};	// NNW_DIR


WORD BCRunTableMMOnR[16] = {	 // dir to run based on closest guy, but most opponents on right
	ENE_DIR,		// NORTH_DIR
	EAST_DIR,	// NNE_DIR
	NORTH_DIR,	// NE_DIR
	NNE_DIR,		// ENE_DIR

	NE_DIR,		// EAST_DIR
	NE_DIR,		// ESE_DIR
	NE_DIR,		// SE_DIR
	ENE_DIR,		// SSE_DIR

	ENE_DIR,		// SOUTH_DIR
	EAST_DIR,	// SSW_DIR
	EAST_DIR,	// SW_DIR
	EAST_DIR,	// WSW_DIR

	EAST_DIR,	// WEST_DIR
	EAST_DIR,	// WNW_DIR
	EAST_DIR,	// NW_DIR
	EAST_DIR};	// NNW_DIR



BOOL DefInBox(UWORD lowy,UWORD highy,UWORD highx)
{
EFFVAR i;

	for (i = 11; i < 22; i++)		// check all defensive guys
	{	// now check if non-blocked and inside box
		if(OPBlockingMe(i) == NOBODY	// FALse if Im blocked
			&& OPYPos(i) >= lowy
			&& OPYPos(i) <= highy
			&& OPXPos(i) <= highx
			&& OPXPos(i) >= CPXPos - YARD	// not behind - Sega bug#59
		  )
		{
			return(TRUE);
		}
	}
	return(FALSE);	// nobody in box

}



BOOL BCBreakThruLOS(void)
{
BOOL rv = FALSE;

	if(  (CPXPos > StartBallX - 1*YARD)
		&&(CPXPos < StartBallX + 3*YARD)
		&&(CPYPos > StartBallY - 6*YARD)
		&&(CPYPos < StartBallY + 6*YARD)
	  )
	{	// bc in box near LOS

		if(!DefInBox(CPYPos-3*HALF_YARD,CPYPos+3*HALF_YARD,CPXPos+6*HALF_YARD))
		{	// nobody in box, blockers must have done job
			CPDirToGo = EAST_DIR;
//			TryManyMovesOrInit2nd();
			InitCPUBurst(15);	// frames to burst (if not currently bursting)
			rv = TRUE;
		}
	}
	return(rv);
}


BOOL ShouldBCGoOOB(void)
{
BOOL rv=FALSE;
WORD secs,scdiff;

	if( (secs = SecondsLeftInQuarter()) > ZERO)	//must still be time left
	{
		scdiff = GameTeamData[OffensiveSlotNum].TotalScore -
						GameTeamData[DefensiveSlotNum].TotalScore;

		if(  TheQuarter == 4 	// still time left in 4th Q
			&& scdiff < 0		// my team's behind
			&& scdiff > -17     // but not too far behind
			&& secs < 125
		  )
		{
			rv = TRUE;
		}
		else if ( ((TheQuarter == 2) || (TheQuarter==4 && scdiff==ZERO))
				&& secs < 125
				&& CPXPos > DEF_GOAL_LINE + 40*YARDS
			  )
		{
			rv = TRUE;
		}

	}
	return(rv);

}

WORD BCFindClosestOpponent(WORD *dist, BYTE *left, BYTE *right); // proto

/*************************************************************************

					BALL CARRIER AVOID

	ONLY used for offense !!! (since all movement tables assume
	we want to go EAST.


*************************************************************************/
#if 1	// new as of 2/27/96

STATEDEF(BCAvoid)
{
WORD		*table;
EFFVAR	closestopp = NOBODY,			// No one is close...
		dirtogo,d,
		behinddist,
		i,val,
		closestdist = 0x7FFF,		// Far
		left = 0,
		right = 0;
BOOL refigure=FALSE;


	CPTimer -= TICKS_PER_AI_FRAME; // for Spinning

	if(CPStateInit == ZERO)
	{
		CPStateInit = 1;
		/* don't allow new BC (rec) to do spin for a while - bug #2335 */
		CPTimer = HALF_SECOND+(rand()&31);	// init time until spin
		refigure=TRUE;
	}

/*
	if(CPStateInit > 1)
	{	// bursting??
		if(CPStateInit > BC_SB_DELAY_TICKS)	// Yes -still bursting
		{
			InitSpeedBurst();	// keep doing it
		}
		CPStateInit--;		// no - delay until next burst

	}
*/

	if((AIFrameCounter&3) == ZERO)
		refigure=TRUE;


	if(BCBreakThruLOS())
	{//	return;				// kludey last minute programming

	}
	else if(refigure)
	{
		behinddist = CPXPos - 2 * YARD;	// Don't worry about guys 2 yards behind

		for (i = START_DEF_SLOT_NUM; i < LAST_DEF_SLOT_NUM+1; i++)
		{
			if( OPXPos(i) > behinddist ) // ignore guys behind BC
			{
				d = DistanceToPlayer(i);
			// figure guys to left or right of BC - weighted by closeness
				if (d < 16*YARDS)	// also ignore guys this far away
				{
					if ( d < closestdist)
					{
						closestdist = d;
						closestopp = i;
					}

					if (d > 8*YARDS)
						val = 1;
					else if (d > 4*YARDS)
						val = 2;
					else
						val = 4;

					if (OP_BLOCKED_MACRO(i)) //guys being blocked...
						val /= 2;				// ... are less important

					if (CPYPos > OPYPos(i))
						left += val;
					else
						right += val;

					left = left;	// test only - set BP here
				}
			}
		}

		dirtogo = EAST_DIR;	// default

/*
	if(closestopp == NOBODY
		|| CPYPos < NORTH_SIDE_LINE + 3*HALF_YARD
		|| CPYPos > SOUTH_SIDE_LINE - 3*HALF_YARD)
	{
		dirtogo = EAST_DIR;
	}
	else if(CPYPos < NORTH_SIDE_LINE + 7*HALF_YARD)
	{
		dirtogo = NE_DIR;
	}
	else if(CPYPos > SOUTH_SIDE_LINE - 7*HALF_YARD)
	{
		dirtogo = SE_DIR;
	}
	else*/
		if (closestdist > 3*YARDS)
		{  // if nobody "really close", main dir is EAST
			if (left > right )
				dirtogo = EAST_DIR + min(3,(left-right)/2);
			else
				dirtogo = EAST_DIR - min(3,(right - left)/2);

		}
		else
		{
			if (closestdist < 2*YARDS)
			{ // burst for X frames if not currently bursting
				InitCPUBurst(20);
			}

			if (left > right + 3 )
				table = BCRunTableMMOnL;
			else if (right > left + 3)
				table = BCRunTableMMOnR;
			else
				table = BCRunTableEqMen;

			if( (closestdist < 2*YARDS)
			   && (table != BCRunTableEqMen)
//			   && (CPNumSpinMoves == ZERO)
			   && (CPNumSpinMoves < 3)
			   && (PlayHappening())
			   && (CPTimer < 0)
			  )
			{
				InitSpinMove();
				CPTimer = HALF_SECOND+(rand()&31);	// init time until next spin
			}

			if( (dirtogo = table[FindCPDirectionToPlayer(closestopp)])
				== YOU_DECIDE)
			{
				dirtogo = CPDirToGo;	//assume
				if(dirtogo > NE_DIR && dirtogo < SE_DIR) // heading toward danger?
				{
					if(CPYPos > MID_FIELDY)	// stay on "outside"
						dirtogo = SE_DIR;
					else
						dirtogo = NE_DIR;
				}
			}
		}

	/* adjust near sidelines	if necessary  */

		if(CPYPos < NORTH_SIDE_LINE + 7*HALF_YARD)
		{
			BCStopClock = ShouldBCGoOOB();
			if(BCStopClock && closestdist < 6*HALFYARD)
			{   // try to get OOB
				dirtogo = NE_DIR;
			}
			else if(CPYPos < NORTH_SIDE_LINE + 3*HALF_YARD)
			{	// real close to SL
				if(dirtogo < EAST_DIR || dirtogo > SSE_DIR)
					dirtogo = EAST_DIR;
			}
			else if(dirtogo < NE_DIR || dirtogo > SSE_DIR)
				dirtogo = NE_DIR;
		}
		else if(CPYPos > SOUTH_SIDE_LINE - 7*HALF_YARD)
		{
			BCStopClock = ShouldBCGoOOB();
			if(BCStopClock && closestdist < 6*HALFYARD)
			{   // try to get OOB
				dirtogo = SE_DIR;
			}
			else if(CPYPos > SOUTH_SIDE_LINE - 3*HALF_YARD)
			{
				if(dirtogo > EAST_DIR)	// dir 0-4 okay
					dirtogo = EAST_DIR;
			}
			else if(dirtogo > SE_DIR)	//dir 0-6 okay
				dirtogo = SE_DIR;

		}

		CPDirToGo = dirtogo;
	}
	TryManyMovesOrInit2nd();
}

#else  // original - retired 2/27/96
STATEDEF(BCAvoid)
{
//WORD	closestopp = NOBODY,			// No one is close...
EFFVAR	closestopp = NOBODY,			// No one is close...
		dirtogo,d,
		behinddist,
		i,val,
		closestdist = 0x7FFF,		// Far
		left = 0,
		right = 0;
		
WORD		*table;


	behinddist = CPXPos - 2 * YARD;	// Don't worry about guys 2 yards behind

	for (i = START_DEF_SLOT_NUM; i < LAST_DEF_SLOT_NUM+1; i++)
	{
		if ( (OP_BLOCKED_MACRO(i)) // ignore guys being blocked
			|| (OPXPos(i) < behinddist) )		// ... and guys behind BC
		{
		}
		else
		{
			d = DistanceToPlayer(i);
			// figure guys to left or right of BC - weighted by closeness

			if (d < 16*YARDS)	// also ignore guys this far away
			{
				if ( d < closestdist)
				{
					closestdist = d;
					closestopp = i;
				}

				if (d > 8*YARDS)
					val = 1;
				else if (d > 4*YARDS)
					val = 2;
				else
					val = 4;

				if (CPYPos > OPYPos(i))
					left += val;
				else
					right += val;
			}
		}
	}

	if(closestopp == NOBODY
		|| CPYPos < NORTH_SIDE_LINE + 3*HALF_YARD
		|| CPYPos > SOUTH_SIDE_LINE - 3*HALF_YARD)
	{
		dirtogo = EAST_DIR;
	}
	else if (closestdist > 3*YARDS)
	{
		if (left > right )
			dirtogo = EAST_DIR + min(3,(left-right)/2);
		else
			dirtogo = EAST_DIR - min(3,(right - left)/2);

	}
	else
	{
		if (left > right + 3 )
			table = BCRunTableMMOnR;
		else if (right > left + 3)
			table = BCRunTableMMOnL;
		else
			table = BCRunTableEqMen;

		dirtogo = table[FindCPDirectionToPlayer(closestopp)];

	}
	CPDirToGo = dirtogo;
	TryManyMovesOrInit2nd();

}

#endif

/*************************************************************************/

/*** 	// used by punt returner, interceptor, or def who got fumble	*/
#if 1
STATEDEF(DefBCAvoid)	// uses "old" BCAvoid modified for opposite dir.
{
//WORD	closestopp = NOBODY,			// No one is close...
EFFVAR	closestopp = NOBODY,			// No one is close...
		dirtogo,d,
		behinddist,
		i,val,
		closestdist = 0x7FFF,		// Far
		left = 0,
		right = 0;
		
WORD  *table;


	behinddist = CPXPos + 2 * YARD;	// Don't worry about guys 2 yards behind

	for (i = START_OFF_SLOT_NUM; i < LAST_OFF_SLOT_NUM; i++)
	{
		if ( (OP_BLOCKED_MACRO(i)) // ignore guys being blocked
			|| (OPXPos(i) > behinddist) )		// ... and guys behind BC
		{
		}
		else
		{
			d = DistanceToPlayer(i);
			// figure guys to left or right of BC - weighted by closeness
			if (d < 16*YARDS)	// also ignore guys this far away
			{
				if ( d < closestdist)
				{
					closestdist = d;
					closestopp = i;
				}

				if (d > 8*YARDS)
					val = 1;
				else if (d > 4*YARDS)
					val = 2;
				else
					val = 4;

				if (CPYPos > OPYPos(i))
//					left += val;
					right += val;
				else
					left += val;
//					right += val;
			}
		}
	}

	if(closestopp == NOBODY
		|| CPYPos < NORTH_SIDE_LINE + 3*HALF_YARD
		|| CPYPos > SOUTH_SIDE_LINE - 3*HALF_YARD)
	{
		dirtogo = WEST_DIR;
	}
//	else if (closestdist > 3*YARDS)
	else if (closestdist > 5*YARDS)	// kick in earlier - bug #2687?
	{
		if (left > right )
//			dirtogo = WEST_DIR - min(3,(left-right)/2);
			dirtogo = WEST_DIR + min(3,(left-right));	//helps PR?
		else
//			dirtogo = WEST_DIR + min(3,(right - left)/2);
			dirtogo = WEST_DIR - min(3,(right - left));

	}
	else
	{
		if (left > right + 3 )
			table = BCRunTableMMOnR;
		else if (right > left + 3)
			table = BCRunTableMMOnL;
		else
			table = BCRunTableEqMen;

		// "reverse" index from nornal offensive table
//		dirtogo = (16- (table[(16-(FindCPDirectionToPlayer(closestopp))&15)])) &15;

		if( (dirtogo = table[(16-(FindCPDirectionToPlayer(closestopp))&15)])
				== YOU_DECIDE)
		{
			dirtogo = CPDirToGo;	//assume
			if(dirtogo > SW_DIR && dirtogo < NW_DIR) // heading toward danger?
			{
				if(CPYPos > MID_FIELDY)	// stay on "outside"
					dirtogo = SW_DIR;
				else
					dirtogo = NW_DIR;
			}
		}
		else
			dirtogo = (16- dirtogo) &15;

//	(16 - (DirVsDirTable[(16-(FindCPDirectionToPlayer(BallCarrier))&15)][OPDirToGo(BallCarrier)])&15);

	}
	CPDirToGo = dirtogo;
	TryManyMovesOrInit2nd();

}







#else


STATEDEF(DefBCAvoid)
{
//WORD i;
EFFVAR	i;

	CPDirToGo = WEST_DIR;

	if((i = TryManyMoves(CPDirToGo)) == NO_COLLISION)
	{
		Run();
	}
	else if (i == CANT_MOVE)  /* couldn't move in one of 4 primary directions   */
	{
		Init2ndState(NORMAL_2ND_STATE_TICKS);

	}

}

#endif


#define MIN_BC_SPEED  (2<<SPEED_SHIFT)

void ReduceBCSpeed(void)
{
// no more instant accel (will reset to instant accel when BC reaches max speed)
	OPAcceleration(BallCarrier) = BASIC_ACCELERATION;

	OPSpeed(BallCarrier) -= 3 * BASIC_ACCELERATION;

	if(OPSpeed(BallCarrier) < MIN_BC_SPEED)
		OPSpeed(BallCarrier) = MIN_BC_SPEED;

}

/*********************************************************************

				 C H E C K    F O R    T A C K L E


	USES: gPNum (that's the guy checking for the tackle)

	new 9/9/96 - only elligible for tackle @ 1st collision,
		then must wait X ticks until next next attempt
		based on who I'm tackling

	Need to tell calling routine what I did so graphics can be updated
	On dives, graphics have already been handled, but on
	"TryManyMovePush()", need to tell it player contacted but didn't
	tackle due to the WAIT

	RETURNS:  	1 if hit but can't tackle due to TackleWait
				2 if guy started a tackle


**********************************************************************/
#if 1
WORD  CheckForTackleAndFumble(WORD GuyHit)
{
//WORD  rv = 0;	// assume nothing happens
EFFVAR	rv = 0;	// assume nothing happens

	if(GuyHit >=ZERO && GuyHit == BallCarrier)	// screen NOBODYs
	{
		if( ((gPNum >= START_DEF_INDEX) && (BallCarrier  < START_DEF_INDEX))
			|| ((gPNum < START_DEF_INDEX) && (BallCarrier  >= START_DEF_INDEX)) )
		{   /* I (on defense) just contacted offensive BC OR vice versa  */

			if(!(EventFlags.TACKLE_ENDED_BFLAG))	// sliding QB still has ball
			{

				SetBit(MoreEventFlags,HIT_BC_THIS_FRAME);	// used to figure forward progress
				BCHitter = gPNum;	// used for personal foul

				rv= HIT_BC_BUT_CANT_TACKLE;	// so TMM-Push doesn't push & decel

				if(CPTackleCount <= ZERO)
				{   // okay - give him a chance to tackle

				 /* CPTackleCount is dec'ed @ end of AI loop, so a value of 2
					means there will be a 1 frame wait;	*/

					if(BallCarrier == FutureBC)	// a running play?
					{  // no more tackle attempts for X frames
 //						CPTackleCount = 10; // pre 10/29
//						CPTackleCount = 6;	// 10/29/96 - running too good
						CPTackleCount = 8;	// 11/09/96 - made it too bad
					}
					else if( BallCarrier >= START_DEF_INDEX
							|| OPMyFormPosition(BallCarrier) == QB)
					{
						CPTackleCount = 2;	// QB and Def are easy to tackle
					}
					else
					{
					//	CPTackleCount = 5;	// receivers
						CPTackleCount = 4;	// receivers
					}

					if(!ShouldFumbleHappenOnTackle())    /* will be fumble */
					{
						if(TackleShouldHappenNow())
						{
							StartTackle();
							rv = HIT_BC_AND_CAN_TACKLE;		// tell calling routine I'm tackling
						}
						else	// at least slow him down
						{
							ReduceBCSpeed();
						}
					}
				}
				else	// can't tackle this frame - at least slow him down
				{
#if PC_ALWAYS_MODE
					if(DisplayLevel)
						printf(".");
#endif
					ReduceBCSpeed();
					MissTackleCount++;	// test only counter
					AccumulatedTacklePercent += 1;	//easier to tackle next time
				}
			}
		}
	}
	return(rv);
}



#else 	// old - retired 9/9/96
BOOL  CheckForTackleAndFumble(WORD GuyHit)
{
BOOL  rv = FALSE;

	if(GuyHit >=ZERO && GuyHit == BallCarrier)	// screen NOBODYs
	{
		if( ((gPNum >= START_DEF_INDEX) && (BallCarrier  < START_DEF_INDEX))
			|| ((gPNum < START_DEF_INDEX) && (BallCarrier  >= START_DEF_INDEX)) )
		{   /* I (on defense) just contacted offensive BC OR vice versa  */
//			if(!IsItaTD())
//			{
//				if( !(OPFlags(BallCarrier) & IM_BEING_TACKLED_BFLAG))
				if(!(EventFlags.TACKLE_ENDED_BFLAG))	// sliding QB still has ball
				{
					if(!ShouldFumbleHappenOnTackle())    /* will be fumble */
					{
						if(TackleShouldHappenNow())
						{
							StartTackle();
							rv = TRUE;
						}
						else	// at least slow him down
						{
//							OPSpeed(BallCarrier) -= (1<<SPEED_SHIFT);
							OPSpeed(BallCarrier) -= 3*OPAcceleration(BallCarrier);
							if(OPSpeed(BallCarrier) < ZERO)
								OPSpeed(BallCarrier) = ZERO;
						}
					}
				}
//			}
		}
	}
	return(rv);
}
#endif

#if 0
BOOL IsItaTD(void)
{
	if(BallCarrier >= ZERO)
	{
		if( (BallCarrier <11 && BallXPosition >= OFF_GOAL_LINE)
			|| (BallCarrier >10 && BallXPosition <= DEF_GOAL_LINE) )
			return(TRUE);
		else
			return (FALSE);
	}
	else
		return (FALSE);

}
#endif


BOOL TackleShouldHappenNow(void)	// called by guy doing tackling
{
//WORD tacklepercent;	// = 40;	//60;
EFFVAR	tacklepercent;	// = 40;	//60;
EFFVAR tacklerating,agil;
//specificplayerdata *pdptr;
BOOL  rv = FALSE;
EFFVAR expercent = 2;	// was 4
EFFVAR rnum;

	tacklepercent  = BaseTacklePercent;	//BTP is a global modifed @ debug screen
		// base is 20 (10/2/96)
		// base is 27 (10/16/96)

	if(!OkToTackle)		// a debug variable
		return(FALSE);

	if(BallCarrier < 11)
	{
		if(BallXPosition > StartBallX + 33*YARDS)
			tacklepercent += 15;	// higher chance if long gain (11/4/96)
		else if(BallXPosition > StartBallX + 18*YARDS)
			tacklepercent += 8;	// higher chance if long gain (11/4/96)
	}

	// tackle rating range = 30-100
	tacklerating = CPTackleRating;

	agil = OPAgility(BallCarrier); // handles guys wih no Agility rating

	if(OPMyPNum(BallCarrier) >= FB && OPMyPNum(BallCarrier) <= FB2)
	{ // use RBs Strength if higher than agility
		if(CPStrength > agil)
		{
			agil = CPStrength;
			expercent = ZERO;	// harder to tackle next time
		}
	}

	tacklepercent += tacklerating/6;	// 5-16 = 25-36
	tacklepercent -= agil/10;			// 3-9    16-33

	if(agil > 90)
		tacklepercent -= 5;				//		  	11-33

	if(tacklerating > 90) 	// good guys even better
		tacklepercent += 10;			// 			11-43


	if(CPFlags & IM_DIVING_BFLAG)	//range now = 41-73
	{
//		tacklepercent += 30;	//40,20;
		tacklepercent += 25;	//11/17/96
	}

	if(OPFlags(BallCarrier) & IM_SPINNING_BFLAG)
	{
//		tacklepercent -= 20;
//		tacklepercent -= 13;	// (10/2/96)//range now = 50-66
//		tacklepercent -= 10;	// (10/17/96)
		tacklepercent -= 7;	// (10/31/96)
		if(HumanEffectValid(BallCarrier))
			tacklepercent -= 5;	// (10/31/96)
	}

	tacklepercent = SetMinMax(tacklepercent,8,75);

	tacklepercent += AccumulatedTacklePercent;

	rnum = randper;
#if 0	//PC_ALWAYS_MODE
	if(DisplayLevel)
//		printf("t=%d(%d),%d ",tacklepercent,rnum,AccumulatedTacklePercent);
		printf("t=%d(%d),%d ",tacklepercent,rnum,AIFrameCounter);
#endif

	if( (rnum < tacklepercent)
	   || (OPFlags(BallCarrier) & IM_LYING_ON_GROUND_BFLAG)) // fixes bug #2204&#3091
	{
		rv = TRUE;
	}
	else
	{	// every miss gives next tackle X% better chance
		AccumulatedTacklePercent += expercent;	//4;
		rv= FALSE;
	}
	return(rv);
}


/*******************************************************************

					FUMBLES

	Types of fumbles:	(12 total)

		1) Fumble caused by players hitting ballcarrier
			1a) Def. hits BC (tackle)
			1b) QB hands off

		2) Not catching ball
			2a) Qb gets(misses) snap-regular
			2b) Qb gets(misses) snap-shotgun
			2c) RB gets(misses) pitch
			2d) Punt Returner muffs punt
			2e) Kick Returner muffs kick

		3) Ball hits ground
			3a) RB misses pitch
			3b) Kickoff hits ground (onside???)
			3c) Long hike hits ground (shotgun, punt) (shouldn't happen)

		4) Misc
			4a) BallCarrier just drops it
			4b) FG/Punt blocked by defense (this is NOT really a fumble,
				but the ball bounces and players go after it like it
				was - won't say the phrase "Fumble" and won't set
				 FUMBLE_BFLAG)


	Most are decided by "HeShouldFumbleGeneric(WORD possiblefumbler)"

************************************************************************/
//#define SETTOWHOLEYARD(xxx)
//		((xxx) <= MID_FIELDX) ? (((xxx) + YARD-1)&0xff00) : ((xxx) & 0xff00)
/*
UWORD SetToWholeYard(UWORD Xyardline)
{ // if < 50 more to next highest, else mve to next lowest
	if(Xyardline <= MID_FIELDX)	// x00_00-x00_ff -> 0x01_00 (i.e 1 YARD)
	{
		Xyardline += YARD-1;
	}
	return(Xyardline & 0xff00);		// mask low byte	OR x50_00-x50_ff -> x50_00


}
*/

UWORD RoundToNearestWholeYard(UWORD Xyardline)
{ // if < 50 more to next highest, else mve to next lowest

	if(Xyardline <= DEF_GOAL_LINE+HALF_YARD)	// x00_00-x00_ff -> 0x01_00 (i.e 1 YARD)
	{
		Xyardline += HALF_YARD;
	}
	else if(Xyardline >= OFF_GOAL_LINE-HALF_YARD)
	{
		Xyardline -= HALF_YARD-1;
	}

	Xyardline += HALF_YARD-1;


	return(Xyardline & 0xff00);		// mask low byte	OR x50_00-x50_ff -> x50_00


}


void InitFumble(WORD fumbletype,WORD DirToGo,WORD NeedSFX)
{
//WORD temp;
EFFVAR	temp;

	if(NeedSFX)
	{                   // it is a TRUE fumble, not a blocked kicked
		AIPlayEffectCheck(FUMBLE_SFX);
		SetBit1(EventFlags, FUMBLE_HAPPENED_BFLAG);
		NumFumblesOnPlay++;
		SetBit(BallFlags,BALL_IS_FUMBLED_BFLAG);  // used for stats upkeep

	}

	Tackler = NOBODY;	// reset if half way thru

	SetBit(BallFlags,BALL_FREE_BFLAG);  // used in ChaseBall();

		// only 1 fumble per play!?!  - make sure humans have a chance
//	SetBit(EventFlags, (OK_FOR_HUMAN_OFF_CONTROL_BFLAG |
//						OK_FOR_HUMAN_DEF_CONTROL_BFLAG));
	SetBit1(EventFlags, OK_FOR_HUMAN_OFF_CONTROL_BFLAG);
	SetBit1(EventFlags, OK_FOR_HUMAN_DEF_CONTROL_BFLAG);
	ResetBit1(EventFlags,TACKLE_STARTED_BFLAG);

	/*  Defensive & Offense can now toggle to closest guy to ball */
	SetBit(GameTeamData[DefensiveSlotNum].TeamFlags,TOGGLE_TO_CLOSEST_VALID_BFLAG);
	SetBit(GameTeamData[OffensiveSlotNum].TeamFlags,TOGGLE_TO_CLOSEST_VALID_BFLAG);


	if(fumbletype == BALL_HIT_GROUND_FUMBLE
		&& PossibleFumbleX != ZERO)
	{	// this is a pitch fumble - use pitch point as "loss" yards
		FumbleX = RoundToNearestWholeYard(PossibleFumbleX); // put on exact yard for stats later
	}
	else
	{
		FumbleX = RoundToNearestWholeYard(BallXPosition); // put on exact yard for stats later
	}

	if(BallCarrier != NOBODY)
	{	// BC = NOBODY if missed pitched fumble
		if(OPAIGraphicState(BallCarrier) == AI_PASSING_NUM)
		{	// somebody was passing - bugs #1966,2148,2482
			BallTargetX = BallTargetY = ZERO;	// clear target so camera don't go crazy
			// last frame doesn't have ball in hands
			OPAIGraphicSequence(BallCarrier) = MAX_FRAMES_MACRO(AI_PASSING_NUM)-1;
		}
    	Fumbler = BallCarrier;
	}
	else
	{
		Fumbler = QB;
	}

	BallCarrier = NOBODY;

	temp = gPNum;
	for(gPNum = 0; gPNum < 22; gPNum++)
		SetReactTime(4*8);		// sets players AI to ChaseBall after X ticks

	InitBallFumble(fumbletype,DirToGo);		//in FBBall.c


//	BallXPosition += 5*YARDS;                // test only - force OOB
//	BallYPosition = SOUTH_SIDELINE + 2*YARDS;

	/*  set time for others to get ball	*/
//	FumbleFrameCounter = AIFrameCounter + 3 + (rand()&7);
//	FumbleFrameCounter = AIFrameCounter + 10;	//20; frame (NOT tick based)
	FumbleFrameCounter = AIFrameCounter + 5; //testers report problem getting ball

	gPNum = temp;

	SetAllPlayersGetFumble();	// everybody will "ChaseBall()"

	TurnOffKickInterface();	//make sure ball arc display off - bug #2813

}

//GAME_OPTNS_WEATHER 	            0-normal 1-rain 2-snow
//WORD WeatherFumblePercent[MAX_WEATHER_OPTIONS] = {0,25,10};	//based on 1000%
WORD WeatherFumblePercent[MAX_WEATHER_OPTIONS] = {0,50,60};	//based on 1000%


BOOL ShouldFumbleHappenOnTackle(void)    /* */
{	// gPNum is guy doing tackling
//WORD percent;
EFFVAR	percent;

	if(  NoFumbleMACRO
	   || NumFumblesOnPlay > ZERO
	   || (OPFlags(BallCarrier) & IM_LYING_ON_GROUND_BFLAG)
			 // no fumbles if lying on ground
	  )
	{	// multiple fumbles gives programmer headaches
		return(FALSE);
	}

	if( (!(EventFlags.TACKLE_STARTED_BFLAG)) 	//no fumble if already tackled
	   && (PlayStateNum == PS_PlayHappeningNUM)	// or if TD
	  )
	{

	// worse hands fumbles 50 out of 10000, best 17 of 10000
//		percent = (150 - OPHands(BallCarrier)) /3;	pre-breaking tackles

/* since breaking tackle logic allows 1/5 to 1/10 fewer hits (chances to
	tackle) the fumble number need to be high	(10/2/96)	*/
//		percent = (150 - OPHands(BallCarrier));	// 10/2/96 - after breaking tackles
//		percent = (150 - OPHands(BallCarrier)/2); // 10/17/96 - too many still
		percent = (150 - OPHands(BallCarrier))/2; // 11/21/96 - above was wrong!!!

		percent += WeatherFumblePercent[GameOptions[GAME_OPTNS_WEATHER]];

		if(HumanEffectValid(gPNum))
		{
			percent += 40;	// human is tackling

		}

		if( OPFlags(BallCarrier) & IM_SPINNING_BFLAG)
		{	// spinning guy has high chance of fumbling
//			percent *= 3;
//			percent >>= 1;	// + 50%
//			percent *= 2;  //  +100% (10/2/96)
//			percent *= 3;  //  +100% (10/27/96)
			percent *= 4;  //  +100% (11/1/96)
		}

#if 0	//PC_ALWAYS_MODE
		if(BallCarrier == 1 || BallCarrier == 2)	// test only
//		if(TRUE)	// test only
#else
		if( percent > (Random10000))	// changed to 10000%  due to too many fumbles
#endif
		{
			InitFumble(BALL_HIT_GUY_FUMBLE,(rand()&15),NEED_FUMBLE_SFX);
			return(TRUE);
		}
	}
	return(FALSE);
}


BOOL HeShouldFumbleGeneric(WORD possiblefumbler)
{
//WORD percent;
EFFVAR	percent;

	if(NoFumbleMACRO || NumFumblesOnPlay > ZERO)
	{
		return(FALSE);
	}

	// worse hands fumbles 5%, best 1.6%, average guy 3.3%
	// NEW 8/30/96 worse hands fumbles 0.5%, best 0.16%, average guy 0.33%
//	percent = (150 - OPHands(possiblefumbler)) / 3;
	percent = (150 - OPHands(possiblefumbler)) / 2;
	percent += WeatherFumblePercent[GameOptions[GAME_OPTNS_WEATHER]];

	if( percent > (Random10000))
	{
		return(TRUE);
	}
	return(FALSE);
}

BOOL FumbleShouldHappenOnHike(WORD possiblefumbler)
{
	/* worse hands fumbles 5% , best 1.6%, average guy 3.3% */
	if(NoFumbleMACRO || NumFumblesOnPlay > ZERO)
	{
		return(FALSE);
	}

//	return( (150 - OPHands(possiblefumbler))/3 > Random1000);
	return(possiblefumbler-possiblefumbler);	//i.e. FALSE test only

}


/*****************    After   Play   Over STATES    *********/



STATEDEF(DecelerateState)		/* used after player tackled	*/
{

#if 0	// test only to see if this causes freeze when return to huddle
	GoToNextState(1);		// next state should be Shuffle

#else
//WORD sp;
EFFVAR	sp;

	CPFacing = CPDirToGo;

	sp = DecrementPlayerSpeed(3,0);

	if(sp <	ZERO)		/* <0  is safer than == -1) */
	{	// here if completely stopped
		if(ImBallCarrierMACRO)
			InitGraphics(AI_STAND_NUM);
		else
			FaceBallandShuffle();
		GoToNextState(1);		// next state should be Shuffle
	}
	else
	{
		if(TryMove(CPDirToGo,sp) >= ZERO)
		{	/* he hit somebody	*/
			GoToNextState(1);		// exit this STATE if can't move
		}
		else if (sp > 3)
			Run();
		else
			Walk();

	}
#endif
}


/* WARNING - changings STATES  will NOT do anything to
	anybody "stuck" in a Frozen ACTION, as ACTIONs are at the lowest level
*/

STATEDEF(EverybodyWatchEOP)
{
	if(CPStateInit == ZERO)		// CPStateInit used as internal state counter
	{
		CPZPos = ZERO;	//bug # 2156

		if(gPNum == LastBallCarrier) // must use LBC, as BC is no longer valid
		{
			InitGraphics(AI_STAND_NUM);
			GoToNextState(1);
			return;
		}
		else
		{	// everybody on ground gets up at same time
//			if(CPAction ==	FreezeActionOnGround) impossiblity since if in STATE, ACTION = NOACTION
 //				CPTimer = 3*SECONDS;
 //			else
//				CPTimer = 2*SECONDS + (rand()&0x007f);
			if(gPNum == DefStrutter)
			{
				CPTimer = QUARTER_SECOND + (rand()&0x0031); 	// 1/4 to 3/4 seconds
				CPStateInit = 2; 	//go right to it
			}
			else
			{
				InitGraphics(AI_STAND_NUM);		// stand for a bit
				CPTimer = rand()&0x001f; 	// 0= 1/2 seconds
				CPStateInit = 1;
			}
		}
	}

	if(CPStateInit == 1)
	{
		InitGraphics(AI_STAND_NUM);
		if( (CPTimer -= TICKS_PER_AI_FRAME) < 0)
		{
			CPStateInit = 2; 	//do the shuffle! (for those who lived thru the 70's)
			CPTimer = HALF_SECOND + rand()&0x003f; 	// for this long (1/2  - 1.5 seconds)

		}
	}


	if(CPStateInit == 2)
	{
		FaceBallandShuffle();

		if( (CPTimer -= TICKS_PER_AI_FRAME) < 0)
		{
	//		if(CPAction ==	FreezeActionOnGround) if it was, I'd never be here in this STATE
	//			SetCPAction(GetUpAction);
//			GoToNextState(1); 	// return to huddle

			if(gPNum == DefStrutter)
			{
				InitGraphics(AI_DEFENSE_POSTTACKLE_NUM);
				SetCPAction(CycleAnimActionAndEndCamera); //CameraGuy = NOBODY at end of ACTION
			}

		}
		if(PlayStateNum >= PS_GetUpAfterPlayNUM)
		{
			GoToNextState(1); 	// return to huddle
		}


	}


}

/*--------------------------------------------------------------
				 ReturnToHuddle

	modified 8/11/96 to use delta method -
	gives "cleaner" runs with no twitching
	drawback is speed must be constant - no acceleration
----------------------------------------------------------------*/

STATEDEF(ReturnToHuddle)	// WARNING - never exit this STATE
{
struct formation  *FormationPtr;
//WORD yoff,slotnum;
EFFVAR	yoff,slotnum;
UWORD keyx;


	if(!CPStateInit)   // set up target
	{
		CPStateInit = TRUE;
		keyx = StartBallX;	//assumption - always true for DEF

		if(GameTeamData[CurTeamSlot].OffDefFlag == ON_OFFENSE)
		{
			FormationPtr = AllFormations[OFF_HUDDLE_FORM_NUM];

	/* following allows guys in KO to reach their ending positions faster */
			if(PlayType == KICKOFF_PLAY)
				keyx = StartBallX - 27*YARDS;	// fix bug #2263
			else if(StartBallX < DEF_GOAL_LINE+5*YARDS)
				keyx = DEF_GOAL_LINE+5*YARDS; // don't allow offense to huddle out of EZ

/*			// don't allow offense to huddle out of EZ
			if(StartBallX < DEF_GOAL_LINE+5*YARDS)
				keyx = DEF_GOAL_LINE+5*YARDS;		*/
		}
		else
		{
			FormationPtr = AllFormations[DEF_HUDDLE_FORM_NUM];
		}

	  /*  set Huddle X,Y as target 	*/
		slotnum = gPNum <11?gPNum:gPNum-11;
		CPTargetX = min((UWORD)(keyx + FormationPtr->FormationStart[slotnum].XStart),(UWORD)MAX_DEF_COVERX);

		yoff = FormationPtr->FormationStart[slotnum].YStart;
//		if(GameTeamData[CurTeamSlot].PlayFlipped)
//			yoff = -yoff;	this faces guys the wrong way, beside this
//							is if the LAST play was flipped, not the nextone
		CPTargetY = StartBallY + yoff;

		CPZPos = ZERO;	//bug # 2156
		CPDirToGo = FindCPDirectionToTargetMACRO;

		SetDeltaParameters(CPTargetX,CPTargetY,RETURN_TO_HUDDLE_SPEED);

	}


//	if( (AIFrameCounter&3) == ZERO)
//		CPDirToGo = FindCPDirectionToTargetMACRO;


	if(CPXPos == CPTargetX && CPYPos == CPTargetY)
	{ 	// we're at target
		CPSpeed = ZERO;	// bug #2255 7/18
		slotnum = gPNum <11?gPNum:gPNum-11;
		if(GameTeamData[CurTeamSlot].OffDefFlag == ON_OFFENSE)
		{
			CPFacing = OffHuddleFacing[slotnum];
			InitGraphics(AI_2PTSTANCE_NUM);
		}
		else
		{
			CPFacing = DefHuddleFacing[slotnum];
			InitGraphics(AI_STAND_NUM);
		}

	}
	else
	{
		if((CPDeltaX | CPDeltaY)==ZERO)
		{	// not at target and not going anywhere - this is trouble
			CPDeltaX = CPDeltaY = 20;
		}

		CPXPos += CPDeltaX;
		CPYPos += CPDeltaY;
			// no collision detect or acceleration
//		CPXPos += SpeedTable[RETURN_TO_HUDDLE_SPEED][CPDirToGo].X;
//		CPYPos += SpeedTable[RETURN_TO_HUDDLE_SPEED][CPDirToGo].Y;

		CPSpeed = 12<<SPEED_SHIFT;	// makes the RUN a run, not a walk(2ticks/frame update)
		Run();

		TurnPlayerOptimal(CPDirToGo);

		if(ReachedTarget())
		{
			CPXPos = CPTargetX;	  // make the current EXACTLY what the target is
			CPYPos = CPTargetY;
			GeneratePlayerSphereOfInfluence(gPNum);

			slotnum = gPNum <11?gPNum:gPNum-11;
			CPFacing = 	GameTeamData[CurTeamSlot].OffDefFlag == ON_OFFENSE
			 ? OffHuddleFacing[slotnum]:DefHuddleFacing[slotnum];

			CPSpeed = ZERO;	// bug #2255 7/18
//		GoToNextState(1)	// stay here forever, or until next PlayInit
		}
	}
}

STATEDEF(InHuddleState)	// WARNING - never exit this STATE
{

}

#if 1
STATEDEF(CelebrateState)	// for TD
{
//WORD maxframesinanim;
EFFVAR	maxframesinanim;

	CPZPos = ZERO;		// bug # 2043?
	if(!CPStateInit)
	{
		CPStateInit = TRUE;
		InitGraphics(AI_DEFENSE_POSTTACKLE_NUM);	//assume

		if(  (TDAnimLoadedNum >= 0)
			&& (TDAnimLoadedNum < MAX_CELEBRATIONS)
			&& (rand()&7)		// 1/8 of time do defense_posttackle
		  )
		{
			InitGraphics(Celebrations[TDAnimLoadedNum]);
		}

	}
	else
	{
		maxframesinanim = MAX_FRAMES_MACRO(CPAIGraphicState);
		if(++CPAIGraphicSequence > maxframesinanim)
		{
		/*  if greater than last frame number, jump higher	*/
			CPAIGraphicSequence = maxframesinanim;	// just to be safe
			CameraGuy = NOBODY;
			GoToNextState(1);
		}
	}
}
#else  // ----------- obsolete 10/5/96 ----------------
STATEDEF(CelebrateState)	// for TD
{
//WORD maxframesinanim;
EFFVAR	maxframesinanim;

	if(  (TDAnimLoadedNum >= 0)
	  && (TDAnimLoadedNum < MAX_CELEBRATIONS) )
	{
		CPZPos = ZERO;		// bug # 2043?
		if(!CPStateInit)
		{
			InitGraphics(Celebrations[TDAnimLoadedNum]);
			CPStateInit = TRUE;
		}
		else
		{
			maxframesinanim = MAX_FRAMES_MACRO(CPAIGraphicState);
			if(++CPAIGraphicSequence > maxframesinanim)
			{
			/*  if greater than last frame number, jump higher	*/
				CPAIGraphicSequence = maxframesinanim;	// just to be safe
				CameraGuy = NOBODY;
				GoToNextState(1);
			}
		}

	}
	else
	{
		CameraGuy = NOBODY;
		GoToNextState(1);
	}
}
#endif


void BallBeingBounced(void);




STATEDEF(Celebrate1State)	// for TD - will spike ball over goalposts
{
//WORD maxframesinanim;
EFFVAR	maxframesinanim;


	if(CPStateInit == ZERO)		// set target
	{
		if(gPNum > 10)	// only offense can do this
		{
			GoToNextState(1);
			CameraGuy = NOBODY;
			return;
		}
		CPStateInit = 1;

		BallCarrier = gPNum;
		SetBallAction(PlayerHasBall);

		CPTargetX = OFF_END_LINE-QTR_YARD;
		CPTargetY = CPYPos < MID_FIELDY ? MID_FIELDY-3*YARD : MID_FIELDY+3*YARD;
		CPFacing = CPDirToGo = FindCPDirectionToTargetMACRO;
		SetDeltaParameters(CPTargetX,CPTargetY,19);
		SetBit(PlayOverFlags,BALL_DEAD_BFLAG);

	}

	if(CPStateInit == 1)		// run to target
	{
		if(CPDeltaX < 1)	// running backward ???
		{ 	// had a lock-up in SIM at EOG - hopefully this will fix
			CameraGuy = BallCarrier = NOBODY;
			GoToNextState(1);
		}

		CPXPos += CPDeltaX;
		CPYPos += CPDeltaY;

		CPSpeed = 12<<SPEED_SHIFT;	// makes the RUN a run, not a walk(2ticks/frame update)
		Run();

		TurnPlayerOptimal(CPDirToGo);

		if(CPXPos > CPTargetX)
		{
			CPFacing = EAST_DIR;	// looks better for ball placement
//		if(ReachedTarget())
			CPStateInit = 3;
		}
	}

/*
	if(CPStateInit == 2)		// jump up
	{
		if(!InitGraphicsQ(AI_HANDSUP_NUM))	// if necessary
			CPAIGraphicSequence++;

		maxframesinanim = MAX_FRAMES_MACRO(CPAIGraphicState);
		if(CPAIGraphicSequence >= maxframesinanim)
		{

			CPAIGraphicSequence = maxframesinanim;	// just to be safe
			CPZPos += JUMP_Z_INC;
			if(CPZPos > 3*FEET)
				CPStateInit = 3;
		}
	}
*/

	if(CPStateInit == 3)		// spike it
	{
		if(!InitGraphicsQ(AI_GP_SPIKE_NUM))	// if necessary
			CPAIGraphicSequence++;

		maxframesinanim = MAX_FRAMES_MACRO(CPAIGraphicState);
		if(CPAIGraphicSequence > maxframesinanim)
		{
			CPAIGraphicSequence = maxframesinanim;	// just to be safe
			CPStateInit = 4;
		}
		else if(CPAIGraphicSequence == 10)	// careful - hardwired
		{
			BallZPosition = 13*FEET;

			BallTargetX = CPXPos + 2 * YARD;
			BallTargetY = CPYPos;
			BallDeltaX = 150;
			BallDeltaY = 100;

			CurrentBallDeltaZ = -140;
			CurrentBallGravity = 100;	// high so it never goes to zero

			SetBallAction(BallBeingBounced);	// will do this frame, (ball done AFTER players)

			BallCarrier = NOBODY;
		}
	}

	if(CPStateInit == 4)		// come down
	{
		if(InitGraphicsQ(AI_HANDSUP_NUM))	// if necessary
			CPAIGraphicSequence = 3;

		CPZPos -= JUMP_Z_INC;

		if( CPZPos >0xf000		// ZPOS is UWORD - need High check
			|| CPZPos == ZERO)
		{
			CPZPos = ZERO;
			CPAIGraphicSequence = 2;	// be safe
			GoToNextState(1);
			CameraGuy = NOBODY;
		}

	}

}

/*******************************************************************


			 R E F E R E E      S T A T E S

	WARNING - ref run amin only in 1 dir - (and flipped)

*******************************************************************/

void SayRefPhrase(void);

STATEDEF(RefRunToTargetState)
{	//  warning  -  Target must have already been set up
	// used to run ref ON and then OFF field - ref

	if(!CPStateInit)
	{
		CPStateInit = TRUE;
		CPFacing = CPDirToGo =  FindCPDirectionToTargetMACRO;
		InitGraphics(AI_REF_RUN_NUM);
		SetBit(CPFlags,REF_ACTIVE_BFLAG);
	}

	CPXPos += SpeedTable[REF_RUN_SPEED][CPDirToGo].X;
	CPYPos += SpeedTable[REF_RUN_SPEED][CPDirToGo].Y;

	if( (CPFlags & REF_RUNNING_OUT_BFLAG)	// used for camera control
	  && (PlayStateNum > PS_GetUpAfterPlayNUM)
	  )
	{
		PutCameraOnRefMacro;
	}

	CYCLE_GRAPHICS_MACRO(CPAIGraphicState);
	if(ReachedTarget())
	{
		ResetBit(CPFlags,REF_RUNNING_OUT_BFLAG);	// used for camera control
		GoToNextState(1);
	}
}


STATEDEF(RefDoNothingState)
{
	CPFacing = CPDirToGo =  FindCPDirectionToBallMACRO;	// look at ball
	InitGraphics(AI_REF_STAND_NUM);
	ResetBit(CPFlags,REF_ACTIVE_BFLAG | REF_RUNNING_OUT_BFLAG);
	PutCameraOffRefMacro;	// used by graphics engine - Off just to be sure

}

STATEDEF(RefDoHisThingState)		// do anim an/or SFX
{

	CPFacing = CPDirToGo =  WEST_DIR;	// face offense
//	CPFacing = CPDirToGo =  SOUTH_DIR;	// face dir ref will run off

	InitGraphics(RefAIAnimPending);
//	CPAIGraphicSequence = 0;

	if(CPAIGraphicSequence == MAX_FRAMES_MACRO(CPAIGraphicState)/2)
	{
		if(RefSFXPending >= ZERO)	// something to say??
		{
			SayRefPhrase();
//			AIPlayEffectCheck(RefSFXPending);
//			RefSFXPending = -1;	// not until done with anim
		}
	}

	if(IncFramesCheckAtEnd())
	{
		if(GameFlags & NEED_2MIN_WARNING_BFLAG)
		{
			ResetBit(GameFlags,NEED_2MIN_WARNING_BFLAG);
			SetBit(GameFlags,GAVE_2MIN_WARNING_BFLAG);
		}


		CPTargetX = CPXPos;
//		CPTargetY = NORTH_SIDELINE - 2 *YARDS;
//		CPTargetY = SOUTH_SIDELINE + 2 *YARDS;	// bug # 1750
	// run off same dir as run on due to Sideline camera - bug #3080
		CPTargetY = NORTH_SIDELINE - 3 *YARDS;


		RefSFXPending = -1;	// also okay to load Play call in PLAYSTATE(PS_Wait7)
		PutCameraOffRefMacro;	// used by graphics engine
		SetBallAction(BallPreSnap);	// ... and have it go to new ball location

		GoToNextState(1);

	}

}



STATEDEF(RefPenaltyOutcomeState)
{
	if(PenaltyResolved)		//
	{
		PenaltyResolved = 0;
		GoToNextState(1);

	}

}

/**   following subs save code (speed hit is okay)		***/

WORD FindCPDirectionToTargetSub(void)
{
	return(FindDirection(CPTargetX-CPXPos,CPTargetY-CPYPos));
}

WORD FindCPDirectionToBallSub(void)
{
	return(FindDirection(BallXPosition-CPXPos,BallYPosition-CPYPos));
}

WORD FindCPDirectionToPlayer(WORD opnum)
{
	return(FindDirection((OPXPos(opnum)-CPXPos),(OPYPos(opnum)-CPYPos)));
}

void CycleGraphics(UWORD graphnum)
{
	if(++CPAIGraphicSequence > MaxAIFrames[graphnum])
		CPAIGraphicSequence = ZERO;
}


/*************************  end of  file - FBMOVE.C  *********************/
